@@ -38,30 +38,15 @@
  * ***** END LICENSE BLOCK ***** */
 
 // NOTE: alphabetically ordered
-#include "nsAccessible.h"
-#include "nsFormControlAccessible.h"
-#include "nsHTMLAtoms.h"
 #include "nsHTMLFormControlAccessible.h"
-#include "nsIClipboard.h"
-#include "nsIDOMHTMLButtonElement.h"
-#include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMHTMLInputElement.h"
 #include "nsIDOMNSHTMLButtonElement.h"
-#include "nsIDOMHTMLLabelElement.h"
 #include "nsIDOMHTMLTextAreaElement.h"
-#include "nsIDOMXULCheckboxElement.h"
-#include "nsIDOMXULButtonElement.h"
-#include "nsIDOMXULSelectCntrlItemEl.h"
-#include "nsIDOMXULSelectCntrlEl.h"
-#include "nsIEditor.h"
-#include "nsIEventStateManager.h"
 #include "nsIFrame.h"
-#include "nsIGfxTextControlFrame.h"
-#include "nsINameSpaceManager.h"
-#include "nsIPlaintextEditor.h"
 #include "nsISelectionController.h"
-#include "nsReadableUtils.h"
-#include "nsWeakReference.h"
+#ifdef MOZ_XUL
+#include "nsIDOMXULTextboxElement.h"
+#endif
 
 // --- checkbox -----
 
@@ -225,6 +210,11 @@ NS_IMETHODIMP nsHTMLButtonAccessible::GetAccName(nsAString& _retval)
   nsAutoString name;
   button->GetValue(name);
   name.CompressWhitespace();
+  if (name.IsEmpty()) {
+    nsCOMPtr<nsIDOMElement> elt(do_QueryInterface(mDOMNode));
+    elt->GetAttribute(NS_LITERAL_STRING("title"), name);
+  }
+
   _retval.Assign(name);
 
   return NS_OK;
@@ -315,7 +305,7 @@ nsFormControlAccessible(aNode, aShell)
 { 
 }
 
-NS_IMPL_ISUPPORTS_INHERITED1(nsHTMLTextFieldAccessible, nsFormControlAccessible, nsIAccessibleEditableText)
+NS_IMPL_ISUPPORTS_INHERITED0(nsHTMLTextFieldAccessible, nsFormControlAccessible)
 
 NS_IMETHODIMP nsHTMLTextFieldAccessible::GetAccRole(PRUint32 *_retval)
 {
@@ -332,16 +322,21 @@ NS_IMETHODIMP nsHTMLTextFieldAccessible::GetAccValue(nsAString& _retval)
 
   nsCOMPtr<nsIDOMHTMLTextAreaElement> textArea(do_QueryInterface(mDOMNode));
   if (textArea) {
-    textArea->GetValue(_retval);
-    return NS_OK;
+    return textArea->GetValue(_retval);
   }
   
   nsCOMPtr<nsIDOMHTMLInputElement> inputElement(do_QueryInterface(mDOMNode));
   if (inputElement) {
-    inputElement->GetValue(_retval);
-    return NS_OK;
+    return inputElement->GetValue(_retval);
   }
 
+#ifdef MOZ_XUL
+  nsCOMPtr<nsIDOMXULTextboxElement> textBox(do_QueryInterface(mDOMNode));
+  if (textBox) {
+    return textBox->GetValue(_retval);
+  }
+#endif
+
   return NS_ERROR_FAILURE;
 }
 
@@ -349,6 +344,25 @@ NS_IMETHODIMP nsHTMLTextFieldAccessible::GetAccState(PRUint32 *_retval)
 {
   // can be
   // focusable, focused, protected. readonly, unavailable, selected
+#ifdef MOZ_XUL
+  nsCOMPtr<nsIDOMXULTextboxElement> textBox(do_QueryInterface(mDOMNode));
+  if (textBox) {
+    nsCOMPtr<nsIDOMHTMLInputElement> inputField;
+    textBox->GetInputField(getter_AddRefs(inputField));
+    if (!inputField) {
+      return NS_ERROR_FAILURE;
+    }
+    // Create a temporary accessible from the HTML text field
+    // to get the accessible state from. Doesn't add to cache
+    // because Init() is not called.
+    nsHTMLTextFieldAccessible tempAccessible(inputField, mWeakShell);
+    return tempAccessible.GetAccState(_retval);
+  }
+#endif
+
+  if (!mDOMNode) {
+    return NS_ERROR_FAILURE;  // Node has been Shutdown()
+  }
 
   nsAccessible::GetAccState(_retval);
   *_retval |= STATE_FOCUSABLE;
@@ -363,16 +377,16 @@ NS_IMETHODIMP nsHTMLTextFieldAccessible::GetAccState(PRUint32 *_retval)
     *_retval |= STATE_READONLY;
 
   // Get current selection and find out if current node is in it
-  nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mPresShell));
+  nsCOMPtr<nsIPresShell> shell(GetPresShell());
   if (!shell) {
      return NS_ERROR_FAILURE;  
   }
 
-  nsCOMPtr<nsIPresContext> context;
-  shell->GetPresContext(getter_AddRefs(context));
   nsCOMPtr<nsIContent> content(do_QueryInterface(mDOMNode));
   nsIFrame *frame = nsnull;
   if (content && NS_SUCCEEDED(shell->GetPrimaryFrameFor(content, &frame)) && frame) {
+    nsCOMPtr<nsIPresContext> context;
+    shell->GetPresContext(getter_AddRefs(context));
     nsCOMPtr<nsISelectionController> selCon;
     frame->GetSelectionController(context,getter_AddRefs(selCon));
     if (selCon) {
@@ -407,107 +421,6 @@ NS_IMETHODIMP nsHTMLTextFieldAccessible::GetAccState(PRUint32 *_retval)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsHTMLTextFieldAccessible::SetAttributes(PRInt32 aStartPos, PRInt32 aEndPos, nsISupports *aAttributes)
-{
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::SetTextContents(const nsAString &aText)
-{
-  nsCOMPtr<nsIDOMHTMLTextAreaElement> textArea(do_QueryInterface(mDOMNode));
-  if (textArea)
-    return textArea->SetValue(aText);
-  
-  nsCOMPtr<nsIDOMHTMLInputElement> inputElement(do_QueryInterface(mDOMNode));
-  if (inputElement)
-    return inputElement->SetValue(aText);
-
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::MakeSelection(PRInt32 aStartPos, PRInt32 aEndPos, nsIEditor **aEditor)
-{
-  nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mPresShell));
-  if (!shell)
-    return NS_ERROR_FAILURE;  
-
-  AccTakeFocus();
-
-  nsCOMPtr<nsIContent> content(do_QueryInterface(mDOMNode));
-  nsIFrame *frame = nsnull;
-  shell->GetPrimaryFrameFor(content, &frame);
-  nsCOMPtr<nsIGfxTextControlFrame2> tframe(do_QueryInterface(frame));
-  if (!tframe)
-    return NS_ERROR_FAILURE;  
-
-  nsCOMPtr<nsIEditor> editor;
-  tframe->SetSelectionRange(aStartPos, aEndPos);
-  tframe->GetEditor(getter_AddRefs(editor));
-  if (!editor)
-    return NS_ERROR_FAILURE;  
-
-  *aEditor = editor;
-  NS_ADDREF(*aEditor);
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::InsertText(const nsAString &aText, PRInt32 aPosition)
-{
-  nsCOMPtr<nsIEditor> editor;
-  if (NS_SUCCEEDED(MakeSelection(aPosition, aPosition, getter_AddRefs(editor)))) {
-    nsCOMPtr<nsIPlaintextEditor> peditor(do_QueryInterface(editor));
-    peditor->InsertText(aText);
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::CopyText(PRInt32 aStartPos, PRInt32 aEndPos)
-{
-  nsCOMPtr<nsIEditor> editor;
-  if (NS_SUCCEEDED(MakeSelection(aStartPos, aEndPos, getter_AddRefs(editor)))) {
-    editor->Copy();
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::CutText(PRInt32 aStartPos, PRInt32 aEndPos)
-{
-  nsCOMPtr<nsIEditor> editor;
-  if (NS_SUCCEEDED(MakeSelection(aStartPos, aEndPos, getter_AddRefs(editor)))) {
-    editor->Cut();
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::DeleteText(PRInt32 aStartPos, PRInt32 aEndPos)
-{
-  nsCOMPtr<nsIEditor> editor;
-  if (NS_SUCCEEDED(MakeSelection(aStartPos, aEndPos, getter_AddRefs(editor)))) {
-    editor->DeleteSelection(nsIEditor::eNone);
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::PasteText(PRInt32 aPosition)
-{
-  nsCOMPtr<nsIEditor> editor;
-  if (NS_SUCCEEDED(MakeSelection(aPosition, aPosition, getter_AddRefs(editor)))) {
-    editor->Paste(nsIClipboard::kGlobalClipboard);
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
 // --- groupbox  -----
 
 /*
@@ -515,7 +428,7 @@ NS_IMETHODIMP nsHTMLTextFieldAccessible::PasteText(PRInt32 aPosition)
  */
 
 nsHTMLGroupboxAccessible::nsHTMLGroupboxAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsAccessible(aNode, aShell)
+nsAccessibleWrap(aNode, aShell)
 { 
 }
 
