@@ -24,6 +24,7 @@
  *                 Karl Guertin <grayrest@grayrest.com> 
  *                 Mike Norton <xor@ivwnet.com>
  *                 ArentJan Banck <ajbanck@planet.nl> 
+ *                 Eric Belhaire <belhaire@ief.u-psud.fr>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -44,28 +45,17 @@
 *   Garth Smedley
 *
 * REQUIRED INCLUDES 
-*        <script type="application/x-javascript" src="chrome://global/content/strres.js"/>
 *        <script type="application/x-javascript" src="chrome://calendar/content/calendarEvent.js"/>
 *
 * NOTES
-*   Code for the oe-calendar.
+*   Code for the calendar.
 *
 *   What is in this file:
 *     - Global variables and functions - Called directly from the XUL
 *     - Several classes:
-*           CalendarWindow - Owns the 3 subviews, keeps track of the selection etc.
-*           CalendarView - Super class for three views
-*                  MonthView 
-*                  WeekView 
-*                  DayView 
-*     - Unifinder functions - fill in content of the unifinder.
 *  
 * IMPLEMENTATION NOTES 
 *
-*  WARNING  - We currently use CalendarEvent objects in what will be an unsafe way
-*             when the datasource changes. We assume that there is exactly one instance of CalendarEvent for 
-*             each unique event. We will probably have to switch to using the calendar event id 
-*             property instead of the object. G.S. April 2001.
 **********
 */
 
@@ -93,21 +83,22 @@ var gEventSource = null;
 // single global instance of CalendarWindow
 var gCalendarWindow;
 
+// style sheet number for calendar
+var gCalendarStyleSheet;
+
 //an array of indexes to boxes for the week view
 var gHeaderDateItemArray = null;
 
-var gClockId;
-
-// Show event details on mouseover
+// Show event details on mouseover (sometimes this is false in the code)
 var showTooltip = true;
 
+//Show only the working days (changed in different menus)
+var gOnlyWorkdayChecked ;
+
 // DAY VIEW VARIABLES
 var kDayViewHourLeftStart = 105;
 
 var kWeekViewHourHeight = 50;
-var kWeekViewHourWidth = 78;
-var kWeekViewItemWidth = 20;
-var kWeekViewHourLeftStart = 97;
 var kWeekViewHourHeightDifference = 2;
 var kDaysInWeek = 7;
 
@@ -134,14 +125,21 @@ function calendarInit()
    
    // get the Ical Library
    gICalLib = gEventSource.getICalLib();
+
+   // this suspends feedbacks to observers until all is settled
+   gICalLib.batchMode = true;
    
    // set up the CalendarWindow instance
    
-   gCalendarWindow = new CalendarWindow( gEventSource );
+   gCalendarWindow = new CalendarWindow();
    
    //when you switch to a view, it takes care of refreshing the events, so that call is not needed.
    gCalendarWindow.currentView.switchTo( gCalendarWindow.currentView );
 
+   // set up the checkboxes variables
+   gOnlyWorkdayChecked = document.getElementById( "only-workday-checkbox-1" ).getAttribute("checked") ;
+   gDisplayToDoInViewChecked = document.getElementById( "display-todo-inview-checkbox-1" ).getAttribute("checked") ;
+
    // set up the unifinder
    
    prepareCalendarUnifinder();
@@ -152,6 +150,44 @@ function calendarInit()
    	
 	checkForMailNews();
 
+   // Change made by CofC for Calendar Coloring
+   // initialize calendar color style rules in the calendar's styleSheet
+
+   // find calendar's style sheet index
+   for (var i=0; i<document.styleSheets.length; i++)
+   {
+      if (document.styleSheets[i].href == "chrome://calendar/skin/calendar.css")
+	  {
+          gCalendarStyleSheet = document.styleSheets[i];
+		  break;
+	  }
+   }
+
+   var calendarNode;
+   var calendarColor;
+
+   // loop through the calendars via the rootSequence of the RDF datasource
+   var seq = gCalendarWindow.calendarManager.rdf.getRootSeq("urn:calendarcontainer");
+   var list = seq.getSubNodes();
+
+	for(var i=0; i<list.length;i++)
+	{
+
+     calendarNode = gCalendarWindow.calendarManager.rdf.getNode( list[i].subject );
+     
+	 // grab the container name and use it for the name of the style rule
+	 containerName = list[i].subject.split(":")[2];
+
+	 // obtain calendar color from the rdf datasource
+     calendarColor = calendarNode.getAttribute("http://home.netscape.com/NC-rdf#color");
+
+	 // if the calendar had a color attribute create a style sheet for it
+     if (calendarColor != null)
+       gCalendarStyleSheet.insertRule("." + containerName + " { background-color:" + calendarColor + "!important;}", 1);
+
+   }
+   // CofC Calendar Coloring Change
+
    if( window.arguments && window.arguments[0].channel )
    {
       gCalendarWindow.calendarManager.checkCalendarURL( window.arguments[0].channel );
@@ -160,6 +196,11 @@ function calendarInit()
    //a bit of a hack since the menulist doesn't remember the selected value     
    var value = document.getElementById( 'event-filter-menulist' ).value;
    document.getElementById( 'event-filter-menulist' ).selectedItem = document.getElementById( 'event-filter-'+value );
+
+   gEventSource.alarmObserver.firePendingAlarms();
+
+   //All is settled, enable feedbacks to observers
+   gICalLib.batchMode = false;
 }
 
 // Set the date and time on the clock and set up a timeout to refresh the clock when the 
@@ -176,7 +217,7 @@ function update_date()
    
    gCalendarWindow.currentView.hiliteTodaysDate();
 
-   gClockId = setTimeout( "update_date()", milliSecsTillTomorrow ); 
+   setTimeout( "update_date()", milliSecsTillTomorrow ); 
 }
 
 /** 
@@ -190,6 +231,8 @@ function calendarFinish()
    finishCalendarToDoUnifinder();
 
    gCalendarWindow.close();
+
+   gICalLib.removeObserver( gEventSource.alarmObserver );
 }
 
 function launchPreferences()
@@ -243,6 +286,9 @@ function dayEventItemClick( eventBox, event )
 
 function dayEventItemDoubleClick( eventBox, event )
 {
+   // we only care about button 0 (left click) events
+   if (event.button != 0) return;
+   
    editEvent( eventBox.calendarEventDisplay.event );
 
    if ( event ) 
@@ -293,6 +339,9 @@ function dayViewHourContextClick( event )
 
 function dayViewHourDoubleClick( event )
 {
+   // we only care about button 0 (left click) events
+   if (event.button != 0) return;
+   
    var startDate = gCalendarWindow.dayView.getNewEventDate();
    
    newEvent( startDate );
@@ -339,6 +388,9 @@ function weekEventItemClick( eventBox, event )
 
 function weekEventItemDoubleClick( eventBox, event )
 {
+   // we only care about button 0 (left click) events
+   if (event.button != 0) return;
+   
    editEvent( eventBox.calendarEventDisplay.event );
 
    if ( event ) 
@@ -394,6 +446,9 @@ function weekViewContextClick( event )
 
 function weekViewHourDoubleClick( event )
 {
+   // we only care about button 0 (left click) events
+   if (event.button != 0) return;
+        
    var startDate = gCalendarWindow.weekView.getNewEventDate();
    
    newEvent( startDate );
@@ -419,7 +474,7 @@ function monthEventBoxClickEvent( eventBox, event )
 
       newDate.setDate( eventBox.calendarEventDisplay.event.start.day );
 
-      gCalendarWindow.setSelectedDate( newDate );
+      gCalendarWindow.setSelectedDate( newDate, false );
    }
 
    if ( event ) 
@@ -439,6 +494,9 @@ function monthEventBoxClickEvent( eventBox, event )
 
 function monthEventBoxDoubleClickEvent( eventBox, event )
 {
+   // we only care about button 0 (left click) events
+   if (event.button != 0) return;
+   
    gCalendarWindow.monthView.clearSelectedDate();
    
    editEvent( eventBox.calendarEventDisplay.event );
@@ -450,6 +508,60 @@ function monthEventBoxDoubleClickEvent( eventBox, event )
 }
    
 
+/** 
+* Called on single click on an todo box in the multiweek view
+*
+* PARAMETERS
+*    todoBox - The XUL box clicked on
+*    event      - the click event
+*/
+
+function multiweekToDoBoxClickEvent( todoBox, event )
+{
+   //do this check, otherwise on double click you get into an infinite loop
+   if( event.detail == 1 )
+   {
+      gCalendarWindow.EventSelection.replaceSelection( todoBox.calendarToDo );
+      
+      var newDate = gCalendarWindow.getSelectedDate();
+
+      newDate.setDate( todoBox.calendarToDo.due.day );
+
+      gCalendarWindow.setSelectedDate( newDate, false );
+   }
+
+   if ( event ) 
+   {
+      event.stopPropagation();
+   }
+}
+
+
+/** 
+* Called on double click on an todo box in the multiweek view
+* launches the edit dialog on the event
+*
+* PARAMETERS
+*    todoBox - The XUL box clicked on
+*    event      - the click event
+*/
+
+function multiweekToDoBoxDoubleClickEvent( todoBox, event )
+{
+   // we only care about button 0 (left click) events
+   if (event.button != 0) return;
+   
+   gCalendarWindow.multiweekView.clearSelectedDate();
+   
+   editToDo( todoBox.calendarToDo );
+
+   if ( event ) 
+   {
+      event.stopPropagation();
+   }
+}
+   
+
 /** 
 * Called when the new event button is clicked
 */
@@ -557,7 +669,7 @@ function newEvent( startDate, endDate )
    
    if( !endDate )
    {
-      var MinutesToAddOn = getIntPref(gCalendarWindow.calendarPreferences.calendarPref, "event.defaultlength", 60 );
+     var MinutesToAddOn = getIntPref(gCalendarWindow.calendarPreferences.calendarPref, "event.defaultlength", gCalendarBundle.getString("defaultEventLength" ) );
    
       var endDateTime = startDate.getTime() + ( 1000 * 60 * MinutesToAddOn );
    
@@ -575,7 +687,7 @@ function newEvent( startDate, endDate )
 
    if( selectedCalendarItem )
    {
-      server = selectedCalendarItem.calendarPath;
+      server = selectedCalendarItem.getAttribute( "calendarPath" );
    }
    
    editNewEvent( calendarEvent, server );
@@ -614,7 +726,7 @@ function editNewEvent( calendarEvent, server )
 
 function addEventDialogResponse( calendarEvent, Server )
 {
-   gICalLib.addEvent( calendarEvent, Server );
+   refreshRemoteCalendarAndRunFunction( calendarEvent, Server, "addEvent" );
 }
 
 
@@ -679,7 +791,7 @@ function editToDo( calendarToDo )
 
 function modifyEventDialogResponse( calendarEvent, Server )
 {
-   gICalLib.modifyEvent( calendarEvent, Server );
+   refreshRemoteCalendarAndRunFunction( calendarEvent, Server, "modifyEvent" );
 }
 
 
@@ -714,18 +826,53 @@ function editEventCommand()
 }
 
 
+function refreshRemoteCalendarAndRunFunction( calendarEvent, Server, functionToRun )
+{
+   var calendarServer = gCalendarWindow.calendarManager.getCalendarByName( Server )
+   
+   if( calendarServer )
+   {
+      if( calendarServer.getAttribute( "http://home.netscape.com/NC-rdf#publishAutomatically" ) == "true" )
+      {
+         var onResponseExtra = function( )
+         {
+            //add the event
+            eval( "gICalLib."+functionToRun+"( calendarEvent, Server )" );
+   
+            gCalendarWindow.clearSelectedEvent( calendarEvent );
+            
+            //publish the changes back to the server
+            if( calendarServer.getAttribute( "http://home.netscape.com/NC-rdf#publishAutomatically" ) == "true" )
+            {
+               gCalendarWindow.calendarManager.publishCalendar( calendarServer );
+            }
+         }
+   
+         //refresh the calendar file.
+         gCalendarWindow.calendarManager.retrieveAndSaveRemoteCalendar( calendarServer, onResponseExtra );
+      }
+      else
+      {
+         eval( "gICalLib."+functionToRun+"( calendarEvent, Server )" );
+   
+         gCalendarWindow.clearSelectedEvent( calendarEvent );
+      }
+   }
+   else
+   {
+      eval( "gICalLib."+functionToRun+"( calendarEvent, Server )" );
+   
+      gCalendarWindow.clearSelectedEvent( calendarEvent );
+   }
+}
+
+
 /**
 *  This is called from the unifinder's delete command
 */
 
 function deleteEventCommand( DoNotConfirm )
 {
-   if( unifinderToDoHasFocus() )
-   {
-      deleteToDoCommand( DoNotConfirm );
-      return;
-   }
-   
    var SelectedItems = gCalendarWindow.EventSelection.selectedEvents;
    
    if( SelectedItems.length == 1 )
@@ -734,16 +881,15 @@ function deleteEventCommand( DoNotConfirm )
 
       if ( calendarEvent.title != "" ) {
          if( !DoNotConfirm ) {        
-            if ( confirm( confirmDeleteEvent+" "+calendarEvent.title+"?" ) ) {
-               gICalLib.deleteEvent( calendarEvent.id );
-
-               gCalendarWindow.clearSelectedEvent( calendarEvent );
+            if ( confirm( confirmDeleteEvent+" "+calendarEvent.title+"?" ) ) 
+            {
+               refreshRemoteCalendarAndRunFunction( calendarEvent.id, calendarEvent.parent.server, "deleteEvent" );
             }
          }
          else
          {
-            gICalLib.deleteEvent( calendarEvent.id );
-
+            refreshRemoteCalendarAndRunFunction( calendarEvent.id, calendarEvent.parent.server, "deleteEvent" );
+            
             gCalendarWindow.clearSelectedEvent( calendarEvent );
          }
       }
@@ -751,14 +897,14 @@ function deleteEventCommand( DoNotConfirm )
       {
          if( !DoNotConfirm ) {        
             if ( confirm( confirmDeleteUntitledEvent ) ) {
-               gICalLib.deleteEvent( calendarEvent.id );
+               refreshRemoteCalendarAndRunFunction( calendarEvent.id, calendarEvent.parent.server, "deleteEvent" );
 
                gCalendarWindow.clearSelectedEvent( calendarEvent );
             }
          }
          else
          {
-            gICalLib.deleteEvent( calendarEvent.id );
+            refreshRemoteCalendarAndRunFunction( calendarEvent.id, calendarEvent.parent.server, "deleteEvent" );
 
             gCalendarWindow.clearSelectedEvent( calendarEvent );
          }
@@ -840,7 +986,15 @@ function playSound( ThisURL )
    
    sample = sample.QueryInterface(Components.interfaces.nsISound);
 
-   sample.play( url );
+   try
+   {
+      sample.play( url );
+   }
+   catch ( ex )
+   {
+      sample.beep();
+      //alert( ex );
+   }
 }
 
 var gSelectAll = false;
@@ -849,7 +1003,7 @@ function selectAllEvents()
 {
    gSelectAll = true;
 
-   gCalendarWindow.EventSelection.setArrayToSelection( gCalendarWindow.eventSource.currentEvents );
+   gCalendarWindow.EventSelection.setArrayToSelection( gEventSource.currentEvents );
 }
 
 function closeCalendar()
@@ -865,6 +1019,71 @@ function launchWizard()
    openDialog("chrome://calendar/content/wizard.xul", "caWizard", "chrome,modal", args );
 }
 
+/**
+*  Called when a user hovers over a todo element and the text for the mouse over is changed.
+*/
+
+function getPreviewTextForTask( toDoItem )
+{
+   var HolderBox = document.createElement( "vbox" );
+   var textString ;
+
+   if( toDoItem )
+   {
+      showTooltip = true; //needed to show the tooltip.
+         
+      if (toDoItem.title)
+      {
+         var TitleHtml = document.createElement( "description" );
+	 textString = gCalendarBundle.getFormattedString("tooltipTitleElement", [toDoItem.title]);
+	 var TitleText = document.createTextNode( textString );
+         TitleHtml.appendChild( TitleText );
+         HolderBox.appendChild( TitleHtml );
+      }
+   
+      var DateHtml = document.createElement( "description" );
+      var startDate = new Date( toDoItem.start.getTime() );
+      textString = gCalendarBundle.getFormattedString("tooltipTaskStart", 
+						   [gCalendarWindow.dateFormater.getFormatedDate( startDate )]);
+      var DateText = document.createTextNode( textString );
+      DateHtml.appendChild( DateText );
+      HolderBox.appendChild( DateHtml );
+   
+      DateHtml = document.createElement( "description" );
+      var dueDate = new Date( toDoItem.due.getTime() );
+      textString = gCalendarBundle.getFormattedString("tooltipTaskEnd", 
+						      [gCalendarWindow.dateFormater.getFormatedDate( dueDate )]);
+      DateText = document.createTextNode( textString );
+      DateHtml.appendChild( DateText );
+      HolderBox.appendChild( DateHtml );
+   
+      if (toDoItem.description)
+      {
+	var text = gCalendarBundle.getFormattedString("tooltipTaskDescription", [toDoItem.description]);
+	
+	var lines = text.split("\n");
+	var nbmaxlines = 5 ;
+	var nblines = lines.length ;
+	if( nblines > nbmaxlines ) {
+	  var nblines = nbmaxlines ;
+	  lines[ nblines - 1 ] = "..." ;
+	}
+	
+	for (var i = 0; i < nblines; i++) {
+	  var DescriptionHtml = document.createElement("description");
+	  var DescriptionText = document.createTextNode(lines[i]);
+	  DescriptionHtml.appendChild(DescriptionText);
+	  HolderBox.appendChild(DescriptionHtml);
+	}
+      }
+      
+      return ( HolderBox );
+   } 
+   else
+   {
+      showTooltip = false; //Don't show the tooltip
+   }
+}
 
 /**
 *  Called when a user hovers over an element and the text for the mouse over is changed.
@@ -875,11 +1094,12 @@ function getPreviewTextForRepeatingEvent( calendarEventDisplay )
 	showTooltip = true;
       
    var HolderBox = document.createElement( "vbox" );
+  var textString ;
     
    if (calendarEventDisplay.event.title)
    {
       var TitleHtml = document.createElement( "description" );
-      var TitleText = "Title: "+calendarEventDisplay.event.title;
+      textString = gCalendarBundle.getFormattedString("tooltipTitleElement", [calendarEventDisplay.event.title]);
       
       /*
       if( calendarEventDisplay.event.recurUnits == "years" )
@@ -888,45 +1108,65 @@ function getPreviewTextForRepeatingEvent( calendarEventDisplay )
          
          TitleText = TitleText+" "+getNumberOfRepeatTimes( calendarEventDisplay.event, false );
       }*/
-      var TitleTextNode = document.createTextNode( TitleText );
+      var TitleTextNode = document.createTextNode( textString );
       TitleHtml.appendChild( TitleTextNode );
       HolderBox.appendChild( TitleHtml );
    }
 
-   var DateHtml = document.createElement( "description" );
    var startDate = new Date( calendarEventDisplay.displayDate );
-   var DateText = document.createTextNode( "Start: "+gCalendarWindow.dateFormater.getFormatedDate( startDate )+" "+gCalendarWindow.dateFormater.getFormatedTime( startDate ) );
+   var endDate = new Date( calendarEventDisplay.displayEndDate );
+
+   var DateHtml = document.createElement( "description" );
+   if (!calendarEventDisplay.event.allDay) {
+     textString = gCalendarBundle.getFormattedString("tooltipEventStart", 
+						  ["",
+					           gCalendarWindow.dateFormater.getFormatedTime( startDate )]);
+   
+     var DateText = document.createTextNode( textString );
    DateHtml.appendChild( DateText );
    HolderBox.appendChild( DateHtml );
 
-   if( calendarEventDisplay.recur == true )
-   {
-      DateHtml = document.createElement( "description" );
-      DateText = document.createTextNode( "This is a repeating event." );
-      DateHtml.appendChild( DateText );
-      HolderBox.appendChild( DateHtml );
+   DateHtml = document.createElement( "description" );
+     textString = gCalendarBundle.getFormattedString("tooltipEventEnd", 
+						  ["",
+						   gCalendarWindow.dateFormater.getFormatedTime( endDate )]);
+
+     DateText = document.createTextNode( textString );
+   DateHtml.appendChild( DateText );
+   HolderBox.appendChild( DateHtml );
    }
-   else
+   
+   if (calendarEventDisplay.event.location)
    {
-      DateHtml = document.createElement( "description" );
-      startDate = new Date( calendarEventDisplay.event.end.getTime() );
-      DateText = document.createTextNode( "End: "+gCalendarWindow.dateFormater.getFormatedDate( startDate )+" "+gCalendarWindow.dateFormater.getFormatedTime( startDate ) );
-      DateHtml.appendChild( DateText );
-      HolderBox.appendChild( DateHtml );
+      var LocationHtml = document.createElement( "description" );
+      textString = gCalendarBundle.getFormattedString("tooltipEventLocation", [calendarEventDisplay.event.location]);
+      var LocationText = document.createTextNode( textString );
+      LocationHtml.appendChild( LocationText );
+      HolderBox.appendChild( LocationHtml );
    }
 
    if (calendarEventDisplay.event.description)
    {
-      var DescriptionHtml = document.createElement( "description" );
-      var DescriptionText = document.createTextNode( "Description: "+calendarEventDisplay.event.description );
-      DescriptionHtml.appendChild( DescriptionText );
-      HolderBox.appendChild( DescriptionHtml );
+     textString = gCalendarBundle.getFormattedString("tooltipEventDescription", [calendarEventDisplay.event.description]);
+     var lines = textString.split("\n");
+     var nbmaxlines = 5 ;
+     var nblines = lines.length ;
+     if( nblines > nbmaxlines ) {
+       var nblines = nbmaxlines ;
+       lines[ nblines - 1 ] = "..." ;
+     }
+  
+     for (var i = 0; i < nblines; i++) {
+       var DescriptionHtml = document.createElement("description");
+       var DescriptionText = document.createTextNode(lines[i]);
+       DescriptionHtml.appendChild(DescriptionText);
+       HolderBox.appendChild(DescriptionHtml);
+     }
    }
 
    return ( HolderBox );
 }
 
-
 function getNumberOfRepeatTimes( Event, DateToCompare )
 {
    if( !DateToCompare )
@@ -948,7 +1188,7 @@ function getNumberOfRepeatTimes( Event, DateToCompare )
 
 function reloadApplication()
 {
-	gCalendarWindow.eventSource.calendarManager.refreshAllRemoteCalendars();
+	gEventSource.calendarManager.refreshAllRemoteCalendars();
 }
 
 
@@ -996,10 +1236,84 @@ function printEventArray( calendarEventArray, stylesheetName )
 
 function print()
 {
-   printEventArray( gCalendarWindow.EventSelection.selectedEvents, "chrome://calendar/content/converters/sortEvents.xsl" );
+   var args = new Object();
+
+   args.eventSource = gEventSource;
+   args.selectedEvents = gCalendarWindow.EventSelection.selectedEvents ;
+   args.selectedDate=gNewDateVariable = gCalendarWindow.getSelectedDate();
+
+   var Offset = getIntPref(gCalendarWindow.calendarPreferences.calendarPref, 
+			   "week.start", 
+			   gCalendarBundle.getString("defaultWeekStart" ) );
+   var WeeksInView = getIntPref(gCalendarWindow.calendarPreferences.calendarPref, 
+				"weeks.inview", 
+				gCalendarBundle.getString("defaultWeeksInView" ) );
+   WeeksInView = ( WeeksInView >= 6 ) ? 6 : WeeksInView ;
+
+   var PreviousWeeksInView = getIntPref(gCalendarWindow.calendarPreferences.calendarPref, 
+					"previousweeks.inview", 
+					gCalendarBundle.getString("defaultPreviousWeeksInView" ) );
+   PreviousWeeksInView = ( PreviousWeeksInView >= WeeksInView - 1 ) ? WeeksInView - 1 : PreviousWeeksInView ;
+
+   args.startOfWeek=Offset;
+   args.weeksInView=WeeksInView;
+   args.prevWeeksInView=PreviousWeeksInView;
+
+   window.openDialog("chrome://calendar/content/printDialog.xul","printdialog","chrome",args);
 }
 
 
+function publishEntireCalendar()
+{
+   var args = new Object();
+   
+   args.onOk =  self.publishEntireCalendarDialogResponse;
+   var name = gCalendarWindow.calendarManager.getSelectedCalendarId();
+   var node = gCalendarWindow.calendarManager.rdf.getNode( name );
+
+   var remotePath = node.getAttribute( "http://home.netscape.com/NC-rdf#remotePath" );
+   
+   if( remotePath != "" && remotePath != null )
+   {
+      var publishObject = new Object( );
+      publishObject.username = node.getAttribute( "http://home.netscape.com/NC-rdf#username" );
+      
+      publishObject.remotePath = remotePath;
+      
+      publishObject.password = node.getAttribute( "http://home.netscape.com/NC-rdf#password" );
+      args.publishObject = publishObject;
+   }
+   
+   openDialog("chrome://calendar/content/publishDialog.xul", "caPublishEvents", "chrome,modal", args );
+}
+
+function publishEntireCalendarDialogResponse( CalendarPublishObject )
+{
+   //update the calendar object with the publish information
+   var name = gCalendarWindow.calendarManager.getSelectedCalendarId();
+   
+   //get the node
+   var node = gCalendarWindow.calendarManager.rdf.getNode( name );
+   
+   node.setAttribute("http://home.netscape.com/NC-rdf#username", CalendarPublishObject.username);
+   
+   node.setAttribute("http://home.netscape.com/NC-rdf#password", CalendarPublishObject.password);
+   
+   node.setAttribute( "http://home.netscape.com/NC-rdf#remotePath", CalendarPublishObject.remotePath );
+   
+   node.setAttribute("http://home.netscape.com/NC-rdf#publishAutomatically", "false");
+
+   gCalendarWindow.calendarManager.rdf.flush();
+      
+   calendarUploadFile(node.getAttribute( "http://home.netscape.com/NC-rdf#path" ), 
+                      CalendarPublishObject.remotePath, 
+                      CalendarPublishObject.username, 
+                      CalendarPublishObject.password, 
+                      "text/calendar");
+
+   return( false );
+}
+
 function publishCalendarData()
 {
    var args = new Object();
@@ -1013,7 +1327,7 @@ function publishCalendarDataDialogResponse( CalendarPublishObject )
 {
    var calendarString = eventArrayToICalString( gCalendarWindow.EventSelection.selectedEvents );
    
-   calendarPublish(calendarString, CalendarPublishObject.url, CalendarPublishObject.remotePath, CalendarPublishObject.username, CalendarPublishObject.password, "text/calendar");
+   calendarPublish(calendarString, CalendarPublishObject.remotePath, CalendarPublishObject.username, CalendarPublishObject.password, "text/calendar");
 }
 
 /*
@@ -1074,6 +1388,7 @@ function GetUnicharPref(prefObj, prefName, defaultValue)
     }
     catch(e)
     {
+      SetUnicharPref(prefObj, prefName, defaultValue);
         return defaultValue;
     }
 }
@@ -1088,3 +1403,52 @@ function SetUnicharPref(aPrefObj, aPrefName, aPrefValue)
     }
     catch(e) {}
 }
+
+/* Change the only-workday checkbox */
+function changeOnlyWorkdayCheckbox( menuindex ) {
+  var check = document.getElementById( "only-workday-checkbox-" + menuindex ).getAttribute("checked") ;
+  switch(menuindex){
+  case 1:
+    var changemenu = 2 ;
+    break;
+  case 2:
+    var changemenu = 1 ;
+    break;
+  default:
+    return(false);
+  }
+  if(check == "true") {
+    document.getElementById( "only-workday-checkbox-" + changemenu ).setAttribute("checked","true");
+    gOnlyWorkdayChecked = "true" ;
+  }
+  else {
+    document.getElementById( "only-workday-checkbox-" + changemenu ).removeAttribute("checked");
+    gOnlyWorkdayChecked = "false" ;
+  }
+  gCalendarWindow.currentView.refreshDisplay( );
+  gCalendarWindow.currentView.refreshEvents( );
+}
+
+/* Change the display-todo-inview checkbox */
+function changeDisplayToDoInViewCheckbox( menuindex ) {
+  var check = document.getElementById( "display-todo-inview-checkbox-" + menuindex ).getAttribute("checked") ;
+  switch(menuindex){
+  case 1:
+    var changemenu = 2 ;
+    break;
+  case 2:
+    var changemenu = 1 ;
+    break;
+  default:
+    return(false);
+  }
+  if(check == "true") {
+    document.getElementById( "display-todo-inview-checkbox-" + changemenu ).setAttribute("checked","true");
+    gDisplayToDoInViewChecked = "true" ;
+  }
+  else {
+    document.getElementById( "display-todo-inview-checkbox-" + changemenu ).removeAttribute("checked");
+    gDisplayToDoInViewChecked = "false" ;
+  }
+  gCalendarWindow.currentView.refreshEvents( );
+}
