@@ -22,7 +22,6 @@ END_USAGE
 }
 
 my %map = (
-  'absyncsvc',          'mailnews/absync',
   'access-proxy',       'extensions/access-builtin/accessproxy',
   'accessibility',      'accessible',
   'addrbook',           'mailnews/addrbook',
@@ -66,9 +65,7 @@ my %map = (
   'imgicon',            'modules/libpr0n/decoders/icon',
   'imgjpeg',            'modules/libpr0n/decoders/jpeg',
   'imglib2',            'modules/libpr0n/public  modules/libpr0n/src',
-  'imgmng',             'modules/libpr0n/decoders/mng',
   'imgpng',             'modules/libpr0n/decoders/png',
-  'imgppm',             'modules/libpr0n/decoders/ppm',
   'imgxbm',             'modules/libpr0n/decoders/xbm',
   'impComm4xMail',      'mailnews/import/comm4x',
   'impEudra',           'mailnews/import/eudora',
@@ -93,7 +90,6 @@ my %map = (
   'liveconnect',        'js/src/liveconnect',
   'locale',             'intl/locale',
   'lwbrk',              'intl/lwbrk',
-  'mai',                'widget/src/gtk2/mai',
   'mailnews',           'mailnews',
   'MapiProxy',          'mailnews/mapi/mapihook/build',
   'mfcEmbed',           'embedding/tests/winEmbed',
@@ -143,7 +139,6 @@ my %map = (
   'rdf',                'rdf',
   'rdfutil',            'rdf/util',
   'rdfldapds',          'directory/xpcom/tests',
-  'regviewer',          'xpfe/components/regviewer',
   'SanePlugin',         'modules/plugin/samples/SanePlugin',
   'shistory',           'xpfe/components/shistory',
   'sidebar',            'xpfe/components/sidebar',
