@@ -44,6 +44,7 @@
 #include "nsIJSContextStack.h"
 #include "nsIScriptContext.h"
 #include "nsCOMArray.h"
+#include "nsIStatefulFrame.h"
 
 class nsIScriptGlobalObject;
 class nsIXPConnect;
@@ -54,7 +55,8 @@ class nsINameSpaceManager;
 class nsIScriptSecurityManager;
 class nsIThreadJSContextStack;
 class nsIParserService;
-
+class nsIIOService;
+class nsIURI;
 
 class nsContentUtils
 {
@@ -198,6 +200,9 @@ public:
   static const nsDependentSubstring TrimCharsInSet(const char* aSet,
                                                    const nsAString& aValue);
 
+  static const nsDependentSubstring TrimWhitespace(const nsAString& aStr,
+                                                   PRBool aTrimTrailing = PR_TRUE);
+
   static void Shutdown();
   
   /**
@@ -237,10 +242,32 @@ public:
   
   static nsINameSpaceManager* GetNSManagerWeakRef()
   {
-      return sNameSpaceManager;
+    return sNameSpaceManager;
   };
 
+  static nsIIOService* GetIOServiceWeakRef()
+  {
+    return sIOService;
+  };
+  
+  static nsresult GenerateStateKey(nsIContent* aContent,
+                                   nsIStatefulFrame::SpecialStateID aID,
+                                   nsACString& aKey);
+
+  /**
+   * Create a new nsIURI from aSpec, using aBaseURI as the base.  The
+   * origin charset of the new nsIURI will be the document charset of
+   * aDocument.
+   */
+  static nsresult NewURIWithDocumentCharset(nsIURI** aResult,
+                                            const nsAString& aSpec,
+                                            nsIDocument* aDocument,
+                                            nsIURI* aBaseURI);
 private:
+  static nsresult GetDocumentAndPrincipal(nsIDOMNode* aNode,
+                                          nsIDocument** aDocument,
+                                          nsIPrincipal** aPrincipal);
+
   static nsresult doReparentContentWrapper(nsIContent *aChild,
                                            nsIDocument *aNewDocument,
                                            nsIDocument *aOldDocument,
@@ -259,6 +286,8 @@ private:
   static nsIParserService *sParserService;
 
   static nsINameSpaceManager *sNameSpaceManager;
+
+  static nsIIOService *sIOService;
 };
 
 
