@@ -65,7 +65,7 @@ public:
   NS_DECL_ISUPPORTS
 
   // nsIContentSerializer
-  NS_IMETHOD Init(PRUint32 flags, PRUint32 dummy, nsIAtom* aCharSet, 
+  NS_IMETHOD Init(PRUint32 flags, PRUint32 dummy, const char* aCharSet, 
                   PRBool aIsCopying);
 
   NS_IMETHOD AppendText(nsIDOMText* aText, PRInt32 aStartOffset,
@@ -100,13 +100,14 @@ public:
   NS_IMETHOD SetParser(nsIParser* aParser) { return NS_OK; }
   NS_IMETHOD OpenContainer(const nsIParserNode& aNode);
   NS_IMETHOD CloseContainer(const nsHTMLTag aTag);
+  NS_IMETHOD AddHeadContent(const nsIParserNode& aNode);
   NS_IMETHOD AddLeaf(const nsIParserNode& aNode);
   NS_IMETHOD AddComment(const nsIParserNode& aNode) { return NS_OK; }
   NS_IMETHOD AddProcessingInstruction(const nsIParserNode& aNode)
                                                     { return NS_OK; }
   NS_IMETHOD AddDocTypeDecl(const nsIParserNode& aNode);
   NS_IMETHOD FlushPendingNotifications() { return NS_OK; }
-  NS_IMETHOD SetDocumentCharset(nsAString& aCharset);
+  NS_IMETHOD SetDocumentCharset(nsACString& aCharset);
   NS_IMETHOD NotifyTagObservers(nsIParserNode* aNode) { return NS_OK; }
 
   // nsIHTMLContentSink
@@ -123,9 +124,8 @@ public:
   NS_IMETHOD CloseMap();
   NS_IMETHOD OpenFrameset(const nsIParserNode& aNode);
   NS_IMETHOD CloseFrameset();
-  NS_IMETHOD GetPref(PRInt32 aTag,PRBool& aPref);
+  NS_IMETHOD IsEnabled(PRInt32 aTag, PRBool* aReturn);
   NS_IMETHOD_(PRBool) IsFormOnStack() { return PR_FALSE; }
-  NS_IMETHOD DoFragment(PRBool aFlag);
   NS_IMETHOD BeginContext(PRInt32 aPosition) { return NS_OK; }
   NS_IMETHOD EndContext(PRInt32 aPosition) { return NS_OK; }
   NS_IMETHOD WillProcessTokens(void) { return NS_OK; }
@@ -163,7 +163,7 @@ protected:
   nsCOMPtr<nsIParserService>   mParserService;
 };
 
-extern nsresult NS_NewSanitizingHTMLSerializer(
-                                         nsIContentSerializer** aSerializer);
+nsresult
+NS_NewSanitizingHTMLSerializer(nsIContentSerializer** aSerializer);
 
 #endif
