@@ -135,7 +135,7 @@ ContentToParentOffset(nsIContent *aContent, nsIDOMNode **aParent, PRInt32 *aOffs
     return;
 
   nsCOMPtr<nsIContent> parent;
-  nsresult rv = aContent->GetParent(*getter_AddRefs(parent));
+  nsresult rv = aContent->GetParent(getter_AddRefs(parent));
 
   if (NS_FAILED(rv) || !parent)
     return;
@@ -160,6 +160,20 @@ ContentIsInTraversalRange(nsIContent *aContent,   PRBool aIsPreMode,
   if (!aStartNode || !aEndNode || !aContent)
     return PR_FALSE;
 
+  nsCOMPtr<nsIDOMCharacterData> cData(do_QueryInterface(aContent));
+
+  if (cData)
+  {
+    // If a chardata node contains an end point of the traversal range,
+    // it is always in the traversal range.
+
+    nsCOMPtr<nsIContent> startContent(do_QueryInterface(aStartNode));
+    nsCOMPtr<nsIContent> endContent(do_QueryInterface(aEndNode));
+
+    if (aContent == startContent || aContent == endContent)
+      return PR_TRUE;
+  }
+
   nsCOMPtr<nsIDOMNode> parentNode;
   PRInt32 indx = 0;
 
@@ -284,18 +298,22 @@ public:
 nsresult NS_NewContentIterator(nsIContentIterator** aInstancePtrResult)
 {
   nsContentIterator * iter = new nsContentIterator();
-  if (iter)
-    return iter->QueryInterface(NS_GET_IID(nsIContentIterator), (void**) aInstancePtrResult);
-  return NS_ERROR_OUT_OF_MEMORY;
+  if (!iter) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  return CallQueryInterface(iter, aInstancePtrResult);
 }
 
 
 nsresult NS_NewPreContentIterator(nsIContentIterator** aInstancePtrResult)
 {
   nsContentIterator * iter = new nsPreContentIterator();
-  if (iter)
-    return iter->QueryInterface(NS_GET_IID(nsIContentIterator), (void**) aInstancePtrResult);
-  return NS_ERROR_OUT_OF_MEMORY;
+  if (!iter) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  return CallQueryInterface(iter, aInstancePtrResult);
 }
 
 
@@ -344,7 +362,6 @@ nsContentIterator::nsContentIterator() :
   // don't need to explicitly initialize |nsCOMPtr|s, they will automatically be NULL
   mCachedIndex(0), mIsDone(PR_FALSE), mPre(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 
@@ -461,7 +478,7 @@ nsresult nsContentIterator::Init(nsIDOMRange* aRange)
   nsCOMPtr<nsIContent> cChild;
 
   if (!cData && ContentHasChildren(startCon))
-    startCon->ChildAt(startIndx,*getter_AddRefs(cChild));
+    startCon->ChildAt(startIndx, getter_AddRefs(cChild));
 
   if (!cChild) // no children, must be a text node
   {
@@ -535,7 +552,7 @@ nsresult nsContentIterator::Init(nsIDOMRange* aRange)
   {
     PRInt32 indx = endIndx;
 
-    endCon->ChildAt(--indx, *getter_AddRefs(cChild));
+    endCon->ChildAt(--indx, getter_AddRefs(cChild));
 
     if (!cChild)  // No child at offset!
     {
@@ -592,7 +609,7 @@ nsresult nsContentIterator::RebuildIndexStack()
   current = mCurNode;
   while (current && current != mCommonParent)
   {
-    if (NS_FAILED(current->GetParent(*getter_AddRefs(parent))))
+    if (NS_FAILED(current->GetParent(getter_AddRefs(parent))))
       return NS_ERROR_FAILURE;
     if (!parent || NS_FAILED(parent->IndexOf(current, indx)))
       return NS_ERROR_FAILURE;
@@ -623,7 +640,7 @@ nsCOMPtr<nsIContent> nsContentIterator::GetDeepFirstChild(nsCOMPtr<nsIContent> a
   {  
     nsCOMPtr<nsIContent> cN = aRoot;
     nsCOMPtr<nsIContent> cChild;
-    cN->ChildAt(0,*getter_AddRefs(cChild));
+    cN->ChildAt(0, getter_AddRefs(cChild));
     while ( cChild )
     {
       if (aIndexes)
@@ -632,7 +649,7 @@ nsCOMPtr<nsIContent> nsContentIterator::GetDeepFirstChild(nsCOMPtr<nsIContent> a
         aIndexes->AppendElement(NS_INT32_TO_PTR(0));
       }
       cN = cChild;
-      cN->ChildAt(0,*getter_AddRefs(cChild));
+      cN->ChildAt(0, getter_AddRefs(cChild));
     }
     deepFirstChild = cN;
   }
@@ -654,7 +671,7 @@ nsCOMPtr<nsIContent> nsContentIterator::GetDeepLastChild(nsCOMPtr<nsIContent> aR
 
     while ( numChildren )
     {
-      cN->ChildAt(--numChildren,*getter_AddRefs(cChild));
+      cN->ChildAt(--numChildren, getter_AddRefs(cChild));
       if (cChild)
       {
         if (aIndexes)
@@ -690,7 +707,7 @@ nsresult nsContentIterator::GetNextSibling(nsCOMPtr<nsIContent> aNode,
   nsCOMPtr<nsIContent> parent;
   PRInt32              indx;
   
-  if (NS_FAILED(aNode->GetParent(*getter_AddRefs(parent))) || !parent)
+  if (NS_FAILED(aNode->GetParent(getter_AddRefs(parent))) || !parent)
     return NS_ERROR_FAILURE;
 
   if (aIndexes)
@@ -704,7 +721,7 @@ nsresult nsContentIterator::GetNextSibling(nsCOMPtr<nsIContent> aNode,
   // reverify that the index of the current node hasn't changed.
   // not super cheap, but a lot cheaper than IndexOf(), and still O(1).
   // ignore result this time - the index may now be out of range.
-  (void) parent->ChildAt(indx, *getter_AddRefs(sib)); // sib defaults to nsnull
+  (void) parent->ChildAt(indx, getter_AddRefs(sib)); // sib defaults to nsnull
   if (sib != aNode)
   {
     // someone changed our index - find the new index the painful way
@@ -713,7 +730,7 @@ nsresult nsContentIterator::GetNextSibling(nsCOMPtr<nsIContent> aNode,
   }
 
   // indx is now canonically correct
-  if (NS_SUCCEEDED(parent->ChildAt(++indx, *getter_AddRefs(sib))) && sib)
+  if (NS_SUCCEEDED(parent->ChildAt(++indx, getter_AddRefs(sib))) && sib)
   {
     *aSibling = sib;
     // update index cache
@@ -758,7 +775,7 @@ nsresult nsContentIterator::GetPrevSibling(nsCOMPtr<nsIContent> aNode,
   nsCOMPtr<nsIContent> parent;
   PRInt32              indx;
   
-  if (NS_FAILED(aNode->GetParent(*getter_AddRefs(parent))) || !parent)
+  if (NS_FAILED(aNode->GetParent(getter_AddRefs(parent))) || !parent)
     return NS_ERROR_FAILURE;
 
   if (aIndexes)
@@ -771,7 +788,7 @@ nsresult nsContentIterator::GetPrevSibling(nsCOMPtr<nsIContent> aNode,
   
   // reverify that the index of the current node hasn't changed
   // ignore result this time - the index may now be out of range.
-  (void) parent->ChildAt(indx, *getter_AddRefs(sib)); // sib defaults to nsnull
+  (void) parent->ChildAt(indx, getter_AddRefs(sib)); // sib defaults to nsnull
   if (sib != aNode)
   {
     // someone changed our index - find the new index the painful way
@@ -781,7 +798,7 @@ nsresult nsContentIterator::GetPrevSibling(nsCOMPtr<nsIContent> aNode,
 
   // indx is now canonically correct
   if (indx > 0 && 
-      NS_SUCCEEDED(parent->ChildAt(--indx, *getter_AddRefs(sib))) && sib)
+      NS_SUCCEEDED(parent->ChildAt(--indx, getter_AddRefs(sib))) && sib)
   {
     *aSibling = sib;
     // update index cache
@@ -826,7 +843,7 @@ nsresult nsContentIterator::NextNode(nsCOMPtr<nsIContent> *ioNextNode, nsVoidArr
     // if it has children then next node is first child
     if (numChildren)
     {
-      if (NS_FAILED(cN->ChildAt(0,*getter_AddRefs(cFirstChild))))
+      if (NS_FAILED(cN->ChildAt(0, getter_AddRefs(cFirstChild))))
         return NS_ERROR_FAILURE;
       if (!cFirstChild)
         return NS_ERROR_FAILURE;
@@ -853,7 +870,7 @@ nsresult nsContentIterator::NextNode(nsCOMPtr<nsIContent> *ioNextNode, nsVoidArr
     PRInt32              indx;
   
     // get next sibling if there is one
-    if (NS_FAILED(cN->GetParent(*getter_AddRefs(parent))))
+    if (NS_FAILED(cN->GetParent(getter_AddRefs(parent))))
       return NS_ERROR_FAILURE;
       
     // get the cached index
@@ -869,7 +886,7 @@ nsresult nsContentIterator::NextNode(nsCOMPtr<nsIContent> *ioNextNode, nsVoidArr
     // not super cheap, but a lot cheaper than IndexOf(), and still O(1).
     // ignore result this time - the index may now be out of range.
     if (indx >= 0)
-      (void) parent->ChildAt(indx, *getter_AddRefs(cSibling));
+      (void) parent->ChildAt(indx, getter_AddRefs(cSibling));
     if (cSibling != cN)
     {
       // someone changed our index - find the new index the painful way
@@ -878,7 +895,7 @@ nsresult nsContentIterator::NextNode(nsCOMPtr<nsIContent> *ioNextNode, nsVoidArr
     }
 
     // indx is now canonically correct
-    if (NS_SUCCEEDED(parent->ChildAt(++indx,*getter_AddRefs(cSibling))) && cSibling)
+    if (NS_SUCCEEDED(parent->ChildAt(++indx, getter_AddRefs(cSibling))) && cSibling)
     {
       // update cache
       if (aIndexes)
@@ -923,7 +940,7 @@ nsresult nsContentIterator::PrevNode(nsCOMPtr<nsIContent> *ioNextNode, nsVoidArr
     PRInt32              indx;
   
     // get prev sibling if there is one
-    if (NS_FAILED(cN->GetParent(*getter_AddRefs(parent))))
+    if (NS_FAILED(cN->GetParent(getter_AddRefs(parent))))
       return NS_ERROR_FAILURE;
 
     // get the cached index
@@ -939,7 +956,7 @@ nsresult nsContentIterator::PrevNode(nsCOMPtr<nsIContent> *ioNextNode, nsVoidArr
     // not super cheap, but a lot cheaper than IndexOf(), and still O(1).
     // ignore result this time - the index may now be out of range.
     if (indx >= 0)
-      (void) parent->ChildAt(indx, *getter_AddRefs(cSibling));
+      (void) parent->ChildAt(indx, getter_AddRefs(cSibling));
     if (cSibling != cN)
     {
       // someone changed our index - find the new index the painful way
@@ -948,7 +965,7 @@ nsresult nsContentIterator::PrevNode(nsCOMPtr<nsIContent> *ioNextNode, nsVoidArr
     }
 
     // indx is now canonically correct
-    if (indx && NS_SUCCEEDED(parent->ChildAt(--indx,*getter_AddRefs(cSibling))) && cSibling)
+    if (indx && NS_SUCCEEDED(parent->ChildAt(--indx, getter_AddRefs(cSibling))) && cSibling)
     {
       // update cache
       if (aIndexes)
@@ -983,7 +1000,7 @@ nsresult nsContentIterator::PrevNode(nsCOMPtr<nsIContent> *ioNextNode, nsVoidArr
     // if it has children then prev node is last child
     if (numChildren)
     {
-      if (NS_FAILED(cN->ChildAt(--numChildren,*getter_AddRefs(cLastChild))))
+      if (NS_FAILED(cN->ChildAt(--numChildren, getter_AddRefs(cLastChild))))
         return NS_ERROR_FAILURE;
       if (!cLastChild)
         return NS_ERROR_FAILURE;
@@ -1156,7 +1173,7 @@ nsresult nsContentIterator::PositionAt(nsIContent* aCurNode)
     // Insert at head since we're walking up
     oldParentStack.InsertElementAt(tempNode,0);
 
-    if (NS_FAILED(tempNode->GetParent(*getter_AddRefs(parent))))
+    if (NS_FAILED(tempNode->GetParent(getter_AddRefs(parent))))
       return NS_ERROR_FAILURE;
 
     if (!parent)  // this node has no parent, and thus no index
@@ -1179,7 +1196,7 @@ nsresult nsContentIterator::PositionAt(nsIContent* aCurNode)
   {
     PRInt32 indx;
 
-    if (NS_FAILED(newCurNode->GetParent(*getter_AddRefs(parent))))
+    if (NS_FAILED(newCurNode->GetParent(getter_AddRefs(parent))))
       return NS_ERROR_FAILURE;
 
     if (!parent)  // this node has no parent, and thus no index
@@ -1218,11 +1235,11 @@ nsresult nsContentIterator::PositionAt(nsIContent* aCurNode)
 
 nsresult nsContentIterator::CurrentNode(nsIContent **aNode)
 {
-  if (!mCurNode) 
+  if (!mCurNode || mIsDone) {
     return NS_ERROR_FAILURE;
-  if (mIsDone) 
-    return NS_ERROR_FAILURE;
-  return mCurNode->QueryInterface(NS_GET_IID(nsIContent), (void**) aNode);
+  }
+
+  return CallQueryInterface(mCurNode, aNode);
 }
 
 
@@ -1298,9 +1315,11 @@ nsresult NS_NewContentSubtreeIterator(nsIContentIterator** aInstancePtrResult);
 nsresult NS_NewContentSubtreeIterator(nsIContentIterator** aInstancePtrResult)
 {
   nsContentIterator * iter = new nsContentSubtreeIterator();
-  if (iter)
-    return iter->QueryInterface(NS_GET_IID(nsIContentIterator), (void**) aInstancePtrResult);
-  return NS_ERROR_OUT_OF_MEMORY;
+  if (!iter) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  return CallQueryInterface(iter, aInstancePtrResult);
 }
 
 
@@ -1359,7 +1378,7 @@ nsresult nsContentSubtreeIterator::Init(nsIDOMRange* aRange)
   // short circuit when start node == end node
   if (startParent == endParent)
   {
-    cStartP->ChildAt(0,*getter_AddRefs(cChild));
+    cStartP->ChildAt(0, getter_AddRefs(cChild));
   
     if (!cChild) // no children, must be a text node or empty container
     {
@@ -1422,8 +1441,10 @@ nsresult nsContentSubtreeIterator::Init(nsIDOMRange* aRange)
   // does not fully contain any node.
   
   PRBool nodeBefore, nodeAfter;
-  if (NS_FAILED(CompareNodeToRange(firstCandidate, aRange, &nodeBefore, &nodeAfter)))
+  if (NS_FAILED(nsRange::CompareNodeToRange(firstCandidate, aRange,
+                                            &nodeBefore, &nodeAfter)))
     return NS_ERROR_FAILURE;
+
   if (nodeBefore || nodeAfter)
   {
     MakeEmpty();
@@ -1485,8 +1506,10 @@ nsresult nsContentSubtreeIterator::Init(nsIDOMRange* aRange)
   // is indeed contained.  Else we have a range that
   // does not fully contain any node.
   
-  if (NS_FAILED(CompareNodeToRange(lastCandidate, aRange, &nodeBefore, &nodeAfter)))
+  if (NS_FAILED(nsRange::CompareNodeToRange(lastCandidate, aRange, &nodeBefore,
+                                            &nodeAfter)))
     return NS_ERROR_FAILURE;
+
   if (nodeBefore || nodeAfter)
   {
     MakeEmpty();
@@ -1559,7 +1582,7 @@ nsresult nsContentSubtreeIterator::Next()
     // as long as we are finding ancestors of the endpoint of the range,
     // dive down into their children
     nsCOMPtr<nsIContent> cChild;
-    nextNode->ChildAt(0,*getter_AddRefs(cChild));
+    nextNode->ChildAt(0, getter_AddRefs(cChild));
     if (!cChild) return NS_ERROR_NULL_POINTER;
     // should be impossible to get a null pointer.  If we went all the 
     // down the child chain to the bottom without finding an interior node, 
@@ -1616,15 +1639,17 @@ nsresult nsContentSubtreeIterator::GetTopAncestorInRange(
   
   // sanity check: aNode is itself in the range
   PRBool nodeBefore, nodeAfter;
-  if (NS_FAILED(CompareNodeToRange(aNode, mRange, &nodeBefore, &nodeAfter)))
+  if (NS_FAILED(nsRange::CompareNodeToRange(aNode, mRange, &nodeBefore,
+                                            &nodeAfter)))
     return NS_ERROR_FAILURE;
+
   if (nodeBefore || nodeAfter)
     return NS_ERROR_FAILURE;
   
   nsCOMPtr<nsIContent> parent, tmp;
   while (aNode)
   {
-    if (NS_FAILED(aNode->GetParent(*getter_AddRefs(parent))))
+    if (NS_FAILED(aNode->GetParent(getter_AddRefs(parent))))
       return NS_ERROR_FAILURE;
     if (!parent)
     {
@@ -1635,8 +1660,10 @@ nsresult nsContentSubtreeIterator::GetTopAncestorInRange(
       }
       else return NS_ERROR_FAILURE;
     }
-    if (NS_FAILED(CompareNodeToRange(parent, mRange, &nodeBefore, &nodeAfter)))
+    if (NS_FAILED(nsRange::CompareNodeToRange(parent, mRange, &nodeBefore,
+                                              &nodeAfter)))
       return NS_ERROR_FAILURE;
+
     if (nodeBefore || nodeAfter)
     {
       *outAnestor = aNode;
