@@ -55,7 +55,6 @@ nsDOMAttribute::nsDOMAttribute(nsIContent* aContent, nsINodeInfo *aNodeInfo,
 {
   NS_ABORT_IF_FALSE(mNodeInfo, "We must get a nodeinfo here!");
 
-  NS_INIT_ISUPPORTS();
 
   // We don't add a reference to our content. It will tell us
   // to drop our reference when it goes away.
@@ -71,7 +70,7 @@ nsDOMAttribute::~nsDOMAttribute()
 // QueryInterface implementation for nsDOMAttribute
 NS_INTERFACE_MAP_BEGIN(nsDOMAttribute)
   NS_INTERFACE_MAP_ENTRY(nsIDOMAttr)
-  NS_INTERFACE_MAP_ENTRY(nsIDOMAttributePrivate)
+  NS_INTERFACE_MAP_ENTRY(nsIAttribute)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNode)
   NS_INTERFACE_MAP_ENTRY(nsIDOM3Node)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDOMAttr)
@@ -108,6 +107,14 @@ nsDOMAttribute::GetContent(nsIContent** aContent)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsDOMAttribute::GetNodeInfo(nsINodeInfo** aNodeInfo)
+{
+  NS_IF_ADDREF(*aNodeInfo = mNodeInfo);
+
+  return NS_OK;
+}
+
 nsresult
 nsDOMAttribute::GetName(nsAString& aName)
 {
@@ -121,23 +128,21 @@ nsDOMAttribute::GetValue(nsAString& aValue)
 {
   NS_ENSURE_TRUE(mNodeInfo, NS_ERROR_FAILURE);
 
-  nsresult result = NS_OK;
   if (mContent) {
-    nsresult attrResult;
-    PRInt32 nameSpaceID;
-    nsCOMPtr<nsIAtom> name;
-
-    mNodeInfo->GetNameAtom(*getter_AddRefs(name));
-    mNodeInfo->GetNamespaceID(nameSpaceID);
+    nsCOMPtr<nsIAtom> name = mNodeInfo->GetNameAtom();
+    PRInt32 nameSpaceID = mNodeInfo->GetNamespaceID();
 
     nsAutoString tmpValue;
-    attrResult = mContent->GetAttr(nameSpaceID, name, tmpValue);
+    nsresult attrResult = mContent->GetAttr(nameSpaceID, name,
+                                            tmpValue);
     if (NS_CONTENT_ATTR_NOT_THERE != attrResult) {
       mValue = tmpValue;
     }
   }
-  aValue=mValue;
-  return result;
+
+  aValue = mValue;
+
+  return NS_OK;
 }
 
 nsresult
@@ -160,28 +165,17 @@ nsDOMAttribute::GetSpecified(PRBool* aSpecified)
   NS_ENSURE_TRUE(mNodeInfo, NS_ERROR_FAILURE);
   NS_ENSURE_ARG_POINTER(aSpecified);
 
-  nsresult result = NS_OK;
-  if (nsnull == mContent) {
+  if (!mContent) {
     *aSpecified = PR_FALSE;
-  } else {
-    nsAutoString value;
-    nsresult attrResult;
-    PRInt32 nameSpaceID;
-    nsCOMPtr<nsIAtom> name;
-
-    mNodeInfo->GetNameAtom(*getter_AddRefs(name));
-    mNodeInfo->GetNamespaceID(nameSpaceID);
-
-    attrResult = mContent->GetAttr(nameSpaceID, name, value);
-    if (NS_CONTENT_ATTR_HAS_VALUE == attrResult) {
-      *aSpecified = PR_TRUE;
-    }
-    else {
-      *aSpecified = PR_FALSE;
-    }
+    return NS_OK;
   }
 
-  return result;
+  nsCOMPtr<nsIAtom> name = mNodeInfo->GetNameAtom();
+  PRInt32 nameSpaceID = mNodeInfo->GetNamespaceID();
+
+  *aSpecified = mContent->HasAttr(nameSpaceID, name);
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -190,8 +184,7 @@ nsDOMAttribute::GetOwnerElement(nsIDOMElement** aOwnerElement)
   NS_ENSURE_ARG_POINTER(aOwnerElement);
 
   if (mContent) {
-    return mContent->QueryInterface(NS_GET_IID(nsIDOMElement),
-                                    (void **)aOwnerElement);
+    return CallQueryInterface(mContent, aOwnerElement);
   }
 
   *aOwnerElement = nsnull;
@@ -292,13 +285,13 @@ nsDOMAttribute::GetFirstChild(nsIDOMNode** aFirstChild)
       nsCOMPtr<nsITextContent> content;
 
       result = NS_NewTextNode(getter_AddRefs(content));
-      if (NS_OK != result) {
+      if (NS_FAILED(result)) {
         return result;
       }
-      result = content->QueryInterface(NS_GET_IID(nsIDOMText), (void**)&mChild);
+      result = CallQueryInterface(content, &mChild);
     }
     mChild->SetData(value);
-    result = mChild->QueryInterface(NS_GET_IID(nsIDOMNode), (void**)aFirstChild);
+    result = CallQueryInterface(mChild, aFirstChild);
   }
   else {
     *aFirstChild = nsnull;
@@ -369,13 +362,10 @@ nsDOMAttribute::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
   nsDOMAttribute* newAttr;
 
   if (mContent) {
-    nsAutoString value;
-    PRInt32 nameSpaceID;
-    nsCOMPtr<nsIAtom> name;
-
-    mNodeInfo->GetNameAtom(*getter_AddRefs(name));
-    mNodeInfo->GetNamespaceID(nameSpaceID);
+    nsCOMPtr<nsIAtom> name = mNodeInfo->GetNameAtom();
+    PRInt32 nameSpaceID = mNodeInfo->GetNamespaceID();
   
+    nsAutoString value;
     mContent->GetAttr(nameSpaceID, name, value);
     newAttr = new nsDOMAttribute(nsnull, mNodeInfo, value); 
   }
@@ -387,7 +377,7 @@ nsDOMAttribute::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  return newAttr->QueryInterface(NS_GET_IID(nsIDOMNode), (void**)aReturn);
+  return CallQueryInterface(newAttr, aReturn);
 }
 
 NS_IMETHODIMP 
@@ -395,11 +385,9 @@ nsDOMAttribute::GetOwnerDocument(nsIDOMDocument** aOwnerDocument)
 {
   nsresult result = NS_OK;
   if (mContent) {
-    nsIDOMNode* node;
-    result = mContent->QueryInterface(NS_GET_IID(nsIDOMNode), (void**)&node);
+    nsCOMPtr<nsIDOMNode> node = do_QueryInterface(mContent, &result);
     if (NS_SUCCEEDED(result)) {
       result = node->GetOwnerDocument(aOwnerDocument);
-      NS_RELEASE(node);
     }
   }
   else {
@@ -431,22 +419,19 @@ nsDOMAttribute::SetPrefix(const nsAString& aPrefix)
   NS_ENSURE_TRUE(mNodeInfo, NS_ERROR_FAILURE);
   nsCOMPtr<nsINodeInfo> newNodeInfo;
   nsCOMPtr<nsIAtom> prefix;
-  nsresult rv = NS_OK;
 
-  if (!aPrefix.IsEmpty() && !DOMStringIsNull(aPrefix))
-    prefix = dont_AddRef(NS_NewAtom(aPrefix));
+  if (!aPrefix.IsEmpty() && !DOMStringIsNull(aPrefix)) {
+    prefix = do_GetAtom(aPrefix);
+  }
 
-  rv = mNodeInfo->PrefixChanged(prefix, *getter_AddRefs(newNodeInfo));
+  nsresult rv = mNodeInfo->PrefixChanged(prefix, getter_AddRefs(newNodeInfo));
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (mContent) {
-    nsCOMPtr<nsIAtom> name;
-    PRInt32 nameSpaceID;
+    nsCOMPtr<nsIAtom> name = mNodeInfo->GetNameAtom();
+    PRInt32 nameSpaceID = mNodeInfo->GetNamespaceID();
+
     nsAutoString tmpValue;
-
-    mNodeInfo->GetNameAtom(*getter_AddRefs(name));
-    mNodeInfo->GetNamespaceID(nameSpaceID);
-
     rv = mContent->GetAttr(nameSpaceID, name, tmpValue);
     if (rv == NS_CONTENT_ATTR_HAS_VALUE) {
       mContent->UnsetAttr(nameSpaceID, name, PR_TRUE);
@@ -511,8 +496,8 @@ nsDOMAttribute::CompareDocumentPosition(nsIDOMNode* aOther,
     // then based upon order between the root container of each node that
     // is in no container. In this case, the result is disconnected
     // and implementation-dependent.
-    mask |= (nsIDOMNode::DOCUMENT_POSITION_DISCONNECTED |
-             nsIDOMNode::DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC);
+    mask |= (nsIDOM3Node::DOCUMENT_POSITION_DISCONNECTED |
+             nsIDOM3Node::DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC);
 
     *aReturn = mask;
 
@@ -534,7 +519,7 @@ nsDOMAttribute::CompareDocumentPosition(nsIDOMNode* aOther,
         // nodeType is the same for both determining nodes, then an
         // implementation-dependent order between the determining nodes
         // is returned.
-        mask |= nsIDOMNode::DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC;
+        mask |= nsIDOM3Node::DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC;
       }
 
       // If the two nodes being compared are the same node,
@@ -575,8 +560,8 @@ nsDOMAttribute::CompareDocumentPosition(nsIDOMNode* aOther,
       // Throw a party!  Celebrate by returning that it is contained
       // and following this node.
 
-      mask |= (nsIDOMNode::DOCUMENT_POSITION_IS_CONTAINED |
-               nsIDOMNode::DOCUMENT_POSITION_FOLLOWING);
+      mask |= (nsIDOM3Node::DOCUMENT_POSITION_CONTAINED_BY |
+               nsIDOM3Node::DOCUMENT_POSITION_FOLLOWING);
 
       *aReturn = mask;
       return NS_OK;
@@ -594,8 +579,8 @@ nsDOMAttribute::CompareDocumentPosition(nsIDOMNode* aOther,
   parent->IsSameNode(aOther, &sameNode);
   if (sameNode) {
     // If the other node contains us, then it precedes us.
-    mask |= (nsIDOMNode::DOCUMENT_POSITION_CONTAINS |
-             nsIDOMNode::DOCUMENT_POSITION_PRECEDING);
+    mask |= (nsIDOM3Node::DOCUMENT_POSITION_CONTAINS |
+             nsIDOM3Node::DOCUMENT_POSITION_PRECEDING);
 
     *aReturn = mask;
     return NS_OK;
@@ -607,7 +592,7 @@ nsDOMAttribute::CompareDocumentPosition(nsIDOMNode* aOther,
   // We already established earlier that the node is not contained
   // by this attribute.  So if it is contained by our owner element,
   // unset the flag.
-  mask |= parentMask & ~nsIDOMNode::DOCUMENT_POSITION_IS_CONTAINED;
+  mask |= parentMask & ~nsIDOM3Node::DOCUMENT_POSITION_CONTAINED_BY;
 
   *aReturn = mask;
   return NS_OK;
@@ -638,7 +623,7 @@ nsDOMAttribute::IsSameNode(nsIDOMNode* aOther,
       // Check to see if we're in HTML.
       if (content->IsContentOfType(nsIContent::eHTML)) {
         nsCOMPtr<nsINodeInfo> ni;
-        content->GetNodeInfo(*getter_AddRefs(ni));
+        content->GetNodeInfo(getter_AddRefs(ni));
         if (ni) {
           // If there is no namespace, we're in HTML (as opposed to XHTML)
           // and we'll need to compare node names case insensitively.
@@ -661,19 +646,82 @@ nsDOMAttribute::IsSameNode(nsIDOMNode* aOther,
   return NS_OK;
 }
 
-NS_IMETHODIMP    
-nsDOMAttribute::LookupNamespacePrefix(const nsAString& aNamespaceURI,
-                                      nsAString& aPrefix) 
+NS_IMETHODIMP
+nsDOMAttribute::IsEqualNode(nsIDOMNode* aOther,
+                            PRBool* aReturn)
 {
-  aPrefix.Truncate();
-  nsresult rv = NS_OK;
-  nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(mContent));
-  if (node)
-    rv = node->LookupNamespacePrefix(aNamespaceURI, aPrefix);
-  return rv;
+  NS_NOTYETIMPLEMENTED("nsDocument::IsEqualNode()");
+
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP    
+NS_IMETHODIMP
+nsDOMAttribute::IsDefaultNamespace(const nsAString& aNamespaceURI,
+                                   PRBool* aReturn)
+{
+  NS_NOTYETIMPLEMENTED("nsDOMAttribute::IsDefaultNamespace()");
+
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsDOMAttribute::GetTextContent(nsAString &aTextContent)
+{
+  return GetNodeValue(aTextContent);
+}
+
+NS_IMETHODIMP
+nsDOMAttribute::SetTextContent(const nsAString& aTextContent)
+{
+  return SetNodeValue(aTextContent);
+}
+
+
+NS_IMETHODIMP
+nsDOMAttribute::GetFeature(const nsAString& aFeature,
+                           const nsAString& aVersion,
+                           nsISupports** aReturn)
+{
+  NS_NOTYETIMPLEMENTED("nsDocument::GetFeature()");
+
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsDOMAttribute::SetUserData(const nsAString& aKey,
+                            nsIVariant* aData,
+                            nsIDOMUserDataHandler* aHandler,
+                            nsIVariant** aReturn)
+{
+  NS_NOTYETIMPLEMENTED("nsDocument::SetUserData()");
+
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsDOMAttribute::GetUserData(const nsAString& aKey,
+                            nsIVariant** aReturn)
+{
+  NS_NOTYETIMPLEMENTED("nsDocument::GetUserData()");
+
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsDOMAttribute::LookupPrefix(const nsAString& aNamespaceURI,
+                             nsAString& aPrefix) 
+{
+  aPrefix.Truncate();
+
+  nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(mContent));
+  if (node) {
+    return node->LookupPrefix(aNamespaceURI, aPrefix);
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
 nsDOMAttribute::LookupNamespaceURI(const nsAString& aNamespacePrefix,
                                    nsAString& aNamespaceURI)
 {
