@@ -38,6 +38,9 @@
 #ifndef nsDocument_h___
 #define nsDocument_h___
 
+#include "nsCOMPtr.h"
+#include "nsAutoPtr.h"
+#include "nsCRT.h"
 #include "nsIDocument.h"
 #include "nsWeakReference.h"
 #include "nsWeakPtr.h"
@@ -56,7 +59,9 @@
 #include "nsIScriptGlobalObject.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIContent.h"
+#include "nsIEventListenerManager.h"
 #include "nsGenericDOMNodeList.h"
+#include "nsIDOM3Node.h"
 #include "nsIPrincipal.h"
 #include "nsIBindingManager.h"
 #include "nsINodeInfo.h"
@@ -76,6 +81,13 @@
 
 #include "pldhash.h"
 
+
+#define XML_DECLARATION_BITS_DECLARATION_EXISTS   (1 << 0)
+#define XML_DECLARATION_BITS_ENCODING_EXISTS      (1 << 1)
+#define XML_DECLARATION_BITS_STANDALONE_EXISTS    (1 << 2)
+#define XML_DECLARATION_BITS_STANDALONE_YES       (1 << 3)
+
+
 class nsIEventListenerManager;
 class nsDOMStyleSheetList;
 class nsIOutputStream;
@@ -84,51 +96,25 @@ class nsIDTD;
 class nsXPathDocumentTearoff;
 class nsIRadioVisitor;
 class nsIFormControl;
-
 struct nsRadioGroupStruct;
 
-#if 0
-class nsPostData : public nsIPostData {
-public:
-  nsPostData(PRBool aIsFile, char* aData);
-
-  NS_DECL_ISUPPORTS
-
-  virtual PRBool      IsFile();
-  virtual const char* GetData();
-  virtual PRInt32     GetDataLength();
-
-protected:
-  virtual ~nsPostData();
-
-  PRBool  mIsFile;
-  char*   mData;
-  PRInt32 mDataLen;
-};
-#endif
 
 class nsDocHeaderData
 {
 public:
   nsDocHeaderData(nsIAtom* aField, const nsAString& aData)
+    : mField(aField), mData(aData), mNext(nsnull)
   {
-    mField = aField;
-    NS_IF_ADDREF(mField);
-    mData.Assign(aData);
-    mNext = nsnull;
-  }
-  ~nsDocHeaderData(void)
-  {
-    NS_IF_RELEASE(mField);
-    if (nsnull != mNext) {
-      delete mNext;
-      mNext = nsnull;
-    }
   }
 
-  nsIAtom*         mField;
-  nsString         mData;
-  nsDocHeaderData* mNext;
+  ~nsDocHeaderData(void)
+  {
+    delete mNext;
+  }
+
+  nsCOMPtr<nsIAtom> mField;
+  nsString          mData;
+  nsDocHeaderData*  mNext;
 };
 
 // Represents the children of a document (prolog, epilog and
@@ -145,6 +131,8 @@ public:
   void DropReference();
 
 protected:
+  nsDocumentChildNodes(); // Not implemented
+
   nsIDocument* mDocument;
 };
 
@@ -159,7 +147,7 @@ public:
   NS_DECL_ISUPPORTS
 
   NS_DECL_NSIDOMSTYLESHEETLIST
-  
+
   NS_IMETHOD BeginUpdate(nsIDocument *aDocument) { return NS_OK; }
   NS_IMETHOD EndUpdate(nsIDocument *aDocument) { return NS_OK; }
   NS_IMETHOD BeginLoad(nsIDocument *aDocument) { return NS_OK; }
@@ -167,7 +155,7 @@ public:
   NS_IMETHOD BeginReflow(nsIDocument *aDocument,
                          nsIPresShell* aShell) { return NS_OK; }
   NS_IMETHOD EndReflow(nsIDocument *aDocument,
-                       nsIPresShell* aShell) { return NS_OK; } 
+                       nsIPresShell* aShell) { return NS_OK; }
   NS_IMETHOD ContentChanged(nsIDocument *aDocument,
                             nsIContent* aContent,
                             nsISupports* aSubContent) { return NS_OK; }
@@ -183,7 +171,7 @@ public:
                               nsChangeHint aHint) { return NS_OK; }
   NS_IMETHOD ContentAppended(nsIDocument *aDocument,
                              nsIContent* aContainer,
-                             PRInt32     aNewIndexInContainer) 
+                             PRInt32     aNewIndexInContainer)
                              { return NS_OK; }
   NS_IMETHOD ContentInserted(nsIDocument *aDocument,
                              nsIContent* aContainer,
@@ -207,8 +195,8 @@ public:
                                               PRBool aApplicable) { return NS_OK; }
   NS_IMETHOD StyleRuleChanged(nsIDocument *aDocument,
                               nsIStyleSheet* aStyleSheet,
-                              nsIStyleRule* aStyleRule,
-                              nsChangeHint aHint) { return NS_OK; }
+                              nsIStyleRule* aOldStyleRule,
+                              nsIStyleRule* aNewStyleRule) { return NS_OK; }
   NS_IMETHOD StyleRuleAdded(nsIDocument *aDocument,
                             nsIStyleSheet* aStyleSheet,
                             nsIStyleRule* aStyleRule) { return NS_OK; }
@@ -224,28 +212,6 @@ protected:
 };
 
 
-// Helper structs for the content->subdoc map
-
-class SubDocMapEntry : public PLDHashEntryHdr
-{
-public:
-  // Both of these are strong references
-  nsIContent *mKey; // must be first, to look like PLDHashEntryStub
-  nsIDocument *mSubDocument;
-};
-
-struct FindContentData
-{
-  FindContentData(nsIDocument *aSubDoc)
-    : mSubDocument(aSubDoc), mResult(nsnull)
-  {
-  }
-
-  nsISupports *mSubDocument;
-  nsIContent *mResult;
-};
-
-
 // Base class for our document implementations.
 //
 // Note that this class *implements* nsIDOMXMLDocument, but it's not
@@ -255,8 +221,8 @@ struct FindContentData
 // nsIDOMXMLDocument's. nsDocument's QI should *not* claim to support
 // nsIDOMXMLDocument unless someone writes a real implementation of
 // the interface.
-class nsDocument : public nsIDocument, 
-                   public nsIDOMXMLDocument, // inherits nsIDOMDocument 
+class nsDocument : public nsIDocument,
+                   public nsIDOMXMLDocument, // inherits nsIDOMDocument
                    public nsIDOMNSDocument,
                    public nsIDOMDocumentEvent,
                    public nsIDOM3DocumentEvent,
@@ -275,7 +241,7 @@ class nsDocument : public nsIDocument,
 public:
   NS_DECL_ISUPPORTS
 
-  NS_IMETHOD GetArena(nsIArena** aArena);
+  NS_DECL_AND_IMPL_ZEROING_OPERATOR_NEW
 
   NS_IMETHOD Reset(nsIChannel* aChannel, nsILoadGroup* aLoadGroup);
   NS_IMETHOD ResetToURI(nsIURI *aURI, nsILoadGroup *aLoadGroup);
@@ -293,7 +259,7 @@ public:
   /**
    * Return the title of the document. May return null.
    */
-  virtual const nsString* GetDocumentTitle() const;
+  NS_IMETHOD GetDocumentTitle(nsAString& aTitle) const;
 
   /**
    * Return the URL for the document. May return null.
@@ -311,6 +277,17 @@ public:
    */
   NS_IMETHOD AddPrincipal(nsIPrincipal *aPrincipal);
 
+  /**
+   * Get the Content-Type of this document.
+   */
+  // NS_IMETHOD GetContentType(nsAString& aContentType);
+  // Already declared in nsIDOMNSDocument
+
+  /**
+   * Set the Content-Type of this document.
+   */
+  NS_IMETHOD SetContentType(const nsAString& aContentType);
+
   /**
    * Return the content language of this document.
    */
@@ -324,7 +301,7 @@ public:
   /**
    * Return the base URL for relative URLs in the document. May return null (or the document URL).
    */
-  NS_IMETHOD GetBaseURL(nsIURI*& aURL) const;
+  NS_IMETHOD GetBaseURL(nsIURI** aURL) const;
   NS_IMETHOD SetBaseURL(nsIURI* aURL);
 
   /**
@@ -337,8 +314,8 @@ public:
    * Return a standard name for the document's character set. This will
    * trigger a startDocumentLoad if necessary to answer the question.
    */
-  NS_IMETHOD GetDocumentCharacterSet(nsAString& oCharsetID);
-  NS_IMETHOD SetDocumentCharacterSet(const nsAString& aCharSetID);
+  NS_IMETHOD GetDocumentCharacterSet(nsACString& oCharsetID);
+  NS_IMETHOD SetDocumentCharacterSet(const nsACString& aCharSetID);
 
   NS_IMETHOD GetDocumentCharacterSetSource(PRInt32* aCharsetSource);
   NS_IMETHOD SetDocumentCharacterSetSource(PRInt32 aCharsetSource);
@@ -353,7 +330,6 @@ public:
    */
   NS_IMETHOD RemoveCharSetObserver(nsIObserver* aObserver);
 
-#ifdef IBMBIDI
   /**
    *  Check if the document contains bidi data.
    *  If so, we have to apply the Unicode Bidi Algorithm.
@@ -363,7 +339,6 @@ public:
    *  Indicate the document contains RTL characters.
    */
   NS_IMETHOD SetBidiEnabled(PRBool aBidiEnabled);
-#endif // IBMBIDI
 
   /**
    * Return the Line Breaker for the document
@@ -387,13 +362,10 @@ public:
    * it's presentation context (presentation context's <b>must not</b> be
    * shared among multiple presentation shell's).
    */
-#if 0
-  // XXX Temp hack: moved to nsMarkupDocument
   NS_IMETHOD CreateShell(nsIPresContext* aContext,
                          nsIViewManager* aViewManager,
                          nsIStyleSet* aStyleSet,
                          nsIPresShell** aInstancePtrResult);
-#endif
   virtual PRBool DeleteShell(nsIPresShell* aShell);
   virtual PRInt32 GetNumberOfShells();
   NS_IMETHOD GetShellAt(PRInt32 aIndex, nsIPresShell** aShell);
@@ -416,11 +388,11 @@ public:
   NS_IMETHOD GetRootContent(nsIContent** aRoot);
   NS_IMETHOD SetRootContent(nsIContent* aRoot);
 
-  /** 
+  /**
    * Get the direct children of the document - content in
    * the prolog, the root content and content in the epilog.
    */
-  NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent*& aResult) const;
+  NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent** aResult) const;
   NS_IMETHOD IndexOf(nsIContent* aPossibleChild, PRInt32& aIndex) const;
   NS_IMETHOD GetChildCount(PRInt32& aCount);
 
@@ -435,7 +407,7 @@ public:
   NS_IMETHOD GetIndexOfStyleSheet(nsIStyleSheet* aSheet, PRInt32* aIndex);
   virtual void AddStyleSheet(nsIStyleSheet* aSheet, PRUint32 aFlags);
   virtual void RemoveStyleSheet(nsIStyleSheet* aSheet);
-  
+
   NS_IMETHOD UpdateStyleSheets(nsCOMArray<nsIStyleSheet>& aOldSheets,
                                nsCOMArray<nsIStyleSheet>& aNewSheets);
   virtual void AddStyleSheetToStyleSets(nsIStyleSheet* aSheet);
@@ -447,7 +419,7 @@ public:
 
   /**
    * Set the object from which a document can get a script context.
-   * This is the context within which all scripts (during document 
+   * This is the context within which all scripts (during document
    * creation and during event handling) will run.
    */
   NS_IMETHOD GetScriptGlobalObject(nsIScriptGlobalObject** aGlobalObject);
@@ -455,7 +427,7 @@ public:
 
   /**
    * Get the script loader for this document
-   */ 
+   */
   NS_IMETHOD GetScriptLoader(nsIScriptLoader** aScriptLoader);
 
   /**
@@ -505,8 +477,8 @@ public:
                             PRInt32 aIndexInContainer);
 
   NS_IMETHOD StyleRuleChanged(nsIStyleSheet* aStyleSheet,
-                              nsIStyleRule* aStyleRule,
-                              nsChangeHint aHint); // See nsStyleConsts fot hint values
+                              nsIStyleRule* aOldStyleRule,
+                              nsIStyleRule* aNewStyleRule);
   NS_IMETHOD StyleRuleAdded(nsIStyleSheet* aStyleSheet,
                             nsIStyleRule* aStyleRule);
   NS_IMETHOD StyleRuleRemoved(nsIStyleSheet* aStyleSheet,
@@ -516,12 +488,20 @@ public:
                                        PRBool aUpdateViews = PR_FALSE);
   NS_IMETHOD GetAndIncrementContentID(PRInt32* aID);
   NS_IMETHOD GetBindingManager(nsIBindingManager** aResult);
-  NS_IMETHOD GetNodeInfoManager(nsINodeInfoManager*& aNodeInfoManager);
+  NS_IMETHOD GetNodeInfoManager(nsINodeInfoManager** aNodeInfoManager);
   NS_IMETHOD AddReference(void *aKey, nsISupports *aReference);
   NS_IMETHOD RemoveReference(void *aKey, nsISupports **aOldReference);
   NS_IMETHOD SetContainer(nsISupports *aContainer);
   NS_IMETHOD GetContainer(nsISupports **aContainer);
   NS_IMETHOD GetScriptEventManager(nsIScriptEventManager **aResult);
+  NS_IMETHOD SetXMLDeclaration(const nsAString& aVersion,
+                               const nsAString& aEncoding,
+                               const nsAString& Standalone);
+  NS_IMETHOD GetXMLDeclaration(nsAString& aVersion,
+                               nsAString& aEncoding,
+                               nsAString& Standalone);
+  NS_IMETHOD_(PRBool) IsCaseSensitive();
+  NS_IMETHOD_(PRBool) IsScriptEnabled();
 
   // nsIRadioGroupContainer
   NS_IMETHOD WalkRadioGroup(const nsAString& aName,
@@ -534,7 +514,7 @@ public:
                              nsIFormControl* aRadio);
   NS_IMETHOD RemoveFromRadioGroup(const nsAString& aName,
                                   nsIFormControl* aRadio);
-                                        
+
   // for radio group
   nsresult GetRadioGroup(const nsAString& aName,
                          nsRadioGroupStruct **aRadioGroup);
@@ -590,15 +570,10 @@ public:
   // nsIDOM3EventTarget
   NS_DECL_NSIDOM3EVENTTARGET
 
-  NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, 
-                            nsEvent* aEvent, 
-                            nsIDOMEvent** aDOMEvent,
-                            PRUint32 aFlags,
+  NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
+                            nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
                             nsEventStatus* aEventStatus);
 
-  NS_IMETHOD_(PRBool) EventCaptureRegistration(PRInt32 aCapturerIncrement);
-
-
   NS_IMETHOD SetDocumentURL(nsIURI* aURI);
 
   virtual nsresult Init();
@@ -612,71 +587,85 @@ protected:
   virtual already_AddRefed<nsIStyleSheet> InternalGetStyleSheetAt(PRInt32 aIndex);
   virtual PRInt32 InternalGetNumberOfStyleSheets();
 
+  virtual void RetrieveRelevantHeaders(nsIChannel *aChannel);
+
+  nsresult doCreateShell(nsIPresContext* aContext,
+                         nsIViewManager* aViewManager, nsIStyleSet* aStyleSet,
+                         nsCompatibility aCompatMode,
+                         nsIPresShell** aInstancePtrResult);
+
   nsDocument();
-  virtual ~nsDocument(); 
+  virtual ~nsDocument();
 
-  nsIArena* mArena;
   nsString mDocumentTitle;
-  nsIURI* mDocumentURL;
+  nsCString mLastModified;
+  nsCOMPtr<nsIURI> mDocumentURL;
   nsCOMPtr<nsIURI> mDocumentBaseURL;
   nsCOMPtr<nsIPrincipal> mPrincipal;
-  PRUint32 mLoadFlags; // load flags of the document's channel
+
   nsWeakPtr mDocumentLoadGroup;
+
   nsWeakPtr mDocumentContainer;
 
-  nsString mCharacterSet;
+  nsCString mCharacterSet;
   PRInt32 mCharacterSetSource;
-  
+
   nsVoidArray mCharSetObservers;
   nsIDocument* mParentDocument;
 
   PLDHashTable *mSubDocuments;
 
-  nsVoidArray mPresShells;
-  nsCOMArray<nsIContent> mChildren; // contains owning references
-  nsIContent* mRootContent; // a weak reference to the only element in
-                            // mChildren, or null if no such element exists.
+  nsSmallVoidArray mPresShells;
+
+  // Array of owning references to all children
+  nsCOMArray<nsIContent> mChildren;
+
+  // A weak reference to the only element in mChildren, or null if no
+  // such element exists.
+  nsIContent* mRootContent;
+
   nsCOMArray<nsIStyleSheet> mStyleSheets;
-  nsAutoVoidArray mObservers; // basically always has at least 1 entry
+
+  // Basically always has at least 1 entry
+  nsAutoVoidArray mObservers;
   nsCOMPtr<nsIScriptGlobalObject> mScriptGlobalObject;
-  nsIEventListenerManager* mListenerManager;
-  PRBool mInDestructor;
+  nsCOMPtr<nsIEventListenerManager> mListenerManager;
   nsCOMPtr<nsIDOMStyleSheetList> mDOMStyleSheets;
   nsCOMPtr<nsIScriptLoader> mScriptLoader;
   nsDocHeaderData* mHeaderData;
   nsCOMPtr<nsILineBreaker> mLineBreaker;
   nsCOMPtr<nsIWordBreaker> mWordBreaker;
-  nsDocumentChildNodes* mChildNodes;
-  // A content ID counter used to give a monotonically increasing ID to the content
-  // objects in the document's content model
+
+  nsRefPtr<nsDocumentChildNodes> mChildNodes;
+
+  // A content ID counter used to give a monotonically increasing ID
+  // to the content objects in the document's content model
   PRInt32 mNextContentID;
 
   nsHashtable mRadioGroups;
 
-#ifdef IBMBIDI
-  PRBool        mBidiEnabled;
-#endif // IBMBIDI
-
-  // disk file members
-  nsCOMPtr<nsIURI>    mDocumentURI;
-  PRInt32             mModCount;
-
-  nsIDTD* mDTD;
-
   nsCOMPtr<nsIBindingManager> mBindingManager;
-  nsCOMPtr<nsINodeInfoManager> mNodeInfoManager; // OWNER
+  nsCOMPtr<nsINodeInfoManager> mNodeInfoManager;
 
-  PRBool mIsGoingAway; // True if the document is being destroyed.
+  // True if the document is being destroyed.
+  PRPackedBool mIsGoingAway;
+
+  // True if BIDI is enabled.
+  PRPackedBool mBidiEnabled;
+
+  // True if the document is being destroyed.
+  PRPackedBool mInDestructor;
+
+  PRUint8 mXMLDeclarationBits;
 
   nsSupportsHashtable* mBoxObjectTable;
-  PRInt32 mNumCapturers; //Number of capturing event handlers in doc.  Used to optimize event delivery.
 
   nsSupportsHashtable mContentWrapperHash;
 
   nsCOMPtr<nsICSSLoader> mCSSLoader;
 
-  nsString mContentLanguage;
-  nsString mContentType;
+  nsXPIDLCString mContentLanguage;
+  nsCString mContentType;
 
   nsCOMPtr<nsIScriptEventManager> mScriptEventManager;
 
