@@ -61,7 +61,6 @@
 #include "nsIDOMRange.h"
 #include "nsIDOMDocument.h"
 #include "nsICharsetConverterManager.h"
-#include "nsICharsetConverterManager2.h"
 #include "nsHTMLAtoms.h"
 #include "nsITextContent.h"
 #include "nsIEnumerator.h"
@@ -74,6 +73,7 @@
 #include "nsIScriptSecurityManager.h"
 #include "nsContentUtils.h"
 #include "nsUnicharUtils.h"
+#include "nsReadableUtils.h"
 
 static NS_DEFINE_CID(kCharsetConverterManagerCID,
                      NS_ICHARSETCONVERTERMANAGER_CID);
@@ -107,8 +107,9 @@ public:
   // Inherited methods from nsIDocumentEncoder
   NS_IMETHOD SetSelection(nsISelection* aSelection);
   NS_IMETHOD SetRange(nsIDOMRange* aRange);
+  NS_IMETHOD SetNode(nsIDOMNode* aNode);
   NS_IMETHOD SetWrapColumn(PRUint32 aWC);
-  NS_IMETHOD SetCharset(const nsAString& aCharset);
+  NS_IMETHOD SetCharset(const nsACString& aCharset);
   NS_IMETHOD GetMimeType(nsAString& aMimeType);
   NS_IMETHOD EncodeToStream(nsIOutputStream* aStream);
   NS_IMETHOD EncodeToString(nsAString& aOutputString);
@@ -143,15 +144,16 @@ protected:
   nsCOMPtr<nsIDocument>          mDocument;
   nsCOMPtr<nsISelection>         mSelection;
   nsCOMPtr<nsIDOMRange>          mRange;
+  nsCOMPtr<nsIDOMNode>           mNode;
   nsCOMPtr<nsIOutputStream>      mStream;
   nsCOMPtr<nsIContentSerializer> mSerializer;
   nsCOMPtr<nsIUnicodeEncoder>    mUnicodeEncoder;
   nsCOMPtr<nsIDOMNode>           mCommonParent;
   nsCOMPtr<nsIDocumentEncoderNodeFixup> mNodeFixup;
-  nsCOMPtr<nsICharsetConverterManager2> mCharsetConverterManager;
+  nsCOMPtr<nsICharsetConverterManager> mCharsetConverterManager;
 
   nsString          mMimeType;
-  nsString          mCharset;
+  nsCString         mCharset;
   PRUint32          mFlags;
   PRUint32          mWrapColumn;
   PRUint32          mStartDepth;
@@ -182,7 +184,6 @@ NS_INTERFACE_MAP_END
 
 nsDocumentEncoder::nsDocumentEncoder()
 {
-  NS_INIT_ISUPPORTS();
 
   mMimeType.Assign(NS_LITERAL_STRING("text/plain"));
 
@@ -197,34 +198,6 @@ nsDocumentEncoder::~nsDocumentEncoder()
 {
 }
 
-static PRBool
-IsScriptEnabled(nsIDocument *aDoc)
-{
-  NS_ENSURE_TRUE(aDoc, PR_TRUE);
-
-  nsCOMPtr<nsIScriptSecurityManager> sm(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
-  NS_ENSURE_TRUE(sm, PR_TRUE);
-
-  nsCOMPtr<nsIPrincipal> principal;
-  aDoc->GetPrincipal(getter_AddRefs(principal));
-  NS_ENSURE_TRUE(principal, PR_TRUE);
-
-  nsCOMPtr<nsIScriptGlobalObject> globalObject;
-  aDoc->GetScriptGlobalObject(getter_AddRefs(globalObject));
-  NS_ENSURE_TRUE(globalObject, PR_TRUE);
-
-  nsCOMPtr<nsIScriptContext> scriptContext;
-  globalObject->GetContext(getter_AddRefs(scriptContext));
-  NS_ENSURE_TRUE(scriptContext, PR_TRUE);
-
-  JSContext* cx = (JSContext *) scriptContext->GetNativeContext();
-  NS_ENSURE_TRUE(cx, PR_TRUE);
-
-  PRBool enabled = PR_TRUE;
-  sm->CanExecuteScripts(cx, principal, &enabled);
-  return enabled;
-}
-
 NS_IMETHODIMP
 nsDocumentEncoder::Init(nsIDocument* aDocument,
                         const nsAString& aMimeType,
@@ -265,7 +238,14 @@ nsDocumentEncoder::SetRange(nsIDOMRange* aRange)
 }
 
 NS_IMETHODIMP
-nsDocumentEncoder::SetCharset(const nsAString& aCharset)
+nsDocumentEncoder::SetNode(nsIDOMNode* aNode)
+{
+  mNode = aNode;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDocumentEncoder::SetCharset(const nsACString& aCharset)
 {
   mCharset = aCharset;
   return NS_OK;
@@ -416,7 +396,7 @@ nsDocumentEncoder::IsTag(nsIDOMNode* aNode, nsIAtom* aAtom)
     nsCOMPtr<nsIAtom> atom;
     nsCOMPtr<nsIContent> content = do_QueryInterface(aNode);
     if (content)
-      content->GetTag(*getter_AddRefs(atom));
+      content->GetTag(getter_AddRefs(atom));
     if (atom)
     {
       if (atom.get() == aAtom)
@@ -524,10 +504,10 @@ static nsresult ChildAt(nsIDOMNode* aNode, PRInt32 aIndex, nsIDOMNode*& aChild)
 
   NS_ENSURE_TRUE(node, NS_ERROR_FAILURE);
 
-  node->ChildAt(aIndex, *getter_AddRefs(child));
+  node->ChildAt(aIndex, getter_AddRefs(child));
 
   if (child)
-    child->QueryInterface(NS_GET_IID(nsIDOMNode), (void **)&aChild);
+    return CallQueryInterface(child, &aChild);
 
   return NS_OK;
 }
@@ -759,7 +739,7 @@ nsDocumentEncoder::SerializeRangeNodes(nsIDOMRange* aRange,
       // serialize the children of this node that are in the range
       for (j=startOffset; j<endOffset; j++)
       {
-        rv = content->ChildAt(j, *getter_AddRefs(child));
+        rv = content->ChildAt(j, getter_AddRefs(child));
         childAsNode = do_QueryInterface(child);
         NS_ENSURE_SUCCESS(rv, rv);
         if ((j==startOffset) || (j==endOffset-1))
@@ -918,10 +898,8 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
       mCharsetConverterManager = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
       NS_ENSURE_SUCCESS(rv, rv);
     }
-    rv = mCharsetConverterManager->GetCharsetAtom(mCharset.get(), getter_AddRefs(charsetAtom));
-    NS_ENSURE_SUCCESS(rv, rv);
   }
-  mSerializer->Init(mFlags, mWrapColumn, charsetAtom, mIsCopying);
+  mSerializer->Init(mFlags, mWrapColumn, mCharset.get(), mIsCopying);
 
   if (mSelection) {
     nsCOMPtr<nsIDOMRange> range;
@@ -942,6 +920,9 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
       rv = SerializeRangeToString(mRange, aOutputString);
 
       mRange = nsnull;
+  } else if (mNode) {
+    rv = SerializeToStringRecursive(mNode, aOutputString);
+    mNode = nsnull;
   } else {
     nsCOMPtr<nsIDOMDocument> domdoc(do_QueryInterface(mDocument));
     rv = mSerializer->AppendDocumentStart(domdoc, aOutputString);
@@ -972,11 +953,7 @@ nsDocumentEncoder::EncodeToStream(nsIOutputStream* aStream)
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  nsCOMPtr<nsIAtom> charsetAtom;
-  rv = mCharsetConverterManager->GetCharsetAtom(mCharset.get(), getter_AddRefs(charsetAtom));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = mCharsetConverterManager->GetUnicodeEncoder(charsetAtom,
+  rv = mCharsetConverterManager->GetUnicodeEncoder(mCharset.get(),
                                                    getter_AddRefs(mUnicodeEncoder));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1107,7 +1084,7 @@ nsHTMLCopyEncoder::Init(nsIDocument* aDocument,
   // (see related bugs #57296, #41924, #58646, #32768)
   mFlags = aFlags | OutputAbsoluteLinks;
 
-  if (!IsScriptEnabled(mDocument))
+  if (!mDocument->IsScriptEnabled())
     mFlags |= OutputNoScriptContent;
 
   return NS_OK;
@@ -1148,7 +1125,7 @@ nsHTMLCopyEncoder::SetSelection(nsISelection* aSelection)
   {
     // checking for selection inside a plaintext form widget
     nsCOMPtr<nsIAtom> atom;
-    selContent->GetTag(*getter_AddRefs(atom));
+    selContent->GetTag(getter_AddRefs(atom));
     if (atom.get() == nsHTMLAtoms::input ||
         atom.get() == nsHTMLAtoms::textarea)
     {
@@ -1169,7 +1146,7 @@ nsHTMLCopyEncoder::SetSelection(nsISelection* aSelection)
         break;
       }
     }
-    selContent->GetParent(*getter_AddRefs(tmp));
+    selContent->GetParent(getter_AddRefs(tmp));
     selContent = tmp;
   }
   
@@ -1298,7 +1275,7 @@ nsHTMLCopyEncoder::IncludeInContext(nsIDOMNode *aNode)
 
   nsCOMPtr<nsIAtom> tag;
 
-  content->GetTag(*getter_AddRefs(tag));
+  content->GetTag(getter_AddRefs(tag));
 
   if (tag.get() == nsHTMLAtoms::b        ||
       tag.get() == nsHTMLAtoms::i        ||
@@ -1493,7 +1470,7 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
           {
             PRBool isBlock = PR_FALSE;
             PRInt32 id;
-            content->GetTag(*getter_AddRefs(atom));
+            content->GetTag(getter_AddRefs(atom));
             atom->ToString(tag);
             parserService->HTMLStringTagToId(tag, &id);
             parserService->IsBlock(id, isBlock);
@@ -1582,7 +1559,7 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
           {
             PRBool isBlock = PR_FALSE;
             PRInt32 id;
-            content->GetTag(*getter_AddRefs(atom));
+            content->GetTag(getter_AddRefs(atom));
             atom->ToString(tag);
             parserService->HTMLStringTagToId(tag, &id);
             parserService->IsBlock(id, isBlock);
@@ -1634,7 +1611,7 @@ nsHTMLCopyEncoder::GetChildAt(nsIDOMNode *aParent, PRInt32 aOffset)
   nsCOMPtr<nsIContent> cChild;
   NS_PRECONDITION(content, "null content in nsHTMLCopyEncoder::GetChildAt");
   
-  if (NS_FAILED(content->ChildAt(aOffset, *getter_AddRefs(cChild))))
+  if (NS_FAILED(content->ChildAt(aOffset, getter_AddRefs(cChild))))
     return resultNode;
   
   resultNode = do_QueryInterface(cChild);
