@@ -48,7 +48,8 @@
 #include "nsReadableUtils.h"
 #include "nsMutationEvent.h"
 #include "nsINameSpaceManager.h"
-
+#include "nsIDOM3Node.h"
+#include "nsIURI.h"
 #include "nsIPrivateDOMEvent.h"
 #include "nsIDOMEvent.h"
 #include "nsIDOMText.h"
@@ -69,7 +70,6 @@ nsGenericDOMDataNode::Shutdown()
 nsGenericDOMDataNode::nsGenericDOMDataNode()
   : mText(), mDocument(nsnull), mParentPtrBits(0)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsGenericDOMDataNode::~nsGenericDOMDataNode()
@@ -155,7 +155,7 @@ nsGenericDOMDataNode::GetPreviousSibling(nsIDOMNode** aPrevSibling)
     PRInt32 pos;
     parent_weak->IndexOf(this, pos);
     if (pos > 0 ) {
-      parent_weak->ChildAt(--pos, *getter_AddRefs(sibling));
+      parent_weak->ChildAt(--pos, getter_AddRefs(sibling));
     }
   } else if (mDocument) {
     // Nodes that are just below the document (their parent is the
@@ -163,7 +163,7 @@ nsGenericDOMDataNode::GetPreviousSibling(nsIDOMNode** aPrevSibling)
     PRInt32 pos;
     mDocument->IndexOf(this, pos);
     if (pos > 0 ) {
-      mDocument->ChildAt(--pos, *getter_AddRefs(sibling));
+      mDocument->ChildAt(--pos, getter_AddRefs(sibling));
     }
   }
 
@@ -189,7 +189,7 @@ nsGenericDOMDataNode::GetNextSibling(nsIDOMNode** aNextSibling)
     PRInt32 pos;
     parent_weak->IndexOf(this, pos);
     if (pos > -1 ) {
-      parent_weak->ChildAt(++pos, *getter_AddRefs(sibling));
+      parent_weak->ChildAt(++pos, getter_AddRefs(sibling));
     }
   }
   else if (mDocument) {
@@ -198,7 +198,7 @@ nsGenericDOMDataNode::GetNextSibling(nsIDOMNode** aNextSibling)
     PRInt32 pos;
     mDocument->IndexOf(this, pos);
     if (pos > -1 ) {
-      mDocument->ChildAt(++pos, *getter_AddRefs(sibling));
+      mDocument->ChildAt(++pos, getter_AddRefs(sibling));
     }
   }
 
@@ -287,30 +287,21 @@ nsGenericDOMDataNode::IsSupported(const nsAString& aFeature,
 nsresult
 nsGenericDOMDataNode::GetBaseURI(nsAString& aURI)
 {
-  aURI.Truncate();
-  nsresult rv = NS_OK;
+  nsCOMPtr<nsIURI> baseURI;
+  nsresult rv = GetBaseURL(getter_AddRefs(baseURI));
 
-  // DOM Data Node inherits the base from its parent element/document
-  nsCOMPtr<nsIDOM3Node> node;
-
-  nsIContent *parent_weak = GetParentWeak();
-
-  if (parent_weak) {
-    node = do_QueryInterface(parent_weak);
-  } else if (mDocument) {
-    node = do_QueryInterface(mDocument);
-  }
-
-  if (node) {
-    rv = node->GetBaseURI(aURI);
+  nsCAutoString spec;
+  if (NS_SUCCEEDED(rv) && baseURI) {
+    baseURI->GetSpec(spec);
   }
+  CopyUTF8toUTF16(spec, aURI);
 
   return rv;
 }
 
 nsresult
-nsGenericDOMDataNode::LookupNamespacePrefix(const nsAString& aNamespaceURI,
-                                            nsAString& aPrefix)
+nsGenericDOMDataNode::LookupPrefix(const nsAString& aNamespaceURI,
+                                   nsAString& aPrefix)
 {
   aPrefix.Truncate();
 
@@ -318,9 +309,8 @@ nsGenericDOMDataNode::LookupNamespacePrefix(const nsAString& aNamespaceURI,
 
   // DOM Data Node passes the query on to its parent
   nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(parent_weak));
-
   if (node) {
-    return node->LookupNamespacePrefix(aNamespaceURI, aPrefix);
+    return node->LookupPrefix(aNamespaceURI, aPrefix);
   }
 
   return NS_OK;
@@ -639,9 +629,9 @@ nsGenericDOMDataNode::ToCString(nsAString& aBuf, PRInt32 aOffset,
 #endif
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::GetDocument(nsIDocument*& aResult) const
+nsGenericDOMDataNode::GetDocument(nsIDocument** aResult) const
 {
-  aResult = mDocument;
+  *aResult = mDocument;
   NS_IF_ADDREF(mDocument);
   return NS_OK;
 }
@@ -653,20 +643,17 @@ nsGenericDOMDataNode::SetDocument(nsIDocument* aDocument, PRBool aDeep,
 {
   mDocument = aDocument;
 
-#ifdef IBMBIDI
   if (mDocument && mText.IsBidi()) {
     mDocument->SetBidiEnabled(PR_TRUE);
   }
-#endif
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::GetParent(nsIContent*& aResult) const
+nsGenericDOMDataNode::GetParent(nsIContent** aResult) const
 {
-  aResult = GetParentWeak();
-  NS_IF_ADDREF(aResult);
+  NS_IF_ADDREF(*aResult = GetParentWeak());
 
   return NS_OK;;
 }
@@ -697,17 +684,17 @@ nsGenericDOMDataNode::SetNativeAnonymous(PRBool aAnonymous)
 }
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::GetNameSpaceID(PRInt32& aID) const
+nsGenericDOMDataNode::GetNameSpaceID(PRInt32* aID) const
 {
-  aID = kNameSpaceID_None;
+  *aID = kNameSpaceID_None;
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsGenericDOMDataNode::NormalizeAttrString(const nsAString& aStr,
-                                          nsINodeInfo*& aNodeInfo)
+                                          nsINodeInfo** aNodeInfo)
 {
-  aNodeInfo = nsnull;
+  *aNodeInfo = nsnull;
   return NS_OK;
 }
 
@@ -743,10 +730,10 @@ nsGenericDOMDataNode::GetAttr(PRInt32 aNameSpaceID, nsIAtom *aAttr,
 
 NS_IMETHODIMP
 nsGenericDOMDataNode::GetAttr(PRInt32 aNameSpaceID, nsIAtom *aAttr,
-                              nsIAtom*& aPrefix,
+                              nsIAtom** aPrefix,
                               nsAString& aResult) const
 {
-  aPrefix = nsnull;
+  *aPrefix = nsnull;
   aResult.Truncate();
 
   return NS_CONTENT_ATTR_NOT_THERE;
@@ -759,12 +746,13 @@ nsGenericDOMDataNode::HasAttr(PRInt32 aNameSpaceID, nsIAtom *aAttribute) const
 }
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::GetAttrNameAt(PRInt32 aIndex, PRInt32& aNameSpaceID,
-                                    nsIAtom*& aName, nsIAtom*& aPrefix) const
+nsGenericDOMDataNode::GetAttrNameAt(PRInt32 aIndex, PRInt32* aNameSpaceID,
+                                    nsIAtom** aName, nsIAtom** aPrefix) const
 {
-  aNameSpaceID = kNameSpaceID_None;
-  aName = nsnull;
-  aPrefix = nsnull;
+  *aNameSpaceID = kNameSpaceID_None;
+  *aName = nsnull;
+  *aPrefix = nsnull;
+
   return NS_ERROR_ILLEGAL_VALUE;
 }
 
@@ -874,9 +862,9 @@ nsGenericDOMDataNode::SetContentID(PRUint32 aID)
 }
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::GetNodeInfo(nsINodeInfo*& aResult) const
+nsGenericDOMDataNode::GetNodeInfo(nsINodeInfo** aResult) const
 {
-  aResult = nsnull;
+  *aResult = nsnull;
   return NS_OK;
 }
 
@@ -895,9 +883,9 @@ nsGenericDOMDataNode::ChildCount(PRInt32& aResult) const
 }
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::ChildAt(PRInt32 aIndex, nsIContent*& aResult) const
+nsGenericDOMDataNode::ChildAt(PRInt32 aIndex, nsIContent** aResult) const
 {
-  aResult = nsnull;
+  *aResult = nsnull;
   return NS_OK;
 }
 
@@ -1021,9 +1009,9 @@ nsGenericDOMDataNode::RangeRemove(nsIDOMRange* aRange)
 
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::GetRangeList(nsVoidArray*& aResult) const
+nsGenericDOMDataNode::GetRangeList(nsVoidArray** aResult) const
 {
-  aResult = LookupRangeList();
+  *aResult = LookupRangeList();
   return NS_OK;
 }
 
@@ -1070,19 +1058,24 @@ nsGenericDOMDataNode::DumpContent(FILE* out, PRInt32 aIndent,
 {
   return NS_OK;
 }
+#endif
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
+nsGenericDOMDataNode::GetBaseURL(nsIURI** aURI) const
 {
-  if (!aResult) {
-    return NS_ERROR_NULL_POINTER;
+  // DOM Data Node inherits the base from its parent element/document
+  nsIContent* parent_weak = GetParentWeak();
+  if (parent_weak) {
+    return parent_weak->GetBaseURL(aURI);
   }
-  PRUint32 sum = sizeof(*this);
-  sum += mText.GetLength() * (mText.Is2b() ? sizeof(PRUnichar) : sizeof(char));
-  *aResult = sum;
+
+  if (mDocument) {
+    return mDocument->GetBaseURL(aURI);
+  }
+
+  *aURI = nsnull;
   return NS_OK;
 }
-#endif
 
 //----------------------------------------------------------------------
 
@@ -1133,7 +1126,7 @@ nsGenericDOMDataNode::SplitText(PRUint32 aOffset, nsIDOMText** aReturn)
   }
 
   nsCOMPtr<nsIContent> parentNode;
-  GetParent(*getter_AddRefs(parentNode));
+  GetParent(getter_AddRefs(parentNode));
 
   if (parentNode) {
     PRInt32 index;
@@ -1146,7 +1139,7 @@ nsGenericDOMDataNode::SplitText(PRUint32 aOffset, nsIDOMText** aReturn)
     }
   }
 
-  return newNode->QueryInterface(NS_GET_IID(nsIDOMText), (void**)aReturn);
+  return CallQueryInterface(newNode, aReturn);
 }
 
 //----------------------------------------------------------------------
@@ -1207,9 +1200,7 @@ nsGenericDOMDataNode::SetText(const PRUnichar* aBuffer,
 
   mText.SetTo(aBuffer, aLength);
 
-#ifdef IBMBIDI
   SetBidiStatus();
-#endif // IBMBIDI
 
   if (mDocument && nsGenericElement::HasMutationListeners(this, NS_EVENT_BITS_MUTATION_CHARACTERDATAMODIFIED)) {
     nsCOMPtr<nsIDOMEventTarget> node(do_QueryInterface(this));
@@ -1221,7 +1212,7 @@ nsGenericDOMDataNode::SetText(const PRUnichar* aBuffer,
     // XXX Handle the setting of prevValue!
     nsAutoString newVal(aBuffer);
     if (!newVal.IsEmpty())
-      mutation.mNewAttrValue = getter_AddRefs(NS_NewAtom(newVal));
+      mutation.mNewAttrValue = do_GetAtom(newVal);
     nsEventStatus status = nsEventStatus_eIgnore;
     HandleDOMEvent(nsnull, &mutation, nsnull,
                    NS_EVENT_FLAG_INIT, &status);
@@ -1263,7 +1254,7 @@ nsGenericDOMDataNode::SetText(const char* aBuffer, PRInt32 aLength,
     // XXX Handle the setting of prevValue!
     nsAutoString newVal; newVal.AssignWithConversion(aBuffer);
     if (!newVal.IsEmpty())
-      mutation.mNewAttrValue = getter_AddRefs(NS_NewAtom(newVal));
+      mutation.mNewAttrValue = do_GetAtom(newVal);
     nsEventStatus status = nsEventStatus_eIgnore;
     HandleDOMEvent(nsnull, &mutation, nsnull,
                    NS_EVENT_FLAG_INIT, &status);
@@ -1288,9 +1279,7 @@ nsGenericDOMDataNode::SetText(const nsAString& aStr,
 
   mText = aStr;
 
-#ifdef IBMBIDI
   SetBidiStatus();
-#endif // IBMBIDI
 
   if (mDocument && nsGenericElement::HasMutationListeners(this, NS_EVENT_BITS_MUTATION_CHARACTERDATAMODIFIED)) {
     nsCOMPtr<nsIDOMEventTarget> node(do_QueryInterface(this));
@@ -1302,7 +1291,7 @@ nsGenericDOMDataNode::SetText(const nsAString& aStr,
     // XXX Handle the setting of prevValue!
     nsAutoString newVal(aStr);
     if (!newVal.IsEmpty())
-      mutation.mNewAttrValue = getter_AddRefs(NS_NewAtom(newVal));
+      mutation.mNewAttrValue = do_GetAtom(newVal);
     nsEventStatus status = nsEventStatus_eIgnore;
     HandleDOMEvent(nsnull, &mutation, nsnull,
                    NS_EVENT_FLAG_INIT, &status);
@@ -1405,7 +1394,6 @@ nsGenericDOMDataNode::LookupRangeList() const
   return nsnull;
 }
 
-#ifdef IBMBIDI
 void nsGenericDOMDataNode::SetBidiStatus()
 {
   if (mDocument) {
@@ -1426,4 +1414,3 @@ void nsGenericDOMDataNode::SetBidiStatus()
     mDocument->SetBidiEnabled(PR_TRUE);
   }
 }
-#endif // IBMBIDI
