@@ -48,6 +48,7 @@
 #include "nsIDOMLinkStyle.h"
 #include "nsIDOMEventReceiver.h"
 #include "nsIDOM3EventTarget.h"
+#include "nsIDOM3Node.h"
 #include "nsIStyleSheetLinkingElement.h"
 #include "nsICSSStyleSheet.h"
 #include "nsICSSLoaderObserver.h"
@@ -229,14 +230,23 @@ public:
  */
 class nsNode3Tearoff : public nsIDOM3Node
 {
+public:
   NS_DECL_ISUPPORTS
 
   NS_DECL_NSIDOM3NODE
 
   nsNode3Tearoff(nsIContent *aContent) : mContent(aContent)
   {
-    NS_INIT_ISUPPORTS();
   }
+
+  static nsresult GetTextContent(nsIDocument *aDoc,
+                                 nsIDOMNode *aNode,
+                                 nsAString &aTextContent);
+
+  static nsresult SetTextContent(nsIContent *aContent,
+                                 const nsAString &aTextContent);
+
+protected:
   virtual ~nsNode3Tearoff() {};
 
 private:
@@ -349,19 +359,19 @@ public:
   static void Shutdown();
 
   // nsIContent interface methods
-  NS_IMETHOD GetDocument(nsIDocument*& aResult) const;
+  NS_IMETHOD GetDocument(nsIDocument** aResult) const;
   NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep,
                          PRBool aCompileEventHandlers);
-  NS_IMETHOD GetParent(nsIContent*& aResult) const;
+  NS_IMETHOD GetParent(nsIContent** aResult) const;
   NS_IMETHOD SetParent(nsIContent* aParent);
   NS_IMETHOD_(PRBool) IsNativeAnonymous() const;
   NS_IMETHOD_(void) SetNativeAnonymous(PRBool aAnonymous);
-  NS_IMETHOD GetNameSpaceID(PRInt32& aNameSpaceID) const;
-  NS_IMETHOD GetTag(nsIAtom*& aResult) const;
-  NS_IMETHOD GetNodeInfo(nsINodeInfo*& aResult) const;
+  NS_IMETHOD GetNameSpaceID(PRInt32* aNameSpaceID) const;
+  NS_IMETHOD GetTag(nsIAtom** aResult) const;
+  NS_IMETHOD GetNodeInfo(nsINodeInfo** aResult) const;
   // NS_IMETHOD CanContainChildren(PRBool& aResult) const;
   // NS_IMETHOD ChildCount(PRInt32& aResult) const;
-  // NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent*& aResult) const;
+  // NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent** aResult) const;
   // NS_IMETHOD IndexOf(nsIContent* aPossibleChild, PRInt32& aResult) const;
   // NS_IMETHOD InsertChildAt(nsIContent* aKid, PRInt32 aIndex,
   //                          PRBool aNotify);
@@ -370,7 +380,7 @@ public:
   // NS_IMETHOD AppendChildTo(nsIContent* aKid, PRBool aNotify);
   // NS_IMETHOD RemoveChildAt(PRInt32 aIndex, PRBool aNotify);
   // NS_IMETHOD NormalizeAttrString(const nsAString& aStr,
-  //                                nsINodeInfo*& aNodeInfo);
+  //                                nsINodeInfo** aNodeInfo);
   // NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, 
   //                    const nsAString& aValue,
   //                    PRBool aNotify);
@@ -380,14 +390,14 @@ public:
   // NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, 
   //                    nsAString& aResult) const;
   // NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, 
-  //                    nsIAtom*& aPrefix,
+  //                    nsIAtom** aPrefix,
   //                    nsAString& aResult) const;
   // NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute, 
   //                      PRBool aNotify);
   // NS_IMETHOD GetAttrNameAt(PRInt32 aIndex,
   //                          PRInt32& aNameSpaceID, 
-  //                          nsIAtom*& aName,
-  //                          nsIAtom*& aPrefix) const;
+  //                          nsIAtom** aName,
+  //                          nsIAtom** aPrefix) const;
   // NS_IMETHOD GetAttrCount(PRInt32& aResult) const;
 #ifdef DEBUG
   // NS_IMETHOD List(FILE* out, PRInt32 aIndent) const;
@@ -395,7 +405,7 @@ public:
 #endif
   NS_IMETHOD RangeAdd(nsIDOMRange* aRange);
   NS_IMETHOD RangeRemove(nsIDOMRange* aRange);
-  NS_IMETHOD GetRangeList(nsVoidArray*& aResult) const;
+  NS_IMETHOD GetRangeList(nsVoidArray** aResult) const;
   NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext,
                             nsEvent* aEvent,
                             nsIDOMEvent** aDOMEvent,
@@ -409,11 +419,12 @@ public:
   NS_IMETHOD SetBindingParent(nsIContent* aParent);
   NS_IMETHOD_(PRBool) IsContentOfType(PRUint32 aFlags);
   NS_IMETHOD GetListenerManager(nsIEventListenerManager** aInstancePtrResult);
+  NS_IMETHOD GetBaseURL(nsIURI** aBaseURL) const;
   NS_IMETHOD DoneCreatingElement();
 
 
   // nsIStyledContent interface methods
-  NS_IMETHOD GetID(nsIAtom*& aResult) const;
+  NS_IMETHOD GetID(nsIAtom** aResult) const;
   NS_IMETHOD GetClasses(nsVoidArray& aArray) const;
   NS_IMETHOD_(PRBool) HasClass(nsIAtom* aClass, PRBool aCaseSensitive) const;
   NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
@@ -422,8 +433,7 @@ public:
                                       PRInt32 aModType, nsChangeHint& aHint) const;
 
   // nsIXMLContent interface methods
-  NS_IMETHOD MaybeTriggerAutoLink(nsIWebShell *aShell);
-  NS_IMETHOD GetXMLBaseURI(nsIURI **aURI);
+  NS_IMETHOD MaybeTriggerAutoLink(nsIDocShell *aShell);
 
   // nsIHTMLContent interface methods
   NS_IMETHOD Compact();
@@ -439,7 +449,6 @@ public:
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
                                const nsAString& aValue,
                                nsHTMLValue& aResult);
-  NS_IMETHOD GetBaseURL(nsIURI*& aBaseURL) const;
   NS_IMETHOD GetBaseTarget(nsAString& aBaseTarget) const;
 
   // nsIDOMNode method implementation
@@ -525,19 +534,22 @@ public:
                                   const nsAString& aValue);
 
   /**
-   * Load a link, putting together the proper URL from the pieces given.
+   * Trigger a link with uri aLinkURI.  If aClick is false, this triggers a
+   * mouseover on the link, otherwise it triggers a load, after doing a
+   * security check.
    * @param aPresContext the pres context.
    * @param aVerb how the link will be loaded (replace page, new window, etc.)
-   * @param aBaseURL the base URL to start with (content.baseURL, may be null)
-   * @param aURLSpec the URL of the link (may be relative)
-   * @param aTargetSpec the target (like target=, may be null)
+   * @param aOriginURI the URI the request originates from.  Used as the origin
+   *        uri for a CheckLoadURI call. 
+   * @param aLinkURI the URI of the link
+   * @param aTargetSpec the target (like target=, may be empty)
    * @param aClick whether this was a click or not (if false, it assumes you
    *        just hovered over the link)
    */
   nsresult TriggerLink(nsIPresContext* aPresContext,
                        nsLinkVerb aVerb,
-                       nsIURI* aBaseURL,
-                       const nsAString& aURLSpec,
+                       nsIURI* aOriginURI,
+                       nsIURI* aLinkURI,
                        const nsAFlatString& aTargetSpec,
                        PRBool aClick);
   /**
@@ -593,10 +605,6 @@ public:
   static PLDHashTable sRangeListsHash;
 
 protected:
-#ifdef DEBUG
-  virtual PRUint32 BaseSizeOf(nsISizeOfHandler *aSizer) const;
-#endif
-
   PRBool HasDOMSlots() const
   {
     return !(mFlagsOrSlots & GENERIC_ELEMENT_DOESNT_HAVE_DOMSLOTS);
@@ -766,7 +774,7 @@ public:
 
   // Remainder of nsIContent
   NS_IMETHOD NormalizeAttrString(const nsAString& aStr,
-                                 nsINodeInfo*& aNodeInfo);
+                                 nsINodeInfo** aNodeInfo);
   NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                      const nsAString& aValue,
                      PRBool aNotify);
@@ -776,14 +784,14 @@ public:
   NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                      nsAString& aResult) const;
   NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                     nsIAtom*& aPrefix, nsAString& aResult) const;
+                     nsIAtom** aPrefix, nsAString& aResult) const;
   NS_IMETHOD_(PRBool) HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const;
   NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
                        PRBool aNotify);
   NS_IMETHOD GetAttrNameAt(PRInt32 aIndex,
-                           PRInt32& aNameSpaceID,
-                           nsIAtom*& aName,
-                           nsIAtom*& aPrefix) const;
+                           PRInt32* aNameSpaceID,
+                           nsIAtom** aName,
+                           nsIAtom** aPrefix) const;
   NS_IMETHOD GetAttrCount(PRInt32& aResult) const;
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out, PRInt32 aIndent) const;
@@ -791,7 +799,7 @@ public:
 #endif
   NS_IMETHOD CanContainChildren(PRBool& aResult) const;
   NS_IMETHOD ChildCount(PRInt32& aResult) const;
-  NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent*& aResult) const;
+  NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent** aResult) const;
   NS_IMETHOD IndexOf(nsIContent* aPossibleChild, PRInt32& aResult) const;
   NS_IMETHOD InsertChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify,
                            PRBool aDeepSetDocument);
@@ -806,10 +814,6 @@ public:
 #endif
 
 protected:
-#ifdef DEBUG
-  virtual PRUint32 BaseSizeOf(nsISizeOfHandler *aSizer) const;
-#endif
-
   /**
    * The attributes (stored as nsGenericAttribute*)
    * @see nsGenericAttribute
