@@ -381,16 +381,9 @@ nsImageLoadingContent::ImageURIChanged(const nsACString& aNewURI)
     return NS_OK;
   }
 
-  if (aNewURI.IsEmpty()) {
-    // Do not take down the already loaded image... (for compat with
-    // the old code that loaded images from frames)    
-    // XXXbz is this what we really want?
-    return NS_OK;
-  }
-
   nsresult rv;   // XXXbz Should failures in this method fire onerror?
 
-  // First, get a document (needed for security checks, base URI and the like)
+  // First, get a document (needed for security checks and the like)
   nsCOMPtr<nsIDocument> doc;
   rv = GetOurDocument(getter_AddRefs(doc));
   if (!doc) {
@@ -467,13 +460,13 @@ nsImageLoadingContent::ImageURIChanged(const nsACString& aNewURI)
   NS_ENSURE_TRUE(thisContent, rv);
 
   nsCOMPtr<nsIDocument> tempDoc;
-  thisContent->GetDocument(*getter_AddRefs(tempDoc));
+  thisContent->GetDocument(getter_AddRefs(tempDoc));
   if (!doc) {
     return NS_OK;
   }
 
   nsCOMPtr<nsIContent> parent;
-  thisContent->GetParent(*getter_AddRefs(parent));
+  thisContent->GetParent(getter_AddRefs(parent));
   if (!parent) {
     return NS_OK;
   }
@@ -569,12 +562,12 @@ nsImageLoadingContent::GetOurDocument(nsIDocument** aDocument)
   nsCOMPtr<nsIContent> thisContent = do_QueryInterface(this, &rv);
   NS_ENSURE_TRUE(thisContent, rv);
 
-  rv = thisContent->GetDocument(*aDocument);
+  rv = thisContent->GetDocument(aDocument);
   if (!*aDocument) {  // nodeinfo time
     nsCOMPtr<nsINodeInfo> nodeInfo;
-    rv = thisContent->GetNodeInfo(*getter_AddRefs(nodeInfo));
+    rv = thisContent->GetNodeInfo(getter_AddRefs(nodeInfo));
     if (nodeInfo) {
-      rv = nodeInfo->GetDocument(*aDocument);
+      rv = nodeInfo->GetDocument(aDocument);
     }
   }
 
@@ -593,25 +586,19 @@ nsImageLoadingContent::StringToURI(const nsACString& aSpec,
   
   // (1) Get the base URI
   nsCOMPtr<nsIURI> baseURL;
-  nsCOMPtr<nsIHTMLContent> thisContent = do_QueryInterface(this);
-  if (thisContent) {
-    rv = thisContent->GetBaseURL(*getter_AddRefs(baseURL));
-  } else {
-    rv = aDocument->GetBaseURL(*getter_AddRefs(baseURL));
-    if (!baseURL) {
-      rv = aDocument->GetDocumentURL(getter_AddRefs(baseURL));
-    }
-  }
+  nsCOMPtr<nsIContent> thisContent = do_QueryInterface(this);
+  NS_ASSERTION(thisContent, "An image loading content must be an nsIContent");
+  rv = thisContent->GetBaseURL(getter_AddRefs(baseURL));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // (2) Get the charset
-  nsAutoString charset;
+  nsCAutoString charset;
   aDocument->GetDocumentCharacterSet(charset);
 
   // (3) Construct the silly thing
   return NS_NewURI(aURI,
                    aSpec,
-                   charset.IsEmpty() ? nsnull : NS_ConvertUCS2toUTF8(charset).get(),
+                   charset.IsEmpty() ? nsnull : charset.get(),
                    baseURL,
                    sIOService);
 }
@@ -622,16 +609,17 @@ nsImageLoadingContent::StringToURI(const nsACString& aSpec,
  */
 MOZ_DECL_CTOR_COUNTER(ImageEvent)
 
-struct ImageEvent : PLEvent
+class ImageEvent : public PLEvent,
+                   public nsDummyLayoutRequest
 {
+public:
   ImageEvent(nsIPresContext* aPresContext, nsIContent* aContent,
-             const nsAString& aMessage, nsILoadGroup *aLoadGroup,
-             nsIRequest *aRequest)
-    : mPresContext(aPresContext),
+             const nsAString& aMessage, nsILoadGroup *aLoadGroup)
+    : nsDummyLayoutRequest(nsnull),
+      mPresContext(aPresContext),
       mContent(aContent),
       mMessage(aMessage),
-      mLoadGroup(aLoadGroup),
-      mDummyRequest(aRequest)
+      mLoadGroup(aLoadGroup)
   {
     MOZ_COUNT_CTOR(ImageEvent);
   }
@@ -644,7 +632,6 @@ struct ImageEvent : PLEvent
   nsCOMPtr<nsIContent> mContent;
   nsString mMessage;
   nsCOMPtr<nsILoadGroup> mLoadGroup;
-  nsCOMPtr<nsIRequest> mDummyRequest;
 };
 
 PR_STATIC_CALLBACK(void*)
@@ -664,8 +651,8 @@ HandleImagePLEvent(PLEvent* aEvent)
   evt->mContent->HandleDOMEvent(evt->mPresContext, &event, nsnull,
                                 NS_EVENT_FLAG_INIT, &estatus);
 
-  evt->mLoadGroup->RemoveRequest(evt->mDummyRequest, nsnull, NS_OK);
-  
+  evt->mLoadGroup->RemoveRequest(evt, nsnull, NS_OK);
+
   return nsnull;
 }
 
@@ -673,7 +660,12 @@ PR_STATIC_CALLBACK(void)
 DestroyImagePLEvent(PLEvent* aEvent)
 {
   ImageEvent* evt = NS_STATIC_CAST(ImageEvent*, aEvent);
-  delete evt;
+
+  // We're reference counted, and we hold a strong reference to
+  // ourselves while we're a 'live' PLEvent. Now that the PLEvent is
+  // destroyed, release ourselves.
+
+  NS_RELEASE(evt);
 }
 
 nsresult
@@ -703,10 +695,6 @@ nsImageLoadingContent::FireEvent(const nsAString& aEventType)
   nsCOMPtr<nsILoadGroup> loadGroup;
   document->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
 
-  nsCOMPtr<nsIRequest> dummyRequest;
-  rv = nsDummyLayoutRequest::Create(getter_AddRefs(dummyRequest), nsnull);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   nsCOMPtr<nsIPresShell> shell;
   document->GetShellAt(0, getter_AddRefs(shell));
   NS_ENSURE_TRUE(shell, NS_ERROR_FAILURE);
@@ -718,18 +706,23 @@ nsImageLoadingContent::FireEvent(const nsAString& aEventType)
   nsCOMPtr<nsIContent> ourContent = do_QueryInterface(this);
   
   ImageEvent* evt = new ImageEvent(presContext, ourContent, aEventType,
-                                   loadGroup, dummyRequest);
+                                   loadGroup);
 
   NS_ENSURE_TRUE(evt, NS_ERROR_OUT_OF_MEMORY);
 
   PL_InitEvent(evt, this, ::HandleImagePLEvent, ::DestroyImagePLEvent);
 
+  // The event will own itself while it's in the event queue, once
+  // removed, it will release itself, and if there are no other
+  // references, it will be deleted.
+  NS_ADDREF(evt);
+
   rv = eventQ->PostEvent(evt);
 
   if (rv == PR_SUCCESS) {
-    // Add the dummy request to the load group only after all the early
-    // returns here!
-    loadGroup->AddRequest(dummyRequest, nsnull);
+    // Add the dummy request (the ImageEvent) to the load group only
+    // after all the early returns here!
+    loadGroup->AddRequest(evt, nsnull);
   } else {
     PL_DestroyEvent(evt);
   }
