@@ -95,7 +95,6 @@ nsNodeInfoManager::NodeInfoInnerKeyCompare(const void *key1, const void *key2)
 nsNodeInfoManager::nsNodeInfoManager()
   : mDocument(nsnull)
 {
-  NS_INIT_ISUPPORTS();
 
   if (gNodeManagerCount == 1 && gAnonymousNodeInfoManager) {
     /*
@@ -186,7 +185,7 @@ nsNodeInfoManager::DropDocumentReference()
 
 NS_IMETHODIMP
 nsNodeInfoManager::GetNodeInfo(nsIAtom *aName, nsIAtom *aPrefix,
-                               PRInt32 aNamespaceID, nsINodeInfo*& aNodeInfo)
+                               PRInt32 aNamespaceID, nsINodeInfo** aNodeInfo)
 {
   NS_ENSURE_ARG_POINTER(aName);
 
@@ -195,9 +194,9 @@ nsNodeInfoManager::GetNodeInfo(nsIAtom *aName, nsIAtom *aPrefix,
   void *node = PL_HashTableLookup(mNodeInfoHash, &tmpKey);
 
   if (node) {
-    aNodeInfo = NS_STATIC_CAST(nsINodeInfo *, node);
+    *aNodeInfo = NS_STATIC_CAST(nsINodeInfo *, node);
 
-    NS_ADDREF(aNodeInfo);
+    NS_ADDREF(*aNodeInfo);
 
     return NS_OK;
   }
@@ -214,7 +213,7 @@ nsNodeInfoManager::GetNodeInfo(nsIAtom *aName, nsIAtom *aPrefix,
   he = PL_HashTableAdd(mNodeInfoHash, &newNodeInfo->mInner, newNodeInfo);
   NS_ENSURE_TRUE(he, NS_ERROR_OUT_OF_MEMORY);
 
-  aNodeInfo = newNodeInfo;
+  *aNodeInfo = newNodeInfo;
 
   return NS_OK;
 }
@@ -222,11 +221,11 @@ nsNodeInfoManager::GetNodeInfo(nsIAtom *aName, nsIAtom *aPrefix,
 
 NS_IMETHODIMP
 nsNodeInfoManager::GetNodeInfo(const nsAString& aName, nsIAtom *aPrefix,
-                               PRInt32 aNamespaceID, nsINodeInfo*& aNodeInfo)
+                               PRInt32 aNamespaceID, nsINodeInfo** aNodeInfo)
 {
   NS_ENSURE_ARG(!aName.IsEmpty());
 
-  nsCOMPtr<nsIAtom> name(dont_AddRef(NS_NewAtom(aName)));
+  nsCOMPtr<nsIAtom> name = do_GetAtom(aName);
   NS_ENSURE_TRUE(name, NS_ERROR_OUT_OF_MEMORY);
 
   return GetNodeInfo(name, aPrefix, aNamespaceID, aNodeInfo);
@@ -236,17 +235,17 @@ nsNodeInfoManager::GetNodeInfo(const nsAString& aName, nsIAtom *aPrefix,
 NS_IMETHODIMP
 nsNodeInfoManager::GetNodeInfo(const nsAString& aName,
                                const nsAString& aPrefix, PRInt32 aNamespaceID,
-                               nsINodeInfo*& aNodeInfo)
+                               nsINodeInfo** aNodeInfo)
 {
   NS_ENSURE_ARG(!aName.IsEmpty());
 
-  nsCOMPtr<nsIAtom> name(dont_AddRef(NS_NewAtom(aName)));
+  nsCOMPtr<nsIAtom> name = do_GetAtom(aName);
   NS_ENSURE_TRUE(name, NS_ERROR_OUT_OF_MEMORY);
 
   nsCOMPtr<nsIAtom> prefix;
 
   if (!aPrefix.IsEmpty()) {
-    prefix = dont_AddRef(NS_NewAtom(aPrefix));
+    prefix = do_GetAtom(aPrefix);
     NS_ENSURE_TRUE(prefix, NS_ERROR_OUT_OF_MEMORY);
   }
 
@@ -258,17 +257,17 @@ NS_IMETHODIMP
 nsNodeInfoManager::GetNodeInfo(const nsAString& aName,
                                const nsAString& aPrefix,
                                const nsAString& aNamespaceURI,
-                               nsINodeInfo*& aNodeInfo)
+                               nsINodeInfo** aNodeInfo)
 {
   NS_ENSURE_ARG(!aName.IsEmpty());
 
-  nsCOMPtr<nsIAtom> name(dont_AddRef(NS_NewAtom(aName)));
+  nsCOMPtr<nsIAtom> name = do_GetAtom(aName);
   NS_ENSURE_TRUE(name, NS_ERROR_OUT_OF_MEMORY);
 
   nsCOMPtr<nsIAtom> prefix;
 
   if (!aPrefix.IsEmpty()) {
-    prefix = dont_AddRef(NS_NewAtom(aPrefix));
+    prefix = do_GetAtom(aPrefix);
     NS_ENSURE_TRUE(prefix, NS_ERROR_OUT_OF_MEMORY);
   }
 
@@ -283,7 +282,7 @@ nsNodeInfoManager::GetNodeInfo(const nsAString& aName,
 NS_IMETHODIMP
 nsNodeInfoManager::GetNodeInfo(const nsAString& aQualifiedName,
                                const nsAString& aNamespaceURI,
-                               nsINodeInfo*& aNodeInfo)
+                               nsINodeInfo** aNodeInfo)
 {
   NS_ENSURE_ARG(!aQualifiedName.IsEmpty());
 
@@ -295,13 +294,13 @@ nsNodeInfoManager::GetNodeInfo(const nsAString& aQualifiedName,
     name.Cut(0, nsoffset+1);
   }
 
-  nsCOMPtr<nsIAtom> nameAtom(dont_AddRef(NS_NewAtom(name)));
+  nsCOMPtr<nsIAtom> nameAtom = do_GetAtom(name);
   NS_ENSURE_TRUE(nameAtom, NS_ERROR_OUT_OF_MEMORY);
 
   nsCOMPtr<nsIAtom> prefixAtom;
 
   if (!prefix.IsEmpty()) {
-    prefixAtom = dont_AddRef(NS_NewAtom(prefix));
+    prefixAtom = do_GetAtom(prefix);
     NS_ENSURE_TRUE(prefixAtom, NS_ERROR_OUT_OF_MEMORY);
   }
 
@@ -316,11 +315,9 @@ nsNodeInfoManager::GetNodeInfo(const nsAString& aQualifiedName,
 }
 
 NS_IMETHODIMP
-nsNodeInfoManager::GetDocument(nsIDocument*& aDocument)
+nsNodeInfoManager::GetDocument(nsIDocument** aDocument)
 {
-  aDocument = mDocument;
-
-  NS_IF_ADDREF(aDocument);
+  NS_IF_ADDREF(*aDocument = mDocument);
 
   return NS_OK;
 }
@@ -408,7 +405,7 @@ nsNodeInfoManager::RemoveNodeInfo(nsNodeInfo *aNodeInfo)
 
 
 nsresult
-nsNodeInfoManager::GetAnonymousManager(nsINodeInfoManager*& aNodeInfoManager)
+nsNodeInfoManager::GetAnonymousManager(nsINodeInfoManager** aNodeInfoManager)
 {
   if (!gAnonymousNodeInfoManager) {
     gAnonymousNodeInfoManager = new nsNodeInfoManager;
@@ -419,7 +416,7 @@ nsNodeInfoManager::GetAnonymousManager(nsINodeInfoManager*& aNodeInfoManager)
     NS_ADDREF(gAnonymousNodeInfoManager);
   }
 
-  aNodeInfoManager = gAnonymousNodeInfoManager;
+  *aNodeInfoManager = gAnonymousNodeInfoManager;
 
   /*
    * If the only nodeinfo manager is the global one we don't hold a ref
@@ -427,7 +424,7 @@ nsNodeInfoManager::GetAnonymousManager(nsINodeInfoManager*& aNodeInfoManager)
    * even if it's the last one arround.
    */
   if (gNodeManagerCount > 1) {
-    NS_ADDREF(aNodeInfoManager);
+    NS_ADDREF(*aNodeInfoManager);
   }
 
   return NS_OK;
