@@ -25,7 +25,6 @@
 #include "nsIDOMCharacterData.h"
 #include "nsParserUtils.h"
 #include "nsICharsetConverterManager.h"
-#include "nsICharsetConverterManager2.h"
 #include "nsIUnicodeDecoder.h"
 #include "nsICharsetAlias.h"
 #include "nsIContent.h"
@@ -42,7 +41,11 @@
 #include "nsIScriptElement.h"
 #include "nsIDocShell.h"
 #include "jsapi.h"
+#include "nsContentUtils.h"
+#include "nsUnicharUtils.h"
 
+
+static NS_DEFINE_CID(kCharsetAliasCID, NS_CHARSETALIAS_CID);
 static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
 
 //////////////////////////////////////////////////////////////
@@ -85,7 +88,6 @@ nsScriptLoadRequest::nsScriptLoadRequest(nsIDOMHTMLScriptElement* aElement,
   mLoading(PR_TRUE), mWasPending(PR_FALSE),
   mIsInline(PR_TRUE), mJSVersion(aVersionString), mLineNo(1)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsScriptLoadRequest::~nsScriptLoadRequest()
@@ -125,7 +127,6 @@ nsScriptLoadRequest::FireScriptEvaluated(nsresult aResult)
 nsScriptLoader::nsScriptLoader() 
   : mDocument(nsnull), mEnabled(PR_TRUE)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsScriptLoader::~nsScriptLoader()
@@ -208,12 +209,11 @@ nsScriptLoader::InNonScriptingContainer(nsIDOMHTMLScriptElement* aScriptElement)
     }
     
     nsCOMPtr<nsINodeInfo> nodeInfo;
-    content->GetNodeInfo(*getter_AddRefs(nodeInfo));
+    content->GetNodeInfo(getter_AddRefs(nodeInfo));
     NS_ASSERTION(nodeInfo, "element without node info");
 
     if (nodeInfo) {
-      nsCOMPtr<nsIAtom> localName;
-      nodeInfo->GetNameAtom(*getter_AddRefs(localName));
+      nsCOMPtr<nsIAtom> localName = nodeInfo->GetNameAtom();
       
       // XXX noframes and noembed are currently unconditionally not
       // displayed and processed. This might change if we support either
@@ -234,6 +234,72 @@ nsScriptLoader::InNonScriptingContainer(nsIDOMHTMLScriptElement* aScriptElement)
   return PR_FALSE;
 }
 
+// Helper method for checking if the script element is an event-handler
+// This means that it has both a for-attribute and a event-attribute.
+// Also, if the for-attribute has a value that matches "\s*window\s*",
+// and the event-attribute matches "\s*onload([ \(].*)?" then it isn't an
+// eventhandler. (both matches are case insensitive).
+// This is how IE seems to filter out a window's onload handler from a
+// <script for=... event=...> element.
+
+PRBool
+nsScriptLoader::IsScriptEventHandler(nsIDOMHTMLScriptElement *aScriptElement)
+{
+  nsCOMPtr<nsIContent> contElement = do_QueryInterface(aScriptElement);
+  if (!contElement ||
+      !contElement->HasAttr(kNameSpaceID_None, nsHTMLAtoms::_event) ||
+      !contElement->HasAttr(kNameSpaceID_None, nsHTMLAtoms::_for)) {
+      return PR_FALSE;
+  }
+
+  nsAutoString str;
+  nsresult rv = contElement->GetAttr(kNameSpaceID_None, nsHTMLAtoms::_for,
+                                     str);
+  NS_ENSURE_SUCCESS(rv, PR_FALSE);
+
+  const nsAString& for_str = nsContentUtils::TrimWhitespace(str);
+
+  if (!for_str.Equals(NS_LITERAL_STRING("window"),
+                      nsCaseInsensitiveStringComparator())) {
+    return PR_TRUE;
+  }
+
+  // We found for="window", now check for event="onload".
+
+  rv = contElement->GetAttr(kNameSpaceID_None, nsHTMLAtoms::_event, str);
+  NS_ENSURE_SUCCESS(rv, PR_FALSE);
+
+  const nsAString& event_str = nsContentUtils::TrimWhitespace(str, PR_FALSE);
+
+  if (event_str.Length() < 6) {
+    // String too short, can't be "onload".
+
+    return PR_TRUE;
+  }
+
+  if (!Substring(event_str, 0, 6).Equals(NS_LITERAL_STRING("onload"),
+                                         nsCaseInsensitiveStringComparator())) {
+    // It ain't "onload.*".
+
+    return PR_TRUE;
+  }
+
+  nsAutoString::const_iterator start, end;
+  event_str.BeginReading(start);
+  event_str.EndReading(end);
+
+  start.advance(6); // advance past "onload"
+
+  if (start != end && *start != '(' && *start != ' ') {
+    // We got onload followed by something other than space or
+    // '('. Not good enough.
+
+    return PR_TRUE;
+  }
+
+  return PR_FALSE;
+}
+
 /* void processScriptElement (in nsIDOMHTMLScriptElement aElement, in nsIScriptLoaderObserver aObserver); */
 NS_IMETHODIMP 
 nsScriptLoader::ProcessScriptElement(nsIDOMHTMLScriptElement *aElement, 
@@ -254,6 +320,12 @@ nsScriptLoader::ProcessScriptElement(nsIDOMHTMLScriptElement *aElement,
     return FireErrorNotification(NS_ERROR_NOT_AVAILABLE, aElement, aObserver);
   }
 
+  // Check that the script is not an eventhandler
+  if (IsScriptEventHandler(aElement)) {
+    return FireErrorNotification(NS_CONTENT_SCRIPT_IS_EVENTHANDLER, aElement,
+                                 aObserver);
+  }
+
   // See if script evaluation is enabled.
   PRBool scriptsEnabled = PR_TRUE;
   nsCOMPtr<nsIScriptGlobalObject> globalObject;
@@ -329,7 +401,9 @@ nsScriptLoader::ProcessScriptElement(nsIDOMHTMLScriptElement *aElement,
     nsCOMPtr<nsIURI> baseURI, scriptURI;
     
     // Use the SRC attribute value to load the URL
-    mDocument->GetBaseURL(*getter_AddRefs(baseURI));
+    nsCOMPtr<nsIContent> content(do_QueryInterface(aElement));
+    NS_ASSERTION(content, "nsIDOMHTMLScriptElement not implementing nsIContent");
+    content->GetBaseURL(getter_AddRefs(baseURI));
     rv = NS_NewURI(getter_AddRefs(scriptURI), src, nsnull, baseURI);
     if (NS_FAILED(rv)) {
       return FireErrorNotification(rv, aElement, aObserver);
@@ -603,7 +677,7 @@ nsScriptLoader::ProcessPendingReqests()
 
 // This function is copied from nsParser.cpp. It was simplified though, unnecessary part is removed.
 static PRBool 
-DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsString& oCharset) 
+DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsCString& oCharset) 
 {
   if (aLen < 2)
     return PR_FALSE;
@@ -613,25 +687,25 @@ DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsString& oCharse
     if( aLen >= 3 && (0xBB==aBytes[1]) && (0xBF==aBytes[2])) {
         // EF BB BF
         // Win2K UTF-8 BOM
-        oCharset.AssignWithConversion("UTF-8"); 
+        oCharset.Assign("UTF-8"); 
     }
     break;
   case 0xFE:
     if(0xFF==aBytes[1]) {
       // FE FF
       // UTF-16, big-endian 
-      oCharset.AssignWithConversion("UTF-16BE"); 
+      oCharset.Assign("UTF-16BE"); 
     }
     break;
   case 0xFF:
     if(0xFE==aBytes[1]) {
       // FF FE
       // UTF-16, little-endian 
-      oCharset.AssignWithConversion("UTF-16LE"); 
+      oCharset.Assign("UTF-16LE"); 
     }
     break;
   }  // switch
-  return oCharset.Length() > 0;
+  return !oCharset.IsEmpty();
 }
 
 
@@ -685,22 +759,25 @@ nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
   }
 
   if (stringLen) {
-    nsAutoString characterSet, preferred;
+    nsCAutoString characterSet, preferred;
     nsCOMPtr<nsIUnicodeDecoder> unicodeDecoder;
 
     nsCOMPtr<nsIChannel> channel;
 
     channel = do_QueryInterface(req);
+    nsAutoString charset;
     if (channel) {
       nsCAutoString charsetVal;
       rv = channel->GetContentCharset(charsetVal);
     
       if (NS_SUCCEEDED(rv)) {
-        characterSet = NS_ConvertASCIItoUCS2(charsetVal);      
+        charset = NS_ConvertASCIItoUCS2(charsetVal);      
         nsCOMPtr<nsICharsetAlias> calias(do_GetService(kCharsetAliasCID,&rv));
 
         if(NS_SUCCEEDED(rv) && calias) {
-          rv = calias->GetPreferred(characterSet, preferred);
+          NS_LossyConvertUCS2toASCII asciiCharset(charset);
+
+          rv = calias->GetPreferred(asciiCharset, preferred);
 
           if(NS_SUCCEEDED(rv)) {
             characterSet = preferred;
@@ -710,7 +787,6 @@ nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
     }
 
     if (NS_FAILED(rv) || characterSet.IsEmpty()) {
-      nsAutoString charset;
       // Check the charset attribute to determine script charset.
       request->mElement->GetCharset(charset);
       if (!charset.IsEmpty()) {
@@ -718,7 +794,9 @@ nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
         // is one.
         nsCOMPtr<nsICharsetAlias> calias(do_GetService(kCharsetAliasCID,&rv));
         if (NS_SUCCEEDED(rv)) {
-          rv = calias->GetPreferred(charset, preferred);
+          NS_LossyConvertUCS2toASCII asciiCharset(charset);
+
+          rv = calias->GetPreferred(asciiCharset, preferred);
           
           if(NS_SUCCEEDED(rv)) {
             characterSet = preferred;
@@ -740,15 +818,15 @@ nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
 
     if (characterSet.IsEmpty()) {
       // fall back to ISO-8851-1, see bug 118404
-      characterSet = NS_LITERAL_STRING("ISO-8859-1");
+      characterSet = NS_LITERAL_CSTRING("ISO-8859-1");
     }
     
     nsCOMPtr<nsICharsetConverterManager> charsetConv =
       do_GetService(kCharsetConverterManagerCID, &rv);
 
     if (NS_SUCCEEDED(rv) && charsetConv) {
-      rv = charsetConv->GetUnicodeDecoder(&characterSet,
-                                          getter_AddRefs(unicodeDecoder));
+      rv = charsetConv->GetUnicodeDecoderRaw(characterSet.get(),
+                                             getter_AddRefs(unicodeDecoder));
     }
 
     // converts from the charset to unicode
