@@ -55,7 +55,6 @@
 #include "prprf.h"
 #include "nsUnicharUtils.h"
 #include "nsCRT.h"
-#include "nsIXMLDocument.h"
 #include "nsContentUtils.h"
 
 typedef struct {
@@ -71,12 +70,11 @@ nsresult NS_NewXMLContentSerializer(nsIContentSerializer** aSerializer)
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  return it->QueryInterface(NS_GET_IID(nsIContentSerializer), (void**)aSerializer);
+  return CallQueryInterface(it, aSerializer);
 }
 
 nsXMLContentSerializer::nsXMLContentSerializer()
 {
-  NS_INIT_ISUPPORTS();
   mPrefixIndex = 0;
   mInAttribute = PR_FALSE;
 }
@@ -89,7 +87,7 @@ NS_IMPL_ISUPPORTS1(nsXMLContentSerializer, nsIContentSerializer)
 
 NS_IMETHODIMP 
 nsXMLContentSerializer::Init(PRUint32 flags, PRUint32 aWrapColumn,
-                             nsIAtom* aCharSet, PRBool aIsCopying)
+                             const char* aCharSet, PRBool aIsCopying)
 {
   return NS_OK;
 }
@@ -282,11 +280,12 @@ nsXMLContentSerializer::AppendDoctype(nsIDOMDocumentType *aDoctype,
   }
   
   if (!internalSubset.IsEmpty()) {
-    AppendToString(PRUnichar(' '), aStr);
+    AppendToString(NS_LITERAL_STRING(" ["), aStr);
     AppendToString(internalSubset, aStr);
+    AppendToString(PRUnichar(']'), aStr);
   }
     
-  AppendToString(NS_LITERAL_STRING(">"), aStr);
+  AppendToString(PRUnichar('>'), aStr);
 
   return NS_OK;
 }
@@ -396,17 +395,76 @@ nsXMLContentSerializer::SerializeAttr(const nsAString& aPrefix,
   AppendToString(PRUnichar(' '), aStr);
   if (!aPrefix.IsEmpty()) {
     AppendToString(aPrefix, aStr);
-    AppendToString(NS_LITERAL_STRING(":"), aStr);
+    AppendToString(PRUnichar(':'), aStr);
   }
   AppendToString(aName, aStr);
   
-  AppendToString(NS_LITERAL_STRING("=\""), aStr);
+  if ( aDoEscapeEntities ) {
+    // if problem characters are turned into character entity references
+    // then there will be no problem with the value delimiter characters
+    AppendToString(NS_LITERAL_STRING("=\""), aStr);
 
-  mInAttribute = PR_TRUE;
-  AppendToString(aValue, aStr, aDoEscapeEntities);
-  mInAttribute = PR_FALSE;
+    mInAttribute = PR_TRUE;
+    AppendToString(aValue, aStr, PR_TRUE);
+    mInAttribute = PR_FALSE;
 
-  AppendToString(NS_LITERAL_STRING("\""), aStr);
+    AppendToString(PRUnichar('"'), aStr);
+  }
+  else {
+    // Depending on whether the attribute value contains quotes or apostrophes we
+    // need to select the delimiter character and escape characters using
+    // character entity references, ignoring the value of aDoEscapeEntities.
+    // See http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2.2 for
+    // the standard on character entity references in values. 
+    PRBool bIncludesSingle = PR_FALSE;
+    PRBool bIncludesDouble = PR_FALSE;
+    nsAString::const_iterator iCurr, iEnd;
+    PRUint32 uiSize, i;
+    aValue.BeginReading(iCurr);
+    aValue.EndReading(iEnd);
+    for ( ; iCurr != iEnd; iCurr.advance(uiSize) ) {
+      const PRUnichar * buf = iCurr.get();
+      uiSize = iCurr.size_forward();
+      for ( i = 0; i < uiSize; i++, buf++ ) {
+        if ( *buf == PRUnichar('\'') )
+        {
+          bIncludesSingle = PR_TRUE;
+          if ( bIncludesDouble ) break;
+        }
+        else if ( *buf == PRUnichar('"') )
+        {
+          bIncludesDouble = PR_TRUE;
+          if ( bIncludesSingle ) break;
+        }
+      }
+      // if both have been found we don't need to search further
+      if ( bIncludesDouble && bIncludesSingle ) break;
+    }
+
+    // Delimiter and escaping is according to the following table
+    //    bIncludesDouble     bIncludesSingle     Delimiter       Escape Double Quote
+    //    FALSE               FALSE               "               FALSE
+    //    FALSE               TRUE                "               FALSE
+    //    TRUE                FALSE               '               FALSE
+    //    TRUE                TRUE                "               TRUE
+    PRUnichar cDelimiter = 
+        (bIncludesDouble && !bIncludesSingle) ? PRUnichar('\'') : PRUnichar('"');
+    AppendToString(PRUnichar('='), aStr);
+    AppendToString(cDelimiter, aStr);
+    if (bIncludesDouble && bIncludesSingle) {
+      nsAutoString sValue(aValue);
+      sValue.ReplaceSubstring(NS_LITERAL_STRING("\"").get(), NS_LITERAL_STRING("&quot;").get());
+      mInAttribute = PR_TRUE;
+      AppendToString(sValue, aStr, PR_FALSE);
+      mInAttribute = PR_FALSE;
+    }
+    else {
+      mInAttribute = PR_TRUE;
+      AppendToString(aValue, aStr, PR_FALSE);
+      mInAttribute = PR_FALSE;
+    }
+    AppendToString(cDelimiter, aStr);
+  }
 }
 
 NS_IMETHODIMP 
@@ -438,13 +496,12 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   // First scan for namespace declarations, pushing each on the stack
   for (index = 0; index < count; index++) {
     
-    content->GetAttrNameAt(index, 
-                           namespaceID,
-                           *getter_AddRefs(attrName),
-                           *getter_AddRefs(attrPrefix));
+    content->GetAttrNameAt(index,
+                           &namespaceID,
+                           getter_AddRefs(attrName),
+                           getter_AddRefs(attrPrefix));
     
     if (namespaceID == kNameSpaceID_XMLNS) {
-      PRBool hasPrefix = attrPrefix ? PR_TRUE : PR_FALSE;
       content->GetAttr(namespaceID, attrName, uriStr);
 
       if (!attrPrefix) {
@@ -479,10 +536,10 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   // XXX Unfortunately we need a namespace manager to get
   // attribute URIs.
   for (index = 0; index < count; index++) {
-    content->GetAttrNameAt(index, 
-                           namespaceID,
-                           *getter_AddRefs(attrName),
-                           *getter_AddRefs(attrPrefix));
+    content->GetAttrNameAt(index,
+                           &namespaceID,
+                           getter_AddRefs(attrName),
+                           getter_AddRefs(attrPrefix));
 
     if (attrPrefix) {
       attrPrefix->ToString(prefixStr);
@@ -509,10 +566,10 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
 
     if (namespaceID == kNameSpaceID_None) {
       PRInt32 elementNsID;
-      content->GetNameSpaceID(elementNsID);
+      content->GetNameSpaceID(&elementNsID);
       if (elementNsID == kNameSpaceID_XHTML) {
         nsCOMPtr<nsIAtom> elementName;
-        content->GetTag(*getter_AddRefs(elementName));
+        content->GetTag(getter_AddRefs(elementName));
         if (IsShorthandAttr(attrName, elementName) &&
             valueStr.IsEmpty()) {
           valueStr = nameStr;
@@ -591,7 +648,7 @@ nsXMLContentSerializer::AppendToString(const PRUnichar aChar,
   aOutputStr.Append(aChar);
 }
 
-static PRUint16 kGTVal = 62;
+static const PRUint16 kGTVal = 62;
 static const char* kEntities[] = {
   "", "", "", "", "", "", "", "", "", "",
   "", "", "", "", "", "", "", "", "", "",
@@ -758,13 +815,13 @@ nsXMLContentSerializer::AppendDocumentStart(nsIDOMDocument *aDocument,
 {
   NS_ENSURE_ARG_POINTER(aDocument);
 
-  nsCOMPtr<nsIXMLDocument> xml(do_QueryInterface(aDocument));
-  if (!xml) {
+  nsCOMPtr<nsIDocument> doc(do_QueryInterface(aDocument));
+  if (!doc) {
     return NS_OK;
   }
 
   nsAutoString version, encoding, standalone;
-  xml->GetXMLDeclaration(version, encoding, standalone);
+  doc->GetXMLDeclaration(version, encoding, standalone);
 
   if (version.IsEmpty())
     return NS_OK; // A declaration must have version, or there is no decl
