@@ -38,7 +38,6 @@
  *
  * ***** END LICENSE BLOCK ***** */
 #include "nsCOMPtr.h"
-#include "nsHTMLUtils.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
 #include "nsIDOMHTMLAnchorElement.h"
@@ -106,7 +105,7 @@ public:
   // nsILink
   NS_IMETHOD GetLinkState(nsLinkState &aState);
   NS_IMETHOD SetLinkState(nsLinkState aState);
-  NS_IMETHOD GetHrefCString(char* &aBuf);
+  NS_IMETHOD GetHrefURI(nsIURI** aURI);
 
   NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep,
                          PRBool aCompileEventHandlers);
@@ -128,10 +127,6 @@ public:
   NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
                        PRBool aNotify);
 
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
-
 protected:
   // The cached visited state
   nsLinkState mLinkState;
@@ -168,12 +163,10 @@ NS_NewHTMLAnchorElement(nsIHTMLContent** aInstancePtrResult,
 
 nsHTMLAnchorElement::nsHTMLAnchorElement() : mLinkState(eLinkState_Unknown)
 {
-  nsHTMLUtils::AddRef(); // for GetHrefCString
 }
 
 nsHTMLAnchorElement::~nsHTMLAnchorElement()
 {
-  nsHTMLUtils::Release(); // for GetHrefCString
 }
 
 
@@ -222,6 +215,7 @@ nsHTMLAnchorElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
 
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Charset, charset)
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Coords, coords)
+NS_IMPL_URI_ATTR(nsHTMLAnchorElement, Href, href)
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Hreflang, hreflang)
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Name, name)
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Rel, rel)
@@ -314,7 +308,7 @@ nsHTMLAnchorElement::RemoveFocus(nsIPresContext* aPresContext)
 
   if (esm) {
     nsCOMPtr<nsIDocument> doc;
-    GetDocument(*getter_AddRefs(doc));
+    GetDocument(getter_AddRefs(doc));
     if (!doc)
       return NS_ERROR_NULL_POINTER;
 
@@ -332,7 +326,7 @@ nsHTMLAnchorElement::StringToAttribute(nsIAtom* aAttribute,
                                        nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::tabindex) {
-    if (ParseValue(aValue, 0, 32767, aResult, eHTMLUnit_Integer)) {
+    if (aResult.ParseIntWithBounds(aValue, eHTMLUnit_Integer, 0, 32767)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -354,30 +348,10 @@ nsHTMLAnchorElement::HandleDOMEvent(nsIPresContext* aPresContext,
                                     PRUint32 aFlags,
                                     nsEventStatus* aEventStatus)
 {
-  return HandleDOMEventForAnchors(this, aPresContext, aEvent, aDOMEvent, 
+  return HandleDOMEventForAnchors(aPresContext, aEvent, aDOMEvent, 
                                   aFlags, aEventStatus);
 }
 
-NS_IMETHODIMP
-nsHTMLAnchorElement::GetHref(nsAString& aValue)
-{
-  char *buf;
-  nsresult rv = GetHrefCString(buf);
-  if (NS_FAILED(rv)) return rv;
-  if (buf) {
-    aValue.Assign(NS_ConvertUTF8toUCS2(buf));
-    nsCRT::free(buf);
-  }
-  // NS_IMPL_STRING_ATTR does nothing where we have (buf == null)
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLAnchorElement::SetHref(const nsAString& aValue)
-{
-  return SetAttr(kNameSpaceID_None, nsHTMLAtoms::href, aValue, PR_TRUE);
-}
-
 NS_IMETHODIMP
 nsHTMLAnchorElement::GetTarget(nsAString& aValue)
 {
@@ -398,17 +372,6 @@ nsHTMLAnchorElement::SetTarget(const nsAString& aValue)
   return SetAttr(kNameSpaceID_None, nsHTMLAtoms::target, aValue, PR_TRUE);
 }
 
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLAnchorElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
-
-
 NS_IMETHODIMP    
 nsHTMLAnchorElement::GetProtocol(nsAString& aProtocol)
 {
@@ -419,7 +382,7 @@ nsHTMLAnchorElement::GetProtocol(nsAString& aProtocol)
     return rv;
 
   nsCOMPtr<nsIDocument> doc;
-  mNodeInfo->GetDocument(*getter_AddRefs(doc));
+  mNodeInfo->GetDocument(getter_AddRefs(doc));
 
   return GetProtocolFromHrefString(href, aProtocol, doc);
 }
@@ -676,39 +639,9 @@ nsHTMLAnchorElement::SetLinkState(nsLinkState aState)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::GetHrefCString(char* &aBuf)
+nsHTMLAnchorElement::GetHrefURI(nsIURI** aURI)
 {
-  // Get href= attribute (relative URL).
-  nsAutoString relURLSpec;
-
-  if (NS_CONTENT_ATTR_HAS_VALUE ==
-      GetAttr(kNameSpaceID_None, nsHTMLAtoms::href, relURLSpec)) {
-    // Clean up any leading or trailing whitespace
-    relURLSpec.Trim(" \t\n\r");
-
-    // Get base URL.
-    nsCOMPtr<nsIURI> baseURL;
-    GetBaseURL(*getter_AddRefs(baseURL));
-
-    if (baseURL) {
-      // Get absolute URL.
-      nsCAutoString buf;
-      NS_MakeAbsoluteURIWithCharset(buf, relURLSpec, mDocument, baseURL,
-                                    nsHTMLUtils::IOService,
-                                    nsHTMLUtils::CharsetMgr);
-      aBuf = ToNewCString(buf);
-    }
-    else {
-      // Absolute URL is same as relative URL.
-      aBuf = ToNewUTF8String(relURLSpec);
-    }
-  }
-  else {
-    // Absolute URL is null to say we have no HREF.
-    aBuf = nsnull;
-  }
-
-  return NS_OK;
+  return GetHrefURIForAnchors(aURI);
 }
 
 NS_IMETHODIMP
