@@ -40,8 +40,8 @@
 #include "nsIDOMEventReceiver.h"
 #include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
+#include "nsImageLoadingContent.h"
 #include "nsHTMLAtoms.h"
-#include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
@@ -59,7 +59,6 @@
 #include "nsIServiceManager.h"
 #include "nsNetUtil.h"
 #include "nsContentUtils.h"
-#include "nsIWebShell.h"
 #include "nsIFrame.h"
 #include "nsIImageFrame.h"
 #include "nsLayoutAtoms.h"
@@ -67,7 +66,8 @@
 #include "nsGUIEvent.h"
 #include "nsContentPolicyUtils.h"
 #include "nsIDOMWindow.h"
-#include "nsIPref.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
 
 #include "imgIContainer.h"
 #include "imgILoader.h"
@@ -81,15 +81,13 @@
 #include "nsIJSContextStack.h"
 #include "nsIView.h"
 
-static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
-
 // XXX nav attrs: suppress
 
 class nsHTMLImageElement : public nsGenericHTMLLeafElement,
+                           public nsImageLoadingContent,
                            public nsIDOMHTMLImageElement,
                            public nsIDOMNSHTMLImageElement,
-                           public nsIJSNativeInitializer,
-                           public imgIDecoderObserver
+                           public nsIJSNativeInitializer
 {
 public:
   nsHTMLImageElement();
@@ -113,9 +111,6 @@ public:
   // nsIDOMNSHTMLImageElement
   NS_DECL_NSIDOMNSHTMLIMAGEELEMENT
 
-  NS_DECL_IMGIDECODEROBSERVER
-  NS_DECL_IMGICONTAINEROBSERVER
-
   // nsIJSNativeInitializer
   NS_IMETHOD Initialize(JSContext* aContext, JSObject *aObj,
                         PRUint32 argc, jsval *argv);
@@ -134,18 +129,21 @@ public:
   NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
                             nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
                             nsEventStatus* aEventStatus);
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
 
+  // SetAttr override.  C++ is stupid, so have to override both
+  // overloaded methods.
+  NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                     const nsAString& aValue, PRBool aNotify);
+  NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo, const nsAString& aValue,
+                     PRBool aNotify);
+  
+  // XXXbz What about UnsetAttr?  We don't seem to unload images when
+  // that happens...
+  
 protected:
-  nsresult SetSrcInner(nsIURI* aBaseURL, const nsAString& aSrc);
-
   void GetImageFrame(nsIImageFrame** aImageFrame);
   nsresult GetXY(PRInt32* aX, PRInt32* aY);
   nsresult GetWidthHeight(PRInt32* aWidth, PRInt32* aHeight);
-
-  nsCOMPtr<imgIRequest> mRequest;
 };
 
 nsresult
@@ -169,12 +167,12 @@ NS_NewHTMLImageElement(nsIHTMLContent** aInstancePtrResult,
     NS_ENSURE_TRUE(doc, NS_ERROR_UNEXPECTED);
 
     nsCOMPtr<nsINodeInfoManager> nodeInfoManager;
-    doc->GetNodeInfoManager(*getter_AddRefs(nodeInfoManager));
+    doc->GetNodeInfoManager(getter_AddRefs(nodeInfoManager));
     NS_ENSURE_TRUE(nodeInfoManager, NS_ERROR_UNEXPECTED);
 
     rv = nodeInfoManager->GetNodeInfo(nsHTMLAtoms::img, nsnull,
                                       kNameSpaceID_None,
-                                      *getter_AddRefs(nodeInfo));
+                                      getter_AddRefs(nodeInfo));
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -219,6 +217,7 @@ NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLImageElement,
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSHTMLImageElement)
   NS_INTERFACE_MAP_ENTRY(nsIJSNativeInitializer)
   NS_INTERFACE_MAP_ENTRY(imgIDecoderObserver)
+  NS_INTERFACE_MAP_ENTRY(nsIImageLoadingContent)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(HTMLImageElement)
 NS_HTML_CONTENT_INTERFACE_MAP_END
 
@@ -255,10 +254,12 @@ nsHTMLImageElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, Name, name)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, Align, align)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, Alt, alt)
-NS_IMPL_PIXEL_ATTR(nsHTMLImageElement, Border, border)
+NS_IMPL_STRING_ATTR(nsHTMLImageElement, Border, border)
 NS_IMPL_PIXEL_ATTR(nsHTMLImageElement, Hspace, hspace)
 NS_IMPL_BOOL_ATTR(nsHTMLImageElement, IsMap, ismap)
-NS_IMPL_STRING_ATTR(nsHTMLImageElement, LongDesc, longdesc)
+NS_IMPL_URI_ATTR(nsHTMLImageElement, LongDesc, longdesc)
+NS_IMPL_STRING_ATTR(nsHTMLImageElement, Lowsrc, lowsrc)
+NS_IMPL_URI_ATTR_GETTER(nsHTMLImageElement, Src, src)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, UseMap, usemap)
 NS_IMPL_PIXEL_ATTR(nsHTMLImageElement, Vspace, vspace)
 
@@ -280,21 +281,19 @@ nsHTMLImageElement::GetImageFrame(nsIImageFrame** aImageFrame)
 NS_IMETHODIMP
 nsHTMLImageElement::GetComplete(PRBool* aComplete)
 {
-  NS_ENSURE_ARG_POINTER(aComplete);
-  *aComplete = PR_FALSE;
+  NS_PRECONDITION(aComplete, "Null out param!");
+  *aComplete = PR_TRUE;
 
-  nsresult result;
-  nsIImageFrame* imageFrame;
-
-  GetImageFrame(&imageFrame);
-  if (imageFrame) {
-    result = imageFrame->IsImageComplete(aComplete);
-  } else {
-    result = NS_OK;
-
-    *aComplete = !mRequest;
+  if (!mCurrentRequest) {
+    return NS_OK;
   }
 
+  PRUint32 status;
+  mCurrentRequest->GetImageStatus(&status);
+  *aComplete =
+    (status &
+     (imgIRequest::STATUS_LOAD_COMPLETE | imgIRequest::STATUS_ERROR)) != 0;
+
   return NS_OK;
 }
 
@@ -384,6 +383,13 @@ nsHTMLImageElement::GetWidthHeight(PRInt32* aWidth, PRInt32* aHeight)
     *aWidth = 0;
   }
 
+  if (mDocument) {
+    // Flush all pending notifications so that our frames are up to date.
+    // If we're not in a document, we don't have a frame anyway, so we
+    // don't care.
+    mDocument->FlushPendingNotifications();
+  }
+
   nsIImageFrame* imageFrame;
   GetImageFrame(&imageFrame);
 
@@ -395,6 +401,8 @@ nsHTMLImageElement::GetWidthHeight(PRInt32* aWidth, PRInt32* aHeight)
   }
 
   if (frame) {
+    // XXX we could put an accessor on nsIImageFrame to return its
+    // mComputedSize.....
     nsSize size;
     frame->GetSize(size);
 
@@ -421,12 +429,18 @@ nsHTMLImageElement::GetWidthHeight(PRInt32* aWidth, PRInt32* aHeight)
     }
   } else {
     nsHTMLValue value;
+    nsCOMPtr<imgIContainer> image;
+    if (mCurrentRequest) {
+      mCurrentRequest->GetImage(getter_AddRefs(image));
+    }
 
     if (aWidth) {
       nsresult rv = GetHTMLAttribute(nsHTMLAtoms::width, value);
 
       if (rv == NS_CONTENT_ATTR_HAS_VALUE) {
         *aWidth = value.GetPixelValue();
+      } else if (image) {
+        image->GetWidth(aWidth);
       }
     }
 
@@ -435,6 +449,8 @@ nsHTMLImageElement::GetWidthHeight(PRInt32* aWidth, PRInt32* aHeight)
 
       if (rv == NS_CONTENT_ATTR_HAS_VALUE) {
         *aHeight = value.GetPixelValue();
+      } else if (image) {
+        image->GetHeight(aHeight);
       }
     }
   }
@@ -542,18 +558,21 @@ nsHTMLImageElement::GetMappedAttributeImpact(const nsIAtom* aAttribute,
                                              PRInt32 aModType,
                                              nsChangeHint& aHint) const
 {
-  if ((aAttribute == nsHTMLAtoms::usemap) ||
-      (aAttribute == nsHTMLAtoms::ismap)  ||
-      (aAttribute == nsHTMLAtoms::align)) {
-    aHint = NS_STYLE_HINT_FRAMECHANGE;
-  }
-  else if (!GetCommonMappedAttributesImpact(aAttribute, aHint)) {
-    if (!GetImageMappedAttributesImpact(aAttribute, aHint)) {
-      if (!GetImageBorderAttributeImpact(aAttribute, aHint)) {
-        aHint = NS_STYLE_HINT_CONTENT;
-      }
-    }
-  }
+  static const AttributeImpactEntry attributes[] = {
+    { &nsHTMLAtoms::usemap, NS_STYLE_HINT_FRAMECHANGE },
+    { &nsHTMLAtoms::ismap, NS_STYLE_HINT_FRAMECHANGE },
+    { &nsHTMLAtoms::align, NS_STYLE_HINT_FRAMECHANGE },
+    { nsnull, NS_STYLE_HINT_NONE },
+  };
+
+  static const AttributeImpactEntry* const map[] = {
+    attributes,
+    sCommonAttributeMap,
+    sImageAttributeMap,
+    sImageBorderAttributeMap
+  };
+
+  FindAttributeImpact(aAttribute, aHint, map, NS_ARRAY_LENGTH(map));
 
   return NS_OK;
 }
@@ -591,6 +610,27 @@ nsHTMLImageElement::HandleDOMEvent(nsIPresContext* aPresContext,
                                                   aEventStatus);
 }
 
+NS_IMETHODIMP
+nsHTMLImageElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                            const nsAString& aValue, PRBool aNotify)
+{
+  // Call ImageURIChanged first so that the image load kicks off
+  // _before_ the reflow triggered by the SetAttr
+  if (aNameSpaceID == kNameSpaceID_None && aName == nsHTMLAtoms::src) {
+    ImageURIChanged(aValue);
+  }
+    
+  return nsGenericHTMLLeafElement::SetAttr(aNameSpaceID, aName,
+                                           aValue, aNotify);
+}
+
+NS_IMETHODIMP
+nsHTMLImageElement::SetAttr(nsINodeInfo* aNodeInfo, const nsAString& aValue,
+                            PRBool aNotify)
+{
+  return nsGenericHTMLLeafElement::SetAttr(aNodeInfo, aValue, aNotify);
+}
+
 NS_IMETHODIMP
 nsHTMLImageElement::Initialize(JSContext* aContext, JSObject *aObj,
                                PRUint32 argc, jsval *argv)
@@ -624,227 +664,6 @@ nsHTMLImageElement::Initialize(JSContext* aContext, JSObject *aObj,
   return rv;
 }
 
-NS_IMETHODIMP
-nsHTMLImageElement::GetSrc(nsAString& aSrc)
-{
-  // Resolve url to an absolute url
-  nsresult rv = NS_OK;
-  nsAutoString relURLSpec;
-  nsCOMPtr<nsIURI> baseURL;
-
-  // Get base URL.
-  GetBaseURL(*getter_AddRefs(baseURL));
-
-  // Get href= attribute (relative URL).
-  nsGenericHTMLLeafElement::GetAttr(kNameSpaceID_None, nsHTMLAtoms::src,
-                                    relURLSpec);
-  relURLSpec.Trim(" \t\n\r");
-
-  if (baseURL && !relURLSpec.IsEmpty()) {
-    // Get absolute URL.
-    rv = NS_MakeAbsoluteURI(aSrc, relURLSpec, baseURL);
-  }
-  else {
-    // Absolute URL is same as relative URL.
-    aSrc = relURLSpec;
-  }
-
-  return rv;
-}
-
-
-NS_IMETHODIMP
-nsHTMLImageElement::OnStartDecode(imgIRequest *request, nsISupports *cx)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLImageElement::OnStartContainer(imgIRequest *request, nsISupports *cx,
-                                     imgIContainer *image)
-{
-  nsCOMPtr<nsIPresContext> pc(do_QueryInterface(cx));
-
-  NS_ASSERTION(pc, "not a prescontext!");
-
-  float t2p;
-  pc->GetTwipsToPixels(&t2p);
-
-  nsSize size;
-  image->GetWidth(&size.width);
-  image->GetHeight(&size.height);
-
-  nsAutoString tmpStr;
-  tmpStr.AppendInt(size.width);
-  SetAttr(kNameSpaceID_None, nsHTMLAtoms::width, tmpStr, PR_FALSE);
-
-  tmpStr.Truncate();
-  tmpStr.AppendInt(size.height);
-  SetAttr(kNameSpaceID_None, nsHTMLAtoms::height, tmpStr, PR_FALSE);
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLImageElement::OnStartFrame(imgIRequest *request, nsISupports *cx,
-                                 gfxIImageFrame *frame)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLImageElement::OnDataAvailable(imgIRequest *request, nsISupports *cx,
-                                    gfxIImageFrame *frame, const nsRect * rect)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLImageElement::OnStopFrame(imgIRequest *request, nsISupports *cx,
-                                gfxIImageFrame *frame)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLImageElement::OnStopContainer(imgIRequest *request, nsISupports *cx,
-                                    imgIContainer *image)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLImageElement::OnStopDecode(imgIRequest *request, nsISupports *cx,
-                                 nsresult status, const PRUnichar *statusArg)
-{
-  nsCOMPtr<nsIPresContext> pc(do_QueryInterface(cx));
-
-  NS_ASSERTION(pc, "not a prescontext!");
-
-  // We set mRequest = nsnull to indicate that we're complete.
-  mRequest = nsnull;
-
-  // Fire the onload event.
-  nsEventStatus estatus = nsEventStatus_eIgnore;
-  nsEvent event;
-  event.eventStructType = NS_EVENT;
-
-  if (NS_SUCCEEDED(status))
-    event.message = NS_IMAGE_LOAD;
-  else
-    event.message = NS_IMAGE_ERROR;
-
-  HandleDOMEvent(pc, &event, nsnull, NS_EVENT_FLAG_INIT, &estatus);
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsHTMLImageElement::FrameChanged(imgIContainer *container, nsISupports *cx, gfxIImageFrame *newframe, nsRect * dirtyRect)
-{
-  return NS_OK;
-}
-
-nsresult
-nsHTMLImageElement::SetSrcInner(nsIURI* aBaseURL,
-                                const nsAString& aSrc)
-{
-  nsresult result = SetAttr(kNameSpaceID_None, nsHTMLAtoms::src, aSrc,
-                            PR_TRUE);
-  NS_ENSURE_SUCCESS(result, result);
-
-  if (!mDocument) {
-    nsCOMPtr<nsIDocument> doc;
-    mNodeInfo->GetDocument(*getter_AddRefs(doc));
-
-    nsCOMPtr<nsIPresShell> shell;
-
-    doc->GetShellAt(0, getter_AddRefs(shell));
-    if (shell) {
-      nsCOMPtr<nsIPresContext> context;
-
-      result = shell->GetPresContext(getter_AddRefs(context));
-      if (context) {
-        nsAutoString url;
-        if (aBaseURL) {
-          result = NS_MakeAbsoluteURI(url, aSrc, aBaseURL);
-          if (NS_FAILED(result)) {
-            url.Assign(aSrc);
-          }
-        }
-        else {
-          url.Assign(aSrc);
-        }
-
-        nsCOMPtr<nsIURI> uri;
-        result = NS_NewURI(getter_AddRefs(uri), aSrc, nsnull, aBaseURL);
-        if (NS_FAILED(result)) return result;
-
-        nsCOMPtr<nsIDocument> document;
-        result = shell->GetDocument(getter_AddRefs(document));
-        if (NS_FAILED(result)) return result;
-
-        nsCOMPtr<nsIScriptGlobalObject> globalObject;
-        result = document->GetScriptGlobalObject(getter_AddRefs(globalObject));
-        if (NS_FAILED(result)) return result;
-
-        nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(globalObject));
-
-        PRBool shouldLoad = PR_TRUE;
-        result =
-          NS_CheckContentLoadPolicy(nsIContentPolicy::IMAGE,
-                                    uri,
-                                    NS_STATIC_CAST(nsIDOMHTMLImageElement *,
-                                                   this),
-                                    domWin, &shouldLoad);
-        if (NS_SUCCEEDED(result) && !shouldLoad) {
-          return NS_OK;
-        }
-
-        // If we have a loader we're in the middle of loading a image,
-        // we'll cancel that load and start a new one.
-
-        // if (mRequest) {
-        //   mRequest->Cancel() ?? cancel the load?
-        // }
-
-        nsCOMPtr<imgILoader> il(do_GetService("@mozilla.org/image/loader;1"));
-        if (!il) {
-          return NS_ERROR_FAILURE;
-        }
-
-        nsCOMPtr<nsIDocument> doc;
-        nsCOMPtr<nsILoadGroup> loadGroup;
-        nsCOMPtr<nsIURI> documentURI;
-        shell->GetDocument(getter_AddRefs(doc));
-        if (doc) {
-          doc->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
-
-          // Get the documment URI for the referrer.
-          doc->GetDocumentURL(getter_AddRefs(documentURI));
-        }
-
-        // XXX: initialDocumentURI is NULL!
-        il->LoadImage(uri, nsnull, documentURI, loadGroup, this, context, nsIRequest::LOAD_NORMAL,
-                      nsnull, nsnull, getter_AddRefs(mRequest));
-      }
-    }
-  } else {
-    nsIImageFrame* imageFrame;
-    GetImageFrame(&imageFrame);
-    if (!imageFrame) {
-      // If we don't have an image frame, reconstruct the frame
-      // so that the new image can load.
-      nsCOMPtr<nsIPresShell> shell;
-      mDocument->GetShellAt(0, getter_AddRefs(shell));
-      if (shell) {
-        shell->RecreateFramesFor(this);
-      }
-    }
-  }
-
-  return result;
-}
-
 NS_IMETHODIMP
 nsHTMLImageElement::SetSrc(const nsAString& aSrc)
 {
@@ -853,55 +672,24 @@ nsHTMLImageElement::SetSrc(const nsAString& aSrc)
    * prevent setting image.src by exiting early
    */
 
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID));
-  if (prefs) {
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
+  if (prefBranch) {
     PRBool disableImageSrcSet = PR_FALSE;
-    prefs->GetBoolPref("dom.disable_image_src_set", &disableImageSrcSet);
+    prefBranch->GetBoolPref("dom.disable_image_src_set", &disableImageSrcSet);
 
     if (disableImageSrcSet && !nsContentUtils::IsCallerChrome()) {
       return NS_OK;
     }
   }
 
-  nsCOMPtr<nsIURI> baseURL;
-  nsresult rv = NS_OK;
+  // Call ImageURIChanged first so that our image load will kick off
+  // before the SetAttr triggers a reflow
+  ImageURIChanged(aSrc);
 
-  nsCOMPtr<nsIDOMDocument> domDoc;
-  nsContentUtils::GetDocumentFromCaller(getter_AddRefs(domDoc));
-
-  nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
-  if (doc) {
-    // XXX GetBaseURL should do the GetDocumentURL for us
-    rv = doc->GetBaseURL(*getter_AddRefs(baseURL));
-    if (!baseURL) {
-      rv = doc->GetDocumentURL(getter_AddRefs(baseURL));
-    }
-  }
-
-  if (!baseURL) {
-    mNodeInfo->GetDocument(*getter_AddRefs(doc));
-    if (doc) {
-      rv = doc->GetBaseURL(*getter_AddRefs(baseURL));
-    }
-  }
-
-  if (NS_SUCCEEDED(rv)) {
-    rv = SetSrcInner(baseURL, aSrc);
-  }
-
-  return rv;
+  return nsGenericHTMLLeafElement::SetAttr(kNameSpaceID_None,
+                                           nsHTMLAtoms::src, aSrc, PR_TRUE);
 }
 
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLImageElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
-
 NS_IMETHODIMP
 nsHTMLImageElement::GetNaturalHeight(PRInt32* aNaturalHeight)
 {
@@ -909,21 +697,17 @@ nsHTMLImageElement::GetNaturalHeight(PRInt32* aNaturalHeight)
 
   *aNaturalHeight = 0;
 
-  nsIImageFrame* imageFrame;
-  GetImageFrame(&imageFrame);
-
-  if (!imageFrame)
-    return NS_OK; // don't throw JS exceptions in this case
-
-  PRUint32 width, height;
-
-  nsresult rv = imageFrame->GetNaturalImageSize(&width, &height);
-
-  if (NS_FAILED(rv))
+  if (!mCurrentRequest) {
     return NS_OK;
+  }
+  
+  nsCOMPtr<imgIContainer> image;
+  mCurrentRequest->GetImage(getter_AddRefs(image));
+  if (!image) {
+    return NS_OK;
+  }
 
-  *aNaturalHeight = (PRInt32)height;
-
+  image->GetHeight(aNaturalHeight);
   return NS_OK;
 }
 
@@ -934,21 +718,17 @@ nsHTMLImageElement::GetNaturalWidth(PRInt32* aNaturalWidth)
 
   *aNaturalWidth = 0;
 
-  nsIImageFrame* imageFrame;
-  GetImageFrame(&imageFrame);
-
-  if (!imageFrame)
-    return NS_OK; // don't throw JS exceptions in this case
-
-  PRUint32 width, height;
-
-  nsresult rv = imageFrame->GetNaturalImageSize(&width, &height);
-
-  if (NS_FAILED(rv))
+  if (!mCurrentRequest) {
     return NS_OK;
+  }
+  
+  nsCOMPtr<imgIContainer> image;
+  mCurrentRequest->GetImage(getter_AddRefs(image));
+  if (!image) {
+    return NS_OK;
+  }
 
-  *aNaturalWidth = (PRInt32)width;
-
+  image->GetWidth(aNaturalWidth);
   return NS_OK;
 }
 
