@@ -106,6 +106,7 @@
 
 // input type=image
 #include "nsImageLoadingContent.h"
+#include "nsIDOMWindowInternal.h"
 
 // XXX align=left, hspace, vspace, border? other nav4 attrs
 
@@ -675,7 +676,7 @@ nsHTMLInputElement::GetValue(nsAString& aValue)
       if (!GET_BOOLBIT(mBitField, BF_VALUE_CHANGED) || !mValue) {
         GetDefaultValue(aValue);
       } else {
-        aValue = NS_ConvertUTF8toUCS2(mValue);
+        CopyUTF8toUTF16(mValue, aValue);
       }
     }
 
@@ -1054,6 +1055,8 @@ nsHTMLInputElement::SetFocus(nsIPresContext* aPresContext)
   PRBool isActive = PR_FALSE;
   focusController->GetActive(&isActive);
   if (!isActive) {
+    nsCOMPtr<nsIDOMWindowInternal> domWin(do_QueryInterface(globalObj));
+    focusController->SetFocusedWindow(domWin);
     focusController->SetFocusedElement(this);
     return NS_OK;
   }
@@ -1098,7 +1101,7 @@ nsHTMLInputElement::RemoveFocus(nsIPresContext* aPresContext)
   if (esm) {
     nsCOMPtr<nsIDocument> doc;
 
-    GetDocument(*getter_AddRefs(doc));
+    GetDocument(getter_AddRefs(doc));
 
     if (!doc)
       return NS_ERROR_NULL_POINTER;
@@ -1145,6 +1148,8 @@ nsHTMLInputElement::Select()
     PRBool isActive = PR_FALSE;
     focusController->GetActive(&isActive);
     if (!isActive) {
+      nsCOMPtr<nsIDOMWindowInternal> domWin(do_QueryInterface(globalObj));
+      focusController->SetFocusedWindow(domWin);
       focusController->SetFocusedElement(this);
       SelectAll(presContext);
       return NS_OK;
@@ -1231,7 +1236,7 @@ nsHTMLInputElement::Click()
       mType == NS_FORM_INPUT_SUBMIT) {
 
     nsCOMPtr<nsIDocument> doc; // Strong
-    rv = GetDocument(*getter_AddRefs(doc));
+    rv = GetDocument(getter_AddRefs(doc));
 
     if (doc) {
       PRInt32 numShells = doc->GetNumberOfShells();
@@ -1295,9 +1300,7 @@ nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
       nsIFrame* formFrame = nsnull;
       CallQueryInterface(formControlFrame, &formFrame);
       if (formFrame) {
-        const nsStyleUserInterface* uiStyle;
-        formFrame->GetStyleData(eStyleStruct_UserInterface,
-                                (const nsStyleStruct *&)uiStyle);
+        const nsStyleUserInterface* uiStyle = formFrame->GetStyleUserInterface();
 
         if (uiStyle->mUserInput == NS_STYLE_USER_INPUT_NONE ||
             uiStyle->mUserInput == NS_STYLE_USER_INPUT_DISABLED) {
@@ -1798,7 +1801,7 @@ nsHTMLInputElement::SetDocument(nsIDocument* aDocument, PRBool aDeep,
 
 // nsIHTMLContent
 
-static nsHTMLValue::EnumTable kInputTypeTable[] = {
+static const nsHTMLValue::EnumTable kInputTypeTable[] = {
   { "browse", NS_FORM_BROWSE }, // XXX not valid html, but it is convenient
   { "button", NS_FORM_INPUT_BUTTON },
   { "checkbox", NS_FORM_INPUT_CHECKBOX },
@@ -1819,7 +1822,7 @@ nsHTMLInputElement::StringToAttribute(nsIAtom* aAttribute,
                                       nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::type) {
-    nsHTMLValue::EnumTable *table = kInputTypeTable;
+    const nsHTMLValue::EnumTable *table = kInputTypeTable;
     nsAutoString valueStr(aValue);
     while (nsnull != table->tag) { 
       if (valueStr.EqualsIgnoreCase(table->tag)) {
