@@ -51,6 +51,7 @@
 #include "nsHTMLParts.h"
 #include "nsRuleNode.h"
 #include "nsStyleContext.h"
+#include "nsIDocument.h"
 
 /* for collections */
 #include "nsIDOMElement.h"
@@ -465,7 +466,7 @@ nsHTMLTableElement::GetTHead(nsIDOMHTMLTableSectionElement** aValue)
       nsCOMPtr<nsIAtom> tag;
       nsCOMPtr<nsIContent> content = do_QueryInterface(section);
 
-      content->GetTag(*getter_AddRefs(tag));
+      content->GetTag(getter_AddRefs(tag));
       if (tag.get() == nsHTMLAtoms::thead) {
         *aValue = section;
         NS_ADDREF(*aValue);
@@ -517,7 +518,7 @@ nsHTMLTableElement::GetTFoot(nsIDOMHTMLTableSectionElement** aValue)
       nsCOMPtr<nsIAtom> tag;
       nsCOMPtr<nsIContent> content = do_QueryInterface(section);
 
-      content->GetTag(*getter_AddRefs(tag));
+      content->GetTag(getter_AddRefs(tag));
 
       if (tag.get() == nsHTMLAtoms::tfoot) {
         *aValue = section;
@@ -599,7 +600,7 @@ nsHTMLTableElement::CreateTHead(nsIDOMHTMLElement** aValue)
     nsCOMPtr<nsIHTMLContent> newHead;
     nsCOMPtr<nsINodeInfo> nodeInfo;
 
-    mNodeInfo->NameChanged(nsHTMLAtoms::thead, *getter_AddRefs(nodeInfo));
+    mNodeInfo->NameChanged(nsHTMLAtoms::thead, getter_AddRefs(nodeInfo));
 
     rv = NS_NewHTMLTableSectionElement(getter_AddRefs(newHead),nodeInfo);
 
@@ -656,7 +657,7 @@ nsHTMLTableElement::CreateTFoot(nsIDOMHTMLElement** aValue)
     nsCOMPtr<nsIHTMLContent> newFoot;
 
     nsCOMPtr<nsINodeInfo> nodeInfo;
-    mNodeInfo->NameChanged(nsHTMLAtoms::tfoot, *getter_AddRefs(nodeInfo));
+    mNodeInfo->NameChanged(nsHTMLAtoms::tfoot, getter_AddRefs(nodeInfo));
 
     rv = NS_NewHTMLTableSectionElement(getter_AddRefs(newFoot),nodeInfo);
 
@@ -703,7 +704,7 @@ nsHTMLTableElement::CreateCaption(nsIDOMHTMLElement** aValue)
     nsCOMPtr<nsIHTMLContent> newCaption;
 
     nsCOMPtr<nsINodeInfo> nodeInfo;
-    mNodeInfo->NameChanged(nsHTMLAtoms::caption, *getter_AddRefs(nodeInfo));
+    mNodeInfo->NameChanged(nsHTMLAtoms::caption, getter_AddRefs(nodeInfo));
 
     rv = NS_NewHTMLTableCaptionElement(getter_AddRefs(newCaption),nodeInfo);
 
@@ -781,7 +782,7 @@ nsHTMLTableElement::InsertRow(PRInt32 aIndex, nsIDOMHTMLElement** aValue)
     nsCOMPtr<nsIHTMLContent> newRow;
 
     nsCOMPtr<nsINodeInfo> nodeInfo;
-    mNodeInfo->NameChanged(nsHTMLAtoms::tr, *getter_AddRefs(nodeInfo));
+    mNodeInfo->NameChanged(nsHTMLAtoms::tr, getter_AddRefs(nodeInfo));
 
     rv = NS_NewHTMLTableRowElement(getter_AddRefs(newRow), nodeInfo);
 
@@ -848,7 +849,7 @@ nsHTMLTableElement::InsertRow(PRInt32 aIndex, nsIDOMHTMLElement** aValue)
       nsCOMPtr<nsIHTMLContent> newRowGroup;
 
       nsCOMPtr<nsINodeInfo> nodeInfo;
-      mNodeInfo->NameChanged(nsHTMLAtoms::tbody, *getter_AddRefs(nodeInfo));
+      mNodeInfo->NameChanged(nsHTMLAtoms::tbody, getter_AddRefs(nodeInfo));
 
       rv = NS_NewHTMLTableSectionElement(getter_AddRefs(newRowGroup),
                                          nodeInfo);
@@ -864,7 +865,7 @@ nsHTMLTableElement::InsertRow(PRInt32 aIndex, nsIDOMHTMLElement** aValue)
       nsCOMPtr<nsIHTMLContent> newRow;
 
       nsCOMPtr<nsINodeInfo> nodeInfo;
-      mNodeInfo->NameChanged(nsHTMLAtoms::tr, *getter_AddRefs(nodeInfo));
+      mNodeInfo->NameChanged(nsHTMLAtoms::tr, getter_AddRefs(nodeInfo));
 
       rv = NS_NewHTMLTableRowElement(getter_AddRefs(newRow), nodeInfo);
       nsCOMPtr<nsIContent> rowGroupContent(do_QueryInterface(rowGroup));
@@ -933,7 +934,7 @@ nsHTMLTableElement::DeleteRow(PRInt32 aValue)
   return parent->RemoveChild(row, getter_AddRefs(deleted_row));
 }
 
-static nsHTMLValue::EnumTable kFrameTable[] = {
+static const nsHTMLValue::EnumTable kFrameTable[] = {
   { "void",   NS_STYLE_TABLE_FRAME_NONE },
   { "above",  NS_STYLE_TABLE_FRAME_ABOVE },
   { "below",  NS_STYLE_TABLE_FRAME_BELOW },
@@ -946,7 +947,7 @@ static nsHTMLValue::EnumTable kFrameTable[] = {
   { 0 }
 };
 
-static nsHTMLValue::EnumTable kRulesTable[] = {
+static const nsHTMLValue::EnumTable kRulesTable[] = {
   { "none",   NS_STYLE_TABLE_RULES_NONE },
   { "groups", NS_STYLE_TABLE_RULES_GROUPS },
   { "rows",   NS_STYLE_TABLE_RULES_ROWS },
@@ -955,7 +956,7 @@ static nsHTMLValue::EnumTable kRulesTable[] = {
   { 0 }
 };
 
-static nsHTMLValue::EnumTable kLayoutTable[] = {
+static const nsHTMLValue::EnumTable kLayoutTable[] = {
   { "auto",   NS_STYLE_TABLE_LAYOUT_AUTO },
   { "fixed",  NS_STYLE_TABLE_LAYOUT_FIXED },
   { 0 }
@@ -1029,7 +1030,11 @@ nsHTMLTableElement::StringToAttribute(nsIAtom* aAttribute,
   }
   else if (aAttribute == nsHTMLAtoms::bgcolor ||
            aAttribute == nsHTMLAtoms::bordercolor) {
-    if (aResult.ParseColor(aValue, mDocument)) {
+    nsCOMPtr<nsIDocument> doc(mDocument);
+    if (!doc) {
+      mNodeInfo->GetDocument(getter_AddRefs(doc));
+    }
+    if (aResult.ParseColor(aValue, doc)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -1100,14 +1105,14 @@ MapTableFrameInto(const nsIHTMLMappedAttributes* aAttributes,
     return;
 
   // set up defaults
-  if (aData->mMarginData->mBorderStyle->mLeft.GetUnit() == eCSSUnit_Null)
-    aData->mMarginData->mBorderStyle->mLeft.SetIntValue(aBorderStyle, eCSSUnit_Enumerated);
-  if (aData->mMarginData->mBorderStyle->mRight.GetUnit() == eCSSUnit_Null)
-    aData->mMarginData->mBorderStyle->mRight.SetIntValue(aBorderStyle, eCSSUnit_Enumerated);
-  if (aData->mMarginData->mBorderStyle->mTop.GetUnit() == eCSSUnit_Null)
-    aData->mMarginData->mBorderStyle->mTop.SetIntValue(aBorderStyle, eCSSUnit_Enumerated);
-  if (aData->mMarginData->mBorderStyle->mBottom.GetUnit() == eCSSUnit_Null)
-    aData->mMarginData->mBorderStyle->mBottom.SetIntValue(aBorderStyle, eCSSUnit_Enumerated);
+  if (aData->mMarginData->mBorderStyle.mLeft.GetUnit() == eCSSUnit_Null)
+    aData->mMarginData->mBorderStyle.mLeft.SetIntValue(aBorderStyle, eCSSUnit_Enumerated);
+  if (aData->mMarginData->mBorderStyle.mRight.GetUnit() == eCSSUnit_Null)
+    aData->mMarginData->mBorderStyle.mRight.SetIntValue(aBorderStyle, eCSSUnit_Enumerated);
+  if (aData->mMarginData->mBorderStyle.mTop.GetUnit() == eCSSUnit_Null)
+    aData->mMarginData->mBorderStyle.mTop.SetIntValue(aBorderStyle, eCSSUnit_Enumerated);
+  if (aData->mMarginData->mBorderStyle.mBottom.GetUnit() == eCSSUnit_Null)
+    aData->mMarginData->mBorderStyle.mBottom.SetIntValue(aBorderStyle, eCSSUnit_Enumerated);
 
   nsHTMLValue frameValue;
   // 0 out the sides that we want to hide based on the frame attribute
@@ -1118,38 +1123,38 @@ MapTableFrameInto(const nsIHTMLMappedAttributes* aAttributes,
     switch (frameValue.GetIntValue())
     {
     case NS_STYLE_TABLE_FRAME_NONE:
-      aData->mMarginData->mBorderStyle->mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
       break;
     case NS_STYLE_TABLE_FRAME_ABOVE:
-      aData->mMarginData->mBorderStyle->mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
       break;
     case NS_STYLE_TABLE_FRAME_BELOW: 
-      aData->mMarginData->mBorderStyle->mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
       break;
     case NS_STYLE_TABLE_FRAME_HSIDES:
-      aData->mMarginData->mBorderStyle->mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
       break;
     case NS_STYLE_TABLE_FRAME_LEFT:
-      aData->mMarginData->mBorderStyle->mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
       break;
     case NS_STYLE_TABLE_FRAME_RIGHT:
-      aData->mMarginData->mBorderStyle->mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
       break;
     case NS_STYLE_TABLE_FRAME_VSIDES:
-      aData->mMarginData->mBorderStyle->mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle->mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
       break;
     // BOX and BORDER are ignored, the caller has already set all the border sides
     // any illegal value is also ignored
@@ -1207,14 +1212,14 @@ MapTableBorderInto(const nsIHTMLMappedAttributes* aAttributes,
 
     if (aData->mMarginData) {
       // by default, set all border sides to the specified width
-      if (aData->mMarginData->mBorderWidth->mLeft.GetUnit() == eCSSUnit_Null)
-        aData->mMarginData->mBorderWidth->mLeft.SetFloatValue((float)borderThickness, eCSSUnit_Pixel);
-      if (aData->mMarginData->mBorderWidth->mRight.GetUnit() == eCSSUnit_Null)
-        aData->mMarginData->mBorderWidth->mRight.SetFloatValue((float)borderThickness, eCSSUnit_Pixel);
-      if (aData->mMarginData->mBorderWidth->mTop.GetUnit() == eCSSUnit_Null)
-        aData->mMarginData->mBorderWidth->mTop .SetFloatValue((float)borderThickness, eCSSUnit_Pixel);
-      if (aData->mMarginData->mBorderWidth->mBottom.GetUnit() == eCSSUnit_Null)
-        aData->mMarginData->mBorderWidth->mBottom.SetFloatValue((float)borderThickness, eCSSUnit_Pixel);
+      if (aData->mMarginData->mBorderWidth.mLeft.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderWidth.mLeft.SetFloatValue((float)borderThickness, eCSSUnit_Pixel);
+      if (aData->mMarginData->mBorderWidth.mRight.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderWidth.mRight.SetFloatValue((float)borderThickness, eCSSUnit_Pixel);
+      if (aData->mMarginData->mBorderWidth.mTop.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderWidth.mTop .SetFloatValue((float)borderThickness, eCSSUnit_Pixel);
+      if (aData->mMarginData->mBorderWidth.mBottom.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderWidth.mBottom.SetFloatValue((float)borderThickness, eCSSUnit_Pixel);
 
       // now account for the frame attribute
       MapTableFrameInto(aAttributes, aData, aBorderStyle);
@@ -1236,9 +1241,8 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
   aData->mPresContext->GetCompatibilityMode(&mode);
 
   if (aData->mSID == eStyleStruct_TableBorder && aData->mTableData) {
-    const nsStyleDisplay* readDisplay = (nsStyleDisplay*)
-                  aData->mStyleContext->GetStyleData(eStyleStruct_Display);
-    if (readDisplay && readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL) {
+    const nsStyleDisplay* readDisplay = aData->mStyleContext->GetStyleDisplay();
+    if (readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL) {
       // cellspacing 
       nsHTMLValue value;
       aAttributes->GetAttribute(nsHTMLAtoms::cellspacing, value);
@@ -1259,9 +1263,8 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
   }
 
   if (aData->mSID == eStyleStruct_Table && aData->mTableData) {
-    const nsStyleDisplay* readDisplay = (nsStyleDisplay*)
-                  aData->mStyleContext->GetStyleData(eStyleStruct_Display);
-    if (readDisplay && readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL) {
+    const nsStyleDisplay* readDisplay = aData->mStyleContext->GetStyleDisplay();
+    if (readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL) {
       MapTableBorderInto(aAttributes, aData, 0);
 
       nsHTMLValue value;
@@ -1288,10 +1291,9 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
     }
   }
   else if (aData->mSID == eStyleStruct_Margin && aData->mMarginData) {
-    const nsStyleDisplay* readDisplay = (nsStyleDisplay*)
-                  aData->mStyleContext->GetStyleData(eStyleStruct_Display);
+    const nsStyleDisplay* readDisplay = aData->mStyleContext->GetStyleDisplay();
   
-    if (readDisplay && readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL) {
+    if (readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL) {
       // align; Check for enumerated type (it may be another type if
       // illegal)
       nsHTMLValue value;
@@ -1300,11 +1302,11 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
       if (value.GetUnit() == eHTMLUnit_Enumerated) {
         if ((NS_STYLE_TEXT_ALIGN_CENTER == value.GetIntValue()) ||
             (NS_STYLE_TEXT_ALIGN_MOZ_CENTER == value.GetIntValue())) {
-          nsCSSRect* margin = aData->mMarginData->mMargin;
-          if (margin->mLeft.GetUnit() == eCSSUnit_Null)
-            margin->mLeft.SetAutoValue();
-          if (margin->mRight.GetUnit() == eCSSUnit_Null)
-            margin->mRight.SetAutoValue();
+          nsCSSRect& margin = aData->mMarginData->mMargin;
+          if (margin.mLeft.GetUnit() == eCSSUnit_Null)
+            margin.mLeft.SetAutoValue();
+          if (margin.mRight.GetUnit() == eCSSUnit_Null)
+            margin.mRight.SetAutoValue();
         }
       }
 
@@ -1315,31 +1317,29 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
         aAttributes->GetAttribute(nsHTMLAtoms::hspace, value);
 
         if (value.GetUnit() == eHTMLUnit_Pixel) {
-          nsCSSRect* margin = aData->mMarginData->mMargin;
-          if (margin->mLeft.GetUnit() == eCSSUnit_Null)
-            margin->mLeft.SetFloatValue((float)value.GetPixelValue(), eCSSUnit_Pixel); 
-          if (margin->mRight.GetUnit() == eCSSUnit_Null)
-            margin->mRight.SetFloatValue((float)value.GetPixelValue(), eCSSUnit_Pixel);
+          nsCSSRect& margin = aData->mMarginData->mMargin;
+          if (margin.mLeft.GetUnit() == eCSSUnit_Null)
+            margin.mLeft.SetFloatValue((float)value.GetPixelValue(), eCSSUnit_Pixel); 
+          if (margin.mRight.GetUnit() == eCSSUnit_Null)
+            margin.mRight.SetFloatValue((float)value.GetPixelValue(), eCSSUnit_Pixel);
         }
 
         aAttributes->GetAttribute(nsHTMLAtoms::vspace, value);
 
         if (value.GetUnit() == eHTMLUnit_Pixel) {
-          nsCSSRect* margin = aData->mMarginData->mMargin;
-          if (margin->mTop.GetUnit() == eCSSUnit_Null)
-            margin->mTop.SetFloatValue((float)value.GetPixelValue(), eCSSUnit_Pixel); 
-          if (margin->mBottom.GetUnit() == eCSSUnit_Null)
-            margin->mBottom.SetFloatValue((float)value.GetPixelValue(), eCSSUnit_Pixel); 
+          nsCSSRect& margin = aData->mMarginData->mMargin;
+          if (margin.mTop.GetUnit() == eCSSUnit_Null)
+            margin.mTop.SetFloatValue((float)value.GetPixelValue(), eCSSUnit_Pixel); 
+          if (margin.mBottom.GetUnit() == eCSSUnit_Null)
+            margin.mBottom.SetFloatValue((float)value.GetPixelValue(), eCSSUnit_Pixel); 
         }
       }
     }
   }
   else if (aData->mSID == eStyleStruct_Padding && aData->mMarginData) {
-    const nsStyleDisplay* readDisplay = (nsStyleDisplay*)
-                  aData->mStyleContext->GetStyleData(eStyleStruct_Display);
+    const nsStyleDisplay* readDisplay = aData->mStyleContext->GetStyleDisplay();
   
-    if (readDisplay &&
-        (readDisplay->mDisplay == NS_STYLE_DISPLAY_TABLE_CELL)) {
+    if (readDisplay->mDisplay == NS_STYLE_DISPLAY_TABLE_CELL) {
       nsHTMLValue value;
       aAttributes->GetAttribute(nsHTMLAtoms::cellpadding, value);
       if (value.GetUnit() == eHTMLUnit_Pixel || value.GetUnit() == eHTMLUnit_Percent) {
@@ -1359,23 +1359,21 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
           //  padVal.SetPercentValue(pctVal);
           //}
         }
-        if (aData->mMarginData->mPadding->mLeft.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mPadding->mLeft = padVal;
-        if (aData->mMarginData->mPadding->mRight.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mPadding->mRight = padVal;
-        if (aData->mMarginData->mPadding->mTop.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mPadding->mTop = padVal;
-        if (aData->mMarginData->mPadding->mBottom.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mPadding->mBottom = padVal;
+        if (aData->mMarginData->mPadding.mLeft.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mPadding.mLeft = padVal;
+        if (aData->mMarginData->mPadding.mRight.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mPadding.mRight = padVal;
+        if (aData->mMarginData->mPadding.mTop.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mPadding.mTop = padVal;
+        if (aData->mMarginData->mPadding.mBottom.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mPadding.mBottom = padVal;
       }
     }
   }
   else if (aData->mPositionData) {
-    const nsStyleDisplay* readDisplay = (nsStyleDisplay*)
-                  aData->mStyleContext->GetStyleData(eStyleStruct_Display);
+    const nsStyleDisplay* readDisplay = aData->mStyleContext->GetStyleDisplay();
   
-    if (readDisplay &&
-        (readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL)) {
+    if (readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL) {
       nsHTMLValue value;
       // width: value
       if (aData->mPositionData->mWidth.GetUnit() == eCSSUnit_Null) {
@@ -1397,21 +1395,16 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
     }
   }
   else if (aData->mSID == eStyleStruct_Visibility) {
-    const nsStyleDisplay* readDisplay = (nsStyleDisplay*)
-                  aData->mStyleContext->GetStyleData(eStyleStruct_Display);
+    const nsStyleDisplay* readDisplay = aData->mStyleContext->GetStyleDisplay();
   
-    if (readDisplay &&
-        (readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL))
+    if (readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL)
       nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aData);
   }
   else if (aData->mSID == eStyleStruct_Border && aData->mMarginData) {
     if (!aData->mStyleContext) return;
-    const nsStyleTableBorder* tableStyle =
-      (const nsStyleTableBorder*)aData->mStyleContext->GetStyleData(eStyleStruct_TableBorder);
-    const nsStyleDisplay* readDisplay = 
-      (nsStyleDisplay*) aData->mStyleContext->GetStyleData(eStyleStruct_Display);  
-    if (readDisplay &&
-        (readDisplay->mDisplay == NS_STYLE_DISPLAY_TABLE_CELL)) {
+    const nsStyleTableBorder* tableStyle = aData->mStyleContext->GetStyleTableBorder();
+    const nsStyleDisplay* readDisplay = aData->mStyleContext->GetStyleDisplay();
+    if (readDisplay->mDisplay == NS_STYLE_DISPLAY_TABLE_CELL) {
       if (NS_STYLE_BORDER_SEPARATE == tableStyle->mBorderCollapse) {
         // Set the cell's border from the table in the separate border model. If there is a border
         // on the table, then the mapping to rules=all will take care of borders in the collapsing model.
@@ -1420,28 +1413,28 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
         if (((value.GetUnit() == eHTMLUnit_Pixel) &&
              (value.GetPixelValue() > 0)) ||
             (value.GetUnit() == eHTMLUnit_Empty)) {
-          if (aData->mMarginData->mBorderWidth->mLeft.GetUnit() == eCSSUnit_Null)
-            aData->mMarginData->mBorderWidth->mLeft.SetFloatValue(1.0f, eCSSUnit_Pixel);
-          if (aData->mMarginData->mBorderWidth->mRight.GetUnit() == eCSSUnit_Null)
-            aData->mMarginData->mBorderWidth->mRight.SetFloatValue(1.0f, eCSSUnit_Pixel);
-          if (aData->mMarginData->mBorderWidth->mTop.GetUnit() == eCSSUnit_Null)
-            aData->mMarginData->mBorderWidth->mTop.SetFloatValue(1.0f, eCSSUnit_Pixel);
-          if (aData->mMarginData->mBorderWidth->mBottom.GetUnit() == eCSSUnit_Null)
-            aData->mMarginData->mBorderWidth->mBottom.SetFloatValue(1.0f, eCSSUnit_Pixel);
+          if (aData->mMarginData->mBorderWidth.mLeft.GetUnit() == eCSSUnit_Null)
+            aData->mMarginData->mBorderWidth.mLeft.SetFloatValue(1.0f, eCSSUnit_Pixel);
+          if (aData->mMarginData->mBorderWidth.mRight.GetUnit() == eCSSUnit_Null)
+            aData->mMarginData->mBorderWidth.mRight.SetFloatValue(1.0f, eCSSUnit_Pixel);
+          if (aData->mMarginData->mBorderWidth.mTop.GetUnit() == eCSSUnit_Null)
+            aData->mMarginData->mBorderWidth.mTop.SetFloatValue(1.0f, eCSSUnit_Pixel);
+          if (aData->mMarginData->mBorderWidth.mBottom.GetUnit() == eCSSUnit_Null)
+            aData->mMarginData->mBorderWidth.mBottom.SetFloatValue(1.0f, eCSSUnit_Pixel);
 
           PRUint8 borderStyle = (eCompatibility_NavQuirks == mode) 
                                 ? NS_STYLE_BORDER_STYLE_BG_INSET : NS_STYLE_BORDER_STYLE_INSET;
           // BG_INSET results in a border color based on background colors
           // used for NavQuirks only...
 
-          if (aData->mMarginData->mBorderStyle->mLeft.GetUnit() == eCSSUnit_Null)
-            aData->mMarginData->mBorderStyle->mLeft.SetIntValue(borderStyle, eCSSUnit_Enumerated);
-          if (aData->mMarginData->mBorderStyle->mRight.GetUnit() == eCSSUnit_Null)
-            aData->mMarginData->mBorderStyle->mRight.SetIntValue(borderStyle, eCSSUnit_Enumerated);
-          if (aData->mMarginData->mBorderStyle->mTop.GetUnit() == eCSSUnit_Null)
-            aData->mMarginData->mBorderStyle->mTop.SetIntValue(borderStyle, eCSSUnit_Enumerated);
-          if (aData->mMarginData->mBorderStyle->mBottom.GetUnit() == eCSSUnit_Null)
-            aData->mMarginData->mBorderStyle->mBottom.SetIntValue(borderStyle, eCSSUnit_Enumerated);
+          if (aData->mMarginData->mBorderStyle.mLeft.GetUnit() == eCSSUnit_Null)
+            aData->mMarginData->mBorderStyle.mLeft.SetIntValue(borderStyle, eCSSUnit_Enumerated);
+          if (aData->mMarginData->mBorderStyle.mRight.GetUnit() == eCSSUnit_Null)
+            aData->mMarginData->mBorderStyle.mRight.SetIntValue(borderStyle, eCSSUnit_Enumerated);
+          if (aData->mMarginData->mBorderStyle.mTop.GetUnit() == eCSSUnit_Null)
+            aData->mMarginData->mBorderStyle.mTop.SetIntValue(borderStyle, eCSSUnit_Enumerated);
+          if (aData->mMarginData->mBorderStyle.mBottom.GetUnit() == eCSSUnit_Null)
+            aData->mMarginData->mBorderStyle.mBottom.SetIntValue(borderStyle, eCSSUnit_Enumerated);
         }
       }
     }
@@ -1462,28 +1455,28 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
       if ((eHTMLUnit_Color == value.GetUnit()) ||
           (eHTMLUnit_ColorName == value.GetUnit())) {
         nscolor color = value.GetColorValue();
-        if (aData->mMarginData->mBorderColor->mLeft.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mBorderColor->mLeft.SetColorValue(color);
-        if (aData->mMarginData->mBorderColor->mRight.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mBorderColor->mRight.SetColorValue(color);
-        if (aData->mMarginData->mBorderColor->mTop.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mBorderColor->mTop.SetColorValue(color);
-        if (aData->mMarginData->mBorderColor->mBottom.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mBorderColor->mBottom.SetColorValue(color);
+        if (aData->mMarginData->mBorderColor.mLeft.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mBorderColor.mLeft.SetColorValue(color);
+        if (aData->mMarginData->mBorderColor.mRight.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mBorderColor.mRight.SetColorValue(color);
+        if (aData->mMarginData->mBorderColor.mTop.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mBorderColor.mTop.SetColorValue(color);
+        if (aData->mMarginData->mBorderColor.mBottom.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mBorderColor.mBottom.SetColorValue(color);
 
         borderStyle = NS_STYLE_BORDER_STYLE_OUTSET; // use css outset
       }
       else if (NS_STYLE_BORDER_COLLAPSE == tableStyle->mBorderCollapse) {
         // make the color grey
         nscolor color = NS_RGB(80, 80, 80);
-        if (aData->mMarginData->mBorderColor->mLeft.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mBorderColor->mLeft.SetColorValue(color);
-        if (aData->mMarginData->mBorderColor->mRight.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mBorderColor->mRight.SetColorValue(color);
-        if (aData->mMarginData->mBorderColor->mTop.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mBorderColor->mTop.SetColorValue(color);
-        if (aData->mMarginData->mBorderColor->mBottom.GetUnit() == eCSSUnit_Null)
-          aData->mMarginData->mBorderColor->mBottom.SetColorValue(color);
+        if (aData->mMarginData->mBorderColor.mLeft.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mBorderColor.mLeft.SetColorValue(color);
+        if (aData->mMarginData->mBorderColor.mRight.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mBorderColor.mRight.SetColorValue(color);
+        if (aData->mMarginData->mBorderColor.mTop.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mBorderColor.mTop.SetColorValue(color);
+        if (aData->mMarginData->mBorderColor.mBottom.GetUnit() == eCSSUnit_Null)
+          aData->mMarginData->mBorderColor.mBottom.SetColorValue(color);
       }
 
       // border and frame
@@ -1492,11 +1485,9 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
   }
 
   if (aData->mSID == eStyleStruct_Background) {
-    const nsStyleDisplay* readDisplay = (nsStyleDisplay*)
-                  aData->mStyleContext->GetStyleData(eStyleStruct_Display);
+    const nsStyleDisplay* readDisplay = aData->mStyleContext->GetStyleDisplay();
   
-    if (readDisplay &&
-        (readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL))
+    if (readDisplay->mDisplay != NS_STYLE_DISPLAY_TABLE_CELL)
       nsGenericHTMLElement::MapBackgroundAttributesInto(aAttributes, aData);
   }
 }
