@@ -46,7 +46,6 @@
 #include "nsGenericHTMLElement.h"
 #include "GenericElementCollection.h"
 #include "nsHTMLAtoms.h"
-#include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsHTMLParts.h"
@@ -98,11 +97,11 @@ nsTableCellCollection::GetLength(PRUint32* aLength)
     nsCOMPtr<nsIContent> child;
     PRUint32 childIndex = 0;
 
-    mParent->ChildAt(childIndex, *getter_AddRefs(child));
+    mParent->ChildAt(childIndex, getter_AddRefs(child));
 
     while (child) {
       nsCOMPtr<nsIAtom> childTag;
-      child->GetTag(*getter_AddRefs(childTag));
+      child->GetTag(getter_AddRefs(childTag));
 
       if ((nsHTMLAtoms::td == childTag.get()) ||
           (nsHTMLAtoms::th == childTag.get())) {
@@ -110,7 +109,7 @@ nsTableCellCollection::GetLength(PRUint32* aLength)
       }
 
       childIndex++;
-      mParent->ChildAt(childIndex, *getter_AddRefs(child));
+      mParent->ChildAt(childIndex, getter_AddRefs(child));
     }
   }
 
@@ -129,18 +128,17 @@ nsTableCellCollection::Item(PRUint32     aIndex,
     nsCOMPtr<nsIContent> child;
     PRUint32 childIndex = 0;
 
-    mParent->ChildAt(childIndex, *getter_AddRefs(child));
+    mParent->ChildAt(childIndex, getter_AddRefs(child));
 
     while (child) {
       nsCOMPtr<nsIAtom> childTag;
 
-      child->GetTag(*getter_AddRefs(childTag));
+      child->GetTag(getter_AddRefs(childTag));
 
       if ((nsHTMLAtoms::td == childTag.get()) ||
           (nsHTMLAtoms::th == childTag.get())) {
         if (aIndex == theIndex) {
-          child->QueryInterface(NS_GET_IID(nsIDOMNode), (void**)aReturn);
-
+          CallQueryInterface(child, aReturn);
           NS_ASSERTION(aReturn, "content element must be an nsIDOMNode");
 
           break;
@@ -149,7 +147,7 @@ nsTableCellCollection::Item(PRUint32     aIndex,
       }
 
       childIndex++;
-      mParent->ChildAt(childIndex, *getter_AddRefs(child));
+      mParent->ChildAt(childIndex, getter_AddRefs(child));
     }
   }
 
@@ -188,10 +186,7 @@ public:
                                nsAString& aResult) const;
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                      PRInt32& aHint) const;
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
+                                      nsChangeHint& aHint) const;
 
 protected:
   nsresult GetSection(nsIDOMHTMLTableSectionElement** aSection);
@@ -202,29 +197,26 @@ protected:
 #ifdef XXX_debugging
 static
 void DebugList(nsIDOMHTMLTableElement* aTable) {
-  nsIHTMLContent* content = nsnull;
-  nsresult result = aTable->QueryInterface(NS_GET_IID(nsIHTMLContent), (void**)&content);
-  if (NS_SUCCEEDED(result) && (nsnull != content)) {
-    nsIDocument* doc = nsnull;
-    result = content->GetDocument(doc);
-    if (NS_SUCCEEDED(result) && (nsnull != doc)) {
-      nsIContent* root = nsnull;
-      doc->GetRootContent(&root);
+  nsCOMPtr<nsIHTMLContent> content = do_QueryInterface(aTable);
+  if (content) {
+    nsCOMPtr<nsIDocument> doc;
+    result = content->GetDocument(getter_AddRefs(doc));
+    if (doc) {
+      nsCOMPtr<nsIContent> root;
+      doc->GetRootContent(getter_AddRefs(root));
       if (root) {
         root->List();
       }
       nsCOMPtr<nsIPresShell> shell;
       doc->GetShellAt(0, getter_AddRefs(shell));
-      if (nsnull != shell) {
+      if (shell) {
         nsIFrame* rootFrame;
         shell->GetRootFrame(rootFrame);
-        if (nsnull != rootFrame) {
+        if (rootFrame) {
           rootFrame->List(stdout, 0);
         }
       }
-      NS_RELEASE(doc);
     }
-    NS_RELEASE(content);
   }
 }
 #endif 
@@ -314,45 +306,38 @@ nsHTMLTableRowElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
 nsresult
 nsHTMLTableRowElement::GetSection(nsIDOMHTMLTableSectionElement** aSection)
 {
+  NS_ENSURE_ARG_POINTER(aSection);
   *aSection = nsnull;
-  if (nsnull == aSection) {
-    return NS_ERROR_OUT_OF_MEMORY;
+
+  nsCOMPtr<nsIDOMNode> sectionNode;
+  nsresult rv = GetParentNode(getter_AddRefs(sectionNode));
+  if (NS_SUCCEEDED(rv) && sectionNode) {
+    rv = CallQueryInterface(sectionNode, aSection);
   }
-  nsIDOMNode *sectionNode = nsnull;
-  nsresult result = GetParentNode(&sectionNode);
-  if (NS_SUCCEEDED(result) && (nsnull != sectionNode)) {
-    result = sectionNode->QueryInterface(NS_GET_IID(nsIDOMHTMLTableSectionElement), (void**)aSection);
-    NS_RELEASE(sectionNode);
-  }
-  return result;
+
+  return rv;
 }
 
 // protected method
 nsresult
 nsHTMLTableRowElement::GetTable(nsIDOMHTMLTableElement** aTable)
 {
+  NS_ENSURE_ARG_POINTER(aTable);
   *aTable = nsnull;
 
-  if (!aTable) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
   nsCOMPtr<nsIDOMNode> sectionNode;
-
-  nsresult result = GetParentNode(getter_AddRefs(sectionNode));
-
-  if (NS_SUCCEEDED(result) && sectionNode) {
-    nsCOMPtr<nsIDOMNode> tableNode;
-
-    result = sectionNode->GetParentNode(getter_AddRefs(tableNode));
-
-    if (NS_SUCCEEDED(result) && tableNode) {
-      result = tableNode->QueryInterface(NS_GET_IID(nsIDOMHTMLTableElement),
-                                         (void**)aTable);
-    }
+  nsresult rv = GetParentNode(getter_AddRefs(sectionNode));
+  if (!sectionNode) {
+    return rv;
   }
 
-  return result;
+  nsCOMPtr<nsIDOMNode> tableNode;
+  rv = sectionNode->GetParentNode(getter_AddRefs(tableNode));
+  if (!tableNode) {
+    return rv;
+  }
+  
+  return CallQueryInterface(tableNode, aTable);
 }
 
 NS_IMETHODIMP
@@ -432,9 +417,7 @@ nsHTMLTableRowElement::GetCells(nsIDOMHTMLCollection** aValue)
     NS_ADDREF(mCells); // this table's reference, released in the destructor
   }
 
-  mCells->QueryInterface(NS_GET_IID(nsIDOMHTMLCollection), (void **)aValue);
-
-  return NS_OK;
+  return CallQueryInterface(mCells, aValue);
 }
 
 NS_IMETHODIMP
@@ -442,53 +425,48 @@ nsHTMLTableRowElement::InsertCell(PRInt32 aIndex, nsIDOMHTMLElement** aValue)
 {
   *aValue = nsnull;
 
-  if (aIndex < 0)
+  if (aIndex < -1) {
     return NS_ERROR_DOM_INDEX_SIZE_ERR;
+  }
   
   nsCOMPtr<nsIDOMHTMLCollection> cells;
-
   GetCells(getter_AddRefs(cells));
 
   PRUint32 cellCount;
   cells->GetLength(&cellCount);
 
-  if (aIndex > PRInt32(cellCount))
+  if (aIndex > PRInt32(cellCount)) {
     return NS_ERROR_DOM_INDEX_SIZE_ERR;
+  }
 
-  PRBool doInsert = (aIndex < PRInt32(cellCount));
+  PRBool doInsert = (aIndex < PRInt32(cellCount)) && (aIndex != -1);
 
   // create the cell
-  nsCOMPtr<nsIHTMLContent> cellContent;
-
   nsCOMPtr<nsINodeInfo> nodeInfo;
-  mNodeInfo->NameChanged(nsHTMLAtoms::td, *getter_AddRefs(nodeInfo));
+  mNodeInfo->NameChanged(nsHTMLAtoms::td, getter_AddRefs(nodeInfo));
 
+  nsCOMPtr<nsIHTMLContent> cellContent;
   nsresult rv = NS_NewHTMLTableCellElement(getter_AddRefs(cellContent),
                                            nodeInfo);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  if (NS_SUCCEEDED(rv) && cellContent) {
-    nsCOMPtr<nsIDOMNode> cellNode(do_QueryInterface(cellContent));
+  nsCOMPtr<nsIDOMNode> cellNode(do_QueryInterface(cellContent));
+  NS_ASSERTION(cellNode, "Should implement nsIDOMNode!");
 
-    if (NS_SUCCEEDED(rv) && cellNode) {
-      nsCOMPtr<nsIDOMNode> retChild;
+  nsCOMPtr<nsIDOMNode> retChild;
 
-      if (doInsert) {
-        PRInt32 refIndex = PR_MAX(aIndex, 0);   
-        nsCOMPtr<nsIDOMNode> refCell;
+  if (doInsert) {
+    nsCOMPtr<nsIDOMNode> refCell;
+    cells->Item(aIndex, getter_AddRefs(refCell));
 
-        cells->Item(refIndex, getter_AddRefs(refCell));
+    rv = InsertBefore(cellNode, refCell, getter_AddRefs(retChild));
+  } else {
+    rv = AppendChild(cellNode, getter_AddRefs(retChild));
+  }
 
-        rv = InsertBefore(cellNode, refCell, getter_AddRefs(retChild));
-      } else {
-        rv = AppendChild(cellNode, getter_AddRefs(retChild));
-      }
-
-      if (retChild) {
-        retChild->QueryInterface(NS_GET_IID(nsIDOMHTMLElement),
-                                 (void **)aValue);
-      }
-    }
-  }  
+  if (retChild) {
+    CallQueryInterface(retChild, aValue);
+  }
 
   return NS_OK;
 }
@@ -497,16 +475,31 @@ nsHTMLTableRowElement::InsertCell(PRInt32 aIndex, nsIDOMHTMLElement** aValue)
 NS_IMETHODIMP
 nsHTMLTableRowElement::DeleteCell(PRInt32 aValue)
 {
-  if (aValue < 0)
+  if (aValue < -1) {
     return NS_ERROR_DOM_INDEX_SIZE_ERR;
-  
-  nsCOMPtr<nsIDOMHTMLCollection> cells;
+  }
 
+  nsCOMPtr<nsIDOMHTMLCollection> cells;
   GetCells(getter_AddRefs(cells));
 
-  nsCOMPtr<nsIDOMNode> cell;
+  nsresult rv;
+  PRUint32 refIndex;
+  if (aValue == -1) {
+    rv = cells->GetLength(&refIndex);
+    NS_ENSURE_SUCCESS(rv, rv);
 
-  nsresult rv = cells->Item(aValue, getter_AddRefs(cell));
+    if (refIndex == 0) {
+      return NS_OK;
+    }
+
+    --refIndex;
+  }
+  else {
+    refIndex = (PRUint32)aValue;
+  }
+
+  nsCOMPtr<nsIDOMNode> cell;
+  rv = cells->Item(refIndex, getter_AddRefs(cell));
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!cell) {
@@ -514,10 +507,7 @@ nsHTMLTableRowElement::DeleteCell(PRInt32 aValue)
   }
 
   nsCOMPtr<nsIDOMNode> retChild;
-
-  RemoveChild(cell, getter_AddRefs(retChild));
-
-  return NS_OK;
+  return RemoveChild(cell, getter_AddRefs(retChild));
 }
 
 NS_IMPL_STRING_ATTR_DEFAULT_VALUE(nsHTMLTableRowElement, Align, align, "left")
@@ -529,29 +519,32 @@ NS_IMPL_STRING_ATTR_DEFAULT_VALUE(nsHTMLTableRowElement, VAlign, valign, "middle
 
 NS_IMETHODIMP
 nsHTMLTableRowElement::StringToAttribute(nsIAtom* aAttribute,
-                                  const nsAString& aValue,
-                                  nsHTMLValue& aResult)
+                                         const nsAString& aValue,
+                                         nsHTMLValue& aResult)
 {
-  /* ignore these attributes, stored simply as strings
-     ch
+  /*
+   * ignore these attributes, stored simply as strings
+   *
+   * ch
    */
-  /* attributes that resolve to integers with default=0*/
+
+  /* attributes that resolve to integers with default = 0 */
   if (aAttribute == nsHTMLAtoms::charoff) {
-    if (ParseValue(aValue, 0, aResult, eHTMLUnit_Integer)) {
+    if (aResult.ParseIntWithBounds(aValue, eHTMLUnit_Integer, 0)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
   else if (aAttribute == nsHTMLAtoms::height) {
     /* attributes that resolve to integers or percents */
 
-    if (ParseValueOrPercent(aValue, aResult, eHTMLUnit_Pixel)) {
+    if (aResult.ParseSpecialIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE, PR_FALSE)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
   else if (aAttribute == nsHTMLAtoms::width) {
     /* attributes that resolve to integers or percents */
 
-    if (ParseValueOrPercent(aValue, aResult, eHTMLUnit_Pixel)) {
+    if (aResult.ParseSpecialIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE, PR_FALSE)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -563,7 +556,11 @@ nsHTMLTableRowElement::StringToAttribute(nsIAtom* aAttribute,
     }
   }
   else if (aAttribute == nsHTMLAtoms::bgcolor) {
-    if (ParseColor(aValue, mDocument, aResult)) {
+    nsCOMPtr<nsIDocument> doc(mDocument);
+    if (!doc) {
+      mNodeInfo->GetDocument(getter_AddRefs(doc));
+    }
+    if (aResult.ParseColor(aValue, doc)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -646,19 +643,23 @@ void MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes, nsRuleDat
 
 NS_IMETHODIMP
 nsHTMLTableRowElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                                PRInt32& aHint) const
+                                                nsChangeHint& aHint) const
 {
-  if ((aAttribute == nsHTMLAtoms::align) ||
-      (aAttribute == nsHTMLAtoms::valign) ||
-      (aAttribute == nsHTMLAtoms::height)) {
-    aHint = NS_STYLE_HINT_REFLOW;
-  }
-  else if (!GetCommonMappedAttributesImpact(aAttribute, aHint)) {
-    if (!GetBackgroundAttributesImpact(aAttribute, aHint)) {
-      aHint = NS_STYLE_HINT_CONTENT;
-    }
-  }
+  static const AttributeImpactEntry attributes[] = {
+    { &nsHTMLAtoms::align, NS_STYLE_HINT_REFLOW },
+    { &nsHTMLAtoms::valign, NS_STYLE_HINT_REFLOW }, 
+    { &nsHTMLAtoms::height, NS_STYLE_HINT_REFLOW },
+    { nsnull, NS_STYLE_HINT_NONE }
+  };
 
+  static const AttributeImpactEntry* const map[] = {
+    attributes,
+    sCommonAttributeMap,
+    sBackgroundAttributeMap,
+  };
+
+  FindAttributeImpact(aAttribute, aHint, map, NS_ARRAY_LENGTH(map));
+  
   return NS_OK;
 }
 
@@ -668,14 +669,3 @@ nsHTMLTableRowElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aM
   aMapRuleFunc = &MapAttributesIntoRule;
   return NS_OK;
 }
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLTableRowElement::SizeOf(nsISizeOfHandler* aSizer,
-                              PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
