@@ -27,31 +27,25 @@
  * Date             Modified by     Description of modification
  * 04/20/2000       IBM Corp.      OS/2 VisualAge build.
  */
-#include "nsICSSLoader.h"
-#include "nsICSSLoaderObserver.h"
-#include "nsICSSParser.h"
-#include "nsICSSStyleSheet.h"
-
-#include "nsIParser.h"
 #include "nsIContent.h"
 #include "nsIDOMNode.h"
-#include "nsIStyleSheetLinkingElement.h"
+#include "nsIDOMWindow.h"
 #include "nsIDocument.h"
 #include "nsINameSpaceManager.h"
-#include "nsIUnicharStreamLoader.h"
 #include "nsIUnicharInputStream.h"
 #include "nsIConverterInputStream.h"
 #include "nsICharsetAlias.h"
 #include "nsUnicharUtils.h"
 #include "nsHashtable.h"
 #include "nsIURL.h"
-#include "nsIURL.h"
 #include "nsIServiceManager.h"
 #include "nsNetUtil.h"
 #include "nsCRT.h"
 #include "nsCOMArray.h"
 #include "nsCOMPtr.h"
 #include "nsIScriptSecurityManager.h"
+#include "nsContentPolicyUtils.h"
+#include "nsIScriptGlobalObject.h"
 #include "nsITimelineService.h"
 #include "nsIHttpChannel.h"
 #include "nsIConsoleService.h"
@@ -59,8 +53,10 @@
 #include "nsIStringBundle.h"
 #include "nsMimeTypes.h"
 #include "nsIAtom.h"
+#include "nsCSSLoader.h"
+#include "nsIDOM3Node.h"
 
-#ifdef INCLUDE_XUL
+#ifdef MOZ_XUL
 #include "nsIXULPrototypeCache.h"
 #endif
 
@@ -104,197 +100,9 @@ static PRLogModuleInfo *gLoaderLog = PR_NewLogModule("nsCSSLoader");
 #define LOG_URI(format, uri)
 #endif // PR_LOGGING
 
+static NS_DEFINE_CID(kCharsetAliasCID, NS_CHARSETALIAS_CID);
 static NS_DEFINE_CID(kCStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 
-class CSSLoaderImpl;
-
-/**
- * OVERALL ARCHITECTURE
- *
- * The CSS Loader gets requests to load various sorts of style sheets:
- * inline style from <style> elements, linked style, @import-ed child
- * sheets, agent sheets.  The loader handles the following tasks:
- *
- * 1) Creation of the actual style sheet objects: CreateSheet()
- * 2) setting of the right media, title, enabled state, etc on the
- *    sheet: PrepareSheet()
- * 3) Insertion of the sheet in the proper cascade order:
- *    InsertSheetInDoc() and InsertChildSheet()
- * 4) Load of the sheet: LoadSheet()
- * 5) Parsing of the sheet: ParseSheet()
- * 6) Cleanup: SheetComplete()
- *
- * The detailed documentation for these functions is found with the
- * function implementations.
- *
- * The following helper objects are used:
- * A) URLKey -- a nsHashKey implementation that uses an nsIURI as the key
- * B) SheetLoadData -- a small class that is used to store all the
- *                     information needed for the loading of a sheet;
- *                     this class handles listening for the stream
- *                     loader completion and also handles charset
- *                     determination.
- */
-
-/************************************
- * URLKey class used for hashtables *
- ************************************/
-                                                            
-MOZ_DECL_CTOR_COUNTER(URLKey)
-
-class URLKey: public nsHashKey {
-public:
-  URLKey(nsIURI* aURL)
-    : mURL(aURL)
-  {
-    MOZ_COUNT_CTOR(URLKey);
-    mHashValue = 0;
-
-    nsCAutoString spec;
-    mURL->GetSpec(spec);
-    if (!spec.IsEmpty()) {
-      mHashValue = nsCRT::HashCode(spec.get());
-    }
-  }
-
-  URLKey(const URLKey& aKey)
-    : mURL(aKey.mURL),
-      mHashValue(aKey.mHashValue)
-  {
-    MOZ_COUNT_CTOR(URLKey);
-  }
-
-  virtual ~URLKey(void)
-  {
-    MOZ_COUNT_DTOR(URLKey);
-  }
-
-  virtual PRUint32 HashCode(void) const
-  {
-    return mHashValue;
-  }
-
-  virtual PRBool Equals(const nsHashKey* aKey) const
-  {
-    URLKey* key = (URLKey*)aKey;
-    PRBool equals = PR_FALSE;
-    nsresult result = mURL->Equals(key->mURL, &equals);
-    return (NS_SUCCEEDED(result) && equals);
-  }
-
-  virtual nsHashKey *Clone(void) const
-  {
-    return new URLKey(*this);
-  }
-
-  nsCOMPtr<nsIURI>  mURL;
-  PRUint32          mHashValue;
-};
-
-/*********************************************
- * Data needed to properly load a stylesheet *
- *********************************************/
-
-class SheetLoadData : public nsIUnicharStreamLoaderObserver
-{
-public:
-  virtual ~SheetLoadData(void);
-  // Data for loading a sheet linked from a document
-  SheetLoadData(CSSLoaderImpl* aLoader,
-                const nsAString& aTitle,
-                nsIParser* aParserToUnblock,
-                nsIURI* aURI,
-                nsICSSStyleSheet* aSheet,
-                nsIStyleSheetLinkingElement* aOwningElement,
-                nsICSSLoaderObserver* aObserver);                 
-
-  // Data for loading a sheet linked from an @import rule
-  SheetLoadData(CSSLoaderImpl* aLoader,
-                nsIURI* aURI,
-                nsICSSStyleSheet* aSheet,
-                SheetLoadData* aParentData,
-                nsICSSLoaderObserver* aObserver);                 
-
-  // Data for loading an agent sheet
-  SheetLoadData(CSSLoaderImpl* aLoader,
-                nsIURI* aURI,
-                nsICSSStyleSheet* aSheet,
-                PRBool aSyncLoad,
-                nsICSSLoaderObserver* aObserver);
-                 
-  
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSIUNICHARSTREAMLOADEROBSERVER
-
-  // Hold a ref to the CSSLoader so we can call back to it to let it
-  // know the load finished
-  CSSLoaderImpl*             mLoader; // strong ref
-
-  // Title needed to pull datas out of the pending datas table when
-  // the preferred title is changed
-  nsString                   mTitle;
-
-  // Parser to be told to continue parsing once the load completes
-  nsCOMPtr<nsIParser>        mParserToUnblock;
-
-  // URI we're loading.  Null for inline sheets
-  nsCOMPtr<nsIURI>           mURI;
-
-  // The sheet we're loading data for
-  nsCOMPtr<nsICSSStyleSheet> mSheet;
-
-  // Linked list of datas for the same URI as us
-  SheetLoadData*             mNext;  // strong ref
-
-  // Load data for the sheet that @import-ed us if we were @import-ed
-  // during the parse
-  SheetLoadData*             mParentData;  // strong ref
-
-  // Number of sheets we @import-ed that are still loading
-  PRUint32                   mPendingChildren;
-
-  // mSyncLoad is true when the load needs to be synchronous -- right
-  // now only for LoadAgentSheet with no observer
-  PRPackedBool               mSyncLoad;
-
-  // mIsAgent is true if the load was triggered by LoadAgentSheet or
-  // an @import from an agent sheet.  Agent loads can proceed even if
-  // we have no document.
-  PRPackedBool               mIsAgent;
-
-  // mIsLoading is true from the moment we are placed in the loader's
-  // "loading datas" table (right after the async channel is opened)
-  // to the moment we are removed from said table (due to the load
-  // completing or being cancelled).
-  PRPackedBool               mIsLoading; // Set once the data is in the "loading" table
-
-  // mIsCancelled is set to true when a sheet load is stopped by
-  // Stop() or StopLoadingSheet().  SheetLoadData::OnStreamComplete()
-  // checks this to avoid parsing sheets that have been cancelled and
-  // such.
-  PRPackedBool               mIsCancelled;
-  
-  // This is the element that imported the sheet.  Needed to get the
-  // charset set on it.
-  nsCOMPtr<nsIStyleSheetLinkingElement> mOwningElement;
-
-  // The observer that wishes to be notified of load completion
-  nsCOMPtr<nsICSSLoaderObserver>        mObserver;
-};
-
-NS_IMPL_ISUPPORTS1(SheetLoadData, nsIUnicharStreamLoaderObserver);
-
-/***********************************************************************
- * Enum that describes the state of the sheet returned by CreateSheet. *
- ***********************************************************************/
-enum StyleSheetState {
-  eSheetStateUnknown = 0,
-  eSheetNeedsParser,
-  eSheetPending,
-  eSheetLoading,
-  eSheetComplete
-};
-
 // And some convenience strings...
 #ifdef PR_LOGGING
 static const char* const gStateStrings[] = {
@@ -306,144 +114,10 @@ static const char* const gStateStrings[] = {
 };
 #endif
 
-/**********************
- * Loader Declaration *
- **********************/
-
-class CSSLoaderImpl : public nsICSSLoader
-{
-public:
-  CSSLoaderImpl(void);
-  virtual ~CSSLoaderImpl(void);
-
-  NS_DECL_ISUPPORTS
-
-  // nsICSSLoader methods
-  NS_IMETHOD Init(nsIDocument* aDocument);
-  NS_IMETHOD DropDocumentReference(void);
-
-  NS_IMETHOD SetCaseSensitive(PRBool aCaseSensitive);
-  NS_IMETHOD SetCompatibilityMode(nsCompatibility aCompatMode);
-  NS_IMETHOD SetPreferredSheet(const nsAString& aTitle);
-
-  NS_IMETHOD GetParserFor(nsICSSStyleSheet* aSheet,
-                          nsICSSParser** aParser);
-  NS_IMETHOD RecycleParser(nsICSSParser* aParser);
-
-  NS_IMETHOD LoadInlineStyle(nsIContent* aElement,
-                             nsIUnicharInputStream* aIn, 
-                             const nsAString& aTitle, 
-                             const nsAString& aMedia, 
-                             PRInt32 aDefaultNameSpaceID,
-                             nsIParser* aParserToUnblock,
-                             PRBool& aCompleted,
-                             nsICSSLoaderObserver* aObserver);
-
-  NS_IMETHOD LoadStyleLink(nsIContent* aElement,
-                           nsIURI* aURL, 
-                           const nsAString& aTitle, 
-                           const nsAString& aMedia, 
-                           PRInt32 aDefaultNameSpaceID,
-                           nsIParser* aParserToUnblock,
-                           PRBool& aCompleted,
-                           nsICSSLoaderObserver* aObserver);
-
-  NS_IMETHOD LoadChildSheet(nsICSSStyleSheet* aParentSheet,
-                            nsIURI* aURL, 
-                            const nsAString& aMedia,
-                            PRInt32 aDefaultNameSpaceID,
-                            nsICSSImportRule* aRule);
-
-  NS_IMETHOD LoadAgentSheet(nsIURI* aURL, nsICSSStyleSheet** aSheet);
-
-  NS_IMETHOD LoadAgentSheet(nsIURI* aURL, nsICSSLoaderObserver* aObserver);
-
-  // stop loading all sheets
-  NS_IMETHOD Stop(void);
-
-  // stop loading one sheet
-  NS_IMETHOD StopLoadingSheet(nsIURI* aURL);
-
-  /**
-   * Is the loader enabled or not.
-   * When disabled, processing of new styles is disabled and an attempt
-   * to do so will fail with a return code of
-   * NS_ERROR_NOT_AVAILABLE. Note that this DOES NOT disable
-   * currently loading styles or already processed styles.
-   */
-  NS_IMETHOD GetEnabled(PRBool *aEnabled);
-  NS_IMETHOD SetEnabled(PRBool aEnabled);
-
-  // local helper methods (some are public for access from statics)
-  PRBool IsAlternate(const nsAString& aTitle);
-
-private:
-  nsresult CreateSheet(nsIURI* aURI,
-                       PRUint32 aDefaultNameSpaceID,
-                       PRBool aSyncLoad,
-                       StyleSheetState& aSheetState,
-                       nsICSSStyleSheet** aSheet);
-
-  nsresult PrepareSheet(nsICSSStyleSheet* aSheet,
-                        const nsAString& aTitle,
-                        const nsAString& aMedia);
-
-  nsresult InsertSheetInDoc(nsICSSStyleSheet* aSheet,
-                            nsIContent* aLinkingElement,
-                            nsIDocument* aDocument);
-
-  nsresult InsertChildSheet(nsICSSStyleSheet* aSheet,
-                            nsICSSStyleSheet* aParentSheet,
-                            nsICSSImportRule* aParentRule);
-
-  nsresult InternalLoadAgentSheet(nsIURI* aURL,
-                                  nsICSSStyleSheet** aSheet,
-                                  nsICSSLoaderObserver* aObserver);
-  
-public:
-  nsresult LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState);
-
-protected:
-  friend class SheetLoadData;
-
-  // Protected functions and members are ones that SheetLoadData needs
-  // access to
-  nsresult ParseSheet(nsIUnicharInputStream* aStream,
-                      SheetLoadData* aLoadData,
-                      PRBool& aCompleted);
-
-public:
-  void SheetComplete(SheetLoadData* aLoadData, PRBool aSucceeded);
-
-protected:
-  // the load data needs access to the document...
-  nsIDocument*      mDocument;  // the document we live for
-
-#ifdef DEBUG
-  PRPackedBool            mSyncCallback;
-#endif
-
-private:
-  PRPackedBool      mCaseSensitive; // is document CSS case sensitive
-  PRPackedBool      mEnabled; // is enabled to load new styles
-  nsCompatibility   mCompatMode;
-  nsString          mPreferredSheet;  // title of preferred sheet
-
-  nsCOMArray<nsICSSParser> mParsers;  // array of CSS parsers
-
-  nsSupportsHashtable mCompleteSheets;
-  nsHashtable         mLoadingDatas;  // weak refs
-  nsHashtable         mPendingDatas;  // weak refs
-  
-  // We're not likely to have many levels of @import...  But likely to have
-  // some.  Allocate some storage, what the hell.
-  nsAutoVoidArray   mParsingDatas;
-
-};
-
 /********************************
  * SheetLoadData implementation *
  ********************************/
+NS_IMPL_ISUPPORTS1(SheetLoadData, nsIUnicharStreamLoaderObserver);
 
 SheetLoadData::SheetLoadData(CSSLoaderImpl* aLoader,
                              const nsAString& aTitle,
@@ -537,6 +211,9 @@ SheetLoadData::~SheetLoadData()
  * Loader Implementation *
  *************************/
 
+// static
+nsCOMArray<nsICSSParser>* CSSLoaderImpl::gParsers = nsnull;
+
 CSSLoaderImpl::CSSLoaderImpl(void)
   : mDocument(nsnull), 
     mCaseSensitive(PR_FALSE),
@@ -555,6 +232,12 @@ CSSLoaderImpl::~CSSLoaderImpl(void)
 
 NS_IMPL_ISUPPORTS1(CSSLoaderImpl, nsICSSLoader)
 
+void
+CSSLoaderImpl::Shutdown()
+{
+  delete gParsers;
+}
+
 NS_IMETHODIMP
 CSSLoaderImpl::Init(nsIDocument* aDocument)
 {
@@ -632,21 +315,25 @@ NS_IMETHODIMP
 CSSLoaderImpl::GetParserFor(nsICSSStyleSheet* aSheet, 
                             nsICSSParser** aParser)
 {
-  NS_ASSERTION(aParser, "null pointer");
-  if (! aParser) {
-    return NS_ERROR_NULL_POINTER;
+  NS_PRECONDITION(aParser, "Null out param");
+
+  *aParser = nsnull;
+
+  if (!gParsers) {
+    gParsers = new nsCOMArray<nsICSSParser>;
+    if (!gParsers) {
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+  }
+
+  PRInt32 count = gParsers->Count();
+  if (0 < count--) {
+    *aParser = gParsers->ObjectAt(count);
+    NS_ADDREF(*aParser);
+    gParsers->RemoveObjectAt(count);
   }
 
   nsresult result = NS_OK;
-
-  *aParser = nsnull;
-  PRInt32 count = mParsers.Count();
-  if (0 < count--) {
-    *aParser = mParsers[count];
-    NS_ADDREF(*aParser);
-    mParsers.RemoveObjectAt(count);
-  }
-
   if (! *aParser) {
     result = NS_NewCSSParser(aParser);
   }
@@ -666,36 +353,20 @@ NS_IMETHODIMP
 CSSLoaderImpl::RecycleParser(nsICSSParser* aParser)
 {
   NS_PRECONDITION(aParser, "Recycle only good parsers, please");
+  NS_ENSURE_TRUE(gParsers, NS_ERROR_UNEXPECTED);
 
-  if (!mParsers.AppendObject(aParser)) {
+  if (!gParsers->AppendObject(aParser)) {
     return NS_ERROR_FAILURE;
   }
   
   return NS_OK;
 }
 
-// XXX We call this function a good bit.  Consider caching the service
-// in a static global or something?
-static nsresult ResolveCharset(const nsAString& aCharsetAlias,
-                               nsAString& aCharset)
-{
-  nsresult rv = NS_ERROR_NOT_AVAILABLE;
-  if (! aCharsetAlias.IsEmpty()) {
-    nsCOMPtr<nsICharsetAlias> calias(do_GetService(kCharsetAliasCID, &rv));
-    NS_ASSERTION(calias, "cannot find charset alias service");
-    if (calias)
-    {
-      rv = calias->GetPreferred(aCharsetAlias, aCharset);
-    }
-  }
-  return rv;
-}
-
 static const char kCharsetSym[] = "@charset";
 
 static nsresult GetCharsetFromData(const unsigned char* aStyleSheetData,
                                    PRUint32 aDataLength,
-                                   nsAString& aCharset)
+                                   nsACString& aCharset)
 {
   aCharset.Truncate();
   if (aDataLength <= sizeof(kCharsetSym) - 1)
@@ -857,7 +528,8 @@ static nsresult GetCharsetFromData(const unsigned char* aStyleSheetData,
       break;
     }
     
-    aCharset.Append(PRUnichar(aStyleSheetData[pos]));
+    // casting to avoid ambiguities
+    aCharset.Append(char(aStyleSheetData[pos]));
     pos += step;
   }
 
@@ -887,6 +559,8 @@ SheetLoadData::OnDetermineCharset(nsIUnicharStreamLoader* aLoader,
   if (NS_FAILED(result))
     channel = nsnull;
 
+  aCharset.Truncate();
+
   /*
    * First determine the charset (if one is indicated)
    * 1)  Check nsIChannel::contentCharset
@@ -897,67 +571,63 @@ SheetLoadData::OnDetermineCharset(nsIUnicharStreamLoader* aLoader,
    * default (document charset or ISO-8859-1 if we have no document
    * charset)
    */
-  nsAutoString charset;
-  nsAutoString charsetCandidate;
   if (channel) {
-    nsCAutoString charsetVal;
-    channel->GetContentCharset(charsetVal);
-    CopyASCIItoUCS2(charsetVal, charsetCandidate);
+    channel->GetContentCharset(aCharset);
   }
 
   result = NS_ERROR_NOT_AVAILABLE;
-  if (! charsetCandidate.IsEmpty()) {
-#ifdef DEBUG_bzbarsky
-    fprintf(stderr, "Setting from HTTP to: %s\n", NS_ConvertUCS2toUTF8(charsetCandidate).get());
-#endif
-    result = ResolveCharset(charsetCandidate, charset);
-  }
 
-  if (NS_FAILED(result)) {
-    //  We have no charset or the HTTP charset is not recognized.
+#ifdef DEBUG_bzbarsky
+  if (! aCharset.IsEmpty()) {
+    fprintf(stderr, "Setting from HTTP to: %s\n",
+            PromiseFlatCString(aCharset).get());
+  }
+#endif
+
+  if (aCharset.IsEmpty()) {
+    //  We have no charset
     //  Try @charset rule
     result = GetCharsetFromData((const unsigned char*)aData,
-                                aDataLength, charsetCandidate);
-    if (NS_SUCCEEDED(result)) {
+                                aDataLength, aCharset);
 #ifdef DEBUG_bzbarsky
+    if (NS_SUCCEEDED(result)) {
       fprintf(stderr, "Setting from @charset rule: %s\n",
-              NS_ConvertUCS2toUTF8(charsetCandidate).get());
-#endif
-      result = ResolveCharset(charsetCandidate, charset);
+              PromiseFlatCString(aCharset).get());
     }
+#endif
   }
 
-  if (NS_FAILED(result)) {
+  if (aCharset.IsEmpty()) {
     // Now try the charset on the <link> or processing instruction
     // that loaded us
     if (mOwningElement) {
-      mOwningElement->GetCharset(charsetCandidate);
-      if (! charsetCandidate.IsEmpty()) {
+      nsAutoString elementCharset;
+      mOwningElement->GetCharset(elementCharset);
+      CopyUCS2toASCII(elementCharset, aCharset);
 #ifdef DEBUG_bzbarsky
+      if (! aCharset.IsEmpty()) {
         fprintf(stderr, "Setting from property on element: %s\n",
-                NS_ConvertUCS2toUTF8(charsetCandidate).get());
-#endif
-        result = ResolveCharset(charsetCandidate, charset);
+                PromiseFlatCString(aCharset).get());
       }
+#endif
     }
   }
 
-  if (NS_FAILED(result) && mLoader->mDocument) {
+  if (aCharset.IsEmpty() && mLoader->mDocument) {
     // no useful data on charset.  Try the document charset.
     // That needs no resolution, since it's already fully resolved
-    mLoader->mDocument->GetDocumentCharacterSet(charset);
+    mLoader->mDocument->GetDocumentCharacterSet(aCharset);
 #ifdef DEBUG_bzbarsky
     fprintf(stderr, "Set from document: %s\n",
-            NS_ConvertUCS2toUTF8(charset).get());
+            PromiseFlatCString(aCharset).get());
 #endif
   }      
 
-  if (charset.IsEmpty()) {
+  if (aCharset.IsEmpty()) {
     NS_WARNING("Unable to determine charset for sheet, using ISO-8859-1!");
-    charset = NS_LITERAL_STRING("ISO-8859-1");
+    aCharset = NS_LITERAL_CSTRING("ISO-8859-1");
   }
   
-  aCharset = NS_ConvertUCS2toUTF8(charset);
   return NS_OK;
 }
 
@@ -1120,7 +790,7 @@ SheetLoadData::OnStreamComplete(nsIUnicharStreamLoader* aLoader,
   return mLoader->ParseSheet(aDataStream, this, completed);
 }
 
-#ifdef INCLUDE_XUL
+#ifdef MOZ_XUL
 static PRBool IsChromeURI(nsIURI* aURI)
 {
   NS_ASSERTION(aURI, "Have to pass in a URI");
@@ -1218,6 +888,63 @@ CSSLoaderImpl::IsAlternate(const nsAString& aTitle)
   return PR_FALSE;
 }
 
+/**
+ * CheckLoadAllowed will return success if the load is allowed,
+ * failure otherwise. 
+ *
+ * @param aSourceURI the uri of the document or parent sheet loading the sheet
+ * @param aTargetURI the uri of the sheet to be loaded
+ * @param aContext the context.  This is the element or the @import
+ *        rule doing the loading
+ */
+nsresult
+CSSLoaderImpl::CheckLoadAllowed(nsIURI* aSourceURI,
+                                nsIURI* aTargetURI,
+                                nsISupports* aContext)
+{
+  LOG(("CSSLoaderImpl::CheckLoadAllowed"));
+  
+  // Check with the security manager
+  nsresult rv;
+  nsCOMPtr<nsIScriptSecurityManager> secMan =
+           do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = secMan->CheckLoadURI(aSourceURI, aTargetURI,
+                            nsIScriptSecurityManager::ALLOW_CHROME);
+  if (NS_FAILED(rv)) { // failure is normal here; don't warn
+    return rv;
+  }
+
+  LOG(("  Passed security check"));
+
+  // Check with content policy
+
+  if (!mDocument) {
+    return NS_OK;
+  }
+  
+  nsCOMPtr<nsIScriptGlobalObject> globalObject;
+  rv = mDocument->GetScriptGlobalObject(getter_AddRefs(globalObject));
+  if (NS_FAILED(rv) || !globalObject) {
+    LOG(("  No script global object"));
+    return rv;
+  }
+  
+  nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(globalObject));
+  NS_ASSERTION(domWin, "Global object not DOM window?");
+
+  PRBool shouldLoad = PR_TRUE;
+  rv = NS_CheckContentLoadPolicy(nsIContentPolicy::STYLESHEET, aTargetURI,
+                                 aContext, domWin, &shouldLoad);
+  if (NS_SUCCEEDED(rv) && !shouldLoad) {
+    LOG(("  Blocked by content policy"));
+    return NS_ERROR_FAILURE;
+  }
+
+  return rv;
+}
+
 /**
  * CreateSheet() creates an nsICSSStyleSheet object for the given URI,
  * if any.  If there is no URI given, we just create a new style sheet
@@ -1229,6 +956,7 @@ CSSLoaderImpl::IsAlternate(const nsAString& aTitle)
  */
 nsresult
 CSSLoaderImpl::CreateSheet(nsIURI* aURI,
+                           nsIContent* aLinkingContent,
                            PRUint32 aDefaultNameSpaceID,
                            PRBool aSyncLoad,
                            StyleSheetState& aSheetState,
@@ -1246,7 +974,7 @@ CSSLoaderImpl::CreateSheet(nsIURI* aURI,
     nsCOMPtr<nsICSSStyleSheet> sheet;
 
     // First, the XUL cache
-#ifdef INCLUDE_XUL
+#ifdef MOZ_XUL
     if (IsChromeURI(aURI)) {
       nsCOMPtr<nsIXULPrototypeCache> cache(do_GetService("@mozilla.org/xul/xul-prototype-cache;1"));
       if (cache) {
@@ -1298,7 +1026,7 @@ CSSLoaderImpl::CreateSheet(nsIURI* aURI,
       sheet->GetComplete(complete);
       if (!modified || !complete) {
         // Proceed on failures; at worst we'll try to create one below
-        sheet->Clone(*aSheet);
+        sheet->Clone(nsnull, nsnull, nsnull, nsnull, aSheet);
         NS_ASSERTION(complete || aSheetState != eSheetComplete,
                      "Sheet thinks it's not complete while we think it is");
       }
@@ -1312,8 +1040,8 @@ CSSLoaderImpl::CreateSheet(nsIURI* aURI,
     if (!sheetURI) {
       // Inline style.  Use the document's base URL so that @import in
       // the inline sheet picks up the right base.
-      NS_ASSERTION(mDocument, "How did we get in here without a document?");
-      mDocument->GetBaseURL(*getter_AddRefs(sheetURI));
+      NS_ASSERTION(aLinkingContent, "Inline stylesheet without linking content?");
+      aLinkingContent->GetBaseURL(getter_AddRefs(sheetURI));
     }
 
     rv = NS_NewCSSStyleSheet(aSheet, sheetURI);
@@ -1367,7 +1095,7 @@ CSSLoaderImpl::PrepareSheet(nsICSSStyleSheet* aSheet,
  */
 nsresult
 CSSLoaderImpl::InsertSheetInDoc(nsICSSStyleSheet* aSheet,
-                                nsIContent* aLinkingElement,
+                                nsIContent* aLinkingContent,
                                 nsIDocument* aDocument)
 {
   LOG(("CSSLoaderImpl::InsertSheetInDoc"));
@@ -1376,8 +1104,8 @@ CSSLoaderImpl::InsertSheetInDoc(nsICSSStyleSheet* aSheet,
 
   // all nodes that link in sheets should be implementing nsIDOM3Node
   nsresult rv = NS_OK;
-  nsCOMPtr<nsIDOM3Node> linkingNode = do_QueryInterface(aLinkingElement);
-  NS_ASSERTION(linkingNode || !aLinkingElement,
+  nsCOMPtr<nsIDOM3Node> linkingNode = do_QueryInterface(aLinkingContent);
+  NS_ASSERTION(linkingNode || !aLinkingContent,
                "Need to implement nsIDOM3Node to get insertion order right");
 
   // XXX Need to cancel pending sheet loads for this element, if any
@@ -1420,7 +1148,7 @@ CSSLoaderImpl::InsertSheetInDoc(nsICSSStyleSheet* aSheet,
     rv = linkingNode->CompareDocumentPosition(sheetOwner, &comparisonFlags);
     // If we can't get the order right, just bail...
     NS_ENSURE_SUCCESS(rv, rv);
-    NS_ASSERTION(!(comparisonFlags & nsIDOMNode::DOCUMENT_POSITION_DISCONNECTED),
+    NS_ASSERTION(!(comparisonFlags & nsIDOM3Node::DOCUMENT_POSITION_DISCONNECTED),
                  "Why are these elements in different documents?");
 #ifdef DEBUG
     {
@@ -1429,7 +1157,7 @@ CSSLoaderImpl::InsertSheetInDoc(nsICSSStyleSheet* aSheet,
       NS_ASSERTION(!sameNode, "Why do we still have our old sheet?");
     }
 #endif // DEBUG
-    if (comparisonFlags & nsIDOMNode::DOCUMENT_POSITION_PRECEDING) {
+    if (comparisonFlags & nsIDOM3Node::DOCUMENT_POSITION_PRECEDING) {
       // The current sheet comes before us, and it better be the first
       // such, because now we break
       break;
@@ -1441,7 +1169,7 @@ CSSLoaderImpl::InsertSheetInDoc(nsICSSStyleSheet* aSheet,
   // XXX <meta> elements do not implement nsIStyleSheetLinkingElement;
   // need to fix this for them to be ordered correctly.
   nsCOMPtr<nsIStyleSheetLinkingElement>
-    linkingElement = do_QueryInterface(aLinkingElement);
+    linkingElement = do_QueryInterface(aLinkingContent);
   if (linkingElement) {
     linkingElement->SetStyleSheet(aSheet); // This sets the ownerNode on the sheet
   }
@@ -1540,7 +1268,7 @@ CSSLoaderImpl::LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState)
 
     // 8192 is a nice magic number that happens to be what a lot of
     // other things use for buffer sizes.
-    rv = converterStream->Init(stream, NS_LITERAL_STRING("UTF-8").get(),
+    rv = converterStream->Init(stream, "UTF-8",
                                8192, PR_TRUE);
     
     if (NS_FAILED(rv)) {
@@ -1632,6 +1360,10 @@ CSSLoaderImpl::LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState)
     }
   }
 
+  // Now tell the channel we expect text/css data back....  We do
+  // this before opening it, so it's only treated as a hint.
+  channel->SetContentType(NS_LITERAL_CSTRING("text/css"));
+
   // We don't have to hold on to the stream loader.  The ownership
   // model is: Necko owns the stream loader, which owns the load data,
   // which owns us
@@ -1798,7 +1530,7 @@ CSSLoaderImpl::SheetComplete(SheetLoadData* aLoadData, PRBool aSucceeded)
 
   // Now that it's marked complete, put the sheet in our cache
   if (aSucceeded && aLoadData->mURI) {
-#ifdef INCLUDE_XUL
+#ifdef MOZ_XUL
     if (IsChromeURI(aLoadData->mURI)) {
       nsCOMPtr<nsIXULPrototypeCache> cache(do_GetService("@mozilla.org/xul/xul-prototype-cache;1"));
       if (cache) {
@@ -1818,7 +1550,7 @@ CSSLoaderImpl::SheetComplete(SheetLoadData* aLoadData, PRBool aSucceeded)
 #endif
       URLKey key(aLoadData->mURI);
       mCompleteSheets.Put(&key, aLoadData->mSheet);
-#ifdef INCLUDE_XUL
+#ifdef MOZ_XUL
     }
 #endif
   }
@@ -1858,8 +1590,8 @@ CSSLoaderImpl::LoadInlineStyle(nsIContent* aElement,
   
   StyleSheetState state;
   nsCOMPtr<nsICSSStyleSheet> sheet;
-  nsresult rv = CreateSheet(nsnull, aDefaultNameSpaceID, PR_FALSE, state,
-                            getter_AddRefs(sheet));
+  nsresult rv = CreateSheet(nsnull, aElement, aDefaultNameSpaceID, PR_FALSE,
+                            state, getter_AddRefs(sheet));
   NS_ENSURE_SUCCESS(rv, rv);
   NS_ASSERTION(state == eSheetNeedsParser,
                "Inline sheets should not be cached");
@@ -1909,22 +1641,18 @@ CSSLoaderImpl::LoadStyleLink(nsIContent* aElement,
   
   NS_ENSURE_TRUE(mDocument, NS_ERROR_NOT_INITIALIZED);
 
-  //-- Make sure this page is allowed to load this URL
-  nsresult rv;
-  nsCOMPtr<nsIScriptSecurityManager> secMan = 
-           do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return rv;
+  // Check whether we should even load
   nsCOMPtr<nsIURI> docURI;
-  rv = mDocument->GetDocumentURL(getter_AddRefs(docURI));
+  nsresult rv = mDocument->GetDocumentURL(getter_AddRefs(docURI));
   if (NS_FAILED(rv) || !docURI) return NS_ERROR_FAILURE;
-  rv = secMan->CheckLoadURI(docURI, aURL, nsIScriptSecurityManager::ALLOW_CHROME);
+  rv = CheckLoadAllowed(docURI, aURL, aElement);
   if (NS_FAILED(rv)) return rv;
 
-  LOG(("  Passed security check"));
+  LOG(("  Passed load check"));
   
   StyleSheetState state;
   nsCOMPtr<nsICSSStyleSheet> sheet;
-  rv = CreateSheet(aURL, aDefaultNameSpaceID, PR_FALSE, state,
+  rv = CreateSheet(aURL, aElement, aDefaultNameSpaceID, PR_FALSE, state,
                    getter_AddRefs(sheet));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1991,18 +1719,14 @@ CSSLoaderImpl::LoadChildSheet(nsICSSStyleSheet* aParentSheet,
   
   LOG_URI("  Child uri: '%s'", aURL);
 
-  //-- Make sure this page is allowed to load this URL
-  nsresult rv;
-  nsCOMPtr<nsIScriptSecurityManager> secMan = 
-           do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return rv;
+  // Check whether we should even load
   nsCOMPtr<nsIURI> sheetURI;
-  rv = aParentSheet->GetURL(*getter_AddRefs(sheetURI));
+  nsresult rv = aParentSheet->GetURL(*getter_AddRefs(sheetURI));
   if (NS_FAILED(rv) || !sheetURI) return NS_ERROR_FAILURE;
-  rv = secMan->CheckLoadURI(sheetURI, aURL, nsIScriptSecurityManager::ALLOW_CHROME);
+  rv = CheckLoadAllowed(sheetURI, aURL, aParentRule);
   if (NS_FAILED(rv)) return rv;
 
-  LOG(("  Passed security check"));
+  LOG(("  Passed load check"));
   
   SheetLoadData* parentData = nsnull;
   nsCOMPtr<nsICSSLoaderObserver> observer;
@@ -2025,7 +1749,7 @@ CSSLoaderImpl::LoadChildSheet(nsICSSStyleSheet* aParentSheet,
       data = data->mParentData;
     }
 
-    NS_ASSERTION(parentData->mSheet = aParentSheet,
+    NS_ASSERTION(parentData->mSheet == aParentSheet,
                  "Unexpected call to LoadChildSheet");
   } else {
     LOG(("  No parent load; must be CSSOM"));
@@ -2038,7 +1762,7 @@ CSSLoaderImpl::LoadChildSheet(nsICSSStyleSheet* aParentSheet,
   // loop) do so
   nsCOMPtr<nsICSSStyleSheet> sheet;
   StyleSheetState state;
-  rv = CreateSheet(aURL, aDefaultNameSpaceID,
+  rv = CreateSheet(aURL, nsnull, aDefaultNameSpaceID,
                    parentData ? parentData->mSyncLoad : PR_FALSE,
                    state, getter_AddRefs(sheet));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -2110,8 +1834,8 @@ CSSLoaderImpl::InternalLoadAgentSheet(nsIURI* aURL,
   nsCOMPtr<nsICSSStyleSheet> sheet;
   PRBool syncLoad = (aObserver == nsnull);
   
-  nsresult rv = CreateSheet(aURL, kNameSpaceID_Unknown, syncLoad, state,
-                            getter_AddRefs(sheet));
+  nsresult rv = CreateSheet(aURL, nsnull, kNameSpaceID_Unknown, syncLoad,
+                            state, getter_AddRefs(sheet));
   NS_ENSURE_SUCCESS(rv, rv);
 
   NS_NAMED_LITERAL_STRING(empty, "");
