@@ -51,7 +51,6 @@
 
 #include "nsCOMPtr.h"
 #include "nsIStyleSet.h"
-#include "nsISizeOfHandler.h"
 #include "nsIDOMCSSStyleSheet.h"
 #include "nsIDOMCSSRule.h"
 #include "nsIDOMCSSImportRule.h"
@@ -71,17 +70,20 @@
 NS_IMETHOD GetStyleSheet(nsIStyleSheet*& aSheet) const; \
 NS_IMETHOD SetStyleSheet(nsICSSStyleSheet* aSheet); \
 NS_IMETHOD SetParentRule(nsICSSGroupRule* aRule); \
+NS_IMETHOD GetDOMRule(nsIDOMCSSRule** aDOMRule); \
 NS_IMETHOD MapRuleInfoInto(nsRuleData* aRuleData);
 
 #define IMPL_STYLE_RULE_INHERIT(_class, super) \
 NS_IMETHODIMP _class::GetStyleSheet(nsIStyleSheet*& aSheet) const { return super::GetStyleSheet(aSheet); }  \
 NS_IMETHODIMP _class::SetStyleSheet(nsICSSStyleSheet* aSheet) { return super::SetStyleSheet(aSheet); }  \
 NS_IMETHODIMP _class::SetParentRule(nsICSSGroupRule* aRule) { return super::SetParentRule(aRule); }  \
+NS_IMETHODIMP _class::GetDOMRule(nsIDOMCSSRule** aDOMRule) { return CallQueryInterface(this, aDOMRule); }  \
 NS_IMETHODIMP _class::MapRuleInfoInto(nsRuleData* aRuleData) { return NS_OK; } 
 
 #define IMPL_STYLE_RULE_INHERIT2(_class, super) \
 NS_IMETHODIMP _class::GetStyleSheet(nsIStyleSheet*& aSheet) const { return super::GetStyleSheet(aSheet); }  \
 NS_IMETHODIMP _class::SetParentRule(nsICSSGroupRule* aRule) { return super::SetParentRule(aRule); }  \
+NS_IMETHODIMP _class::GetDOMRule(nsIDOMCSSRule** aDOMRule) { return CallQueryInterface(this, aDOMRule); }  \
 NS_IMETHODIMP _class::MapRuleInfoInto(nsRuleData* aRuleData) { return NS_OK; } 
 
 // -------------------------------
@@ -152,7 +154,7 @@ CSSGroupRuleRuleListImpl::Item(PRUint32 aIndex, nsIDOMCSSRule** aReturn)
 
     result = mGroupRule->GetStyleRuleAt(aIndex, *getter_AddRefs(rule));
     if (rule) {
-      result = CallQueryInterface(rule, aReturn);
+      result = rule->GetDOMRule(aReturn);
     } else if (result == NS_ERROR_ILLEGAL_VALUE) {
       result = NS_OK; // per spec: "Return Value ... null if ... not a valid index."
     }
@@ -180,8 +182,6 @@ public:
   // nsIStyleRule methods
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out = stdout, PRInt32 aIndent = 0) const;
-
-  virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
 #endif
 
   // nsICSSRule methods
@@ -243,42 +243,6 @@ CSSCharsetRuleImpl::List(FILE* out, PRInt32 aIndent) const
 
   return NS_OK;
 }
-
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as CSSCharSetRuleImpl's size): 
-*    1) sizeof(*this) + the size of the mEncoding string
-*
-*  Contained / Aggregated data (not reported as CSSCharsetRuleImpl's size):
-*    none
-*
-*  Children / siblings / parents:
-*    none
-*    
-******************************************************************************/
-void CSSCharsetRuleImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("CSSCharsetRuleImpl");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-  // add the string for encoding value
-  mEncoding.SizeOf(aSizeOfHandler, &localSize);
-  aSize += localSize;
-  aSize -= sizeof(mEncoding); // counted in sizeof(*this) and nsString->SizeOf()
-  aSizeOfHandler->AddSize(tag,aSize);
-}
 #endif
 
 NS_IMETHODIMP
@@ -314,7 +278,7 @@ CSSCharsetRuleImpl::SetEncoding(const nsAString& aEncoding)
 }
 
 
-NS_EXPORT nsresult
+nsresult
 NS_NewCSSCharsetRule(nsICSSRule** aInstancePtrResult, const nsAString& aEncoding)
 {
   if (! aInstancePtrResult) {
@@ -368,7 +332,7 @@ NS_IMETHODIMP
 CSSCharsetRuleImpl::GetParentRule(nsIDOMCSSRule** aParentRule)
 {
   if (mParentRule) {
-    return CallQueryInterface(mParentRule, aParentRule);
+    return mParentRule->GetDOMRule(aParentRule);
   }
   *aParentRule = nsnull;
   return NS_OK;
@@ -395,8 +359,6 @@ public:
   // nsIStyleRule methods
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out = stdout, PRInt32 aIndent = 0) const;
-
-  virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
 #endif
 
   // nsICSSRule methods
@@ -437,10 +399,8 @@ CSSImportRuleImpl::CSSImportRuleImpl(const CSSImportRuleImpl& aCopy)
 {
   
   if (aCopy.mChildSheet) {
-    aCopy.mChildSheet->Clone(*getter_AddRefs(mChildSheet));
-    if (mChildSheet) {
-      mChildSheet->SetOwnerRule(this);
-    }      
+    aCopy.mChildSheet->Clone(nsnull, this, nsnull, nsnull,
+                             getter_AddRefs(mChildSheet));
   }
 
   NS_NewMediaList(getter_AddRefs(mMedia));
@@ -491,44 +451,6 @@ CSSImportRuleImpl::List(FILE* out, PRInt32 aIndent) const
 
   return NS_OK;
 }
-
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as CSSImportRuleImpl's size): 
-*    1) sizeof(*this) + the size of the mURLSpec string
-*
-*  Contained / Aggregated data (not reported as CSSImportRuleImpl's size):
-*    none
-*
-*  Children / siblings / parents:
-*    none
-*    
-******************************************************************************/
-void CSSImportRuleImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("CSSImportRuleImpl");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-
-  // add the strings for the URLSpec and the Media
-  mURLSpec.SizeOf(aSizeOfHandler, &localSize);
-  aSize += localSize;
-  aSize -= sizeof(mURLSpec); // counted in sizeof(*this) and nsString->SizeOf()
-  aSizeOfHandler->AddSize(tag,aSize);
-
-}
 #endif
 
 NS_IMETHODIMP
@@ -605,7 +527,7 @@ CSSImportRuleImpl::SetSheet(nsICSSStyleSheet* aSheet)
   return NS_OK;
 }
 
-NS_EXPORT nsresult
+nsresult
 NS_NewCSSImportRule(nsICSSImportRule** aInstancePtrResult, 
                     const nsString& aURLSpec,
                     const nsString& aMedia)
@@ -670,7 +592,7 @@ NS_IMETHODIMP
 CSSImportRuleImpl::GetParentRule(nsIDOMCSSRule** aParentRule)
 {
   if (mParentRule) {
-    return CallQueryInterface(mParentRule, aParentRule);
+    return mParentRule->GetDOMRule(aParentRule);
   }
   *aParentRule = nsnull;
   return NS_OK;
@@ -726,8 +648,6 @@ public:
   // nsIStyleRule methods
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out = stdout, PRInt32 aIndent = 0) const;
-
-  virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
 #endif
 
   // nsICSSRule methods
@@ -745,6 +665,7 @@ public:
   NS_IMETHOD  EnumerateRulesForwards(nsISupportsArrayEnumFunc aFunc, void * aData) const;
   NS_IMETHOD  DeleteStyleRuleAt(PRUint32 aIndex);
   NS_IMETHOD  InsertStyleRulesAt(PRUint32 aIndex, nsISupportsArray* aRules);
+  NS_IMETHOD  ReplaceStyleRule(nsICSSRule *aOld, nsICSSRule *aNew);
   
   // nsIDOMCSSRule interface
   NS_DECL_NSIDOMCSSRULE
@@ -905,64 +826,6 @@ CSSMediaRuleImpl::List(FILE* out, PRInt32 aIndent) const
   fputs("}\n", out);
   return NS_OK;
 }
-
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as CSSMediaRuleImpl's size): 
-*    1) sizeof(*this) + the size of each unique atom in the mMedia collection
-*
-*  Contained / Aggregated data (not reported as CSSMediaRuleImpl's size):
-*    1) Delegate to the rules in the mRules collection to report theri own size
-*
-*  Children / siblings / parents:
-*    none
-*    
-******************************************************************************/
-void CSSMediaRuleImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("CSSMediaRuleImpl");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-
-  if (mMedia) {
-    // get the sizes of the media atoms (if unique)
-    PRUint32 index = 0;
-    PRUint32 count;
-    mMedia->Count(&count);
-    while (index < count) {
-      nsCOMPtr<nsIAtom> medium = dont_AddRef((nsIAtom*)mMedia->ElementAt(index++));
-      if(medium && uniqueItems->AddItem(medium)){
-        medium->SizeOf(aSizeOfHandler, &localSize);
-        aSize += localSize;
-      }
-    }
-  }
-  // we are done with the size we report for ourself
-  aSizeOfHandler->AddSize(tag,aSize);
-
-  if (mRules) {
-    // delegate to the rules themselves (do not sum into our size)
-    PRUint32 index = 0;
-    PRUint32 count;
-    mRules->Count(&count);
-    while (index < count) {
-      nsCOMPtr<nsICSSRule> rule = dont_AddRef((nsICSSRule*)mRules->ElementAt(index++));
-      rule->SizeOf(aSizeOfHandler, localSize);
-    }
-  }
-}
 #endif
 
 NS_IMETHODIMP
@@ -1013,6 +876,8 @@ CSSMediaRuleImpl::AppendStyleRule(nsICSSRule* aRule)
     aRule->SetStyleSheet(mSheet);
     aRule->SetParentRule(this);
     if (mSheet) {
+      // XXXldb Shouldn't we be using |WillDirty| and |DidDirty| (and
+      // shouldn't |SetModified| be removed?
       mSheet->SetModified(PR_TRUE);
     }
   }
@@ -1093,6 +958,18 @@ CSSMediaRuleImpl::InsertStyleRulesAt(PRUint32 aIndex, nsISupportsArray* aRules)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+CSSMediaRuleImpl::ReplaceStyleRule(nsICSSRule* aOld, nsICSSRule* aNew)
+{
+  PRInt32 index = mRules->IndexOf(aOld);
+  NS_ENSURE_TRUE(index != -1, NS_ERROR_UNEXPECTED);
+  mRules->ReplaceElementAt(aNew, index);
+  aNew->SetStyleSheet(mSheet);
+  aNew->SetParentRule(this);
+  aOld->SetStyleSheet(nsnull);
+  aOld->SetParentRule(nsnull);
+  return NS_OK;
+}
 
 nsresult
 NS_NewCSSMediaRule(nsICSSMediaRule** aInstancePtrResult)
@@ -1145,14 +1022,16 @@ CSSMediaRuleImpl::GetCssText(nsAString& aCssText)
   if (mRules) {
     mRules->Count(&count);
     for (index = 0; index < count; index++) {
-      nsCOMPtr<nsIDOMCSSRule> rule;
-      mRules->QueryElementAt(index, NS_GET_IID(nsIDOMCSSRule), getter_AddRefs(rule));
-      if (rule) {
-        nsAutoString tempString;
-        rule->GetCssText(tempString);
-        aCssText.Append(NS_LITERAL_STRING("  "));
-        aCssText.Append(tempString);
-        aCssText.Append(NS_LITERAL_STRING("\n"));
+      nsCOMPtr<nsICSSRule> rule;
+      mRules->GetElementAt(index, getter_AddRefs(rule));
+      nsCOMPtr<nsIDOMCSSRule> domRule;
+      rule->GetDOMRule(getter_AddRefs(domRule));
+      if (domRule) {
+        nsAutoString cssText;
+        domRule->GetCssText(cssText);
+        aCssText.Append(NS_LITERAL_STRING("  ") +
+                        cssText +
+                        NS_LITERAL_STRING("\n"));
       }
     }
   }
@@ -1182,7 +1061,7 @@ NS_IMETHODIMP
 CSSMediaRuleImpl::GetParentRule(nsIDOMCSSRule** aParentRule)
 {
   if (mParentRule) {
-    return CallQueryInterface(mParentRule, aParentRule);
+    return mParentRule->GetDOMRule(aParentRule);
   }
   *aParentRule = nsnull;
   return NS_OK;
@@ -1269,8 +1148,6 @@ public:
   // nsIStyleRule methods
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out = stdout, PRInt32 aIndent = 0) const;
-
-  virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
 #endif
 
   // nsICSSRule methods
@@ -1348,49 +1225,6 @@ CSSNameSpaceRuleImpl::List(FILE* out, PRInt32 aIndent) const
   fputs(")\n", out);
   return NS_OK;
 }
-
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as CSSNamespaceRuleImpl's size): 
-*    1) sizeof(*this) + the size of the mURLSpec string +
-*       the sizeof the mPrefix atom (if it ieists)
-*
-*  Contained / Aggregated data (not reported as CSSNamespaceRuleImpl's size):
-*    none
-*
-*  Children / siblings / parents:
-*    none
-*    
-******************************************************************************/
-void CSSNameSpaceRuleImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("CSSNameSpaceRuleImpl");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-  
-  // get the member data as part of this dump
-  mURLSpec.SizeOf(aSizeOfHandler, &localSize);
-  aSize += localSize;
-  aSize -= sizeof(mURLSpec); // counted in sizeof(*this) and nsString->SizeOf()
-
-  if(mPrefix && uniqueItems->AddItem(mPrefix)){
-    mPrefix->SizeOf(aSizeOfHandler, &localSize);
-    aSize += localSize;
-  }
-  aSizeOfHandler->AddSize(tag, aSize);
-}
 #endif
 
 NS_IMETHODIMP
@@ -1504,7 +1338,7 @@ NS_IMETHODIMP
 CSSNameSpaceRuleImpl::GetParentRule(nsIDOMCSSRule** aParentRule)
 {
   if (mParentRule) {
-    return CallQueryInterface(mParentRule, aParentRule);
+    return mParentRule->GetDOMRule(aParentRule);
   }
   *aParentRule = nsnull;
   return NS_OK;
