@@ -49,11 +49,14 @@
 #include "nsHTMLTokens.h"
 #include "nsIURI.h"
 #include "nsTextFragment.h"
+#ifdef MOZ_XUL
 #include "nsXULElement.h"
+#endif
 #include "nsXULAtoms.h"
 #include "nsXBLProtoImplProperty.h"
 #include "nsXBLProtoImplMethod.h"
 #include "nsXBLProtoImplField.h"
+#include "nsXBLPrototypeBinding.h"
 #include "nsIConsoleService.h"
 #include "nsIScriptError.h"
 
@@ -61,32 +64,34 @@ nsresult
 NS_NewXBLContentSink(nsIXMLContentSink** aResult,
                      nsIDocument* aDoc,
                      nsIURI* aURL,
-                     nsIWebShell* aWebShell)
+                     nsISupports* aContainer)
 {
-  NS_PRECONDITION(nsnull != aResult, "null ptr");
-  if (!aResult)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_ARG_POINTER(aResult);
+
   nsXBLContentSink* it;
   NS_NEWXPCOM(it, nsXBLContentSink);
-  if (!it)
-    return NS_ERROR_OUT_OF_MEMORY;
-  nsresult rv = it->Init(aDoc, aURL, aWebShell);
-  if (NS_FAILED(rv))
-    return rv;
-  return it->QueryInterface(NS_GET_IID(nsIXMLContentSink), (void **)aResult);
+  NS_ENSURE_TRUE(it, NS_ERROR_OUT_OF_MEMORY);
+
+  nsCOMPtr<nsIXMLContentSink> kungFuDeathGrip = it;
+  nsresult rv = it->Init(aDoc, aURL, aContainer);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return CallQueryInterface(it, aResult);
 }
 
 nsXBLContentSink::nsXBLContentSink()
+  : mState(eXBL_InDocument),
+    mSecondaryState(eXBL_None),
+    mDocInfo(nsnull),
+    mIsChromeOrResource(PR_FALSE),
+    mBinding(nsnull),
+    mHandler(nsnull),
+    mImplementation(nsnull),
+    mImplMember(nsnull),
+    mProperty(nsnull),
+    mMethod(nsnull),
+    mField(nsnull)
 {
-  mState = eXBL_InDocument;
-  mSecondaryState = eXBL_None;
-  mDocInfo = nsnull;
-  mIsChromeOrResource = PR_FALSE;
-  mImplementation = nsnull;
-  mImplMember = nsnull;
-  mProperty = nsnull;
-  mMethod = nsnull;
-  mField = nsnull;
   mPrettyPrintXML = PR_FALSE;
 }
 
@@ -97,7 +102,7 @@ nsXBLContentSink::~nsXBLContentSink()
 nsresult
 nsXBLContentSink::Init(nsIDocument* aDoc,
                        nsIURI* aURL,
-                       nsIWebShell* aContainer)
+                       nsISupports* aContainer)
 {
   nsresult rv;
   rv = nsXMLContentSink::Init(aDoc, aURL, aContainer, nsnull);
@@ -132,11 +137,11 @@ nsXBLContentSink::FlushText(PRBool aCreateTextNode,
       // BindingAttached and BindingDetached, and they're still implemented
       // using handlers.  At some point, we need to change these to just
       // be special functions on the class instead.
-      nsCOMPtr<nsIXBLPrototypeHandler> handler;
+      nsXBLPrototypeHandler* handler;
       if (mSecondaryState == eXBL_InConstructor)
-        mBinding->GetConstructor(getter_AddRefs(handler));
+        handler = mBinding->GetConstructor();
       else
-        mBinding->GetDestructor(getter_AddRefs(handler));
+        handler = mBinding->GetDestructor();
 
       // Get the text and add it to the constructor/destructor.
       handler->AppendHandlerText(text);
@@ -323,7 +328,8 @@ PRBool
 nsXBLContentSink::OnOpenContainer(const PRUnichar **aAtts, 
                                   PRUint32 aAttsCount, 
                                   PRInt32 aNameSpaceID, 
-                                  nsIAtom* aTagName)
+                                  nsIAtom* aTagName,
+                                  PRUint32 aLineNumber)
 {
   PRBool ret = PR_TRUE;
   if (aNameSpaceID == kNameSpaceID_XBL) {
@@ -357,7 +363,7 @@ nsXBLContentSink::OnOpenContainer(const PRUnichar **aAtts,
     }
     else if (aTagName == nsXBLAtoms::handler) {
       mSecondaryState = eXBL_InHandler;
-      ConstructHandler(aAtts);
+      ConstructHandler(aAtts, aLineNumber);
       ret = PR_FALSE;
     }
     else if (aTagName == nsXBLAtoms::resources) {
@@ -377,42 +383,57 @@ nsXBLContentSink::OnOpenContainer(const PRUnichar **aAtts,
     else if (mState == eXBL_InImplementation) {
       if (aTagName == nsXBLAtoms::constructor) {
         mSecondaryState = eXBL_InConstructor;
-        nsCOMPtr<nsIXBLPrototypeHandler> newHandler;
-        NS_NewXBLPrototypeHandler(nsnull, nsnull, nsnull, nsnull, nsnull,
-                                  nsnull, nsnull, nsnull, nsnull, nsnull,
-                                  getter_AddRefs(newHandler));
+        nsXBLPrototypeHandler* newHandler;
+        newHandler = new nsXBLPrototypeHandler(nsnull, nsnull, nsnull, nsnull,
+                                               nsnull, nsnull, nsnull, nsnull,
+                                               nsnull, nsnull);
         newHandler->SetEventName(nsXBLAtoms::constructor);
+        newHandler->SetLineNumber(aLineNumber);
         mBinding->SetConstructor(newHandler);
       }
       else if (aTagName == nsXBLAtoms::destructor) {
         mSecondaryState = eXBL_InDestructor;
-        nsCOMPtr<nsIXBLPrototypeHandler> newHandler;
-        NS_NewXBLPrototypeHandler(nsnull, nsnull, nsnull, nsnull, nsnull,
-                                  nsnull, nsnull, nsnull, nsnull, nsnull,
-                                  getter_AddRefs(newHandler));
+        nsXBLPrototypeHandler* newHandler;
+        newHandler = new nsXBLPrototypeHandler(nsnull, nsnull, nsnull, nsnull,
+                                               nsnull, nsnull, nsnull, nsnull,
+                                               nsnull, nsnull);
         newHandler->SetEventName(nsXBLAtoms::destructor);
+        newHandler->SetLineNumber(aLineNumber);
         mBinding->SetDestructor(newHandler);
       }
       else if (aTagName == nsXBLAtoms::field) {
         mSecondaryState = eXBL_InField;
-        ConstructField(aAtts);
+        ConstructField(aAtts, aLineNumber);
       }
       else if (aTagName == nsXBLAtoms::property) {
         mSecondaryState = eXBL_InProperty;
         ConstructProperty(aAtts);
       }
-      else if (aTagName == nsXBLAtoms::getter)
+      else if (aTagName == nsXBLAtoms::getter) {
+        if (mSecondaryState == eXBL_InProperty && mProperty) {
+          mProperty->SetGetterLineNumber(aLineNumber);
+        }        
         mSecondaryState = eXBL_InGetter;
-      else if (aTagName == nsXBLAtoms::setter)
+      }
+      else if (aTagName == nsXBLAtoms::setter) {
+        if (mSecondaryState == eXBL_InProperty && mProperty) {
+          mProperty->SetSetterLineNumber(aLineNumber);
+        }
         mSecondaryState = eXBL_InSetter;
+      }
       else if (aTagName == nsXBLAtoms::method) {
         mSecondaryState = eXBL_InMethod;
         ConstructMethod(aAtts);
       }
       else if (aTagName == nsXBLAtoms::parameter)
         ConstructParameter(aAtts);
-      else if (aTagName == nsXBLAtoms::body)
+      else if (aTagName == nsXBLAtoms::body) {
+        if (mSecondaryState == eXBL_InMethod && mMethod) {
+          // stash away the line number
+          mMethod->SetLineNumber(aLineNumber);
+        }
         mSecondaryState = eXBL_InBody;
+      }
 
       ret = PR_FALSE; // Ignore everything we encounter inside an <implementation> block.
     }
@@ -424,13 +445,13 @@ nsXBLContentSink::OnOpenContainer(const PRUnichar **aAtts,
 void 
 nsXBLContentSink::ConstructBinding()
 {
-  nsCOMPtr<nsIContent> binding = getter_AddRefs(GetCurrentContent());
+  nsCOMPtr<nsIContent> binding = GetCurrentContent();
   nsAutoString id;
   binding->GetAttr(kNameSpaceID_None, nsHTMLAtoms::id, id);
   nsCAutoString cid; cid.AssignWithConversion(id);
 
   if (!cid.IsEmpty()) {
-    NS_NewXBLPrototypeBinding(cid, binding, mDocInfo, getter_AddRefs(mBinding));
+    mBinding = new nsXBLPrototypeBinding(cid, mDocInfo, binding);
     mDocInfo->SetPrototypeBinding(cid, mBinding);
     binding->UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::id, PR_FALSE);
   }
@@ -438,7 +459,7 @@ nsXBLContentSink::ConstructBinding()
 
 
 void
-nsXBLContentSink::ConstructHandler(const PRUnichar **aAtts)
+nsXBLContentSink::ConstructHandler(const PRUnichar **aAtts, PRUint32 aLineNumber)
 {
   nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
 
@@ -496,12 +517,14 @@ nsXBLContentSink::ConstructHandler(const PRUnichar **aAtts)
 
   // All of our pointers are now filled in.  Construct our handler with all of these
   // parameters.
-  nsCOMPtr<nsIXBLPrototypeHandler> newHandler;
-  NS_NewXBLPrototypeHandler(event, phase, action, command, 
-                            keycode, charcode, modifiers, button,
-                            clickcount, preventdefault,
-                            getter_AddRefs(newHandler));
+  nsXBLPrototypeHandler* newHandler;
+  newHandler = new nsXBLPrototypeHandler(event, phase, action, command,
+                                         keycode, charcode, modifiers, button,
+                                         clickcount, preventdefault);
+
   if (newHandler) {
+    newHandler->SetLineNumber(aLineNumber);
+    
     // Add this handler to our chain of handlers.
     if (mHandler)
       mHandler->SetNextHandler(newHandler); // Already have a chain. Just append to the end.
@@ -574,7 +597,7 @@ nsXBLContentSink::ConstructImplementation(const PRUnichar **aAtts)
 }
 
 void
-nsXBLContentSink::ConstructField(const PRUnichar **aAtts)
+nsXBLContentSink::ConstructField(const PRUnichar **aAtts, PRUint32 aLineNumber)
 {
   nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
 
@@ -602,6 +625,8 @@ nsXBLContentSink::ConstructField(const PRUnichar **aAtts)
   // parameters.
   mField = new nsXBLProtoImplField(name, readonly);
   if (mField) {
+    mField->SetLineNumber(aLineNumber);
+    
     // Add this member to our chain.
     if (mImplMember)
       mImplMember->SetNext(mField); // Already have a chain. Just append to the end.
@@ -724,10 +749,13 @@ nsXBLContentSink::CreateElement(const PRUnichar** aAtts, PRUint32 aAttsCount,
                                 nsINodeInfo* aNodeInfo, PRUint32 aLineNumber,
                                 nsIContent** aResult, PRBool* aAppendContent)
 {
+#ifdef MOZ_XUL
   if (!aNodeInfo->NamespaceEquals(kNameSpaceID_XUL)) {
+#endif
     return nsXMLContentSink::CreateElement(aAtts, aAttsCount, aNodeInfo,
                                            aLineNumber, aResult,
                                            aAppendContent);
+#ifdef MOZ_XUL
   }
 
   *aAppendContent = PR_TRUE;
@@ -747,6 +775,7 @@ nsXBLContentSink::CreateElement(const PRUnichar** aAtts, PRUint32 aAttsCount,
 
   // Following this function call, the prototype's ref count will be 1.
   return nsXULElement::Create(prototype, mDocument, PR_FALSE, aResult);
+#endif
 }
 
 nsresult 
@@ -759,6 +788,7 @@ nsXBLContentSink::AddAttributes(const PRUnichar** aAtts,
   return nsXMLContentSink::AddAttributes(aAtts, aContent);
 }
 
+#ifdef MOZ_XUL
 nsresult
 nsXBLContentSink::AddAttributesToXULPrototype(const PRUnichar **aAtts, 
                                               PRUint32 aAttsCount, 
@@ -800,12 +830,12 @@ nsXBLContentSink::AddAttributesToXULPrototype(const PRUnichar **aAtts,
 
     if (kNameSpaceID_Unknown == nameSpaceID) {
       nameSpaceID = kNameSpaceID_None;
-      nameAtom = dont_AddRef(NS_NewAtom(key));
+      nameAtom = do_GetAtom(key);
       nameSpacePrefix = nsnull;
     } 
 
     mNodeInfoManager->GetNodeInfo(nameAtom, nameSpacePrefix, nameSpaceID,
-                                  *getter_AddRefs(attrs->mNodeInfo));
+                                  getter_AddRefs(attrs->mNodeInfo));
     
     attrs->mValue.SetValue(nsDependentString(aAtts[1]));
     ++attrs;
@@ -831,12 +861,8 @@ nsXBLContentSink::AddAttributesToXULPrototype(const PRUnichar **aAtts,
 
     if (rv == NS_CONTENT_ATTR_HAS_VALUE) {
       if (!mCSSParser) {
-          rv = nsComponentManager::CreateInstance(kCSSParserCID,
-                                                  nsnull,
-                                                  NS_GET_IID(nsICSSParser),
-                                                  getter_AddRefs(mCSSParser));
-
-          if (NS_FAILED(rv)) return rv;
+          mCSSParser = do_CreateInstance(kCSSParserCID, &rv);
+          NS_ENSURE_SUCCESS(rv, rv);
       }
 
       rv = mCSSParser->ParseStyleAttribute(value, mDocumentURL,
@@ -849,3 +875,4 @@ nsXBLContentSink::AddAttributesToXULPrototype(const PRUnichar **aAtts,
 
   return NS_OK;
 }
+#endif
