@@ -50,9 +50,6 @@
 
 class nsIXBLBinding;
 class nsIXBLDocumentInfo;
-class nsIXBLPrototypeHandler;
-class nsIXBLPrototypeBinding;
-class nsINameSpaceManager;
 class nsIContent;
 class nsIDocument;
 class nsIAtom;
@@ -61,9 +58,10 @@ class nsIURI;
 class nsSupportsHashtable;
 class nsHashtable;
 class nsIXULPrototypeCache;
-class nsIXULContentUtils;
 
-class nsXBLService : public nsIXBLService, public nsIObserver, public nsSupportsWeakReference
+class nsXBLService : public nsIXBLService,
+                     public nsIObserver,
+                     public nsSupportsWeakReference
 {
   NS_DECL_ISUPPORTS
 
@@ -72,20 +70,12 @@ class nsXBLService : public nsIXBLService, public nsIObserver, public nsSupports
   NS_IMETHOD LoadBindings(nsIContent* aContent, const nsAString& aURL, PRBool aAugmentFlag,
                           nsIXBLBinding** aBinding, PRBool* aResolveStyle);
 
-  // This method loads a binding doc and then builds the specific binding required.
-  NS_IMETHOD GetBinding(nsIContent* aBoundElement, const nsCString& aURLStr, nsIXBLBinding** aResult);
-
   // Indicates whether or not a binding is fully loaded.
   NS_IMETHOD BindingReady(nsIContent* aBoundElement, const nsCString& aURLStr, PRBool* aIsReady);
 
-  // This function clears out the bindings on a given content node.
-  NS_IMETHOD FlushStyleBindings(nsIContent* aContent);
-
   // Gets the object's base class type.
   NS_IMETHOD ResolveTag(nsIContent* aContent, PRInt32* aNameSpaceID, nsIAtom** aResult);
 
-  NS_IMETHOD GetXBLDocumentInfo(const nsCString& aURLStr, nsIContent* aBoundElement, nsIXBLDocumentInfo** aResult);
-
   // This method checks the hashtable and then calls FetchBindingDocument on a miss.
   NS_IMETHOD LoadBindingDocumentInfo(nsIContent* aBoundElement, nsIDocument* aBoundDocument,
                                      const nsCString& aURI, const nsCString& aRef,
@@ -95,22 +85,27 @@ class nsXBLService : public nsIXBLService, public nsIObserver, public nsSupports
   NS_IMETHOD AttachGlobalKeyHandler(nsIDOMEventReceiver* aElement);
   NS_IMETHOD AttachGlobalDragHandler(nsIDOMEventReceiver* aElement);
 
-  // Helper method for loading an XML doc.
-  NS_IMETHOD FetchSyncXMLDocument(nsIURI* aURI, nsIDocument** aResult);
-  
   NS_DECL_NSIOBSERVER
 
 public:
   nsXBLService();
   virtual ~nsXBLService();
 
+  // This function clears out the bindings on a given content node.
+  nsresult FlushStyleBindings(nsIContent* aContent);
+
+  // This method loads a binding doc and then builds the specific binding required.
+  nsresult GetBinding(nsIContent* aBoundElement, const nsCString& aURLStr, nsIXBLBinding** aResult);
+
   // Release any memory that we can
   nsresult FlushMemory();
   
   // This method synchronously loads and parses an XBL file.
-  NS_IMETHOD FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoundDocument,
-                                  nsIURI* aURI, const nsCString& aRef, 
-                                  PRBool aForceSyncLoad, nsIDocument** aResult);
+  nsresult FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoundDocument,
+                                nsIURI* aURI, const nsCString& aRef, 
+                                PRBool aForceSyncLoad, nsIDocument** aResult);
+
+  nsresult GetXBLDocumentInfo(const nsCString& aURLStr, nsIContent* aBoundElement, nsIXBLDocumentInfo** aResult);
 
   // This method loads a binding doc and then builds the specific binding required.  It
   // can also peek without building.
@@ -119,11 +114,10 @@ public:
 
 // MEMBER VARIABLES
 public:
-  static nsIXULContentUtils* gXULUtils;
+#ifdef MOZ_XUL
   static nsIXULPrototypeCache* gXULCache;
+#endif
     
-  static nsINameSpaceManager* gNameSpaceManager; // Used to register the XBL namespace
-  
   static PRUint32 gRefCnt;                   // A count of XBLservice instances.
 
   static PRBool gDisableChromeCache;
@@ -134,10 +128,6 @@ public:
   static PRUint32 gClassLRUListLength;       // Number of classes on LRU list.
   static PRUint32 gClassLRUListQuota;        // Quota on class LRU list.
 
-  // XBL Atoms
-  static nsIAtom* kEventAtom;
-  static nsIAtom* kInputAtom;
-
   nsFixedSizeAllocator mPool;
 };
 
