@@ -14,7 +14,7 @@
  *
  * The Original Code is Mozilla Communicator client code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -82,8 +82,6 @@
 
 #include "nsGenericElement.h" // for nsCheapVoidArray
 
-class nsISizeOfHandler;
-
 class nsIDocument;
 class nsIRDFService;
 class nsISupportsArray;
@@ -93,7 +91,7 @@ class nsRDFDOMNodeList;
 class nsString;
 class nsXULAttributes;
 class nsVoidArray;
-class nsIWebShell;
+class nsIDocShell;
 
 class nsIObjectInputStream;
 class nsIObjectOutputStream;
@@ -197,7 +195,7 @@ public:
     Type                     mType;
 
     PRInt32                  mRefCnt;
-          
+
     virtual ~nsXULPrototypeNode() {}
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
                                nsIScriptContext* aContext,
@@ -208,11 +206,11 @@ public:
                                  nsISupportsArray* aNodeInfos) = 0;
 
     void AddRef() { ++mRefCnt; };
-    void Release() 
-    { 
-        --mRefCnt; 
-        if (mRefCnt == 0) 
-            delete this; 
+    void Release()
+    {
+        --mRefCnt;
+        if (mRefCnt == 0)
+            delete this;
     };
     virtual void ReleaseSubtree() { Release(); };
 
@@ -244,7 +242,7 @@ public:
         delete[] mChildren;
     }
 
-    virtual void ReleaseSubtree() 
+    virtual void ReleaseSubtree()
     {
       if (mChildren) {
         for (PRInt32 i = mNumChildren-1; i >= 0; i--) {
@@ -279,8 +277,8 @@ public:
     nsresult GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsAString& aValue);
 
 
-    static void ReleaseGlobals() 
-    { 
+    static void ReleaseGlobals()
+    {
         NS_IF_RELEASE(sCSSParser);
     }
 
@@ -307,12 +305,14 @@ public:
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
                                nsIScriptContext* aContext,
                                nsISupportsArray* aNodeInfos);
+    nsresult SerializeOutOfLine(nsIObjectOutputStream* aStream,
+                                nsIScriptContext* aContext);
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
                                  nsIScriptContext* aContext,
                                  nsIURI* aDocumentURI,
                                  nsISupportsArray* aNodeInfos);
-    virtual nsresult DeserializeOutOfLineScript(nsIObjectInputStream* aInput,
-                                                nsIScriptContext* aContext);
+    nsresult DeserializeOutOfLine(nsIObjectInputStream* aInput,
+                                  nsIScriptContext* aContext);
 
     nsresult Compile(const PRUnichar* aText, PRInt32 aTextLength,
                      nsIURI* aURI, PRUint16 aLineNo,
@@ -327,9 +327,9 @@ public:
     JSObject*                mJSObject;
     const char*              mLangVersion;
 
-    static void ReleaseGlobals() 
-    { 
-        NS_IF_RELEASE(sXULPrototypeCache); 
+    static void ReleaseGlobals()
+    {
+        NS_IF_RELEASE(sXULPrototypeCache);
     }
 
 protected:
@@ -404,17 +404,17 @@ public:
 
     // nsISupports
     NS_DECL_ISUPPORTS
-       
+
     // nsIContent (from nsIStyledContent)
-    NS_IMETHOD GetDocument(nsIDocument*& aResult) const;
+    NS_IMETHOD GetDocument(nsIDocument** aResult) const;
     NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep, PRBool aCompileEventHandlers);
-    NS_IMETHOD GetParent(nsIContent*& aResult) const;
+    NS_IMETHOD GetParent(nsIContent** aResult) const;
     NS_IMETHOD SetParent(nsIContent* aParent);
     NS_IMETHOD_(PRBool) IsNativeAnonymous() const;
     NS_IMETHOD_(void) SetNativeAnonymous(PRBool aAnonymous);
     NS_IMETHOD CanContainChildren(PRBool& aResult) const;
     NS_IMETHOD ChildCount(PRInt32& aResult) const;
-    NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent*& aResult) const;
+    NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent** aResult) const;
     NS_IMETHOD IndexOf(nsIContent* aPossibleChild, PRInt32& aResult) const;
     NS_IMETHOD InsertChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify,
                              PRBool aDeepSetDocument);
@@ -423,23 +423,22 @@ public:
     NS_IMETHOD AppendChildTo(nsIContent* aKid, PRBool aNotify,
                              PRBool aDeepSetDocument);
     NS_IMETHOD RemoveChildAt(PRInt32 aIndex, PRBool aNotify);
-    NS_IMETHOD GetNameSpaceID(PRInt32& aNameSpeceID) const;
-    NS_IMETHOD GetTag(nsIAtom*& aResult) const;
-    NS_IMETHOD GetNodeInfo(nsINodeInfo*& aResult) const;
-    NS_IMETHOD NormalizeAttrString(const nsAString& aStr, nsINodeInfo*& aNodeInfo);
+    NS_IMETHOD GetNameSpaceID(PRInt32* aNameSpeceID) const;
+    NS_IMETHOD GetTag(nsIAtom** aResult) const;
+    NS_IMETHOD GetNodeInfo(nsINodeInfo** aResult) const;
+    NS_IMETHOD NormalizeAttrString(const nsAString& aStr, nsINodeInfo** aNodeInfo);
     NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, const nsAString& aValue, PRBool aNotify);
     NS_IMETHOD SetAttr(nsINodeInfo *aNodeInfo, const nsAString& aValue, PRBool aNotify);
     NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsAString& aResult) const;
-    NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsIAtom*& aPrefix, nsAString& aResult) const;
+    NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsIAtom** aPrefix, nsAString& aResult) const;
     NS_IMETHOD_(PRBool) HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const;
     NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, PRBool aNotify);
-    NS_IMETHOD GetAttrNameAt(PRInt32 aIndex, PRInt32& aNameSpaceID, 
-                             nsIAtom*& aName, nsIAtom*& aPrefix) const;
+    NS_IMETHOD GetAttrNameAt(PRInt32 aIndex, PRInt32* aNameSpaceID,
+                             nsIAtom** aName, nsIAtom** aPrefix) const;
     NS_IMETHOD GetAttrCount(PRInt32& aResult) const;
 #ifdef DEBUG
     NS_IMETHOD List(FILE* out, PRInt32 aIndent) const;
     NS_IMETHOD DumpContent(FILE* out, PRInt32 aIndent,PRBool aDumpAll) const { return NS_OK; }
-    NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
 #endif
     NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext,
                               nsEvent* aEvent,
@@ -452,22 +451,22 @@ public:
     NS_IMETHOD SetContentID(PRUint32 aID);
 
     NS_IMETHOD RangeAdd(nsIDOMRange* aRange);
-    NS_IMETHOD RangeRemove(nsIDOMRange* aRange); 
-    NS_IMETHOD GetRangeList(nsVoidArray*& aResult) const;
+    NS_IMETHOD RangeRemove(nsIDOMRange* aRange);
+    NS_IMETHOD GetRangeList(nsVoidArray** aResult) const;
     NS_IMETHOD SetFocus(nsIPresContext* aPresContext);
     NS_IMETHOD RemoveFocus(nsIPresContext* aPresContext);
 
     NS_IMETHOD GetBindingParent(nsIContent** aContent);
     NS_IMETHOD SetBindingParent(nsIContent* aParent);
     NS_IMETHOD_(PRBool) IsContentOfType(PRUint32 aFlags);
+    NS_IMETHOD GetBaseURL(nsIURI** aURI) const;
     NS_IMETHOD GetListenerManager(nsIEventListenerManager** aResult);
 
     // nsIXMLContent
-    NS_IMETHOD MaybeTriggerAutoLink(nsIWebShell *aShell);
-    NS_IMETHOD GetXMLBaseURI(nsIURI **aURI);
+    NS_IMETHOD MaybeTriggerAutoLink(nsIDocShell *aShell);
 
     // nsIStyledContent
-    NS_IMETHOD GetID(nsIAtom*& aResult) const;
+    NS_IMETHOD GetID(nsIAtom** aResult) const;
     NS_IMETHOD GetClasses(nsVoidArray& aArray) const;
     NS_IMETHOD_(PRBool) HasClass(nsIAtom* aClass, PRBool aCaseSensitive) const;
 
@@ -483,10 +482,10 @@ public:
     NS_IMETHOD ClearLazyState(LazyState aFlags);
     NS_IMETHOD GetLazyState(LazyState aFlag, PRBool& aValue);
     NS_IMETHOD AddScriptEventListener(nsIAtom* aName, const nsAString& aValue);
-    
+
     // nsIDOMNode (from nsIDOMElement)
     NS_DECL_NSIDOMNODE
-  
+
     // nsIDOMElement
     NS_DECL_NSIDOMELEMENT
 
@@ -501,10 +500,6 @@ public:
                                    void** aHandler);
     NS_IMETHOD GetCompiledEventHandler(nsIAtom *aName, void** aHandler);
 
-#ifdef DEBUG
-    virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
-#endif
-    
     // nsIChromeEventHandler
     NS_DECL_NSICHROMEEVENTHANDLER
 
@@ -524,11 +519,6 @@ protected:
     ExecuteJSCode(nsIDOMElement* anElement, nsEvent* aEvent);
 
     // Static helpers
-    static nsresult
-    GetElementsByTagName(nsIDOMNode* aNode,
-                         const nsAString& aTagName,
-                         nsRDFDOMNodeList* aElements);
-
     static nsresult
     GetElementsByAttribute(nsIDOMNode* aNode,
                            const nsAString& aAttributeName,
@@ -653,7 +643,7 @@ protected:
     nsresult AddListenerFor(nsINodeInfo *aNodeInfo,
                             PRBool aCompileEventHandlers);
 
-    
+
     nsresult HideWindowChrome(PRBool aShouldHide);
 
 protected:
