@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; c-file-style: "stroustrup" -*-
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2; c-file-style: "stroustrup" -*-
  *
  * The contents of this file are subject to the Netscape Public
  * License Version 1.1 (the "License"); you may not use this file
@@ -79,6 +79,7 @@
 #include "nsIRDFContainerUtils.h"
 #include "nsXULAtoms.h"
 
+
 ////////////////////////////////////////////////////////////////////////
 
 static NS_DEFINE_CID(kRDFServiceCID,          NS_RDFSERVICE_CID);
@@ -144,14 +145,6 @@ protected:
 
 private:
   static nsrefcnt gRefCnt;
-  static nsIAtom *kStaticHintAtom;
-  static nsIAtom *kStaticsSortLastHintAtom;
-  static nsIAtom *kResourceAtom;
-  static nsIAtom *kSortResourceAtom;
-  static nsIAtom *kSortResource2Atom;
-  static nsIAtom *kSortSeparatorsAtom;
-  static nsIAtom *kRDF_type;
-
   static nsString *kTrueStr;
   static nsString *kNaturalStr;
   static nsString *kAscendingStr;
@@ -195,14 +188,6 @@ nsIRDFService *XULSortServiceImpl::gRDFService = nsnull;
 nsIRDFContainerUtils *XULSortServiceImpl::gRDFC = nsnull;
 nsrefcnt XULSortServiceImpl::gRefCnt = 0;
 
-nsIAtom* XULSortServiceImpl::kStaticHintAtom;
-nsIAtom* XULSortServiceImpl::kStaticsSortLastHintAtom;
-nsIAtom* XULSortServiceImpl::kResourceAtom;
-nsIAtom* XULSortServiceImpl::kSortResourceAtom;
-nsIAtom* XULSortServiceImpl::kSortResource2Atom;
-nsIAtom* XULSortServiceImpl::kSortSeparatorsAtom;
-nsIAtom* XULSortServiceImpl::kRDF_type;
-
 nsString* XULSortServiceImpl::kTrueStr = nsnull;
 nsString* XULSortServiceImpl::kNaturalStr = nsnull;
 nsString* XULSortServiceImpl::kAscendingStr = nsnull;
@@ -212,32 +197,19 @@ nsString* XULSortServiceImpl::kDescendingStr = nsnull;
 
 XULSortServiceImpl::XULSortServiceImpl(void)
 {
-  NS_INIT_ISUPPORTS();
   if (gRefCnt == 0) {
-    kStaticHintAtom = NS_NewAtom("staticHint");
-    kStaticsSortLastHintAtom   = NS_NewAtom("sortStaticsLast");
-    kResourceAtom = NS_NewAtom("resource");
-    kSortResourceAtom = NS_NewAtom("sortResource");
-    kSortResource2Atom = NS_NewAtom("sortResource2");
-    kSortSeparatorsAtom = NS_NewAtom("sortSeparators");
-    kRDF_type = NS_NewAtom("type");
- 
     kTrueStr = new nsString(NS_LITERAL_STRING("true"));
     kNaturalStr = new nsString(NS_LITERAL_STRING("natural"));
     kAscendingStr = new nsString(NS_LITERAL_STRING("ascending"));
     kDescendingStr = new nsString(NS_LITERAL_STRING("descending"));
  
-    nsresult rv = nsServiceManager::GetService(kRDFServiceCID,
-                                               NS_GET_IID(nsIRDFService),
-                                               (nsISupports**) &gRDFService);
-    if (NS_FAILED(rv))
-      NS_ERROR("couldn't create rdf service");
+    nsresult rv = CallGetService(kRDFServiceCID, &gRDFService);
 
-    rv = nsServiceManager::GetService(kRDFContainerUtilsCID,
-                                      NS_GET_IID(nsIRDFContainerUtils),
-                                      (nsISupports**) &gRDFC);
-    if (NS_FAILED(rv))
-      NS_ERROR("couldn't create rdf container utils");
+    NS_ASSERTION(NS_SUCCEEDED(rv), "couldn't create rdf service");
+
+    rv = CallGetService(kRDFContainerUtilsCID, &gRDFC);
+
+    NS_ASSERTION(NS_SUCCEEDED(rv), "couldn't create rdf container utils");
 
     // get a locale service 
     nsCOMPtr<nsILocaleService> localeService = do_GetService(NS_LOCALESERVICE_CONTRACTID, &rv);
@@ -246,8 +218,9 @@ XULSortServiceImpl::XULSortServiceImpl(void)
       if (NS_SUCCEEDED(rv = localeService->GetApplicationLocale(getter_AddRefs(locale))) && (locale)) {
         nsCOMPtr<nsICollationFactory> colFactory = do_CreateInstance(kCollationFactoryCID);
         if (colFactory) {
-          if (NS_FAILED(rv = colFactory->CreateCollation(locale, &gCollation)))
-            NS_ERROR("couldn't create collation instance");
+          rv = colFactory->CreateCollation(locale, &gCollation);
+
+          NS_ASSERTION(NS_SUCCEEDED(rv), "couldn't create collation instance");
         } else
           NS_ERROR("couldn't create instance of collation factory");
       } else
@@ -276,22 +249,10 @@ XULSortServiceImpl::~XULSortServiceImpl(void) {
     delete kNaturalStr;
     kNaturalStr = nsnull;
 
-    NS_IF_RELEASE(kStaticHintAtom);
-    NS_IF_RELEASE(kStaticsSortLastHintAtom);
-    NS_IF_RELEASE(kResourceAtom);
-    NS_IF_RELEASE(kSortResourceAtom);
-    NS_IF_RELEASE(kSortResource2Atom);
-    NS_IF_RELEASE(kSortSeparatorsAtom);
-    NS_IF_RELEASE(kRDF_type);
-
     NS_IF_RELEASE(gCollation);
 
-    if (gRDFService) {
-      nsServiceManager::ReleaseService(kRDFServiceCID, gRDFService);
-      gRDFService = nsnull;
-    }
-    if (gRDFC)
-      nsServiceManager::ReleaseService(kRDFContainerUtilsCID, gRDFC);
+    NS_IF_RELEASE(gRDFService);
+    NS_IF_RELEASE(gRDFC);
   }
 }
 
@@ -317,7 +278,7 @@ XULSortServiceImpl::FindDatabaseElement(nsIContent *aElement, nsIContent **aData
     }
     
     nsCOMPtr<nsIContent> parent;
-    content->GetParent(*getter_AddRefs(parent));
+    content->GetParent(getter_AddRefs(parent));
     content = parent;
   }
 
@@ -330,7 +291,7 @@ XULSortServiceImpl::FindSortableContainer(nsIContent *aRoot, nsIContent **aConta
   nsresult rv;
 
   nsCOMPtr<nsIAtom> tag;
-  if (NS_FAILED(rv = aRoot->GetTag(*getter_AddRefs(tag))))  return rv;
+  if (NS_FAILED(rv = aRoot->GetTag(getter_AddRefs(tag))))  return rv;
 
   if (tag == nsXULAtoms::templateAtom) // ignore content within templates
     return NS_OK;    
@@ -347,8 +308,8 @@ XULSortServiceImpl::FindSortableContainer(nsIContent *aRoot, nsIContent **aConta
 
   if (NS_FAILED(rv = aRoot->ChildCount(numChildren))) return rv;
   for (childIndex = 0; childIndex < numChildren; childIndex++) {
-    if (NS_FAILED(rv = aRoot->ChildAt(childIndex, *getter_AddRefs(child)))) return rv;
-    if (NS_FAILED(rv = child->GetNameSpaceID(nameSpaceID)))  return rv;
+    if (NS_FAILED(rv = aRoot->ChildAt(childIndex, getter_AddRefs(child)))) return rv;
+    if (NS_FAILED(rv = child->GetNameSpaceID(&nameSpaceID)))  return rv;
     if (nameSpaceID == kNameSpaceID_XUL) {
       rv = FindSortableContainer(child, aContainer);
       if (*aContainer)
@@ -368,24 +329,24 @@ XULSortServiceImpl::SetSortHints(nsIContent *tree, const nsAString &sortResource
     // set hints on tree root node
     tree->SetAttr(kNameSpaceID_None, nsXULAtoms::sortActive, *kTrueStr, PR_FALSE);
     tree->SetAttr(kNameSpaceID_None, nsXULAtoms::sortDirection, sortDirection, PR_FALSE);
-    tree->SetAttr(kNameSpaceID_None, kSortResourceAtom, sortResource, PR_FALSE);
+    tree->SetAttr(kNameSpaceID_None, nsXULAtoms::sortResource, sortResource, PR_FALSE);
 
     if (!sortResource2.IsEmpty())
-      tree->SetAttr(kNameSpaceID_None, kSortResource2Atom, sortResource2, PR_FALSE);
+      tree->SetAttr(kNameSpaceID_None, nsXULAtoms::sortResource2, sortResource2, PR_FALSE);
     else
-      tree->UnsetAttr(kNameSpaceID_None, kSortResource2Atom, PR_FALSE);
+      tree->UnsetAttr(kNameSpaceID_None, nsXULAtoms::sortResource2, PR_FALSE);
   } else {
     tree->UnsetAttr(kNameSpaceID_None, nsXULAtoms::sortActive, PR_FALSE);
     tree->UnsetAttr(kNameSpaceID_None, nsXULAtoms::sortDirection, PR_FALSE);
-    tree->UnsetAttr(kNameSpaceID_None, kSortResourceAtom, PR_FALSE);
-    tree->UnsetAttr(kNameSpaceID_None, kSortResource2Atom, PR_FALSE);
+    tree->UnsetAttr(kNameSpaceID_None, nsXULAtoms::sortResource, PR_FALSE);
+    tree->UnsetAttr(kNameSpaceID_None, nsXULAtoms::sortResource2, PR_FALSE);
   }
 
   // optional hint
   if (inbetweenSeparatorSort == PR_TRUE)
-    tree->SetAttr(kNameSpaceID_None, kSortSeparatorsAtom, *kTrueStr, PR_FALSE);
+    tree->SetAttr(kNameSpaceID_None, nsXULAtoms::sortSeparators, *kTrueStr, PR_FALSE);
   else
-    tree->UnsetAttr(kNameSpaceID_None, kSortSeparatorsAtom, PR_FALSE);
+    tree->UnsetAttr(kNameSpaceID_None, nsXULAtoms::sortSeparators, PR_FALSE);
 
   SetSortColumnHints(tree, sortResource, sortDirection);
 
@@ -402,12 +363,12 @@ XULSortServiceImpl::SetSortColumnHints(nsIContent *content, const nsAString &sor
   if (NS_FAILED(rv = content->ChildCount(numChildren))) return rv;
   
   for (childIndex = 0; childIndex < numChildren; ++childIndex) {
-    if (NS_FAILED(rv = content->ChildAt(childIndex, *getter_AddRefs(child)))) return rv;
-    if (NS_FAILED(rv = child->GetNameSpaceID(nameSpaceID))) return rv;
+    if (NS_FAILED(rv = content->ChildAt(childIndex, getter_AddRefs(child)))) return rv;
+    if (NS_FAILED(rv = child->GetNameSpaceID(&nameSpaceID))) return rv;
     
     if (nameSpaceID == kNameSpaceID_XUL) {
       nsCOMPtr<nsIAtom> tag;
-      if (NS_FAILED(rv = child->GetTag(*getter_AddRefs(tag)))) return rv;
+      if (NS_FAILED(rv = child->GetTag(getter_AddRefs(tag)))) return rv;
         
       if (tag == nsXULAtoms::treecols || tag == nsXULAtoms::listcols || tag == nsXULAtoms::listhead) {
         rv = SetSortColumnHints(child, sortResource, sortDirection);
@@ -415,7 +376,7 @@ XULSortServiceImpl::SetSortColumnHints(nsIContent *content, const nsAString &sor
                  tag == nsXULAtoms::listcol || tag == nsXULAtoms::listheader) {
         nsAutoString value;
 
-        if (NS_SUCCEEDED(rv = child->GetAttr(kNameSpaceID_None, kResourceAtom, value))
+        if (NS_SUCCEEDED(rv = child->GetAttr(kNameSpaceID_None, nsXULAtoms::resource, value))
             && rv == NS_CONTENT_ATTR_HAS_VALUE)
         {
           if (value == sortResource) {
@@ -448,7 +409,7 @@ XULSortServiceImpl::GetSortColumnInfo(nsIContent *tree, nsAString &sortResource,
   {
     if (value.EqualsIgnoreCase("true"))
     {
-      if (NS_SUCCEEDED(rv = tree->GetAttr(kNameSpaceID_None, kSortResourceAtom, sortResource))
+      if (NS_SUCCEEDED(rv = tree->GetAttr(kNameSpaceID_None, nsXULAtoms::sortResource, sortResource))
           && (rv == NS_CONTENT_ATTR_HAS_VALUE))
       {
         if (NS_SUCCEEDED(rv = tree->GetAttr(kNameSpaceID_None, nsXULAtoms::sortDirection, sortDirection))
@@ -457,7 +418,7 @@ XULSortServiceImpl::GetSortColumnInfo(nsIContent *tree, nsAString &sortResource,
           rv = NS_OK;
 
           // sort separator flag is optional
-          if (NS_SUCCEEDED(rv = tree->GetAttr(kNameSpaceID_None, kSortSeparatorsAtom,
+          if (NS_SUCCEEDED(rv = tree->GetAttr(kNameSpaceID_None, nsXULAtoms::sortSeparators,
             value)) && (rv == NS_CONTENT_ATTR_HAS_VALUE))
           {
             if (value.EqualsIgnoreCase("true"))
@@ -467,7 +428,7 @@ XULSortServiceImpl::GetSortColumnInfo(nsIContent *tree, nsAString &sortResource,
           }
 
           // secondary sort info is optional
-          if (NS_FAILED(rv = tree->GetAttr(kNameSpaceID_None, kSortResource2Atom, sortResource2))
+          if (NS_FAILED(rv = tree->GetAttr(kNameSpaceID_None, nsXULAtoms::sortResource2, sortResource2))
               || (rv != NS_CONTENT_ATTR_HAS_VALUE))
           {
             sortResource2.Truncate();
@@ -817,7 +778,7 @@ XULSortServiceImpl::GetNodeValue(nsIContent *node1, sortPtr sortInfo, PRBool fir
       if (NS_SUCCEEDED(rv = node1->GetAttr(kNameSpaceID_None, nsXULAtoms::id, htmlID)) 
           && (rv == NS_CONTENT_ATTR_HAS_VALUE))
       {
-        if (NS_FAILED(rv = gRDFService->GetUnicodeResource(htmlID.get(), getter_AddRefs(res1))))
+        if (NS_FAILED(rv = gRDFService->GetUnicodeResource(htmlID, getter_AddRefs(res1))))
           res1 = nsnull;
       }
     }
@@ -895,7 +856,7 @@ XULSortServiceImpl::GetNodeValue(contentSortInfo *info1, sortPtr sortInfo, PRBoo
       if (NS_SUCCEEDED(rv = node1->GetAttr(kNameSpaceID_None, nsXULAtoms::id, htmlID))
         && (rv == NS_CONTENT_ATTR_HAS_VALUE))
       {
-        if (NS_FAILED(rv = gRDFService->GetUnicodeResource(htmlID.get(), getter_AddRefs(res1))))
+        if (NS_FAILED(rv = gRDFService->GetUnicodeResource(htmlID, getter_AddRefs(res1))))
           res1 = nsnull;
         info1->resource = res1;
       }
@@ -1130,7 +1091,7 @@ XULSortServiceImpl::SortContainer(nsIContent *container, sortPtr sortInfo, PRBoo
   if (numChildren < 1) return NS_OK;
 
   nsCOMPtr<nsIDocument> doc;
-  container->GetDocument(*getter_AddRefs(doc));
+  container->GetDocument(getter_AddRefs(doc));
   if (!doc) return NS_ERROR_UNEXPECTED;
 
   // Note: This is a straight allocation (not a COMPtr) so we
@@ -1146,10 +1107,10 @@ XULSortServiceImpl::SortContainer(nsIContent *container, sortPtr sortInfo, PRBoo
   nsCOMPtr<nsIAtom> tag;
   currentElement = numChildren;
   for (childIndex = numChildren-1; childIndex >= 0; --childIndex) {
-    if (NS_FAILED(rv = container->ChildAt(childIndex, *getter_AddRefs(child)))) continue;
-    if (NS_FAILED(rv = child->GetNameSpaceID(nameSpaceID))) continue;
+    if (NS_FAILED(rv = container->ChildAt(childIndex, getter_AddRefs(child)))) continue;
+    if (NS_FAILED(rv = child->GetNameSpaceID(&nameSpaceID))) continue;
     if (nameSpaceID == kNameSpaceID_XUL) {
-      if (NS_FAILED(rv = child->GetTag(*getter_AddRefs(tag)))) continue;
+      if (NS_FAILED(rv = child->GetTag(getter_AddRefs(tag)))) continue;
       if (tag == nsXULAtoms::listitem || tag == nsXULAtoms::treeitem
           || tag == nsXULAtoms::menu || tag == nsXULAtoms::menuitem) {
         --currentElement;
@@ -1172,7 +1133,7 @@ XULSortServiceImpl::SortContainer(nsIContent *container, sortPtr sortInfo, PRBoo
       nsAutoString  type;
       for (loop=currentElement; loop< currentElement + numElements; ++loop) {
         if (NS_SUCCEEDED(rv = contentSortInfoArray[loop]->content->GetAttr(kNameSpaceID_RDF,
-            kRDF_type, type)) && (rv == NS_CONTENT_ATTR_HAS_VALUE))
+            nsXULAtoms::type, type)) && (rv == NS_CONTENT_ATTR_HAS_VALUE))
         {
           if (type.EqualsWithConversion(kURINC_BookmarkSeparator)) {
             if (loop > startIndex+1) {
@@ -1205,10 +1166,10 @@ XULSortServiceImpl::SortContainer(nsIContent *container, sortPtr sortInfo, PRBoo
 
     for (childIndex=numChildren-1; childIndex >= 0; childIndex--)
     {
-      if (NS_FAILED(rv = container->ChildAt(childIndex, *getter_AddRefs(child)))) continue;
-      if (NS_FAILED(rv = child->GetNameSpaceID(nameSpaceID))) continue;
+      if (NS_FAILED(rv = container->ChildAt(childIndex, getter_AddRefs(child)))) continue;
+      if (NS_FAILED(rv = child->GetNameSpaceID(&nameSpaceID))) continue;
       if (nameSpaceID == kNameSpaceID_XUL) {
-        if (NS_FAILED(rv = child->GetTag(*getter_AddRefs(tag)))) continue;
+        if (NS_FAILED(rv = child->GetTag(getter_AddRefs(tag)))) continue;
         if (tag == nsXULAtoms::listitem || tag == nsXULAtoms::treeitem
             || tag == nsXULAtoms::menu || tag == nsXULAtoms::menuitem) {
           // immediately remove the child node, and ignore any errors
@@ -1239,12 +1200,12 @@ XULSortServiceImpl::SortContainer(nsIContent *container, sortPtr sortInfo, PRBoo
       if (NS_FAILED(rv = parentNode->ChildCount(numChildren))) continue;
 
       for (childIndex=0; childIndex<numChildren; childIndex++) {
-        if (NS_FAILED(rv = parentNode->ChildAt(childIndex, *getter_AddRefs(child))))
+        if (NS_FAILED(rv = parentNode->ChildAt(childIndex, getter_AddRefs(child))))
           continue;
-        if (NS_FAILED(rv = child->GetNameSpaceID(nameSpaceID))) continue;
+        if (NS_FAILED(rv = child->GetNameSpaceID(&nameSpaceID))) continue;
         if (nameSpaceID != kNameSpaceID_XUL) continue;
 
-        if (NS_FAILED(rv = child->GetTag(*getter_AddRefs(tag)))) continue;
+        if (NS_FAILED(rv = child->GetTag(getter_AddRefs(tag)))) continue;
         if (tag != nsXULAtoms::treechildren && tag != nsXULAtoms::menupopup) continue;
 
         sortInfo->parentContainer = parentNode;
@@ -1306,7 +1267,7 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
 
   // look for sorting info on root node
 
-  if (NS_SUCCEEDED(rv = root->GetAttr(kNameSpaceID_None, kSortResourceAtom, sortResource))
+  if (NS_SUCCEEDED(rv = root->GetAttr(kNameSpaceID_None, nsXULAtoms::sortResource, sortResource))
       && (rv == NS_CONTENT_ATTR_HAS_VALUE))
   {
     if (NS_SUCCEEDED(rv = root->GetAttr(kNameSpaceID_None, nsXULAtoms::sortDirection, sortDirection))
@@ -1314,7 +1275,7 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
     {
       sortInfoAvailable = PR_TRUE;
 
-      if (NS_FAILED(rv = root->GetAttr(kNameSpaceID_None, kSortResource2Atom, sortResource2)) 
+      if (NS_FAILED(rv = root->GetAttr(kNameSpaceID_None, nsXULAtoms::sortResource2, sortResource2)) 
           || (rv != NS_CONTENT_ATTR_HAS_VALUE))
       {
         sortResource2.Truncate();
@@ -1342,38 +1303,38 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
       sortState->sortPropertySort = nsnull;
       sortState->sortPropertySort2 = nsnull;
 
-      rv = gRDFService->GetUnicodeResource(sortResource.get(), getter_AddRefs(sortInfo.sortProperty));
+      rv = gRDFService->GetUnicodeResource(sortResource, getter_AddRefs(sortInfo.sortProperty));
       if (NS_FAILED(rv)) return rv;
       sortState->sortResource = sortResource;
       sortState->sortProperty = sortInfo.sortProperty;
       
       nsAutoString resourceUrl = sortResource;
       resourceUrl.Append(NS_LITERAL_STRING("?collation=true"));
-      rv = gRDFService->GetUnicodeResource(resourceUrl.get(), getter_AddRefs(sortInfo.sortPropertyColl));
+      rv = gRDFService->GetUnicodeResource(resourceUrl, getter_AddRefs(sortInfo.sortPropertyColl));
       if (NS_FAILED(rv)) return rv;
       sortState->sortPropertyColl = sortInfo.sortPropertyColl;
 
       resourceUrl = sortResource;
       resourceUrl.Append(NS_LITERAL_STRING("?sort=true"));
-      rv = gRDFService->GetUnicodeResource(resourceUrl.get(), getter_AddRefs(sortInfo.sortPropertySort));
+      rv = gRDFService->GetUnicodeResource(resourceUrl, getter_AddRefs(sortInfo.sortPropertySort));
       if (NS_FAILED(rv)) return rv;
       sortState->sortPropertySort = sortInfo.sortPropertySort;
 
       if (!sortResource2.IsEmpty()) {
-        rv = gRDFService->GetUnicodeResource(sortResource2.get(), getter_AddRefs(sortInfo.sortProperty2));
+        rv = gRDFService->GetUnicodeResource(sortResource2, getter_AddRefs(sortInfo.sortProperty2));
         if (NS_FAILED(rv)) return rv;
         sortState->sortResource2 = sortResource2;
         sortState->sortProperty2 = sortInfo.sortProperty2;
 
         resourceUrl = sortResource2;
         resourceUrl.Append(NS_LITERAL_STRING("?collation=true"));
-        rv = gRDFService->GetUnicodeResource(resourceUrl.get(), getter_AddRefs(sortInfo.sortPropertyColl2));
+        rv = gRDFService->GetUnicodeResource(resourceUrl, getter_AddRefs(sortInfo.sortPropertyColl2));
         if (NS_FAILED(rv)) return rv;
         sortState->sortPropertyColl2 = sortInfo.sortPropertyColl2;
 
         resourceUrl = sortResource2;
         resourceUrl.Append(NS_LITERAL_STRING("?sort=true"));
-        rv = gRDFService->GetUnicodeResource(resourceUrl.get(), getter_AddRefs(sortInfo.sortPropertySort2));
+        rv = gRDFService->GetUnicodeResource(resourceUrl, getter_AddRefs(sortInfo.sortPropertySort2));
         if (NS_FAILED(rv)) return rv;
         sortState->sortPropertySort2 = sortInfo.sortPropertySort2;
       }
@@ -1409,7 +1370,7 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
 
     nsCOMPtr<nsIDocument> doc;
     if (NS_SUCCEEDED(rv) && parent) {
-      rv = parent->GetDocument(*getter_AddRefs(doc));
+      rv = parent->GetDocument(getter_AddRefs(doc));
       if (!doc)
         parent = nsnull;
     }
@@ -1418,7 +1379,7 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
       nsAutoString id;
       nsCOMPtr<nsIAtom> tag;
 
-      if (NS_FAILED(rv = trueParent->GetTag(*getter_AddRefs(tag))))
+      if (NS_FAILED(rv = trueParent->GetTag(getter_AddRefs(tag))))
         return rv;
 
       rv = trueParent->GetAttr(kNameSpaceID_None, nsXULAtoms::ref, id);
@@ -1450,7 +1411,7 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
     PRInt32 staticCount = 0;
 
     nsAutoString staticValue;
-    if (NS_SUCCEEDED(rv = container->GetAttr(kNameSpaceID_None, kStaticHintAtom, staticValue))
+    if (NS_SUCCEEDED(rv = container->GetAttr(kNameSpaceID_None, nsXULAtoms::staticHint, staticValue))
         && (rv == NS_CONTENT_ATTR_HAS_VALUE))
     {
       // found "static" XUL element count hint
@@ -1462,7 +1423,7 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
       // compute the "static" XUL element count
       nsAutoString  valueStr;
       for (PRInt32 childLoop = 0; childLoop < numChildren; ++childLoop) {
-        container->ChildAt(childLoop, *getter_AddRefs(child));
+        container->ChildAt(childLoop, getter_AddRefs(child));
         if (!child) break;
 
         if (NS_SUCCEEDED(rv = child->GetAttr(kNameSpaceID_None, nsXULAtoms::templateAtom, valueStr))
@@ -1472,7 +1433,7 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
           ++staticCount;
       }
       
-      if (NS_SUCCEEDED(rv = root->GetAttr(kNameSpaceID_None, kStaticsSortLastHintAtom, valueStr))
+      if (NS_SUCCEEDED(rv = root->GetAttr(kNameSpaceID_None, nsXULAtoms::sortStaticsLast, valueStr))
           && (rv == NS_CONTENT_ATTR_HAS_VALUE) && (valueStr.EqualsIgnoreCase("true")))
       {
         // indicate that static XUL comes after RDF-generated content by making negative
@@ -1482,7 +1443,7 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
       // save the "static" XUL element count hint
       valueStr.Truncate();
       valueStr.AppendInt(staticCount);
-      container->SetAttr(kNameSpaceID_None, kStaticHintAtom, valueStr, PR_FALSE);
+      container->SetAttr(kNameSpaceID_None, nsXULAtoms::staticHint, valueStr, PR_FALSE);
     }
 
     if (staticCount <= 0) {
@@ -1502,7 +1463,7 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
       // binary search insertion sort... with interpolation at either end-point.
 
       if (sortState->lastWasFirst) {
-        container->ChildAt(staticCount, *getter_AddRefs(child));
+        container->ChildAt(staticCount, getter_AddRefs(child));
         temp = child.get();
         direction = inplaceSortCallback(&node, &temp, &sortInfo);
         if (direction < 0) {
@@ -1511,7 +1472,7 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
         } else
           sortState->lastWasFirst = PR_FALSE;
       } else if (sortState->lastWasLast) {
-        container->ChildAt(realNumChildren-1, *getter_AddRefs(child));
+        container->ChildAt(realNumChildren-1, getter_AddRefs(child));
         temp = child.get();
         direction = inplaceSortCallback(&node, &temp, &sortInfo);
         if (direction > 0) {
@@ -1524,7 +1485,7 @@ XULSortServiceImpl::InsertContainerNode(nsIRDFCompositeDataSource *db, nsRDFSort
       PRInt32  left = staticCount+1, right = realNumChildren, x;
       while (!childAdded && right >= left) {
         x = (left + right) / 2;
-        container->ChildAt(x-1, *getter_AddRefs(child));
+        container->ChildAt(x-1, getter_AddRefs(child));
         temp = child.get();
 
         // rjc says: since cacheFirstHint is PR_TRUE, the first node passed
@@ -1590,7 +1551,7 @@ XULSortServiceImpl::Sort(nsIDOMNode* node, const nsAString& sortResource, const
       && (rv == NS_CONTENT_ATTR_HAS_VALUE) 
       && (value.EqualsIgnoreCase("true")))
   {
-    if (NS_SUCCEEDED(rv = dbNode->GetAttr(kNameSpaceID_None, kSortResourceAtom, value))
+    if (NS_SUCCEEDED(rv = dbNode->GetAttr(kNameSpaceID_None, nsXULAtoms::sortResource, value))
         && (rv == NS_CONTENT_ATTR_HAS_VALUE) 
         && (value.Equals(sortResource, nsCaseInsensitiveStringComparator())))
     {
@@ -1609,9 +1570,9 @@ XULSortServiceImpl::Sort(nsIDOMNode* node, const nsAString& sortResource, const
   // remove any sort hints on tree root node
   dbNode->UnsetAttr(kNameSpaceID_None, nsXULAtoms::sortActive, PR_FALSE);
   dbNode->UnsetAttr(kNameSpaceID_None, nsXULAtoms::sortDirection, PR_FALSE);
-  dbNode->UnsetAttr(kNameSpaceID_None, kSortSeparatorsAtom, PR_FALSE);
-  dbNode->UnsetAttr(kNameSpaceID_None, kSortResourceAtom, PR_FALSE);
-  dbNode->UnsetAttr(kNameSpaceID_None, kSortResource2Atom, PR_FALSE);
+  dbNode->UnsetAttr(kNameSpaceID_None, nsXULAtoms::sortSeparators, PR_FALSE);
+  dbNode->UnsetAttr(kNameSpaceID_None, nsXULAtoms::sortResource, PR_FALSE);
+  dbNode->UnsetAttr(kNameSpaceID_None, nsXULAtoms::sortResource2, PR_FALSE);
 
   nsCOMPtr<nsIRDFCompositeDataSource>  cds;
   if (NS_SUCCEEDED(rv = dbXULNode->GetDatabase(getter_AddRefs(cds))))
@@ -1634,33 +1595,33 @@ XULSortServiceImpl::Sort(nsIDOMNode* node, const nsAString& sortResource, const
   GetSortColumnInfo(contentNode, unused, unused, sortResource2, sortInfo.inbetweenSeparatorSort);
 
   // build resource url for first sort resource
-  rv = gRDFService->GetUnicodeResource(PromiseFlatString(sortResource).get(), getter_AddRefs(sortInfo.sortProperty));
+  rv = gRDFService->GetUnicodeResource(sortResource, getter_AddRefs(sortInfo.sortProperty));
   if (NS_FAILED(rv)) return rv;
 
   nsAutoString resourceUrl;
   resourceUrl.Assign(sortResource);
   resourceUrl.Append(NS_LITERAL_STRING("?collation=true"));
-  rv = gRDFService->GetUnicodeResource(resourceUrl.get(), getter_AddRefs(sortInfo.sortPropertyColl));
+  rv = gRDFService->GetUnicodeResource(resourceUrl, getter_AddRefs(sortInfo.sortPropertyColl));
   if (NS_FAILED(rv)) return rv;
 
   resourceUrl.Assign(sortResource);
   resourceUrl.Append(NS_LITERAL_STRING("?sort=true"));
-  rv = gRDFService->GetUnicodeResource(resourceUrl.get(), getter_AddRefs(sortInfo.sortPropertySort));
+  rv = gRDFService->GetUnicodeResource(resourceUrl, getter_AddRefs(sortInfo.sortPropertySort));
   if (NS_FAILED(rv)) return rv;
 
   // build resource url for second sort resource
   if (!sortResource2.IsEmpty()) {
-    rv = gRDFService->GetUnicodeResource(sortResource2.get(), getter_AddRefs(sortInfo.sortProperty2));
+    rv = gRDFService->GetUnicodeResource(sortResource2, getter_AddRefs(sortInfo.sortProperty2));
     if (NS_FAILED(rv)) return rv;
 
     resourceUrl = sortResource2;
     resourceUrl.Append(NS_LITERAL_STRING("?collation=true"));
-    rv = gRDFService->GetUnicodeResource(resourceUrl.get(), getter_AddRefs(sortInfo.sortPropertyColl2));
+    rv = gRDFService->GetUnicodeResource(resourceUrl, getter_AddRefs(sortInfo.sortPropertyColl2));
     if (NS_FAILED(rv)) return rv;
 
     resourceUrl = sortResource2;
     resourceUrl.Append(NS_LITERAL_STRING("?sort=true"));
-    rv = gRDFService->GetUnicodeResource(resourceUrl.get(), getter_AddRefs(sortInfo.sortPropertySort2));
+    rv = gRDFService->GetUnicodeResource(resourceUrl, getter_AddRefs(sortInfo.sortPropertySort2));
     if (NS_FAILED(rv)) return rv;
   }
 
@@ -1669,14 +1630,14 @@ XULSortServiceImpl::Sort(nsIDOMNode* node, const nsAString& sortResource, const
 
   // start sorting the content depending on the tag
   nsCOMPtr<nsIAtom> dbNodeTag;
-  dbNode->GetTag(*getter_AddRefs(dbNodeTag));
+  dbNode->GetTag(getter_AddRefs(dbNodeTag));
   nsCOMPtr<nsIContent> container;
   if (NS_FAILED(rv = FindSortableContainer(dbNode, getter_AddRefs(container)))) return rv;
   SortContainer(container, &sortInfo, invertTreeFlag);
   
   // Now remove the db node and re-insert it to force the frames to be rebuilt.
   nsCOMPtr<nsIContent> containerParent;
-  if (NS_FAILED(rv = container->GetParent(*getter_AddRefs(containerParent))))  return rv;
+  if (NS_FAILED(rv = container->GetParent(getter_AddRefs(containerParent))))  return rv;
   PRInt32 containerIndex;
   if (NS_FAILED(rv = containerParent->IndexOf(container, containerIndex)))  return rv;
   PRInt32 childCount;
