@@ -70,6 +70,7 @@
 #include "nsVoidArray.h"
 #include "nsUnicharUtils.h"
 #include "nsINameSpaceManager.h"
+#include "nsIDOMClassInfo.h"
 
 // For security check
 #include "nsIDocument.h"
@@ -92,9 +93,6 @@ public:
     // nsITreeView
     NS_DECL_NSITREEVIEW
 
-    // nsXULTemplateBuilder
-    NS_IMETHOD Rebuild();
-
     NS_IMETHOD DocumentWillBeDestroyed(nsIDocument *aDocument);
 
 protected:
@@ -119,6 +117,9 @@ protected:
     virtual nsresult
     InitializeRuleNetworkForSimpleRules(InnerNode** aChildNode);
 
+    virtual nsresult
+    RebuildAll();
+
     /**
      * Override default behavior to additionally handle the <row>
      * condition.
@@ -308,9 +309,16 @@ NS_NewXULTreeBuilder(nsISupports* aOuter, REFNSIID aIID, void** aResult)
     return rv;
 }
 
-NS_IMPL_ISUPPORTS_INHERITED2(nsXULTreeBuilder, nsXULTemplateBuilder,
-                             nsIXULTreeBuilder,
-                             nsITreeView)
+NS_IMPL_ADDREF(nsXULTreeBuilder)
+NS_IMPL_RELEASE(nsXULTreeBuilder)
+
+NS_INTERFACE_MAP_BEGIN(nsXULTreeBuilder)
+  NS_INTERFACE_MAP_ENTRY(nsIXULTreeBuilder)
+  NS_INTERFACE_MAP_ENTRY(nsITreeView)
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIXULTreeBuilder)
+  NS_INTERFACE_MAP_ENTRY_DOM_CLASSINFO(XULTreeBuilder)
+NS_INTERFACE_MAP_END_INHERITING(nsXULTemplateBuilder)
+
 
 nsXULTreeBuilder::nsXULTreeBuilder()
     : mSortVariable(0),
@@ -407,60 +415,66 @@ nsXULTreeBuilder::Sort(nsIDOMElement* aElement)
     if (! header)
         return NS_ERROR_FAILURE;
 
+    nsAutoString sortLocked;
+    header->GetAttr(kNameSpaceID_None, nsXULAtoms::sortLocked, sortLocked);
+    if (sortLocked.Equals(NS_LITERAL_STRING("true")))
+        return NS_OK;
+
     nsAutoString sort;
     header->GetAttr(kNameSpaceID_None, nsXULAtoms::sort, sort);
 
-    if (!sort.IsEmpty()) {
-        // Grab the new sort variable
-        mSortVariable = mRules.LookupSymbol(sort.get());
+    if (sort.IsEmpty())
+        return NS_OK;
 
-        // Cycle the sort direction
-        nsAutoString dir;
-        header->GetAttr(kNameSpaceID_None, nsXULAtoms::sortDirection, dir);
+    // Grab the new sort variable
+    mSortVariable = mRules.LookupSymbol(sort.get());
 
-        if (dir == NS_LITERAL_STRING("ascending")) {
-            dir = NS_LITERAL_STRING("descending");
-            mSortDirection = eDirection_Descending;
-        }
-        else if (dir == NS_LITERAL_STRING("descending")) {
-            dir = NS_LITERAL_STRING("natural");
-            mSortDirection = eDirection_Natural;
-        }
-        else {
-            dir = NS_LITERAL_STRING("ascending");
-            mSortDirection = eDirection_Ascending;
-        }
+    // Cycle the sort direction
+    nsAutoString dir;
+    header->GetAttr(kNameSpaceID_None, nsXULAtoms::sortDirection, dir);
 
-        // Sort it
-        SortSubtree(mRows.GetRoot());
-        mRows.InvalidateCachedRow();
-        if (mBoxObject) 
-            mBoxObject->Invalidate();
+    if (dir == NS_LITERAL_STRING("ascending")) {
+        dir = NS_LITERAL_STRING("descending");
+        mSortDirection = eDirection_Descending;
+    }
+    else if (dir == NS_LITERAL_STRING("descending")) {
+        dir = NS_LITERAL_STRING("natural");
+        mSortDirection = eDirection_Natural;
+    }
+    else {
+        dir = NS_LITERAL_STRING("ascending");
+        mSortDirection = eDirection_Ascending;
+    }
 
-        header->SetAttr(kNameSpaceID_None, nsXULAtoms::sortDirection, dir, PR_TRUE);
-        header->SetAttr(kNameSpaceID_None, nsXULAtoms::sortActive, NS_LITERAL_STRING("true"), PR_TRUE);
+    // Sort it.
+    SortSubtree(mRows.GetRoot());
+    mRows.InvalidateCachedRow();
+    if (mBoxObject) 
+        mBoxObject->Invalidate();
 
-        // Unset sort attribute(s) on the other columns
-        nsCOMPtr<nsIContent> parentContent;
-        header->GetParent(*getter_AddRefs(parentContent));
-        if (parentContent) {
-            nsCOMPtr<nsIAtom> parentTag;
-            parentContent->GetTag(*getter_AddRefs(parentTag));
-            if (parentTag == nsXULAtoms::treecols) {
-                PRInt32 numChildren;
-                parentContent->ChildCount(numChildren);
-                for (int i = 0; i < numChildren; ++i) {
-                    nsCOMPtr<nsIContent> childContent;
-                    nsCOMPtr<nsIAtom> childTag;
-                    parentContent->ChildAt(i, *getter_AddRefs(childContent));
-                    if (childContent) {
-                        childContent->GetTag(*getter_AddRefs(childTag));
-                        if (childTag == nsXULAtoms::treecol && childContent != header) {
-                            childContent->UnsetAttr(kNameSpaceID_None,
-                                                    nsXULAtoms::sortDirection, PR_TRUE);
-                            childContent->UnsetAttr(kNameSpaceID_None,
-                                                    nsXULAtoms::sortActive, PR_TRUE);
-                        }
+    header->SetAttr(kNameSpaceID_None, nsXULAtoms::sortDirection, dir, PR_TRUE);
+    header->SetAttr(kNameSpaceID_None, nsXULAtoms::sortActive, NS_LITERAL_STRING("true"), PR_TRUE);
+
+    // Unset sort attribute(s) on the other columns
+    nsCOMPtr<nsIContent> parentContent;
+    header->GetParent(getter_AddRefs(parentContent));
+    if (parentContent) {
+        nsCOMPtr<nsIAtom> parentTag;
+        parentContent->GetTag(getter_AddRefs(parentTag));
+        if (parentTag == nsXULAtoms::treecols) {
+            PRInt32 numChildren;
+            parentContent->ChildCount(numChildren);
+            for (int i = 0; i < numChildren; ++i) {
+                nsCOMPtr<nsIContent> childContent;
+                nsCOMPtr<nsIAtom> childTag;
+                parentContent->ChildAt(i, getter_AddRefs(childContent));
+                if (childContent) {
+                    childContent->GetTag(getter_AddRefs(childTag));
+                    if (childTag == nsXULAtoms::treecol && childContent != header) {
+                        childContent->UnsetAttr(kNameSpaceID_None,
+                                                nsXULAtoms::sortDirection, PR_TRUE);
+                        childContent->UnsetAttr(kNameSpaceID_None,
+                                                nsXULAtoms::sortActive, PR_TRUE);
                     }
                 }
             }
@@ -794,8 +808,12 @@ nsXULTreeBuilder::SetTree(nsITreeBoxObject* tree)
 
     mBoxObject = tree;
 
+    // If this is teardown time, then we're done.
+    if (! mBoxObject)
+        return NS_OK;
+
     nsCOMPtr<nsIDocument> doc;
-    mRoot->GetDocument(*getter_AddRefs(doc));
+    mRoot->GetDocument(getter_AddRefs(doc));
     NS_ASSERTION(doc, "element has no document");
     if (!doc)
         return NS_ERROR_UNEXPECTED;
@@ -1051,56 +1069,6 @@ nsXULTreeBuilder::PerformActionOnCell(const PRUnichar* action, PRInt32 row, cons
     return NS_OK;
 }
 
-//----------------------------------------------------------------------
-//
-// nsIXULTemplateBuilder methods
-//
-
-NS_IMETHODIMP
-nsXULTreeBuilder::Rebuild()
-{
-    NS_PRECONDITION(mRoot != nsnull, "not initialized");
-    if (! mRoot)
-        return NS_ERROR_NOT_INITIALIZED;
-
-    nsresult rv;
-
-    PRInt32 count = mRows.Count();
-    mRows.Clear();
-    mConflictSet.Clear();
-    if (mBoxObject)
-      mBoxObject->RowCountChanged(0, -count);
-
-    rv = CompileRules();
-    if (NS_FAILED(rv)) return rv;
-
-    // Seed the rule network with assignments for the tree row
-    // variable
-    nsCOMPtr<nsIRDFResource> root;
-    nsXULContentUtils::GetElementRefResource(mRoot, getter_AddRefs(root));
-    mRows.SetRootResource(root);
-
-#ifdef PR_LOGGING
-    if (PR_LOG_TEST(gXULTemplateLog, PR_LOG_DEBUG)) {
-        const char* s = "(null)";
-        if (root)
-            root->GetValueConst(&s);
-
-        PR_LOG(gXULTemplateLog, PR_LOG_DEBUG,
-               ("xultemplate[%p] root=%s", this, s));
-    }
-#endif
-
-    if (root)
-        OpenContainer(-1, root);
-
-    return NS_OK;
-}
-
-//----------------------------------------------------------------------
-//
-// nsXULTemplateBuilder abstract methods
-//
 
 NS_IMETHODIMP
 nsXULTreeBuilder::DocumentWillBeDestroyed(nsIDocument* aDocument)
@@ -1301,9 +1269,9 @@ nsXULTreeBuilder::EnsureSortVariables()
     treecols->ChildCount(count);
     for (PRInt32 i = 0; i < count; i++) {
         nsCOMPtr<nsIContent> child;
-        treecols->ChildAt(i, *getter_AddRefs(child));
+        treecols->ChildAt(i, getter_AddRefs(child));
         nsCOMPtr<nsIAtom> tag;
-        child->GetTag(*getter_AddRefs(tag));
+        child->GetTag(getter_AddRefs(tag));
         if (tag == nsXULAtoms::treecol) {
             nsAutoString sortActive;
             child->GetAttr(kNameSpaceID_None, nsXULAtoms::sortActive, sortActive);
@@ -1371,6 +1339,58 @@ nsXULTreeBuilder::InitializeRuleNetworkForSimpleRules(InnerNode** aChildNode)
     return NS_OK;
 }
 
+nsresult
+nsXULTreeBuilder::RebuildAll()
+{
+    NS_PRECONDITION(mRoot != nsnull, "not initialized");
+    if (! mRoot)
+        return NS_ERROR_NOT_INITIALIZED;
+
+    nsCOMPtr<nsIDocument> doc;
+    nsresult rv = mRoot->GetDocument(getter_AddRefs(doc));
+    if (NS_FAILED(rv)) return rv;
+
+    // Bail out early if we are being torn down.
+    if (!doc)
+        return NS_OK;
+
+    if (mBoxObject) {
+        mBoxObject->BeginUpdateBatch();
+    }
+
+    mRows.Clear();
+    mConflictSet.Clear();
+
+    rv = CompileRules();
+    if (NS_FAILED(rv)) return rv;
+
+    // Seed the rule network with assignments for the tree row
+    // variable
+    nsCOMPtr<nsIRDFResource> root;
+    nsXULContentUtils::GetElementRefResource(mRoot, getter_AddRefs(root));
+    mRows.SetRootResource(root);
+
+#ifdef PR_LOGGING
+    if (PR_LOG_TEST(gXULTemplateLog, PR_LOG_DEBUG)) {
+        const char* s = "(null)";
+        if (root)
+            root->GetValueConst(&s);
+
+        PR_LOG(gXULTemplateLog, PR_LOG_DEBUG,
+               ("xultemplate[%p] root=%s", this, s));
+    }
+#endif
+
+    if (root)
+        OpenContainer(-1, root);
+
+    if (mBoxObject) {
+        mBoxObject->EndUpdateBatch();
+    }
+
+    return NS_OK;
+}
+
 nsresult
 nsXULTreeBuilder::CompileCondition(nsIAtom* aTag,
                                        nsTemplateRule* aRule,
@@ -1484,9 +1504,9 @@ nsXULTreeBuilder::GetTemplateActionCellFor(PRInt32 aRow,
         PRInt32 j = 0;
         for (PRInt32 i = 0; i < count; ++i) {
             nsCOMPtr<nsIContent> child;
-            row->ChildAt(i, *getter_AddRefs(child));
+            row->ChildAt(i, getter_AddRefs(child));
             nsCOMPtr<nsIAtom> tag;
-            child->GetTag(*getter_AddRefs(tag));
+            child->GetTag(getter_AddRefs(tag));
             if (tag == nsXULAtoms::treecell) {
                 nsAutoString ref;
                 child->GetAttr(kNameSpaceID_None, nsXULAtoms::ref, ref);
@@ -1989,7 +2009,7 @@ nsXULTreeBuilder::CompareMatches(nsTemplateMatch* aLeft, nsTemplateMatch* aRight
                 r->GetLength(&rlen);
                 
                 mCollation->CompareRawSortKey(lval, llen, rval, rlen, &result);
-                return result;
+                return result * mSortDirection;
             }
         }
     }
