@@ -42,8 +42,13 @@
 #include "nsIEventStateManager.h"
 #include "nsIDocShell.h"
 #include "nsIBaseWindow.h"
+#include "nsIWindowWatcher.h"
+#include "nsIDocShellTreeItem.h"
+#include "nsIDocShellTreeOwner.h"
+#include "nsIInterfaceRequestorUtils.h"
+#include "nsIServiceManagerUtils.h"
 
-#ifdef INCLUDE_XUL
+#ifdef MOZ_XUL
 #include "nsIDOMXULDocument.h"
 #include "nsIDOMXULElement.h"
 #endif
@@ -53,9 +58,9 @@
 nsFocusController::nsFocusController(void)
 : mSuppressFocus(0), 
   mSuppressFocusScroll(PR_FALSE), 
-	mActive(PR_FALSE)
+  mActive(PR_FALSE),
+  mUpdateWindowWatcher(PR_FALSE)
 {
-	NS_INIT_REFCNT();
 }
 
 nsFocusController::~nsFocusController(void)
@@ -100,7 +105,11 @@ nsFocusController::GetFocusedWindow(nsIDOMWindowInternal** aWindow)
 NS_IMETHODIMP
 nsFocusController::SetFocusedElement(nsIDOMElement* aElement)
 {
-  mPreviousElement = mCurrentElement;
+  if (mCurrentElement) 
+    mPreviousElement = mCurrentElement;
+  else if (aElement) 
+    mPreviousElement = aElement;
+
   mCurrentElement = aElement;
 
   if (!mSuppressFocus) {
@@ -138,6 +147,14 @@ nsFocusController::SetFocusedWindow(nsIDOMWindowInternal* aWindow)
   else if (aWindow) mPreviousWindow = aWindow;
 
   mCurrentWindow = aWindow;
+
+  if (mUpdateWindowWatcher) {
+    NS_ASSERTION(mActive, "This shouldn't happen");
+    if (mCurrentWindow)
+      UpdateWWActiveWindow();
+    mUpdateWindowWatcher = PR_FALSE;
+  }
+
   return NS_OK;
 }
 
@@ -173,7 +190,7 @@ nsFocusController::GetControllers(nsIControllers** aResult)
   //     so this code would have no special knowledge of what object might have controllers.
   if (mCurrentElement) {
 
-#ifdef INCLUDE_XUL
+#ifdef MOZ_XUL
     nsCOMPtr<nsIDOMXULElement> xulElement(do_QueryInterface(mCurrentElement));
     if (xulElement)
       return xulElement->GetControllers(aResult);
@@ -205,12 +222,12 @@ nsFocusController::MoveFocus(PRBool aForward, nsIDOMElement* aElt)
   nsCOMPtr<nsIContent> content;
   if (aElt) {
     content = do_QueryInterface(aElt);
-    content->GetDocument(*getter_AddRefs(doc));
+    content->GetDocument(getter_AddRefs(doc));
   }
   else {
     if (mCurrentElement) {
       content = do_QueryInterface(mCurrentElement);
-      content->GetDocument(*getter_AddRefs(doc));
+      content->GetDocument(getter_AddRefs(doc));
       content = nsnull;
     }
     else if (mCurrentWindow) {
@@ -296,8 +313,10 @@ nsFocusController::Focus(nsIDOMEvent* aEvent)
           nsCOMPtr<nsIDOMDocument> windowDoc;
           mCurrentWindow->GetDocument(getter_AddRefs(windowDoc));
           if (ownerDoc != windowDoc)
-            mCurrentElement = nsnull;
+            mCurrentElement = mPreviousElement = nsnull;
         }
+        else
+          mPreviousElement = nsnull;
 
         if (!mCurrentElement)
           UpdateCommands(NS_LITERAL_STRING("focus"));
@@ -469,9 +488,45 @@ NS_IMETHODIMP
 nsFocusController::SetActive(PRBool aActive)
 {
   mActive = aActive;
+
+  // We may be activated before we ever have a focused window set.
+  // This happens on window creation, where the FocusController
+  // is activated just prior to setting the focused window.
+  // (see nsEventStateManager::PreHandleEvent/NS_ACTIVATE)
+  // If this is the case, we need to queue a notification of the
+  // WindowWatcher until SetFocusedWindow is called.
+  if (mActive) {
+    if (mCurrentWindow)
+      UpdateWWActiveWindow();
+    else
+      mUpdateWindowWatcher = PR_TRUE;
+  }
+
   return NS_OK;
 }
 
+void
+nsFocusController::UpdateWWActiveWindow()
+{
+  // Inform the window watcher of the new active window.
+  nsCOMPtr<nsIWindowWatcher> wwatch = do_GetService("@mozilla.org/embedcomp/window-watcher;1");
+  if (!wwatch) return;
+
+  // This gets the toplevel DOMWindow
+  nsCOMPtr<nsIScriptGlobalObject> sgo = do_QueryInterface(mCurrentWindow);
+  nsCOMPtr<nsIDocShell> docShell;
+  sgo->GetDocShell(getter_AddRefs(docShell));
+  if (!docShell) return;
+
+  nsCOMPtr<nsIDocShellTreeItem> docShellAsItem(do_QueryInterface(docShell));
+  nsCOMPtr<nsIDocShellTreeItem> rootItem;
+  docShellAsItem->GetRootTreeItem(getter_AddRefs(rootItem));
+  NS_ASSERTION(rootItem, "Invalid docshell tree - no root!");
+
+  nsCOMPtr<nsIDOMWindow> domWin = do_GetInterface(rootItem);
+  wwatch->SetActiveWindow(domWin);
+}
+
 NS_IMETHODIMP
 nsFocusController::GetPopupNode(nsIDOMNode** aNode)
 {
