@@ -45,7 +45,7 @@
 #include "nsIObserver.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIXPCScriptNotify.h"
-#include "nsITimerCallback.h"
+#include "nsITimer.h"
 
 
 class nsJSContext : public nsIScriptContext,
@@ -99,7 +99,7 @@ public:
                                             nsIAtom *aName,
                                             void *aHandler);
   NS_IMETHOD       CompileFunction(void* aTarget,
-                                   const nsCString& aName,
+                                   const nsACString& aName,
                                    PRUint32 aArgCount,
                                    const char** aArgArray,
                                    const nsAString& aBody,
@@ -131,7 +131,7 @@ public:
 
   NS_DECL_NSIXPCSCRIPTNOTIFY
 
-  NS_IMETHOD_(void) Notify(nsITimer *timer);
+  NS_DECL_NSITIMERCALLBACK
 
 protected:
   nsresult InitClasses();
@@ -144,7 +144,6 @@ private:
   JSContext *mContext;
   PRUint32 mNumEvaluations;
 
-  nsCOMPtr<nsIScriptSecurityManager> mSecurityManager; // [OWNER]
   nsIScriptContextOwner* mOwner;  /* NB: weak reference, not ADDREF'd */
   nsScriptTerminationFunc mTerminationFunc;
 
