@@ -40,9 +40,11 @@
 
 #include "nsEditorSpellCheck.h"
 
-#include "nsTextServicesCID.h"
 #include "nsITextServicesDocument.h"
 #include "nsISpellChecker.h"
+#include "nsISelection.h"
+#include "nsIDOMRange.h"
+#include "nsIEditor.h"
 
 #include "nsIComponentManager.h"
 #include "nsXPIDLString.h"
@@ -55,15 +57,12 @@
 #include "nsReadableUtils.h"
 #include "nsComposeTxtSrvFilter.h"
 
-static NS_DEFINE_CID(kCTextServicesDocumentCID, NS_TEXTSERVICESDOCUMENT_CID);
-
 NS_IMPL_ISUPPORTS1(nsEditorSpellCheck, nsIEditorSpellCheck);
 
 nsEditorSpellCheck::nsEditorSpellCheck()
   : mSuggestedWordIndex(0)
   , mDictionaryIndex(0)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsEditorSpellCheck::~nsEditorSpellCheck()
@@ -74,16 +73,13 @@ nsEditorSpellCheck::~nsEditorSpellCheck()
 }
 
 NS_IMETHODIMP    
-nsEditorSpellCheck::InitSpellChecker(nsIEditor* editor)
+nsEditorSpellCheck::InitSpellChecker(nsIEditor* aEditor, PRBool aEnableSelectionChecking)
 {
   nsresult rv;
 
   // We can spell check with any editor type
-  nsCOMPtr<nsITextServicesDocument>tsDoc;
-  rv = nsComponentManager::CreateInstance(kCTextServicesDocumentCID,
-                                          nsnull,
-                                          NS_GET_IID(nsITextServicesDocument),
-                                          (void **)getter_AddRefs(tsDoc));
+  nsCOMPtr<nsITextServicesDocument>tsDoc =
+     do_CreateInstance("@mozilla.org/textservices/textservicesdocument;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!tsDoc)
@@ -92,9 +88,57 @@ nsEditorSpellCheck::InitSpellChecker(nsIEditor* editor)
   tsDoc->SetFilter(mTxtSrvFilter);
 
   // Pass the editor to the text services document
-  rv = tsDoc->InitWithEditor(editor);
+  rv = tsDoc->InitWithEditor(aEditor);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  if (aEnableSelectionChecking) {
+    // Find out if the section is collapsed or not.
+    // If it isn't, we want to spellcheck just the selection.
+
+    nsCOMPtr<nsISelection> selection;
+
+    rv = aEditor->GetSelection(getter_AddRefs(selection));
+    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ENSURE_TRUE(selection, NS_ERROR_FAILURE);
+
+    PRInt32 count = 0;
+
+    rv = selection->GetRangeCount(&count);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    if (count > 0) {
+      nsCOMPtr<nsIDOMRange> range;
+
+      rv = selection->GetRangeAt(0, getter_AddRefs(range));
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      PRBool collapsed = PR_FALSE;
+      rv = range->GetCollapsed(&collapsed);
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      if (!collapsed) {
+        // We don't want to touch the range in the selection,
+        // so create a new copy of it.
+
+        nsCOMPtr<nsIDOMRange> rangeBounds;
+        rv =  range->CloneRange(getter_AddRefs(rangeBounds));
+        NS_ENSURE_SUCCESS(rv, rv);
+        NS_ENSURE_TRUE(rangeBounds, NS_ERROR_FAILURE);
+
+        // Make sure the new range spans complete words.
+
+        rv = tsDoc->ExpandRangeToWordBoundaries(rangeBounds);
+        NS_ENSURE_SUCCESS(rv, rv);
+
+        // Now tell the text services that you only want
+        // to iterate over the text in this range.
+
+        rv = tsDoc->SetExtent(rangeBounds);
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
+    }
+  }
+
   rv = nsComponentManager::CreateInstance(NS_SPELLCHECKER_CONTRACTID,
                                           nsnull,
                                           NS_GET_IID(nsISpellChecker),
