@@ -42,9 +42,7 @@
 #include "nsIDOMRange.h"
 #include "nsISelection.h"
 #include "nsEditor.h"
-#include "nsLayoutCID.h"
-
-static NS_DEFINE_CID(kCRangeCID, NS_RANGE_CID);
+#include "nsEditorUtils.h"
 
 
 /***************************************************************************
@@ -63,10 +61,9 @@ nsresult
 nsSelectionState::SaveSelection(nsISelection *aSel)
 {
   if (!aSel) return NS_ERROR_NULL_POINTER;
-  nsresult res = NS_OK;
   PRInt32 i,rangeCount, arrayCount = mArray.Count();
-  nsRangeStore *item;
   aSel->GetRangeCount(&rangeCount);
+  nsRangeStore *item;
   
   // if we need more items in the array, new them
   if (arrayCount<rangeCount)
@@ -91,6 +88,7 @@ nsSelectionState::SaveSelection(nsISelection *aSel)
   }
   
   // now store the selection ranges
+  nsresult res = NS_OK;
   for (i=0; i<rangeCount; i++)
   {
     item = (nsRangeStore*)mArray.ElementAt(i);
@@ -166,7 +164,7 @@ nsSelectionState::IsEqual(nsSelectionState *aSelState)
     itsItem->GetRange(address_of(itsRange));
     if (!myRange || !itsRange) return PR_FALSE;
   
-    PRInt32 compResult;
+    PRInt16 compResult;
     myRange->CompareBoundaryPoints(nsIDOMRange::START_TO_START, itsRange, &compResult);
     if (compResult) return PR_FALSE;
     myRange->CompareBoundaryPoints(nsIDOMRange::END_TO_END, itsRange, &compResult);
@@ -241,7 +239,7 @@ nsRangeUpdater::RegisterSelectionState(nsSelectionState &aSelState)
     RegisterRangeItem(item);
   }
 
-  return NS_OK;;
+  return NS_OK;
 }
 
 nsresult 
@@ -258,7 +256,7 @@ nsRangeUpdater::DropSelectionState(nsSelectionState &aSelState)
     DropRangeItem(item);
   }
 
-  return NS_OK;;
+  return NS_OK;
 }
 
 // gravity methods:
@@ -303,11 +301,12 @@ nsRangeUpdater::SelAdjDeleteNode(nsIDOMNode *aNode)
 
   nsCOMPtr<nsIDOMNode> parent;
   PRInt32 offset = 0;
-  nsRangeStore *item;
   
   nsresult res = nsEditor::GetNodeLocation(aNode, address_of(parent), &offset);
   NS_ENSURE_SUCCESS(res, res);
   
+  // check for range endpoints that are after aNode and in the same parent
+  nsRangeStore *item;
   for (i=0; i<count; i++)
   {
     item = (nsRangeStore*)mArray.ElementAt(i);
@@ -317,9 +316,35 @@ nsRangeUpdater::SelAdjDeleteNode(nsIDOMNode *aNode)
       item->startOffset--;
     if ((item->endNode.get() == parent) && (item->endOffset > offset))
       item->endOffset--;
+      
+    // check for range endpoints that are in aNode
+    if (item->startNode == aNode)
+    {
+      item->startNode   = parent;
+      item->startOffset = offset;
+    }
+    if (item->endNode == aNode)
+    {
+      item->endNode   = parent;
+      item->endOffset = offset;
+    }
+
+    // check for range endpoints that are in descendants of aNode
+    nsCOMPtr<nsIDOMNode> oldStart;
+    if (nsEditorUtils::IsDescendantOf(item->startNode, aNode))
+    {
+      oldStart = item->startNode;  // save for efficiency hack below.
+      item->startNode   = parent;
+      item->startOffset = offset;
+    }
+
+    // avoid having to call IsDescendantOf() for common case of range startnode == range endnode.
+    if ((item->endNode == oldStart) || nsEditorUtils::IsDescendantOf(item->endNode, aNode))
+    {
+      item->endNode   = parent;
+      item->endOffset = offset;
+    }
   }
-  // MOOSE: also check inside of aNode, expensive.  But in theory, we shouldn't
-  // actually hit this case in the usage i forsee for this.
   return NS_OK;
 }
 
@@ -395,9 +420,9 @@ nsRangeUpdater::SelAdjJoinNodes(nsIDOMNode *aLeftNode,
     item = (nsRangeStore*)mArray.ElementAt(i);
     if (!item) return NS_ERROR_NULL_POINTER;
     
-    // adjust endpoints in aParent
     if (item->startNode.get() == aParent)
     {
+      // adjust start point in aParent
       if (item->startOffset > aOffset)
       {
         item->startOffset--;
@@ -409,8 +434,20 @@ nsRangeUpdater::SelAdjJoinNodes(nsIDOMNode *aLeftNode,
         item->startOffset = aOldLeftNodeLength;
       }
     }
+    else if (item->startNode.get() == aRightNode)
+    {
+      // adjust start point in aRightNode
+      item->startOffset += aOldLeftNodeLength;
+    }
+    else if (item->startNode.get() == aLeftNode)
+    {
+      // adjust start point in aLeftNode
+      item->startNode = aRightNode;
+    }
+
     if (item->endNode.get() == aParent)
     {
+      // adjust end point in aParent
       if (item->endOffset > aOffset)
       {
         item->endOffset--;
@@ -422,11 +459,16 @@ nsRangeUpdater::SelAdjJoinNodes(nsIDOMNode *aLeftNode,
         item->endOffset = aOldLeftNodeLength;
       }
     }
-    // adjust endpoints in aRightNode
-    if (item->startNode.get() == aRightNode)
-      item->startOffset += aOldLeftNodeLength;
-    if (item->endNode.get() == aRightNode)
-      item->endOffset += aOldLeftNodeLength;
+    else if (item->endNode.get() == aRightNode)
+    {
+      // adjust end point in aRightNode
+       item->endOffset += aOldLeftNodeLength;
+    }
+    else if (item->endNode.get() == aLeftNode)
+    {
+      // adjust end point in aLeftNode
+      item->endNode = aRightNode;
+    }
   }
   
   return NS_OK;
@@ -438,13 +480,13 @@ nsRangeUpdater::SelAdjInsertText(nsIDOMCharacterData *aTextNode, PRInt32 aOffset
 {
   if (mLock) return NS_OK;  // lock set by Will/DidReplaceParent, etc...
 
-  if (!aTextNode) return NS_ERROR_NULL_POINTER;
-  PRInt32 len=aString.Length(), i, count = mArray.Count();
+  PRInt32 count = mArray.Count();
   if (!count) return NS_OK;
-  nsRangeStore *item;
   nsCOMPtr<nsIDOMNode> node(do_QueryInterface(aTextNode));
   if (!node) return NS_ERROR_NULL_POINTER;
   
+  PRInt32 len=aString.Length(), i;
+  nsRangeStore *item;
   for (i=0; i<count; i++)
   {
     item = (nsRangeStore*)mArray.ElementAt(i);
@@ -464,7 +506,6 @@ nsRangeUpdater::SelAdjDeleteText(nsIDOMCharacterData *aTextNode, PRInt32 aOffset
 {
   if (mLock) return NS_OK;  // lock set by Will/DidReplaceParent, etc...
 
-  if (!aTextNode) return NS_ERROR_NULL_POINTER;
   PRInt32 i, count = mArray.Count();
   if (!count) return NS_OK;
   nsRangeStore *item;
@@ -661,10 +702,8 @@ nsresult nsRangeStore::StoreRange(nsIDOMRange *aRange)
 nsresult nsRangeStore::GetRange(nsCOMPtr<nsIDOMRange> *outRange)
 {
   if (!outRange) return NS_ERROR_NULL_POINTER;
-  nsresult res = nsComponentManager::CreateInstance(kCRangeCID,
-                             nsnull,
-                             NS_GET_IID(nsIDOMRange),
-                             getter_AddRefs(*outRange));
+  nsresult res;
+  *outRange = do_CreateInstance("@mozilla.org/content/range;1", &res);
   if(NS_FAILED(res)) return res;
 
   res = (*outRange)->SetStart(startNode, startOffset);
