@@ -45,27 +45,7 @@
  *                     XPCOM cruft 
  *******************************************************************/
 
-NS_IMPL_ADDREF(TypeInState)
-NS_IMPL_RELEASE(TypeInState)
-
-NS_IMETHODIMP
-TypeInState::QueryInterface(REFNSIID aIID, void** aInstancePtr)
-{
-  if (nsnull == aInstancePtr) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  if (aIID.Equals(NS_GET_IID(nsISupports))) {
-    *aInstancePtr = (void*)(nsISupports*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsISelectionListener))) {
-    *aInstancePtr = (void*)(nsISelectionListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  return NS_NOINTERFACE;
-}
+NS_IMPL_ISUPPORTS1(TypeInState, nsISelectionListener)
 
 /********************************************************************
  *                   public methods
@@ -77,7 +57,6 @@ TypeInState::TypeInState() :
 ,mRelativeFontSize(0)
 ,mLastSelectionOffset(0)
 {
-  NS_INIT_ISUPPORTS();
   Reset();
 }
 
@@ -89,7 +68,24 @@ TypeInState::~TypeInState()
   Reset();
 }
 
-NS_IMETHODIMP TypeInState::NotifySelectionChanged(nsIDOMDocument *, nsISelection *aSelection,short)
+nsresult TypeInState::UpdateSelState(nsISelection *aSelection)
+{
+  if (!aSelection) return NS_ERROR_NULL_POINTER;
+  
+  PRBool isCollapsed = PR_FALSE;
+  nsresult result = aSelection->GetIsCollapsed(&isCollapsed);
+
+  if (NS_FAILED(result)) return result;
+
+  if (isCollapsed)
+  {
+    result = nsEditor::GetStartNodeAndOffset(aSelection, address_of(mLastSelectionContainer), &mLastSelectionOffset);
+  }
+  return result;
+}
+
+
+NS_IMETHODIMP TypeInState::NotifySelectionChanged(nsIDOMDocument *, nsISelection *aSelection, short)
 {
   // XXX: Selection currently generates bogus selection changed notifications
   // XXX: (bug 140303). It can notify us when the selection hasn't actually
@@ -174,12 +170,12 @@ nsresult TypeInState::SetProp(nsIAtom *aProp, const nsString &aAttr)
 nsresult TypeInState::SetProp(nsIAtom *aProp, const nsString &aAttr, const nsString &aValue)
 {
   // special case for big/small, these nest
-  if (nsIEditProperty::big == aProp)
+  if (nsEditProperty::big == aProp)
   {
     mRelativeFontSize++;
     return NS_OK;
   }
-  if (nsIEditProperty::small == aProp)
+  if (nsEditProperty::small == aProp)
   {
     mRelativeFontSize--;
     return NS_OK;
@@ -429,8 +425,8 @@ PRBool TypeInState::IsPropCleared(nsIAtom *aProp,
 }
 
 PRBool TypeInState::FindPropInList(nsIAtom *aProp, 
-                                   const nsString &aAttr,
-                                   nsString *outValue,
+                                   const nsAString &aAttr,
+                                   nsAString *outValue,
                                    nsVoidArray &aList,
                                    PRInt32 &outIndex)
 {
@@ -456,7 +452,7 @@ PRBool TypeInState::FindPropInList(nsIAtom *aProp,
  *    PropItem: helper struct for TypeInState
  *******************************************************************/
 
-PropItem::PropItem(nsIAtom *aTag, const nsString &aAttr, const nsString &aValue) :
+PropItem::PropItem(nsIAtom *aTag, const nsAString &aAttr, const nsAString &aValue) :
  tag(aTag)
 ,attr(aAttr)
 ,value(aValue)
