@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *    Charles Manske (cmanske@netscape.com)
+ *    Daniel Glazman (glazman@netscape.com)
  *
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -40,6 +41,7 @@
 #include "nsString.h"
 
 #include "nsIDOMEvent.h"
+#include "nsIDOMNSEvent.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMMouseEvent.h"
 #include "nsISelection.h"
@@ -53,8 +55,10 @@
 
 #include "nsIEditor.h"
 #include "nsIHTMLEditor.h"
-#include "nsIEditProperty.h"
+#include "nsIHTMLObjectResizer.h"
+#include "nsEditProperty.h"
 #include "nsTextEditUtils.h"
+#include "nsIHTMLInlineTableEditor.h"
 
 /*
  * nsHTMLEditorMouseListener implementation
@@ -63,8 +67,10 @@
  *  moves the caret or selects an element as it does for normal click
  */
 
-nsHTMLEditorMouseListener::nsHTMLEditorMouseListener() 
+nsHTMLEditorMouseListener::nsHTMLEditorMouseListener(nsHTMLEditor *aHTMLEditor)
+  : mHTMLEditor(aHTMLEditor)
 {
+  SetEditor(mHTMLEditor); // Tell the base class about the editor.
 }
 
 nsHTMLEditorMouseListener::~nsHTMLEditorMouseListener() 
@@ -73,6 +79,36 @@ nsHTMLEditorMouseListener::~nsHTMLEditorMouseListener()
 
 NS_IMPL_ISUPPORTS_INHERITED1(nsHTMLEditorMouseListener, nsTextEditorMouseListener, nsIDOMMouseListener)
 
+nsresult
+nsHTMLEditorMouseListener::MouseUp(nsIDOMEvent* aMouseEvent)
+{
+  NS_ENSURE_ARG_POINTER(aMouseEvent);
+  nsCOMPtr<nsIDOMMouseEvent> mouseEvent ( do_QueryInterface(aMouseEvent) );
+  if (!mouseEvent) {
+    //non-ui event passed in.  bad things.
+    return NS_OK;
+  }
+
+  // Don't do anything special if not an HTML editor
+  nsCOMPtr<nsIHTMLEditor> htmlEditor = do_QueryInterface(mEditor);
+  if (htmlEditor)
+  {
+    nsCOMPtr<nsIDOMEventTarget> target;
+    nsresult res = aMouseEvent->GetTarget(getter_AddRefs(target));
+    if (NS_FAILED(res)) return res;
+    if (!target) return NS_ERROR_NULL_POINTER;
+    nsCOMPtr<nsIDOMElement> element = do_QueryInterface(target);
+
+    nsCOMPtr<nsIHTMLObjectResizer> objectResizer = do_QueryInterface(htmlEditor);
+    PRInt32 clientX, clientY;
+    mouseEvent->GetClientX(&clientX);
+    mouseEvent->GetClientY(&clientY);
+    objectResizer->MouseUp(clientX, clientY, element);
+  }
+
+  return nsTextEditorMouseListener::MouseUp(aMouseEvent);
+}
+
 nsresult
 nsHTMLEditorMouseListener::MouseDown(nsIDOMEvent* aMouseEvent)
 {
@@ -82,7 +118,6 @@ nsHTMLEditorMouseListener::MouseDown(nsIDOMEvent* aMouseEvent)
     //non-ui event passed in.  bad things.
     return NS_OK;
   }
-  nsresult res;
 
   // Don't do anything special if not an HTML editor
   nsCOMPtr<nsIHTMLEditor> htmlEditor = do_QueryInterface(mEditor);
@@ -92,7 +127,7 @@ nsHTMLEditorMouseListener::MouseDown(nsIDOMEvent* aMouseEvent)
     //XXX This should be easier to do!
     // But eDOMEvents_contextmenu and NS_CONTEXTMENU is not exposed in any event interface :-(
     PRUint16 buttonNumber;
-    res = mouseEvent->GetButton(&buttonNumber);
+    nsresult res = mouseEvent->GetButton(&buttonNumber);
     if (NS_FAILED(res)) return res;
 
     PRBool isContextClick;
@@ -110,14 +145,15 @@ nsHTMLEditorMouseListener::MouseDown(nsIDOMEvent* aMouseEvent)
     res = mouseEvent->GetDetail(&clickCount);
     if (NS_FAILED(res)) return res;
 
+    nsCOMPtr<nsIDOMEventTarget> target;
+    nsCOMPtr<nsIDOMNSEvent> internalEvent = do_QueryInterface(aMouseEvent);
+    res = internalEvent->GetExplicitOriginalTarget(getter_AddRefs(target));
+    if (NS_FAILED(res)) return res;
+    if (!target) return NS_ERROR_NULL_POINTER;
+    nsCOMPtr<nsIDOMElement> element = do_QueryInterface(target);
+
     if (isContextClick || (buttonNumber == 0 && clickCount == 2))
     {
-      nsCOMPtr<nsIDOMEventTarget> target;
-      res = aMouseEvent->GetTarget(getter_AddRefs(target));
-      if (NS_FAILED(res)) return res;
-      if (!target) return NS_ERROR_NULL_POINTER;
-      nsCOMPtr<nsIDOMElement> element = do_QueryInterface(target);
-
       nsCOMPtr<nsISelection> selection;
       mEditor->GetSelection(getter_AddRefs(selection));
       if (!selection) return NS_OK;
@@ -171,7 +207,6 @@ nsHTMLEditorMouseListener::MouseDown(nsIDOMEvent* aMouseEvent)
       nsCOMPtr<nsIDOMNode> node = do_QueryInterface(target);
       if (node && !nodeIsInSelection)
       {
-        PRBool elementIsLink = PR_FALSE;
         if (!element)
         {
           if (isContextClick)
@@ -200,6 +235,23 @@ nsHTMLEditorMouseListener::MouseDown(nsIDOMEvent* aMouseEvent)
         //   and not the entire body, or table-related elements
         if (element)
         {
+          nsCOMPtr<nsIDOMNode> eleNode = do_QueryInterface(element);
+
+          if (eleNode)
+          {
+            nsCOMPtr<nsIDOMNode> selectAllNode = mHTMLEditor->FindUserSelectAllNode(eleNode);
+
+            if (selectAllNode)
+            {
+              nsCOMPtr<nsIDOMElement> newElement = do_QueryInterface(selectAllNode);
+              if (newElement)
+              {
+                node = selectAllNode;
+                element = newElement;
+              }
+            }
+          }
+
           if (nsTextEditUtils::NodeIsType(node, NS_LITERAL_STRING("body")) ||
               nsTextEditUtils::NodeIsType(node, NS_LITERAL_STRING("td")) ||
               nsTextEditUtils::NodeIsType(node, NS_LITERAL_STRING("th")) ||
@@ -216,6 +268,10 @@ nsHTMLEditorMouseListener::MouseDown(nsIDOMEvent* aMouseEvent)
           }
         }
       }
+      // HACK !!! Context click places the caret but the context menu consumes
+      // the event; so we need to check resizing state ourselves
+      htmlEditor->CheckSelectionStateForAnonymousButtons(selection);
+
       // Prevent bubbling if we changed selection or 
       //   for all context clicks
       if (element || isContextClick)
@@ -224,20 +280,54 @@ nsHTMLEditorMouseListener::MouseDown(nsIDOMEvent* aMouseEvent)
         return NS_OK;
       }
     }
+    else if (!isContextClick && buttonNumber == 0 && clickCount == 1)
+    {
+      // if the target element is an image, we have to display resizers
+      nsCOMPtr<nsIHTMLObjectResizer> objectResizer = do_QueryInterface(htmlEditor);
+      PRInt32 clientX, clientY;
+      mouseEvent->GetClientX(&clientX);
+      mouseEvent->GetClientY(&clientY);
+      objectResizer->MouseDown(clientX, clientY, element);
+    }
   }
 
   return nsTextEditorMouseListener::MouseDown(aMouseEvent);
 }
 
 nsresult
-NS_NewHTMLEditorMouseListener(nsIDOMEventListener ** aInstancePtrResult, 
-                              nsIEditor *aEditor)
+nsHTMLEditorMouseListener::MouseClick(nsIDOMEvent* aMouseEvent)
 {
-  nsHTMLEditorMouseListener* listener = new nsHTMLEditorMouseListener();
+  NS_ENSURE_ARG_POINTER(aMouseEvent);
+  nsCOMPtr<nsIDOMMouseEvent> mouseEvent ( do_QueryInterface(aMouseEvent) );
+  if (!mouseEvent) {
+    //non-ui event passed in.  bad things.
+    return NS_OK;
+  }
+
+  // Don't do anything special if not an HTML editor
+  nsCOMPtr<nsIHTMLEditor> htmlEditor = do_QueryInterface(mEditor);
+  if (htmlEditor)
+  {
+    nsCOMPtr<nsIDOMEventTarget> target;
+    nsresult res = aMouseEvent->GetTarget(getter_AddRefs(target));
+    if (NS_FAILED(res)) return res;
+    if (!target) return NS_ERROR_NULL_POINTER;
+    nsCOMPtr<nsIDOMElement> element = do_QueryInterface(target);
+
+    nsCOMPtr<nsIHTMLInlineTableEditor> inlineTableEditing = do_QueryInterface(htmlEditor);
+    inlineTableEditing->DoInlineTableEditingAction(element);
+  }
+
+  return nsTextEditorMouseListener::MouseClick(aMouseEvent);
+}
+
+nsresult
+NS_NewHTMLEditorMouseListener(nsIDOMEventListener ** aInstancePtrResult, 
+                              nsHTMLEditor *aHTMLEditor)
+{
+  nsHTMLEditorMouseListener* listener = new nsHTMLEditorMouseListener(aHTMLEditor);
   if (!listener)
     return NS_ERROR_OUT_OF_MEMORY;
 
-  listener->SetEditor(aEditor);
-
   return listener->QueryInterface(NS_GET_IID(nsIDOMEventListener), (void **) aInstancePtrResult);   
 }
