@@ -48,36 +48,13 @@
 
 nsAOLCiter::nsAOLCiter()
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsAOLCiter::~nsAOLCiter()
 {
 }
 
-NS_IMPL_ADDREF(nsAOLCiter)
-
-NS_IMPL_RELEASE(nsAOLCiter)
-
-NS_IMETHODIMP
-nsAOLCiter::QueryInterface(REFNSIID aIID, void** aInstancePtr)
-{
-  if (nsnull == aInstancePtr) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  if (aIID.Equals(NS_GET_IID(nsISupports)))
-  {
-    *aInstancePtr = (void*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsICiter))) {
-    *aInstancePtr = (void*)(nsICiter*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  return NS_NOINTERFACE;
-}
+NS_IMPL_ISUPPORTS1(nsAOLCiter, nsICiter)
 
 NS_IMETHODIMP
 nsAOLCiter::GetCiteString(const nsAString& aInString, nsAString& aOutString)
