@@ -66,6 +66,7 @@
 #include "nsILookAndFeel.h"
 #include "nsIPresContext.h"
 #include "nsGUIEvent.h"
+#include "nsICaret.h"
 // for repainting hack only
 #include "nsIView.h"
 #include "nsIViewManager.h"
@@ -73,7 +74,6 @@
 
 // Drag & Drop, Clipboard
 #include "nsIServiceManager.h"
-#include "nsWidgetsCID.h"
 #include "nsIClipboard.h"
 #include "nsIDragService.h"
 #include "nsIDragSession.h"
@@ -83,11 +83,10 @@
 #include "nsIContent.h"
 #include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
-#include "nsLayoutCID.h"
 #include "nsIDOMNSRange.h"
-
-// Drag & Drop, Clipboard Support
-static NS_DEFINE_CID(kLookAndFeelCID,          NS_LOOKANDFEEL_CID);
+#include "nsEditorUtils.h"
+#include "nsIDOMEventTarget.h"
+#include "nsIEventStateManager.h"
 
 //#define DEBUG_IME
 
@@ -95,14 +94,11 @@ static NS_DEFINE_CID(kLookAndFeelCID,          NS_LOOKANDFEEL_CID);
  * nsTextEditorKeyListener implementation
  */
 
-NS_IMPL_ADDREF(nsTextEditorKeyListener)
-
-NS_IMPL_RELEASE(nsTextEditorKeyListener)
+NS_IMPL_ISUPPORTS2(nsTextEditorKeyListener, nsIDOMEventListener, nsIDOMKeyListener)
 
 
 nsTextEditorKeyListener::nsTextEditorKeyListener()
 {
-  NS_INIT_ISUPPORTS();
 }
 
 
@@ -112,33 +108,6 @@ nsTextEditorKeyListener::~nsTextEditorKeyListener()
 }
 
 
-
-nsresult
-nsTextEditorKeyListener::QueryInterface(REFNSIID aIID, void** aInstancePtr)
-{
-  if (nsnull == aInstancePtr) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  if (aIID.Equals(NS_GET_IID(nsISupports))) {
-    *aInstancePtr = (void*)(nsISupports*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsIDOMEventListener))) {
-    *aInstancePtr = (void*)(nsIDOMEventListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsIDOMKeyListener))) {
-    *aInstancePtr = (void*)(nsIDOMKeyListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  return NS_NOINTERFACE;
-}
-
-
-
 nsresult
 nsTextEditorKeyListener::HandleEvent(nsIDOMEvent* aEvent)
 {
@@ -173,6 +142,12 @@ nsTextEditorKeyListener::KeyPress(nsIDOMEvent* aKeyEvent)
     return NS_OK;
   }
   
+  // DOM event handling happens in two passes, the client pass and the system
+  // pass.  We do all of our processing in the system pass, to allow client
+  // handlers the opporunity to cancel events and prevent typing in the editor.
+  // If the client pass cancelled the event, defaultPrevented will be true
+  // below.
+
   nsCOMPtr<nsIDOMNSUIEvent> nsUIEvent = do_QueryInterface(aKeyEvent);
   if(nsUIEvent) 
   {
@@ -182,22 +157,6 @@ nsTextEditorKeyListener::KeyPress(nsIDOMEvent* aKeyEvent)
       return NS_OK;
   }
 
-  // DOM event handling happens in two passes, the client pass and the system
-  // pass.  All system handlers need to move eventually to the system pass
-  // but due to inconsistencies that would create with XBL handlers they
-  // need to stay here for now and continue to process command keys.  Character
-  // generation, however, should be handled in the second system pass to allow
-  // proper cancellation of character generating events.
-  PRBool isSystemPass = PR_FALSE;
-  nsCOMPtr<nsIPrivateDOMEvent> privDOMEvent(do_QueryInterface(aKeyEvent));
-  if (privDOMEvent) {
-    nsEvent* innerEvent;
-    privDOMEvent->GetInternalNSEvent(&innerEvent);
-    if (innerEvent) {
-      isSystemPass = innerEvent->flags & NS_EVENT_FLAG_SYSTEM_EVENT;
-    }
-  }
-
   // we should check a flag here to see if we should be using built-in key bindings
   // mEditor->GetFlags(&flags);
   // if (flags & ...)
@@ -221,8 +180,7 @@ nsTextEditorKeyListener::KeyPress(nsIDOMEvent* aKeyEvent)
 
   // if there is no charCode, then it's a key that doesn't map to a character,
   // so look for special keys using keyCode.
-  // Also, process these keys only during the Client Pass of the event loop.
-  if (0 != keyCode && !isSystemPass)
+  if (0 != keyCode)
   {
     PRBool isAnyModifierKeyButShift;
     nsresult rv;
@@ -286,7 +244,7 @@ nsTextEditorKeyListener::KeyPress(nsIDOMEvent* aKeyEvent)
 
         // else we insert the tab straight through
         textEditor->HandleKeyPress(keyEvent);
-        aKeyEvent->PreventDefault(); // consumed
+        // let HandleKeyPress consume the event
         return NS_OK; 
 
       case nsIDOMKeyEvent::DOM_VK_RETURN:
@@ -304,7 +262,6 @@ nsTextEditorKeyListener::KeyPress(nsIDOMEvent* aKeyEvent)
   }
 
   textEditor->HandleKeyPress(keyEvent);
-
   return NS_OK; // we don't PreventDefault() here or keybindings like control-x won't work 
 }
 
@@ -321,14 +278,11 @@ nsTextEditorKeyListener::ProcessShortCutKeys(nsIDOMEvent* aKeyEvent, PRBool& aPr
  * nsTextEditorMouseListener implementation
  */
 
-NS_IMPL_ADDREF(nsTextEditorMouseListener)
-
-NS_IMPL_RELEASE(nsTextEditorMouseListener)
+NS_IMPL_ISUPPORTS2(nsTextEditorMouseListener, nsIDOMEventListener, nsIDOMMouseListener)
 
 
 nsTextEditorMouseListener::nsTextEditorMouseListener() 
 {
-  NS_INIT_ISUPPORTS();
 }
 
 
@@ -338,34 +292,6 @@ nsTextEditorMouseListener::~nsTextEditorMouseListener()
 }
 
 
-
-nsresult
-nsTextEditorMouseListener::QueryInterface(REFNSIID aIID, void** aInstancePtr)
-{
-  if (nsnull == aInstancePtr) {
-    return NS_ERROR_NULL_POINTER;
-  }
-
-  if (aIID.Equals(NS_GET_IID(nsISupports))) {
-    *aInstancePtr = (void*)(nsISupports*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsIDOMEventListener))) {
-    *aInstancePtr = (void*)(nsIDOMEventListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsIDOMMouseListener))) {
-    *aInstancePtr = (void*)(nsIDOMMouseListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  return NS_NOINTERFACE;
-}
-
-
-
 nsresult
 nsTextEditorMouseListener::HandleEvent(nsIDOMEvent* aEvent)
 {
@@ -505,49 +431,20 @@ nsTextEditorMouseListener::MouseOut(nsIDOMEvent* aMouseEvent)
  * nsTextEditorTextListener implementation
  */
 
-NS_IMPL_ADDREF(nsTextEditorTextListener)
-
-NS_IMPL_RELEASE(nsTextEditorTextListener)
+NS_IMPL_ISUPPORTS2(nsTextEditorTextListener, nsIDOMEventListener, nsIDOMTextListener)
 
 
 nsTextEditorTextListener::nsTextEditorTextListener()
 :   mCommitText(PR_FALSE),
    mInTransaction(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 
-
 nsTextEditorTextListener::~nsTextEditorTextListener() 
 {
 }
 
-nsresult
-nsTextEditorTextListener::QueryInterface(REFNSIID aIID, void** aInstancePtr)
-{
-  if (nsnull == aInstancePtr) {
-    return NS_ERROR_NULL_POINTER;
-  }
-
-  if (aIID.Equals(NS_GET_IID(nsISupports))) {
-    *aInstancePtr = (void*)(nsISupports*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsIDOMEventListener))) {
-    *aInstancePtr = (void*)(nsIDOMEventListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsIDOMTextListener))) {
-    *aInstancePtr = (void*)(nsIDOMTextListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  return NS_NOINTERFACE;
-}
-
 nsresult
 nsTextEditorTextListener::HandleEvent(nsIDOMEvent* aEvent)
 {
@@ -603,46 +500,18 @@ nsTextEditorTextListener::HandleText(nsIDOMEvent* aTextEvent)
  * nsTextEditorDragListener implementation
  */
 
-NS_IMPL_ADDREF(nsTextEditorDragListener)
-
-NS_IMPL_RELEASE(nsTextEditorDragListener)
-
-
 nsTextEditorDragListener::nsTextEditorDragListener() 
+: mEditor(nsnull)
+, mPresShell(nsnull)
+, mCaretDrawn(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsTextEditorDragListener::~nsTextEditorDragListener() 
 {
 }
 
-nsresult
-nsTextEditorDragListener::QueryInterface(REFNSIID aIID, void** aInstancePtr)
-{
-  if (nsnull == aInstancePtr) {
-    return NS_ERROR_NULL_POINTER;
-  }
-
-  if (aIID.Equals(NS_GET_IID(nsISupports))) {
-    *aInstancePtr = (void*)(nsISupports*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsIDOMEventListener))) {
-    *aInstancePtr = (void*)(nsIDOMEventListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsIDOMDragListener))) {
-    *aInstancePtr = (void*)(nsIDOMDragListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  return NS_NOINTERFACE;
-}
-
-
+NS_IMPL_ISUPPORTS2(nsTextEditorDragListener, nsIDOMEventListener, nsIDOMDragListener);
 
 nsresult
 nsTextEditorDragListener::HandleEvent(nsIDOMEvent* aEvent)
@@ -670,6 +539,20 @@ nsTextEditorDragListener::DragGesture(nsIDOMEvent* aDragEvent)
 nsresult
 nsTextEditorDragListener::DragEnter(nsIDOMEvent* aDragEvent)
 {
+  if (mPresShell)
+  {
+    if (!mCaret)
+    {
+      mCaret = do_CreateInstance("@mozilla.org/layout/caret;1");
+      if (mCaret)
+      {
+        mCaret->Init(mPresShell);
+        mCaret->SetCaretReadOnly(PR_TRUE);
+      }
+      mCaretDrawn = PR_FALSE;
+    }
+  }
+  
   return DragOver(aDragEvent);
 }
 
@@ -677,124 +560,37 @@ nsTextEditorDragListener::DragEnter(nsIDOMEvent* aDragEvent)
 nsresult
 nsTextEditorDragListener::DragOver(nsIDOMEvent* aDragEvent)
 {
+  // XXX cache this between drag events?
   nsresult rv;
-  nsCOMPtr<nsIDragService> dragService = 
-           do_GetService( "@mozilla.org/widget/dragservice;1", &rv );
-  if ( NS_SUCCEEDED(rv) ) {
-    nsCOMPtr<nsIDragSession> dragSession;
-    dragService->GetCurrentSession(getter_AddRefs(dragSession));
-    if ( dragSession ) {
-      PRUint32 flags;
-      if (NS_SUCCEEDED(mEditor->GetFlags(&flags))) {
-        if ((flags & nsIPlaintextEditor::eEditorDisabledMask) || 
-            (flags & nsIPlaintextEditor::eEditorReadonlyMask)) {
-          dragSession->SetCanDrop(PR_FALSE);
-          aDragEvent->PreventDefault(); // consumed
-          return NS_OK;
-        }
-      }
-      PRBool flavorSupported = PR_FALSE;
-      dragSession->IsDataFlavorSupported(kUnicodeMime, &flavorSupported);
-      if ( !flavorSupported ) 
-        dragSession->IsDataFlavorSupported(kHTMLMime, &flavorSupported);
-      if ( !flavorSupported ) 
-        dragSession->IsDataFlavorSupported(kFileMime, &flavorSupported);
-      if ( !flavorSupported ) 
-        dragSession->IsDataFlavorSupported(kJPEGImageMime, &flavorSupported);
-      if ( flavorSupported ) {
-        dragSession->SetCanDrop(PR_TRUE);
-        aDragEvent->PreventDefault();
-      }
-    } 
+  nsCOMPtr<nsIDragService> dragService = do_GetService("@mozilla.org/widget/dragservice;1", &rv);
+  if (!dragService) return rv;
+
+  // does the drag have flavors we can accept?
+  nsCOMPtr<nsIDragSession> dragSession;
+  dragService->GetCurrentSession(getter_AddRefs(dragSession));
+  if (!dragSession) return NS_ERROR_FAILURE;
+
+  PRBool canDrop = CanDrop(aDragEvent);
+  if (canDrop)
+  {
+    nsCOMPtr<nsIDOMDocument> domdoc;
+    mEditor->GetDocument(getter_AddRefs(domdoc));
+    canDrop = nsEditorHookUtils::DoAllowDropHook(domdoc, aDragEvent, dragSession);
   }
 
-  return NS_OK;
-}
+  dragSession->SetCanDrop(canDrop);
 
+  // We need to consume the event to prevent the browser's
+  // default drag listeners from being fired. (Bug 199133)
 
-nsresult
-nsTextEditorDragListener::DragExit(nsIDOMEvent* aDragEvent)
-{
-  return NS_OK;
-}
-
-
-
-nsresult
-nsTextEditorDragListener::DragDrop(nsIDOMEvent* aMouseEvent)
-{
-  if ( mEditor )
-  {
-    nsresult rv;
-    nsCOMPtr<nsIDragService> dragService = 
-             do_GetService("@mozilla.org/widget/dragservice;1", &rv);
-    if (NS_FAILED(rv)) return rv;
-
-    PRUint32 flags;
-    if (NS_SUCCEEDED(mEditor->GetFlags(&flags))
-        && ((flags & nsIPlaintextEditor::eEditorDisabledMask)
-           || (flags & nsIPlaintextEditor::eEditorReadonlyMask)) )
-    {
-      return aMouseEvent->StopPropagation(); // it was decided to "eat" the event as this is the "least surprise"
-                                      // since someone else handling it might be unintentional and the 
-                                      // user could probably re-drag to be not over the disabled/readonly 
-                                      // editfields if that is what is desired.
-    }
-
-    nsCOMPtr<nsIDragSession> dragSession;
-    dragService->GetCurrentSession(getter_AddRefs(dragSession));
-    if (dragSession)
-    {
-       PRBool flavorSupported = PR_FALSE;
-      dragSession->IsDataFlavorSupported(kUnicodeMime, &flavorSupported);
-      if ( !flavorSupported ) 
-        dragSession->IsDataFlavorSupported(kHTMLMime, &flavorSupported);
-      if ( !flavorSupported ) 
-        dragSession->IsDataFlavorSupported(kFileMime, &flavorSupported);
-      if ( !flavorSupported ) 
-        dragSession->IsDataFlavorSupported(kJPEGImageMime, &flavorSupported);
-      if (! flavorSupported ) 
-        return NS_OK;     
-    }
-
-    nsCOMPtr<nsIDOMNSUIEvent> nsuiEvent (do_QueryInterface(aMouseEvent));
-    if (!nsuiEvent) return NS_OK;
-
-    //some day we want to use another way to stop this from bubbling.
-    nsCOMPtr<nsIDOMNSEvent> nsevent(do_QueryInterface(aMouseEvent));
-
-    if (nsevent) {
-      nsevent->PreventBubble();
-    }
-
-    aMouseEvent->PreventDefault();
-
-    /* for bug 47399, when dropping a drag session, if you are over your original
-       selection, nothing should happen. 
-       cmanske: But do this only if drag source is not the same as target (current) document!
-    */
-    nsCOMPtr<nsISelection> selection;
-    rv = mEditor->GetSelection(getter_AddRefs(selection));
-    if (NS_FAILED(rv))
-      return rv;
-    if (!selection)
-      return NS_ERROR_FAILURE;
+  aDragEvent->PreventDefault(); // consumed
     
-    nsCOMPtr<nsIDOMDocument> domdoc;
-    rv = mEditor->GetDocument(getter_AddRefs(domdoc));
-    if (NS_FAILED(rv)) return rv;
-
-    nsCOMPtr<nsIDOMDocument> sourceDoc;
-    rv = dragSession->GetSourceDocument(getter_AddRefs(sourceDoc));
-    if (NS_FAILED(rv)) return rv;
-    if (domdoc == sourceDoc)
+  if (canDrop)
+  {
+    if (mCaret)
     {
-      PRBool isCollapsed;
-      rv = selection->GetIsCollapsed(&isCollapsed);
-      if (NS_FAILED(rv)) return rv;
-  
-      // Don't bother if collapsed - can always drop
-      if (!isCollapsed)
+      nsCOMPtr<nsIDOMNSUIEvent> nsuiEvent (do_QueryInterface(aDragEvent));
+      if (nsuiEvent)
       {
         nsCOMPtr<nsIDOMNode> parent;
         rv = nsuiEvent->GetRangeParent(getter_AddRefs(parent));
@@ -805,74 +601,196 @@ nsTextEditorDragListener::DragDrop(nsIDOMEvent* aMouseEvent)
         rv = nsuiEvent->GetRangeOffset(&offset);
         if (NS_FAILED(rv)) return rv;
 
-        PRInt32 rangeCount;
-        rv = selection->GetRangeCount(&rangeCount);
-        if (NS_FAILED(rv)) return rv;
-
-        for (PRInt32 i = 0; i < rangeCount; i++)
-        {
-          nsCOMPtr<nsIDOMRange> range;
-
-          rv = selection->GetRangeAt(i, getter_AddRefs(range));
-          if (NS_FAILED(rv) || !range) 
-            continue;//dont bail yet, iterate through them all
-
-          nsCOMPtr<nsIDOMNSRange> nsrange(do_QueryInterface(range));
-          if (NS_FAILED(rv) || !nsrange) 
-            continue;//dont bail yet, iterate through them all
-
-          PRBool inrange;
-          rv = nsrange->IsPointInRange(parent, offset, &inrange);
-          if(inrange)
-            return NS_ERROR_FAILURE;//okay, now you can bail, we are over the orginal selection
-        }
+        // to avoid flicker, we could track the node and offset to see if we moved
+        if (mCaretDrawn)
+          mCaret->EraseCaret();
+        
+        //mCaret->SetCaretVisible(PR_TRUE);   // make sure it's visible
+        mCaret->DrawAtPosition(parent, offset);
+        mCaretDrawn = PR_TRUE;
       }
     }
-    // if we are not over orginal selection, drop that baby!
-    return mEditor->InsertFromDrop(aMouseEvent);
+  }
+  else
+  {
+    if (mCaret && mCaretDrawn)
+    {
+      mCaret->EraseCaret();
+      mCaretDrawn = PR_FALSE;
+    } 
   }
 
   return NS_OK;
 }
 
 
+nsresult
+nsTextEditorDragListener::DragExit(nsIDOMEvent* aDragEvent)
+{
+  if (mCaret)
+  {
+    if (mCaretDrawn)
+      mCaret->EraseCaret();
+    mCaretDrawn = PR_FALSE;
+  }
+
+  return NS_OK;
+}
+
+
+
+nsresult
+nsTextEditorDragListener::DragDrop(nsIDOMEvent* aMouseEvent)
+{
+  if (mCaret)
+  {
+    if (mCaretDrawn)
+      mCaret->EraseCaret();
+    mCaretDrawn = PR_FALSE;
+    mCaret->SetCaretVisible(PR_FALSE);    // hide it, so that it turns off its timer
+    mCaret = nsnull;      // release it
+  }
+
+  if (!mEditor)
+    return NS_ERROR_FAILURE;
+
+  PRBool canDrop = CanDrop(aMouseEvent);
+  if (!canDrop)
+  {
+    // was it because we're read-only?
+
+    PRUint32 flags;
+    if (NS_SUCCEEDED(mEditor->GetFlags(&flags))
+        && ((flags & nsIPlaintextEditor::eEditorDisabledMask) ||
+            (flags & nsIPlaintextEditor::eEditorReadonlyMask)) )
+    {
+      // it was decided to "eat" the event as this is the "least surprise"
+                                      // since someone else handling it might be unintentional and the 
+                                      // user could probably re-drag to be not over the disabled/readonly 
+                                      // editfields if that is what is desired.
+      return aMouseEvent->StopPropagation();
+    }
+    return NS_OK;
+  }
+
+  //some day we want to use another way to stop this from bubbling.
+  nsCOMPtr<nsIDOMNSEvent> nsevent(do_QueryInterface(aMouseEvent));
+  if (nsevent)
+    nsevent->PreventBubble();
+
+  aMouseEvent->PreventDefault();
+  return mEditor->InsertFromDrop(aMouseEvent);
+}
+
+PRBool
+nsTextEditorDragListener::CanDrop(nsIDOMEvent* aEvent)
+{
+  // if the target doc is read-only, we can't drop
+  PRUint32 flags = 0;
+  nsresult rv = mEditor->GetFlags(&flags);
+  if (NS_FAILED(rv)) return PR_FALSE;
+
+  if ((flags & nsIPlaintextEditor::eEditorDisabledMask) || 
+      (flags & nsIPlaintextEditor::eEditorReadonlyMask)) {
+    return PR_FALSE;
+  }
+
+  // XXX cache this between drag events?
+  nsCOMPtr<nsIDragService> dragService = do_GetService("@mozilla.org/widget/dragservice;1", &rv);
+  if (!dragService) return PR_FALSE;
+
+  // does the drag have flavors we can accept?
+  nsCOMPtr<nsIDragSession> dragSession;
+  dragService->GetCurrentSession(getter_AddRefs(dragSession));
+  if (!dragSession) return PR_FALSE;
+
+  PRBool flavorSupported = PR_FALSE;
+  dragSession->IsDataFlavorSupported(kUnicodeMime, &flavorSupported);
+
+  // if we aren't plaintext editing, we can accept more flavors
+  if (!flavorSupported 
+     && (flags & nsIPlaintextEditor::eEditorPlaintextMask) == 0)
+  {
+    dragSession->IsDataFlavorSupported(kHTMLMime, &flavorSupported);
+    if (!flavorSupported)
+      dragSession->IsDataFlavorSupported(kFileMime, &flavorSupported);
+#if 0
+    if (!flavorSupported)
+      dragSession->IsDataFlavorSupported(kJPEGImageMime, &flavorSupported);
+#endif
+  }
+
+  if (!flavorSupported)
+    return PR_FALSE;     
+
+  nsCOMPtr<nsISelection> selection;
+  rv = mEditor->GetSelection(getter_AddRefs(selection));
+  if (NS_FAILED(rv) || !selection)
+    return PR_FALSE;
+    
+  nsCOMPtr<nsIDOMDocument> domdoc;
+  rv = mEditor->GetDocument(getter_AddRefs(domdoc));
+  if (NS_FAILED(rv)) return PR_FALSE;
+
+  nsCOMPtr<nsIDOMDocument> sourceDoc;
+  rv = dragSession->GetSourceDocument(getter_AddRefs(sourceDoc));
+  if (NS_FAILED(rv)) return PR_FALSE;
+  if (domdoc == sourceDoc)      // source and dest are the same document; disallow drops within the selection
+  {
+    PRBool isCollapsed;
+    rv = selection->GetIsCollapsed(&isCollapsed);
+    if (NS_FAILED(rv)) return PR_FALSE;
+  
+    // Don't bother if collapsed - can always drop
+    if (!isCollapsed)
+    {
+      nsCOMPtr<nsIDOMNSUIEvent> nsuiEvent (do_QueryInterface(aEvent));
+      if (!nsuiEvent) return PR_FALSE;
+
+      nsCOMPtr<nsIDOMNode> parent;
+      rv = nsuiEvent->GetRangeParent(getter_AddRefs(parent));
+      if (NS_FAILED(rv) || !parent) return PR_FALSE;
+
+      PRInt32 offset = 0;
+      rv = nsuiEvent->GetRangeOffset(&offset);
+      if (NS_FAILED(rv)) return PR_FALSE;
+
+      PRInt32 rangeCount;
+      rv = selection->GetRangeCount(&rangeCount);
+      if (NS_FAILED(rv)) return PR_FALSE;
+
+      for (PRInt32 i = 0; i < rangeCount; i++)
+      {
+        nsCOMPtr<nsIDOMRange> range;
+        rv = selection->GetRangeAt(i, getter_AddRefs(range));
+        if (NS_FAILED(rv) || !range) 
+        continue; //dont bail yet, iterate through them all
+
+        nsCOMPtr<nsIDOMNSRange> nsrange(do_QueryInterface(range));
+        if (NS_FAILED(rv) || !nsrange) 
+        continue; //dont bail yet, iterate through them all
+
+        PRBool inRange = PR_TRUE;
+        (void)nsrange->IsPointInRange(parent, offset, &inRange);
+        if (inRange)
+          return PR_FALSE;  //okay, now you can bail, we are over the orginal selection
+      }
+    }
+  }
+  
+  return PR_TRUE;
+}
+
+
 nsTextEditorCompositionListener::nsTextEditorCompositionListener()
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsTextEditorCompositionListener::~nsTextEditorCompositionListener() 
 {
 }
 
-
-nsresult
-nsTextEditorCompositionListener::QueryInterface(REFNSIID aIID, void** aInstancePtr)
-{
-  if (nsnull == aInstancePtr) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  if (aIID.Equals(NS_GET_IID(nsISupports))) {
-    *aInstancePtr = (void*)(nsISupports*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsIDOMEventListener))) {
-    *aInstancePtr = (void*)(nsIDOMEventListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsIDOMCompositionListener))) {
-    *aInstancePtr = (void*)(nsIDOMCompositionListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  return NS_NOINTERFACE;
-}
-
-NS_IMPL_ADDREF(nsTextEditorCompositionListener)
-
-NS_IMPL_RELEASE(nsTextEditorCompositionListener)
+NS_IMPL_ISUPPORTS2(nsTextEditorCompositionListener, nsIDOMEventListener, nsIDOMCompositionListener)
 
 nsresult
 nsTextEditorCompositionListener::HandleEvent(nsIDOMEvent* aEvent)
@@ -1007,7 +925,7 @@ NS_NewEditorTextListener(nsIDOMEventListener** aInstancePtrResult, nsIEditor* aE
 
 
 nsresult
-NS_NewEditorDragListener(nsIDOMEventListener ** aInstancePtrResult, 
+NS_NewEditorDragListener(nsIDOMEventListener ** aInstancePtrResult, nsIPresShell* aPresShell,
                           nsIEditor *aEditor)
 {
   nsTextEditorDragListener* it = new nsTextEditorDragListener();
@@ -1016,6 +934,7 @@ NS_NewEditorDragListener(nsIDOMEventListener ** aInstancePtrResult,
   }
 
   it->SetEditor(aEditor);
+  it->SetPresShell(aPresShell);
 
   return it->QueryInterface(NS_GET_IID(nsIDOMEventListener), (void **) aInstancePtrResult);   
 }
@@ -1049,56 +968,78 @@ NS_NewEditorFocusListener(nsIDOMEventListener ** aInstancePtrResult,
  * nsTextEditorFocusListener implementation
  */
 
-NS_IMPL_ADDREF(nsTextEditorFocusListener)
-
-NS_IMPL_RELEASE(nsTextEditorFocusListener)
+NS_IMPL_ISUPPORTS2(nsTextEditorFocusListener, nsIDOMEventListener, nsIDOMFocusListener)
 
 
 nsTextEditorFocusListener::nsTextEditorFocusListener() 
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsTextEditorFocusListener::~nsTextEditorFocusListener() 
 {
 }
 
-nsresult
-nsTextEditorFocusListener::QueryInterface(REFNSIID aIID, void** aInstancePtr)
-{
-  if (nsnull == aInstancePtr) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  static NS_DEFINE_IID(kIDOMFocusListenerIID, NS_IDOMFOCUSLISTENER_IID);
-  static NS_DEFINE_IID(kIDOMEventListenerIID, NS_IDOMEVENTLISTENER_IID);
-  static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
-  if (aIID.Equals(kISupportsIID)) {
-    *aInstancePtr = (void*)(nsISupports*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(kIDOMEventListenerIID)) {
-    *aInstancePtr = (void*)(nsIDOMEventListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(kIDOMFocusListenerIID)) {
-    *aInstancePtr = (void*)(nsIDOMFocusListener*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  return NS_NOINTERFACE;
-}
-
 nsresult
 nsTextEditorFocusListener::HandleEvent(nsIDOMEvent* aEvent)
 {
   return NS_OK;
 }
 
+static PRBool
+IsTargetFocused(nsIDOMEventTarget* aTarget)
+{
+  // The event target could be either a content node or a document.
+  nsCOMPtr<nsIDocument> doc;
+  nsCOMPtr<nsIContent> content = do_QueryInterface(aTarget);
+  if (content)
+    content->GetDocument(getter_AddRefs(doc));
+  else
+    doc = do_QueryInterface(aTarget);
+
+  if (!doc)
+    return PR_FALSE;
+
+  nsCOMPtr<nsIPresShell> shell;
+  doc->GetShellAt(0, getter_AddRefs(shell));
+  if (!shell)
+    return PR_FALSE;
+
+  nsCOMPtr<nsIPresContext> presContext;
+  shell->GetPresContext(getter_AddRefs(presContext));
+  if (!presContext)
+    return PR_FALSE;
+
+  nsCOMPtr<nsIEventStateManager> esm;
+  presContext->GetEventStateManager(getter_AddRefs(esm));
+  if (!esm)
+    return PR_FALSE;
+
+  nsCOMPtr<nsIContent> focusedContent;
+  esm->GetFocusedContent(getter_AddRefs(focusedContent));
+
+  // focusedContent will be null in the case where the document has focus,
+  // and so will content.
+
+  return (focusedContent == content);
+}
+
 nsresult
 nsTextEditorFocusListener::Focus(nsIDOMEvent* aEvent)
 {
+  // It's possible for us to receive a focus when we're really not focused.
+  // This happens, for example, when an onfocus handler that's hooked up
+  // before this listener focuses something else.  In that case, all of the
+  // onblur handlers will be fired synchronously, then the remaining focus
+  // handlers will be fired from the original event.  So, check to see that
+  // we're really focused.  (Note that the analogous situation does not
+  // happen for blurs, due to the ordering in
+  // nsEventStateManager::SendFocuBlur().
+
+  nsCOMPtr<nsIDOMEventTarget> target;
+  aEvent->GetTarget(getter_AddRefs(target));
+  if (!IsTargetFocused(target))
+    return NS_OK;
+
   // turn on selection and caret
   if (mEditor)
   {
@@ -1126,7 +1067,7 @@ nsTextEditorFocusListener::Focus(nsIDOMEvent* aEvent)
             nsresult result;
 
             nsCOMPtr<nsILookAndFeel> look = 
-                     do_GetService(kLookAndFeelCID, &result);
+                     do_GetService("@mozilla.org/widget/lookandfeel;1", &result);
 
             if (NS_SUCCEEDED(result) && look)
             {
