@@ -55,7 +55,6 @@
 #include "nsIContent.h"
 #include "nsIContentIterator.h"
 #include "nsIEnumerator.h"
-#include "nsLayoutCID.h"
 #include "nsEditorUtils.h"
 #include "EditTxn.h"
 #include "nsIPrefBranch.h"
@@ -67,9 +66,6 @@
 #include "nsIPresShell.h"
 #include "nsIPresContext.h"
 
-static NS_DEFINE_CID(kContentIteratorCID,   NS_CONTENTITERATOR_CID);
-static NS_DEFINE_IID(kRangeCID, NS_RANGE_CID);
-
 #define CANCEL_OPERATION_IF_READONLY_OR_DISABLED \
   if ((mFlags & nsIPlaintextEditor::eEditorReadonlyMask) || (mFlags & nsIPlaintextEditor::eEditorDisabledMask)) \
   {                     \
@@ -102,6 +98,7 @@ nsTextEditRules::nsTextEditRules()
 , mFlags(0) // initialized to 0 ("no flags set").  Real initial value is given in Init()
 , mActionNesting(0)
 , mLockRulesSniffing(PR_FALSE)
+, mDidExplicitlySetInterline(PR_FALSE)
 , mTheAction(0)
 {
 }
@@ -115,9 +112,7 @@ nsTextEditRules::~nsTextEditRules()
  *  XPCOM Cruft
  ********************************************************/
 
-NS_IMPL_ADDREF(nsTextEditRules)
-NS_IMPL_RELEASE(nsTextEditRules)
-NS_IMPL_QUERY_INTERFACE1(nsTextEditRules, nsIEditRules)
+NS_IMPL_ISUPPORTS1(nsTextEditRules, nsIEditRules)
 
 
 /********************************************************
@@ -156,7 +151,7 @@ nsTextEditRules::Init(nsPlaintextEditor *aEditor, PRUint32 aFlags)
   }
   
   // create a range that is the entire body contents
-  nsCOMPtr<nsIDOMRange> wholeDoc = do_CreateInstance(kRangeCID);
+  nsCOMPtr<nsIDOMRange> wholeDoc = do_CreateInstance("@mozilla.org/content/range;1");
   if (!wholeDoc) return NS_ERROR_NULL_POINTER;
   wholeDoc->SetStart(mBody,0);
   nsCOMPtr<nsIDOMNodeList> list;
@@ -202,6 +197,7 @@ nsTextEditRules::BeforeEdit(PRInt32 action, nsIEditor::EDirection aDirection)
   if (mLockRulesSniffing) return NS_OK;
   
   nsAutoLockRulesSniffing lockIt(this);
+  mDidExplicitlySetInterline = PR_FALSE;
   
   if (!mActionNesting)
   {
@@ -904,7 +900,7 @@ nsTextEditRules::WillDeleteSelection(nsISelection *aSelection,
       if (!content) return NS_ERROR_NULL_POINTER;
       if (aCollapsedAction == nsIEditor::ePrevious)
         --startOffset;
-      res = content->ChildAt(startOffset, *getter_AddRefs(child));
+      res = content->ChildAt(startOffset, getter_AddRefs(child));
       if (NS_FAILED(res)) return res;
       nextNode = do_QueryInterface(child);
       
@@ -926,7 +922,7 @@ nsTextEditRules::WillDeleteSelection(nsISelection *aSelection,
         if (aCollapsedAction == nsIEditor::ePrevious)
           --startOffset;
           // don't need to increment startOffset for nsIEditor::eNext
-        res = content->ChildAt(startOffset, *getter_AddRefs(child));
+        res = content->ChildAt(startOffset, getter_AddRefs(child));
         if (NS_FAILED(res)) return res;
         nextNode = do_QueryInterface(child);
       }
@@ -977,11 +973,13 @@ nsTextEditRules::DidDeleteSelection(nsISelection *aSelection,
       if (NS_FAILED(res)) return res;
     }
   }
-  // We prevent the caret from sticking on the left of prior BR
-  // (i.e. the end of previous line) after this deletion.  Bug 92124
-  nsCOMPtr<nsISelectionPrivate> selPriv = do_QueryInterface(aSelection);
-  if (selPriv) res = selPriv->SetInterlinePosition(PR_TRUE);
-
+  if (!mDidExplicitlySetInterline)
+  {
+    // We prevent the caret from sticking on the left of prior BR
+    // (i.e. the end of previous line) after this deletion.  Bug 92124
+    nsCOMPtr<nsISelectionPrivate> selPriv = do_QueryInterface(aSelection);
+    if (selPriv) res = selPriv->SetInterlinePosition(PR_TRUE);
+  }
   return res;
 }
 
@@ -1124,20 +1122,18 @@ nsTextEditRules::ReplaceNewlines(nsIDOMRange *aRange)
   // into normal breaks.  this is because layout wont give us a place 
   // to put the cursor on empty lines otherwise.
 
-  nsCOMPtr<nsIContentIterator> iter;
+  nsresult res;
+  nsCOMPtr<nsIContentIterator> iter =
+       do_CreateInstance("@mozilla.org/content/post-content-iterator;1", &res);
+  if (NS_FAILED(res)) return res;
+
+  res = iter->Init(aRange);
+  if (NS_FAILED(res)) return res;
+  
   nsCOMPtr<nsISupports> isupports;
   PRInt32 nodeCount,j;
   nsCOMArray<nsIDOMCharacterData> arrayOfNodes;
   
-  // need an iterator
-  nsresult res = nsComponentManager::CreateInstance(kContentIteratorCID,
-                                                    nsnull,
-                                                    NS_GET_IID(nsIContentIterator), 
-                                                    getter_AddRefs(iter));
-  if (NS_FAILED(res)) return res;
-  res = iter->Init(aRange);
-  if (NS_FAILED(res)) return res;
-  
   // gather up a list of editable preformatted text nodes
   while (NS_ENUMERATOR_FALSE == iter->IsDone())
   {
