@@ -39,7 +39,10 @@
  * ***** END LICENSE BLOCK ***** */
 #include "stdafx.h"
 
+#ifdef XPCOM_GLUE
 #include "nsXPCOMGlue.h"
+#endif
+
 #include "nsCOMPtr.h"
 #include "nsComponentManagerUtils.h"
 #include "nsIServiceManagerUtils.h"
@@ -60,7 +63,6 @@
 
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIEventListenerManager.h"
-#include "nsGUIEvent.h"
 
 #include "nsIScriptEventManager.h"
 #include "jsapi.h"
@@ -68,6 +70,16 @@
 #include "LegacyPlugin.h"
 #include "XPConnect.h"
 
+#ifdef XPC_IDISPATCH_SUPPORT
+#include "nsIDOMWindowInternal.h"
+#include "nsIDOMLocation.h"
+#include "nsNetUtil.h"
+#ifdef XPCOM_GLUE
+#include "nsEmbedString.h"
+#endif
+#include "nsIURI.h"
+#endif
+
 static NS_DEFINE_IID(kIClassInfoIID, NS_ICLASSINFO_IID);
 static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
 
@@ -77,7 +89,6 @@ static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
 nsScriptablePeer::nsScriptablePeer() :
     mTearOff(new nsScriptablePeerTearOff(this))
 {
-    NS_INIT_ISUPPORTS();
     NS_ASSERTION(mTearOff, "can't create tearoff");
     MozAxPlugin::AddRef();
 }
@@ -377,7 +388,8 @@ nsScriptablePeer::ConvertVariants(VARIANT *aIn, nsIVariant **aOut)
                 if (pManager)
                 {
                     rv = pManager->CreateInstanceByContractID("@mozilla.org/variant;1",
-                        nsnull, NS_GET_IID(nsIWritableVariant), (void **) &v);
+                        nsnull, NS_GET_IID(nsIWritableVariant),
+                        getter_AddRefs(v));
                     pManager->Release();
                 }
             }
@@ -741,10 +753,11 @@ nsEventSink::InternalInvoke(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wF
     pFuncDesc = NULL;
 
     nsCOMPtr<nsIDOMElement> element;
-    NPN_GetValue(mPlugin->pPluginInstance, NPNVDOMElement, (void *) &element);
+    NPN_GetValue(mPlugin->pPluginInstance, NPNVDOMElement, 
+        NS_STATIC_CAST(nsIDOMElement **, getter_AddRefs(element)));
     if (!element)
     {
-        NS_ASSERTION(element, "can't get the object element");
+        NS_ERROR("can't get the object element");
         return S_OK;
     }
     nsAutoString id;
@@ -759,7 +772,8 @@ nsEventSink::InternalInvoke(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wF
 
     // Fire the script event handler...
     nsCOMPtr<nsIDOMWindow> window;
-    NPN_GetValue(mPlugin->pPluginInstance, NPNVDOMWindow, (void *)&window);
+    NPN_GetValue(mPlugin->pPluginInstance, NPNVDOMWindow, 
+        NS_STATIC_CAST(nsIDOMWindow **, getter_AddRefs(window)));
     
     nsCOMPtr<nsIScriptEventManager> eventManager(do_GetInterface(window));
     if (!eventManager) return S_OK;
@@ -802,7 +816,8 @@ nsEventSink::InternalInvoke(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wF
         nsCOMPtr<nsIDispatchSupport> disp(do_GetService("@mozilla.org/nsdispatchsupport;1"));
         for (UINT i = 0; i < argc; i++)
         {
-            disp->COMVariant2JSVal(&pDispParams->rgvarg[i], &args[i]);
+            // Arguments are listed backwards, intentionally, in rgvarg
+            disp->COMVariant2JSVal(&pDispParams->rgvarg[argc - 1 - i], &args[i]);
         }
     }
 
@@ -917,7 +932,9 @@ void MozAxPlugin::AddRef()
 {
     if (gInstances == 0)
     {
+#ifdef XPCOM_GLUE
         XPCOMGlueStartup(nsnull);
+#endif
         MozAxPlugin::PrefGetHostingFlags(); // Initial call to set it up
     }
     gInstances++;
@@ -930,10 +947,46 @@ void MozAxPlugin::Release()
 #ifdef XPC_IDISPATCH_SUPPORT
         MozAxPlugin::ReleasePrefObserver();
 #endif
+#ifdef XPCOM_GLUE
         XPCOMGlueShutdown();
+#endif
     }
 }
 
+#ifdef XPC_IDISPATCH_SUPPORT
+nsresult MozAxPlugin::GetCurrentLocation(NPP instance, nsIURI **aLocation)
+{
+    NS_ENSURE_ARG_POINTER(aLocation);
+    *aLocation = nsnull;
+    nsCOMPtr<nsIDOMWindow> domWindow;
+    NPN_GetValue(instance, NPNVDOMWindow, (void *) &domWindow);
+    if (!domWindow)
+    {
+        return NS_ERROR_FAILURE;
+    }
+    nsCOMPtr<nsIDOMWindowInternal> windowInternal = do_QueryInterface(domWindow);
+    if (!windowInternal)
+    {
+        return NS_ERROR_FAILURE;
+    }
+
+    nsCOMPtr<nsIDOMLocation> location;
+#ifdef XPCOM_GLUE
+    nsEmbedString href;
+#else
+    nsAutoString href;
+#endif
+    windowInternal->GetLocation(getter_AddRefs(location));
+    if (!location ||
+        NS_FAILED(location->GetHref(href)))
+    {
+        return NS_ERROR_FAILURE;
+    }
+
+    return NS_NewURI(aLocation, href);
+}
+#endif
+
 CLSID MozAxPlugin::GetCLSIDForType(const char *mimeType)
 {
     if (mimeType == NULL)
@@ -1024,7 +1077,7 @@ MozAxPlugin::GetValue(NPP instance, NPPVariable variable, void *value)
         {
             PRBool classExists = PR_FALSE;
             nsCID cid;
-            memcpy(&iid, &pData->clsid, sizeof(cid));
+            memcpy(&cid, &pData->clsid, sizeof(cid));
             dispSupport->IsClassMarkedSafeForScripting(cid, &classExists, &isScriptable);
         }
         if (!isScriptable)
