@@ -68,7 +68,6 @@
 #include "nsIPrompt.h"
 #include "nsRect.h"
 #include "nsIWebBrowserChromeFocus.h"
-#include "nsIDragDropOverride.h"
 #include "nsIContent.h"
 #include "imgIContainer.h"
 #include "nsContextMenuInfo.h"
@@ -632,6 +631,20 @@ NS_IMETHODIMP nsDocShellTreeOwner::SetEnabled(PRBool aEnabled)
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP
+nsDocShellTreeOwner::GetBlurSuppression(PRBool *aBlurSuppression)
+{
+  NS_ENSURE_ARG_POINTER(aBlurSuppression);
+  *aBlurSuppression = PR_FALSE;
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsDocShellTreeOwner::SetBlurSuppression(PRBool aBlurSuppression)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP nsDocShellTreeOwner::GetMainWidget(nsIWidget** aMainWidget)
 {
     return NS_ERROR_NULL_POINTER;
@@ -829,11 +842,8 @@ nsDocShellTreeOwner :: AddChromeListeners ( )
     if ( mChromeDragHandler ) {
       nsCOMPtr<nsIDOMEventReceiver> rcvr;
       GetEventReceiver(mWebBrowser, getter_AddRefs(rcvr));
-      nsCOMPtr<nsIOverrideDragSource> srcOverride ( do_QueryInterface(mWebBrowserChrome) );
-      nsCOMPtr<nsIOverrideDropSite> siteOverride ( do_QueryInterface(mWebBrowserChrome) );
       nsCOMPtr<nsIDOMEventTarget> rcvrTarget(do_QueryInterface(rcvr));
-      mChromeDragHandler->HookupTo(rcvrTarget, NS_STATIC_CAST(nsIWebNavigation*, mWebBrowser),
-                                    srcOverride, siteOverride);
+      mChromeDragHandler->HookupTo(rcvrTarget, NS_STATIC_CAST(nsIWebNavigation*, mWebBrowser));
     }
   }
 
@@ -908,7 +918,7 @@ NS_IMETHODIMP DefaultTooltipTextProvider::GetNodeText(nsIDOMNode *aNode, PRUnich
       nsCOMPtr<nsIContent> content(do_QueryInterface(currElement));
       if (content) {
         nsCOMPtr<nsIAtom> tagAtom;
-        content->GetTag(*getter_AddRefs(tagAtom));
+        content->GetTag(getter_AddRefs(tagAtom));
         if (tagAtom != mTag_dialog &&
             tagAtom != mTag_dialogheader &&
             tagAtom != mTag_window) {
@@ -1566,19 +1576,19 @@ ChromeContextMenuListener :: ContextMenu ( nsIDOMEvent* aMouseEvent )
       }
       else if (tag.Equals(NS_LITERAL_STRING("html"), nsCaseInsensitiveStringComparator()))
       {
-        // first check if this is a background image that the user was trying to click on
-        // and if the listener is ready for that (only nsIContextMenuListener2 and up)
-        if (menuInfoImpl && menuInfoImpl->HasBackgroundImage(node)) {
-          flags2 |= nsIContextMenuListener2::CONTEXT_BACKGROUND_IMAGE;
-          targetDOMnode = node;
-        }
-
         if (!flags && !flags2) { 
         // only care about this if no other context was found.
             flags |= nsIContextMenuListener::CONTEXT_DOCUMENT;
             flags2 |= nsIContextMenuListener2::CONTEXT_DOCUMENT;
             targetDOMnode = node;
         }
+        if (!(flags & nsIContextMenuListener::CONTEXT_IMAGE)) {
+          // first check if this is a background image that the user was trying to click on
+          // and if the listener is ready for that (only nsIContextMenuListener2 and up)
+          if (menuInfoImpl && menuInfoImpl->HasBackgroundImage(node)) {
+            flags2 |= nsIContextMenuListener2::CONTEXT_BACKGROUND_IMAGE;
+          }
+        }
         break; // exit do-while
       }
       else if ( tag.EqualsWithConversion("object", PR_TRUE) ||  /* XXX what about images and documents? */
