@@ -32,6 +32,7 @@
 #include "nsIComponentManager.h"
 #include "nsIDocument.h"
 #include "nsIDOMDocument.h"
+#include "nsIDOMXULDocument.h"
 #include "nsIDOMWindow.h"
 #include "nsIDOMElement.h"
 #include "nsIInterfaceRequestor.h"
@@ -70,7 +71,14 @@
 #include "nsToolkitCompsCID.h"
 #endif
 
+#if (defined(XP_MAC) || defined(XP_MACOSX)) && !defined(MOZ_WIDGET_COCOA)
+#include <MacWindows.h>
+#include "nsWidgetSupport.h"
+#include "nsIEventSink.h"
+#endif
+
 static NS_DEFINE_CID(kWebShellCID, NS_WEB_SHELL_CID);
+static NS_DEFINE_IID(kWindowCID, NS_WINDOW_CID);
 static NS_DEFINE_CID(kChildCID, NS_CHILD_CID);
 static NS_DEFINE_CID(kLookAndFeelCID, NS_LOOKANDFEEL_CID);
 
@@ -92,6 +100,9 @@ nsWebBrowser::nsWebBrowser() : mDocShellTreeOwner(nsnull),
    mParentWidget(nsnull),
    mParent(nsnull),
    mListenerArray(nsnull)
+#if (defined(XP_MAC) || defined(XP_MACOSX)) && !defined(MOZ_WIDGET_COCOA)
+   , mTopLevelWidget(nsnull)
+#endif
 {
     mInitInfo = new nsWebBrowserInitInfo();
     mWWatch = do_GetService(NS_WINDOWWATCHER_CONTRACTID);
@@ -134,6 +145,10 @@ NS_IMETHODIMP nsWebBrowser::InternalDestroy()
       mListenerArray = nsnull;
    }
 
+#if (defined(XP_MAC) || defined(XP_MACOSX)) && !defined(MOZ_WIDGET_COCOA)   
+   NS_IF_RELEASE(mTopLevelWidget);
+#endif
+
    return NS_OK;
 }
 
@@ -172,6 +187,11 @@ NS_IMETHODIMP nsWebBrowser::GetInterface(const nsIID& aIID, void** aSink)
    if(NS_SUCCEEDED(QueryInterface(aIID, aSink)))
       return NS_OK;
 
+#if (defined(XP_MAC) || defined(XP_MACOSX)) && !defined(MOZ_WIDGET_COCOA)
+   if (aIID.Equals(NS_GET_IID(nsIEventSink)) && mTopLevelWidget)
+      return mTopLevelWidget->QueryInterface(NS_GET_IID(nsIEventSink), aSink);
+#endif
+
    if (mDocShell) {
        if (aIID.Equals(NS_GET_IID(nsIWebBrowserPrint))) {
            nsCOMPtr<nsIContentViewer> viewer;
@@ -330,6 +350,40 @@ NS_IMETHODIMP nsWebBrowser::EnableGlobalHistory(PRBool aEnable)
     return rv;
 }
 
+#if (defined(XP_MAC) || defined(XP_MACOSX)) && !defined(MOZ_WIDGET_COCOA)
+NS_IMETHODIMP nsWebBrowser::EnsureTopLevelWidget(nativeWindow aWindow)
+{
+    WindowPtr macWindow = NS_STATIC_CAST(WindowPtr, aWindow);
+    nsIWidget *widget = nsnull;
+    nsCOMPtr<nsIWidget> newWidget;
+    nsresult rv = NS_ERROR_FAILURE;
+
+    ::GetWindowProperty(macWindow, kTopLevelWidgetPropertyCreator,
+        kTopLevelWidgetRefPropertyTag, sizeof(nsIWidget*), nsnull, (void*)&widget);
+    if (!widget) {
+      newWidget = do_CreateInstance(kWindowCID, &rv);
+      if (NS_SUCCEEDED(rv)) {
+        
+        // Create it with huge bounds. The actual bounds that matters is that of the
+        // nsIBaseWindow. The bounds of the top level widget clips its children so
+        // we just have to make sure it is big enough to always contain the children.
+
+        nsRect r(0, 0, 32000, 32000);
+        rv = newWidget->Create(macWindow, r, nsnull, nsnull, nsnull, nsnull, nsnull);
+        if (NS_SUCCEEDED(rv))
+          widget = newWidget;
+      }
+    }
+    NS_ASSERTION(widget, "Failed to get or create a toplevel widget!!");
+    if (widget) {
+      mTopLevelWidget = widget;
+      NS_ADDREF(mTopLevelWidget); // Allows multiple nsWebBrowsers to be in 1 window.
+      rv = NS_OK;
+    }
+    return rv;
+}
+#endif
+
 NS_IMETHODIMP nsWebBrowser::GetContainerWindow(nsIWebBrowserChrome** aTopWindow)
 {
    NS_ENSURE_ARG_POINTER(aTopWindow);
@@ -604,7 +658,7 @@ NS_IMETHODIMP nsWebBrowser::GoForward()
 
 NS_IMETHODIMP nsWebBrowser::LoadURI(const PRUnichar* aURI,
                                     PRUint32 aLoadFlags,
-                                    nsIURI* aReferingURI,
+                                    nsIURI* aReferringURI,
                                     nsIInputStream* aPostDataStream,
                                     nsIInputStream* aExtraHeaderStream)
 {
@@ -612,7 +666,7 @@ NS_IMETHODIMP nsWebBrowser::LoadURI(const PRUnichar* aURI,
 
    return mDocShellAsNav->LoadURI(aURI,
                                   aLoadFlags,
-                                  aReferingURI,
+                                  aReferringURI,
                                   aPostDataStream,
                                   aExtraHeaderStream);
 }
@@ -645,11 +699,11 @@ NS_IMETHODIMP nsWebBrowser::GetCurrentURI(nsIURI** aURI)
    return mDocShellAsNav->GetCurrentURI(aURI);
 }
 
-NS_IMETHODIMP nsWebBrowser::GetReferingURI(nsIURI** aURI)
+NS_IMETHODIMP nsWebBrowser::GetReferringURI(nsIURI** aURI)
 {
     NS_ENSURE_STATE(mDocShell);
 
-    return mDocShellAsNav->GetReferingURI(aURI);
+    return mDocShellAsNav->GetReferringURI(aURI);
 }
 
 NS_IMETHODIMP nsWebBrowser::SetSessionHistory(nsISHistory* aSessionHistory)
@@ -1042,7 +1096,9 @@ NS_IMETHODIMP nsWebBrowser::Create()
       nsWidgetInitData  widgetInit;
 
       widgetInit.clipChildren = PR_TRUE;
-      widgetInit.mContentType = mContentType;
+      widgetInit.mContentType = (mContentType == typeChrome || 
+        mContentType == typeChromeWrapper)? eContentTypeUI: eContentTypeContent;
+
       widgetInit.mWindowType = eWindowType_child;
       nsRect bounds(mInitInfo->x, mInitInfo->y, mInitInfo->cx, mInitInfo->cy);
       
@@ -1289,8 +1345,14 @@ NS_IMETHODIMP nsWebBrowser::GetParentNativeWindow(nativeWindow* aParentNativeWin
 NS_IMETHODIMP nsWebBrowser::SetParentNativeWindow(nativeWindow aParentNativeWindow)
 {
    NS_ENSURE_STATE(!mDocShell);
-
+   
+#if (defined(XP_MAC) || defined(XP_MACOSX)) && !defined(MOZ_WIDGET_COCOA)
+   nsresult rv = EnsureTopLevelWidget(aParentNativeWindow);
+   NS_ENSURE_SUCCESS(rv, rv);
+   mParentNativeWindow = mTopLevelWidget->GetNativeData(NS_NATIVE_WINDOW);
+#else
    mParentNativeWindow = aParentNativeWindow;
+#endif
 
    return NS_OK;
 }
@@ -1335,6 +1397,20 @@ NS_IMETHODIMP nsWebBrowser::SetEnabled(PRBool aEnabled)
   return NS_ERROR_FAILURE;
 }
 
+NS_IMETHODIMP
+nsWebBrowser::GetBlurSuppression(PRBool *aBlurSuppression)
+{
+  NS_ENSURE_ARG_POINTER(aBlurSuppression);
+  *aBlurSuppression = PR_FALSE;
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsWebBrowser::SetBlurSuppression(PRBool aBlurSuppression)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP nsWebBrowser::GetMainWidget(nsIWidget** mainWidget)
 {
    NS_ENSURE_ARG_POINTER(mainWidget);
