@@ -98,11 +98,7 @@ static NS_DEFINE_CID(kStringBundleServiceCID,  NS_STRINGBUNDLESERVICE_CID);
 // For PrintDlgEx
 // needed because there are unicode/ansi versions of this routine
 // and we need to make sure we get the correct one.
-#ifdef UNICODE
-#define GetPrintDlgExQuoted "PrintDlgExW"
-#else
 #define GetPrintDlgExQuoted "PrintDlgExA"
-#endif
 
 // Default labels for the radio buttons
 static const char* kAsLaidOutOnScreenStr = "As &laid out on the screen";
@@ -298,6 +294,7 @@ SetPrintSettingsFromDevMode(nsIPrintSettings* aPrintSettings,
     return NS_ERROR_FAILURE;
   }
 
+  aPrintSettings->SetIsInitializedFromPrinter(PR_TRUE);
   if (aDevMode->dmFields & DM_ORIENTATION) {
     PRInt32 orientation  = aDevMode->dmOrientation == DMORIENT_PORTRAIT?
                            nsIPrintSettings::kPortraitOrientation:nsIPrintSettings::kLandscapeOrientation;
@@ -326,6 +323,12 @@ SetPrintSettingsFromDevMode(nsIPrintSettings* aPrintSettings,
   if (aDevMode->dmFields & DM_PAPERSIZE) {
     aPrintSettings->SetPaperSizeType(nsIPrintSettings::kPaperSizeNativeData);
     aPrintSettings->SetPaperData(aDevMode->dmPaperSize);
+    for (PRInt32 i=0;i<kNumPaperSizes;i++) {
+      if (kPaperSizes[i].mPaperSize == aDevMode->dmPaperSize) {
+        aPrintSettings->SetPaperSizeUnit(kPaperSizes[i].mIsInches?nsIPrintSettings::kPaperSizeInches:nsIPrintSettings::kPaperSizeMillimeters);
+        break;
+      }
+    }
 
   } else if (aDevMode->dmFields & DM_PAPERLENGTH && aDevMode->dmFields & DM_PAPERWIDTH) {
     PRBool found = PR_FALSE;
@@ -334,7 +337,7 @@ SetPrintSettingsFromDevMode(nsIPrintSettings* aPrintSettings,
         aPrintSettings->SetPaperSizeType(nsIPrintSettings::kPaperSizeDefined);
         aPrintSettings->SetPaperWidth(kPaperSizes[i].mWidth);
         aPrintSettings->SetPaperHeight(kPaperSizes[i].mHeight);
-        aPrintSettings->SetPaperSizeUnit(kPaperSizes[i].mIsInches?nsIPrintSettings::kPaperSizeInches:nsIPrintSettings::kPaperSizeInches);
+        aPrintSettings->SetPaperSizeUnit(kPaperSizes[i].mIsInches?nsIPrintSettings::kPaperSizeInches:nsIPrintSettings::kPaperSizeMillimeters);
         found = PR_TRUE;
         break;
       }
@@ -760,7 +763,12 @@ static UINT CALLBACK PrintHookProc(HWND hdlg, UINT uiMsg, WPARAM wParam, LPARAM
 
 //----------------------------------------------------------------------------------
 // Returns a Global Moveable Memory Handle to a DevMode
-// from the Printer byt the name of aPrintName
+// from the Printer by the name of aPrintName
+//
+// NOTE:
+//   This function assumes that aPrintName has already been converted to 
+//   Double Byte chars typically via the GetACPString helper function
+//
 static HGLOBAL CreateGlobalDevModeAndInit(LPTSTR aPrintName, nsIPrintSettings* aPS)
 {
   HGLOBAL hGlobalDevMode = NULL;
@@ -773,13 +781,6 @@ static HGLOBAL CreateGlobalDevModeAndInit(LPTSTR aPrintName, nsIPrintSettings* a
     LPDEVMODE   pNewDevMode;
     DWORD       dwNeeded, dwRet;
 
-    nsString prtName;
-#ifdef UNICODE
-    prtName.AppendWithConversion((PRUnichar *)aPrintName);
-#else 
-    prtName.AssignWithConversion((char*)aPrintName);
-#endif
-
     // Get the buffer size
     dwNeeded = ::DocumentProperties(gParentWnd, hPrinter, aPrintName, NULL, NULL, 0);
     if (dwNeeded == 0) {
@@ -812,6 +813,17 @@ static HGLOBAL CreateGlobalDevModeAndInit(LPTSTR aPrintName, nsIPrintSettings* a
       memcpy(devMode, pNewDevMode, dwNeeded);
       // Initialize values from the PrintSettings
       SetupDevModeFromSettings(devMode, aPS);
+
+      // Sets back the changes we made to the DevMode into the Printer Driver
+      dwRet = ::DocumentProperties(gParentWnd, hPrinter, aPrintName, devMode, devMode, DM_IN_BUFFER | DM_OUT_BUFFER);
+      if (dwRet != IDOK) {
+        ::GlobalUnlock(hGlobalDevMode);
+        ::GlobalFree(hGlobalDevMode);
+        ::HeapFree(::GetProcessHeap(), 0, pNewDevMode);
+        ::ClosePrinter(hPrinter);
+         return NULL;
+      }
+
       ::GlobalUnlock(hGlobalDevMode);
     } else {
       ::GlobalFree(hGlobalDevMode);
@@ -871,15 +883,19 @@ ShowNativePrintDialog(HWND              aHWnd,
   if (!printerName) return NS_ERROR_FAILURE;
 
   // Now create a DEVNAMES struct so the the dialog is initialized correctly.
-  PRUint32 len = nsCRT::strlen(printerName);
+  char* dbStr = GetACPString(nsString(printerName));
+  NS_ENSURE_TRUE(dbStr, NS_ERROR_FAILURE);
+
+  PRUint32 len = strlen(dbStr);
   hDevNames = (HGLOBAL)::GlobalAlloc(GHND, len+sizeof(DEVNAMES)+1);
   DEVNAMES* pDevNames = (DEVNAMES*)::GlobalLock(hDevNames);
   pDevNames->wDriverOffset = sizeof(DEVNAMES);
   pDevNames->wDeviceOffset = sizeof(DEVNAMES);
   pDevNames->wOutputOffset = sizeof(DEVNAMES)+len+1;
   pDevNames->wDefault      = 0;
+
   char* device = &(((char*)pDevNames)[pDevNames->wDeviceOffset]);
-  strcpy(device, (char*)NS_ConvertUCS2toUTF8(printerName).get());
+  strcpy(device, dbStr);
   ::GlobalUnlock(hDevNames);
 
   // Create a Moveable Memory Object that holds a new DevMode
@@ -888,11 +904,8 @@ ShowNativePrintDialog(HWND              aHWnd,
   // NOTE: We only need to free hGlobalDevMode when the dialog is cancelled
   // When the user prints, it comes back in the printdlg struct and 
   // is used and cleaned up later
-#ifdef UNICODE
-  hGlobalDevMode = CreateGlobalDevModeAndInit(printerName, aPrintSettings);
-#else 
-  hGlobalDevMode = CreateGlobalDevModeAndInit((char*)NS_ConvertUCS2toUTF8(printerName).get(), aPrintSettings);
-#endif
+  hGlobalDevMode = CreateGlobalDevModeAndInit(dbStr, aPrintSettings);
+  free(dbStr);
 
   // Prepare to Display the Print Dialog
   PRINTDLG  prntdlg;
@@ -903,7 +916,7 @@ ShowNativePrintDialog(HWND              aHWnd,
   prntdlg.hDevMode    = hGlobalDevMode;
   prntdlg.hDevNames   = hDevNames;
   prntdlg.hDC         = NULL;
-  prntdlg.Flags       = PD_ALLPAGES | PD_RETURNIC | PD_HIDEPRINTTOFILE | PD_USEDEVMODECOPIESANDCOLLATE;
+  prntdlg.Flags       = PD_ALLPAGES | PD_RETURNIC | PD_USEDEVMODECOPIESANDCOLLATE;
 
   // if there is a current selection then enable the "Selection" radio button
   PRInt16 howToEnableFrameUI = nsIPrintSettings::kFrameEnableNone;
@@ -927,8 +940,13 @@ ShowNativePrintDialog(HWND              aHWnd,
     }
   }
 
-  prntdlg.nFromPage           = 0xFFFF;
-  prntdlg.nToPage             = 0xFFFF;
+  PRInt32 pg = 1;
+  aPrintSettings->GetStartPageRange(&pg);
+  prntdlg.nFromPage           = pg;
+  
+  aPrintSettings->GetEndPageRange(&pg);
+  prntdlg.nToPage             = pg;
+
   prntdlg.nMinPage            = 1;
   prntdlg.nMaxPage            = 0xFFFF;
   prntdlg.nCopies             = 1;
@@ -953,70 +971,108 @@ ShowNativePrintDialog(HWND              aHWnd,
   BOOL result = ::PrintDlg(&prntdlg);
 
   if (TRUE == result) {
-    if (aPrintSettings && prntdlg.hDevMode != NULL) {
-      // Transfer the settings from the native data to the PrintSettings
-      LPDEVMODE devMode = (LPDEVMODE)::GlobalLock(prntdlg.hDevMode);
-      SetPrintSettingsFromDevMode(aPrintSettings, devMode);
-      ::GlobalUnlock(prntdlg.hDevMode);
+    // check to make sure we don't have any NULL pointers
+    NS_ENSURE_TRUE(aPrintSettings && prntdlg.hDevMode, NS_ERROR_FAILURE);
+
+    if (prntdlg.hDevNames == NULL) {
+      ::GlobalFree(hGlobalDevMode);
+      return NS_ERROR_FAILURE;
     }
+    // Lock the deviceNames and check for NULL
     DEVNAMES *devnames = (DEVNAMES *)::GlobalLock(prntdlg.hDevNames);
-    if ( NULL != devnames ) {
+    if (devnames == NULL) {
+      ::GlobalFree(hGlobalDevMode);
+      return NS_ERROR_FAILURE;
+    }
 
-      char* device = &(((char *)devnames)[devnames->wDeviceOffset]);
-      char* driver = &(((char *)devnames)[devnames->wDriverOffset]);
+    char* device = &(((char *)devnames)[devnames->wDeviceOffset]);
+    char* driver = &(((char *)devnames)[devnames->wDriverOffset]);
 
-      nsCOMPtr<nsIPrintSettingsWin> psWin(do_QueryInterface(aPrintSettings));
-      // Setup local Data members
-      psWin->SetDeviceName(device);
-      psWin->SetDriverName(driver);
+    // Check to see if the "Print To File" control is checked
+    // then take the name from devNames and set it in the PrintSettings
+    //
+    // NOTE:
+    // As per Microsoft SDK documentation the returned value offset from
+    // devnames->wOutputOffset is either "FILE:" or NULL
+    // if the "Print To File" checkbox is checked it MUST be "FILE:"
+    // We assert as an extra safety check.
+    if (prntdlg.Flags & PD_PRINTTOFILE) {
+      char* fileName = &(((char *)devnames)[devnames->wOutputOffset]);
+      NS_ASSERTION(strcmp(fileName, "FILE:") == 0, "FileName must be `FILE:`");
+      aPrintSettings->SetToFileName(NS_ConvertASCIItoUCS2(fileName).get());
+      aPrintSettings->SetPrintToFile(PR_TRUE);
+    } else {
+      // clear "print to file" info
+      aPrintSettings->SetPrintToFile(PR_FALSE);
+      aPrintSettings->SetToFileName(nsnull);
+    }
+
+    nsCOMPtr<nsIPrintSettingsWin> psWin(do_QueryInterface(aPrintSettings));
+    if (!psWin) {
+      ::GlobalFree(hGlobalDevMode);
+      return NS_ERROR_FAILURE;
+    }
+
+    // Setup local Data members
+    psWin->SetDeviceName(device);
+    psWin->SetDriverName(driver);
 
 #if defined(DEBUG_rods) || defined(DEBUG_dcone)
-      printf("printer: driver %s, device %s  flags: %d\n", driver, device, prntdlg.Flags);
+    printf("printer: driver %s, device %s  flags: %d\n", driver, device, prntdlg.Flags);
 #endif
-      // fill the print options with the info from the dialog
-      if (aPrintSettings != nsnull) {
-        nsString printerName;
-        printerName.AssignWithConversion(device);
+    // fill the print options with the info from the dialog
+    nsString printerName;
+    printerName.AssignWithConversion(device);
 
-        aPrintSettings->SetPrinterName(printerName.get());
+    aPrintSettings->SetPrinterName(printerName.get());
 
-        if (prntdlg.Flags & PD_SELECTION) {
-          aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeSelection);
+    if (prntdlg.Flags & PD_SELECTION) {
+      aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeSelection);
 
-        } else if (prntdlg.Flags & PD_PAGENUMS) {
-          aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeSpecifiedPageRange);
-          aPrintSettings->SetStartPageRange(prntdlg.nFromPage);
-          aPrintSettings->SetEndPageRange(prntdlg.nToPage);
+    } else if (prntdlg.Flags & PD_PAGENUMS) {
+      aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeSpecifiedPageRange);
+      aPrintSettings->SetStartPageRange(prntdlg.nFromPage);
+      aPrintSettings->SetEndPageRange(prntdlg.nToPage);
 
-        } else { // (prntdlg.Flags & PD_ALLPAGES)
-          aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeAllPages);
-        }
+    } else { // (prntdlg.Flags & PD_ALLPAGES)
+      aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeAllPages);
+    }
 
-        if (howToEnableFrameUI != nsIPrintSettings::kFrameEnableNone) {
-          // make sure the dialog got extended
-          if (gDialogWasExtended) {
-            // check to see about the frame radio buttons
-            switch (gFrameSelectedRadioBtn) {
-              case rad4: 
-                aPrintSettings->SetPrintFrameType(nsIPrintSettings::kFramesAsIs);
-                break;
-              case rad5: 
-                aPrintSettings->SetPrintFrameType(nsIPrintSettings::kSelectedFrame);
-                break;
-              case rad6: 
-                aPrintSettings->SetPrintFrameType(nsIPrintSettings::kEachFrameSep);
-                break;
-            } // switch
-          } else {
-            // if it didn't get extended then have it default to printing
-            // each frame separately
+    if (howToEnableFrameUI != nsIPrintSettings::kFrameEnableNone) {
+      // make sure the dialog got extended
+      if (gDialogWasExtended) {
+        // check to see about the frame radio buttons
+        switch (gFrameSelectedRadioBtn) {
+          case rad4: 
+            aPrintSettings->SetPrintFrameType(nsIPrintSettings::kFramesAsIs);
+            break;
+          case rad5: 
+            aPrintSettings->SetPrintFrameType(nsIPrintSettings::kSelectedFrame);
+            break;
+          case rad6: 
             aPrintSettings->SetPrintFrameType(nsIPrintSettings::kEachFrameSep);
-          }
-        } else {
-          aPrintSettings->SetPrintFrameType(nsIPrintSettings::kNoFrames);
-        }
+            break;
+        } // switch
+      } else {
+        // if it didn't get extended then have it default to printing
+        // each frame separately
+        aPrintSettings->SetPrintFrameType(nsIPrintSettings::kEachFrameSep);
       }
-      ::GlobalUnlock(prntdlg.hDevNames);
+    } else {
+      aPrintSettings->SetPrintFrameType(nsIPrintSettings::kNoFrames);
+    }
+    // Unlock DeviceNames
+    ::GlobalUnlock(prntdlg.hDevNames);
+
+    // Transfer the settings from the native data to the PrintSettings
+    LPDEVMODE devMode = (LPDEVMODE)::GlobalLock(prntdlg.hDevMode);
+    if (devMode == NULL) {
+      ::GlobalFree(hGlobalDevMode);
+      return NS_ERROR_FAILURE;
+    }
+    psWin->SetDevMode(devMode); // copies DevMode
+    SetPrintSettingsFromDevMode(aPrintSettings, devMode);
+    ::GlobalUnlock(prntdlg.hDevMode);
 
 #if defined(DEBUG_rods) || defined(DEBUG_dcone)
     PRBool  printSelection = prntdlg.Flags & PD_SELECTION;
@@ -1040,15 +1096,9 @@ ShowNativePrintDialog(HWND              aHWnd,
     }
 #endif
     
-      LPDEVMODE devMode = (LPDEVMODE)::GlobalLock(prntdlg.hDevMode);
-      psWin->SetDevMode(devMode);
-      SetPrintSettingsFromDevMode(aPrintSettings, devMode);
-      ::GlobalUnlock(prntdlg.hDevMode);
-
-    }
   } else {
     aPrintSettings->SetIsCancelled(PR_TRUE);
-    ::GlobalFree(hGlobalDevMode);
+    if (hGlobalDevMode) ::GlobalFree(hGlobalDevMode);
     return NS_ERROR_ABORT;
   }
 
@@ -1195,11 +1245,10 @@ ShowNativePrintDialogEx(HWND              aHWnd,
   aPrintSettings->GetPrinterName(&printerName);
   HGLOBAL hGlobalDevMode = NULL;
   if (printerName) {
-#ifdef UNICODE
-    hGlobalDevMode = CreateGlobalDevModeAndInit(printerName, aPrintSettings);
-#else 
-    hGlobalDevMode = CreateGlobalDevModeAndInit((char*)NS_ConvertUCS2toUTF8(printerName).get(), aPrintSettings);
-#endif
+    char* dbStr = GetACPString(nsString(printerName));
+    NS_ENSURE_TRUE(dbStr, NS_ERROR_FAILURE);
+    hGlobalDevMode = CreateGlobalDevModeAndInit(dbStr, aPrintSettings);
+    free(dbStr);
   }
 
   // Prepare to Display the Print Dialog
@@ -1209,7 +1258,7 @@ ShowNativePrintDialogEx(HWND              aHWnd,
   prntdlg.lStructSize = sizeof(prntdlg);
   prntdlg.hwndOwner   = aHWnd;
   prntdlg.hDevMode    = hGlobalDevMode;
-  prntdlg.Flags       = PD_ALLPAGES | PD_RETURNDC | PD_HIDEPRINTTOFILE | PD_USEDEVMODECOPIESANDCOLLATE |
+  prntdlg.Flags       = PD_ALLPAGES | PD_RETURNDC | PD_USEDEVMODECOPIESANDCOLLATE |
                         PD_NOCURRENTPAGE;
   prntdlg.nStartPage  = START_PAGE_GENERAL;
 
@@ -1272,67 +1321,110 @@ ShowNativePrintDialogEx(HWND              aHWnd,
   HRESULT result = ::PrintDlgEx(&prntdlg);
 
   if (S_OK == result && (prntdlg.dwResultAction == PD_RESULT_PRINT)) {
-    if (aPrintSettings && prntdlg.hDevMode != NULL) {
-      LPDEVMODE devMode = (LPDEVMODE)::GlobalLock(prntdlg.hDevMode);
-      SetPrintSettingsFromDevMode(aPrintSettings, devMode);
-      ::GlobalUnlock(prntdlg.hDevMode);
+
+    // check to make sure we don't have any NULL pointers
+    NS_ENSURE_TRUE(aPrintSettings && prntdlg.hDevMode, NS_ERROR_FAILURE);
+
+    if (prntdlg.hDevNames == NULL) {
+      ::GlobalFree(hGlobalDevMode);
+      return NS_ERROR_FAILURE;
     }
+    // Lock the deviceNames and check for NULL
     DEVNAMES *devnames = (DEVNAMES *)::GlobalLock(prntdlg.hDevNames);
-    if ( NULL != devnames ) {
+    if (devnames == NULL) {
+      ::GlobalFree(hGlobalDevMode);
+      return NS_ERROR_FAILURE;
+    }
 
-      char* device = &(((char *)devnames)[devnames->wDeviceOffset]);
-      char* driver = &(((char *)devnames)[devnames->wDriverOffset]);
+    char* device = &(((char *)devnames)[devnames->wDeviceOffset]);
+    char* driver = &(((char *)devnames)[devnames->wDriverOffset]);
 
-      nsCOMPtr<nsIPrintSettingsWin> psWin(do_QueryInterface(aPrintSettings));
-      // Setup local Data members
-      psWin->SetDeviceName(device);
-      psWin->SetDriverName(driver);
+    // Check to see if the "Print To File" control is checked
+    // then take the name from devNames and set it in the PrintSettings
+    //
+    // NOTE:
+    // As per Microsoft SDK documentation the returned value offset from
+    // devnames->wOutputOffset is either "FILE:" or NULL
+    // if the "Print To File" checkbox is checked it MUST be "FILE:"
+    // We assert as an extra safety check.
+    if (prntdlg.Flags & PD_PRINTTOFILE) {
+      char* fileName = &(((char *)devnames)[devnames->wOutputOffset]);
+      NS_ASSERTION(strcmp(fileName, "FILE:") == 0, "FileName must be `FILE:`");
+      aPrintSettings->SetToFileName(NS_ConvertASCIItoUCS2(fileName).get());
+      aPrintSettings->SetPrintToFile(PR_TRUE);
+    } else {
+      // clear "print to file" info
+      aPrintSettings->SetPrintToFile(PR_FALSE);
+      aPrintSettings->SetToFileName(nsnull);
+    }
+
+    nsCOMPtr<nsIPrintSettingsWin> psWin(do_QueryInterface(aPrintSettings));
+    if (!psWin) {
+      ::GlobalFree(hGlobalDevMode);
+      return NS_ERROR_FAILURE;
+    }
+
+    // Setup local Data members
+    psWin->SetDeviceName(device);
+    psWin->SetDriverName(driver);
 
 #if defined(DEBUG_rods) || defined(DEBUG_dcone)
-      printf("printer: driver %s, device %s  flags: %d\n", driver, device, prntdlg.Flags);
+    printf("printer: driver %s, device %s  flags: %d\n", driver, device, prntdlg.Flags);
 #endif
-      ::GlobalUnlock(prntdlg.hDevNames);
+    ::GlobalUnlock(prntdlg.hDevNames);
 
-      // fill the print options with the info from the dialog
-      if (aPrintSettings != nsnull) {
+    // fill the print options with the info from the dialog
+    if (aPrintSettings != nsnull) {
 
-        if (prntdlg.Flags & PD_SELECTION) {
-          aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeSelection);
+      if (prntdlg.Flags & PD_SELECTION) {
+        aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeSelection);
 
-        } else if (prntdlg.Flags & PD_PAGENUMS) {
-          aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeSpecifiedPageRange);
-          aPrintSettings->SetStartPageRange(pPageRanges->nFromPage);
-          aPrintSettings->SetEndPageRange(pPageRanges->nToPage);
+      } else if (prntdlg.Flags & PD_PAGENUMS) {
+        aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeSpecifiedPageRange);
+        aPrintSettings->SetStartPageRange(pPageRanges->nFromPage);
+        aPrintSettings->SetEndPageRange(pPageRanges->nToPage);
 
-        } else { // (prntdlg.Flags & PD_ALLPAGES)
-          aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeAllPages);
-        }
-
-        if (howToEnableFrameUI != nsIPrintSettings::kFrameEnableNone) {
-          // make sure the dialog got extended
-          if (gDialogWasExtended) {
-            // check to see about the frame radio buttons
-            switch (gFrameSelectedRadioBtn) {
-              case rad4: 
-                aPrintSettings->SetPrintFrameType(nsIPrintSettings::kFramesAsIs);
-                break;
-              case rad5: 
-                aPrintSettings->SetPrintFrameType(nsIPrintSettings::kSelectedFrame);
-                break;
-              case rad6: 
-                aPrintSettings->SetPrintFrameType(nsIPrintSettings::kEachFrameSep);
-                break;
-            } // switch
-          } else {
-            // if it didn't get extended then have it default to printing
-            // each frame separately
-            aPrintSettings->SetPrintFrameType(nsIPrintSettings::kEachFrameSep);
-          }
-        } else {
-          aPrintSettings->SetPrintFrameType(nsIPrintSettings::kNoFrames);
-        }
+      } else { // (prntdlg.Flags & PD_ALLPAGES)
+        aPrintSettings->SetPrintRange(nsIPrintSettings::kRangeAllPages);
       }
 
+      if (howToEnableFrameUI != nsIPrintSettings::kFrameEnableNone) {
+        // make sure the dialog got extended
+        if (gDialogWasExtended) {
+          // check to see about the frame radio buttons
+          switch (gFrameSelectedRadioBtn) {
+            case rad4: 
+              aPrintSettings->SetPrintFrameType(nsIPrintSettings::kFramesAsIs);
+              break;
+            case rad5: 
+              aPrintSettings->SetPrintFrameType(nsIPrintSettings::kSelectedFrame);
+              break;
+            case rad6: 
+              aPrintSettings->SetPrintFrameType(nsIPrintSettings::kEachFrameSep);
+              break;
+          } // switch
+        } else {
+          // if it didn't get extended then have it default to printing
+          // each frame separately
+          aPrintSettings->SetPrintFrameType(nsIPrintSettings::kEachFrameSep);
+        }
+      } else {
+        aPrintSettings->SetPrintFrameType(nsIPrintSettings::kNoFrames);
+      }
+    }
+
+    // Unlock DeviceNames
+    ::GlobalUnlock(prntdlg.hDevNames);
+
+    // Transfer the settings from the native data to the PrintSettings
+    LPDEVMODE devMode = (LPDEVMODE)::GlobalLock(prntdlg.hDevMode);
+    if (devMode == NULL) {
+      ::GlobalFree(hGlobalDevMode);
+      return NS_ERROR_FAILURE;
+    }
+    psWin->SetDevMode(devMode); // copies DevMode
+    SetPrintSettingsFromDevMode(aPrintSettings, devMode);
+    ::GlobalUnlock(prntdlg.hDevMode);
 
 #if defined(DEBUG_rods) || defined(DEBUG_dcone)
     PRBool  printSelection = prntdlg.Flags & PD_SELECTION;
@@ -1356,12 +1448,6 @@ ShowNativePrintDialogEx(HWND              aHWnd,
     }
 #endif
     
-    LPDEVMODE devMode = (LPDEVMODE)::GlobalLock(prntdlg.hDevMode);
-    psWin->SetDevMode(devMode);
-    SetPrintSettingsFromDevMode(aPrintSettings, devMode);
-    ::GlobalUnlock(prntdlg.hDevMode);
-
-    }
   } else {
     if (hGlobalDevMode) ::GlobalFree(hGlobalDevMode);
     return NS_ERROR_ABORT;
