@@ -65,8 +65,8 @@
 //         clicking on the "Choose..." button), the specified app
 //         is run after the content is downloaded
 //
-//      3. In either case, CHelperAppLauncherDialog::ShowProgressDialog()
-//         will be called which is used to display a download progress
+//      3. In either case, an instance of nsIProgressDialog will be created
+//         which is used to display a download progress
 //         dialog box. This dialog box registers itself as a
 //         WebProgressListener so that it can receive the progress change
 //         messages, using which it can update the progress bar
@@ -125,7 +125,6 @@ NS_IMPL_ISUPPORTS1(CHelperAppLauncherDialogFactory, nsIFactory)
 
 CHelperAppLauncherDialogFactory::CHelperAppLauncherDialogFactory() 
 {
-    NS_INIT_ISUPPORTS();
 }
 
 CHelperAppLauncherDialogFactory::~CHelperAppLauncherDialogFactory() {
@@ -163,7 +162,6 @@ NS_IMPL_ISUPPORTS1(CHelperAppLauncherDialog, nsIHelperAppLauncherDialog)
 CHelperAppLauncherDialog::CHelperAppLauncherDialog() :
       mWWatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID))
 {
-    NS_INIT_ISUPPORTS();
 }
 
 CHelperAppLauncherDialog::~CHelperAppLauncherDialog() 
@@ -211,7 +209,8 @@ CWnd* CHelperAppLauncherDialog::GetParentFromContext(nsISupports *aWindowContext
 // when Gecko encounters a mime type it cannot handle
 //
 NS_IMETHODIMP CHelperAppLauncherDialog::Show(nsIHelperAppLauncher *aLauncher, 
-                                             nsISupports *aContext)
+                                             nsISupports *aContext,
+                                             PRBool aForced)
 {
     ResourceState setState;
 
@@ -241,8 +240,11 @@ NS_IMETHODIMP CHelperAppLauncherDialog::Show(nsIHelperAppLauncher *aLauncher,
 
         m_FileName = dlg.m_OpenWithAppName;
 
+        USES_CONVERSION;
+        nsCAutoString fileName(T2CA(m_FileName));
+
         nsCOMPtr<nsILocalFile> openWith;
-        nsresult rv = NS_NewNativeLocalFile(nsDependentCString(m_FileName), PR_FALSE, getter_AddRefs(openWith));
+        nsresult rv = NS_NewNativeLocalFile(fileName, PR_FALSE, getter_AddRefs(openWith));
         if (NS_FAILED(rv))
             return aLauncher->LaunchWithApplication(nsnull, PR_FALSE);
         else
@@ -263,42 +265,21 @@ NS_IMETHODIMP CHelperAppLauncherDialog::PromptForSaveToFile(nsISupports *aWindow
 
     NS_ENSURE_ARG_POINTER(_retval);
 
-    char *lpszFilter = "All Files (*.*)|*.*||";
-    CFileDialog cf(FALSE, W2T(aSuggestedFileExtension), W2T(aDefaultFile),
+    TCHAR *lpszFilter = _T("All Files (*.*)|*.*||");
+    CFileDialog cf(FALSE, W2CT(aSuggestedFileExtension), W2CT(aDefaultFile),
                     OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                     lpszFilter, GetParentFromContext(aWindowContext));
     if(cf.DoModal() == IDOK)
     {
         m_FileName = cf.GetPathName(); // Will be like: c:\tmp\junk.exe
-
-        return NS_NewNativeLocalFile(nsDependentCString(m_FileName), PR_FALSE, _retval);
+        USES_CONVERSION;
+        nsCAutoString fileName(T2CA(m_FileName));
+        return NS_NewNativeLocalFile(fileName, PR_FALSE, _retval);
     }
     else
         return NS_ERROR_FAILURE;
 }
 
-// Displays a download progress dialog whether the user chose a
-// "Save To Disk" or an "Open Using An App" option earlier
-//
-NS_IMETHODIMP CHelperAppLauncherDialog::ShowProgressDialog(nsIHelperAppLauncher *aLauncher, 
-                                                           nsISupports *aContext)
-{
-    ResourceState setState;
-
-    NS_ENSURE_ARG_POINTER(aLauncher);
-
-    CProgressDlg *pProgressDlg = new CProgressDlg(aLauncher, m_HandleContentOp, m_FileName);
-    if(!pProgressDlg)
-        return NS_ERROR_OUT_OF_MEMORY;
-
-    nsCOMPtr<nsIWebProgressListener> listener = NS_STATIC_CAST(nsIWebProgressListener*, pProgressDlg);
-    aLauncher->SetWebProgressListener(listener);
-
-    pProgressDlg->Create(IDD_PROGRESS_DIALOG);
-
-    return NS_OK;
-}
-
 //*****************************************************************************
 // CChooseActionDlg
 //***************************************************************************** 
@@ -337,7 +318,10 @@ BOOL CChooseActionDlg::OnInitDialog()
         {
             CStatic *pMimeType = (CStatic *)GetDlgItem(IDC_CONTENT_TYPE);
             if(pMimeType)
-                pMimeType->SetWindowText(mimeType.get());
+            {
+                USES_CONVERSION;
+                pMimeType->SetWindowText(A2CT(mimeType.get()));
+            }
         }
 
         // See if we can get the preferred action from the mime info
@@ -378,7 +362,7 @@ void CChooseActionDlg::InitWithPreferredAction(nsIMIMEInfo* aMimeInfo)
     if(NS_SUCCEEDED(rv)) 
     {
         USES_CONVERSION;
-        m_OpenWithAppName = W2T(appDesc.get());
+        m_OpenWithAppName = W2CT(appDesc.get());
 
         // Update with the app name
         //
@@ -451,9 +435,9 @@ void CChooseActionDlg::OnSaveToDiskRadioBtnClicked()
 
 void CChooseActionDlg::OnChooseAppClicked() 
 {	
-    char *lpszFilter =
-        "EXE Files Only (*.exe)|*.exe|"
-        "All Files (*.*)|*.*||";
+    TCHAR *lpszFilter =
+        _T("EXE Files Only (*.exe)|*.exe|")
+        _T("All Files (*.*)|*.*||");
 
     CFileDialog cf(TRUE, NULL, NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                     lpszFilter, this);
@@ -476,8 +460,9 @@ void CChooseActionDlg::OnOK()
         //
         if(m_OpenWithAppName.IsEmpty())
         {
-            ::MessageBox(this->m_hWnd, "You have chosen to open the content with an external application, but,\nno application has been specified.\n\nPlease click the Choose... button to select an application",
-            "MfcEmbed", MB_OK);
+            ::MessageBox(this->m_hWnd,
+                _T("You have chosen to open the content with an external application, but,\nno application has been specified.\n\nPlease click the Choose... button to select an application"),
+                _T("MfcEmbed"), MB_OK);
             return;
         }
         else
@@ -510,8 +495,6 @@ CProgressDlg::CProgressDlg(nsIHelperAppLauncher *aLauncher, int aHandleContentOp
                            CString& aFileName, CWnd* pParent /*=NULL*/)
 	: CDialog(CProgressDlg::IDD, pParent)
 {
-    NS_INIT_ISUPPORTS();
-
     m_HelperAppLauncher = aLauncher;
     m_HandleContentOp = aHandleContentOp;
     m_FileName = aFileName;
@@ -586,16 +569,16 @@ BOOL CProgressDlg::OnInitDialog()
         {
             nsCAutoString uriString;
             srcUri->GetSpec(uriString);
-
-            m_SavingFrom.SetWindowText(uriString.get());
+            USES_CONVERSION;
+            m_SavingFrom.SetWindowText(A2CT(uriString.get()));
         }
     }
 
     // Set the "Action" field
     if(m_HandleContentOp == CONTENT_SAVE_TO_DISK)
-        m_Action.SetWindowText("[Saving file to:] " + m_FileName);
+        m_Action.SetWindowText(_T("[Saving file to:] ") + m_FileName);
     else if(m_HandleContentOp == CONTENT_LAUNCH_WITH_APP)
-        m_Action.SetWindowText("[Opening file with:] " + m_FileName);
+        m_Action.SetWindowText(_T("[Opening file with:] ") + m_FileName);
 
     return TRUE;
 }
