@@ -1,23 +1,31 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: Mozilla-sample-code 1.0
+ *
+ * Copyright (c) 2002 Netscape Communications Corporation and
+ * other contributors
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this Mozilla sample software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to permit
+ * persons to whom the Software is furnished to do so, subject to the
+ * following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  *
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- * 
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
- * 
- * The Original Code is the Mozilla browser.
- * 
- * The Initial Developer of the Original Code is Netscape
- * Communications, Inc.  Portions created by Netscape are
- * Copyright (C) 1999, Mozilla.  All Rights Reserved.
- * 
  * Contributor(s):
- */
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 // Mozilla Includes
 #include "nsIGenericFactory.h"
@@ -36,6 +44,7 @@
 #include "nsCWebBrowser.h"
 #include "nsWidgetsCID.h"
 #include "nsIProfileChangeStatus.h"
+#include "nsCRT.h"
 
 // Local includes
 #include "resource.h"
@@ -46,7 +55,6 @@
 
 WebBrowserChrome::WebBrowserChrome()
 {
-    NS_INIT_REFCNT();
     mNativeWindow = nsnull;
     mSizeSet = PR_FALSE;
 }
@@ -185,12 +193,13 @@ NS_IMETHODIMP WebBrowserChrome::DestroyBrowserWindow(void)
 }
 
 
-NS_IMETHODIMP WebBrowserChrome::SizeBrowserTo(PRInt32 aCX, PRInt32 aCY)
+// IN: The desired browser client area dimensions.
+NS_IMETHODIMP WebBrowserChrome::SizeBrowserTo(PRInt32 aWidth, PRInt32 aHeight)
 {
   /* This isn't exactly correct: we're setting the whole window to
      the size requested for the browser. At time of writing, though,
      it's fine and useful for winEmbed's purposes. */
-  WebBrowserChromeUI::SizeTo(this, aCX, aCY);
+  WebBrowserChromeUI::SizeTo(this, aWidth, aHeight);
   mSizeSet = PR_TRUE;
   return NS_OK;
 }
@@ -249,7 +258,7 @@ NS_IMETHODIMP WebBrowserChrome::OnProgressChange(nsIWebProgress *progress, nsIRe
 }
 
 NS_IMETHODIMP WebBrowserChrome::OnStateChange(nsIWebProgress *progress, nsIRequest *request,
-                                               PRInt32 progressStateFlags, PRUint32 status)
+                                               PRUint32 progressStateFlags, nsresult status)
 {
     if ((progressStateFlags & STATE_START) && (progressStateFlags & STATE_IS_DOCUMENT))
     {
@@ -303,9 +312,9 @@ WebBrowserChrome::OnStatusChange(nsIWebProgress* aWebProgress,
 NS_IMETHODIMP 
 WebBrowserChrome::OnSecurityChange(nsIWebProgress *aWebProgress, 
                                     nsIRequest *aRequest, 
-                                    PRInt32 state)
+                                    PRUint32 state)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 //*****************************************************************************
@@ -363,10 +372,10 @@ WebBrowserChrome::OnHistoryPurge(PRInt32 aNumEntries, PRBool *aContinue)
 nsresult
 WebBrowserChrome::SendHistoryStatusMessage(nsIURI * aURI, char * operation, PRInt32 info1, PRUint32 aReloadFlags)
 {
-    nsXPIDLCString uriCStr;
+    nsCAutoString uriCStr;
     if (aURI)
     {
-        aURI->GetSpec(getter_Copies(uriCStr));
+        aURI->GetSpec(uriCStr);
     }
 
     nsString uriAStr;
