@@ -58,7 +58,6 @@ inFlasher::inFlasher() :
   mThickness(0),
   mInvert(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
   mCSSUtils = do_GetService(kInspectorCSSUtilsCID);
 }
 
@@ -152,9 +151,8 @@ inFlasher::RepaintElement(nsIDOMElement* aElement)
   nsIFrame* parentWithView = nsnull;
   frame->GetParentWithView(pcontext, &parentWithView);
   if (parentWithView) {
-    nsIView* view = nsnull;
-    nsresult rv= parentWithView->GetView(pcontext, &view);
-    if (NS_SUCCEEDED(rv) && view) {
+    nsIView* view = parentWithView->GetViewExternal(pcontext);
+    if (view) {
       nsCOMPtr<nsIViewManager> viewManager;
       view->GetViewManager(*getter_AddRefs(viewManager));
       if (viewManager) {
