@@ -103,7 +103,7 @@ AttributeValueTemplate* ExprParser::createAttributeValueTemplate
                     }
                     else {
                         if (!buffer.IsEmpty()) {
-                            Expr* strExpr = new StringExpr(buffer);
+                            Expr* strExpr = new txLiteralExpr(buffer);
                             if (!strExpr) {
                                 // XXX ErrorReport: out of memory
                                 delete avt;
@@ -154,7 +154,7 @@ AttributeValueTemplate* ExprParser::createAttributeValueTemplate
     }
 
     if (!buffer.IsEmpty()) {
-        Expr* strExpr = new StringExpr(buffer);
+        Expr* strExpr = new txLiteralExpr(buffer);
         if (!strExpr) {
             // XXX ErrorReport: out of memory
             delete avt;
@@ -351,11 +351,11 @@ Expr* ExprParser::createFilterExpr(ExprLexer& lexer, txIParseContext* aContext)
             }
             break;
         case Token::LITERAL :
-            expr = new StringExpr(tok->value);
+            expr = new txLiteralExpr(tok->value);
             break;
         case Token::NUMBER:
         {
-            expr = new NumberExpr(Double::toDouble(tok->value));
+            expr = new txLiteralExpr(Double::toDouble(tok->value));
             break;
         }
         default:
@@ -499,17 +499,16 @@ Expr* ExprParser::createFunctionCall(ExprLexer& lexer,
         }
         rv = aContext->resolveFunctionCall(lName, namespaceID, fnCall);
 
-        // XXX this should be removed once we don't return
-        // NS_ERROR_NOT_IMPLEMENTED for unparsed-entity-uri(). As should the
-        // code in parseParameters that deals with fnCall = 0.
-        // Bug 65981
-        if (rv == NS_ERROR_NOT_IMPLEMENTED) {
+        // XXX We should have an errorfunction that always fails
+        // and use that here
+        if (rv == NS_ERROR_NOT_IMPLEMENTED ||
+            rv == NS_ERROR_XPATH_UNKNOWN_FUNCTION) {
             NS_ASSERTION(!fnCall, "Now is it implemented or not?");
             if (!parseParameters(0, lexer, aContext)) {
                 return 0;
             }
-            return new StringExpr(tok->value +
-                                  NS_LITERAL_STRING(" not implemented."));
+            return new txLiteralExpr(tok->value +
+                                     NS_LITERAL_STRING(" not implemented."));
         }
 
         if (NS_FAILED(rv)) {
@@ -537,7 +536,7 @@ LocationStep* ExprParser::createLocationStep(ExprLexer& lexer,
 {
     //-- child axis is default
     LocationStep::LocationStepType axisIdentifier = LocationStep::CHILD_AXIS;
-    txNodeTest* nodeTest = 0;
+    nsAutoPtr<txNodeTest> nodeTest;
 
     //-- get Axis Identifier or AbbreviatedStep, if present
     Token* tok = lexer.peek();
@@ -643,7 +642,8 @@ LocationStep* ExprParser::createLocationStep(ExprLexer& lexer,
                     nsresult rv = resolveQName(tok->value,
                                                getter_AddRefs(prefix),
                                                aContext,
-                                               getter_AddRefs(lName), nspace);
+                                               getter_AddRefs(lName), nspace,
+                                               PR_TRUE);
                     if (NS_FAILED(rv)) {
                         // XXX error report namespace resolve failed
                         return 0;
@@ -676,7 +676,6 @@ LocationStep* ExprParser::createLocationStep(ExprLexer& lexer,
     LocationStep* lstep = new LocationStep(nodeTest, axisIdentifier);
     if (!lstep) {
         //XXX out of memory
-        delete nodeTest;
         return 0;
     }
 
@@ -1012,7 +1011,8 @@ short ExprParser::precedenceLevel(short tokenType) {
 
 nsresult ExprParser::resolveQName(const nsAString& aQName,
                                   nsIAtom** aPrefix, txIParseContext* aContext,
-                                  nsIAtom** aLocalName, PRInt32& aNamespace)
+                                  nsIAtom** aLocalName, PRInt32& aNamespace,
+                                  PRBool aIsNameTest)
 {
     aNamespace = kNameSpaceID_None;
     PRInt32 idx = aQName.FindChar(':');
@@ -1031,7 +1031,14 @@ nsresult ExprParser::resolveQName(const nsAString& aQName,
     }
     // the lexer dealt with idx == 0
     *aPrefix = 0;
-    *aLocalName = NS_NewAtom(aQName);
+    if (aIsNameTest && aContext->caseInsensitiveNameTests()) {
+        nsAutoString lcname;
+        TX_ToLowerCase(aQName, lcname);
+        *aLocalName = NS_NewAtom(lcname);
+    }
+    else {
+        *aLocalName = NS_NewAtom(aQName);
+    }
     if (!*aLocalName) {
         return NS_ERROR_OUT_OF_MEMORY;
     }
