@@ -40,6 +40,8 @@
 
 #include "nsIDocument.h"
 #include "nsIDocShell.h"
+#include "nsIScriptLoader.h"
+#include "nsIDOMDocument.h"
 #include "nsIDOMComment.h"
 #include "nsIDOMDocumentType.h"
 #include "nsIDOMDOMImplementation.h"
@@ -53,7 +55,6 @@
 #include "nsIRefreshURI.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsITextContent.h"
-#include "nsIDocumentTransformer.h"
 #include "nsIXMLContent.h"
 #include "nsContentCID.h"
 #include "nsNetUtil.h"
@@ -63,6 +64,12 @@
 #include "nsIConsoleService.h"
 #include "nsIDOMDocumentFragment.h"
 #include "nsINameSpaceManager.h"
+#include "nsICSSStyleSheet.h"
+#include "txStringUtils.h"
+#include "txURIUtils.h"
+#include "nsIHTMLDocument.h"
+#include "nsIStyleSheetLinkingElement.h"
+#include "nsIDocumentTransformer.h"
 
 extern nsINameSpaceManager* gTxNameSpaceManager;
 
@@ -76,7 +83,7 @@ static NS_DEFINE_CID(kHTMLDocumentCID, NS_HTMLDOCUMENT_CID);
     if (!mCurrentNode)                                  \
         return
 
-txMozillaXMLOutput::txMozillaXMLOutput(const String& aRootName,
+txMozillaXMLOutput::txMozillaXMLOutput(const nsAString& aRootName,
                                        PRInt32 aRootNsID,
                                        txOutputFormat* aFormat,
                                        nsIDOMDocument* aSourceDocument,
@@ -86,15 +93,18 @@ txMozillaXMLOutput::txMozillaXMLOutput(const String& aRootName,
       mDontAddCurrent(PR_FALSE),
       mHaveTitleElement(PR_FALSE),
       mHaveBaseElement(PR_FALSE),
-      mInTransform(PR_FALSE),
       mCreatingNewDocument(PR_TRUE)
 {
-    NS_INIT_ISUPPORTS();
+    if (aObserver) {
+        mNotifier = new txTransformNotifier();
+        if (mNotifier) {
+            mNotifier->Init(aObserver);
+        }
+    }
+
     mOutputFormat.merge(*aFormat);
     mOutputFormat.setFromDefaults();
 
-    mObserver = do_GetWeakReference(aObserver);
-
     createResultDocument(aRootName, aRootNsID, aSourceDocument, aResultDocument);
 }
 
@@ -104,15 +114,16 @@ txMozillaXMLOutput::txMozillaXMLOutput(txOutputFormat* aFormat,
       mDontAddCurrent(PR_FALSE),
       mHaveTitleElement(PR_FALSE),
       mHaveBaseElement(PR_FALSE),
-      mInTransform(PR_FALSE),
       mCreatingNewDocument(PR_FALSE)
 {
-    NS_INIT_ISUPPORTS();
     mOutputFormat.merge(*aFormat);
     mOutputFormat.setFromDefaults();
 
     aFragment->GetOwnerDocument(getter_AddRefs(mDocument));
 
+    nsCOMPtr<nsIDocument> doc = do_QueryInterface(mDocument);
+    mDocumentIsHTML = doc && !doc->IsCaseSensitive();
+
     mCurrentNode = aFragment;
 }
 
@@ -120,13 +131,9 @@ txMozillaXMLOutput::~txMozillaXMLOutput()
 {
 }
 
-NS_IMPL_ISUPPORTS2(txMozillaXMLOutput,
-                   txIOutputXMLEventHandler,
-                   nsIScriptLoaderObserver);
-
-void txMozillaXMLOutput::attribute(const String& aName,
+void txMozillaXMLOutput::attribute(const nsAString& aName,
                                    const PRInt32 aNsID,
-                                   const String& aValue)
+                                   const nsAString& aValue)
 {
     if (!mParentNode)
         // XXX Signal this? (can't add attributes after element closed)
@@ -144,9 +151,9 @@ void txMozillaXMLOutput::attribute(const String& aName,
 
     if ((mOutputFormat.mMethod == eHTMLOutput) && (aNsID == kNameSpaceID_None)) {
         // Outputting HTML as XHTML, lowercase attribute names
-        nsAutoString lowerName(aName);
-        ToLowerCase(lowerName);
-        element->SetAttributeNS(NS_LITERAL_STRING(""), lowerName,
+        nsAutoString lowerName;
+        TX_ToLowerCase(aName, lowerName);
+        element->SetAttributeNS(nsString(), lowerName,
                                 aValue);
     }
     else {
@@ -156,7 +163,7 @@ void txMozillaXMLOutput::attribute(const String& aName,
     }
 }
 
-void txMozillaXMLOutput::characters(const String& aData)
+void txMozillaXMLOutput::characters(const nsAString& aData, PRBool aDOE)
 {
     closePrevious(eCloseElement);
 
@@ -167,7 +174,7 @@ void txMozillaXMLOutput::characters(const String& aData)
     mText.Append(aData);
 }
 
-void txMozillaXMLOutput::comment(const String& aData)
+void txMozillaXMLOutput::comment(const nsAString& aData)
 {
     closePrevious(eCloseElement | eFlushText);
 
@@ -193,7 +200,7 @@ void txMozillaXMLOutput::endDocument()
     if (mCreatingNewDocument && !mHaveTitleElement) {
         nsCOMPtr<nsIDOMNSDocument> domDoc = do_QueryInterface(mDocument);
         if (domDoc) {
-            domDoc->SetTitle(NS_LITERAL_STRING(""));
+            domDoc->SetTitle(nsString());
         }
     }
 
@@ -207,17 +214,18 @@ void txMozillaXMLOutput::endDocument()
             nsCOMPtr<nsIRefreshURI> refURI = do_QueryInterface(docShell);
             if (refURI) {
                 nsCOMPtr<nsIURI> baseURI;
-                doc->GetBaseURL(*getter_AddRefs(baseURI));
+                doc->GetBaseURL(getter_AddRefs(baseURI));
                 refURI->SetupRefreshURIFromHeader(baseURI, mRefreshString);
             }
         }
     }
 
-    mInTransform = PR_FALSE;
-    SignalTransformEnd();
+    if (mNotifier) {
+        mNotifier->OnTransformEnd();
+    }
 }
 
-void txMozillaXMLOutput::endElement(const String& aName, const PRInt32 aNsID)
+void txMozillaXMLOutput::endElement(const nsAString& aName, const PRInt32 aNsID)
 {
     TX_ENSURE_CURRENTNODE;
 
@@ -279,7 +287,7 @@ void txMozillaXMLOutput::getOutputDocument(nsIDOMDocument** aDocument)
     NS_IF_ADDREF(*aDocument);
 }
 
-void txMozillaXMLOutput::processingInstruction(const String& aTarget, const String& aData)
+void txMozillaXMLOutput::processingInstruction(const nsAString& aTarget, const nsAString& aData)
 {
     if (mOutputFormat.mMethod == eHTMLOutput)
         return;
@@ -312,25 +320,25 @@ void txMozillaXMLOutput::processingInstruction(const String& aTarget, const Stri
 
     if (ssle) {
         ssle->SetEnableUpdates(PR_TRUE);
-        ssle->UpdateStyleSheet(nsnull);
-    }
-}
-
-void txMozillaXMLOutput::removeScriptElement(nsIDOMHTMLScriptElement *aElement)
-{
-    if (mScriptElements) {
-        PRInt32 index = mScriptElements->IndexOf(aElement);
-        if (index > -1)
-            mScriptElements->RemoveElementAt(index);
+        rv = ssle->UpdateStyleSheet(nsnull, mNotifier);
+        if (rv == NS_ERROR_HTMLPARSER_BLOCK) {
+            nsCOMPtr<nsIStyleSheet> stylesheet;
+            ssle->GetStyleSheet(*getter_AddRefs(stylesheet));
+            if (mNotifier) {
+                mNotifier->AddStyleSheet(stylesheet);
+            }
+        }
     }
 }
 
 void txMozillaXMLOutput::startDocument()
 {
-    mInTransform = PR_TRUE;
+    if (mNotifier) {
+        mNotifier->OnTransformStart();
+    }
 }
 
-void txMozillaXMLOutput::startElement(const String& aName,
+void txMozillaXMLOutput::startElement(const nsAString& aName,
                                       const PRInt32 aNsID)
 {
     TX_ENSURE_CURRENTNODE;
@@ -358,8 +366,17 @@ void txMozillaXMLOutput::startElement(const String& aName,
     mDontAddCurrent = PR_FALSE;
 
     if ((mOutputFormat.mMethod == eHTMLOutput) && (aNsID == kNameSpaceID_None)) {
-        rv = mDocument->CreateElement(aName,
-                                      getter_AddRefs(element));
+        if (mDocumentIsHTML) {
+            rv = mDocument->CreateElement(aName,
+                                          getter_AddRefs(element));
+        }
+        else {
+            nsAutoString lcname;
+            ToLowerCase(aName, lcname);
+            rv = mDocument->CreateElementNS(NS_LITERAL_STRING(kXHTMLNameSpaceURI),
+                                            lcname,
+                                            getter_AddRefs(element));
+        }
         if (NS_FAILED(rv)) {
             return;
         }
@@ -415,7 +432,7 @@ void txMozillaXMLOutput::closePrevious(PRInt8 aAction)
             PRInt32 childCount, i;
             document->GetChildCount(childCount);
             for (i = 0; i < childCount; ++i) {
-                document->ChildAt(0, *getter_AddRefs(childContent));
+                document->ChildAt(0, getter_AddRefs(childContent));
                 if (childContent == mRootContent) {
                     document->SetRootContent(nsnull);
                 }
@@ -467,7 +484,7 @@ void txMozillaXMLOutput::closePrevious(PRInt8 aAction)
         NS_ASSERTION(NS_SUCCEEDED(rv), "Can't create text node");
 
         nsCOMPtr<nsIDOMNode> resultNode;
-        mCurrentNode->AppendChild(text, getter_AddRefs(resultNode));
+        rv = mCurrentNode->AppendChild(text, getter_AddRefs(resultNode));
         NS_ASSERTION(NS_SUCCEEDED(rv), "Can't append text node");
 
         mText.Truncate();
@@ -478,19 +495,36 @@ void txMozillaXMLOutput::startHTMLElement(nsIDOMElement* aElement)
 {
     nsCOMPtr<nsIAtom> atom;
     nsCOMPtr<nsIContent> content = do_QueryInterface(aElement);
-    content->GetTag(*getter_AddRefs(atom));
+    content->GetTag(getter_AddRefs(atom));
 
     mDontAddCurrent = (atom == txHTMLAtoms::script);
 
-    if (mCreatingNewDocument) {
+    if (atom == txHTMLAtoms::head) {
+        // Insert META tag, according to spec, 16.2, like
+        // <META http-equiv="Content-Type" content="text/html; charset=EUC-JP">
+        nsresult rv;
+        nsCOMPtr<nsIDOMNode> foo;
+        nsCOMPtr<nsIDOMElement> meta;
+        rv = mDocument->CreateElement(NS_LITERAL_STRING("meta"),
+                                      getter_AddRefs(meta));
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Can't create meta element");
+        rv = meta->SetAttribute(NS_LITERAL_STRING("http-equiv"),
+                                NS_LITERAL_STRING("Content-Type"));
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Can't set http-equiv on meta");
+        nsAutoString metacontent;
+        metacontent.Append(mOutputFormat.mMediaType);
+        metacontent.Append(NS_LITERAL_STRING("; charset="));
+        metacontent.Append(mOutputFormat.mEncoding);
+        rv = meta->SetAttribute(NS_LITERAL_STRING("content"),
+                                metacontent);
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Can't set content on meta");
+        rv = aElement->AppendChild(meta, getter_AddRefs(foo));
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Can't append meta");
+    }
+    else if (mCreatingNewDocument) {
         nsCOMPtr<nsIStyleSheetLinkingElement> ssle =
             do_QueryInterface(aElement);
         if (ssle) {
-            // XXX Trick nsCSSLoader into blocking/notifying us?
-            //     We would need to implement nsIParser and
-            //     pass ourselves as first parameter to
-            //     InitStyleLinkElement. We would then be notified
-            //     of stylesheet loads/load failures.
             ssle->InitStyleLinkElement(nsnull, PR_FALSE);
             ssle->SetEnableUpdates(PR_FALSE);
         }
@@ -505,7 +539,7 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
     NS_ASSERTION(content, "Can't QI to nsIContent");
 
     nsCOMPtr<nsIAtom> atom;
-    content->GetTag(*getter_AddRefs(atom));
+    content->GetTag(getter_AddRefs(atom));
 
     // add <tbody> to tables if there is none (not in xhtml)
     if (atom == txHTMLAtoms::table && !aXHTML) {
@@ -516,7 +550,7 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
 
         content->ChildCount(count);
         while (!section && (i < count)) {
-            rv = content->ChildAt(i, *getter_AddRefs(childContent));
+            rv = content->ChildAt(i, getter_AddRefs(childContent));
             NS_ASSERTION(NS_SUCCEEDED(rv), "Something went wrong while getting a child");
             section = do_QueryInterface(childContent);
             ++i;
@@ -526,9 +560,15 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
             // If no section, wrap table's children in a tbody.
             nsCOMPtr<nsIDOMElement> wrapper;
 
-            rv = mDocument->CreateElementNS(NS_LITERAL_STRING(kXHTMLNameSpaceURI),
-                                            NS_LITERAL_STRING("tbody"),
-                                            getter_AddRefs(wrapper));
+            if (mDocumentIsHTML) {
+                rv = mDocument->CreateElement(NS_LITERAL_STRING("tbody"),
+                                              getter_AddRefs(wrapper));
+            }
+            else {
+                rv = mDocument->CreateElementNS(NS_LITERAL_STRING(kXHTMLNameSpaceURI),
+                                                NS_LITERAL_STRING("tbody"),
+                                                getter_AddRefs(wrapper));
+            }
             NS_ASSERTION(NS_SUCCEEDED(rv), "Can't create tbody element");
 
             if (wrapper) {
@@ -541,17 +581,12 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
         }
     }
     // Load scripts
-    else if (mCreatingNewDocument &&
-             atom == txHTMLAtoms::script) {
+    else if (mNotifier && atom == txHTMLAtoms::script) {
         // Add this script element to the array of loading script elements.
         nsCOMPtr<nsIDOMHTMLScriptElement> scriptElement =
             do_QueryInterface(mCurrentNode);
         NS_ASSERTION(scriptElement, "Need script element");
-        if (!mScriptElements)
-            NS_NewISupportsArray(getter_AddRefs(mScriptElements));
-        NS_ASSERTION(mScriptElements, "Can't create array");
-        if (mScriptElements)
-            mScriptElements->AppendElement(scriptElement);
+        mNotifier->AddScriptElement(scriptElement);
     }
     // Set document title
     else if (mCreatingNewDocument &&
@@ -568,8 +603,8 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
             domDoc->SetTitle(text);
         }
     }
-    else if (mCreatingNewDocument &&
-             atom == txHTMLAtoms::base && !mHaveBaseElement) {
+    else if (mCreatingNewDocument && atom == txHTMLAtoms::base &&
+             !mHaveBaseElement) {
         // The first base wins
         mHaveBaseElement = PR_TRUE;
 
@@ -586,8 +621,7 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
             return;
         doc->SetBaseURL(baseURI); // The document checks if it is legal to set this base
     }
-    else if (mCreatingNewDocument &&
-             atom == txHTMLAtoms::meta) {
+    else if (mCreatingNewDocument && atom == txHTMLAtoms::meta) {
         // handle HTTP-EQUIV data
         nsAutoString httpEquiv;
         content->GetAttr(kNameSpaceID_None, txHTMLAtoms::httpEquiv, httpEquiv);
@@ -599,7 +633,7 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
         if (value.IsEmpty())
             return;
         
-        ToLowerCase(httpEquiv);
+        TX_ToLowerCase(httpEquiv);
         nsCOMPtr<nsIAtom> header = dont_AddRef(NS_NewAtom(httpEquiv));
         processHTTPEquiv(header, value);
     }
@@ -610,7 +644,14 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
             do_QueryInterface(aElement);
         if (ssle) {
             ssle->SetEnableUpdates(PR_TRUE);
-            ssle->UpdateStyleSheet(nsnull);
+            rv = ssle->UpdateStyleSheet(nsnull, mNotifier);
+            if (rv == NS_ERROR_HTMLPARSER_BLOCK) {
+                nsCOMPtr<nsIStyleSheet> stylesheet;
+                ssle->GetStyleSheet(*getter_AddRefs(stylesheet));
+                if (mNotifier) {
+                    mNotifier->AddStyleSheet(stylesheet);
+                }
+            }
         }
     }
 }
@@ -645,7 +686,7 @@ void txMozillaXMLOutput::wrapChildren(nsIDOMNode* aCurrentNode,
 }
 
 nsresult
-txMozillaXMLOutput::createResultDocument(const String& aName, PRInt32 aNsID,
+txMozillaXMLOutput::createResultDocument(const nsAString& aName, PRInt32 aNsID,
                                          nsIDOMDocument* aSourceDocument,
                                          nsIDOMDocument* aResultDocument)
 {
@@ -657,48 +698,49 @@ txMozillaXMLOutput::createResultDocument(const String& aName, PRInt32 aNsID,
         if (mOutputFormat.mMethod == eHTMLOutput) {
             doc = do_CreateInstance(kHTMLDocumentCID, &rv);
             NS_ENSURE_SUCCESS(rv, rv);
+
+            mDocumentIsHTML = PR_TRUE;
         }
         else {
             // We should check the root name/namespace here and create the
             // appropriate document
             doc = do_CreateInstance(kXMLDocumentCID, &rv);
             NS_ENSURE_SUCCESS(rv, rv);
+
+            mDocumentIsHTML = PR_FALSE;
         }
         mDocument = do_QueryInterface(doc);
     }
     else {
         mDocument = aResultDocument;
         doc = do_QueryInterface(aResultDocument);
+        
+        nsCOMPtr<nsIDocument> doc = do_QueryInterface(aResultDocument);
+        mDocumentIsHTML = doc && !doc->IsCaseSensitive();
     }
+
     mCurrentNode = mDocument;
 
     // Reset and set up the document
-    nsCOMPtr<nsILoadGroup> loadGroup;
-    nsCOMPtr<nsIChannel> channel;
-    nsCOMPtr<nsIDocument> sourceDoc = do_QueryInterface(aSourceDocument);
-    sourceDoc->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
-    nsCOMPtr<nsIIOService> serv = do_GetService(NS_IOSERVICE_CONTRACTID);
-    if (serv) {
-        // Create a temporary channel to get nsIDocument->Reset to
-        // do the right thing. We want the output document to get
-        // much of the input document's characteristics.
-        nsCOMPtr<nsIURI> docURL;
-        sourceDoc->GetDocumentURL(getter_AddRefs(docURL));
-        serv->NewChannelFromURI(docURL, getter_AddRefs(channel));
+    URIUtils::ResetWithSource(doc, aSourceDocument);
+
+    // Set the mime-type
+    if (!mOutputFormat.mMediaType.IsEmpty()) {
+        doc->SetContentType(mOutputFormat.mMediaType);
+    }
+    else if (mOutputFormat.mMethod == eHTMLOutput) {
+        doc->SetContentType(NS_LITERAL_STRING("text/html"));
+    }
+    else {
+        doc->SetContentType(NS_LITERAL_STRING("text/xml"));
     }
-    doc->Reset(channel, loadGroup);
-    nsCOMPtr<nsIURI> baseURL;
-    sourceDoc->GetBaseURL(*getter_AddRefs(baseURL));
-    doc->SetBaseURL(baseURL);
-    // XXX We might want to call SetDefaultStylesheets here
 
     // Set up script loader of the result document.
     nsCOMPtr<nsIScriptLoader> loader;
     doc->GetScriptLoader(getter_AddRefs(loader));
-    nsCOMPtr<nsITransformObserver> observer = do_QueryReferent(mObserver);
     if (loader) {
-        if (observer) {
-            loader->AddObserver(this);
+        if (mNotifier) {
+            loader->AddObserver(mNotifier);
         }
         else {
             // Don't load scripts, we can't notify the caller when they're loaded.
@@ -706,13 +748,19 @@ txMozillaXMLOutput::createResultDocument(const String& aName, PRInt32 aNsID,
         }
     }
 
-    // Notify the contentsink that the document is created
-    if (observer) {
-        observer->OnDocumentCreated(mDocument);
+    if (mNotifier) {
+        mNotifier->SetOutputDocument(mDocument);
+    }
+
+    // Do this after calling OnDocumentCreated to ensure that the
+    // PresShell/PresContext has been hooked up and get notified.
+    nsCOMPtr<nsIHTMLDocument> htmlDoc = do_QueryInterface(doc);
+    if (htmlDoc) {
+        htmlDoc->SetCompatibilityMode(eCompatibility_FullStandards);
     }
 
     // Add a doc-type if requested
-    if (!mOutputFormat.mSystemId.isEmpty()) {
+    if (!mOutputFormat.mSystemId.IsEmpty()) {
         nsCOMPtr<nsIDOMDOMImplementation> implementation;
         rv = aSourceDocument->GetImplementation(getter_AddRefs(implementation));
         NS_ENSURE_SUCCESS(rv, rv);
@@ -721,7 +769,7 @@ txMozillaXMLOutput::createResultDocument(const String& aName, PRInt32 aNsID,
             qName.Assign(NS_LITERAL_STRING("html"));
         }
         else {
-            qName.Assign(aName.getConstNSString());
+            qName.Assign(aName);
         }
         nsCOMPtr<nsIDOMDocumentType> documentType;
         rv = implementation->CreateDocumentType(qName,
@@ -736,17 +784,31 @@ txMozillaXMLOutput::createResultDocument(const String& aName, PRInt32 aNsID,
     return NS_OK;
 }
 
+txTransformNotifier::txTransformNotifier()
+    : mInTransform(PR_FALSE)
+      
+{
+}
+
+txTransformNotifier::~txTransformNotifier()
+{
+}
+
+NS_IMPL_ISUPPORTS2(txTransformNotifier,
+                   nsIScriptLoaderObserver,
+                   nsICSSLoaderObserver);
+
 NS_IMETHODIMP
-txMozillaXMLOutput::ScriptAvailable(nsresult aResult, 
-                                    nsIDOMHTMLScriptElement *aElement, 
-                                    PRBool aIsInline,
-                                    PRBool aWasPending,
-                                    nsIURI *aURI, 
-                                    PRInt32 aLineNo,
-                                    const nsAString& aScript)
+txTransformNotifier::ScriptAvailable(nsresult aResult, 
+                                     nsIDOMHTMLScriptElement *aElement, 
+                                     PRBool aIsInline,
+                                     PRBool aWasPending,
+                                     nsIURI *aURI, 
+                                     PRInt32 aLineNo,
+                                     const nsAString& aScript)
 {
     if (NS_FAILED(aResult)) {
-        removeScriptElement(aElement);
+        mScriptElements.RemoveObject(aElement);
         SignalTransformEnd();
     }
 
@@ -754,42 +816,77 @@ txMozillaXMLOutput::ScriptAvailable(nsresult aResult,
 }
 
 NS_IMETHODIMP 
-txMozillaXMLOutput::ScriptEvaluated(nsresult aResult, 
-                                    nsIDOMHTMLScriptElement *aElement,
-                                    PRBool aIsInline,
-                                    PRBool aWasPending)
+txTransformNotifier::ScriptEvaluated(nsresult aResult, 
+                                     nsIDOMHTMLScriptElement *aElement,
+                                     PRBool aIsInline,
+                                     PRBool aWasPending)
 {
-    removeScriptElement(aElement);
+    mScriptElements.RemoveObject(aElement);
+    SignalTransformEnd();
+    return NS_OK;
+}
+
+NS_IMETHODIMP 
+txTransformNotifier::StyleSheetLoaded(nsICSSStyleSheet* aSheet, PRBool aNotify)
+{
+    // aSheet might not be in our list if the load was done synchronously
+    mStylesheets.RemoveObject(aSheet);
     SignalTransformEnd();
     return NS_OK;
 }
 
 void
-txMozillaXMLOutput::SignalTransformEnd()
+txTransformNotifier::Init(nsITransformObserver* aObserver)
 {
-    if (mInTransform) {
-        return;
-    }
+    mObserver = aObserver;
+}
 
-    nsCOMPtr<nsITransformObserver> observer = do_QueryReferent(mObserver);
-    if (!observer) {
-        return;
-    }
+void
+txTransformNotifier::AddScriptElement(nsIDOMHTMLScriptElement* aElement)
+{
+    mScriptElements.AppendObject(aElement);
+}
 
-    if (mScriptElements) {
-        PRUint32 scriptCount = 0;
-        mScriptElements->Count(&scriptCount);
-        if (scriptCount != 0) {
-            return;
-        }
+void
+txTransformNotifier::AddStyleSheet(nsIStyleSheet* aStyleSheet)
+{
+    mStylesheets.AppendObject(aStyleSheet);
+}
+
+void
+txTransformNotifier::OnTransformEnd()
+{
+    mInTransform = PR_FALSE;
+    SignalTransformEnd();
+}
+
+void
+txTransformNotifier::OnTransformStart()
+{
+    mInTransform = PR_TRUE;
+}
+
+void
+txTransformNotifier::SetOutputDocument(nsIDOMDocument* aDocument)
+{
+    mDocument = aDocument;
+
+    // Notify the contentsink that the document is created
+    mObserver->OnDocumentCreated(mDocument);
+}
+
+void
+txTransformNotifier::SignalTransformEnd()
+{
+    if (mInTransform || mScriptElements.Count() > 0 ||
+        mStylesheets.Count() > 0) {
+        return;
     }
 
     // Make sure that we don't get deleted while this function is executed and
     // we remove ourselfs from the scriptloader
     nsCOMPtr<nsIScriptLoaderObserver> kungFuDeathGrip(this);
 
-    mObserver = nsnull;
-
     // XXX Need a better way to determine transform success/failure
     if (mDocument) {
         nsCOMPtr<nsIScriptLoader> loader;
@@ -799,10 +896,10 @@ txMozillaXMLOutput::SignalTransformEnd()
             loader->RemoveObserver(this);
         }
 
-        observer->OnTransformDone(NS_OK, mDocument);
+        mObserver->OnTransformDone(NS_OK, mDocument);
     }
     else {
         // XXX Need better error message and code.
-        observer->OnTransformDone(NS_ERROR_FAILURE, nsnull);
+        mObserver->OnTransformDone(NS_ERROR_FAILURE, nsnull);
     }
 }
