@@ -548,6 +548,42 @@ nsTypeAheadFind::HandleEvent(nsIDOMEvent* aEvent)
   else if (eventType.Equals(NS_LITERAL_STRING("popuphidden"))) {
     mIsMenuPopupActive = PR_FALSE;
   }
+  else if (eventType.Equals(NS_LITERAL_STRING("unload"))) {
+    // When document is unloaded, check to see if it's the 
+    // current typeahead doc. If it is, cancel find
+    // and reset member variables so we don't leak
+    nsCOMPtr<nsIDOMNSEvent> event(do_QueryInterface(aEvent));
+    NS_ENSURE_TRUE(event, NS_ERROR_FAILURE);
+
+    nsCOMPtr<nsIDOMEventTarget> eventTarget;
+    event->GetOriginalTarget(getter_AddRefs(eventTarget));
+    nsCOMPtr<nsIDocument> doc(do_QueryInterface(eventTarget));
+    nsCOMPtr<nsIPresShell> focusedShell(do_QueryReferent(mFocusedWeakShell));
+    if (!focusedShell || !doc) {
+      return NS_ERROR_FAILURE;
+    }
+
+    PRInt32 numShells = doc->GetNumberOfShells();
+    nsCOMPtr<nsIPresShell> shellToBeDestroyed;
+    PRBool cancelFind = PR_FALSE;
+
+    for (PRInt32 count = 0; count < numShells; count ++) {
+      doc->GetShellAt(count, getter_AddRefs(shellToBeDestroyed));
+      if (shellToBeDestroyed == focusedShell) {
+        cancelFind = PR_TRUE;
+        break;
+      }
+    }
+
+    if (cancelFind) {
+      RemoveDocListeners();
+      mSearchRange = do_CreateInstance(kRangeCID);
+      mStartPointRange = do_CreateInstance(kRangeCID);
+      mEndPointRange = do_CreateInstance(kRangeCID);
+      mFocusedWindow = nsnull;
+      CancelFind();
+    }
+  }
 
   return NS_OK;
 }
@@ -664,7 +700,13 @@ nsTypeAheadFind::KeyPress(nsIDOMEvent* aEvent)
 
   // ----------- Back space -------------------------
   if (keyCode == nsIDOMKeyEvent::DOM_VK_BACK_SPACE) {
-    if (HandleBackspace()) {
+    // The order of seeing keystrokes is:
+    // 1) Chrome, 2) Typeahead, 3) [platform]HTMLBindings.xml
+    // If chrome handles backspace, it needs to do this work
+    // Otherwise, we handle backspace here.
+    PRBool backspaceUsed;
+    BackOneChar(&backspaceUsed);
+    if (backspaceUsed) {
       aEvent->PreventDefault(); // Prevent normal processing of this keystroke
     }
 
@@ -693,13 +735,15 @@ nsTypeAheadFind::KeyPress(nsIDOMEvent* aEvent)
 }
 
 
-PRBool
-nsTypeAheadFind::HandleBackspace()
+NS_IMETHODIMP
+nsTypeAheadFind::BackOneChar(PRBool *aIsBackspaceUsed)
 {
   // In normal type ahead find, remove a printable char from 
   // mTypeAheadBuffer, then search for buffer contents
   // Or, in repeated char find, go backwards
 
+  *aIsBackspaceUsed = PR_TRUE;
+
   // ---------- No chars in string ------------
   if (mTypeAheadBuffer.IsEmpty() || !mStartFindRange) {
     if (!mFindNextBuffer.IsEmpty() &&
@@ -722,9 +766,12 @@ nsTypeAheadFind::HandleBackspace()
           soundInterface->Beep(); // beep to warn
         }
         mIsBackspaceProtectOn = PR_FALSE;
-        return PR_TRUE;
       }
-      return PR_FALSE;
+      else {
+        *aIsBackspaceUsed = PR_FALSE;
+      }
+
+      return NS_OK;
     }
   }
 
@@ -742,7 +789,7 @@ nsTypeAheadFind::HandleBackspace()
     CancelFind();
     mIsBackspaceProtectOn = PR_TRUE;
 
-    return PR_TRUE;
+    return NS_OK;
   }
 
   // ---------- Multiple chars in string ----------
@@ -763,7 +810,7 @@ nsTypeAheadFind::HandleBackspace()
     --mBadKeysSinceMatch;
     DisplayStatus(PR_FALSE, nsnull, PR_FALSE); // Display failure status
     SaveFind();
-    return PR_TRUE;
+    return NS_OK;
   }
 
   mBadKeysSinceMatch = 0;
@@ -785,7 +832,8 @@ nsTypeAheadFind::HandleBackspace()
       }
     }
     if (!presShell) {
-      return PR_FALSE;
+      *aIsBackspaceUsed = PR_FALSE;
+      return NS_ERROR_FAILURE;
     }
     // Set the selection to the where the first character was found
     // so that find starts from there
@@ -808,7 +856,7 @@ nsTypeAheadFind::HandleBackspace()
 
   SaveFind();
 
-  return PR_TRUE;   // Backspace handled
+  return NS_OK;   // Backspace handled
 }
 
 
@@ -1545,7 +1593,7 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
 
   if (NS_SUCCEEDED(startContent->CanContainChildren(canContainChildren)) &&
       canContainChildren) {
-    startContent->ChildAt(startOffset, *getter_AddRefs(childContent));
+    startContent->ChildAt(startOffset, getter_AddRefs(childContent));
     if (childContent) {
       startContent = childContent;
     }
@@ -1604,9 +1652,9 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
 
     // Get the parent
     nsCOMPtr<nsIContent> parent, parentsFirstChild;
-    startContent->GetParent(*getter_AddRefs(parent));
+    startContent->GetParent(getter_AddRefs(parent));
     if (parent) {
-      parent->ChildAt(0, *getter_AddRefs(parentsFirstChild));
+      parent->ChildAt(0, getter_AddRefs(parentsFirstChild));
       nsCOMPtr<nsITextContent> textContent =
         do_QueryInterface(parentsFirstChild);
 
@@ -1615,7 +1663,7 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
         PRBool isOnlyWhitespace;
         textContent->IsOnlyWhitespace(&isOnlyWhitespace);
         if (isOnlyWhitespace)
-          parent->ChildAt(1, *getter_AddRefs(parentsFirstChild));
+          parent->ChildAt(1, getter_AddRefs(parentsFirstChild));
       }
 
       if (parentsFirstChild != startContent) {
@@ -2284,6 +2332,10 @@ nsTypeAheadFind::RemoveWindowListeners(nsIDOMWindow *aDOMWin)
                                           genericEventListener, 
                                           PR_TRUE);
 
+  chromeEventHandler->RemoveEventListener(NS_LITERAL_STRING("unload"), 
+                                          genericEventListener, 
+                                          PR_TRUE);
+
   // Remove DOM Text listener for IME text events
   nsCOMPtr<nsIDOMEventReceiver> chromeEventReceiver = 
     do_QueryInterface(chromeEventHandler);
@@ -2333,6 +2385,10 @@ nsTypeAheadFind::AttachWindowListeners(nsIDOMWindow *aDOMWin)
                                        genericEventListener, 
                                        PR_TRUE);
 
+  chromeEventHandler->AddEventListener(NS_LITERAL_STRING("unload"), 
+                                       genericEventListener, 
+                                       PR_TRUE);
+
   // Add DOM Text listener for IME text events
   nsCOMPtr<nsIDOMEventReceiver> chromeEventReceiver =
     do_QueryInterface(chromeEventHandler);
@@ -2387,7 +2443,7 @@ nsTypeAheadFind::IsTargetContentOkay(nsIContent *aContent)
     // having a table of atoms just for it. Instead, we're paying for 1 extra 
     // string compare per keystroke, which isn't too bad.
     nsCOMPtr<nsIAtom> targetTagAtom;
-    aContent->GetTag(*getter_AddRefs(targetTagAtom));
+    aContent->GetTag(getter_AddRefs(targetTagAtom));
     nsAutoString targetTagString;
     targetTagAtom->ToString(targetTagString);
     if (targetTagString.Equals(NS_LITERAL_STRING("isindex"))) {
@@ -2435,7 +2491,7 @@ nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent,
   // ---------- Is the keystroke in a new window? -------------------
 
   nsCOMPtr<nsIDocument> doc;
-  if (NS_FAILED(targetContent->GetDocument(*getter_AddRefs(doc))) || !doc) {
+  if (NS_FAILED(targetContent->GetDocument(getter_AddRefs(doc))) || !doc) {
     return NS_OK;
   }
 
@@ -2545,9 +2601,7 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
     return PR_FALSE;
   }
 
-  const nsStyleVisibility* vis = nsnull;
-  ::GetStyleData(frame, &vis);
-  if (!vis || !vis->IsVisible()) {
+  if (!frame->GetStyleVisibility()->IsVisible()) {
     return PR_FALSE;
   }
 
@@ -2820,6 +2874,10 @@ nsTypeAheadController::nsTypeAheadController(nsIFocusController *aFocusControlle
 {
 }
 
+nsTypeAheadController::~nsTypeAheadController()
+{
+}
+
 NS_IMETHODIMP
 nsTypeAheadController::IsCommandEnabled(const char *aCommand, PRBool *aResult)
 {
@@ -2884,8 +2942,6 @@ nsTypeAheadController::SupportsCommand(const char *aCommand, PRBool *aResult)
 NS_IMETHODIMP
 nsTypeAheadController::DoCommand(const char *aCommand)
 {
-  nsresult rv = NS_OK;
-
   PRBool isLinkSearch = PR_FALSE;
 
   if (nsCRT::strcmp(sLinkFindString, aCommand) == 0) {
