@@ -36,6 +36,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "jsapi.h"
 #include "nsDOMParser.h"
 #include "nsIURI.h"
 #include "nsIChannel.h"
@@ -51,27 +52,19 @@
 #include "nsIDocument.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMDOMImplementation.h"
+#include "nsIDOMWindow.h"
 #include "nsIPrivateDOMImplementation.h"
 #include "nsIJSContextStack.h"
 #include "nsIScriptSecurityManager.h"
-#include "nsICodebasePrincipal.h"
-#include "nsIDOMClassInfo.h"
-#include "nsReadableUtils.h"
-
-#ifdef IMPLEMENT_SYNC_LOAD
+#include "nsIPrincipal.h"
 #include "nsIScriptContext.h"
 #include "nsIScriptGlobalObject.h"
-#include "nsIDocShell.h"
-#include "nsIDocShellTreeItem.h"
-#include "nsIDocShellTreeOwner.h"
-#include "nsIEventQueueService.h"
-#include "nsIInterfaceRequestor.h"
-#include "nsIInterfaceRequestorUtils.h"
+#include "nsIDOMClassInfo.h"
+#include "nsReadableUtils.h"
+#include "nsCRT.h"
 #include "nsIDOMEventReceiver.h"
-#include "jsapi.h"
 #include "nsLoadListenerProxy.h"
 static NS_DEFINE_IID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
-#endif
 
 static const char* kLoadAsData = "loadAsData";
 
@@ -103,7 +96,6 @@ protected:
 
 nsDOMParserChannel::nsDOMParserChannel(nsIURI* aURI, const char* aContentType)
 {
-  NS_INIT_ISUPPORTS();
   mURI = aURI;
   mContentType.Assign(aContentType);
   mStatus = NS_OK;
@@ -119,7 +111,7 @@ NS_IMPL_ISUPPORTS2(nsDOMParserChannel,
                    nsIRequest)
 
 /* boolean isPending (); */
-NS_IMETHODIMP nsDOMParserChannel::GetName(PRUnichar* *result)
+NS_IMETHODIMP nsDOMParserChannel::GetName(nsACString &result)
 {
     NS_NOTREACHED("nsDOMParserChannel::GetName");
     return NS_ERROR_NOT_IMPLEMENTED;
@@ -188,20 +180,29 @@ NS_IMETHODIMP nsDOMParserChannel::GetURI(nsIURI * *aURI)
   return NS_OK;
 }
 
-/* attribute string contentType; */
-NS_IMETHODIMP nsDOMParserChannel::GetContentType(char * *aContentType)
+/* attribute ACString contentType; */
+NS_IMETHODIMP nsDOMParserChannel::GetContentType(nsACString &aContentType)
 {
-  NS_ENSURE_ARG_POINTER(aContentType);
-  *aContentType = ToNewCString(mContentType);
+  aContentType = mContentType;
   return NS_OK;
 }
-NS_IMETHODIMP nsDOMParserChannel::SetContentType(const char * aContentType)
+NS_IMETHODIMP nsDOMParserChannel::SetContentType(const nsACString &aContentType)
 {
-  NS_ENSURE_ARG(aContentType);
-  mContentType.Assign(aContentType);
+  mContentType = aContentType;
   return NS_OK;
 }
 
+/* attribute ACString contentCharset; */
+NS_IMETHODIMP nsDOMParserChannel::GetContentCharset(nsACString &aContentCharset)
+{
+  aContentCharset.Truncate();
+  return NS_OK;
+}
+NS_IMETHODIMP nsDOMParserChannel::SetContentCharset(const nsACString &aContentCharset)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 /* attribute long contentLength; */
 NS_IMETHODIMP nsDOMParserChannel::GetContentLength(PRInt32 *aContentLength)
 {
@@ -284,20 +285,6 @@ NS_IMETHODIMP nsDOMParserChannel::AsyncOpen(nsIStreamListener *listener, nsISupp
 //
 /////////////////////////////////////////////
 
-#ifdef IMPLEMENT_SYNC_LOAD
-// See if we have a modal event loop
-static inline PRBool IsModal(nsIWebBrowserChrome *aWindow)
-{
-  if (aWindow) {
-    PRBool isModal;
-    if (NS_SUCCEEDED(aWindow->IsWindowModal(&isModal))) {
-      return isModal;
-    }
-  }
-
-  return PR_FALSE;
-}
-
 // nsIDOMEventListener
 nsresult
 nsDOMParser::HandleEvent(nsIDOMEvent* aEvent)
@@ -309,11 +296,7 @@ nsDOMParser::HandleEvent(nsIDOMEvent* aEvent)
 nsresult
 nsDOMParser::Load(nsIDOMEvent* aEvent)
 {
-  if (IsModal(mChromeWindow)) {
-    mChromeWindow->ExitModalEventLoop(NS_OK);
-  }
-
-  mChromeWindow = nsnull;
+  mLoopingForSyncLoad = PR_FALSE;
 
   return NS_OK;
 }
@@ -327,11 +310,7 @@ nsDOMParser::Unload(nsIDOMEvent* aEvent)
 nsresult
 nsDOMParser::Abort(nsIDOMEvent* aEvent)
 {
-  if (IsModal(mChromeWindow)) {
-    mChromeWindow->ExitModalEventLoop(NS_OK);
-  }
-
-  mChromeWindow = nsnull;
+  mLoopingForSyncLoad = PR_FALSE;
 
   return NS_OK;
 }
@@ -339,28 +318,21 @@ nsDOMParser::Abort(nsIDOMEvent* aEvent)
 nsresult
 nsDOMParser::Error(nsIDOMEvent* aEvent)
 {
-  if (IsModal(mChromeWindow)) {
-    mChromeWindow->ExitModalEventLoop(NS_OK);
-  }
-
-  mChromeWindow = nsnull;
+  mLoopingForSyncLoad = PR_FALSE;
 
   return NS_OK;
 }
-#endif
 
 nsDOMParser::nsDOMParser()
+  : mLoopingForSyncLoad(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
+  mEventQService = do_GetService(kEventQueueServiceCID);
 }
 
 nsDOMParser::~nsDOMParser()
 {
-#ifdef IMPLEMENT_SYNC_LOAD
-  if (IsModal(mChromeWindow)) {
-    mChromeWindow->ExitModalEventLoop(NS_OK);
-  }
-#endif
+  NS_ABORT_IF_FALSE(!mLoopingForSyncLoad, "we rather crash than hang");
+  mLoopingForSyncLoad = PR_FALSE;
 }
 
 
@@ -368,11 +340,9 @@ nsDOMParser::~nsDOMParser()
 NS_INTERFACE_MAP_BEGIN(nsDOMParser)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDOMParser)
   NS_INTERFACE_MAP_ENTRY(nsIDOMParser)
-#ifdef IMPLEMENT_SYNC_LOAD
   NS_INTERFACE_MAP_ENTRY(nsIDOMLoadListener)
   NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
-#endif
-  NS_INTERFACE_MAP_ENTRY_DOM_CLASSINFO(DOMParser)
+  NS_INTERFACE_MAP_ENTRY_EXTERNAL_DOM_CLASSINFO(DOMParser)
 NS_INTERFACE_MAP_END
 
 
@@ -387,7 +357,6 @@ ConvertWStringToStream(const PRUnichar* aStr,
 {
   nsresult rv;
   nsCOMPtr<nsIUnicodeEncoder> encoder;
-  nsAutoString charsetStr;
   char* charBuf;
 
   // We want to encode the string as utf-8, so get the right encoder
@@ -395,8 +364,7 @@ ConvertWStringToStream(const PRUnichar* aStr,
            do_GetService(kCharsetConverterManagerCID, &rv);
   NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
   
-  charsetStr.Assign(NS_LITERAL_STRING("UTF-8"));
-  rv = charsetConv->GetUnicodeEncoder(&charsetStr,
+  rv = charsetConv->GetUnicodeEncoderRaw("UTF-8",
                                       getter_AddRefs(encoder));
   NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
   
@@ -472,10 +440,6 @@ nsDOMParser::ParseFromStream(nsIInputStream *stream,
   NS_ENSURE_ARG_POINTER(_retval);
   *_retval = nsnull;
 
-  nsresult rv;
-  nsCOMPtr<nsIURI> baseURI;
-  nsCOMPtr<nsIPrincipal> principal;
-
   // For now, we can only create XML documents.
   if (nsCRT::strcmp(contentType, "text/xml") != 0 &&
     nsCRT::strcmp(contentType, "application/xml") != 0 &&
@@ -483,7 +447,17 @@ nsDOMParser::ParseFromStream(nsIInputStream *stream,
     return NS_ERROR_NOT_IMPLEMENTED;
   }
 
-  // First try to find a base URI for the document we're creating
+  nsresult rv;
+  nsCOMPtr<nsIPrincipal> principal;
+  nsCOMPtr<nsIScriptSecurityManager> secMan = 
+    do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
+  if (NS_SUCCEEDED(rv)) {
+    secMan->GetSubjectPrincipal(getter_AddRefs(principal));
+  }
+
+  // Try to find a base URI for the document we're creating.
+  nsCOMPtr<nsIURI> baseURI;
+
   nsCOMPtr<nsIXPCNativeCallContext> cc;
   nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID(), &rv));
   if(NS_SUCCEEDED(rv)) {
@@ -494,15 +468,21 @@ nsDOMParser::ParseFromStream(nsIInputStream *stream,
     JSContext* cx;
     rv = cc->GetJSContext(&cx);
     if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-  
-    nsCOMPtr<nsIScriptSecurityManager> secMan = 
-             do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
-    if (NS_SUCCEEDED(rv)) {
-      rv = secMan->GetSubjectPrincipal(getter_AddRefs(principal));
-      if (NS_SUCCEEDED(rv)) {
-        nsCOMPtr<nsICodebasePrincipal> codebase(do_QueryInterface(principal));
-        if (codebase) {
-          codebase->GetURI(getter_AddRefs(baseURI));
+
+    nsCOMPtr<nsIScriptContext> scriptContext;
+    GetScriptContextFromJSContext(cx, getter_AddRefs(scriptContext));
+    if (scriptContext) {
+      nsCOMPtr<nsIScriptGlobalObject> globalObject;
+      scriptContext->GetGlobalObject(getter_AddRefs(globalObject));
+
+      nsCOMPtr<nsIDOMWindow> window = do_QueryInterface(globalObject);
+      if (window) {
+        nsCOMPtr<nsIDOMDocument> domdoc;
+        window->GetDocument(getter_AddRefs(domdoc));
+
+        nsCOMPtr<nsIDocument> doc = do_QueryInterface(domdoc);
+        if (doc) {
+          doc->GetBaseURL(getter_AddRefs(baseURI));
         }
       }
     }
@@ -541,7 +521,6 @@ nsDOMParser::ParseFromStream(nsIInputStream *stream,
                                       getter_AddRefs(domDocument));
   if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-#ifdef IMPLEMENT_SYNC_LOAD
   // Register as a load listener on the document
   nsCOMPtr<nsIDOMEventReceiver> target(do_QueryInterface(domDocument));
   if (target) {
@@ -555,7 +534,6 @@ nsDOMParser::ParseFromStream(nsIInputStream *stream,
                                        NS_GET_IID(nsIDOMLoadListener));
     if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
   }
-#endif
 
   // Create a fake channel 
   nsDOMParserChannel* parserChannel = new nsDOMParserChannel(baseURI, contentType);
@@ -573,67 +551,30 @@ nsDOMParser::ParseFromStream(nsIInputStream *stream,
   nsCOMPtr<nsIDocument> document(do_QueryInterface(domDocument));
   if (!document) return NS_ERROR_FAILURE;
 
-#ifdef IMPLEMENT_SYNC_LOAD
   nsCOMPtr<nsIEventQueue> modalEventQueue;
-  nsCOMPtr<nsIEventQueueService> eventQService;
-  
-  if (cc) {
-    JSContext* cx;
-    rv = cc->GetJSContext(&cx);
-    if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-    // We can only do this if we're called from a DOM script context
-    nsIScriptContext* scriptCX = (nsIScriptContext*)JS_GetContextPrivate(cx);
-    if (!scriptCX) return NS_OK;
+  if(!mEventQService) {
+    return NS_ERROR_FAILURE;
+  }
 
-    // Get the nsIDocShellTreeOwner associated with the window
-    // containing this script context
-    // XXX Need to find a better way to do this rather than
-    // chaining through a bunch of getters and QIs
-    nsCOMPtr<nsIScriptGlobalObject> global;
-    scriptCX->GetGlobalObject(getter_AddRefs(global));
-    if (!global) return NS_ERROR_FAILURE;
+  mLoopingForSyncLoad = PR_TRUE;
 
-    nsCOMPtr<nsIDocShell> docshell;
-    rv = global->GetDocShell(getter_AddRefs(docshell));
-    if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-    
-    nsCOMPtr<nsIDocShellTreeItem> item = do_QueryInterface(docshell);
-    if (!item) return NS_ERROR_FAILURE;
-
-    nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
-    rv = item->GetTreeOwner(getter_AddRefs(treeOwner));
-    if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-
-    nsCOMPtr<nsIInterfaceRequestor> treeRequestor(do_GetInterface(treeOwner));
-    if (!treeRequestor) return NS_ERROR_FAILURE;
-
-    treeRequestor->GetInterface(NS_GET_IID(nsIWebBrowserChrome), getter_AddRefs(mChromeWindow));
-    if (!mChromeWindow) return NS_ERROR_FAILURE;
-
-    eventQService = do_GetService(kEventQueueServiceCID);
-    if(!eventQService || 
-       NS_FAILED(eventQService->PushThreadEventQueue(getter_AddRefs(modalEventQueue)))) {
-      return NS_ERROR_FAILURE;
-    }
+  rv = mEventQService->PushThreadEventQueue(getter_AddRefs(modalEventQueue));
+  if (NS_FAILED(rv)) {
+    return rv;
   }
-#endif
 
   rv = document->StartDocumentLoad(kLoadAsData, channel, 
                                    nsnull, nsnull, 
                                    getter_AddRefs(listener),
                                    PR_FALSE);
 
-#ifdef IMPLEMENT_SYNC_LOAD
   if (NS_FAILED(rv) || !listener) {
     if (modalEventQueue) {
-      eventQService->PopThreadEventQueue(modalEventQueue);
+      mEventQService->PopThreadEventQueue(modalEventQueue);
     }
     return NS_ERROR_FAILURE;
   }
-#else
-  if (NS_FAILED(rv) || !listener) return NS_ERROR_FAILURE;
-#endif
 
   // Now start pumping data to the listener
   nsresult status;
@@ -647,29 +588,19 @@ nsDOMParser::ParseFromStream(nsIInputStream *stream,
   }
 
   rv = listener->OnStopRequest(request, nsnull, status);
-#ifdef IMPLEMENT_SYNC_LOAD
+
   if (NS_FAILED(rv)) {
     if (modalEventQueue) {
-      eventQService->PopThreadEventQueue(modalEventQueue);
+      mEventQService->PopThreadEventQueue(modalEventQueue);
     }
     return NS_ERROR_FAILURE;
   }
-#else
-  if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-#endif
 
-#ifdef IMPLEMENT_SYNC_LOAD
-  // Spin an event loop here and wait
-  if (mChromeWindow) {
-    rv = mChromeWindow->ShowAsModal();
-    
-    eventQService->PopThreadEventQueue(modalEventQueue);
-    
-    if (NS_FAILED(rv)) return NS_ERROR_FAILURE;      
-  } else if (modalEventQueue) {
-    eventQService->PopThreadEventQueue(modalEventQueue);
+  while (mLoopingForSyncLoad) {
+    modalEventQueue->ProcessPendingEvents();
   }
-#endif
+
+  mEventQService->PopThreadEventQueue(modalEventQueue);
 
   *_retval = domDocument;
   NS_ADDREF(*_retval);
