@@ -52,6 +52,7 @@
 #include "nsIDOMWindowInternal.h"
 #include "nsVoidArray.h"
 #include "nsSupportsArray.h"
+#include "prprf.h"
 
 // For Prefs
 #include "nsIPrefBranch.h"
@@ -116,8 +117,6 @@ nsFont* nsPrintOptions::sDefaultFont = nsnull;
  */
 nsPrintOptions::nsPrintOptions()
 {
-  NS_INIT_ISUPPORTS();
-
   /* member initializers and constructor code */
   if (sDefaultFont == nsnull) {
     sDefaultFont = new nsFont("Times", NS_FONT_STYLE_NORMAL,NS_FONT_VARIANT_NORMAL,
@@ -170,7 +169,6 @@ nsPrinterListEnumerator : public nsISimpleEnumerator
 nsPrinterListEnumerator::nsPrinterListEnumerator() :
   mPrinters(nsnull), mCount(0), mIndex(0)
 {
-   NS_INIT_ISUPPORTS();
 }
 
 nsPrinterListEnumerator::~nsPrinterListEnumerator()
@@ -233,7 +231,7 @@ NS_IMETHODIMP nsPrinterListEnumerator::GetNext(nsISupports **aPrinter)
 NS_IMETHODIMP 
 nsPrintOptions::SetFontNamePointSize(nsString& aFontName, PRInt32 aPointSize)
 {
-  if (sDefaultFont != nsnull && aFontName.Length() > 0 && aPointSize > 0) {
+  if (sDefaultFont != nsnull && !aFontName.IsEmpty() && aPointSize > 0) {
     sDefaultFont->name = aFontName;
     sDefaultFont->size = NSIntPointsToTwips(aPointSize);
   }
@@ -326,16 +324,16 @@ const char* nsPrintOptions::GetPrefName(const char *    aPrefName,
     return aPrefName;
   }
 
-  nsCAutoString prefName;
+  mPrefName.Truncate(); /* mPrefName = ""; */
 
   if (aPrinterName.Length()) {
-    prefName.AppendWithConversion(NS_LITERAL_STRING("printer_"));
-    prefName.AppendWithConversion(aPrinterName);
-    prefName.AppendWithConversion(NS_LITERAL_STRING("."));
+    mPrefName.Append("printer_");
+    mPrefName.AppendWithConversion(aPrinterName);
+    mPrefName.Append(".");
   }
-  prefName += aPrefName;
+  mPrefName += aPrefName;
 
-  return prefName.get();
+  return mPrefName.get();
 }
 
 //----------------------------------------------------------------------
@@ -802,13 +800,12 @@ NS_IMETHODIMP nsPrintOptions::AvailablePrinters(nsISimpleEnumerator **aPrinterEn
    nsCOMPtr<nsPrinterListEnumerator> printerListEnum = new nsPrinterListEnumerator();
    NS_ENSURE_TRUE(printerListEnum.get(), NS_ERROR_OUT_OF_MEMORY);
 
-   // if Init fails NS_OK is return (script needs NS_OK) but the enumerator will be null
    nsresult rv = printerListEnum->Init();
    if (NS_SUCCEEDED(rv)) {
      *aPrinterEnumerator = NS_STATIC_CAST(nsISimpleEnumerator*, printerListEnum);
      NS_ADDREF(*aPrinterEnumerator);
    }
-   return NS_OK;
+   return rv;
 }
 
 NS_IMETHODIMP nsPrintOptions::DisplayJobProperties( const PRUnichar *aPrinter, nsIPrintSettings* aPrintSettings, PRBool *aDisplayed)
@@ -1102,7 +1099,7 @@ nsresult nsPrintOptions::ReadPrefDouble(const char * aPrefId,
   nsresult rv = mPrefBranch->GetCharPref(aPrefId, &str);
   if (NS_SUCCEEDED(rv) && str) {
     float f;
-    sscanf(str, "%f", &f);
+    PR_sscanf(str, "%f", &f);
     aVal = double(f);
     nsMemory::Free(str);
   }
