@@ -46,15 +46,13 @@
 
 #include <errno.h>
 #include <string.h>
-static int gGotAllFontNames = 0;
 
 // XXX many of these statics need to be freed at shutdown time
-static PLHashTable* gFamilies = nsnull;
 static nsHashtable* gFontMetricsCache = nsnull;
-static int gFontMetricsCacheCount = 0;
 static nsCString **gFontNames = nsnull;
 static FontDetails *gFontDetails = nsnull;
 static int gnFonts = 0;
+static nsIPref* gPref = nsnull;
 
 #undef USER_DEFINED
 #define USER_DEFINED "x-user-def"
@@ -64,6 +62,7 @@ static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 
 nsFontMetricsPh::nsFontMetricsPh()
 {
+	NS_INIT_ISUPPORTS();
 	mDeviceContext = nsnull;
 	mFont = nsnull;
 
@@ -89,11 +88,16 @@ nsFontMetricsPh::nsFontMetricsPh()
 	mAveCharWidth = 0;
 }
 
-static void InitGlobals()
+static nsresult InitGlobals()
 {
+  nsServiceManager::GetService(kPrefCID, NS_GET_IID(nsIPref), (nsISupports**) &gPref);
+  if (!gPref) return NS_ERROR_FAILURE;
+
 	gFontMetricsCache = new nsHashtable();
+	return NS_OK;
 }
 
+#if 0
 static PRBool FreeFontMetricsCache(nsHashKey* aKey, void* aData, void* aClosure)
 {
 	FontQueryInfo * node = (FontQueryInfo*) aData;
@@ -104,7 +108,6 @@ static PRBool FreeFontMetricsCache(nsHashKey* aKey, void* aData, void* aClosure)
 	
 	return PR_TRUE;
 }
-
 static void FreeGlobals()
 {
 	if (gFontMetricsCache)
@@ -112,9 +115,9 @@ static void FreeGlobals()
 		gFontMetricsCache->Reset(FreeFontMetricsCache, nsnull);
 		delete gFontMetricsCache;
 		gFontMetricsCache = nsnull;
-		gFontMetricsCacheCount = 0;
 	}
 }
+#endif
 
 nsFontMetricsPh :: ~nsFontMetricsPh( )
 {
@@ -130,7 +133,6 @@ nsFontMetricsPh :: ~nsFontMetricsPh( )
     mDeviceContext->FontMetricsDeleted(this);
     mDeviceContext = nsnull;
   }
-	FreeGlobals();
 }
 
 NS_IMPL_ISUPPORTS1( nsFontMetricsPh, nsIFontMetrics )
@@ -143,115 +145,108 @@ NS_IMETHODIMP nsFontMetricsPh::Init ( const nsFont& aFont, nsIAtom* aLangGroup,
 	char          *str = nsnull;
 	nsresult      result;
 	PhRect_t      extent;
-	
-	if( !gFontMetricsCacheCount )
-		InitGlobals( );
+
+	if( !gFontMetricsCache ) {
+		nsresult res = InitGlobals( );
+		if( NS_FAILED(res) ) return res;
+		}
 	
 	mFont = new nsFont(aFont);
 	mLangGroup = aLangGroup;
 	
 	mDeviceContext = aContext;
 	
-	float app2dev;
-	mDeviceContext->GetAppUnitsToDevUnits(app2dev);
-
 	result = aContext->FirstExistingFont(aFont, firstFace);
 
 	str = ToNewCString(firstFace);
 
+#ifdef DEBUG_Adrian
+printf( "\n\n\t\t\tIn nsFontMetricsPh::Init str=%s\n", str );
+#endif
+
 	if( !str || !str[0] )
 	{
-		free (str);
+		if( str ) free (str);
 		str = strdup("serif");
 	}
 	
-	const PRUnichar *uc;
-	aLangGroup->GetUnicode( &uc );
-	nsString language( uc );
-	char *cstring = ToNewCString(language);
+	const char *cstring;
+	aLangGroup->GetUTF8String( &cstring );
 	
 	char prop[256];
 	sprintf( prop, "font.name.%s.%s", str, cstring );
 
-	/* Free cstring.  */
-	if (cstring)
-		free (cstring);
-
 	char *font_default = NULL;
-	nsresult res = NS_ERROR_FAILURE;
-	nsIPref* prefs = nsnull;
-	nsServiceManager::GetService(kPrefCID, NS_GET_IID(nsIPref), (nsISupports**) &prefs);
-	if (prefs)
-	  {
-		  prefs->CopyCharPref( prop, &font_default );
-		  if( font_default )
-			{
-				//delete [] str;
-				free (str);
-			/* font_default was allocated. in CopyCharPref. */
-				str = font_default;
-			}
-	  }
-	NS_IF_RELEASE(prefs);
-
-	float scale = 1.0;
+	gPref->CopyCharPref( prop, &font_default );
+	if( font_default )
+		{
+		free (str);
+		/* font_default was allocated. in CopyCharPref. */
+		str = font_default;
+		}
 
+	float app2dev;
 	mDeviceContext->GetAppUnitsToDevUnits(app2dev);
-	mDeviceContext->GetCanonicalPixelScale(scale);
 
-	PRInt32 sizePoints = NSToIntRound(app2dev * mFont->size * 0.8);
+	PRInt32 sizePoints = NSToIntRound( app2dev * mFont->size * 0.74 );
 	
-	char NSFontSuffix[5];
 	char NSFullFontName[MAX_FONT_TAG];
 
-	NSFontSuffix[0] = nsnull;
-
 	unsigned int uiFlags = 0L;
 
 	if(aFont.weight > NS_FONT_WEIGHT_NORMAL)
 	   uiFlags |= PF_STYLE_BOLD;
 
-	if(aFont.style & NS_FONT_STYLE_ITALIC)
+	if(aFont.style & (NS_FONT_STYLE_ITALIC|NS_FONT_STYLE_OBLIQUE) )
 	   uiFlags |= PF_STYLE_ITALIC;
 
-	if(aFont.style & NS_FONT_STYLE_OBLIQUE)
-	   uiFlags |= PF_STYLE_ANTIALIAS;
+	if(aFont.style & NS_FONT_STYLE_ANTIALIAS)
+		uiFlags |= PF_STYLE_ANTIALIAS;
 
-#if (Ph_LIB_VERSION > 200) // a header changed in RTP 6.2
 	if( PfGenerateFontName( (char *)str, uiFlags, sizePoints, (char *)NSFullFontName ) == NULL )
 	  {
-		  PfGenerateFontName( (char *)"Courier 10 Pitch BT", uiFlags, sizePoints, (char *)NSFullFontName );
-	  }
-#else
-	if( PfGenerateFontName( (uchar_t *)str, uiFlags, sizePoints, (uchar_t *)NSFullFontName ) == NULL )
-	  {
-		  PfGenerateFontName( (uchar_t *)"Courier 10 Pitch BT", uiFlags, sizePoints, (uchar_t *)NSFullFontName );
-	  }
+#ifdef DEBUG_Adrian
+printf( "!!!!!!!!!!!! PfGenerateFontName failed\n" );
 #endif
+		  PfGenerateFontName( "TextFont", uiFlags, sizePoints, (char *)NSFullFontName );
+	  }
 
 	/* Once the Photon Font String is built get the attributes */
 	FontQueryInfo *node;
 
-	//nsStringKey key((char *)(NSFullFontName));
 	nsCStringKey key((char *)(NSFullFontName));
 	node = (FontQueryInfo *) gFontMetricsCache->Get(&key);
+
+#ifdef DEBUG_Adrian
+printf( "\t\t\tThe generated font name is NSFullFontName=%s\n", NSFullFontName );
+if( node ) printf( "\t\t\t( cached ) The real font is desc=%s\n", node->desc );
+#endif
+
 	if( !node )
 	  {
-		  node = (FontQueryInfo *)calloc(sizeof(FontQueryInfo), 1);
-		  PfQueryFont(NSFullFontName, node);
-		  gFontMetricsCache->Put(&key, node);
-		  gFontMetricsCacheCount++;
+		node = (FontQueryInfo *)calloc(sizeof(FontQueryInfo), 1);
+		PfQueryFont(NSFullFontName, node);
+
+#ifdef DEBUG_Adrian
+printf( "\t\t\t(not cached ) The real font is desc=%s\n", node->desc );
+printf( "\tCall PfLoadMetrics for NSFullFontName=%s\n", NSFullFontName );
+#endif
+
+		gFontMetricsCache->Put(&key, node);
+
+		PfLoadFont( NSFullFontName, PHFONT_LOAD_METRICS, nsnull );
+		PfLoadMetrics( NSFullFontName );
 	  }
 
 	float dev2app;
-	int height;
+	double height;
 	nscoord onePixel;
 
 	mDeviceContext->GetDevUnitsToAppUnits(dev2app);
 	onePixel = NSToCoordRound(1 * dev2app);
 	height = node->descender - node->ascender + 1.0;
 	PfExtentText(&extent, NULL, NSFullFontName, " ", 1);
-	mSpaceWidth = (int) ((extent.lr.x - extent.ul.x + 1) * dev2app);
+	mSpaceWidth = NSToCoordRound((extent.lr.x - extent.ul.x + 1) * dev2app);
 
 	mLeading = NSToCoordRound(0);
 	mEmHeight = NSToCoordRound(height * dev2app);
@@ -261,9 +256,7 @@ NS_IMETHODIMP nsFontMetricsPh::Init ( const nsFont& aFont, nsIAtom* aLangGroup,
 	mAscent = mMaxAscent = NSToCoordRound(node->ascender * dev2app * -1.0);
 	mDescent = mMaxDescent = NSToCoordRound(node->descender * dev2app);
 	mMaxAdvance = NSToCoordRound(node->width * dev2app);
-
-	PfExtentText(&extent, NULL, NSFullFontName, "x", 1);
-	mAveCharWidth = (int) ((extent.lr.x - extent.ul.x + 1) * dev2app);
+	mAveCharWidth = mSpaceWidth;
 
 	mXHeight = NSToCoordRound((float)node->ascender * dev2app * 0.56f * -1.0); // 56% of ascent, best guess for non-true type
 	mSuperscriptOffset = mXHeight;     // XXX temporary code!
@@ -278,24 +271,10 @@ NS_IMETHODIMP nsFontMetricsPh::Init ( const nsFont& aFont, nsIAtom* aLangGroup,
 	   free (mFontHandle);
 	mFontHandle = strdup(NSFullFontName);
 
-	//delete [] str;
 	free (str);
 	return NS_OK;
 }
 
-NS_IMETHODIMP nsFontMetricsPh :: Destroy( )
-{
-	mDeviceContext = nsnull;
-	return NS_OK;
-}
-
-static void apGenericFamilyToFont( const nsString& aGenericFamily, nsIDeviceContext* aDC, nsString& aFontFace )
-{
-	char *str = ToNewCString(aGenericFamily);
-	//delete [] str;
-	free (str);
-}
-
 struct FontEnumData
 {
 	FontEnumData(nsIDeviceContext* aContext, char* aFaceName)
@@ -307,117 +286,11 @@ struct FontEnumData
 	char* mFaceName;
 };
 
-static PRBool FontEnumCallback( const nsString& aFamily, PRBool aGeneric, void *aData )
-{
-	return PR_TRUE;
-}
-
 void nsFontMetricsPh::RealizeFont()
 {
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetXHeight( nscoord& aResult )
-{
-	aResult = mXHeight;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetSuperscriptOffset( nscoord& aResult )
-{
-	aResult = mSuperscriptOffset;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetSubscriptOffset( nscoord& aResult )
-{
-	aResult = mSubscriptOffset;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetStrikeout( nscoord& aOffset, nscoord& aSize )
-{
-	aOffset = mStrikeoutOffset;
-	aSize = mStrikeoutSize;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetUnderline( nscoord& aOffset, nscoord& aSize )
-{
-	aOffset = mUnderlineOffset;
-	aSize = mUnderlineSize;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetHeight( nscoord &aHeight )
-{
-	aHeight = mHeight;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh ::GetNormalLineHeight( nscoord &aHeight )
-{
-	aHeight = mEmHeight + mLeading;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetLeading( nscoord &aLeading )
-{
-	aLeading = mLeading;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh::GetEmHeight( nscoord &aHeight )
-{
-	aHeight = mEmHeight;
-	return NS_OK;
-}
-
-NS_IMETHODIMP  nsFontMetricsPh::GetEmAscent( nscoord &aAscent )
-{
-	aAscent = mEmAscent;
-	return NS_OK;
-}
-
-NS_IMETHODIMP  nsFontMetricsPh::GetEmDescent( nscoord &aDescent )
-{
-	aDescent = mEmDescent;
-	return NS_OK;
-}
-
-NS_IMETHODIMP  nsFontMetricsPh::GetMaxHeight( nscoord &aHeight )
-{
-	aHeight = mMaxHeight;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetMaxAscent( nscoord &aAscent )
-{
-	aAscent = mMaxAscent;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetMaxDescent( nscoord &aDescent )
-{
-	aDescent = mMaxDescent;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetMaxAdvance( nscoord &aAdvance )
-{
-	aAdvance = mMaxAdvance;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetAveCharWidth( nscoord &aAveCharWidth)
-{
-	aAveCharWidth = mAveCharWidth;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh :: GetFont(const nsFont *&aFont)
-{
-	aFont = mFont;
-	return NS_OK;
+#ifdef DEBUG_Adrian
+printf( "In RealizeFont\n" );
+#endif
 }
 
 NS_IMETHODIMP  nsFontMetricsPh::GetLangGroup(nsIAtom** aLangGroup)
@@ -430,18 +303,6 @@ NS_IMETHODIMP  nsFontMetricsPh::GetLangGroup(nsIAtom** aLangGroup)
 	return NS_OK;
 }
 
-NS_IMETHODIMP nsFontMetricsPh::GetFontHandle(nsFontHandle &aHandle)
-{
-	aHandle = (nsFontHandle) mFontHandle;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsFontMetricsPh::GetSpaceWidth(nscoord &aSpaceWidth)
-{
-	aSpaceWidth = mSpaceWidth;
-	return NS_OK;
-}
-
 struct nsFontFamily
 {
 	NS_DECL_AND_IMPL_ZEROING_OPERATOR_NEW
@@ -451,19 +312,11 @@ struct nsFontFamily
 // The Font Enumerator
 nsFontEnumeratorPh::nsFontEnumeratorPh()
 {
+	NS_INIT_ISUPPORTS();
 }
 
 NS_IMPL_ISUPPORTS1(nsFontEnumeratorPh, nsIFontEnumerator)
 
-static int gInitializedFontEnumerator = 0;
-
-static int InitializeFontEnumerator( void )
-{
-	gInitializedFontEnumerator = 1;
-	if( !gGotAllFontNames ) gGotAllFontNames = 1;
-	return 1;
-}
-
 typedef struct EnumerateFamilyInfo
 {
 	PRUnichar** mArray;
@@ -505,50 +358,22 @@ static int CompareFontNames(const void* aArg1, const void* aArg2, void* aClosure
 
 NS_IMETHODIMP nsFontEnumeratorPh::EnumerateAllFonts(PRUint32* aCount, PRUnichar*** aResult)
 {
-	if( aCount ) *aCount = 0;
-	else return NS_ERROR_NULL_POINTER;
-	if( aResult ) *aResult = nsnull;
-	else return NS_ERROR_NULL_POINTER;
-
-	if( !gInitializedFontEnumerator && !InitializeFontEnumerator( ) ) return NS_ERROR_FAILURE;
-
-	if( gFamilies )
-	  {
-		  PRUnichar** array = (PRUnichar**) nsMemory::Alloc(gFamilies->nentries * sizeof(PRUnichar*));
-		  if( !array ) return NS_ERROR_OUT_OF_MEMORY;
-
-		  EnumerateFamilyInfo info = { array, 0
-		  };
-		  PL_HashTableEnumerateEntries(gFamilies, EnumerateFamily, &info);
-		  if( !info.mIndex )
-			{
-				nsMemory::Free(array);
-				return NS_ERROR_OUT_OF_MEMORY;
-			}
-
-		  NS_QuickSort(array, gFamilies->nentries, sizeof(PRUnichar*),
-					   CompareFontNames, nsnull);
-
-		  *aCount = gFamilies->nentries;
-		  *aResult = array;
-		  return NS_OK;
-	  }
-	else return NS_ERROR_FAILURE;
+	return EnumerateFonts( nsnull, nsnull, aCount, aResult );
 }
 
 NS_IMETHODIMP nsFontEnumeratorPh::EnumerateFonts( const char* aLangGroup, const char* aGeneric, PRUint32* aCount, PRUnichar*** aResult )
 {
-	if( !aLangGroup || !aGeneric ) return NS_ERROR_NULL_POINTER;
 
-	if( aCount ) *aCount = 0;
-	else return NS_ERROR_NULL_POINTER;
+	NS_ENSURE_ARG_POINTER(aResult);
+	*aResult = nsnull;
+	NS_ENSURE_ARG_POINTER(aCount);
+	*aCount = 0;
 
-	if( aResult ) *aResult = nsnull;
-	else return NS_ERROR_NULL_POINTER;
-
-	//if( !strcmp(aLangGroup, "x-unicode") || !strcmp(aLangGroup, "x-user-def") ) return EnumerateAllFonts( aCount, aResult );
-
-	if( !gInitializedFontEnumerator && !InitializeFontEnumerator( ) ) return NS_ERROR_FAILURE;
+	// aLangGroup=null or ""  means any (i.e., don't care)
+	// aGeneric=null or ""  means any (i.e, don't care)
+	const char* generic = nsnull;
+	if (aGeneric && *aGeneric)
+		generic = aGeneric;
 
 	int i;
 	if(!gFontDetails)
@@ -559,16 +384,20 @@ NS_IMETHODIMP nsFontEnumeratorPh::EnumerateFonts( const char* aLangGroup, const
 				gFontDetails = new FontDetails[gnFonts];
 				if(gFontDetails)
 				  {
-					  gFontNames = (nsCString**) nsMemory::Alloc(gnFonts * sizeof(nsCString*));
-					  PfQueryFonts('a', PHFONT_DONT_SHOW_LEGACY, gFontDetails, gnFonts);
-					  for(i=0;i<gnFonts;i++)
-						 gFontNames[i] = new nsCString(gFontDetails[i].desc);
+					gFontNames = (nsCString**) nsMemory::Alloc(gnFonts * sizeof(nsCString*));
+					PfQueryFonts('a', PHFONT_DONT_SHOW_LEGACY, gFontDetails, gnFonts);
+
+					int total = 0;
+					for(i=0;i<gnFonts;i++) {
+						gFontNames[total++] = new nsCString(gFontDetails[i].desc);
+						}
+					gnFonts = total;
 				  }
 
 			}
 	  }
 
-	if(gFontDetails)
+	if( gnFonts > 0 )
 	  {
 		  PRUnichar** array = (PRUnichar**) nsMemory::Alloc(gnFonts * sizeof(PRUnichar*));
 		  if(!array)
@@ -577,14 +406,20 @@ NS_IMETHODIMP nsFontEnumeratorPh::EnumerateFonts( const char* aLangGroup, const
 		  int nCount = 0;
 		  for(i=0;i<gnFonts;i++)
 			{
-				if(stricmp(aGeneric, "monospace") == 0)
+				if(!generic) /* select all fonts */
+				  {
+					  if (gFontDetails[i].flags & PHFONT_INFO_PROP) /* always use proportional fonts */
+					  	array[nCount++] = ToNewUnicode(*gFontNames[i]);
+				  }
+				else if(stricmp(generic, "monospace") == 0)
 				  {
 					  if(gFontDetails[i].flags & PHFONT_INFO_FIXED)
 						 array[nCount++] = ToNewUnicode(*gFontNames[i]);
 				  }
+				/* other possible vallues for generic are: serif, sans-serif, cursive, fantasy */
 				else
 				  {
-					  if (gFontDetails[i].flags & PHFONT_INFO_PROP)
+					  if (gFontDetails[i].flags & PHFONT_INFO_PROP) /* always use proportional fonts */
 						 array[nCount++] = ToNewUnicode(*gFontNames[i]);
 				  }
 			}
@@ -593,21 +428,30 @@ NS_IMETHODIMP nsFontEnumeratorPh::EnumerateFonts( const char* aLangGroup, const
 		  return NS_OK;
 	  }
 
-	return EnumerateAllFonts( aCount, aResult );
+	return NS_OK;
 }
 
 NS_IMETHODIMP
    nsFontEnumeratorPh::HaveFontFor(const char* aLangGroup, PRBool* aResult)
 {
 	NS_ENSURE_ARG_POINTER(aResult);
-	*aResult = PR_FALSE;
-	NS_ENSURE_ARG_POINTER(aLangGroup);
-
 	*aResult = PR_TRUE; // always return true for now.
-	// Finish me - ftang
 	return NS_OK;
 }
 
+NS_IMETHODIMP
+nsFontEnumeratorPh::GetDefaultFont(const char *aLangGroup, 
+  const char *aGeneric, PRUnichar **aResult)
+{
+  // aLangGroup=null or ""  means any (i.e., don't care)
+  // aGeneric=null or ""  means any (i.e, don't care)
+
+  NS_ENSURE_ARG_POINTER(aResult);
+  *aResult = nsnull;
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP
    nsFontEnumeratorPh::UpdateFontList(PRBool *updateFontList)
 {
