@@ -167,8 +167,8 @@ void nsDeviceContextWin :: CommonInit(HDC aDC)
 {
   int   rasterCaps = ::GetDeviceCaps(aDC, RASTERCAPS);
 
-  mTwipsToPixels = ((float)::GetDeviceCaps(aDC, LOGPIXELSY)) / (float)NSIntPointsToTwips(72);
-  mPixelsToTwips = 1.0f / mTwipsToPixels;
+  mPixelsToTwips = NSToIntRound((float)NSIntPointsToTwips(72) / ((float)::GetDeviceCaps(aDC, LOGPIXELSY)));
+  mTwipsToPixels = 1.0 / mPixelsToTwips;
 
   mDepth = (PRUint32)::GetDeviceCaps(aDC, BITSPIXEL);
   mPaletteInfo.isPaletteDevice = RC_PALETTE == (rasterCaps & RC_PALETTE);
@@ -571,7 +571,8 @@ NS_IMETHODIMP nsDeviceContextWin :: GetSystemFont(nsSystemFontID anID, nsFont *a
 NS_IMETHODIMP nsDeviceContextWin :: GetDrawingSurface(nsIRenderingContext &aContext, nsDrawingSurface &aSurface)
 {
   if (NULL == mSurface) {
-    aContext.CreateDrawingSurface(nsnull, 0, mSurface);
+    nsRect empty(0,0,0,0); // CreateDrawingSurface(null,...) used width=0,height=0
+    aContext.CreateDrawingSurface(empty, 0, mSurface);
   }
 
   aSurface = mSurface;
@@ -593,20 +594,17 @@ NS_IMETHODIMP nsDeviceContextWin :: CheckFontExistence(const nsString& aFontName
   HDC     hdc = ::GetDC(hwnd);
   PRBool  isthere = PR_FALSE;
 
-  char    fontName[LF_FACESIZE];
-
-  const PRUnichar* unicodefontname = aFontName.get();
-
-  int outlen = ::WideCharToMultiByte(CP_ACP, 0, aFontName.get(), aFontName.Length(), 
-                                   fontName, LF_FACESIZE, NULL, NULL);
-  if(outlen > 0)
-    fontName[outlen] = '\0'; // null terminate
+  LOGFONT logFont;
+  logFont.lfCharSet = DEFAULT_CHARSET;
+  logFont.lfPitchAndFamily = 0;
+  int outlen = WideCharToMultiByte(CP_ACP, 0, aFontName.get(), aFontName.Length() + 1,
+                                   logFont.lfFaceName, sizeof(logFont.lfFaceName), nsnull, nsnull);
 
   // somehow the WideCharToMultiByte failed, let's try the old code
-  if(0 == outlen) 
-    aFontName.ToCString(fontName, LF_FACESIZE);
+  if(0 == outlen)
+    aFontName.ToCString(logFont.lfFaceName, LF_FACESIZE);
 
-  ::EnumFontFamilies(hdc, fontName, (FONTENUMPROC)fontcallback, (LPARAM)&isthere);
+  ::EnumFontFamiliesEx(hdc, &logFont, (FONTENUMPROC)fontcallback, (LPARAM)&isthere, 0);
 
   ::ReleaseDC(hwnd, hdc);
 
@@ -787,7 +785,7 @@ NS_IMETHODIMP nsDeviceContextWin :: BeginDocument(PRUnichar * aTitle, PRUnichar*
 
     char* docName = nsnull;
     nsAutoString str(aPrintToFileName);
-    if (str.Length() > 0) {
+    if (!str.IsEmpty()) {
       docName = ToNewCString(str);
     }
     docinfo.cbSize = sizeof(docinfo);
