@@ -49,7 +49,7 @@ static NS_DEFINE_IID(kILoggingSinkIID, NS_ILOGGING_SINK_IID);
 static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
 
 // list of tags that have skipped content
-static char gSkippedContentTags[] = {
+static const char gSkippedContentTags[] = {
   eHTMLTag_style,
   eHTMLTag_script,
   eHTMLTag_server,
@@ -74,7 +74,6 @@ NS_NewHTMLLoggingSink(nsIContentSink** aInstancePtrResult)
 }
 
 nsLoggingSink::nsLoggingSink() {
-  NS_INIT_ISUPPORTS();
   mOutput = 0;
 	mLevel=-1;
   mSink=0;
@@ -247,6 +246,21 @@ nsLoggingSink::CloseContainer(const nsHTMLTag aTag) {
 
 }
 
+NS_IMETHODIMP
+nsLoggingSink::AddHeadContent(const nsIParserNode& aNode) {
+  LeafNode(aNode);
+
+  nsresult theResult=NS_OK;
+
+  //then proxy the call to the real sink if you have one.
+  if(mSink) {
+    theResult=mSink->AddHeadContent(aNode);
+  }
+  
+  return theResult;
+
+}
+
 NS_IMETHODIMP
 nsLoggingSink::AddLeaf(const nsIParserNode& aNode) {
   LeafNode(aNode);
@@ -595,7 +609,7 @@ nsLoggingSink::WriteAttributes(const nsIParserNode& aNode) {
  
     tmp.Truncate();
     tmp.Append(v);
-    if(tmp.Length() > 0) {
+    if(!tmp.IsEmpty()) {
       PRUnichar first = tmp.First();
       if ((first == '"') || (first == '\'')) {
         if (tmp.Last() == first) {
@@ -655,7 +669,7 @@ nsLoggingSink::WillWriteAttributes(const nsIParserNode& aNode)
     PRInt32 lineNo = 0;
 
     dtd->CollectSkippedContent(aNode.GetNodeType(), content, lineNo);
-    if (content.Length() > 0) {
+    if (!content.IsEmpty()) {
       return PR_TRUE;
     }
   }
@@ -764,19 +778,13 @@ nsLoggingSink::GetNewCString(const nsAString& aValue, char** aResult)
   nsAutoString temp;
   result=QuoteText(aValue,temp);
   if(NS_SUCCEEDED(result)) {
-    if(temp.Length()>0) {
+    if(!temp.IsEmpty()) {
       *aResult = ToNewCString(temp);
     }
   }
   return result;
 }
 
-NS_IMETHODIMP
-nsLoggingSink::DoFragment(PRBool aFlag) 
-{
-  return NS_OK; 
-}
-
 /**
  * This gets called when handling illegal contents, especially
  * in dealing with tables. This method creates a new context.
