@@ -44,12 +44,10 @@
 
 #include "nsIAtom.h"
 #include "nsParser.h"
-#include "nsIContentSink.h"
 #include "nsString.h"
 #include "nsCRT.h" 
 #include "nsScanner.h"
 #include "plstr.h"
-#include "nsIParserFilter.h"
 #include "nsViewSourceHTML.h" 
 #include "nsIStringStream.h"
 #include "nsIChannel.h"
@@ -66,6 +64,7 @@
 #include "nsIEventQueue.h"
 #include "nsIEventQueueService.h"
 #include "nsExpatDriver.h"
+#include "nsIServiceManager.h"
 //#define rickgdebug 
 
 #define NS_PARSER_FLAG_DTD_VERIFICATION       0x00000001
@@ -299,21 +298,15 @@ static PRBool gDumpContent=PR_FALSE;
  *  @param   
  *  @return   
  */
-nsParser::nsParser(nsITokenObserver* anObserver) {
-  NS_INIT_ISUPPORTS();
-
+nsParser::nsParser() {
 #ifdef NS_DEBUG
   if(!gDumpContent) {
     gDumpContent=(PR_GetEnv("PARSER_DUMP_CONTENT"))? PR_TRUE:PR_FALSE;
   }
 #endif
 
-  mCharset.Assign(NS_LITERAL_STRING("ISO-8859-1"));
-  mParserFilter = 0;
-  mObserver = 0;
-  mSink=0;
+  mCharset.Assign(NS_LITERAL_CSTRING("ISO-8859-1"));
   mParserContext=0;
-  mTokenObserver=anObserver;
   mStreamStatus=0;
   mCharsetSource=kCharsetUninitialized;
   mInternalState=NS_OK;;
@@ -361,10 +354,6 @@ nsParser::~nsParser() {
   }
 #endif
 
-  NS_IF_RELEASE(mObserver);
-  NS_IF_RELEASE(mSink);
-  NS_IF_RELEASE(mParserFilter);
-
   //don't forget to add code here to delete 
   //what may be several contexts...
   delete mParserContext;
@@ -444,20 +433,13 @@ nsParser::PostContinueEvent()
  * @param 
  * @return
  */
-nsIParserFilter * nsParser::SetParserFilter(nsIParserFilter * aFilter)
+NS_IMETHODIMP_(void) nsParser::SetParserFilter(nsIParserFilter * aFilter)
 {
-  nsIParserFilter* old=mParserFilter;
-  if(old)
-    NS_RELEASE(old);
-  if(aFilter) {
-    mParserFilter=aFilter;
-    NS_ADDREF(aFilter);
-  }
-  return old;
+  mParserFilter = aFilter;
 }
 
 
-void nsParser::GetCommand(nsString& aCommand)
+NS_IMETHODIMP_(void) nsParser::GetCommand(nsString& aCommand)
 {
   aCommand = mCommandStr;  
 }
@@ -468,10 +450,10 @@ void nsParser::GetCommand(nsString& aCommand)
  *  this allows us to select a DTD which can do, say, view-source.
  *  
  *  @update  gess 01/04/99
- *  @param   aContentSink -- ptr to content sink that will receive output
- *  @return	 ptr to previously set contentsink (usually null)  
+ *  @param   aCommand the command string to set
  */
-void nsParser::SetCommand(const char* aCommand){
+NS_IMETHODIMP_(void) nsParser::SetCommand(const char* aCommand)
+{
   nsCAutoString theCommand(aCommand);
   if(theCommand.Equals(kViewSourceCommand))
     mCommand=eViewSource;
@@ -487,11 +469,11 @@ void nsParser::SetCommand(const char* aCommand){
  *  this allows us to select a DTD which can do, say, view-source.
  *  
  *  @update  gess 01/04/99
- *  @param   aContentSink -- ptr to content sink that will receive output
- *  @return	 ptr to previously set contentsink (usually null)  
+ *  @param   aParserCommand the command to set
  */
-void nsParser::SetCommand(eParserCommands aParserCommand){
-  mCommand=aParserCommand;
+NS_IMETHODIMP_(void) nsParser::SetCommand(eParserCommands aParserCommand)
+{
+  mCommand = aParserCommand;
 }
 
 
@@ -500,18 +482,20 @@ void nsParser::SetCommand(eParserCommands aParserCommand){
  *  about what charset to load
  *  
  *  @update  ftang 4/23/99
- *  @param   aCharset- the charest of a document
- *  @param   aCharsetSource- the soure of the chares
+ *  @param   aCharset- the charset of a document
+ *  @param   aCharsetSource- the source of the charset
  *  @return	 nada
  */
-void nsParser::SetDocumentCharset(const nsAString& aCharset, PRInt32 aCharsetSource){
+NS_IMETHODIMP_(void)
+nsParser::SetDocumentCharset(const nsACString& aCharset, PRInt32 aCharsetSource)
+{
   mCharset = aCharset;
   mCharsetSource = aCharsetSource; 
   if(mParserContext && mParserContext->mScanner)
      mParserContext->mScanner->SetDocumentCharset(aCharset, aCharsetSource);
 }
 
-void nsParser::SetSinkCharset(nsAString& aCharset)
+void nsParser::SetSinkCharset(nsACString& aCharset)
 {
   if (mSink) {
     mSink->SetDocumentCharset(aCharset);
@@ -526,26 +510,23 @@ void nsParser::SetSinkCharset(nsAString& aCharset)
  *  @param   nsIContentSink interface for node receiver
  *  @return  
  */
-nsIContentSink* nsParser::SetContentSink(nsIContentSink* aSink) {
-  NS_PRECONDITION(0!=aSink,"sink cannot be null!");
-  nsIContentSink* old=mSink;
-
-  NS_IF_RELEASE(old);
-  if(aSink) {
-    mSink=aSink;
-    NS_ADDREF(aSink);
+NS_IMETHODIMP_(void) nsParser::SetContentSink(nsIContentSink* aSink)
+{
+  NS_PRECONDITION(aSink,"sink cannot be null!");
+  mSink = aSink;
+  
+  if (mSink) {
     mSink->SetParser(this);
   }
-  return old;
 }
 
 /**
  * retrive the sink set into the parser 
  * @update	gess5/11/98
- * @param   aSink is the new sink to be used by parser
- * @return  old sink, or NULL
+ * @return  current sink 
  */
-nsIContentSink* nsParser::GetContentSink(void){
+NS_IMETHODIMP_(nsIContentSink*) nsParser::GetContentSink(void)
+{
   return mSink;
 }
 
@@ -557,34 +538,23 @@ nsIContentSink* nsParser::GetContentSink(void){
  *  @param   aDTD  is the object to be registered.
  *  @return  nothing.
  */
-nsresult
-nsParser::RegisterDTD(nsIDTD* aDTD){
+NS_IMETHODIMP
+nsParser::RegisterDTD(nsIDTD* aDTD)
+{
   CSharedParserObjects* sharedObjects;
   nsresult rv = GetSharedObjects(&sharedObjects);
   NS_ENSURE_SUCCESS(rv, rv);
   return sharedObjects->RegisterDTD(aDTD);
 }
 
-/**
- *  Retrieve scanner from topmost parsecontext
- *  
- *  @update  gess 01/04/99
- *  @return  ptr to internal scanner
- */
-nsScanner* nsParser::GetScanner(void){
-  if(mParserContext)
-    return mParserContext->mScanner;
-  return 0;
-}
-
-
 /**
  *  Retrieve parsemode from topmost parser context
  *  
  *  @update  gess 01/04/99
  *  @return  parsemode
  */
-nsDTDMode nsParser::GetParseMode(void){
+NS_IMETHODIMP_(nsDTDMode) nsParser::GetParseMode(void)
+{
   if(mParserContext)
     return mParserContext->mDTDMode;
   NS_NOTREACHED("no parser context");
@@ -1208,7 +1178,8 @@ FindSuitableDTD(CParserContext& aParserContext,
 }
 
 NS_IMETHODIMP 
-nsParser::CancelParsingEvents() {
+nsParser::CancelParsingEvents()
+{
   if (mFlags & NS_PARSER_FLAG_PENDING_CONTINUE_EVENT) {
     NS_ASSERTION(mEventQueue,"Event queue is null");
     // Revoke all pending continue parsing events 
@@ -1310,17 +1281,22 @@ void nsParser::PushContext(CParserContext& aContext) {
  * @update	gess7/22/98
  * @return  prev. context
  */
-CParserContext* nsParser::PopContext() {
-  CParserContext* oldContext=mParserContext;
-  if(oldContext) {
-    mParserContext=oldContext->mPrevContext;
-    // If the old context was blocked, propagate the blocked state
-    // back to the new one. Also, propagate the stream listener state
-    // but don't override onStop state to guarantee the call to DidBuildModel().
+CParserContext* nsParser::PopContext() 
+{
+  CParserContext* oldContext = mParserContext;
+  if (oldContext) {
+    mParserContext = oldContext->mPrevContext;
     if (mParserContext) {
-      if(mParserContext->mStreamListenerState!=eOnStop) {
+      // If the old context was blocked, propagate the blocked state
+      // back to the new one. Also, propagate the stream listener state
+      // but don't override onStop state to guarantee the call to DidBuildModel().
+      if (mParserContext->mStreamListenerState != eOnStop) {
         mParserContext->mStreamListenerState = oldContext->mStreamListenerState;
       }
+      // Preserve tokenizer state so that information is not lost
+      // between document.write. This fixes bug 99467
+      if (mParserContext->mTokenizer)
+        mParserContext->mTokenizer->CopyState(oldContext->mTokenizer);
     }
   }
   return oldContext;
@@ -1335,7 +1311,8 @@ CParserContext* nsParser::PopContext() {
  *  @param   aState determines whether we parse/tokenize or just cache.
  *  @return  current state
  */
-void nsParser::SetUnusedInput(nsString& aBuffer) {
+void nsParser::SetUnusedInput(nsString& aBuffer)
+{
   mUnusedInput=aBuffer;
 }
 
@@ -1347,7 +1324,8 @@ void nsParser::SetUnusedInput(nsString& aBuffer) {
  *  @update  gess 7/4/99
  *  @return  should return NS_OK once implemented
  */
-nsresult nsParser::Terminate(void){
+NS_IMETHODIMP nsParser::Terminate(void)
+{
   nsresult result = NS_OK;
   if (mParserContext && mParserContext->mDTD) {
     mParserContext->mDTD->Terminate();
@@ -1374,8 +1352,8 @@ nsresult nsParser::Terminate(void){
  *  @param   aState determines whether we parse/tokenize or just cache.
  *  @return  current state
  */
-nsresult nsParser::ContinueParsing(){
-    
+NS_IMETHODIMP nsParser::ContinueParsing()
+{    
   // If the stream has already finished, there's a good chance
   // that we might start closing things down when the parser
   // is reenabled. To make sure that we're not deleted across
@@ -1402,7 +1380,8 @@ nsresult nsParser::ContinueParsing(){
  *  @update  
  *  @return  
  */
-void nsParser::BlockParser() {
+NS_IMETHODIMP_(void) nsParser::BlockParser()
+{
   mFlags &= ~NS_PARSER_FLAG_PARSER_ENABLED;
   MOZ_TIMER_DEBUGLOG(("Stop: Parse Time: nsParser::BlockParser(), this=%p\n", this));
   MOZ_TIMER_STOP(mParseTime);
@@ -1417,7 +1396,8 @@ void nsParser::BlockParser() {
  *  @update  
  *  @return  
  */
-void nsParser::UnblockParser() {
+NS_IMETHODIMP_(void) nsParser::UnblockParser()
+{
   mFlags |= NS_PARSER_FLAG_PARSER_ENABLED;
   MOZ_TIMER_DEBUGLOG(("Start: Parse Time: nsParser::UnblockParser(), this=%p\n", this));
   MOZ_TIMER_START(mParseTime);
@@ -1429,7 +1409,8 @@ void nsParser::UnblockParser() {
  *  @update  vidur 4/12/99
  *  @return  current state
  */
-PRBool nsParser::IsParserEnabled() {
+NS_IMETHODIMP_(PRBool) nsParser::IsParserEnabled()
+{
   return mFlags & NS_PARSER_FLAG_PARSER_ENABLED;
 }
 
@@ -1439,7 +1420,8 @@ PRBool nsParser::IsParserEnabled() {
  *  @update  rickg 5/12/01
  *  @return  complete state
  */
-PRBool nsParser::IsComplete() {
+NS_IMETHODIMP_(PRBool) nsParser::IsComplete()
+{
   return !(mFlags & NS_PARSER_FLAG_PENDING_CONTINUE_EVENT);
 }
 
@@ -1473,13 +1455,18 @@ void nsParser::SetCanInterrupt(PRBool aCanInterrupt) {
  *  @param   aFilename -- const char* containing file to be parsed.
  *  @return  error code -- 0 if ok, non-zero if error.
  */
-nsresult nsParser::Parse(nsIURI* aURL,nsIRequestObserver* aListener,PRBool aVerifyEnabled, void* aKey,nsDTDMode aMode) {  
+NS_IMETHODIMP
+nsParser::Parse(nsIURI* aURL,
+                nsIRequestObserver* aListener,
+                PRBool aVerifyEnabled,
+                void* aKey,
+                nsDTDMode aMode)
+{  
 
   NS_PRECONDITION(aURL, "Error: Null URL given");
 
   nsresult result=kBadURL;
   mObserver = aListener;
-  NS_IF_ADDREF(mObserver);
  
   if (aVerifyEnabled) {
     mFlags |= NS_PARSER_FLAG_DTD_VERIFICATION;
@@ -1519,8 +1506,13 @@ nsresult nsParser::Parse(nsIURI* aURL,nsIRequestObserver* aListener,PRBool aVeri
  * @param   aStream is the i/o source
  * @return  error code -- 0 if ok, non-zero if error.
  */
-nsresult nsParser::Parse(nsIInputStream& aStream,const nsACString& aMimeType,PRBool aVerifyEnabled, void* aKey,nsDTDMode aMode){
-
+NS_IMETHODIMP
+nsParser::Parse(nsIInputStream* aStream,
+                const nsACString& aMimeType,
+                PRBool aVerifyEnabled,
+                void* aKey,
+                nsDTDMode aMode)
+{
   if (aVerifyEnabled) {
     mFlags |= NS_PARSER_FLAG_DTD_VERIFICATION;
   }
@@ -1533,9 +1525,9 @@ nsresult nsParser::Parse(nsIInputStream& aStream,const nsACString& aMimeType,PRB
   //ok, time to create our tokenizer and begin the process
   nsAutoString theUnknownFilename(NS_LITERAL_STRING("unknown"));
 
-  nsInputStream input(&aStream);
+  // references 
+  nsScanner* theScanner=new nsScanner(theUnknownFilename,aStream,mCharset,mCharsetSource);
 
-  nsScanner* theScanner=new nsScanner(theUnknownFilename,input,mCharset,mCharsetSource);
   CParserContext* pc=new CParserContext(theScanner,aKey,mCommand,0);
   if(pc && theScanner) {
     PushContext(*pc);
@@ -1566,17 +1558,21 @@ nsresult nsParser::Parse(nsIInputStream& aStream,const nsACString& aMimeType,PRB
  * @param   aMimeType tells us what type of content to expect in the given string
  * @return  error code -- 0 if ok, non-zero if error.
  */
-nsresult nsParser::Parse(const nsAString& aSourceBuffer, void* aKey,
-                         const nsACString& aMimeType,
-                         PRBool aVerifyEnabled, PRBool aLastCall,
-                         nsDTDMode aMode){ 
+NS_IMETHODIMP
+nsParser::Parse(const nsAString& aSourceBuffer,
+                void* aKey,
+                const nsACString& aMimeType,
+                PRBool aVerifyEnabled,
+                PRBool aLastCall,
+                nsDTDMode aMode)
+{ 
 
   //NOTE: Make sure that updates to this method don't cause 
   //      bug #2361 to break again! 
 
   nsresult result=NS_OK;
 
-  if(aLastCall && (0==aSourceBuffer.Length())) {
+  if(aLastCall && aSourceBuffer.IsEmpty()) {
     // Nothing is being passed to the parser so return
     // immediately. mUnusedInput will get processed when
     // some data is actually passed in.
@@ -1592,7 +1588,7 @@ nsresult nsParser::Parse(const nsAString& aSourceBuffer, void* aKey,
   // till we're completely done. 
   nsCOMPtr<nsIParser> kungFuDeathGrip(this);
 
-  if(aSourceBuffer.Length() || mUnusedInput.Length()) { 
+  if(!aSourceBuffer.IsEmpty() || !mUnusedInput.IsEmpty()) {
     
     if (aVerifyEnabled) {
       mFlags |= NS_PARSER_FLAG_DTD_VERIFICATION;
@@ -1678,13 +1674,14 @@ nsresult nsParser::Parse(const nsAString& aSourceBuffer, void* aKey,
  *  @param   
  *  @return  
  */
-nsresult nsParser::ParseFragment(const nsAString& aSourceBuffer,
-                                 void* aKey,
-                                 nsVoidArray& aTagStack,
-                                 PRUint32 anInsertPos,
-                                 const nsACString& aMimeType,
-                                 nsDTDMode aMode){
-
+NS_IMETHODIMP
+nsParser::ParseFragment(const nsAString& aSourceBuffer,
+                        void* aKey,
+                        nsVoidArray& aTagStack,
+                        PRUint32 anInsertPos,
+                        const nsACString& aMimeType,
+                        nsDTDMode aMode)
+{
   nsresult result = NS_OK;
   nsAutoString  theContext;
   PRUint32 theCount = aTagStack.Count();
@@ -1755,7 +1752,7 @@ nsresult nsParser::ResumeParse(PRBool allowIteration, PRBool aIsFinalChunk, PRBo
        
       while((result==NS_OK) && (theIterationIsOk)) {
         theFirstTime=PR_FALSE;
-        if(mUnusedInput.Length()>0) {
+        if(!mUnusedInput.IsEmpty()) {
           if(mParserContext->mScanner) {
             // -- Ref: Bug# 22485 --
             // Insert the unused input into the source buffer 
@@ -1903,7 +1900,7 @@ nsresult nsParser::BuildModel() {
     if (theRootDTD) {      
       MOZ_TIMER_START(mDTDTime);
       
-      result = theRootDTD->BuildModel(this, theTokenizer, mTokenObserver, mSink);  
+      result = theRootDTD->BuildModel(this, theTokenizer, nsnull, mSink);  
       
       MOZ_TIMER_STOP(mDTDTime);
     }
@@ -1953,7 +1950,7 @@ nsresult nsParser::OnStartRequest(nsIRequest *request, nsISupports* aContext) {
                   "Parser's nsIStreamListener API was not setup "
                   "correctly in constructor.");
 
-  if (nsnull != mObserver) {
+  if (mObserver) {
     mObserver->OnStartRequest(request, aContext);
   }
   mParserContext->mStreamListenerState = eOnStart;
@@ -1971,8 +1968,6 @@ nsresult nsParser::OnStartRequest(nsIRequest *request, nsISupports* aContext) {
   {
     mParserContext->SetMimeType(contentType);
   }
-  else
-    NS_NOTREACHED("parser needs a content type to find a dtd");
 
 #ifdef rickgdebug
   gOutFile= new fstream("c:/temp/out.file",ios::trunc);
@@ -1982,15 +1977,28 @@ nsresult nsParser::OnStartRequest(nsIRequest *request, nsISupports* aContext) {
 }
 
 
-#define UCS2_BE "UTF-16BE"
-#define UCS2_LE "UTF-16LE"
+#define UTF16_BE "UTF-16BE"
+#define UTF16_LE "UTF-16LE"
 #define UCS4_BE "UTF-32BE"
 #define UCS4_LE "UTF-32LE"
 #define UCS4_2143 "X-ISO-10646-UCS-4-2143"
 #define UCS4_3412 "X-ISO-10646-UCS-4-3412"
 #define UTF8 "UTF-8"
 
-static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsString& oCharset, PRInt32& oCharsetSource) {
+static inline PRBool IsSecondMarker(unsigned char aChar)
+{
+  switch (aChar) {
+    case '!':
+    case '?':
+    case 'h':
+    case 'H':
+      return PR_TRUE;
+    default:
+      return PR_FALSE;
+  }
+}
+
+static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsCString& oCharset, PRInt32& oCharsetSource) {
  oCharsetSource= kCharsetFromAutoDetection;
  oCharset.Truncate();
  // See http://www.w3.org/TR/2000/REC-xml-20001006#sec-guessing
@@ -2003,34 +2011,45 @@ static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsS
    case 0x00:
      if(0x00==aBytes[1]) {
         // 00 00
-        if((0x00==aBytes[2]) && (0x3C==aBytes[3])) {
+        if((0xFE==aBytes[2]) && (0xFF==aBytes[3])) {
+           // 00 00 FE FF UCS-4, big-endian machine (1234 order)
+           oCharset.Assign(UCS4_BE);
+        } else if((0x00==aBytes[2]) && (0x3C==aBytes[3])) {
            // 00 00 00 3C UCS-4, big-endian machine (1234 order)
-           oCharset.AssignWithConversion(UCS4_BE);
+           oCharset.Assign(UCS4_BE);
+        } else if((0xFF==aBytes[2]) && (0xFE==aBytes[3])) {
+           // 00 00 FF FE UCS-4, unusual octet order (2143)
+           oCharset.Assign(UCS4_2143);
         } else if((0x3C==aBytes[2]) && (0x00==aBytes[3])) {
            // 00 00 3C 00 UCS-4, unusual octet order (2143)
-           oCharset.AssignWithConversion(UCS4_2143);
+           oCharset.Assign(UCS4_2143);
         } 
-     } else if(0x3C==aBytes[1]) {
-        // 00 3C
-        if((0x00==aBytes[2]) && (0x00==aBytes[3])) {
+        oCharsetSource = kCharsetFromByteOrderMark;
+     } else if((0x3C==aBytes[1]) && (0x00==aBytes[2])) {
+        // 00 3C 00
+        if(IsSecondMarker(aBytes[3])) {
+           // 00 3C 00 SM UTF-16,  big-endian, no Byte Order Mark 
+           oCharset.Assign(UTF16_BE); 
+        } else if((0x00==aBytes[3])) {
            // 00 3C 00 00 UCS-4, unusual octet order (3412)
-           oCharset.AssignWithConversion(UCS4_3412);
-        } else if((0x3C==aBytes[2]) && (0x3F==aBytes[3])) {
-           // 00 3C 00 3F UTF-16, big-endian, no Byte Order Mark
-           oCharset.AssignWithConversion(UCS2_BE); // should change to UTF-16BE
+           oCharset.Assign(UCS4_3412);
         } 
+        oCharsetSource = kCharsetFromByteOrderMark;
      }
    break;
    case 0x3C:
-     if(0x00==aBytes[1]) {
-        // 3C 00
-        if((0x00==aBytes[2]) && (0x00==aBytes[3])) {
+     if(0x00==aBytes[1] && (0x00==aBytes[3])) {
+        // 3C 00 XX 00
+        if(IsSecondMarker(aBytes[2])) {
+           // 3C 00 SM 00 UTF-16,  little-endian, no Byte Order Mark 
+           oCharset.Assign(UTF16_LE); 
+        } else if((0x00==aBytes[2])) {
            // 3C 00 00 00 UCS-4, little-endian machine (4321 order)
-           oCharset.AssignWithConversion(UCS4_LE);
-        } else if((0x3F==aBytes[2]) && (0x00==aBytes[3])) {
-           // 3C 00 3F 00 UTF-16, little-endian, no Byte Order Mark
-           oCharset.AssignWithConversion(UCS2_LE); // should change to UTF-16LE
+           oCharset.Assign(UCS4_LE); 
         } 
+        oCharsetSource = kCharsetFromByteOrderMark;
+     // For html, meta tag detector is invoked before this so that we have 
+     // to deal only with XML here.
      } else if(                     (0x3F==aBytes[1]) &&
                (0x78==aBytes[2]) && (0x6D==aBytes[3]) &&
                (0 == PL_strncmp("<?xml", (char*)aBytes, 5 ))) {
@@ -2096,7 +2115,7 @@ static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsS
                    // encoding value is invalid if it is UTF-16
                    if (count > 0 && 
                      (0 != PL_strcmp("UTF-16", (char*)(aBytes+encStart)))) {
-                     oCharset.AssignWithConversion((char*)(aBytes+encStart),count);
+                     oCharset.Assign((char*)(aBytes+encStart),count);
                      oCharsetSource = kCharsetFromMetaTag;
                    }
                    encodingFound = PR_TRUE;
@@ -2116,23 +2135,31 @@ static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsS
      if((0xBB==aBytes[1]) && (0xBF==aBytes[2])) {
         // EF BB BF
         // Win2K UTF-8 BOM
-        oCharset.AssignWithConversion(UTF8); 
+        oCharset.Assign(UTF8); 
         oCharsetSource= kCharsetFromByteOrderMark;
      }
    break;
    case 0xFE:
      if(0xFF==aBytes[1]) {
-        // FE FF
-        // UTF-16, big-endian 
-        oCharset.AssignWithConversion(UCS2_BE); // should change to UTF-16BE
+        if(0x00==aBytes[2] && 0x00==aBytes[3]) {
+          // FE FF 00 00  UCS-4, unusual octet order (3412)
+          oCharset.Assign(UCS4_3412);
+        } else {
+          // FE FF UTF-16, big-endian 
+          oCharset.Assign(UTF16_BE); 
+        }
         oCharsetSource= kCharsetFromByteOrderMark;
      }
    break;
    case 0xFF:
      if(0xFE==aBytes[1]) {
+        if(0x00==aBytes[2] && 0x00==aBytes[3]) 
+         // FF FE 00 00  UTF-32, little-endian
+           oCharset.Assign(UCS4_LE); 
+        else
         // FF FE
         // UTF-16, little-endian 
-        oCharset.AssignWithConversion(UCS2_LE); // should change to UTF-16LE
+           oCharset.Assign(UTF16_LE); 
         oCharsetSource= kCharsetFromByteOrderMark;
      }
    break;
@@ -2141,7 +2168,7 @@ static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsS
    // }
    // break;
  }  // switch
- return oCharset.Length() > 0;
+ return !oCharset.IsEmpty();
 }
 
 inline const char GetNextChar(nsACString::const_iterator& aStart,
@@ -2154,7 +2181,7 @@ inline const char GetNextChar(nsACString::const_iterator& aStart,
 PRBool 
 nsParser::DetectMetaTag(const char* aBytes, 
                         PRInt32 aLen, 
-                        nsString& aCharset, 
+                        nsCString& aCharset, 
                         PRInt32& aCharsetSource) 
 {
   aCharsetSource= kCharsetFromMetaTag;
@@ -2253,7 +2280,7 @@ nsParser::DetectMetaTag(const char* aBytes,
 
     // return true if we successfully got something for charset
     if (currPos != tokEnd) {
-      aCharset.Assign(NS_ConvertASCIItoUCS2(currPos.get(), tokEnd.get() - currPos.get()));
+      aCharset.Assign(currPos.get(), tokEnd.get() - currPos.get());
       return PR_TRUE;
     } 
     
@@ -2296,8 +2323,9 @@ ParserWriteFunc(nsIInputStream* in,
   }
 
   if(pws->mNeedCharsetCheck) { 
-    PRInt32 guessSource; 
-    nsAutoString guess, preferred; 
+    PRInt32 guessSource;
+    nsCAutoString guess;
+    nsCAutoString preferred; 
   
     pws->mNeedCharsetCheck = PR_FALSE; 
     if(pws->mParser->DetectMetaTag(buf, theNumRead,
@@ -2311,14 +2339,14 @@ ParserWriteFunc(nsIInputStream* in,
       // one of a designated set that we ignore.
       if (NS_SUCCEEDED(result) &&
           ((kCharsetFromByteOrderMark == guessSource) ||
-           (!preferred.Equals(NS_LITERAL_STRING("UTF-16")) &&
-            !preferred.Equals(NS_LITERAL_STRING("UTF-16BE")) &&
-            !preferred.Equals(NS_LITERAL_STRING("UTF-16LE")) &&
-            !preferred.Equals(NS_LITERAL_STRING("UTF-32BE")) &&
-            !preferred.Equals(NS_LITERAL_STRING("UTF-32LE"))))) {
-        guess.Assign(preferred);
+           (!preferred.Equals(NS_LITERAL_CSTRING("UTF-16")) &&
+            !preferred.Equals(NS_LITERAL_CSTRING("UTF-16BE")) &&
+            !preferred.Equals(NS_LITERAL_CSTRING("UTF-16LE")) &&
+            !preferred.Equals(NS_LITERAL_CSTRING("UTF-32BE")) &&
+            !preferred.Equals(NS_LITERAL_CSTRING("UTF-32LE"))))) {
+        guess = preferred;
         pws->mParser->SetDocumentCharset(guess, guessSource); 
-        pws->mParser->SetSinkCharset(guess);
+        pws->mParser->SetSinkCharset(preferred);
         nsCOMPtr<nsICachingChannel> channel(do_QueryInterface(pws->mRequest));
         if (channel) {
           nsCOMPtr<nsISupports> cacheToken;
@@ -2330,7 +2358,7 @@ ParserWriteFunc(nsIInputStream* in,
               nsresult rv =
 #endif
                 cacheDescriptor->SetMetaDataElement("charset",
-                                                    NS_ConvertUCS2toUTF8(guess).get());
+                                                    guess.get());
               NS_ASSERTION(NS_SUCCEEDED(rv),"cannot SetMetaDataElement");
             }
           }
@@ -2458,7 +2486,7 @@ nsresult nsParser::OnStopRequest(nsIRequest *request, nsISupports* aContext,
 
   // XXX Should we wait to notify our observers as well if the
   // parser isn't yet enabled?
-  if (nsnull != mObserver) {
+  if (mObserver) {
     mObserver->OnStopRequest(request, aContext, status);
   }
 
@@ -2595,36 +2623,10 @@ PRBool nsParser::DidTokenize(PRBool aIsFinalChunk){
 
   if (NS_SUCCEEDED(rv) && theTokenizer) {
     result = theTokenizer->DidTokenize(aIsFinalChunk);
-    if(mTokenObserver) {
-      PRInt32 theCount=theTokenizer->GetCount();
-      PRInt32 theIndex;
-      for(theIndex=0;theIndex<theCount;++theIndex){
-        if((*mTokenObserver)(theTokenizer->GetTokenAt(theIndex))){
-          //add code here to pull unwanted tokens out of the stack...
-        }
-      }//for      
-    }//if
   }
   return result;
 }
 
-#ifdef DEBUG
-void nsParser::DebugDumpSource(nsOutputStream& aStream) {
-  PRInt32 theIndex=-1;
-
-  nsITokenizer* theTokenizer=0;
-  PRInt32 type = mParserContext && mParserContext->mDTD ? 
-    mParserContext->mDTD->GetType() : NS_IPARSER_FLAG_HTML;
-  if(NS_SUCCEEDED(mParserContext->GetTokenizer(type, theTokenizer))){
-    CToken* theToken;
-    while(nsnull != (theToken=theTokenizer->GetTokenAt(++theIndex))) {
-      // theToken->DebugDumpToken(out);
-      theToken->DebugDumpSource(aStream);
-    }
-  }
-}
-#endif
-
 /** 
  * Get the channel associated with this parser
  * @update harishd,gagan 07/17/01
