@@ -241,7 +241,8 @@ nsresult nsDateTimeFormatMac::Initialize(nsILocale* locale)
 
   // use cached info if match with stored locale
   if (NULL == locale) {
-    if (mLocale.Length() && mLocale.EqualsIgnoreCase(mAppLocale)) {
+    if (mLocale.Length() &&
+        mLocale.Equals(mAppLocale, nsCaseInsensitiveStringComparator())) {
       return NS_OK;
     }
   }
@@ -261,7 +262,7 @@ nsresult nsDateTimeFormatMac::Initialize(nsILocale* locale)
   mScriptcode = smSystemScript;
   mLangcode = langEnglish;
   mRegioncode = verUS;
-  mCharset.Assign(NS_LITERAL_STRING("x-mac-roman"));
+  mCharset.Assign(NS_LITERAL_CSTRING("x-mac-roman"));
   
 
   // get application locale
@@ -304,7 +305,7 @@ nsresult nsDateTimeFormatMac::Initialize(nsILocale* locale)
       PRUnichar* mappedCharset = NULL;
       res = platformCharset->GetDefaultCharsetForLocale(mLocale.get(), &mappedCharset);
       if (NS_SUCCEEDED(res) && mappedCharset) {
-        mCharset.Assign(mappedCharset);
+        mCharset.AssignWithConversion(mappedCharset);
         nsMemory::Free(mappedCharset);
       }
       
@@ -315,14 +316,12 @@ nsresult nsDateTimeFormatMac::Initialize(nsILocale* locale)
 
   // Initialize unicode decoder
   nsCOMPtr <nsIAtom>                      charsetAtom;
-  nsCOMPtr <nsICharsetConverterManager2>  charsetConverterManager;
+  nsCOMPtr <nsICharsetConverterManager>  charsetConverterManager;
   charsetConverterManager = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &res);
   if (NS_SUCCEEDED(res)) {
-    res = charsetConverterManager->GetCharsetAtom(mUseDefaultLocale ? mSystemCharset.get() : mCharset.get(), 
-                                                  getter_AddRefs(charsetAtom));
-    if (NS_SUCCEEDED(res)) {
-      res = charsetConverterManager->GetUnicodeDecoder(charsetAtom, getter_AddRefs(mDecoder));
-    }
+    res = charsetConverterManager->GetUnicodeDecoder(mUseDefaultLocale ? mSystemCharset.get() : 
+                                                     mCharset.get(), 
+                                                     getter_AddRefs(mDecoder));
   }
   
   return res;
@@ -338,6 +337,15 @@ nsresult nsDateTimeFormatMac::FormatTime(nsILocale* locale,
   return FormatTMTime(locale, dateFormatSelector, timeFormatSelector, localtime(&timetTime), stringOut);
 }
 
+#if !TARGET_CARBON
+static void CopyPascalStringToC(StringPtr aSrc, char* aDest)
+{
+  int len = aSrc[0];
+  ::BlockMoveData(aSrc + 1, aDest, len);
+  aDest[len] = '\0';
+}
+#endif
+
 // performs a locale sensitive date formatting operation on the struct tm parameter
 nsresult nsDateTimeFormatMac::FormatTMTime(nsILocale* locale, 
                                            const nsDateFormatSelector  dateFormatSelector, 
@@ -427,17 +435,18 @@ nsresult nsDateTimeFormatMac::FormatTMTime(nsILocale* locale,
   }
   
   // construct a C string
-  char *localBuffer;
+  char localBuffer[2 * 255 + 2 + 1]; // 2 255-byte-max pascal strings, plus
+                                     // a space and a null.
   if (dateFormatSelector != kDateFormatNone && timeFormatSelector != kTimeFormatNone) {
-    localBuffer = p2cstr(dateString);
-    strcat(localBuffer, " ");
-    strcat(localBuffer, p2cstr(timeString));
+    CopyPascalStringToC(dateString, localBuffer);
+    localBuffer[dateString[0]] = ' ';
+    CopyPascalStringToC(timeString, &(localBuffer[dateString[0] + 1]));
   }
   else if (dateFormatSelector != kDateFormatNone) {
-    localBuffer = p2cstr(dateString);
+    CopyPascalStringToC(dateString, localBuffer);
   }
   else if (timeFormatSelector != kTimeFormatNone) {
-    localBuffer = p2cstr(timeString);
+    CopyPascalStringToC(timeString, localBuffer);
   }
 
   // convert result to unicode
@@ -476,7 +485,7 @@ nsresult nsDateTimeFormatMac::FormatPRExplodedTime(nsILocale* locale,
                                                    nsString& stringOut)
 {
   struct tm  tmTime;
-  nsCRT::memset( &tmTime, 0, sizeof(tmTime) );
+  memset( &tmTime, 0, sizeof(tmTime) );
 
   tmTime.tm_yday = explodedTime->tm_yday;
   tmTime.tm_wday = explodedTime->tm_wday;
