@@ -56,7 +56,6 @@ NS_IMPL_ISUPPORTS1(nsPlatformCharset, nsIPlatformCharset);
 
 nsPlatformCharset::nsPlatformCharset()
 {
-  NS_INIT_ISUPPORTS();
   PR_AtomicIncrement(&gCnt);
 }
 nsPlatformCharset::~nsPlatformCharset()
@@ -81,16 +80,16 @@ nsresult nsPlatformCharset::InitInfo()
   return NS_OK;
 }
 
-nsresult nsPlatformCharset::MapToCharset(short script, short region, nsAString& outCharset)
+nsresult nsPlatformCharset::MapToCharset(short script, short region, nsACString& outCharset)
 {
   switch (region) {
     case verUS:
     case verFrance:
     case verGermany:
-      outCharset.Assign(NS_LITERAL_STRING("x-mac-roman"));
+      outCharset.Assign(NS_LITERAL_CSTRING("x-mac-roman"));
       return NS_OK;
     case verJapan:
-      outCharset.Assign(NS_LITERAL_STRING("Shift_JIS"));
+      outCharset.Assign(NS_LITERAL_CSTRING("Shift_JIS"));
       return NS_OK;
   }
 
@@ -102,14 +101,19 @@ nsresult nsPlatformCharset::MapToCharset(short script, short region, nsAString&
   nsAutoString key(NS_LITERAL_STRING("region."));
   key.AppendInt(region, 10);
 
-  rv = gInfo->Get(key, outCharset);
-  if (NS_FAILED(rv)) {
+  nsAutoString uCharset;
+  rv = gInfo->Get(key, uCharset);
+  if (NS_SUCCEEDED(rv))
+    CopyUCS2toASCII(uCharset, outCharset);
+  else {
     key.Assign(NS_LITERAL_STRING("script."));
     key.AppendInt(script, 10);
-    rv = gInfo->Get(key, outCharset);
+    rv = gInfo->Get(key, uCharset);
     // not found in the .property file, assign x-mac-roman
-    if (NS_FAILED(rv)) { 
-      outCharset.Assign(NS_LITERAL_STRING("x-mac-roman"));
+    if (NS_SUCCEEDED(rv))
+      CopyUCS2toASCII(uCharset, outCharset);
+    else {
+      outCharset.Assign(NS_LITERAL_CSTRING("x-mac-roman"));
     }
   }
   
@@ -117,7 +121,7 @@ nsresult nsPlatformCharset::MapToCharset(short script, short region, nsAString&
 }
 
 NS_IMETHODIMP 
-nsPlatformCharset::GetCharset(nsPlatformCharsetSel selector, nsAString& oResult)
+nsPlatformCharset::GetCharset(nsPlatformCharsetSel selector, nsACString& oResult)
 {
   nsresult rv;
   if (mCharset.IsEmpty()) {
@@ -129,6 +133,11 @@ nsPlatformCharset::GetCharset(nsPlatformCharsetSel selector, nsAString& oResult)
   }
 
   switch (selector) {
+#ifdef XP_MACOSX  
+    case kPlatformCharsetSel_FileName:
+      oResult.Assign(NS_LITERAL_CSTRING("UTF-8"));
+      break;
+#endif
     case  kPlatformCharsetSel_KeyboardInput:
       rv = MapToCharset(
              (short) (0x0000FFFF & ::GetScriptManagerVariable(smKeyScript)),
@@ -147,7 +156,8 @@ NS_IMETHODIMP
 nsPlatformCharset::GetDefaultCharsetForLocale(const PRUnichar* localeName, PRUnichar** _retValue)
 {
   nsCOMPtr<nsIMacLocale>	pMacLocale;
-  nsAutoString localeAsString(localeName), charset(NS_LITERAL_STRING("x-mac-roman"));
+  nsAutoString localeAsString(localeName);
+  nsCAutoString charset(NS_LITERAL_CSTRING("x-mac-roman"));
   short script, language, region;
 	
   nsresult rv;
@@ -171,13 +181,13 @@ nsPlatformCharset::Init()
 }
 
 nsresult 
-nsPlatformCharset::MapToCharset(nsAString& inANSICodePage, nsAString& outCharset)
+nsPlatformCharset::MapToCharset(nsAString& inANSICodePage, nsACString& outCharset)
 {
   return NS_OK;
 }
 
 nsresult
-nsPlatformCharset::InitGetCharset(nsAString &oString)
+nsPlatformCharset::InitGetCharset(nsACString &oString)
 {
   return NS_OK;
 }
@@ -189,7 +199,7 @@ nsPlatformCharset::ConvertLocaleToCharsetUsingDeprecatedConfig(nsAutoString& loc
 }
 
 nsresult
-nsPlatformCharset::VerifyCharset(nsString &aCharset)
+nsPlatformCharset::VerifyCharset(nsCString &aCharset)
 {
   return NS_OK;
 }
