@@ -43,16 +43,18 @@
 #include "nsIComponentManager.h"
 #include "nsICategoryManager.h"
 #include "nsICharsetConverterManager.h"
-#include "nsICharsetConverterManager2.h"
+#include "nsEncoderDecoderUtils.h"
 #include "nsIUnicodeDecoder.h"
 #include "nsIUnicodeEncoder.h"
 #include "nsICharsetAlias.h"
 #include "nsIServiceManager.h"
 
 
+#include "nsUConvCID.h"
 #include "nsCharsetConverterManager.h"
 #include "nsCharsetAlias.h"
 #include "nsTextToSubURI.h"
+#include "nsUTF8ConverterService.h"
 #include "nsConverterInputStream.h"
 #include "nsPlatformCharset.h"
 
@@ -114,6 +116,8 @@
 #include "nsCP1257ToUnicode.h"
 #include "nsCP1258ToUnicode.h"
 #include "nsCP874ToUnicode.h"
+#include "nsISO885911ToUnicode.h"
+#include "nsTIS620ToUnicode.h"
 #include "nsCP866ToUnicode.h"
 #include "nsKOI8RToUnicode.h"
 #include "nsKOI8UToUnicode.h"
@@ -165,6 +169,8 @@
 #include "nsUnicodeToCP1257.h"
 #include "nsUnicodeToCP1258.h"
 #include "nsUnicodeToCP874.h"
+#include "nsUnicodeToISO885911.h"
+#include "nsUnicodeToTIS620.h"
 #include "nsUnicodeToCP866.h"
 #include "nsUnicodeToKOI8R.h"
 #include "nsUnicodeToKOI8U.h"
@@ -203,6 +209,7 @@
 #include "nsUnicodeToMacGujarati.h"
 #include "nsUnicodeToMacGurmukhi.h"
 #include "nsUnicodeToMacHebrew.h"
+#include "nsUnicodeToTSCII.h"
 #ifdef MOZ_EXTRA_X11CONVERTERS
 #include "nsUnicodeToLangBoxArabic8.h"
 #include "nsUnicodeToLangBoxArabic16.h"
@@ -339,7 +346,9 @@ NS_UCONV_REG_UNREG("windows-1255", NS_CP1255TOUNICODE_CID, NS_UNICODETOCP1255_CI
 NS_UCONV_REG_UNREG("windows-1256", NS_CP1256TOUNICODE_CID, NS_UNICODETOCP1256_CID)
 NS_UCONV_REG_UNREG("windows-1257", NS_CP1257TOUNICODE_CID, NS_UNICODETOCP1257_CID)
 NS_UCONV_REG_UNREG("windows-1258", NS_CP1258TOUNICODE_CID, NS_UNICODETOCP1258_CID)
-NS_UCONV_REG_UNREG("TIS-620", NS_CP874TOUNICODE_CID, NS_UNICODETOCP874_CID)
+NS_UCONV_REG_UNREG("TIS-620", NS_TIS620TOUNICODE_CID, NS_UNICODETOTIS620_CID)
+NS_UCONV_REG_UNREG("windows-874", NS_CP874TOUNICODE_CID, NS_UNICODETOCP874_CID)
+NS_UCONV_REG_UNREG("ISO-8859-11", NS_ISO885911TOUNICODE_CID, NS_UNICODETOISO885911_CID)
 NS_UCONV_REG_UNREG("IBM866", NS_CP866TOUNICODE_CID, NS_UNICODETOCP866_CID)
 NS_UCONV_REG_UNREG("KOI8-R", NS_KOI8RTOUNICODE_CID, NS_UNICODETOKOI8R_CID)
 NS_UCONV_REG_UNREG("KOI8-U", NS_KOI8UTOUNICODE_CID, NS_UNICODETOKOI8U_CID)
@@ -374,6 +383,8 @@ NS_UCONV_REG_UNREG("x-mac-hebrew" , NS_MACHEBREWTOUNICODE_CID, NS_UNICODETOMACHE
 NS_UCONV_REG_UNREG_ENCODER("UTF-16" , NS_UNICODETOUTF16_CID)
 NS_UCONV_REG_UNREG_ENCODER("Adobe-Symbol-Encoding" , NS_UNICODETOSYMBOL_CID)
 NS_UCONV_REG_UNREG_ENCODER("x-zapf-dingbats" , NS_UNICODETOZAPFDINGBATS_CID)
+NS_UCONV_REG_UNREG_ENCODER("x-tscii",  NS_UNICODETOTSCII_CID)
+NS_UCONV_REG_UNREG_ENCODER("x-tamilttf-0",  NS_UNICODETOTAMILTTF_CID)
 #ifdef MOZ_EXTRA_X11CONVERTERS
 NS_UCONV_REG_UNREG_ENCODER("x-iso-8859-6-8-x" , NS_UNICODETOLANGBOXARABIC_CID)
 NS_UCONV_REG_UNREG_ENCODER("x-iso-8859-6-16" , NS_UNICODETOLANGBOXARABIC16_CID)
@@ -471,6 +482,8 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToUTF16LE);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToUTF16);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToUTF32BE);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToUTF32LE);
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToTSCII);
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToTamilTTF);
 #ifdef MOZ_EXTRA_X11CONVERTERS
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToLangBoxArabic8);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToLangBoxArabic16);
@@ -672,6 +685,7 @@ nsConverterManagerDataRegister(nsIComponentManager* aCompMgr,
 
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsCharsetConverterManager)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsTextToSubURI)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsUTF8ConverterService)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsCharsetAlias2)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsConverterInputStream)
 NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsPlatformCharset, Init)
@@ -721,6 +735,11 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODEHELPER_CONTRACTID, 
     nsUnicodeEncodeHelperConstructor 
   },
+  { 
+    "Converter to/from UTF8 with charset", NS_UTF8CONVERTERSERVICE_CID,
+    NS_UTF8CONVERTERSERVICE_CONTRACTID, 
+    nsUTF8ConverterServiceConstructor
+  },
   { 
     "Unicode Encoder / Decoder for Script", NS_ISCRIPTABLEUNICODECONVERTER_CID,
     NS_ISCRIPTABLEUNICODECONVERTER_CONTRACTID, 
@@ -911,8 +930,18 @@ static const nsModuleComponentInfo components[] =
     nsCP1258ToUnicodeConstructor ,
   },
   { 
-    DECODER_NAME_BASE "TIS-620" , NS_CP874TOUNICODE_CID, 
+    DECODER_NAME_BASE "TIS-620" , NS_TIS620TOUNICODE_CID, 
     NS_UNICODEDECODER_CONTRACTID_BASE "TIS-620",
+    nsTIS620ToUnicodeConstructor ,
+  },
+  { 
+    DECODER_NAME_BASE "ISO-8859-11" , NS_ISO885911TOUNICODE_CID, 
+    NS_UNICODEDECODER_CONTRACTID_BASE "ISO-8859-11",
+    nsISO885911ToUnicodeConstructor ,
+  },
+  { 
+    DECODER_NAME_BASE "windows-874" , NS_CP874TOUNICODE_CID, 
+    NS_UNICODEDECODER_CONTRACTID_BASE "windows-874",
     nsCP874ToUnicodeConstructor ,
   },
   { 
@@ -1206,9 +1235,19 @@ static const nsModuleComponentInfo components[] =
     nsUnicodeToCP1258Constructor, 
   },
   { 
-    ENCODER_NAME_BASE "TIS-620" , NS_UNICODETOCP874_CID, 
+    ENCODER_NAME_BASE "TIS-620" , NS_UNICODETOTIS620_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "TIS-620",
-    nsUnicodeToCP874Constructor, 
+    nsUnicodeToTIS620Constructor, 
+  },
+  { 
+    ENCODER_NAME_BASE "ISO-8859-11" , NS_UNICODETOISO885911_CID, 
+    NS_UNICODEENCODER_CONTRACTID_BASE "ISO-8859-11",
+    nsUnicodeToISO885911Constructor ,
+  },
+  { 
+    ENCODER_NAME_BASE "windows-874" , NS_UNICODETOCP874_CID, 
+    NS_UNICODEENCODER_CONTRACTID_BASE "windows-874",
+    nsUnicodeToCP874Constructor ,
   },
   { 
     ENCODER_NAME_BASE "IBM866" , NS_UNICODETOCP866_CID, 
@@ -1380,6 +1419,16 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "x-mac-hebrew",
     nsUnicodeToMacHebrewConstructor, 
   },
+  { 
+    ENCODER_NAME_BASE "x-tscii" , NS_UNICODETOTSCII_CID,
+    NS_UNICODEENCODER_CONTRACTID_BASE "x-tscii",
+    nsUnicodeToTSCIIConstructor, 
+  },
+  { 
+    ENCODER_NAME_BASE "x-tamilttf-0" , NS_UNICODETOTAMILTTF_CID,
+    NS_UNICODEENCODER_CONTRACTID_BASE "x-tamilttf-0",
+    nsUnicodeToTamilTTFConstructor, 
+  },
 #ifdef MOZ_EXTRA_X11CONVERTERS
   { 
     ENCODER_NAME_BASE "x-iso-8859-6-8-x" , NS_UNICODETOLANGBOXARABIC_CID, 
