@@ -95,20 +95,25 @@
     {0xad, 0x26, 0x11, 0x3f, 0x2c, 0x02, 0xd0, 0xfe} \
 }
 
+#define JSDS_MAJOR_VERSION 1
+#define JSDS_MINOR_VERSION 1
+
 #define NS_CATMAN_CTRID   "@mozilla.org/categorymanager;1"
 #define NS_JSRT_CTRID     "@mozilla.org/js/xpc/RuntimeService;1"
 
 #define AUTOREG_CATEGORY  "xpcom-autoregistration"
 #define APPSTART_CATEGORY "app-startup"
-#define JSD_STARTUP_ENTRY "JSDebugger Startup Observer"
+#define JSD_AUTOREG_ENTRY "JSDebugger Startup Observer"
+#define JSD_STARTUP_ENTRY "JSDebugger Startup Observer,service"
 
 JS_STATIC_DLL_CALLBACK (JSBool)
 jsds_GCCallbackProc (JSContext *cx, JSGCStatus status);
 
 /*******************************************************************************
  * global vars
- *******************************************************************************/
+ ******************************************************************************/
 
+const char implementationString[] = "Mozilla JavaScript Debugger Service";
 static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
 
@@ -860,7 +865,6 @@ jsdProperty::jsdProperty (JSDContext *aCx, JSDProperty *aProperty) :
 {
     DEBUG_CREATE ("jsdProperty", gPropertyCount);
     mValid = (aCx && aProperty);
-    NS_INIT_ISUPPORTS();
     mLiveListEntry.value = this;
     jsds_InsertEphemeral (&gLiveProperties, &mLiveListEntry);
 }
@@ -966,7 +970,6 @@ jsdScript::jsdScript (JSDContext *aCx, JSDScript *aScript) : mValid(PR_FALSE),
                                                              mFirstPC(0)
 {
     DEBUG_CREATE ("jsdScript", gScriptCount);
-    NS_INIT_ISUPPORTS();
 
     if (mScript) {
         /* copy the script's information now, so we have it later, when it
@@ -1218,6 +1221,33 @@ jsdScript::GetFunctionName(char **_rval)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+jsdScript::GetFunctionObject(jsdIValue **_rval)
+{
+    JSFunction *fun = JSD_GetJSFunction(mCx, mScript);
+    if (!fun)
+        return NS_ERROR_NOT_AVAILABLE;
+    
+    JSObject *obj = JS_GetFunctionObject(fun);
+    if (!obj)
+        return NS_ERROR_FAILURE;
+
+    JSDContext *cx;
+    gJsds->GetJSDContext (&cx);
+
+    JSDValue *jsdv = JSD_NewValue(cx, OBJECT_TO_JSVAL(obj));
+    if (!jsdv)
+        return NS_ERROR_FAILURE;
+
+    *_rval = jsdValue::FromPtr(cx, jsdv);
+    if (!*_rval) {
+        JSD_DropValue(cx, jsdv);
+        return NS_ERROR_FAILURE;
+    }
+
+    return NS_OK;
+}
+
 NS_IMETHODIMP
 jsdScript::GetFunctionSource(nsAString & aFunctionSource)
 {
@@ -1430,7 +1460,6 @@ jsdContext::jsdContext (JSDContext *aJSDCx, JSContext *aJSCx,
                                               mJSCx(aJSCx), mISCx(aISCx)
 {
     DEBUG_CREATE ("jsdContext", gContextCount);
-    NS_INIT_ISUPPORTS();
     mLiveListEntry.value = this;
     mLiveListEntry.key   = NS_STATIC_CAST (void *, aJSCx);
     jsds_InsertEphemeral (&gLiveContexts, &mLiveListEntry);
@@ -1600,7 +1629,6 @@ jsdStackFrame::jsdStackFrame (JSDContext *aCx, JSDThreadState *aThreadState,
 {
     DEBUG_CREATE ("jsdStackFrame", gFrameCount);
     mValid = (aCx && aThreadState && aStackFrameInfo);
-    NS_INIT_ISUPPORTS();
     if (mValid) {
         mLiveListEntry.key = aStackFrameInfo;
         mLiveListEntry.value = this;
@@ -1838,6 +1866,10 @@ jsdStackFrame::Eval (const nsAString &bytes, const char *fileName,
                      PRUint32 line, jsdIValue **result, PRBool *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
+
+    if (bytes.IsEmpty())
+        return NS_ERROR_INVALID_ARG;
+
     const nsSharedBufferHandle<PRUnichar> *h = bytes.GetSharedBufferHandle();
     const jschar *char_bytes = NS_REINTERPRET_CAST(const jschar *,
                                                    h->DataStart());
@@ -1890,7 +1922,6 @@ jsdValue::jsdValue (JSDContext *aCx, JSDValue *aValue) : mValid(PR_TRUE),
                                                          mValue(aValue)
 {
     DEBUG_CREATE ("jsdValue", gValueCount);
-    NS_INIT_ISUPPORTS();
     mLiveListEntry.value = this;
     jsds_InsertEphemeral (&gLiveValues, &mLiveListEntry);
 }
@@ -2239,7 +2270,7 @@ jsdService::GetInitAtStartup (PRBool *_rval)
         nsresult autoreg_rv, appstart_rv;
         
         autoreg_rv = categoryManager->GetCategoryEntry(AUTOREG_CATEGORY, 
-                                                       JSD_STARTUP_ENTRY,
+                                                       JSD_AUTOREG_ENTRY,
                                                        getter_Copies(notused));
         appstart_rv = categoryManager->GetCategoryEntry(APPSTART_CATEGORY,
                                                         JSD_STARTUP_ENTRY,
@@ -2307,7 +2338,7 @@ jsdService::SetInitAtStartup (PRBool state)
 
     if (state) {
         rv = categoryManager->AddCategoryEntry(AUTOREG_CATEGORY,
-                                               JSD_STARTUP_ENTRY,
+                                               JSD_AUTOREG_ENTRY,
                                                jsdASObserverCtrID,
                                                PR_TRUE, PR_TRUE, nsnull);
         if (NS_FAILED(rv))
@@ -2321,7 +2352,7 @@ jsdService::SetInitAtStartup (PRBool state)
         mInitAtStartup = triYes;
     } else {
         rv = categoryManager->DeleteCategoryEntry(AUTOREG_CATEGORY,
-                                                  JSD_STARTUP_ENTRY, PR_TRUE);
+                                                  JSD_AUTOREG_ENTRY, PR_TRUE);
         if (NS_FAILED(rv))
             return rv;
         rv = categoryManager->DeleteCategoryEntry(APPSTART_CATEGORY,
@@ -2348,6 +2379,29 @@ jsdService::SetFlags (PRUint32 flags)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+jsdService::GetImplementationString(char **_rval)
+{
+    *_rval = PL_strdup(implementationString);
+    if (!*_rval)
+        return NS_ERROR_OUT_OF_MEMORY;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+jsdService::GetImplementationMajor(PRUint32 *_rval)
+{
+    *_rval = JSDS_MAJOR_VERSION;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+jsdService::GetImplementationMinor(PRUint32 *_rval)
+{
+    *_rval = JSDS_MINOR_VERSION;
+    return NS_OK;
+}
+
 NS_IMETHODIMP
 jsdService::GetIsOn (PRBool *_rval)
 {
@@ -3183,7 +3237,6 @@ class jsdASObserver : public nsIObserver
 
     jsdASObserver ()
     {
-        NS_INIT_ISUPPORTS();
     }    
 };
 
