@@ -68,17 +68,6 @@
 #include "jsprf.h"
 #include "jsstr.h"
 
-union dpun {
-    struct {
-#ifdef IS_LITTLE_ENDIAN
-	uint32 lo, hi;
-#else
-	uint32 hi, lo;
-#endif
-    } s;
-    jsdouble d;
-};
-
 static JSBool
 num_isNaN(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
@@ -425,7 +414,7 @@ enum nc_slot {
 /*
  * Some to most C compilers forbid spelling these at compile time, or barf
  * if you try, so all but MAX_VALUE are set up by js_InitRuntimeNumberState
- * using union dpun.
+ * using union jsdpun.
  */
 static JSConstDoubleSpec number_constants[] = {
     {0,                         js_NaN_str,          0,{0,0,0}},
@@ -461,7 +450,7 @@ JSBool
 js_InitRuntimeNumberState(JSContext *cx)
 {
     JSRuntime *rt;
-    union dpun u;
+    jsdpun u;
 
     rt = cx->runtime;
     JS_ASSERT(!rt->jsNaN);
@@ -958,7 +947,7 @@ js_strtointeger(JSContext *cx, const jschar *s, const jschar **ep, jsint base, j
      */
     start = s1; /* Mark - if string is empty, we return NaN. */
     value = 0.0;
-    while (1) {
+    for (;;) {
         uintN digit;
         jschar c = *s1;
         if ('0' <= c && c <= '9')
