@@ -43,6 +43,8 @@
 #include "nsAString.h"
 #include "nsIRequest.h"
 #include "nsCompatibility.h"
+#include "nsCOMPtr.h"
+#include "nsIPresShell.h"
 #ifdef IBMBIDI
 class nsBidiPresUtils;
 #endif // IBMBIDI
@@ -61,7 +63,7 @@ class nsIImage;
 class nsILinkHandler;
 class nsIPresShell;
 class nsIPref;
-class nsIStyleContext;
+class nsStyleContext;
 class nsIAtom;
 class nsString;
 class nsIEventStateManager;
@@ -71,6 +73,8 @@ class nsIIOService;
 class nsICSSPseudoComparator;
 class nsILanguageAtom;
 class nsITheme;
+struct nsStyleStruct;
+struct nsStyleBackground;
 
 #ifdef MOZ_REFLOW_PERF
 class nsIRenderingContext;
@@ -80,12 +84,6 @@ class nsIRenderingContext;
 { 0x0a5d12e0, 0x944e, 0x11d1, \
   {0x93, 0x23, 0x00, 0x80, 0x5f, 0x8a, 0xdd, 0x32} }
 
-enum nsWidgetRendering {
-  eWidgetRendering_Native   = 1,
-  eWidgetRendering_Gfx      = 2,
-  eWidgetRendering_PartialGfx = 3
-};
-
 enum nsWidgetType {
   eWidgetType_Button  	= 1,
   eWidgetType_Checkbox	= 2,
@@ -134,6 +132,11 @@ public:
    * Get the PresentationShell that this context is bound to.
    */
   NS_IMETHOD GetShell(nsIPresShell** aResult) = 0;
+  nsIPresShell* GetPresShell() { return mShell; }
+
+  nsIDocument* GetDocument() { return GetPresShell()->GetDocument(); } 
+  nsIViewManager* GetViewManager() { return GetPresShell()->GetViewManager(); } 
+  nsIStyleSet* GetStyleSet() { return GetPresShell()->GetStyleSet(); } 
 
   /**
    * Access compatibility mode for this context
@@ -144,12 +147,6 @@ public:
   NS_IMETHOD GetCompatibilityMode(nsCompatibility* aModeResult) = 0;
   NS_IMETHOD SetCompatibilityMode(nsCompatibility aMode) = 0;
 
-  /**
-   * Access the widget rendering mode for this context
-   */
-  NS_IMETHOD GetWidgetRenderingMode(nsWidgetRendering* aModeResult) = 0;
-  NS_IMETHOD SetWidgetRenderingMode(nsWidgetRendering aMode) = 0;
-
   /**
    * Access the image animation mode for this context
    */
@@ -190,9 +187,9 @@ public:
    * Resolve style for the given piece of content that will be a child
    * of the aParentContext. Don't use this for pseudo frames.
    */
-  NS_IMETHOD ResolveStyleContextFor(nsIContent* aContent,
-                                    nsIStyleContext* aParentContext,
-                                    nsIStyleContext** aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ResolveStyleContextFor(nsIContent* aContent,
+                         nsStyleContext* aParentContext) = 0;
 
   /**
    * Resolve style for a non-element content node (i.e., one that is
@@ -206,19 +203,19 @@ public:
    * represents everything except the first letter.)
    *
    */
-  NS_IMETHOD ResolveStyleContextForNonElement(
-                                    nsIStyleContext* aParentContext,
-                                    nsIStyleContext** aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ResolveStyleContextForNonElement(nsStyleContext* aParentContext) = 0;
 
   /**
    * Resolve style for a pseudo frame within the given aParentContent & aParentContext.
    * The tag should be lowercase and inclue the colon.
    * ie: NS_NewAtom(":first-line");
    */
-  NS_IMETHOD ResolvePseudoStyleContextFor(nsIContent* aParentContent,
-                                          nsIAtom* aPseudoTag,
-                                          nsIStyleContext* aParentContext,
-                                          nsIStyleContext** aResult) = 0;
+
+  virtual already_AddRefed<nsStyleContext>
+  ResolvePseudoStyleContextFor(nsIContent* aParentContent,
+                               nsIAtom* aPseudoTag,
+                               nsStyleContext* aParentContext) = 0;
 
   /**
    * Resolve style for a pseudo frame within the given aParentContent & aParentContext.
@@ -228,11 +225,11 @@ public:
    * Instead of matching solely on aPseudoTag, a comparator function can be
    * passed in to test.
    */
-  NS_IMETHOD ResolvePseudoStyleWithComparator(nsIContent* aParentContent,
-                                              nsIAtom* aPseudoTag,
-                                              nsIStyleContext* aParentContext,
-                                              nsICSSPseudoComparator* aComparator,
-                                              nsIStyleContext** aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ResolvePseudoStyleWithComparator(nsIContent* aParentContent,
+                                   nsIAtom* aPseudoTag,
+                                   nsStyleContext* aParentContext,
+                                   nsICSSPseudoComparator* aComparator) = 0;
 
   /**
    * Probe style for a pseudo frame within the given aParentContent & aParentContext.
@@ -240,17 +237,24 @@ public:
    * The tag should be lowercase and inclue the colon.
    * ie: NS_NewAtom(":first-line");
    */
-  NS_IMETHOD ProbePseudoStyleContextFor(nsIContent* aParentContent,
-                                        nsIAtom* aPseudoTag,
-                                        nsIStyleContext* aParentContext,
-                                        nsIStyleContext** aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ProbePseudoStyleContextFor(nsIContent* aParentContent,
+                             nsIAtom* aPseudoTag,
+                             nsStyleContext* aParentContext) = 0;
+ 
+   /**
+    * Resolve a new style context for a content node and return the URL
+    * for its XBL binding, or the empty string if it has no binding
+    * specified in CSS.
+    */
+   NS_IMETHOD GetXBLBindingURL(nsIContent* aContent, nsAString& aResult) = 0;
 
   /** 
    * For a given frame tree, get a new style context that is the equivalent
    * but within a new parent
    */
   NS_IMETHOD ReParentStyleContext(nsIFrame* aFrame, 
-                                  nsIStyleContext* aNewParentContext) = 0;
+                                  nsStyleContext* aNewParentContext) = 0;
 
 
   NS_IMETHOD AllocateFromShell(size_t aSize, void** aResult) = 0;
@@ -395,10 +399,12 @@ public:
    */
   NS_IMETHOD GetScaledPixelsToTwips(float* aScale) const = 0;
 
-  //be sure to Relase() after you are done with the Get()
   NS_IMETHOD GetDeviceContext(nsIDeviceContext** aResult) const = 0;
+  nsIDeviceContext* GetDeviceContext() { return mDeviceContext; }
 
   NS_IMETHOD GetEventStateManager(nsIEventStateManager** aManager) = 0;
+  nsIEventStateManager* GetEventStateManager();
+
   NS_IMETHOD GetLanguage(nsILanguageAtom** aLanguage) = 0;
 
   /**
@@ -505,7 +511,7 @@ public:
   /**
    * Get the document charset
    */
-  NS_IMETHOD GetBidiCharset(nsAString &aCharSet) const = 0;
+  NS_IMETHOD GetBidiCharset(nsACString &aCharSet) const = 0;
 
 
 #endif // IBMBIDI
@@ -534,10 +540,34 @@ public:
    */
   NS_IMETHOD SysColorChanged() = 0;
 
+  /*
+   * Fill in an nsStyleBackground to be used to paint the background for an
+   * element.  This applies the rules for propagating backgrounds between
+   * BODY, the root element, and the canvas.
+   */
+  NS_IMETHOD FindFrameBackground(nsIFrame* aFrame,
+                                 const nsStyleBackground** aBackground,
+                                 PRBool* aIsCanavs,
+                                 PRBool* aFoundBackground) = 0;
+
 #ifdef MOZ_REFLOW_PERF
   NS_IMETHOD CountReflows(const char * aName, PRUint32 aType, nsIFrame * aFrame) = 0;
   NS_IMETHOD PaintCount(const char * aName, nsIRenderingContext* aRendingContext, nsIFrame * aFrame, PRUint32 aColor) = 0;
 #endif
+
+protected:
+  // IMPORTANT: The ownership implicit in the following member variables
+  // has been explicitly checked.  If you add any members to this class,
+  // please make the ownership explicit (pinkerton, scc).
+  
+  nsIPresShell*         mShell;         // [WEAK]
+  nsIDeviceContext*     mDeviceContext; // [STRONG] could be weak, but
+                                        // better safe than sorry.
+                                        // Cannot reintroduce cycles
+                                        // since there is no dependency
+                                        // from gfx back to layout.
+  nsIEventStateManager* mEventManager;  // [STRONG]
+
 };
 
 // Bit values for StartLoadImage's aImageStatus
@@ -547,8 +577,8 @@ public:
 
 // Factory method to create a "galley" presentation context (galley is
 // a kind of view that has no limit to the size of a page)
-extern NS_EXPORT nsresult
-  NS_NewGalleyContext(nsIPresContext** aInstancePtrResult);
+nsresult
+NS_NewGalleyContext(nsIPresContext** aInstancePtrResult);
 
 #ifdef MOZ_REFLOW_PERF
 
