@@ -63,7 +63,6 @@ class nsStyleContext;
 class nsIFrameSelection;
 class nsIFrameManager;
 class nsILayoutHistoryState;
-class nsIArena;
 class nsIReflowCallback;
 class nsISupportsArray;
 class nsIDOMNode;
@@ -150,12 +149,16 @@ public:
   NS_IMETHOD AllocateStackMemory(size_t aSize, void** aResult) = 0;
   
   NS_IMETHOD GetDocument(nsIDocument** aResult) = 0;
+  nsIDocument* GetDocument() { return mDocument; }
 
   NS_IMETHOD GetPresContext(nsIPresContext** aResult) = 0;
+  nsIPresContext* GetPresContext() { return mPresContext; }
 
   NS_IMETHOD GetViewManager(nsIViewManager** aResult) = 0;
+  nsIViewManager* GetViewManager() { return mViewManager; }
 
   NS_IMETHOD GetStyleSet(nsIStyleSet** aResult) = 0;
+  nsIStyleSet* GetStyleSet() { return mStyleSet; }
 
   NS_IMETHOD GetActiveAlternateStyleSheet(nsString& aSheetTitle) = 0;
 
@@ -166,7 +169,7 @@ public:
    * all style data for a given pres shell without necessarily reconstructing
    * all of the frames.
    */
-  NS_IMETHOD ReconstructStyleData(PRBool aRebuildRuleTree) = 0;
+  NS_IMETHOD ReconstructStyleData() = 0;
 
   /** Setup all style rules required to implement preferences
    * - used for background/text/link colors and link underlining
@@ -340,8 +343,7 @@ public:
    * The content object associated with aFrame should either be a IMG
    * element, an OBJECT element, or an APPLET element
    */
-  NS_IMETHOD CantRenderReplacedElement(nsIPresContext* aPresContext,
-                                       nsIFrame*       aFrame) = 0;
+  NS_IMETHOD CantRenderReplacedElement(nsIFrame* aFrame) = 0;
 
 
   /**
@@ -485,8 +487,6 @@ public:
    */
 
   NS_IMETHOD CaptureHistoryState(nsILayoutHistoryState** aLayoutHistoryState, PRBool aLeavingPage = PR_FALSE) = 0;
-  NS_IMETHOD GetHistoryState(nsILayoutHistoryState** aLayoutHistoryState) = 0;
-  NS_IMETHOD SetHistoryState(nsILayoutHistoryState* aLayoutHistoryState) = 0;
 
   /**
    * Determine if reflow is currently locked
@@ -611,6 +611,17 @@ public:
   NS_IMETHOD BidiStyleChangeReflow(void) = 0;
 #endif
 
+protected:
+  // IMPORTANT: The ownership implicit in the following member variables
+  // has been explicitly checked.  If you add any members to this class,
+  // please make the ownership explicit (pinkerton, scc).
+
+  // these are the same Document and PresContext owned by the DocViewer.
+  // we must share ownership.
+  nsIDocument*              mDocument;      // [STRONG]
+  nsIPresContext*           mPresContext;   // [STRONG]
+  nsIStyleSet*              mStyleSet;      // [STRONG]
+  nsIViewManager*           mViewManager;   // [WEAK] docViewer owns it so I don't have to
 };
 
 /**
