@@ -48,7 +48,7 @@
 #include "nsIRenderingContext.h"
 #include "nsIURL.h"
 #include "nsIDocument.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsLayoutAtoms.h"
 #include "nsILookAndFeel.h"
 #include "nsWidgetsCID.h"
@@ -64,19 +64,19 @@
 #include "nsNetUtil.h"
 #include "nsXPIDLString.h"
 #include "nsIWeakReferenceUtils.h"
-
+#include "nsCSSRendering.h"
 #include "prprf.h"
 #include "nsContentPolicyUtils.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIDOMDocument.h"
+#include "nsAutoPtr.h"
 #ifdef IBMBIDI
 #include "nsBidiPresUtils.h"
 #endif // IBMBIDI
 
 // Needed for Start/Stop of Image Animation
 #include "imgIContainer.h"
-#include "nsIDOMHTMLImageElement.h"
-#include "nsIImageFrame.h"
+#include "nsIImageLoadingContent.h"
 
 //needed for resetting of image service color
 #include "nsLayoutCID.h"
@@ -109,7 +109,7 @@ nsPresContext::PrefChangedCallback(const char* aPrefName, void* instance_data)
 
 #ifdef IBMBIDI
 PRBool
-IsVisualCharset(const nsAutoString& aCharset)
+IsVisualCharset(const nsCAutoString& aCharset)
 {
   if (aCharset.EqualsIgnoreCase("ibm864")             // Arabic//ahmed
       || aCharset.EqualsIgnoreCase("ibm862")          // Hebrew
@@ -154,9 +154,7 @@ nsPresContext::nsPresContext()
       NS_FONT_WEIGHT_NORMAL, 0, NSIntPointsToTwips(12)),
     mNoTheme(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
   mCompatibilityMode = eCompatibility_FullStandards;
-  mWidgetRenderingMode = eWidgetRendering_Gfx; 
   mImageAnimationMode = imgIContainer::kNormalAnimMode;
   mImageAnimationModePref = imgIContainer::kNormalAnimMode;
 
@@ -210,18 +208,13 @@ nsPresContext::~nsPresContext()
 {
   mImageLoaders.Enumerate(destroy_loads);
 
-  if (mShell) {
-    nsCOMPtr<nsIDocument> doc;
-    mShell->GetDocument(getter_AddRefs(doc));
-    if (doc) {
-      doc->RemoveCharSetObserver(this);
-    }
-  }
+  NS_PRECONDITION(!mShell, "Presshell forgot to clear our mShell pointer");
+  SetShell(nsnull);
 
-  mShell = nsnull;
-
-  if (mEventManager)
+  if (mEventManager) {
     mEventManager->SetPresContext(nsnull);   // unclear if this is needed, but can't hurt
+    NS_RELEASE(mEventManager);
+  }
 
   // Unregister preference callbacks
   if (mPrefs) {
@@ -241,6 +234,8 @@ nsPresContext::~nsPresContext()
     delete mBidiUtils;
   }
 #endif // IBMBIDI
+
+  NS_IF_RELEASE(mDeviceContext);
 }
 
 NS_IMPL_ISUPPORTS2(nsPresContext, nsIPresContext, nsIObserver)
@@ -314,7 +309,10 @@ nsPresContext::GetFontPreferences()
 
   // get font.minimum-size.[langGroup]
   PRInt32 size;
-  pref.Assign("font.minimum-size."); pref.Append(NS_ConvertUCS2toUTF8(langGroup));
+
+  pref.Assign("font.minimum-size.");
+  AppendUTF16toUTF8(langGroup, pref);
+
   rv = mPrefs->GetIntPref(pref.get(), &size);
   if (NS_SUCCEEDED(rv)) {
     if (unit == eUnit_px) {
@@ -329,7 +327,7 @@ nsPresContext::GetFontPreferences()
   nsCAutoString generic_dot_langGroup;
   for (PRInt32 eType = eDefaultFont_Variable; eType < eDefaultFont_COUNT; ++eType) {
     generic_dot_langGroup.Assign(kGenericFont[eType]);
-    generic_dot_langGroup.Append(NS_ConvertUCS2toUTF8(langGroup));
+    AppendUTF16toUTF8(langGroup, generic_dot_langGroup);
 
     nsFont* font;
     switch (eType) {
@@ -400,9 +398,10 @@ nsPresContext::GetFontPreferences()
     }
 
 #ifdef DEBUG_rbs
-    nsCAutoString family(NS_ConvertUCS2toUTF8(font->name));
     printf("%s Family-list:%s size:%d sizeAdjust:%.2f\n",
-            generic_dot_langGroup.get(), family.get(), font->size, font->sizeAdjust);
+            generic_dot_langGroup.get(),
+           NS_ConvertUCS2toUTF8(font->name).get(), font->size,
+           font->sizeAdjust);
 #endif
   }
 }
@@ -459,10 +458,6 @@ nsPresContext::GetUserPreferences()
     mFontScaler = prefInt;
   }
 
-  if (NS_SUCCEEDED(mPrefs->GetIntPref("nglayout.widget.mode", &prefInt))) {
-    mWidgetRenderingMode = (enum nsWidgetRendering)prefInt;  // bad cast
-  }
-
   // * document colors
   GetDocumentColorPreferences();
 
@@ -589,7 +584,7 @@ nsPresContext::ClearStyleDataAndReflow()
     // Clear out all our style data.
     nsCOMPtr<nsIStyleSet> set;
     mShell->GetStyleSet(getter_AddRefs(set));
-    set->ClearStyleData(this, nsnull, nsnull);
+    set->ClearStyleData(this);
 
     // Force a reflow of the root frame
     // XXX We really should only do a reflow if a preference that affects
@@ -631,7 +626,8 @@ nsPresContext::Init(nsIDeviceContext* aDeviceContext)
 {
   NS_ASSERTION(!(mInitialized == PR_TRUE), "attempt to reinit pres context");
 
-  mDeviceContext = dont_QueryInterface(aDeviceContext);
+  mDeviceContext = aDeviceContext;
+  NS_IF_ADDREF(mDeviceContext);
 
   mLangService = do_GetService(NS_LANGUAGEATOMSERVICE_CONTRACTID);
   mPrefs = do_GetService(NS_PREF_CONTRACTID);
@@ -664,13 +660,24 @@ nsPresContext::Init(nsIDeviceContext* aDeviceContext)
 NS_IMETHODIMP
 nsPresContext::SetShell(nsIPresShell* aShell)
 {
+  if (mShell) {
+    // Remove ourselves as the charset observer from the shell's doc, because
+    // this shell may be going away for good.
+    nsCOMPtr<nsIDocument> doc;
+    mShell->GetDocument(getter_AddRefs(doc));
+    if (doc) {
+      doc->RemoveCharSetObserver(this);
+    }
+  }    
+
   mShell = aShell;
-  if (nsnull != mShell) {
+
+  if (mShell) {
     nsCOMPtr<nsIDocument> doc;
     if (NS_SUCCEEDED(mShell->GetDocument(getter_AddRefs(doc)))) {
       NS_ASSERTION(doc, "expect document here");
       if (doc) {
-        doc->GetBaseURL(*getter_AddRefs(mBaseURL));
+        doc->GetBaseURL(getter_AddRefs(mBaseURL));
 
         if (mBaseURL) {
             PRBool isChrome = PR_FALSE;
@@ -685,7 +692,7 @@ nsPresContext::SetShell(nsIPresShell* aShell)
         }
 
         if (mLangService) {
-          nsAutoString charset;
+          nsCAutoString charset;
           doc->AddCharSetObserver(this);
           doc->GetDocumentCharacterSet(charset);
           UpdateCharSet(charset.get());
@@ -706,10 +713,11 @@ nsPresContext::GetShell(nsIPresShell** aResult)
 }
 
 void
-nsPresContext::UpdateCharSet(const PRUnichar* aCharSet)
+nsPresContext::UpdateCharSet(const char* aCharSet)
 {
   if (mLangService) {
-    mLangService->LookupCharSet(aCharSet, getter_AddRefs(mLanguage));
+    mLangService->LookupCharSet(aCharSet,
+                                getter_AddRefs(mLanguage));
     GetFontPreferences();
     if (mLanguage) {
       nsCOMPtr<nsIAtom> langGroupAtom;
@@ -743,7 +751,7 @@ nsPresContext::Observe(nsISupports* aSubject,
                        const PRUnichar* aData)
 {
   if (!nsCRT::strcmp(aTopic, "charset")) {
-    UpdateCharSet(aData);
+    UpdateCharSet(NS_LossyConvertUCS2toASCII(aData).get());
     if (mDeviceContext) {
       mDeviceContext->FlushFontCache();
       ClearStyleDataAndReflow();
@@ -780,23 +788,6 @@ nsPresContext::SetCompatibilityMode(nsCompatibility aMode)
   return NS_OK;
 }
 
-
-NS_IMETHODIMP
-nsPresContext::GetWidgetRenderingMode(nsWidgetRendering* aResult)
-{
-  NS_PRECONDITION(aResult, "null out param");
-  *aResult = mWidgetRenderingMode;
-  return NS_OK;
-}
-
- 
-NS_IMETHODIMP
-nsPresContext::SetWidgetRenderingMode(nsWidgetRendering aMode)
-{
-  mWidgetRenderingMode = aMode;
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsPresContext::GetImageAnimationMode(PRUint16* aModeResult)
 {
@@ -808,7 +799,7 @@ nsPresContext::GetImageAnimationMode(PRUint16* aModeResult)
 // Helper function for setting Anim Mode on image
 static void SetImgAnimModeOnImgReq(imgIRequest* aImgReq, PRUint16 aMode)
 {
-  if (aImgReq != nsnull) {
+  if (aImgReq) {
     nsCOMPtr<imgIContainer> imgCon;
     aImgReq->GetImage(getter_AddRefs(imgCon));
     if (imgCon) {
@@ -835,25 +826,19 @@ PR_STATIC_CALLBACK(PRBool) set_animation_mode(nsHashKey *aKey, void *aData, void
 // this is a way to turn on/off image animations
 void nsPresContext::SetImgAnimations(nsCOMPtr<nsIContent>& aParent, PRUint16 aMode)
 {
-  nsCOMPtr<nsIDOMHTMLImageElement> imgContent(do_QueryInterface(aParent));
+  nsCOMPtr<nsIImageLoadingContent> imgContent(do_QueryInterface(aParent));
   if (imgContent) {
-    nsIFrame* frame;
-    mShell->GetPrimaryFrameFor(aParent, &frame);
-    if (frame != nsnull) {
-      nsIImageFrame* imgFrame = nsnull;
-      CallQueryInterface(frame, &imgFrame);
-      if (imgFrame != nsnull) {
-        nsCOMPtr<imgIRequest> imgReq;
-        imgFrame->GetImageRequest(getter_AddRefs(imgReq));
-        SetImgAnimModeOnImgReq(imgReq, aMode);
-      }
-    }
+    nsCOMPtr<imgIRequest> imgReq;
+    imgContent->GetRequest(nsIImageLoadingContent::CURRENT_REQUEST,
+                           getter_AddRefs(imgReq));
+    SetImgAnimModeOnImgReq(imgReq, aMode);
   }
+  
   PRInt32 count;
   aParent->ChildCount(count);
   for (PRInt32 i=0;i<count;i++) {
     nsCOMPtr<nsIContent> child;
-    aParent->ChildAt(i, *getter_AddRefs(child));
+    aParent->ChildAt(i, getter_AddRefs(child));
     if (child) {
       SetImgAnimations(child, aMode);
     }
@@ -935,106 +920,89 @@ nsPresContext::GetBaseURL(nsIURI** aResult)
   return NS_OK;
 }
 
-NS_IMETHODIMP
+already_AddRefed<nsStyleContext>
 nsPresContext::ResolveStyleContextFor(nsIContent* aContent,
-                                      nsIStyleContext* aParentContext,
-                                      nsIStyleContext** aResult)
+                                      nsStyleContext* aParentContext)
 {
-  NS_PRECONDITION(aResult, "null out param");
-
-  nsIStyleContext* result = nsnull;
-  nsCOMPtr<nsIStyleSet> set;
-  nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
-  if (NS_SUCCEEDED(rv)) {
-    if (set) {
-      result = set->ResolveStyleFor(this, aContent, aParentContext);
-      if (nsnull == result) {
-        rv = NS_ERROR_OUT_OF_MEMORY;
-      }
-    }
-  }
-  *aResult = result;
-  return rv;
-}
-
-NS_IMETHODIMP
-nsPresContext::ResolveStyleContextForNonElement(
-                                      nsIStyleContext* aParentContext,
-                                      nsIStyleContext** aResult)
-{
-  NS_PRECONDITION(aResult, "null out param");
-
-  nsIStyleContext* result = nsnull;
   nsCOMPtr<nsIStyleSet> set;
   nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
   if (NS_SUCCEEDED(rv) && set) {
-    result = set->ResolveStyleForNonElement(this, aParentContext);
-    if (!result)
-      rv = NS_ERROR_OUT_OF_MEMORY;
+    // return the addref'd style context
+    return set->ResolveStyleFor(this, aContent, aParentContext);
   }
-  *aResult = result;
-  return rv;
+
+  return nsnull;
 }
 
-NS_IMETHODIMP
+already_AddRefed<nsStyleContext>
+nsPresContext::ResolveStyleContextForNonElement(nsStyleContext* aParentContext)
+{
+  nsCOMPtr<nsIStyleSet> set;
+  nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
+  if (NS_SUCCEEDED(rv) && set) {
+    // return addrefed style context
+    return set->ResolveStyleForNonElement(this, aParentContext);
+  }
+
+  return nsnull;
+}
+
+already_AddRefed<nsStyleContext>
 nsPresContext::ResolvePseudoStyleContextFor(nsIContent* aParentContent,
                                             nsIAtom* aPseudoTag,
-                                            nsIStyleContext* aParentContext,
-                                            nsIStyleContext** aResult)
+                                            nsStyleContext* aParentContext)
 {
-  return ResolvePseudoStyleWithComparator(aParentContent, aPseudoTag, aParentContext,
-                                          nsnull, aResult);
+  return ResolvePseudoStyleWithComparator(aParentContent, aPseudoTag,
+                                          aParentContext, nsnull);
 }
 
-NS_IMETHODIMP
+already_AddRefed<nsStyleContext>
 nsPresContext::ResolvePseudoStyleWithComparator(nsIContent* aParentContent,
                                                 nsIAtom* aPseudoTag,
-                                                nsIStyleContext* aParentContext,
-                                                nsICSSPseudoComparator* aComparator,
-                                                nsIStyleContext** aResult)
+                                                nsStyleContext* aParentContext,
+                                                nsICSSPseudoComparator* aComparator)
 {
-  NS_PRECONDITION(aResult, "null out param");
-
-  nsIStyleContext* result = nsnull;
   nsCOMPtr<nsIStyleSet> set;
   nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
-  if (NS_SUCCEEDED(rv)) {
-    if (set) {
-      result = set->ResolvePseudoStyleFor(this, aParentContent, aPseudoTag,
-                                          aParentContext, aComparator);
-      if (nsnull == result) {
-        rv = NS_ERROR_OUT_OF_MEMORY;
-      }
-    }
+  if (NS_SUCCEEDED(rv) && set) {
+    // return addrefed style context
+    return set->ResolvePseudoStyleFor(this, aParentContent, aPseudoTag,
+                                      aParentContext, aComparator);
   }
-  *aResult = result;
-  return rv;
+
+  return nsnull;
+}
+
+already_AddRefed<nsStyleContext>
+nsPresContext::ProbePseudoStyleContextFor(nsIContent* aParentContent,
+                                          nsIAtom* aPseudoTag,
+                                          nsStyleContext* aParentContext)
+{
+  nsCOMPtr<nsIStyleSet> set;
+  nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
+  if (NS_SUCCEEDED(rv) && set) {
+    // return addrefed style context
+    return set->ProbePseudoStyleFor(this, aParentContent, aPseudoTag,
+                                    aParentContext);
+  }
+
+  return nsnull;
 }
 
 NS_IMETHODIMP
-nsPresContext::ProbePseudoStyleContextFor(nsIContent* aParentContent,
-                                          nsIAtom* aPseudoTag,
-                                          nsIStyleContext* aParentContext,
-                                          nsIStyleContext** aResult)
+nsPresContext::GetXBLBindingURL(nsIContent* aContent, nsAString& aResult)
 {
-  NS_PRECONDITION(aResult, "null out param");
+  nsRefPtr<nsStyleContext> sc;
+  sc = ResolveStyleContextFor(aContent, nsnull);
+  NS_ENSURE_TRUE(sc, NS_ERROR_FAILURE);
 
-  nsIStyleContext* result = nsnull;
-  nsCOMPtr<nsIStyleSet> set;
-  nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
-  if (NS_SUCCEEDED(rv)) {
-    if (set) {
-      result = set->ProbePseudoStyleFor(this, aParentContent, aPseudoTag,
-                                        aParentContext);
-    }
-  }
-  *aResult = result;
-  return rv;
+  aResult = sc->GetStyleDisplay()->mBinding;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
 nsPresContext::ReParentStyleContext(nsIFrame* aFrame, 
-                                    nsIStyleContext* aNewParentContext)
+                                    nsStyleContext* aNewParentContext)
 {
   NS_PRECONDITION(aFrame, "null ptr");
   if (! aFrame) {
@@ -1044,7 +1012,7 @@ nsPresContext::ReParentStyleContext(nsIFrame* aFrame,
   nsCOMPtr<nsIFrameManager> manager;
   nsresult rv = mShell->GetFrameManager(getter_AddRefs(manager));
   if (NS_SUCCEEDED(rv) && manager) {
-    rv = manager->ReParentStyleContext(this, aFrame, aNewParentContext);
+    rv = manager->ReParentStyleContext(aFrame, aNewParentContext);
   }
   return rv;
 }
@@ -1410,14 +1378,15 @@ nsPresContext::LoadImage(const nsString& aURL,
   if (NS_FAILED(rv)) return rv;
 
   nsCOMPtr<nsIURI> baseURI;
-  doc->GetBaseURL(*getter_AddRefs(baseURI));
+  doc->GetBaseURL(getter_AddRefs(baseURI));
 
   nsCOMPtr<nsIURI> uri;
-  NS_NewURI(getter_AddRefs(uri), aURL, nsnull, baseURI);
+  nsCOMPtr<nsIIOService> ioService;
+  GetIOService(getter_AddRefs(ioService));
+  NS_NewURI(getter_AddRefs(uri), aURL, nsnull, baseURI, ioService);
 
   if (!loader) {
-    nsCOMPtr<nsIContent> content;
-    aTargetFrame->GetContent(getter_AddRefs(content));
+    nsIContent* content = aTargetFrame->GetContent();
 
     // Check with the content-policy things to make sure this load is permitted.
     nsresult rv;
@@ -1425,7 +1394,7 @@ nsPresContext::LoadImage(const nsString& aURL,
 
     if (content && element) {
       nsCOMPtr<nsIDocument> document;
-      rv = content->GetDocument(*getter_AddRefs(document));
+      rv = content->GetDocument(getter_AddRefs(document));
 
       // If there is no document, skip the policy check
       // XXXldb This really means the document is being destroyed, so
@@ -1465,10 +1434,7 @@ nsPresContext::LoadImage(const nsString& aURL,
   // Allow for a null target frame argument (for precached images)
   if (aTargetFrame) {
     // Mark frame as having loaded an image
-    nsFrameState state;
-    aTargetFrame->GetFrameState(&state);
-    state |= NS_FRAME_HAS_LOADED_IMAGES;
-    aTargetFrame->SetFrameState(state);
+    aTargetFrame->AddStateBits(NS_FRAME_HAS_LOADED_IMAGES);
   }
 
   loader->Load(uri);
@@ -1536,24 +1502,30 @@ nsPresContext::GetContainer(nsISupports** aResult)
   return CallQueryReferent(mContainer.get(), aResult);
 }
 
-NS_IMETHODIMP
-nsPresContext::GetEventStateManager(nsIEventStateManager** aManager)
+nsIEventStateManager*
+nsIPresContext::GetEventStateManager()
 {
-  NS_PRECONDITION(aManager, "null ptr");
-
   if (!mEventManager) {
-    nsresult rv;
-    mEventManager = do_CreateInstance(kEventStateManagerCID,&rv);
+    nsresult rv = CallCreateInstance(kEventStateManagerCID, &mEventManager);
     if (NS_FAILED(rv)) {
-      return rv;
+      return nsnull;
     }
 
     //Not refcnted, set null in destructor
     mEventManager->SetPresContext(this);
   }
+  return mEventManager;
+}
 
-  *aManager = mEventManager;
-  NS_IF_ADDREF(*aManager);
+NS_IMETHODIMP
+nsPresContext::GetEventStateManager(nsIEventStateManager** aManager)
+{
+  NS_PRECONDITION(aManager, "null ptr");
+
+  *aManager = GetEventStateManager();
+  if (!*aManager)
+    return NS_ERROR_OUT_OF_MEMORY;
+  NS_ADDREF(*aManager);
   return NS_OK;
 }
 
@@ -1680,7 +1652,7 @@ nsPresContext::GetIsBidiSystem(PRBool& aResult) const
 }
 
 NS_IMETHODIMP
-nsPresContext::GetBidiCharset(nsAString &aCharSet) const
+nsPresContext::GetBidiCharset(nsACString &aCharSet) const
 {
   aCharSet = mCharset;
   return NS_OK;
@@ -1748,7 +1720,7 @@ nsPresContext::ThemeChanged()
   if (!mShell)
     return NS_OK;
 
-  return mShell->ReconstructStyleData(PR_FALSE);
+  return mShell->ReconstructStyleData();
 }
 
 NS_IMETHODIMP
@@ -1765,12 +1737,6 @@ nsPresContext::SysColorChanged()
 
   // Clear out all of the style data since it may contain RGB values
   // which originated from system colors.
-  if (mShell) {
-    // Clear out all our style data.
-    nsCOMPtr<nsIStyleSet> set;
-    mShell->GetStyleSet(getter_AddRefs(set));
-    set->ClearStyleData(this, nsnull, nsnull);
-  }
   nsCOMPtr<nsISelectionImageService> imageService;
   nsresult result;
   imageService = do_GetService(kSelectionImageService, &result);
@@ -1778,6 +1744,22 @@ nsPresContext::SysColorChanged()
   {
     imageService->Reset();
   }
+
+  // We need to do a full reflow (and view update) here. Clearing the style
+  // data without reflowing/updating views will lead to incorrect change hints
+  // later, because when generating change hints, any style structs which have
+  // been cleared and not reread are assumed to not be used at all.
+  return ClearStyleDataAndReflow();
+}
+
+NS_IMETHODIMP
+nsPresContext::FindFrameBackground(nsIFrame* aFrame,
+                                   const nsStyleBackground** aBackground,
+                                   PRBool* aIsCanvas,
+                                   PRBool* aFoundBackground)
+{
+  *aFoundBackground = nsCSSRendering::FindBackground(this, aFrame,
+                                                     aBackground, aIsCanvas);
   return NS_OK;
 }
 
