@@ -112,7 +112,7 @@
 #endif
 
 // handy utilities
-extern void SetFontFromStyle(nsIRenderingContext* aRC, nsIStyleContext* aSC);
+void SetFontFromStyle(nsIRenderingContext* aRC, nsStyleContext* aSC);
 
 //----------------------------------------------------------------------
 
@@ -140,7 +140,7 @@ public:
   void* operator new(size_t sz, nsIPresShell* aPresShell) CPP_THROW_NEW;
 
   // Overridden to prevent the global delete from being called, since the memory
-  // came out of an nsIArena instead of the global delete operator's heap.  
+  // came out of an arena instead of the global delete operator's heap.  
   // XXX Would like to make this private some day, but our UNIX compilers can't 
   // deal with it.
   void operator delete(void* aPtr, size_t sz);
@@ -158,7 +158,7 @@ public:
   NS_IMETHOD  Init(nsIPresContext*  aPresContext,
                    nsIContent*      aContent,
                    nsIFrame*        aParent,
-                   nsIStyleContext* aContext,
+                   nsStyleContext*  aContext,
                    nsIFrame*        asPrevInFlow);
   NS_IMETHOD  SetInitialChildList(nsIPresContext* aPresContext,
                                   nsIAtom*        aListName,
@@ -183,10 +183,9 @@ public:
                            nsIFrame*       aNewFrame);
   NS_IMETHOD  Destroy(nsIPresContext* aPresContext);
   NS_IMETHOD  CalcBorderPadding(nsMargin& aBorderPadding) const;
-  NS_IMETHOD  GetAdditionalStyleContext(PRInt32 aIndex, 
-                                        nsIStyleContext** aStyleContext) const;
-  NS_IMETHOD  SetAdditionalStyleContext(PRInt32 aIndex, 
-                                        nsIStyleContext* aStyleContext);
+  virtual nsStyleContext* GetAdditionalStyleContext(PRInt32 aIndex) const;
+  virtual void SetAdditionalStyleContext(PRInt32 aIndex,
+                                         nsStyleContext* aStyleContext);
   NS_IMETHOD  GetAdditionalChildListName(PRInt32 aIndex, nsIAtom** aListName) const;
   NS_IMETHOD  FirstChild(nsIPresContext* aPresContext,
                          nsIAtom*        aListName,
@@ -235,28 +234,19 @@ public:
                                nsIAtom*        aAttribute,
                                PRInt32         aModType, 
                                PRInt32         aHint);
-  NS_IMETHOD  ContentStateChanged(nsIPresContext* aPresContext,
-                                  nsIContent*     aChild,
-                                  PRInt32         aHint);
   NS_IMETHOD  IsSplittable(nsSplittableType& aIsSplittable) const;
   NS_IMETHOD  GetPrevInFlow(nsIFrame** aPrevInFlow) const;
   NS_IMETHOD  SetPrevInFlow(nsIFrame*);
   NS_IMETHOD  GetNextInFlow(nsIFrame** aNextInFlow) const;
   NS_IMETHOD  SetNextInFlow(nsIFrame*);
-  NS_IMETHOD  GetView(nsIPresContext* aPresContext, nsIView** aView) const;
-  NS_IMETHOD  SetView(nsIPresContext* aPresContext, nsIView* aView);
-  NS_IMETHOD  GetParentWithView(nsIPresContext* aPresContext, nsIFrame** aParent) const;
   NS_IMETHOD  GetOffsetFromView(nsIPresContext* aPresContext, nsPoint& aOffset, nsIView** aView) const;
   NS_IMETHOD  GetOriginToViewOffset(nsIPresContext *aPresContext, nsPoint& aOffset, nsIView **aView) const;
-  NS_IMETHOD  GetWindow(nsIPresContext* aPresContext, nsIWidget**) const;
   NS_IMETHOD  GetFrameType(nsIAtom** aType) const;
   NS_IMETHOD  IsPercentageBase(PRBool& aBase) const;
-  NS_IMETHOD  Scrolled(nsIView *aView);
 #ifdef NS_DEBUG
   NS_IMETHOD  List(nsIPresContext* aPresContext, FILE* out, PRInt32 aIndent) const;
   NS_IMETHOD  GetFrameName(nsAString& aResult) const;
   NS_IMETHOD  DumpRegressionData(nsIPresContext* aPresContext, FILE* out, PRInt32 aIndent, PRBool aIncludeStyleData);
-  NS_IMETHOD  SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const;
   NS_IMETHOD  VerifyTree() const;
 #endif
 
@@ -360,10 +350,6 @@ public:
   PRBool IsFrameTreeTooDeep(const nsHTMLReflowState& aReflowState,
                             nsHTMLReflowMetrics& aMetrics);
 
-  virtual nsresult GetClosestViewForFrame(nsIPresContext* aPresContext,
-                                          nsIFrame *aFrame,
-                                          nsIView **aView);
-
   static nsresult CreateAndPostReflowCommand(nsIPresShell*                   aPresShell,
                                              nsIFrame*                       aTargetFrame,
                                              nsReflowType aReflowType,
@@ -374,14 +360,22 @@ public:
   // Do the work for getting the parent style context frame so that
   // other frame's |GetParentStyleContextFrame| methods can call this
   // method on *another* frame.  (This function handles out-of-flow
-  // frames by using the frame manager's placeholder map and it handles
-  // block-within-inline by calling |GetIBSpecialParent|.)
+  // frames by using the frame manager's placeholder map and it also
+  // handles block-within-inline and generated content wrappers.)
   nsresult DoGetParentStyleContextFrame(nsIPresContext* aPresContext,
                                         nsIFrame**      aProviderFrame,
                                         PRBool*         aIsChild);
 
-  nsresult GetIBSpecialParent(nsIPresContext* aPresContext,
-                              nsIFrame** aSpecialParent);
+  // Return the previously stored overflow area, if the frame does not 
+  // overflow and a creation is not requested it will return nsnull
+  virtual nsRect* GetOverflowAreaProperty(nsIPresContext* aPresContext,
+                                  PRBool          aCreateIfNecessary = PR_FALSE);
+
+  // Set/unset the NS_FRAME_OUTSIDE_CHILDREN flag and store the overflow area
+  // as a frame property in the frame manager so that it can be retrieved
+  // later without reflowing the frame.
+  void StoreOverflow(nsIPresContext*      aPresContext,
+                     nsHTMLReflowMetrics& aMetrics);
 
   //Mouse Capturing code used by the frames to tell the view to capture all the following events
   NS_IMETHOD CaptureMouse(nsIPresContext* aPresContext, PRBool aGrabMouseEvents);
@@ -396,6 +390,8 @@ public:
                                void*                   aPropertyValue,
                                NSFramePropertyDtorFunc aPropDtorFunc);
 
+  virtual const nsStyleStruct* GetStyleDataExternal(nsStyleStructID aSID) const;
+
 #ifdef IBMBIDI
   NS_IMETHOD GetBidiProperty(nsIPresContext* aPresContext,
                              nsIAtom*        aPropertyName,
@@ -462,6 +458,9 @@ public:
                                  nsHTMLReflowMetrics& aMetrics,
                                  PRUint32             aStatus,
                                  void*                aFrameTreeNode);
+
+  static void DisplayReflowStartup();
+  static void DisplayReflowShutdown();
 #endif
 
 protected:
@@ -469,6 +468,17 @@ protected:
   nsFrame();
   virtual ~nsFrame();
 
+  /**
+   * To be called by |Paint| of this class or derived classes to paint
+   * the background, border, and outline, when in the correct layer to
+   * do so.
+   */
+  void PaintSelf(nsIPresContext*      aPresContext,
+                 nsIRenderingContext& aRenderingContext,
+                 const nsRect&        aDirtyRect,
+                 PRIntn               aSkipSides = 0,
+                 PRBool               aUsePrintBackgroundSettings = PR_TRUE);
+
   PRInt16 DisplaySelection(nsIPresContext* aPresContext, PRBool isOkToTurnOn = PR_FALSE);
   
   //this will modify aPos and return the next frame ect.
