@@ -119,26 +119,18 @@ HRuleFrame::Paint(nsIPresContext*      aPresContext,
     return NS_OK;
   }
 
-  const nsStyleVisibility* vis = 
-      (const nsStyleVisibility*)((nsStyleContext*)mStyleContext)->GetStyleData(eStyleStruct_Visibility);
-  if (!vis->IsVisible()) {
+  if (!GetStyleVisibility()->IsVisible()) {
     return NS_OK;
   }
 
-  nsIFrame *parent = nsnull;
-  //hack to get around lack of selection bit setting.
-  GetParent(&parent);//get the parent and check to see if its selected
+  nsIFrame *parent = GetParent();
+  // hack to get around lack of selection bit setting.
+  // get the parent and check to see if its selected
   PRBool isSelected = PR_FALSE;
   if (parent)
   {
-    nsCOMPtr<nsIContent> content;
-    parent->GetContent(getter_AddRefs(content));
-    nsFrameState  frameState;
-    if (content.get() == mContent)
-      parent->GetFrameState(&frameState);
-    else
-      GetFrameState(&frameState);
-    isSelected = (frameState & NS_FRAME_SELECTED_CONTENT) == NS_FRAME_SELECTED_CONTENT;
+    isSelected = (((parent->GetContent() == mContent) ? parent : this)->GetStateBits()
+       & NS_FRAME_SELECTED_CONTENT) == NS_FRAME_SELECTED_CONTENT;
   }
   if (isSelected) //check the display flags to see if we draw this frame selected.
   {
@@ -201,8 +193,8 @@ HRuleFrame::Paint(nsIPresContext*      aPresContext,
   if (!noShadeAttribute) {
     nsRect rect(x0, y0, width, height);
 
-    const nsStyleBorder* border = (const nsStyleBorder*) mStyleContext->GetStyleData(eStyleStruct_Border);
-    const nsStylePadding* padding = (const nsStylePadding*) mStyleContext->GetStyleData(eStyleStruct_Padding);
+    const nsStyleBorder* border = GetStyleBorder();
+    const nsStylePadding* padding = GetStylePadding();
     
     nsCSSRendering::PaintBackground(aPresContext, aRenderingContext,
                                     this,aDirtyRect, rect,
@@ -302,8 +294,7 @@ HRuleFrame::Reflow(nsIPresContext*          aPresContext,
   // Compute height of "line" that hrule will layout within. Use the
   // font-size to do this.
   nscoord minLineHeight = thickness + twoPixels;
-  const nsStyleFont* font;
-  GetStyleData(eStyleStruct_Font, (const nsStyleStruct*&) font);
+  const nsStyleFont* font = GetStyleFont();
   const nsFont& f = font->mFont;
   nsCOMPtr<nsIFontMetrics> fm;
   aPresContext->GetMetricsFor(f, getter_AddRefs(fm));
@@ -382,19 +373,14 @@ HRuleFrame::GetContentAndOffsetsFromPoint(nsIPresContext* aPresContext,
   if (!aNewContent) return NS_ERROR_NULL_POINTER;
   if (!mContent) return NS_ERROR_NULL_POINTER;
 
-  nsIView  *view         = nsnull;
-  rv = GetClosestViewForFrame(aPresContext, this, &view);
-  if (NS_FAILED(rv)) return rv;
-
-  nsRect thisRect;
-  rv = GetRect(thisRect);
-  if (NS_FAILED(rv)) return rv;
+  nsRect thisRect = GetRect();
   nsPoint offsetPoint;
+  nsIView *view;
   GetOffsetFromView(aPresContext, offsetPoint, &view);
   thisRect.x = offsetPoint.x;
   thisRect.y = offsetPoint.y;
 
-  rv = mContent->GetParent(*aNewContent);
+  rv = mContent->GetParent(aNewContent);
   if (!*aNewContent) return rv;
   
   rv = (*aNewContent)->IndexOf(mContent, aContentOffset);
