@@ -39,7 +39,6 @@
 #include "nsSimplePageSequence.h"
 #include "nsIPresContext.h"
 #include "nsIRenderingContext.h"
-#include "nsIStyleContext.h"
 #include "nsHTMLAtoms.h"
 #include "nsIDeviceContext.h"
 #include "nsIViewManager.h"
@@ -54,7 +53,9 @@
 #include "nsRegion.h"
 #include "nsLayoutAtoms.h"
 
-#include "nsIPref.h" // for header/footer gap & ExtraMargin for Print Preview
+// for header/footer gap & ExtraMargin for Print Preview
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
 
 // DateTime Includes
 #include "nsDateTimeFormatCID.h"
@@ -216,23 +217,30 @@ nsSimplePageSequenceFrame::CreateContinuingPageFrame(nsIPresContext* aPresContex
   return rv;
 }
 
-void nsSimplePageSequenceFrame::GetEdgePaperMarginCoord(nsIPref* aPref, char* aPrefName, nscoord& aCoord)
+void
+nsSimplePageSequenceFrame::GetEdgePaperMarginCoord(char* aPrefName,
+                                                   nscoord& aCoord)
 {
-  if (NS_SUCCEEDED(mPageData->mPrintOptions->GetPrinterPrefInt(mPageData->mPrintSettings, 
-                                                               NS_ConvertASCIItoUCS2(aPrefName).get(), &aCoord))) {
+  nsresult rv = mPageData->mPrintOptions->
+    GetPrinterPrefInt(mPageData->mPrintSettings, 
+                      NS_ConvertASCIItoUCS2(aPrefName).get(),
+                      &aCoord);
+
+  if (NS_SUCCEEDED(rv)) {
     nscoord inchInTwips = NS_INCHES_TO_TWIPS(1.0);
     aCoord = PR_MAX(NS_INCHES_TO_TWIPS(float(aCoord)/100.0f), 0);
     aCoord = PR_MIN(aCoord, inchInTwips); // an inch is still probably excessive
   }
 }
 
-void nsSimplePageSequenceFrame::GetEdgePaperMargin(nsIPref* aPref, nsMargin& aMargin)
+void
+nsSimplePageSequenceFrame::GetEdgePaperMargin(nsMargin& aMargin)
 {
   aMargin.SizeTo(0,0,0,0);
-  GetEdgePaperMarginCoord(aPref, "print_edge_top",    aMargin.top);
-  GetEdgePaperMarginCoord(aPref, "print_edge_left",   aMargin.left);
-  GetEdgePaperMarginCoord(aPref, "print_edge_bottom", aMargin.bottom);
-  GetEdgePaperMarginCoord(aPref, "print_edge_right",  aMargin.right);
+  GetEdgePaperMarginCoord("print_edge_top",    aMargin.top);
+  GetEdgePaperMarginCoord("print_edge_left",   aMargin.left);
+  GetEdgePaperMarginCoord("print_edge_bottom", aMargin.bottom);
+  GetEdgePaperMarginCoord("print_edge_right",  aMargin.right);
 }
 
 NS_IMETHODIMP
@@ -309,12 +317,12 @@ nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
   aPresContext->GetPageDim(&pageSize, &adjSize);
 
   nscoord extraGap = 0;
-  nsCOMPtr<nsIPref> pref = do_GetService(NS_PREF_CONTRACTID);
-  if (pref) {
-    GetEdgePaperMargin(pref, mPageData->mEdgePaperMargin);
+  nsCOMPtr<nsIPrefBranch> prefBranch = do_GetService(NS_PREFSERVICE_CONTRACTID);
+  if (prefBranch) {
+    GetEdgePaperMargin(mPageData->mEdgePaperMargin);
     nscoord extraThreshold = PR_MAX(pageSize.width, pageSize.height)/10;
     PRInt32 gapInTwips;
-    if (NS_SUCCEEDED(pref->GetIntPref("print.print_extra_margin", &gapInTwips))) {
+    if (NS_SUCCEEDED(prefBranch->GetIntPref("print.print_extra_margin", &gapInTwips))) {
       gapInTwips = PR_MAX(gapInTwips, 0);
       gapInTwips = PR_MIN(gapInTwips, extraThreshold); // clamp to 1/10 of the largest dim of the page
       extraGap   = nscoord(gapInTwips);
@@ -473,19 +481,19 @@ nsSimplePageSequenceFrame::Reflow(nsIPresContext*          aPresContext,
       }
 
       // Get the next page
-      kidFrame->GetNextSibling(&kidFrame);
+      kidFrame = kidFrame->GetNextSibling();
     }
 
     // Get Total Page Count
     nsIFrame* page;
     PRInt32 pageTot = 0;
-    for (page = mFrames.FirstChild(); nsnull != page; page->GetNextSibling(&page)) {
+    for (page = mFrames.FirstChild(); page; page = page->GetNextSibling()) {
       pageTot++;
     }
 
     // Set Page Number Info
     PRInt32 pageNum = 1;
-    for (page = mFrames.FirstChild(); nsnull != page; page->GetNextSibling(&page)) {
+    for (page = mFrames.FirstChild(); page; page = page->GetNextSibling()) {
       nsPageFrame * pf = NS_STATIC_CAST(nsPageFrame*, page);
       if (pf != nsnull) {
         pf->SetPageNumInfo(pageNum, pageTot);
@@ -553,23 +561,6 @@ nsSimplePageSequenceFrame::GetFrameName(nsAString& aResult) const
 
 //----------------------------------------------------------------------
 
-// Helper function that sends the progress notification. Returns PR_TRUE
-// if printing should continue and PR_FALSE otherwise
-static PRBool
-SendStatusNotification(nsIPrintStatusCallback* aStatusCallback,
-                       PRInt32                 aPageNumber,
-                       PRInt32                 aTotalPages,
-                       nsPrintStatus           aStatus)
-{
-  PRBool  ret = PR_TRUE;
-
-  if (nsnull != aStatusCallback) {
-    aStatusCallback->OnProgress(aPageNumber, aTotalPages, aStatus,ret);
-  }
-
-  return ret;
-}
-
 NS_IMETHODIMP
 nsSimplePageSequenceFrame::SetOffsets(nscoord aStartOffset, nscoord aEndOffset)
 {
@@ -683,15 +674,12 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*   aPresContext,
 
 #if defined(DEBUG_rods) || defined(DEBUG_dcone)
   {
-    nsIView * seqView;
-    GetView(aPresContext, &seqView);
-    nsRect rect;
-    GetRect(rect);
+    nsIView* seqView = GetView();
+    nsRect rect = GetRect();
     PR_PL(("Seq Frame: %p - [%5d,%5d,%5d,%5d] ", this, rect.x, rect.y, rect.width, rect.height));
     PR_PL(("view: %p ", seqView));
-    nsRect viewRect;
     if (seqView) {
-      seqView->GetBounds(viewRect);
+      nsRect viewRect = seqView->GetBounds();
       PR_PL((" [%5d,%5d,%5d,%5d]", viewRect.x, viewRect.y, viewRect.width, viewRect.height));
     }
     PR_PL(("\n"));
@@ -699,14 +687,12 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*   aPresContext,
 
   {
     PRInt32 pageNum = 1;
-    for (nsIFrame* page = mFrames.FirstChild(); nsnull != page; page->GetNextSibling(&page)) {
-      nsIView*  view;
-      page->GetView(aPresContext, &view);
-      NS_ASSERTION(nsnull != view, "no page view");
-      nsRect rect;
-      page->GetRect(rect);
-      nsRect viewRect;
-      view->GetBounds(viewRect);
+    for (nsIFrame* page = mFrames.FirstChild(); page;
+         page = page->GetNextSibling()) {
+      nsIView* view = page->GetView();
+      NS_ASSERTION(view, "no page view");
+      nsRect rect = page->GetRect();
+      nsRect viewRect = view->GetBounds();
       PR_PL((" Page: %p  No: %d - [%5d,%5d,%5d,%5d] ", page, pageNum, rect.x, rect.y, rect.width, rect.height));
       PR_PL((" [%5d,%5d,%5d,%5d]\n", viewRect.x, viewRect.y, viewRect.width, viewRect.height));
       pageNum++;
@@ -728,14 +714,13 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*   aPresContext,
     PRInt32 pageNum = 1;
     nscoord y = 0;//mMargin.top;
 
-    for (nsIFrame* page = mFrames.FirstChild(); nsnull != page; page->GetNextSibling(&page)) {
-      nsIView*  view = nsnull;
-      page->GetView(aPresContext, &view);
-      NS_ASSERTION(nsnull != view, "no page view");
+    for (nsIFrame* page = mFrames.FirstChild(); page;
+         page = page->GetNextSibling()) {
+      nsIView* view = page->GetView();
+      NS_ASSERTION(view, "no page view");
 
-      nsCOMPtr<nsIViewManager> vm;
-      view->GetViewManager(*getter_AddRefs(vm));
-      NS_ASSERTION(nsnull != vm, "no view manager");
+      nsIViewManager* vm = view->GetViewManager();
+      NS_ASSERTION(vm, "no view manager");
 
       if (pageNum < mFromPageNum || pageNum > mToPageNum) {
         // Hide the pages that won't be printed to the Viewmanager
@@ -746,14 +731,12 @@ nsSimplePageSequenceFrame::StartPrint(nsIPresContext*   aPresContext,
         nsRegion emptyRegion;
         vm->SetViewChildClipRegion(view, &emptyRegion);
       } else {
-        nsRect rect;
-        page->GetRect(rect);
+        nsRect rect = page->GetRect();
         rect.y = y;
         rect.height = height;
-        page->SetRect(aPresContext, rect);
+        page->SetRect(rect);
 
-        nsRect viewRect;
-        view->GetBounds(viewRect);
+        nsRect viewRect = view->GetBounds();
         viewRect.y = y;
         viewRect.height = height;
         vm->MoveViewTo(view, viewRect.x, viewRect.y);
@@ -841,12 +824,7 @@ nsSimplePageSequenceFrame::PrintNextPage(nsIPresContext*  aPresContext)
   aPresContext->GetDeviceContext(getter_AddRefs(dc));
   NS_ASSERTION(dc, "nsIDeviceContext can't be NULL!");
 
-  nsCOMPtr<nsIPresShell> presShell;
-  aPresContext->GetShell(getter_AddRefs(presShell));
-  NS_ASSERTION(presShell, "nsIPresShell can't be NULL!");
-
-  nsCOMPtr<nsIViewManager> vm;
-  presShell->GetViewManager(getter_AddRefs(vm));
+  nsIViewManager* vm = aPresContext->GetViewManager();
   NS_ASSERTION(vm, "nsIViewManager can't be NULL!");
 
   nsresult rv = NS_OK;
@@ -885,7 +863,6 @@ nsSimplePageSequenceFrame::PrintNextPage(nsIPresContext*  aPresContext)
     // currently this does not work for IFrames
     // I will soon improve this to work with IFrames 
     PRBool  continuePrinting = PR_TRUE;
-    nscoord selectionHeight  = mSelectionHeight;
     PRInt32 width, height;
     dc->GetDeviceSurfaceDimensions(width, height);
     nsRect clipRect(0, 0, width, height);
@@ -899,9 +876,9 @@ nsSimplePageSequenceFrame::PrintNextPage(nsIPresContext*  aPresContext)
       nsIFrame* childFrame = mFrames.FirstChild();
       nsIFrame* conFrame;
       childFrame->FirstChild(aPresContext, nsnull, &conFrame);
-      conFrame->GetView(aPresContext, &containerView);
-      NS_ASSERTION(containerView != nsnull, "Container view can't be null!");
-      containerView->GetBounds(containerRect);
+      containerView = conFrame->GetView();
+      NS_ASSERTION(containerView, "Container view can't be null!");
+      containerRect = containerView->GetBounds();
       containerRect.y -= mYSelOffset;
       slidingRect.SetRect(0,mYSelOffset,width,height);
       
@@ -932,10 +909,9 @@ nsSimplePageSequenceFrame::PrintNextPage(nsIPresContext*  aPresContext)
       }
 
       // Print the page
-      nsIView*  view;
-      mCurrentPageFrame->GetView(aPresContext, &view);
+      nsIView* view = mCurrentPageFrame->GetView();
 
-      NS_ASSERTION(nsnull != view, "no page view");
+      NS_ASSERTION(view, "no page view");
 
       PR_PL(("SeqFr::Paint -> %p PageNo: %d  View: %p", pf, mPageNum, view));
       PR_PL((" At: %d,%d\n", mMargin.left+mOffsetX, mMargin.top+mOffsetY));
@@ -987,7 +963,7 @@ nsSimplePageSequenceFrame::PrintNextPage(nsIPresContext*  aPresContext)
     }
 
     mPageNum++;
-    rv = mCurrentPageFrame->GetNextSibling(&mCurrentPageFrame);
+    mCurrentPageFrame = mCurrentPageFrame->GetNextSibling();
   }
 
   return rv;
@@ -1019,8 +995,8 @@ nsSimplePageSequenceFrame::DoPageEnd(nsIPresContext*  aPresContext)
 
   mPageNum++;
   
-  if( nsnull != mCurrentPageFrame){
-    rv = mCurrentPageFrame->GetNextSibling(&mCurrentPageFrame);
+  if (mCurrentPageFrame) {
+    mCurrentPageFrame = mCurrentPageFrame->GetNextSibling();
   }
   
   return rv;
@@ -1029,7 +1005,7 @@ nsSimplePageSequenceFrame::DoPageEnd(nsIPresContext*  aPresContext)
 NS_IMETHODIMP
 nsSimplePageSequenceFrame::SuppressHeadersAndFooters(PRBool aDoSup)
 {
-  for (nsIFrame* f = mFrames.FirstChild(); f != nsnull; f->GetNextSibling(&f)) {
+  for (nsIFrame* f = mFrames.FirstChild(); f; f = f->GetNextSibling()) {
     nsPageFrame * pf = NS_STATIC_CAST(nsPageFrame*, f);
     if (pf != nsnull) {
       pf->SuppressHeadersAndFooters(aDoSup);
@@ -1041,7 +1017,7 @@ nsSimplePageSequenceFrame::SuppressHeadersAndFooters(PRBool aDoSup)
 NS_IMETHODIMP
 nsSimplePageSequenceFrame::SetClipRect(nsIPresContext*  aPresContext, nsRect* aRect)
 {
-  for (nsIFrame* f = mFrames.FirstChild(); f != nsnull; f->GetNextSibling(&f)) {
+  for (nsIFrame* f = mFrames.FirstChild(); f; f = f->GetNextSibling()) {
     nsPageFrame * pf = NS_STATIC_CAST(nsPageFrame*, f);
     if (pf != nsnull) {
       pf->SetClipRect(aRect);
@@ -1076,11 +1052,6 @@ nsSimplePageSequenceFrame::Paint(nsIPresContext*      aPresContext,
   return rv;
 }
 
-NS_IMETHODIMP nsSimplePageSequenceFrame::SizeTo(nsIPresContext* aPresContext, nscoord aWidth, nscoord aHeight)
-{
-  return nsFrame::SizeTo(aPresContext, aWidth, aHeight);
-}
-
 NS_IMETHODIMP
 nsSimplePageSequenceFrame::GetFrameType(nsIAtom** aType) const
 {
