@@ -368,9 +368,7 @@ nsHTMLFrameOuterFrame::Init(nsIPresContext*  aPresContext,
   // (e.g., the canvas) when it really needs to have the OuterFrame's
   // view as its parent. So, create the OuterFrame's view right away
   // if we need it, and the InnerFrame's view will get it as the parent.
-  nsIView* view = nsnull;
-  GetView(aPresContext, &view);
-  if (!view) {
+  if (!HasView()) {
     // To properly initialize the view we need to know the frame for the content
     // that is the parent of content for this frame. This might not be our actual
     // frame parent if we are out of flow (e.g., positioned) so our parent frame
@@ -399,17 +397,12 @@ nsHTMLFrameOuterFrame::Init(nsIPresContext*  aPresContext,
     }
   
     nsHTMLContainerFrame::CreateViewForFrame(aPresContext,this,mStyleContext,contentParent,PR_TRUE); 
-    GetView(aPresContext, &view);
   }
+  nsIView* view = GetView();
 
-  const nsStyleDisplay* disp;
-  aParent->GetStyleData(eStyleStruct_Display, ((const nsStyleStruct *&)disp));
-  if (disp->mDisplay == NS_STYLE_DISPLAY_DECK) {
-    nsCOMPtr<nsIWidget> widget;
-    view->GetWidget(*getter_AddRefs(widget));
-
-    if (!widget)
-      view->CreateWidget(kCChildCID);
+  if (aParent->GetStyleDisplay()->mDisplay == NS_STYLE_DISPLAY_DECK
+      && !view->HasWidget()) {
+    view->CreateWidget(kCChildCID);
   }
 
   nsCOMPtr<nsIPresShell> shell;
@@ -588,8 +581,7 @@ nsHTMLFrameOuterFrame::Reflow(nsIPresContext*          aPresContext,
 
   {
     // Invalidate the frame
-    nsRect frameRect;
-    GetRect(frameRect);
+    nsRect frameRect = GetRect();
     nsRect rect(0, 0, frameRect.width, frameRect.height);
     if (!rect.IsEmpty()) {
       Invalidate(aPresContext, rect, PR_FALSE);
@@ -621,7 +613,7 @@ nsHTMLFrameOuterFrame::AttributeChanged(nsIPresContext* aPresContext,
                                         PRInt32 aHint)
 {
   nsCOMPtr<nsIAtom> type;
-  aChild->GetTag(*getter_AddRefs(type));
+  aChild->GetTag(getter_AddRefs(type));
 
   if ((type != nsHTMLAtoms::object && aAttribute == nsHTMLAtoms::src) ||
       (type == nsHTMLAtoms::object && aAttribute == nsHTMLAtoms::data)) {
@@ -635,14 +627,13 @@ nsHTMLFrameOuterFrame::AttributeChanged(nsIPresContext* aPresContext,
   // If the noResize attribute changes, dis/allow frame to be resized
   else if (aAttribute == nsHTMLAtoms::noresize) {
     nsCOMPtr<nsIContent> parentContent;
-    mContent->GetParent(*getter_AddRefs(parentContent));
+    mContent->GetParent(getter_AddRefs(parentContent));
 
     nsCOMPtr<nsIAtom> parentTag;
-    parentContent->GetTag(*getter_AddRefs(parentTag));
+    parentContent->GetTag(getter_AddRefs(parentTag));
 
     if (parentTag == nsHTMLAtoms::frameset) {
-      nsIFrame* parentFrame = nsnull;
-      GetParent(&parentFrame);
+      nsIFrame* parentFrame = GetParent();
 
       if (parentFrame) {
         // There is no interface for nsHTMLFramesetFrame so QI'ing to
@@ -770,15 +761,15 @@ PRBool nsHTMLFrameInnerFrame::GetURL(nsIContent* aContent, nsString& aResult)
 {
   aResult.SetLength(0);
   nsCOMPtr<nsIAtom> type;
-  aContent->GetTag(*getter_AddRefs(type));
+  aContent->GetTag(getter_AddRefs(type));
 
   if (type.get() == nsHTMLAtoms::object) {
     if (NS_CONTENT_ATTR_HAS_VALUE == (aContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::data, aResult)))
-      if (aResult.Length() > 0)
+      if (!aResult.IsEmpty())
         return PR_TRUE;
   }else
     if (NS_CONTENT_ATTR_HAS_VALUE == (aContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, aResult)))
-      if (aResult.Length() > 0)
+      if (!aResult.IsEmpty())
         return PR_TRUE;
 
   return PR_FALSE;
@@ -789,7 +780,7 @@ PRBool nsHTMLFrameInnerFrame::GetName(nsIContent* aContent, nsString& aResult)
   aResult.SetLength(0);
 
   if (NS_CONTENT_ATTR_HAS_VALUE == (aContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, aResult))) {
-    if (aResult.Length() > 0) {
+    if (!aResult.IsEmpty()) {
       return PR_TRUE;
     }
   }
@@ -805,6 +796,9 @@ PRInt32 nsHTMLFrameInnerFrame::GetScrolling(nsIContent* aContent)
 
   if (NS_SUCCEEDED(rv) && content) {
     nsHTMLValue value;
+    // XXXldb This code belongs in the attribute mapping code for the
+    // content node -- otherwise it doesn't follow the CSS cascading
+    // rules correctly.
     if (NS_CONTENT_ATTR_HAS_VALUE == content->GetHTMLAttribute(nsHTMLAtoms::scrolling, value)) {
       if (eHTMLUnit_Enumerated == value.GetUnit()) {
         switch (value.GetIntValue()) {
@@ -829,8 +823,7 @@ PRInt32 nsHTMLFrameInnerFrame::GetScrolling(nsIContent* aContent)
     }
 
     // Check style for overflow
-    const nsStyleDisplay* display;
-    GetStyleData(eStyleStruct_Display, ((const nsStyleStruct *&)display));
+    const nsStyleDisplay* display = GetStyleDisplay();
     if (display->mOverflow)
       returnValue = display->mOverflow;
   }
@@ -946,9 +939,7 @@ nsHTMLFrameInnerFrame::Paint(nsIPresContext*      aPresContext,
     GetDocShell(getter_AddRefs(docShell));
 
     if (!docShell) {
-      const nsStyleBackground* color =
-        (const nsStyleBackground*)mStyleContext->
-        GetStyleData(eStyleStruct_Background);
+      const nsStyleBackground* color = GetStyleBackground();
 
       aRenderingContext.SetColor(color->mBackgroundColor);
       aRenderingContext.FillRect(mRect);
@@ -965,11 +956,10 @@ nsHTMLFrameInnerFrame::GetParentContent(nsIContent** aContent)
 {
   *aContent = nsnull;
 
-  nsIFrame* parent = nsnull;
-  GetParent(&parent);
-
+  nsIFrame* parent = GetParent();
   if (parent) {
-    parent->GetContent(aContent);
+    *aContent = parent->GetContent();
+    NS_IF_ADDREF(*aContent);
   }
 }
 
@@ -1029,18 +1019,14 @@ nsHTMLFrameInnerFrame::DidReflow(nsIPresContext*           aPresContext,
   // The view is created hidden; once we have reflowed it and it has been
   // positioned then we show it.
   if (NS_FRAME_REFLOW_FINISHED == aStatus) {
-    nsIView* view = nsnull;
-    GetView(aPresContext, &view);
+    nsIView* view = GetView();
     if (view) {
-      const nsStyleVisibility* vis;
-      GetStyleData(eStyleStruct_Visibility, ((const nsStyleStruct *&)vis));
-      nsViewVisibility newVis = vis->IsVisible() ? nsViewVisibility_kShow : nsViewVisibility_kHide;
-      nsViewVisibility oldVis;
+      nsViewVisibility newVis = GetStyleVisibility()->IsVisible()
+                                  ? nsViewVisibility_kShow
+                                  : nsViewVisibility_kHide;
       // only change if different.
-      view->GetVisibility(oldVis);
-      if (newVis != oldVis) {
-        nsCOMPtr<nsIViewManager> vm;
-        view->GetViewManager(*getter_AddRefs(vm));
+      if (newVis != view->GetVisibility()) {
+        nsIViewManager* vm = view->GetViewManager();
         if (vm) {
           vm->SetViewVisibility(view, newVis);
         }
@@ -1120,10 +1106,6 @@ nsHTMLFrameInnerFrame::CreateViewAndWidget(nsIPresContext* aPresContext,
   NS_ENSURE_ARG_POINTER(aPresContext);
   NS_ENSURE_ARG_POINTER(aWidget);
 
-  nsCOMPtr<nsIPresShell> presShell;
-  aPresContext->GetShell(getter_AddRefs(presShell));
-  NS_ENSURE_TRUE(presShell, NS_ERROR_FAILURE);
-
   float t2p;
   aPresContext->GetTwipsToPixels(&t2p);
 
@@ -1143,8 +1125,7 @@ nsHTMLFrameInnerFrame::CreateViewAndWidget(nsIPresContext* aPresContext,
   GetOffsetFromView(aPresContext, origin, &parView);
   nsRect viewBounds(origin.x, origin.y, 10, 10);
 
-  nsCOMPtr<nsIViewManager> viewMan;
-  presShell->GetViewManager(getter_AddRefs(viewMan));
+  nsIViewManager* viewMan = aPresContext->GetViewManager();
   rv = view->Init(viewMan, viewBounds, parView);
   // XXX put it at the end of the document order until we can do better
   viewMan->InsertChild(parView, view, nsnull, PR_TRUE);
@@ -1156,18 +1137,17 @@ nsHTMLFrameInnerFrame::CreateViewAndWidget(nsIPresContext* aPresContext,
 
   rv = view->CreateWidget(kCChildCID, nsnull, nsnull, PR_TRUE, PR_TRUE,
                           xulElement? eContentTypeUI: eContentTypeContent);
-  SetView(aPresContext, view);
+  SetView(view);
 
   nsContainerFrame::SyncFrameViewProperties(aPresContext, this, nsnull, view);
 
   // XXX the following should be unnecessary, given the above Sync call
   // if the visibility is hidden, reflect that in the view
-  const nsStyleVisibility* vis;
-  GetStyleData(eStyleStruct_Visibility, ((const nsStyleStruct *&)vis));
-  if (!vis->IsVisible()) {
+  if (!GetStyleVisibility()->IsVisible()) {
     viewMan->SetViewVisibility(view, nsViewVisibility_kHide);
   }
-  view->GetWidget(*aWidget);
+  *aWidget = view->GetWidget();
+  NS_IF_ADDREF(*aWidget);
   return rv;
 }
 
