@@ -38,7 +38,7 @@
 #include "nsIScrollableView.h"
 #include "nsStyleCoord.h"
 #include "nsStyleConsts.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsIDocumentLoader.h"
 #include "nsHTMLParts.h"
 #include "nsILookAndFeel.h"
@@ -54,6 +54,7 @@
 #include "nsINameSpaceManager.h"
 #include "nsCSSPseudoElements.h"
 #include "nsCSSAnonBoxes.h"
+#include "nsAutoPtr.h"
 
 // masks for mEdgeVisibility
 #define LEFT_VIS   0x0001
@@ -283,7 +284,7 @@ nsHTMLFramesetFrame::Observe(nsISupports* aObject, const char* aAction,
   nsAutoString prefName(aPrefName);
   if (prefName.Equals(NS_LITERAL_STRING(kFrameResizePref))) {
     nsCOMPtr<nsIDocument> doc;
-    mContent->GetDocument(*getter_AddRefs(doc));
+    mContent->GetDocument(getter_AddRefs(doc));
     if (doc) {
       doc->BeginUpdate();
       doc->AttributeWillChange(mContent,
@@ -317,23 +318,22 @@ NS_IMETHODIMP
 nsHTMLFramesetFrame::Init(nsIPresContext*  aPresContext,
                           nsIContent*      aContent,
                           nsIFrame*        aParent,
-                          nsIStyleContext* aContext,
+                          nsStyleContext*  aContext,
                           nsIFrame*        aPrevInFlow)
 {
   nsHTMLContainerFrame::Init(aPresContext, aContent, aParent,
                              aContext, aPrevInFlow);
   // find the highest ancestor that is a frameset
   nsresult rv = NS_OK;
-  nsIFrame* parentFrame = nsnull;
-  GetParent((nsIFrame**)&parentFrame);
+  nsIFrame* parentFrame = GetParent();
   mTopLevelFrameset = (nsHTMLFramesetFrame*)this;
   while (parentFrame) {
-    nsHTMLFramesetFrame* frameset;
-    rv = parentFrame->QueryInterface(NS_GET_IID(nsHTMLFramesetFrame),
-				     (void**)&frameset);
-    if (NS_SUCCEEDED(rv)) {
+    nsHTMLFramesetFrame* frameset = nsnull;
+    CallQueryInterface(parentFrame, &frameset);
+    
+    if (frameset) {
       mTopLevelFrameset = frameset;
-      parentFrame->GetParent((nsIFrame**)&parentFrame);
+      parentFrame = parentFrame->GetParent();
     } else {
       break;
     }
@@ -341,22 +341,17 @@ nsHTMLFramesetFrame::Init(nsIPresContext*  aPresContext,
 
   // create the view. a view is needed since it needs to be a mouse grabber
   nsIView* view;
-  nsresult result = nsComponentManager::CreateInstance(kViewCID, nsnull, NS_GET_IID(nsIView),
-                                                 (void **)&view);
-  nsCOMPtr<nsIPresShell> presShell;
-  aPresContext->GetShell(getter_AddRefs(presShell));
-  nsCOMPtr<nsIViewManager> viewMan;
-  presShell->GetViewManager(getter_AddRefs(viewMan));
+  nsresult result = CallCreateInstance(kViewCID, &view);
+  nsIViewManager* viewMan = aPresContext->GetViewManager();
 
   nsIFrame* parWithView;
-  nsIView *parView;
   GetParentWithView(aPresContext, &parWithView);
-  parWithView->GetView(aPresContext, &parView);
+  nsIView *parView = parWithView->GetView();
   nsRect boundBox(0, 0, 0, 0); 
   result = view->Init(viewMan, boundBox, parView);
   // XXX Put it last in document order until we can do better
   viewMan->InsertChild(parView, view, nsnull, PR_TRUE);
-  SetView(aPresContext, view);
+  SetView(view);
 
   nsCOMPtr<nsIPresShell> shell;
   aPresContext->GetShell(getter_AddRefs(shell));
@@ -402,17 +397,16 @@ nsHTMLFramesetFrame::Init(nsIPresContext*  aPresContext,
       break;
     }
     nsCOMPtr<nsIContent> child;
-    mContent->ChildAt(childX, *getter_AddRefs(child));
+    mContent->ChildAt(childX, getter_AddRefs(child));
     if (!child->IsContentOfType(nsIContent::eHTML))
       continue;
     nsCOMPtr<nsIAtom> tag;
-    child->GetTag(*getter_AddRefs(tag));
+    child->GetTag(getter_AddRefs(tag));
     if (tag == nsHTMLAtoms::frameset || tag == nsHTMLAtoms::frame) {
-      nsCOMPtr<nsIStyleContext> kidSC;
+      nsRefPtr<nsStyleContext> kidSC;
       nsresult result;
 
-      aPresContext->ResolveStyleContextFor(child, mStyleContext,
-                                           getter_AddRefs(kidSC));
+      kidSC = aPresContext->ResolveStyleContextFor(child, mStyleContext);
       if (tag == nsHTMLAtoms::frameset) {
         result = NS_NewHTMLFramesetFrame(shell, &frame);
 
@@ -453,10 +447,10 @@ nsHTMLFramesetFrame::Init(nsIPresContext*  aPresContext,
     // XXX the blank frame is using the content of its parent - at some point it 
     // should just have null content, if we support that
     nsHTMLFramesetBlankFrame* blankFrame = new (shell.get()) nsHTMLFramesetBlankFrame;
-    nsCOMPtr<nsIStyleContext> pseudoStyleContext;
-    aPresContext->ResolvePseudoStyleContextFor(nsnull,
-                            nsCSSAnonBoxes::framesetBlank, mStyleContext,
-                            getter_AddRefs(pseudoStyleContext));
+    nsRefPtr<nsStyleContext> pseudoStyleContext;
+    pseudoStyleContext = aPresContext->ResolvePseudoStyleContextFor(nsnull,
+                                                                    nsCSSAnonBoxes::framesetBlank,
+                                                                    mStyleContext);
     if(blankFrame)
       blankFrame->Init(aPresContext, mContent, this, pseudoStyleContext, nsnull);
    
@@ -711,22 +705,22 @@ nsHTMLFramesetFrame::GetDesiredSize(nsIPresContext*          aPresContext,
 nsHTMLFramesetFrame* nsHTMLFramesetFrame::GetFramesetParent(nsIFrame* aChild)
 {
   nsHTMLFramesetFrame* parent = nsnull;
-  nsCOMPtr<nsIContent> content;
-  aChild->GetContent(getter_AddRefs(content));
+  nsIContent* content = aChild->GetContent();
 
   if (content) { 
     nsCOMPtr<nsIContent> contentParent;
-    content->GetParent(*getter_AddRefs(contentParent));
+    content->GetParent(getter_AddRefs(contentParent));
 
     nsCOMPtr<nsIHTMLContent> contentParent2 =
       do_QueryInterface(contentParent);
 
     if (contentParent2) {
       nsCOMPtr<nsIAtom> tag;
-      contentParent2->GetTag(*getter_AddRefs(tag));
+      contentParent2->GetTag(getter_AddRefs(tag));
 
       if (tag == nsHTMLAtoms::frameset) {
-        aChild->GetParent((nsIFrame**)&parent);
+        nsIFrame* fptr = aChild->GetParent();
+        parent = (nsHTMLFramesetFrame*) fptr;
       }
     }
   }
@@ -759,7 +753,7 @@ void nsHTMLFramesetFrame::GetSizeOfChild(nsIFrame* aChild,
   // this assumption is used here
   int i = 0;
   for (nsIFrame* child = mFrames.FirstChild(); child;
-       child->GetNextSibling(&child)) {
+       child = child->GetNextSibling()) {
     if (aChild == child) {
       nsPoint ignore;
       GetSizeOfChildAt(i, aSize, ignore);
@@ -799,11 +793,9 @@ PRBool
 nsHTMLFramesetFrame::IsGrabbingMouse()
 {
   PRBool result = PR_FALSE;
-  nsIView* view;
-  GetView(&view);
+  nsIView* view = GetView();
   if (view) {
-    nsIViewManager* viewMan;
-    view->GetViewManager(viewMan);
+    nsIViewManager* viewMan = view->GetViewManager();
     if (viewMan) {
       nsIView* grabber;
       viewMan->GetMouseEventGrabber(grabber);
@@ -1080,12 +1072,11 @@ nsHTMLFramesetFrame::Reflow(nsIPresContext*          aPresContext,
       offset.y += lastSize.height;
       if (firstTime) { // create horizontal border
         borderFrame = new (shell.get()) nsHTMLFramesetBorderFrame(borderWidth, PR_FALSE, PR_FALSE);
-        nsIStyleContext* pseudoStyleContext;
-        aPresContext->ResolvePseudoStyleContextFor(mContent, nsCSSPseudoElements::horizontalFramesetBorder,
-                                                   mStyleContext,
-                                                   &pseudoStyleContext);
+        nsRefPtr<nsStyleContext> pseudoStyleContext;
+        pseudoStyleContext = aPresContext->ResolvePseudoStyleContextFor(mContent,
+                                                                        nsCSSPseudoElements::horizontalFramesetBorder,
+                                                                        mStyleContext);
         borderFrame->Init(aPresContext, mContent, this, pseudoStyleContext, nsnull);
-        NS_RELEASE(pseudoStyleContext);
 
         mChildCount++;
         lastChild->SetNextSibling(borderFrame);
@@ -1108,12 +1099,11 @@ nsHTMLFramesetFrame::Reflow(nsIPresContext*          aPresContext,
         if (0 == cellIndex.y) { // in 1st row
           if (firstTime) { // create vertical border
             borderFrame = new (shell.get()) nsHTMLFramesetBorderFrame(borderWidth, PR_TRUE, PR_FALSE);
-            nsIStyleContext* pseudoStyleContext;
-            aPresContext->ResolvePseudoStyleContextFor(mContent, nsCSSPseudoElements::verticalFramesetBorder,
-                                                       mStyleContext,
-                                                       &pseudoStyleContext);
+            nsRefPtr<nsStyleContext> pseudoStyleContext;
+            pseudoStyleContext = aPresContext->ResolvePseudoStyleContextFor(mContent,
+                                                                            nsCSSPseudoElements::verticalFramesetBorder,
+                                                                            mStyleContext);
             borderFrame->Init(aPresContext, mContent, this, pseudoStyleContext, nsnull);
-            NS_RELEASE(pseudoStyleContext);
 
             mChildCount++;
             lastChild->SetNextSibling(borderFrame);
@@ -1219,7 +1209,7 @@ nsHTMLFramesetFrame::Reflow(nsIPresContext*          aPresContext,
     lastCol  = cellIndex.x;
     lastSize = size;
     offset.x += size.width;
-    child->GetNextSibling(&child);
+    child = child->GetNextSibling();
   }
 
   if (firstTime) {
@@ -1259,9 +1249,8 @@ nsHTMLFramesetFrame::Reflow(nsIPresContext*          aPresContext,
     delete[] mChildBorderColors;
   }
 
-  if (nsnull != aDesiredSize.maxElementSize) {
-    aDesiredSize.maxElementSize->width = aDesiredSize.width;
-    aDesiredSize.maxElementSize->height = aDesiredSize.height;
+  if (aDesiredSize.mComputeMEW) {
+    aDesiredSize.mMaxElementWidth = aDesiredSize.width;
   }
 
   aStatus = NS_FRAME_COMPLETE;
@@ -1315,8 +1304,7 @@ PRBool
 nsHTMLFramesetFrame::GetNoResize(nsIFrame* aChildFrame) 
 {
   PRBool result = PR_FALSE;
-  nsCOMPtr<nsIContent> content;
-  aChildFrame->GetContent(getter_AddRefs(content));
+  nsIContent* content = aChildFrame->GetContent();
 
   nsCOMPtr<nsIHTMLContent> htmlContent(do_QueryInterface(content));
 
@@ -1360,11 +1348,11 @@ nsHTMLFramesetFrame::RecalculateBorderResize()
   mContent->ChildCount(numChildren);
   for (childIndex = 0; childIndex < numChildren; childIndex++) {
     nsCOMPtr<nsIContent> childCon;
-    mContent->ChildAt(childIndex, *getter_AddRefs(childCon));
+    mContent->ChildAt(childIndex, getter_AddRefs(childCon));
     nsCOMPtr<nsIHTMLContent> child(do_QueryInterface(childCon));
     if (child) {
       nsCOMPtr<nsIAtom> tag;
-      child->GetTag(*getter_AddRefs(tag));
+      child->GetTag(getter_AddRefs(tag));
       if (tag == nsHTMLAtoms::frameset) {
         childTypes[frameOrFramesetChildIndex++] = FRAMESET;
       } else if (tag == nsHTMLAtoms::frame) {
@@ -1459,11 +1447,9 @@ nsHTMLFramesetFrame::StartMouseDrag(nsIPresContext*            aPresContext,
   IndexOf(aBorder, index);
   NS_ASSERTION((nsnull != aBorder) && (index >= 0), "invalid dragger");
 #endif
-  nsIView* view;
-  GetView(aPresContext, &view);
+  nsIView* view = GetView();
   if (view) {
-    nsCOMPtr<nsIViewManager> viewMan;
-    view->GetViewManager(*getter_AddRefs(viewMan));
+    nsIViewManager* viewMan = view->GetViewManager();
     if (viewMan) {
       PRBool ignore;
       viewMan->GrabMouseEvents(view, ignore);
@@ -1546,21 +1532,13 @@ nsHTMLFramesetFrame::MouseDrag(nsIPresContext* aPresContext,
 
   if (change != 0) {
     mDrag.Reset(mDragger->mVertical, mDragger->mPrevNeighbor, change, this);
-    nsIFrame* parentFrame = nsnull;
-    GetParent((nsIFrame**)&parentFrame);
+    nsIFrame* parentFrame = GetParent();
     if (!parentFrame) {
       return;
     }
 
-    nsCOMPtr<nsIPresShell> shell;
-    aPresContext->GetShell(getter_AddRefs(shell));
-    if (!shell) {
-      return;
-    }
-
     // Update the view immediately (make drag appear snappier)
-    nsCOMPtr<nsIViewManager> vm;
-    shell->GetViewManager(getter_AddRefs(vm));
+    nsIViewManager* vm = aPresContext->GetViewManager();
     if (vm) {
       nsIView* root;
       vm->GetRootView(root);
@@ -1574,11 +1552,9 @@ nsHTMLFramesetFrame::MouseDrag(nsIPresContext* aPresContext,
 void
 nsHTMLFramesetFrame::EndMouseDrag(nsIPresContext* aPresContext)
 {
-  nsIView* view;
-  GetView(aPresContext, &view);
+  nsIView* view = GetView();
   if (view) {
-    nsCOMPtr<nsIViewManager> viewMan;
-    view->GetViewManager(*getter_AddRefs(viewMan));
+    nsIViewManager* viewMan = view->GetViewManager();
     if (viewMan) {
       mDragger = nsnull;
       PRBool ignore;
@@ -1767,8 +1743,9 @@ nsHTMLFramesetBorderFrame::HandleEvent(nsIPresContext* aPresContext,
 
   switch (aEvent->message) {
     case NS_MOUSE_LEFT_BUTTON_DOWN:
-      nsHTMLFramesetFrame* parentFrame = nsnull;
-      GetParent((nsIFrame**)&parentFrame);
+      nsHTMLFramesetFrame* parentFrame;
+      nsIFrame* fptr = GetParent();
+      parentFrame = (nsHTMLFramesetFrame*) fptr;
       parentFrame->StartMouseDrag(aPresContext, this, aEvent);
       *aEventStatus = nsEventStatus_eConsumeNoDefault;
 	    break;
