@@ -71,6 +71,7 @@
 #include "nsIDOMEventReceiver.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsILocalFile.h"
+#include "nsITextControlElement.h"
 
 #include "nsContentCID.h"
 static NS_DEFINE_CID(kHTMLElementFactoryCID,   NS_HTML_ELEMENT_FACTORY_CID);
@@ -116,20 +117,37 @@ nsFileControlFrame::~nsFileControlFrame()
   }
 }
 
+NS_IMETHODIMP 
+nsFileControlFrame::Destroy(nsIPresContext* aPresContext)
+{
+  // Toss the value into the control from the anonymous content, which is about
+  // to get lost.
+  if (mTextContent) {
+    nsCOMPtr<nsIDOMHTMLInputElement> input = do_QueryInterface(mTextContent);
+    nsAutoString value;
+    input->GetValue(value);
+
+    // Have it take the value, just like when input type=text goes away
+    nsCOMPtr<nsITextControlElement> fileInput = do_QueryInterface(mContent);
+    fileInput->TakeTextFrameValue(value);
+  }
+  return nsAreaFrame::Destroy(aPresContext);
+}
+
 NS_IMETHODIMP
 nsFileControlFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
                                            nsISupportsArray& aChildList)
 {
   // Get the NodeInfoManager and tag necessary to create input elements
   nsCOMPtr<nsIDocument> doc;
-  mContent->GetDocument(*getter_AddRefs(doc));
+  mContent->GetDocument(getter_AddRefs(doc));
   nsCOMPtr<nsINodeInfoManager> nimgr;
-  nsresult rv = doc->GetNodeInfoManager(*getter_AddRefs(nimgr));
+  nsresult rv = doc->GetNodeInfoManager(getter_AddRefs(nimgr));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsINodeInfo> nodeInfo;
   nimgr->GetNodeInfo(nsHTMLAtoms::input, nsnull, kNameSpaceID_None,
-                     *getter_AddRefs(nodeInfo));
+                     getter_AddRefs(nodeInfo));
 
   nsCOMPtr<nsIElementFactory> ef(do_GetService(kHTMLElementFactoryCID,&rv));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -194,11 +212,10 @@ nsFileControlFrame::QueryInterface(const nsIID& aIID, void** aInstancePtr)
   return nsHTMLContainerFrame::QueryInterface(aIID, aInstancePtr);
 }
 
-NS_IMETHODIMP 
-nsFileControlFrame::GetType(PRInt32* aType) const
+NS_IMETHODIMP_(PRInt32)
+nsFileControlFrame::GetType() const
 {
-  *aType = NS_FORM_INPUT_FILE;
-  return NS_OK;
+  return NS_FORM_INPUT_FILE;
 }
 
 
@@ -207,8 +224,7 @@ nsFileControlFrame::SetFocus(PRBool aOn, PRBool aRepaint)
 {
   // Fix for Bug 6133 
   if (mTextFrame) {
-    nsCOMPtr<nsIContent> content;
-    mTextFrame->GetContent(getter_AddRefs(content));
+    nsIContent* content = mTextFrame->GetContent();
     if (content) {
       content->SetFocus(mPresContext);
     }
@@ -249,13 +265,12 @@ nsFileControlFrame::MouseClick(nsIDOMEvent* aMouseEvent)
   nsresult result;
 
   // Get parent nsIDOMWindowInternal object.
-  nsCOMPtr<nsIContent> content;
-  result = GetContent(getter_AddRefs(content));
+  nsIContent* content = GetContent();
   if (!content)
-    return NS_FAILED(result) ? result : NS_ERROR_FAILURE;
+    return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsIDocument> doc;
-  result = content->GetDocument(*getter_AddRefs(doc));
+  result = content->GetDocument(getter_AddRefs(doc));
   if (!doc)
     return NS_FAILED(result) ? result : NS_ERROR_FAILURE;
 
@@ -357,19 +372,16 @@ NS_IMETHODIMP nsFileControlFrame::Reflow(nsIPresContext*          aPresContext,
   // except for when style is used to change its size.
   nsresult rv = nsAreaFrame::Reflow(aPresContext, aDesiredSize, aReflowState, aStatus);
   if (NS_SUCCEEDED(rv) && mTextFrame != nsnull) {
-    const nsStyleVisibility* vis;
-    GetStyleData(eStyleStruct_Visibility, (const nsStyleStruct*&)vis);
+    const nsStyleVisibility* vis = GetStyleVisibility();
 
     nsIFrame * child;
     FirstChild(aPresContext, nsnull, &child);
-    while (child == mTextFrame) {
-      child->GetNextSibling(&child);
+    if (child == mTextFrame) {
+      child = child->GetNextSibling();
     }
-    if (child != nsnull) {
-      nsRect buttonRect;
-      nsRect txtRect;
-      mTextFrame->GetRect(txtRect);
-      child->GetRect(buttonRect);
+    if (child) {
+      nsRect buttonRect = child->GetRect();
+      nsRect txtRect = mTextFrame->GetRect();
 
       // check to see if we must reflow just the area frame again 
       // in order for the text field to be the correct height
@@ -377,8 +389,8 @@ NS_IMETHODIMP nsFileControlFrame::Reflow(nsIPresContext*          aPresContext,
       // messes up the button's rect
       if (txtRect.width + buttonRect.width != aDesiredSize.width ||
           txtRect.height != aDesiredSize.height) {
+        nsHTMLReflowMetrics txtKidSize(PR_TRUE);
         nsSize txtAvailSize(aReflowState.availableWidth, aDesiredSize.height);
-        nsHTMLReflowMetrics txtKidSize(&txtAvailSize);
         nsHTMLReflowState   txtKidReflowState(aPresContext, aReflowState, this, txtAvailSize,
                                               eReflowReason_Resize);
         txtKidReflowState.mComputedHeight = aDesiredSize.height;
@@ -394,7 +406,7 @@ NS_IMETHODIMP nsFileControlFrame::Reflow(nsIPresContext*          aPresContext,
           // now adjust the frame positions
           txtRect.y      = aReflowState.mComputedBorderPadding.top;
           txtRect.height = aDesiredSize.height;
-          mTextFrame->SetRect(aPresContext, txtRect);
+          mTextFrame->SetRect(txtRect);
         }
       }
 
@@ -404,11 +416,11 @@ NS_IMETHODIMP nsFileControlFrame::Reflow(nsIPresContext*          aPresContext,
       // and we must make sure the text field is the correct height
       if (NS_STYLE_DIRECTION_RTL == vis->mDirection) {
         buttonRect.x      = aReflowState.mComputedBorderPadding.left;
-        child->SetRect(aPresContext, buttonRect);
+        child->SetRect(buttonRect);
         txtRect.x         = aDesiredSize.width - txtRect.width + aReflowState.mComputedBorderPadding.left;
         txtRect.y         = aReflowState.mComputedBorderPadding.top;
         txtRect.height    = aDesiredSize.height;
-        mTextFrame->SetRect(aPresContext, txtRect);
+        mTextFrame->SetRect(txtRect);
       }
 
     }
@@ -442,11 +454,10 @@ nsFileControlFrame::GetTextControlFrame(nsIPresContext* aPresContext, nsIFrame*
 
   while (childFrame) {
     // see if the child is a text control
-    nsCOMPtr<nsIContent> content;
-    nsresult res = childFrame->GetContent(getter_AddRefs(content));
-    if (NS_SUCCEEDED(res) && content) {
+    nsIContent* content = childFrame->GetContent();
+    if (content) {
       nsCOMPtr<nsIAtom> atom;
-      res = content->GetTag(*getter_AddRefs(atom));
+      nsresult res = content->GetTag(getter_AddRefs(atom));
       if (NS_SUCCEEDED(res) && atom) {
         if (atom.get() == nsHTMLAtoms::input) {
 
@@ -466,8 +477,7 @@ nsFileControlFrame::GetTextControlFrame(nsIPresContext* aPresContext, nsIFrame*
     if (frame)
        result = frame;
      
-    res = childFrame->GetNextSibling(&childFrame);
-    NS_ASSERTION(res == NS_OK,"failed to get next child");
+    childFrame = childFrame->GetNextSibling();
   }
 
   return result;
@@ -541,10 +551,7 @@ nsFileControlFrame::GetFrameForPoint(nsIPresContext* aPresContext,
 {
 #ifndef DEBUG_NEWFRAME
   if ( nsFormControlHelper::GetDisabled(mContent) && mRect.Contains(aPoint) ) {
-    const nsStyleVisibility* vis = 
-      (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
-      
-    if (vis->IsVisible()) {
+    if (GetStyleVisibility()->IsVisible()) {
       *aFrame = this;
       return NS_OK;
     }
@@ -566,12 +573,9 @@ nsFileControlFrame::GetFrameName(nsAString& aResult) const
 NS_IMETHODIMP
 nsFileControlFrame::GetFormContent(nsIContent*& aContent) const
 {
-  nsIContent* content;
-  nsresult    rv;
-
-  rv = GetContent(&content);
-  aContent = content;
-  return rv;
+  aContent = GetContent();
+  NS_IF_ADDREF(aContent);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
