@@ -59,6 +59,8 @@
 #include "nsIElementFactory.h"
 #include "nsIHTMLContent.h"
 #include "nsIEditorIMESupport.h"
+#include "nsIPhonetic.h"
+#include "nsIEditorObserver.h"
 #include "nsIDOMHTMLTextAreaElement.h"
 #include "nsINameSpaceManager.h"
 #include "nsINodeInfo.h"
@@ -219,7 +221,6 @@ nsTextInputListener::nsTextInputListener()
 , mHadUndoItems(PR_FALSE)
 , mHadRedoItems(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsTextInputListener::~nsTextInputListener() 
@@ -262,7 +263,7 @@ nsTextInputListener::NotifySelectionChanged(nsIDOMDocument* aDoc, nsISelection*
     if (content) 
     {
       nsCOMPtr<nsIDocument> doc;
-      if (NS_SUCCEEDED(content->GetDocument(*getter_AddRefs(doc)))) 
+      if (NS_SUCCEEDED(content->GetDocument(getter_AddRefs(doc))))
       {
         if (doc) 
         {
@@ -380,12 +381,11 @@ nsTextInputListener::EditAction()
 nsresult
 nsTextInputListener::UpdateTextInputCommands(const nsAString& commandsToUpdate)
 {
-  nsCOMPtr<nsIContent> content;
-  nsresult rv = mFrame->GetContent(getter_AddRefs(content));
+  nsIContent* content = mFrame->GetContent();
   NS_ENSURE_TRUE(content, NS_ERROR_FAILURE);
   
   nsCOMPtr<nsIDocument> doc;
-  rv = content->GetDocument(*getter_AddRefs(doc));
+  nsresult rv = content->GetDocument(getter_AddRefs(doc));
   NS_ENSURE_TRUE(doc, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsIScriptGlobalObject> scriptGlobalObject;
@@ -427,6 +427,7 @@ public:
   NS_IMETHOD SetCaretWidth(PRInt16 twips);
   NS_IMETHOD SetCaretReadOnly(PRBool aReadOnly);
   NS_IMETHOD GetCaretEnabled(PRBool *_retval);
+  NS_IMETHOD SetCaretVisibilityDuringSelection(PRBool aVisibility);
   NS_IMETHOD CharacterMove(PRBool aForward, PRBool aExtend);
   NS_IMETHOD WordMove(PRBool aForward, PRBool aExtend);
   NS_IMETHOD LineMove(PRBool aForward, PRBool aExtend);
@@ -507,7 +508,6 @@ NS_IMPL_ISUPPORTS3(nsTextInputSelectionImpl, nsISelectionController, nsISupports
 
 nsTextInputSelectionImpl::nsTextInputSelectionImpl(nsIFrameSelection *aSel, nsIPresShell *aShell, nsIContent *aLimiter)
 {
-  NS_INIT_ISUPPORTS();
   if (aSel && aShell)
   {
     mFrameSelection = aSel;//we are the owner now!
@@ -676,6 +676,27 @@ nsTextInputSelectionImpl::GetCaretEnabled(PRBool *_retval)
   return NS_ERROR_FAILURE;
 }
 
+NS_IMETHODIMP
+nsTextInputSelectionImpl::SetCaretVisibilityDuringSelection(PRBool aVisibility)
+{
+  if (!mPresShellWeak) return NS_ERROR_NOT_INITIALIZED;
+  nsresult result;
+  nsCOMPtr<nsIPresShell> shell = do_QueryReferent(mPresShellWeak, &result);
+  if (shell)
+  {
+    nsCOMPtr<nsICaret> caret;
+    if (NS_SUCCEEDED(result = shell->GetCaret(getter_AddRefs(caret))))
+    {
+      nsCOMPtr<nsISelection> domSel;
+      if (NS_SUCCEEDED(result = mFrameSelection->GetSelection(nsISelectionController::SELECTION_NORMAL, getter_AddRefs(domSel))))
+      {
+        return caret->SetVisibilityDuringSelection(aVisibility);
+      }
+    }
+
+  }
+  return NS_ERROR_FAILURE;
+}
 
 NS_IMETHODIMP
 nsTextInputSelectionImpl::CharacterMove(PRBool aForward, PRBool aExtend)
@@ -780,11 +801,11 @@ nsTextInputSelectionImpl::CompleteMove(PRBool aForward, PRBool aExtend)
     if (offset > 0)
     {
       nsCOMPtr<nsIContent> child;
-      result = parentDIV->ChildAt(offset - 1, *getter_AddRefs(child));
+      result = parentDIV->ChildAt(offset - 1, getter_AddRefs(child));
       if (NS_SUCCEEDED(result) && child)
       {
         nsCOMPtr<nsIAtom> tagName;
-        result = child->GetTag(*getter_AddRefs(tagName));
+        result = child->GetTag(getter_AddRefs(tagName));
         if (NS_SUCCEEDED(result) && tagName.get() == nsHTMLAtoms::br)
         {
           --offset;
@@ -811,7 +832,7 @@ nsTextInputSelectionImpl::ScrollPage(PRBool aForward)
   if (!scrollableView)
     return NS_ERROR_NOT_INITIALIZED;
 
-  return scrollableView->ScrollByPages(aForward ? 1 : -1);
+  return scrollableView->ScrollByPages(0, aForward ? 1 : -1);
 }
 
 NS_IMETHODIMP
@@ -1120,6 +1141,10 @@ nsTextControlFrame::QueryInterface(const nsIID& aIID, void** aInstancePtr)
     *aInstancePtr = (void*)(nsIScrollableViewProvider*) this;
     return NS_OK;
   }
+  if (aIID.Equals(NS_GET_IID(nsIPhonetic))) {
+    *aInstancePtr = (void*)(nsIPhonetic*) this;
+    return NS_OK;
+  }
 
   return nsBoxFrame::QueryInterface(aIID, aInstancePtr);
 }
@@ -1137,7 +1162,8 @@ NS_IMETHODIMP nsTextControlFrame::GetAccessible(nsIAccessible** aAccessible)
 }
 #endif
 
-nsTextControlFrame::nsTextControlFrame(nsIPresShell* aShell):nsStackFrame(aShell)
+nsTextControlFrame::nsTextControlFrame(nsIPresShell* aShell)
+  : nsStackFrame(aShell)
 {
   mUseEditor = PR_FALSE;
   mIsProcessing = PR_FALSE;
@@ -1252,7 +1278,7 @@ nsTextControlFrame::PreDestroy(nsIPresContext* aPresContext)
           nsCOMPtr<nsIControllerContext> editController = do_QueryInterface(controller);
           if (editController)
           {
-            editController->SetCommandRefCon(nsnull);
+            editController->SetCommandContext(nsnull);
           }
         }
       }
@@ -1307,12 +1333,8 @@ nsTextControlFrame::GetFrameType(nsIAtom** aType) const
 // XXX: wouldn't it be nice to get this from the style context!
 PRBool nsTextControlFrame::IsSingleLineTextControl() const
 {
-  PRInt32 type;
-  GetType(&type);
-  if ((NS_FORM_INPUT_TEXT==type) || (NS_FORM_INPUT_PASSWORD==type)) {
-    return PR_TRUE;
-  }
-  return PR_FALSE; 
+  PRInt32 type = GetType();
+  return (type == NS_FORM_INPUT_TEXT) || (type == NS_FORM_INPUT_PASSWORD);
 }
 
 PRBool nsTextControlFrame::IsTextArea() const
@@ -1321,7 +1343,7 @@ PRBool nsTextControlFrame::IsTextArea() const
     return PR_FALSE;
 
   nsCOMPtr<nsIAtom> tag;
-  mContent->GetTag(*getter_AddRefs(tag));
+  mContent->GetTag(getter_AddRefs(tag));
 
   if (nsHTMLAtoms::textarea == tag)
     return PR_TRUE;
@@ -1338,12 +1360,7 @@ PRBool nsTextControlFrame::IsPlainTextControl() const
 
 PRBool nsTextControlFrame::IsPasswordTextControl() const
 {
-  PRInt32 type;
-  GetType(&type);
-  if (NS_FORM_INPUT_PASSWORD==type) {
-    return PR_TRUE;
-  }
-  return PR_FALSE;
+  return GetType() == NS_FORM_INPUT_PASSWORD;
 }
 
 
@@ -1598,11 +1615,14 @@ nsTextControlFrame::InitEditor()
   return NS_OK;
 }
 
+// XXXldb I'm not sure if we really want the 'text-decoration: inherit',
+// but it's needed to make 'text-decoration' "work" on text inputs.
 #define DIV_STRING \
   "-moz-user-focus: none;" \
   "border: 0px !important;" \
   "padding: 0px;" \
   "margin: 0px;" \
+  "text-decoration: inherit;" \
   ""
 
 #define DIV_STRING_SINGLELINE \
@@ -1612,6 +1632,7 @@ nsTextControlFrame::InitEditor()
   "border: 0px !important;" \
   "padding: 0px;" \
   "margin: 0px;" \
+  "text-decoration: inherit;" \
   ""
 
 NS_IMETHODIMP
@@ -1655,7 +1676,7 @@ nsTextControlFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsINodeInfoManager> nodeInfoManager;
-  rv = doc->GetNodeInfoManager(*getter_AddRefs(nodeInfoManager));
+  rv = doc->GetNodeInfoManager(getter_AddRefs(nodeInfoManager));
 
   if (NS_FAILED(rv))
     return rv;
@@ -1665,7 +1686,7 @@ nsTextControlFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
   nsCOMPtr<nsINodeInfo> nodeInfo;
   rv = nodeInfoManager->GetNodeInfo(nsHTMLAtoms::div, nsnull,
                                     kNameSpaceID_XHTML,
-                                    *getter_AddRefs(nodeInfo));
+                                    getter_AddRefs(nodeInfo));
 
   if (NS_FAILED(rv))
     return rv;
@@ -1690,8 +1711,7 @@ nsTextControlFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
     rv = divContent->SetAttr(kNameSpaceID_None,nsHTMLAtoms::style, NS_ConvertASCIItoUCS2(DIV_STRING_SINGLELINE), PR_FALSE);
   else {
     nsAutoString divStr; divStr.AssignWithConversion(DIV_STRING);
-    const nsStyleDisplay* disp = (const nsStyleDisplay*)
-    mStyleContext->GetStyleData(eStyleStruct_Display);
+    const nsStyleDisplay* disp = GetStyleDisplay();
     if (disp->mOverflow == NS_STYLE_OVERFLOW_SCROLL)
       divStr += NS_LITERAL_STRING("overflow:scroll;");
     else if (disp->mOverflow == NS_STYLE_OVERFLOW_HIDDEN)
@@ -1807,7 +1827,7 @@ nsTextControlFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
           nsCOMPtr<nsIControllerContext> editController = do_QueryInterface(controller);
           if (editController)
           {
-            editController->SetCommandRefCon(mEditor);
+            editController->SetCommandContext(mEditor);
             found = PR_TRUE;
           }
         }
@@ -1939,14 +1959,13 @@ nsTextControlFrame::Reflow(nsIPresContext*   aPresContext,
 
   nsresult rv = nsStackFrame::Reflow(aPresContext, aDesiredSize, aReflowState, aStatus);
   if (NS_SUCCEEDED(rv))
-  { // fix for bug 40596, width:auto means the control sets it's maxElementSize.width to it's default width
-    if (aDesiredSize.maxElementSize)
+  { // fix for bug 40596, width:auto means the control sets it's mMaxElementWidth to it's default width
+    if (aDesiredSize.mComputeMEW)
     {
-      nsStylePosition *stylePosition;
-      GetStyleData(eStyleStruct_Position,  (const nsStyleStruct *&)stylePosition);
+      const nsStylePosition* stylePosition = GetStylePosition();
       nsStyleUnit widthUnit = stylePosition->mWidth.GetUnit();
       if (eStyleUnit_Auto == widthUnit) {
-        aDesiredSize.maxElementSize->width = aDesiredSize.width;
+        aDesiredSize.mMaxElementWidth = aDesiredSize.width;
       }
     }
   }
@@ -2091,10 +2110,10 @@ nsTextControlFrame::GetName(nsAString* aResult)
   return nsFormControlHelper::GetName(mContent, aResult);
 }
 
-NS_IMETHODIMP
-nsTextControlFrame::GetType(PRInt32* aType) const
+NS_IMETHODIMP_(PRInt32)
+nsTextControlFrame::GetType() const
 {
-  return nsFormControlHelper::GetType(mContent, aType);
+  return nsFormControlHelper::GetType(mContent);
 }
 
 void    nsTextControlFrame::SetFocus(PRBool aOn , PRBool aRepaint){}
@@ -2158,12 +2177,9 @@ nsTextControlFrame::GetFont(nsIPresContext* aPresContext,
 NS_IMETHODIMP
 nsTextControlFrame::GetFormContent(nsIContent*& aContent) const
 {
-  nsIContent* content;
-  nsresult    rv;
-
-  rv = GetContent(&content);
-  aContent = content;
-  return rv;
+  aContent = GetContent();
+  NS_IF_ADDREF(aContent);
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsTextControlFrame::SetProperty(nsIPresContext* aPresContext, nsIAtom* aName, const nsAString& aValue)
@@ -2189,7 +2205,14 @@ NS_IMETHODIMP nsTextControlFrame::SetProperty(nsIPresContext* aPresContext, nsIA
     }
     else if (nsHTMLAtoms::select == aName && mSelCon)
     {
-      // select all the text
+      // Select all the text.
+      //
+      // XXX: This is lame, we can't call mEditor->SelectAll()
+      //      because that triggers AutoCopies in unix builds.
+      //      Instead, we have to call our own homegrown version
+      //      of select all which merely builds a range that selects
+      //      all of the content and adds that to the selection.
+
       SelectAllContents();
     }
     mIsProcessing = PR_FALSE;
@@ -2239,178 +2262,117 @@ nsTextControlFrame::GetTextLength(PRInt32* aTextLength)
   return NS_OK;
 }
 
-
-
-NS_IMETHODIMP
-nsTextControlFrame::GetFirstTextNode(nsIDOMCharacterData* *aFirstTextNode)
+nsresult
+nsTextControlFrame::SetSelectionInternal(nsIDOMNode *aStartNode,
+                                         PRInt32 aStartOffset,
+                                         nsIDOMNode *aEndNode,
+                                         PRInt32 aEndOffset)
 {
-  if (!mEditor)
-    return NS_ERROR_NOT_INITIALIZED;
-  nsCOMPtr<nsIDOMElement> rootElement;
-  mEditor->GetRootElement(getter_AddRefs(rootElement));
-  *aFirstTextNode = nsnull;
-  
-  nsCOMPtr<nsIDOMNode> rootNode = do_QueryInterface(rootElement);
-  if (!rootNode) return NS_ERROR_FAILURE;
-  
-  // for a text widget, the text of the document is in a single
-  // text node under the body. Let's make sure that's true.
-  nsCOMPtr<nsIDOMNodeList> childNodesList;
-  rootNode->GetChildNodes(getter_AddRefs(childNodesList));
-  if (!childNodesList)
-  {
-    NS_WARNING("rootNode has no text node list");
-    return NS_ERROR_FAILURE;
-  }
+  // Create a new range to represent the new selection.
+  // Note that we use a new range to avoid having to do
+  // isIncreasing checks to avoid possible errors.
 
-  PRUint32 numChildNodes = 0;
-  childNodesList->GetLength(&numChildNodes);
+  nsCOMPtr<nsIDOMRange> range = do_CreateInstance(kRangeCID);
+  NS_ENSURE_TRUE(range, NS_ERROR_FAILURE);
 
-  nsCOMPtr<nsIDOMNode> firstChild;
-  nsresult rv = rootNode->GetFirstChild(getter_AddRefs(firstChild));
-  if (NS_FAILED(rv)) return rv;
-  if (!firstChild) return NS_ERROR_FAILURE;
+  nsresult rv = range->SetStart(aStartNode, aStartOffset);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIDOMCharacterData> charDataNode = do_QueryInterface(firstChild, &rv);
-  if (NS_FAILED(rv)) return rv;
-  
-  NS_ADDREF(*aFirstTextNode = charDataNode);
-  return NS_OK;
+  rv = range->SetEnd(aEndNode, aEndOffset);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Get the selection, clear it and add the new range to it!
+
+  nsCOMPtr<nsISelection> selection;
+  mTextSelImpl->GetSelection(nsISelectionController::SELECTION_NORMAL, getter_AddRefs(selection));  
+  NS_ENSURE_TRUE(selection, NS_ERROR_FAILURE);
+
+  rv = selection->RemoveAllRanges();  
+
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return selection->AddRange(range);
 }
 
-
 nsresult
 nsTextControlFrame::SelectAllContents()
 {
-  nsresult rv;
-  
-  if (IsSingleLineTextControl())
-  {
-    rv = SetSelectionRange(0, eSelectToEnd);
-  }
-  else
-  {
-    // we have to select all
-    if (!mEditor)
-      return NS_ERROR_NOT_INITIALIZED;
-    NS_ASSERTION(mEditor, "Should have an editor here");    
-    rv = mEditor->SelectAll();
+  if (!mEditor)
+    return NS_OK;
+
+  nsCOMPtr<nsIDOMElement> rootElement;
+  nsresult rv = mEditor->GetRootElement(getter_AddRefs(rootElement));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIContent> rootContent = do_QueryInterface(rootElement);
+  PRInt32 numChildren = 0;
+  rv = rootContent->ChildCount(numChildren);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (numChildren > 0) {
+    // We never want to place the selection after the last
+    // br under the root node!
+    nsCOMPtr<nsIContent> child;
+    rv = rootContent->ChildAt(numChildren - 1, getter_AddRefs(child));
+    if (NS_SUCCEEDED(rv) && child) {
+      nsCOMPtr<nsIAtom> tagName;
+      rv = child->GetTag(getter_AddRefs(tagName));
+      if (NS_SUCCEEDED(rv) && tagName == nsHTMLAtoms::br)
+        --numChildren;
+    }
   }
 
-  return rv;
+  nsCOMPtr<nsIDOMNode> rootNode(do_QueryInterface(rootElement));
+
+  return SetSelectionInternal(rootNode, 0, rootNode, numChildren);
 }
 
-
 nsresult
 nsTextControlFrame::SetSelectionEndPoints(PRInt32 aSelStart, PRInt32 aSelEnd)
 {
-  NS_ASSERTION(IsSingleLineTextControl() || IsTextArea(), "Should only call this on a single line input");
-  NS_ASSERTION(mEditor, "Should have an editor here");
-  NS_ASSERTION(mTextSelImpl,"selection not found!");
+  NS_ASSERTION(aSelStart <= aSelEnd, "Invalid selection offsets!");
 
-  nsCOMPtr<nsIDOMCharacterData> firstTextNode;
-  nsresult rv = GetFirstTextNode(getter_AddRefs(firstTextNode));
-  if (NS_FAILED(rv) || !firstTextNode)
-  {
-    // probably an empty document. not an error
-    return NS_OK;
+  if (aSelStart > aSelEnd)
+    return NS_ERROR_FAILURE;
+
+  nsCOMPtr<nsIDOMNode> startNode, endNode;
+  PRInt32 startOffset, endOffset;
+
+  // Calculate the selection start point.
+
+  nsresult rv = OffsetToDOMPoint(aSelStart, getter_AddRefs(startNode), &startOffset);
+
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (aSelStart == aSelEnd) {
+    // Collapsed selection, so start and end are the same!
+    endNode   = startNode;
+    endOffset = startOffset;
   }
-  
-  nsCOMPtr<nsIDOMNode> firstNode = do_QueryInterface(firstTextNode, &rv);
-  if (!firstNode) return rv;
-  
-  // constrain the selection to this node
-  PRUint32 nodeLengthU;
-  firstTextNode->GetLength(&nodeLengthU);
-  PRInt32 nodeLength = (PRInt32)nodeLengthU;
-    
-  nsCOMPtr<nsISelection> selection;
-  mTextSelImpl->GetSelection(nsISelectionController::SELECTION_NORMAL,getter_AddRefs(selection));  
-  if (!selection) return NS_ERROR_FAILURE;
+  else {
+    // Selection isn't collapsed so we have to calculate
+    // the end point too.
 
-  // are we setting both start and end?
-  if (aSelStart != eIgnoreSelect && aSelEnd != eIgnoreSelect)
-  {
-    if (aSelStart == eSelectToEnd || aSelStart > nodeLength)
-      aSelStart = nodeLength;
-    if (aSelStart < 0)
-      aSelStart = 0;
+    rv = OffsetToDOMPoint(aSelEnd, getter_AddRefs(endNode), &endOffset);
 
-    if (aSelEnd == eSelectToEnd || aSelEnd > nodeLength)
-      aSelEnd = nodeLength;
-    if (aSelEnd < 0)
-      aSelEnd = 0;
-
-    // remove existing ranges
-    selection->RemoveAllRanges();  
-
-    nsCOMPtr<nsIDOMRange> selectionRange(do_CreateInstance(kRangeCID,&rv));
-    if (NS_FAILED(rv)) 
-      return rv;
-    
-    selectionRange->SetStart(firstTextNode, aSelStart);
-    selectionRange->SetEnd(firstTextNode, aSelEnd);
-    
-    selection->AddRange(selectionRange);
-  }
-  else    // we're setting either start or end but not both
-  {
-    // does a range exist?
-    nsCOMPtr<nsIDOMRange> firstRange;
-    selection->GetRangeAt(0, getter_AddRefs(firstRange));
-    PRBool mustAdd = PR_FALSE;
-    PRInt32 selStart = 0, selEnd = 0;
-
-    if (firstRange)
-    {
-     firstRange->GetStartOffset(&selStart);
-     firstRange->GetEndOffset(&selEnd);
-    }
-    else
-    {
-      // no range. Make a new one.
-      firstRange = do_CreateInstance(kRangeCID,&rv);
-      if (NS_FAILED(rv)) 
-        return rv;
-      mustAdd = PR_TRUE;
-    }
-    
-    if (aSelStart == eSelectToEnd)
-      selStart = nodeLength;
-    else if (aSelStart != eIgnoreSelect)
-      selStart = aSelStart;
-
-    if (aSelEnd == eSelectToEnd)
-      selEnd = nodeLength;
-    else if (aSelEnd != eIgnoreSelect)
-      selEnd = aSelEnd;
-    
-    // swap them
-    if (selEnd < selStart)
-    {
-      PRInt32 temp = selStart;
-      selStart = selEnd;
-      selEnd = temp;
-    }
-    
-    firstRange->SetStart(firstTextNode, selStart);
-    firstRange->SetEnd(firstTextNode, selEnd);
-    if (mustAdd)  
-      selection->AddRange(firstRange);
+    NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  return NS_OK;
+  return SetSelectionInternal(startNode, startOffset, endNode, endOffset);
 }
 
 NS_IMETHODIMP
 nsTextControlFrame::SetSelectionRange(PRInt32 aSelStart, PRInt32 aSelEnd)
 {
-  if (!IsSingleLineTextControl()) return NS_ERROR_NOT_IMPLEMENTED;
-  
-  // make sure we have an editor
-  if (!mEditor) 
-    return NS_ERROR_NOT_INITIALIZED;
+  NS_ENSURE_TRUE(mEditor, NS_ERROR_NOT_INITIALIZED);
   
+  if (aSelStart > aSelEnd) {
+    // Simulate what we'd see SetSelectionStart() was called, followed
+    // by a SetSelectionEnd().
+
+    aSelStart   = aSelEnd;
+  }
+
   return SetSelectionEndPoints(aSelStart, aSelEnd);
 }
 
@@ -2418,122 +2380,269 @@ nsTextControlFrame::SetSelectionRange(PRInt32 aSelStart, PRInt32 aSelEnd)
 NS_IMETHODIMP
 nsTextControlFrame::SetSelectionStart(PRInt32 aSelectionStart)
 {
-  if (!IsSingleLineTextControl() && !IsTextArea()) return NS_ERROR_NOT_IMPLEMENTED;
+  NS_ENSURE_TRUE(mEditor, NS_ERROR_NOT_INITIALIZED);
 
-  // make sure we have an editor
-  if (!mEditor) 
-    return NS_ERROR_NOT_INITIALIZED;
+  PRInt32 selStart = 0, selEnd = 0; 
+
+  nsresult rv = GetSelectionRange(&selStart, &selEnd);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (aSelectionStart > selEnd) {
+    // Collapse to the new start point.
+    selEnd = aSelectionStart; 
+  }
+
+  selStart = aSelectionStart;
   
-  return SetSelectionEndPoints(aSelectionStart, eIgnoreSelect);
+  return SetSelectionEndPoints(selStart, selEnd);
 }
 
 NS_IMETHODIMP
 nsTextControlFrame::SetSelectionEnd(PRInt32 aSelectionEnd)
 {
-  if (!IsSingleLineTextControl() && !IsTextArea()) return NS_ERROR_NOT_IMPLEMENTED;
-
-  // make sure we have an editor
-  if (!mEditor) 
-    return NS_ERROR_NOT_INITIALIZED;
+  NS_ENSURE_TRUE(mEditor, NS_ERROR_NOT_INITIALIZED);
   
-  return SetSelectionEndPoints(eIgnoreSelect, aSelectionEnd);
+  PRInt32 selStart = 0, selEnd = 0; 
+
+  nsresult rv = GetSelectionRange(&selStart, &selEnd);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (aSelectionEnd < selStart) {
+    // Collapse to the new end point.
+    selStart = aSelectionEnd; 
+  }
+
+  selEnd = aSelectionEnd;
+  
+  return SetSelectionEndPoints(selStart, selEnd);
 }
 
+nsresult
+nsTextControlFrame::DOMPointToOffset(nsIDOMNode* aNode,
+                                     PRInt32 aNodeOffset,
+                                     PRInt32* aResult)
+{
+  NS_ENSURE_ARG_POINTER(aNode && aResult);
+
+  *aResult = 0;
+
+  nsCOMPtr<nsIDOMElement> rootElement;
+  mEditor->GetRootElement(getter_AddRefs(rootElement));
+  nsCOMPtr<nsIDOMNode> rootNode(do_QueryInterface(rootElement));
+
+  NS_ENSURE_TRUE(rootNode, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsIDOMNodeList> nodeList;
+
+  nsresult rv = rootNode->GetChildNodes(getter_AddRefs(nodeList));
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(nodeList, NS_ERROR_FAILURE);
+
+  PRUint32 length = 0;
+  rv = nodeList->GetLength(&length);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (!length || aNodeOffset < 0)
+    return NS_OK;
+
+  PRInt32 i, textOffset = 0;
+  PRInt32 lastIndex = (PRInt32)length - 1;
+
+  for (i = 0; i < (PRInt32)length; i++) {
+    if (rootNode == aNode && i == aNodeOffset) {
+      *aResult = textOffset;
+      return NS_OK;
+    }
+
+    nsCOMPtr<nsIDOMNode> item;
+    rv = nodeList->Item(i, getter_AddRefs(item));
+    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ENSURE_TRUE(item, NS_ERROR_FAILURE);
+
+    nsCOMPtr<nsIDOMText> domText(do_QueryInterface(item));
+
+    if (domText) {
+      PRUint32 textLength = 0;
+
+      rv = domText->GetLength(&textLength);
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      if (item == aNode) {
+        NS_ASSERTION((aNodeOffset >= 0 && aNodeOffset <= (PRInt32)textLength),
+                     "Invalid aNodeOffset!");
+        *aResult = textOffset + aNodeOffset;
+        return NS_OK;
+      }
+
+      textOffset += textLength;
+    }
+    else {
+      // Must be a BR node. If it's not the last BR node
+      // under the root, count it as a newline.
+
+      if (i != lastIndex)
+        ++textOffset;
+    }
+  }
+
+  NS_ASSERTION((aNode == rootNode && aNodeOffset == (PRInt32)length),
+               "Invalide node offset!");
+
+  *aResult = textOffset;
+  
+  return NS_OK;
+}
+
+nsresult
+nsTextControlFrame::OffsetToDOMPoint(PRInt32 aOffset,
+                                     nsIDOMNode** aResult,
+                                     PRInt32* aPosition)
+{
+  NS_ENSURE_ARG_POINTER(aResult && aPosition);
+
+  *aResult = nsnull;
+  *aPosition = 0;
+
+  nsCOMPtr<nsIDOMElement> rootElement;
+  mEditor->GetRootElement(getter_AddRefs(rootElement));
+  nsCOMPtr<nsIDOMNode> rootNode(do_QueryInterface(rootElement));
+
+  NS_ENSURE_TRUE(rootNode, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsIDOMNodeList> nodeList;
+
+  nsresult rv = rootNode->GetChildNodes(getter_AddRefs(nodeList));
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(nodeList, NS_ERROR_FAILURE);
+
+  PRUint32 length = 0;
+
+  rv = nodeList->GetLength(&length);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (!length || aOffset < 0) {
+    *aPosition = 0;
+    *aResult = rootNode;
+    NS_ADDREF(*aResult);
+    return NS_OK;
+  }
+
+  PRInt32 textOffset = 0;
+  PRUint32 lastIndex = length - 1;
+
+  for (PRUint32 i=0; i<length; i++) {
+    nsCOMPtr<nsIDOMNode> item;
+    rv = nodeList->Item(i, getter_AddRefs(item));
+    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ENSURE_TRUE(item, NS_ERROR_FAILURE);
+
+    nsCOMPtr<nsIDOMText> domText(do_QueryInterface(item));
+
+    if (domText) {
+      PRUint32 textLength = 0;
+
+      rv = domText->GetLength(&textLength);
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      // Check if aOffset falls within this range.
+      if (aOffset >= textOffset && aOffset <= textOffset+(PRInt32)textLength) {
+        *aPosition = aOffset - textOffset;
+        *aResult = item;
+        NS_ADDREF(*aResult);
+        return NS_OK;
+      }
+
+      textOffset += textLength;
+
+      // If there aren't any more siblings after this text node,
+      // return the point at the end of this text node!
+
+      if (i == lastIndex) {
+        *aPosition = textLength;
+        *aResult = item;
+        NS_ADDREF(*aResult);
+        return NS_OK;
+      }
+    }
+    else {
+      // Must be a BR node, count it as a newline.
+
+      if (aOffset == textOffset || i == lastIndex) {
+        // We've found the correct position, or aOffset takes us
+        // beyond the last child under rootNode, just return the point
+        // under rootNode that is in front of this br.
+
+        *aPosition = i;
+        *aResult = rootNode;
+        NS_ADDREF(*aResult);
+        return NS_OK;
+      }
+
+      ++textOffset;
+    }
+  }
+
+  NS_ASSERTION(0, "We should never get here!");
+
+  return NS_ERROR_FAILURE;
+}
 
 NS_IMETHODIMP
 nsTextControlFrame::GetSelectionRange(PRInt32* aSelectionStart, PRInt32* aSelectionEnd)
 {
-    NS_ENSURE_ARG_POINTER((aSelectionStart && aSelectionEnd));
+  // make sure we have an editor
+  NS_ENSURE_TRUE(mEditor, NS_ERROR_NOT_INITIALIZED);
 
-    // make sure we have an editor
-    if (!mEditor) 
-      return NS_ERROR_NOT_INITIALIZED;
+  *aSelectionStart = 0;
+  *aSelectionEnd = 0;
 
-    *aSelectionStart = 0;
-    *aSelectionEnd = 0;
+  nsCOMPtr<nsISelection> selection;
+  nsresult rv = mTextSelImpl->GetSelection(nsISelectionController::SELECTION_NORMAL, getter_AddRefs(selection));  
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(selection, NS_ERROR_FAILURE);
 
-    nsCOMPtr<nsISelection> selection;
-    mTextSelImpl->GetSelection(nsISelectionController::SELECTION_NORMAL,getter_AddRefs(selection));  
-    if (!selection) return NS_ERROR_FAILURE;
+  PRInt32 numRanges = 0;
+  selection->GetRangeCount(&numRanges);
 
-    // we should have only zero or one range
-    PRInt32 numRanges = 0;
-    selection->GetRangeCount(&numRanges);
-    if (numRanges > 1)
-    {
-      NS_ASSERTION(0, "Found more than on range in GetSelectionRange");
-    }
-  
-    if (numRanges != 0)
-    {
-      nsCOMPtr<nsIDOMRange> firstRange;
-      selection->GetRangeAt(0, getter_AddRefs(firstRange));
-      if (!firstRange) 
-        return NS_ERROR_FAILURE;
-
-      if (IsSingleLineTextControl() || IsTextArea())
-      {
-        firstRange->GetStartOffset(aSelectionStart);
-        firstRange->GetEndOffset(aSelectionEnd);
-      }
-      else//multiline
-      {
-        //mContent = parent. iterate over each child. 
-        //when text nodes are reached add text length. 
-        //if you find range-startoffset,startnode then mark aSelecitonStart
-        nsresult rv = NS_ERROR_FAILURE;
-        nsCOMPtr<nsIDOMNode> contentNode;
-        nsCOMPtr<nsIDOMNode> curNode;
-        contentNode = do_QueryInterface(mContent);
-        if (!contentNode || NS_FAILED(rv = contentNode->GetFirstChild(getter_AddRefs(curNode))) || !curNode)
-          return rv;
-        nsCOMPtr<nsIDOMNode> startParent;
-        nsCOMPtr<nsIDOMNode> endParent;
-        PRInt32 startOffset;
-        PRInt32 endOffset;
-
-        firstRange->GetStartContainer(getter_AddRefs(startParent));
-        firstRange->GetStartOffset(&startOffset);
-        firstRange->GetEndContainer(getter_AddRefs(endParent));
-        firstRange->GetEndOffset(&endOffset);
-
-        PRInt32 currentTextOffset = 0;
-        
-        while(curNode)
-        {
-          nsCOMPtr<nsIDOMText> domText;
-          domText = do_QueryInterface(curNode);
-          if (contentNode == startParent)
-          {
-            if (domText)
-              *aSelectionStart = currentTextOffset + startOffset;
-            else
-              *aSelectionStart = currentTextOffset;
-          }
-          if (curNode == endParent)
-          {
-            if (domText)
-              *aSelectionEnd = currentTextOffset + endOffset;
-            else
-              *aSelectionEnd = currentTextOffset;
-            break;
-          }
-          if (domText)
-          {
-            PRUint32 length;
-            if (NS_SUCCEEDED(domText->GetLength(&length)))
-              currentTextOffset += length;
-          }
-          else
-            ++currentTextOffset;
-        }
-        if (!curNode) //something went very wrong...
-        {
-          *aSelectionEnd = *aSelectionStart;//couldnt find the end
-        }
-      }
-    }
+  if (numRanges < 1)
     return NS_OK;
+
+  // We only operate on the first range in the selection!
+
+  nsCOMPtr<nsIDOMRange> firstRange;
+  rv = selection->GetRangeAt(0, getter_AddRefs(firstRange));
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(firstRange, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsIDOMNode> startNode, endNode;
+  PRInt32 startOffset = 0, endOffset = 0;
+
+  // Get the start point of the range.
+
+  rv = firstRange->GetStartContainer(getter_AddRefs(startNode));
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(startNode, NS_ERROR_FAILURE);
+
+  rv = firstRange->GetStartOffset(&startOffset);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Get the end point of the range.
+
+  rv = firstRange->GetEndContainer(getter_AddRefs(endNode));
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(endNode, NS_ERROR_FAILURE);
+
+  rv = firstRange->GetEndOffset(&endOffset);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Convert the start point to a selection offset.
+
+  rv = DOMPointToOffset(startNode, startOffset, aSelectionStart);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Convert the end point to a selection offset.
+
+  return DOMPointToOffset(endNode, endOffset, aSelectionEnd);
 }
 
 
@@ -2680,6 +2789,20 @@ nsTextControlFrame::GetText(nsString* aText)
 }
 
 
+NS_IMETHODIMP
+nsTextControlFrame::GetPhonetic(nsAString& aPhonetic)
+{
+  aPhonetic.Truncate(0); 
+  if (!mEditor)
+    return NS_ERROR_NOT_INITIALIZED;
+  nsCOMPtr<nsIEditorIMESupport> imeSupport = do_QueryInterface(mEditor);
+  if (imeSupport) {
+    nsCOMPtr<nsIPhonetic> phonetic = do_QueryInterface(imeSupport);
+    if (phonetic)
+      phonetic->GetPhonetic(aPhonetic);
+  }
+  return NS_OK;
+}
 
 ///END NSIFRAME OVERLOADS
 /////BEGIN PROTECTED METHODS
@@ -2940,7 +3063,7 @@ nsTextControlFrame::SetValue(const nsAString& aValue)
     nsCOMPtr<nsITextControlElement> textControl = do_QueryInterface(mContent);
     if (textControl)
     {
-      textControl->SetValueGuaranteed(aValue, this);
+      textControl->TakeTextFrameValue(aValue);
     }
   }
 }
@@ -2952,13 +3075,10 @@ nsTextControlFrame::SetInitialChildList(nsIPresContext* aPresContext,
                                   nsIFrame*       aChildList)
 {
   /*nsIFrame *list = aChildList;
-  nsFrameState  frameState;
   while (list)
   {
-    list->GetFrameState(&frameState);
-    frameState |= NS_FRAME_INDEPENDENT_SELECTION;
-    list->SetFrameState(frameState);
-    list->GetNextSibling(&list);
+    list->AddStateBits(NS_FRAME_INDEPENDENT_SELECTION);
+    list = list->GetNextSibling();
   }
   */
   nsresult rv = nsBoxFrame::SetInitialChildList(aPresContext, aListName, aChildList);
@@ -2971,15 +3091,10 @@ nsTextControlFrame::SetInitialChildList(nsIPresContext* aPresContext,
   // Mark the scroll frame as being a reflow root. This will allow
   // incremental reflows to be initiated at the scroll frame, rather
   // than descending from the root frame of the frame hierarchy.
-  nsFrameState state;
-  first->GetFrameState(&state);
-  state |= NS_FRAME_REFLOW_ROOT;
-  first->SetFrameState(state);
+  first->AddStateBits(NS_FRAME_REFLOW_ROOT);
 
 //we must turn off scrollbars for singleline text controls
-  PRInt32 type;
-  GetType(&type);
-  if ((NS_FORM_INPUT_TEXT == type) || (NS_FORM_INPUT_PASSWORD == type)) 
+  if (IsSingleLineTextControl()) 
   {
     nsIScrollableFrame *scrollableFrame = nsnull;
     if (first)
@@ -3005,13 +3120,11 @@ nsTextControlFrame::SetInitialChildList(nsIPresContext* aPresContext,
 
   while(first)
   {
-    nsIScrollableView *scrollView;
-    nsIView *view;
-    first->GetView(aPresContext,&view);
+    nsIView *view = first->GetView();
     if (view)
     {
-      view->QueryInterface(NS_GET_IID(nsIScrollableView),(void **)&scrollView);
-      if (scrollView)
+      nsIScrollableView *scrollView;
+      if (NS_SUCCEEDED(CallQueryInterface(view, &scrollView)))
       {
         mScrollableView = scrollView; // Note: views are not addref'd
         mTextSelImpl->SetScrollableView(scrollView);
@@ -3053,7 +3166,8 @@ nsTextControlFrame::GetWidthInCharacters() const
 }
 
 NS_IMETHODIMP
-nsTextControlFrame::GetScrollableView(nsIScrollableView** aView)
+nsTextControlFrame::GetScrollableView(nsIPresContext* aPresContext,
+                                      nsIScrollableView** aView)
 {
   nsresult rv = NS_OK;
   *aView = mScrollableView;
@@ -3065,7 +3179,7 @@ nsTextControlFrame::GetScrollableView(nsIScrollableView** aView)
       rv = view->QueryInterface(NS_GET_IID(nsIScrollableView), (void **)&scrollableView);
       if (NS_SUCCEEDED(rv) && scrollableView)
         *aView = scrollableView;
-      view->GetParent(view);
+      view = view->GetParent();
     }
   }
   return rv;
@@ -3109,5 +3223,3 @@ nsTextControlFrame::HandleEvent(nsIPresContext* aPresContext,
   return nsStackFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
     
 }
-
-
