@@ -38,7 +38,7 @@
 #include "nsTableOuterFrame.h"
 #include "nsTableFrame.h"
 #include "nsHTMLReflowCommand.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsIRenderingContext.h"
@@ -65,7 +65,7 @@
 nsTableCaptionFrame::nsTableCaptionFrame()
 {
   // shrink wrap 
-  SetFlags(NS_BLOCK_SPACE_MGR|NS_BLOCK_WRAP_SIZE);
+  SetFlags(NS_BLOCK_SPACE_MGR);
 }
 
 nsTableCaptionFrame::~nsTableCaptionFrame()
@@ -166,7 +166,7 @@ NS_IMETHODIMP
 nsTableOuterFrame::Init(nsIPresContext*  aPresContext,
                    nsIContent*           aContent,
                    nsIFrame*             aParent,
-                   nsIStyleContext*      aContext,
+                   nsStyleContext*       aContext,
                    nsIFrame*             aPrevInFlow)
 {
   nsresult rv = nsHTMLContainerFrame::Init(aPresContext, aContent, aParent, 
@@ -300,10 +300,7 @@ nsTableOuterFrame::RemoveFrame(nsIPresContext* aPresContext,
       (NS_SIDE_LEFT == captionSide) || (NS_SIDE_RIGHT == captionSide)) {
     // The old caption width had an effect on the inner table width so
     // we're going to need to reflow it. Mark it dirty
-    nsFrameState  frameState;
-    mInnerTableFrame->GetFrameState(&frameState);
-    frameState |= NS_FRAME_IS_DIRTY;
-    mInnerTableFrame->SetFrameState(frameState);
+    mInnerTableFrame->AddStateBits(NS_FRAME_IS_DIRTY);
   }
 
   // Remove the caption frame and destroy it
@@ -345,8 +342,7 @@ nsTableOuterFrame::Paint(nsIPresContext*      aPresContext,
   // the remaining code was copied from nsContainerFrame::PaintChildren since
   // it only paints the primary child list
 
-  const nsStyleDisplay* disp = (const nsStyleDisplay*)
-    mStyleContext->GetStyleData(eStyleStruct_Display);
+  const nsStyleDisplay* disp = GetStyleDisplay();
 
   // Child elements have the opportunity to override the visibility property
   // of their parent and display even if the parent is hidden
@@ -361,10 +357,8 @@ nsTableOuterFrame::Paint(nsIPresContext*      aPresContext,
   if (mCaptionFrame) {
     PaintChild(aPresContext, aRenderingContext, aDirtyRect, mCaptionFrame, aWhichLayer);
   }
-  nsIFrame* kid = mFrames.FirstChild();
-  while (nsnull != kid) {
+  for (nsIFrame* kid = mFrames.FirstChild(); kid; kid = kid->GetNextSibling()) {
     PaintChild(aPresContext, aRenderingContext, aDirtyRect, kid, aWhichLayer);
-    kid->GetNextSibling(&kid);
   }
 
   if (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow) {
@@ -390,8 +384,9 @@ nsTableOuterFrame::GetFrameForPoint(nsIPresContext* aPresContext,
       return NS_OK;
     }
   }
-  // this should act like a block, so we need to override
-  return GetFrameForPointUsing(aPresContext, aPoint, nsnull, aWhichLayer, (aWhichLayer == NS_FRAME_PAINT_LAYER_BACKGROUND), aFrame);
+  // This frame should never get events (it contains the margins of the
+  // table), so always pass |PR_FALSE| for |aConsiderSelf|.
+  return GetFrameForPointUsing(aPresContext, aPoint, nsnull, aWhichLayer, PR_FALSE, aFrame);
 }
 
 NS_IMETHODIMP nsTableOuterFrame::SetSelected(nsIPresContext* aPresContext,
@@ -414,12 +409,11 @@ nsTableOuterFrame::GetParentStyleContextFrame(nsIPresContext* aPresContext,
   // data by giving the table frame the style context associated with
   // the table content node and creating a style context for the outer
   // frame that is a *child* of the table frame's style context,
-  // matching the :table-outer (should be :-moz-table-outer!)
-  // pseudo-element.  html.css has a rule that causes that
-  // pseudo-element (and thus the outer table) to inherit *some* style
-  // properties from the table frame.  The children of the table inherit
-  // directly from the inner table, and the outer table's style context
-  // is a leaf.
+  // matching the ::-moz-table-outer pseudo-element.  html.css has a
+  // rule that causes that pseudo-element (and thus the outer table)
+  // to inherit *some* style properties from the table frame.  The
+  // children of the table inherit directly from the inner table, and
+  // the outer table's style context is a leaf.
 
   if (!mInnerTableFrame) {
     *aProviderFrame = this;
@@ -433,15 +427,6 @@ nsTableOuterFrame::GetParentStyleContextFrame(nsIPresContext* aPresContext,
 
 // INCREMENTAL REFLOW HELPER FUNCTIONS 
 
-nsSize
-GetFrameSize(nsIFrame& aFrame) 
-{
-  nsRect rect;
-  aFrame.GetRect(rect);
-  nsSize size(rect.width, rect.height);
-  return size;
-}
-
 void
 nsTableOuterFrame::ZeroAutoMargin(nsHTMLReflowState& aReflowState,
                                   nsMargin&          aMargin)
@@ -512,9 +497,7 @@ nsTableOuterFrame::GetMarginPadding(nsIPresContext*          aPresContext,
                             eReflowReason_Resize, PR_FALSE);
   InitChildReflowState(*aPresContext, childRS);
 
-  nsRect childRect;
-  aChildFrame->GetRect(childRect);
-  FixAutoMargins(aAvailWidth, childRect.width, childRS); 
+  FixAutoMargins(aAvailWidth, aChildFrame->GetSize().width, childRS); 
   aMargin = childRS.mComputedMargin;
   aMarginNoAuto = aMargin;
   nsTableOuterFrame::ZeroAutoMargin(childRS, aMarginNoAuto);
@@ -543,10 +526,9 @@ MoveFrameTo(nsIPresContext* aPresContext,
             nscoord         aX,
             nscoord         aY)
 {
-  nsRect oldRect;
-  aFrame->GetRect(oldRect);
-  if ((oldRect.x != aX) || (oldRect.y != aY)) {
-    aFrame->MoveTo(aPresContext, aX, aY);
+  nsPoint pt = aFrame->GetPosition();
+  if ((pt.x != aX) || (pt.y != aY)) {
+    aFrame->SetPosition(nsPoint(aX, aY));
     nsTableFrame::RePositionViews(aPresContext, aFrame);
   }
 }
@@ -585,10 +567,10 @@ nsTableOuterFrame::InvalidateDamage(nsIPresContext* aPresContext,
     damage = nsRect(0, 0, aOuterSize.width, aOuterSize.height);
   }
   else {
-    nsRect innerRect, captionRect(0,0,0,0);
-    mInnerTableFrame->GetRect(innerRect);
+    nsRect captionRect(0,0,0,0);
+    nsRect innerRect = mInnerTableFrame->GetRect();
     if (mCaptionFrame) {
-      mCaptionFrame->GetRect(captionRect);
+      captionRect = mCaptionFrame->GetRect();
     }
     // only works for vertical captions {
     damage.x = 0;
@@ -683,8 +665,7 @@ nsTableOuterFrame::GetCaptionAvailWidth(nsIPresContext*          aPresContext,
     }
   }
   else {
-    nsSize outerSize = GetFrameSize(*this);
-    availWidth = outerSize.width;
+    availWidth = GetSize().width;
   }
 
   if (NS_UNCONSTRAINEDSIZE == availWidth) {
@@ -719,8 +700,7 @@ nsTableOuterFrame::GetInnerTableAvailWidth(nsIPresContext*          aPresContext
     }
   }
   else {
-    nsSize outerSize = GetFrameSize(*this);
-    availWidth = outerSize.width;
+    availWidth = GetSize().width;
   }
 
   if (NS_UNCONSTRAINEDSIZE == availWidth) {
@@ -750,37 +730,35 @@ nsTableOuterFrame::GetInnerTableAvailWidth(nsIPresContext*          aPresContext
   }
 }
 
-nsSize
-nsTableOuterFrame::GetMaxElementSize(PRUint8         aCaptionSide,
-                                     const nsMargin& aInnerMargin,
-                                     const nsMargin& aInnerPadding,
-                                     const nsMargin& aCaptionMargin)
+nscoord
+nsTableOuterFrame::GetMaxElementWidth(PRUint8         aCaptionSide,
+                                      const nsMargin& aInnerMargin,
+                                      const nsMargin& aInnerPadding,
+                                      const nsMargin& aCaptionMargin)
 {
-  nsSize size(0,0);
-  size.width += aInnerMargin.left + 
-                ((nsTableFrame *)mInnerTableFrame)->GetMinWidth() + 
-                aInnerMargin.right;
+  nscoord width = aInnerMargin.left + 
+                  ((nsTableFrame *)mInnerTableFrame)->GetMinWidth() + 
+                  aInnerMargin.right;
   if (mCaptionFrame) { 
     nscoord capWidth = mMinCaptionWidth + aCaptionMargin.left + aCaptionMargin.right;
     switch(aCaptionSide) {
     case NS_SIDE_LEFT:
       if (capWidth > aInnerMargin.left) {
-        size.width += capWidth - aInnerMargin.left;
+        width += capWidth - aInnerMargin.left;
       }
       break;
     case NS_SIDE_RIGHT:
       if(capWidth > aInnerMargin.right) {
-        size.width += capWidth - aInnerMargin.right;
+        width += capWidth - aInnerMargin.right;
       }
       break;
     default:
-      if (capWidth > size.width) {
-        size.width = capWidth;
+      if (capWidth > width) {
+        width = capWidth;
       }
     }
   }
-  // leave max element height set to 0. It is not used for anything
-  return size;
+  return width;
 }
 
 nscoord
@@ -793,13 +771,12 @@ nsTableOuterFrame::GetMaxWidth(PRUint8         aCaptionSide,
   maxWidth = ((nsTableFrame *)mInnerTableFrame)->GetPreferredWidth() + 
                aInnerMargin.left + aInnerMargin.right;
   if (mCaptionFrame) {
-    nsSize captionSize = GetFrameSize(*mCaptionFrame);
     switch(aCaptionSide) {
     case NS_SIDE_LEFT:
     case NS_SIDE_RIGHT:
       // the caption plus it margins should cover the corresponding inner table side
       // margin - don't count it twice.
-      maxWidth = captionSize.width + aCaptionMargin.left + aCaptionMargin.right +
+      maxWidth = mCaptionFrame->GetSize().width + aCaptionMargin.left + aCaptionMargin.right +
                  ((nsTableFrame *)mInnerTableFrame)->GetPreferredWidth();
       maxWidth += (NS_SIDE_LEFT == aCaptionSide) ? aInnerMargin.right : aInnerMargin.left;
       break;
@@ -816,10 +793,8 @@ nsTableOuterFrame::GetMaxWidth(PRUint8         aCaptionSide,
 PRUint8
 nsTableOuterFrame::GetCaptionSide()
 {
-  const nsStyleTableBorder* tableStyle;
   if (mCaptionFrame) {
-    mCaptionFrame->GetStyleData(eStyleStruct_TableBorder, ((const nsStyleStruct *&)tableStyle));
-    return tableStyle->mCaptionSide;
+    return mCaptionFrame->GetStyleTableBorder()->mCaptionSide;
   }
   else {
     return NO_SIDE; // no caption
@@ -829,8 +804,7 @@ nsTableOuterFrame::GetCaptionSide()
 PRUint8
 nsTableOuterFrame::GetCaptionVerticalAlign()
 {
-  const nsStyleTextReset* textStyle;
-  mCaptionFrame->GetStyleData(eStyleStruct_TextReset, ((const nsStyleStruct *&) textStyle));
+  const nsStyleTextReset* textStyle = mCaptionFrame->GetStyleTextReset();
   PRUint8 verticalAlignFlags = NS_STYLE_VERTICAL_ALIGN_TOP;
   if (textStyle->mVerticalAlign.GetUnit() == eStyleUnit_Enumerated) {
     verticalAlignFlags = textStyle->mVerticalAlign.GetIntValue();
@@ -849,14 +823,13 @@ nsTableOuterFrame::SetDesiredSize(nsIPresContext* aPresContext,
 {
   aWidth = aHeight = 0;
 
-  nsRect innerRect;
-  mInnerTableFrame->GetRect(innerRect);
+  nsRect innerRect = mInnerTableFrame->GetRect();
   nscoord innerWidth = innerRect.width;
 
   nsRect captionRect(0,0,0,0);
   nscoord captionWidth = 0;
   if (mCaptionFrame) {
-    mCaptionFrame->GetRect(captionRect);
+    captionRect = mCaptionFrame->GetRect();
     captionWidth = captionRect.width;
     if ((NS_UNCONSTRAINEDSIZE == aAvailableWidth) &&
         ((NS_SIDE_LEFT == aCaptionSide) || (NS_SIDE_RIGHT == aCaptionSide))) {
@@ -898,9 +871,7 @@ nsTableOuterFrame::PctAdjustMinCaptionWidth(nsIPresContext*           aPresConte
       PRBool isPctWidth;
       IsAutoWidth(*mCaptionFrame,&isPctWidth);
       if (isPctWidth) {
-        nsRect priorCaptionRect;
-        mCaptionFrame->GetRect(priorCaptionRect);
-        capMin = priorCaptionRect.width;
+        capMin = mCaptionFrame->GetSize().width;
       }
       capMin += capMarginNoAuto.left + capMarginNoAuto.right;
     }
@@ -932,23 +903,22 @@ nsTableOuterFrame::BalanceLeftRightCaption(nsIPresContext* aPresContext,
   *     aCaptionWidth = capPercent/(1-capPercent) * 
   *                      (aCaptionMargin.left + aCaptionMargin.right + aInnerwidth + aInnerMargin.right)
   *     this result will cause problems for capPercent >= 1, in these cases the algorithm will now bail out
-  *     a similiar expression can be found for the right case
+  *     a similar expression can be found for the right case
   *  d) both frames have percent widths in this case the caption width will be the inner width multiplied 
   *     by the weight capPercent/innerPercent
   */
     
-  const nsStylePosition* position;
 
   float capPercent   = -1.0;
   float innerPercent = -1.0;
-  mCaptionFrame->GetStyleData(eStyleStruct_Position, ((const nsStyleStruct *&)position));
+  const nsStylePosition* position = mCaptionFrame->GetStylePosition();
   if (eStyleUnit_Percent == position->mWidth.GetUnit()) {
     capPercent = position->mWidth.GetPercentValue();
     if (capPercent >= 1.0)
       return;
   }
 
-  mInnerTableFrame->GetStyleData(eStyleStruct_Position, ((const nsStyleStruct *&)position));
+  position = mInnerTableFrame->GetStylePosition();
   if (eStyleUnit_Percent == position->mWidth.GetUnit()) {
     innerPercent = position->mWidth.GetPercentValue();
     if (innerPercent >= 1.0)
@@ -1233,10 +1203,7 @@ nsTableOuterFrame::IsAutoWidth(nsIFrame& aTableOrCaption,
     *aIsPctWidth = PR_FALSE;
   }
 
-  nsCOMPtr<nsIStyleContext> styleContext;
-  aTableOrCaption.GetStyleContext(getter_AddRefs(styleContext)); 
-
-  nsStylePosition* position = (nsStylePosition*)styleContext->GetStyleData(eStyleStruct_Position);
+  const nsStylePosition* position = aTableOrCaption.GetStylePosition();
 
   switch (position->mWidth.GetUnit()) {
     case eStyleUnit_Auto:         // specified auto width
@@ -1338,10 +1305,9 @@ nsTableOuterFrame::OuterReflowChild(nsIPresContext*            aPresContext,
   }
 
   // use the current position as a best guess for placement
-  nsRect childRect;
-  aChildFrame->GetRect(childRect);
+  nsPoint childPt = aChildFrame->GetPosition();
   nsresult rv = ReflowChild(aChildFrame, aPresContext, aMetrics, childRS,
-                            childRect.x, childRect.y, NS_FRAME_NO_MOVE_FRAME, aStatus);
+                            childPt.x, childPt.y, NS_FRAME_NO_MOVE_FRAME, aStatus);
   if (NS_FAILED(rv)) return rv;
   
   FixAutoMargins(aAvailWidth, aMetrics.width, childRS);
@@ -1369,37 +1335,63 @@ nsTableOuterFrame::UpdateReflowMetrics(nsIPresContext*      aPresContext,
   SetDesiredSize(aPresContext, aCaptionSide, aInnerMargin, aCaptionMargin, aAvailableWidth, aMet.width, aMet.height);
 
   // set maxElementSize width if requested
-  if (aMet.maxElementSize) {
-    *aMet.maxElementSize = GetMaxElementSize(aCaptionSide, aInnerMarginNoAuto, aInnerPadding, aCaptionMarginNoAuto);
+  if (aMet.mComputeMEW) {
+    aMet.mMaxElementWidth = GetMaxElementWidth(aCaptionSide, aInnerMarginNoAuto, aInnerPadding, aCaptionMarginNoAuto);
   }
   // set maximum width if requested
   if (aMet.mFlags & NS_REFLOW_CALC_MAX_WIDTH) {
     aMet.mMaximumWidth = GetMaxWidth(aCaptionSide, aInnerMarginNoAuto, aCaptionMarginNoAuto);
   }
-  // see if the caption leaks out
-  if (NS_SIDE_LEFT == aCaptionSide) {
-    if (aCaptionMarginNoAuto.left < 0) {
-      aMet.mOverflowArea.x = aCaptionMarginNoAuto.left;
-      aMet.mOverflowArea.y = 0;
-      aMet.mOverflowArea.width = aMet.width - aCaptionMarginNoAuto.left;
-      aMet.mOverflowArea.height = aMet.height;
-      nsFrameState  frameState;
-      GetFrameState(&frameState);
-      frameState |= NS_FRAME_OUTSIDE_CHILDREN;
-      SetFrameState(frameState);
-    }
+  // see if the caption or the inner table frame leaks out
+  // the overflow area of the inner table and caption frame needs to be factored in once 
+  // one can retrieve them from the frames (see bug 197581)
+  switch (aCaptionSide) {
+    case NS_SIDE_LEFT:
+      aMet.mOverflowArea.x      = PR_MIN(0, aCaptionMarginNoAuto.left);
+      aMet.mOverflowArea.width  = aMet.width - aMet.mOverflowArea.x - 
+                                  PR_MIN(0, aInnerMarginNoAuto.right);
+      aMet.mOverflowArea.y      = PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.top), aInnerMarginNoAuto.top);
+      aMet.mOverflowArea.height = aMet.height - aMet.mOverflowArea.y -
+                                  PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.bottom), aInnerMarginNoAuto.bottom); 
+      break;
+    case NS_SIDE_RIGHT:
+      aMet.mOverflowArea.x      = PR_MIN(0, aInnerMarginNoAuto.left);
+      aMet.mOverflowArea.width  = aMet.width - aMet.mOverflowArea.x - 
+                                  PR_MIN(0, aCaptionMarginNoAuto.right);
+      aMet.mOverflowArea.y      = PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.top), aInnerMarginNoAuto.top);
+      aMet.mOverflowArea.height = aMet.height - aMet.mOverflowArea.y -
+                                  PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.bottom), aInnerMarginNoAuto.bottom);
+      break;
+    case NS_SIDE_BOTTOM:
+      aMet.mOverflowArea.x      = PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.left), aInnerMarginNoAuto.left);
+      aMet.mOverflowArea.width  = aMet.width - aMet.mOverflowArea.x -
+                                  PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.right), aInnerMarginNoAuto.right);
+      aMet.mOverflowArea.y      = PR_MIN(0, aInnerMarginNoAuto.top);
+      aMet.mOverflowArea.height = aMet.height - aMet.mOverflowArea.y - 
+                                  PR_MIN(0, aCaptionMarginNoAuto.bottom);
+      break;
+    case NS_SIDE_TOP:
+      aMet.mOverflowArea.x      = PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.left), aInnerMarginNoAuto.left);
+      aMet.mOverflowArea.width  = aMet.width - aMet.mOverflowArea.x -
+                                  PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.right), aInnerMarginNoAuto.right);
+      aMet.mOverflowArea.y      = PR_MIN(0, aCaptionMarginNoAuto.top);
+      aMet.mOverflowArea.height = aMet.height - aMet.mOverflowArea.y - 
+                                  PR_MIN(0, aInnerMarginNoAuto.bottom);
+      break;
+    default: // no caption
+      aMet.mOverflowArea.x      = PR_MIN(0, aInnerMarginNoAuto.left);
+      aMet.mOverflowArea.width  = aMet.width - aMet.mOverflowArea.x - PR_MIN(0, aInnerMarginNoAuto.right);
+      aMet.mOverflowArea.y      = PR_MIN(0, aInnerMarginNoAuto.top);
+      aMet.mOverflowArea.height = aMet.height - aMet.mOverflowArea.y - PR_MIN(0, aInnerMarginNoAuto.bottom);
   }
-  else if (NS_SIDE_RIGHT == aCaptionSide) {
-    if (aCaptionMarginNoAuto.right <0) {
-      aMet.mOverflowArea.x = 0;
-      aMet.mOverflowArea.y = 0;
-      aMet.mOverflowArea.width = aMet.width - aCaptionMarginNoAuto.right;
-      aMet.mOverflowArea.height = aMet.height;
-      nsFrameState  frameState;
-      GetFrameState(&frameState);
-      frameState |= NS_FRAME_OUTSIDE_CHILDREN;
-      SetFrameState(frameState);
-    }
+  if ((aMet.mOverflowArea.x < 0) ||
+      (aMet.mOverflowArea.y < 0) ||
+      (aMet.mOverflowArea.XMost() > aMet.width) ||
+      (aMet.mOverflowArea.YMost() > aMet.height)) {
+    mState |= NS_FRAME_OUTSIDE_CHILDREN;
+  } 
+  else {
+    mState &= ~NS_FRAME_OUTSIDE_CHILDREN;
   } 
 }
 
@@ -1409,6 +1401,13 @@ nsTableOuterFrame::IncrementalReflow(nsIPresContext*          aPresContext,
                                      const nsHTMLReflowState& aReflowState,
                                      nsReflowStatus&          aStatus)
 {
+  // At this point, we need an inner table frame, and we might have a caption.  
+  // Due to the logic in nsCSSFrameConstructor::ConstructTableFrame, we can 
+  // end here without an inner table frame.
+  if (mFrames.IsEmpty() || !mInnerTableFrame) {
+    NS_ASSERTION(PR_FALSE, "incomplete children");
+    return NS_ERROR_FAILURE;
+  }
   // the outer table is a target if its path has a reflow command
   nsHTMLReflowCommand* command = aReflowState.path->mReflowCommand;
   if (command)
@@ -1444,8 +1443,7 @@ nsTableOuterFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
     rv = IR_TargetIsCaptionFrame(aPresContext, aDesiredSize, aReflowState, aStatus);
   }
   else {
-    const nsStyleDisplay* nextDisplay;
-    aNextFrame->GetStyleData(eStyleStruct_Display, (const nsStyleStruct *&)nextDisplay);
+    const nsStyleDisplay* nextDisplay = aNextFrame->GetStyleDisplay();
     if (NS_STYLE_DISPLAY_TABLE_HEADER_GROUP==nextDisplay->mDisplay ||
         NS_STYLE_DISPLAY_TABLE_FOOTER_GROUP==nextDisplay->mDisplay ||
         NS_STYLE_DISPLAY_TABLE_ROW_GROUP   ==nextDisplay->mDisplay ||
@@ -1480,15 +1478,14 @@ nsTableOuterFrame::IR_TargetIsCaptionFrame(nsIPresContext*           aPresContex
   aStatus = NS_FRAME_COMPLETE;
   PRUint8 captionSide = GetCaptionSide();
 
-  nsSize captionSize, captionMES;
+  nsSize captionSize;
   nsMargin captionMargin, captionMarginNoAuto, captionPadding;
   // reflow the caption frame, getting it's MES
-  nsRect prevInnerRect;
-  mInnerTableFrame->GetRect(prevInnerRect);
+  nsRect prevInnerRect = mInnerTableFrame->GetRect();
   nsMargin prevInnerMargin(prevInnerRect.x, 0, mRect.width - prevInnerRect.x - prevInnerRect.width, 0);
   nscoord availWidth = GetCaptionAvailWidth(aPresContext, mCaptionFrame, aOuterRS, captionMargin, 
                                             captionPadding, &prevInnerRect.width, nsnull, &prevInnerMargin);
-  nsHTMLReflowMetrics captionMet(&captionMES);
+  nsHTMLReflowMetrics captionMet(PR_TRUE);
   nsReflowStatus capStatus; // don't let the caption cause incomplete
   // for now just reflow the table if a style changed. This should be improved
   PRBool needInnerReflow = PR_FALSE;
@@ -1515,9 +1512,9 @@ nsTableOuterFrame::IR_TargetIsCaptionFrame(nsIPresContext*           aPresContex
   nscoord  capMin = mMinCaptionWidth + captionMarginNoAuto.left + captionMarginNoAuto.right;
   
 
-  if (mMinCaptionWidth != captionMES.width) {  
+  if (mMinCaptionWidth != captionMet.mMaxElementWidth) {  
     // set the new caption min width, and set state to reflow the inner table if necessary
-    mMinCaptionWidth = captionMES.width;
+    mMinCaptionWidth = captionMet.mMaxElementWidth;
     // see if the captions min width could cause the table to be wider
     // XXX this really only affects an auto width table
     if ((capMin) > mRect.width) {
@@ -1554,7 +1551,7 @@ nsTableOuterFrame::IR_TargetIsCaptionFrame(nsIPresContext*           aPresContex
   }
   else {
     // reposition the inner frame if necessary and set the caption's origin
-    nsSize innerSize = GetFrameSize(*mInnerTableFrame);
+    nsSize innerSize = mInnerTableFrame->GetSize();
     GetMarginPadding(aPresContext, aOuterRS, mInnerTableFrame, aOuterRS.availableWidth, innerMargin, 
                      innerMarginNoAuto, innerPadding);
     GetInnerOrigin(aPresContext, captionSide, containSize, captionSize, 
@@ -1581,22 +1578,19 @@ nsTableOuterFrame::IR_ReflowDirty(nsIPresContext*           aPresContext,
                                   const nsHTMLReflowState&  aReflowState,
                                   nsReflowStatus&           aStatus)
 {
-  nsFrameState  frameState;
   nsresult      rv = NS_OK;
   PRBool sizeSet = PR_FALSE;
   // See if the caption frame is dirty. This would be because of a newly
   // inserted caption
   if (mCaptionFrame) {
-    mCaptionFrame->GetFrameState(&frameState);
-    if (frameState & NS_FRAME_IS_DIRTY) {
+    if (mCaptionFrame->GetStateBits() & NS_FRAME_IS_DIRTY) {
       rv = IR_CaptionInserted(aPresContext, aDesiredSize, aReflowState, aStatus);
       sizeSet = PR_TRUE;
     }
   }
 
   // See if the inner table frame is dirty
-  mInnerTableFrame->GetFrameState(&frameState);
-  if (frameState & NS_FRAME_IS_DIRTY) {
+  if (mInnerTableFrame->GetStateBits() & NS_FRAME_IS_DIRTY) {
     rv = IR_InnerTableReflow(aPresContext, aDesiredSize, aReflowState, aStatus);
     sizeSet = PR_TRUE;
   } 
@@ -1604,8 +1598,7 @@ nsTableOuterFrame::IR_ReflowDirty(nsIPresContext*           aPresContext,
     // The inner table isn't dirty so we don't need to reflow it, but make
     // sure it's placed correctly. It could be that we're dirty because the
     // caption was removed
-    nsRect   innerRect;
-    mInnerTableFrame->GetRect(innerRect);
+    nsRect   innerRect = mInnerTableFrame->GetRect();
     nsSize   innerSize(innerRect.width, innerRect.height);
     nsPoint  innerOrigin;
     nsMargin innerMargin, innerMarginNoAuto, innerPadding;
@@ -1626,7 +1619,7 @@ nsTableOuterFrame::IR_ReflowDirty(nsIPresContext*           aPresContext,
   }
   if (!sizeSet) {
     // set our desired size to what it was before
-    nsSize size = GetFrameSize(*this);
+    nsSize size = GetSize();
     aDesiredSize.width  = size.width;
     aDesiredSize.height = size.height;
   }
@@ -1678,14 +1671,13 @@ nsTableOuterFrame::IR_InnerTableReflow(nsIPresContext*           aPresContext,
   aStatus = NS_FRAME_COMPLETE;
   PRUint8 captionSide = GetCaptionSide();
 
-  nsRect priorInnerRect;
-  mInnerTableFrame->GetRect(priorInnerRect);
+  nsSize priorInnerSize = mInnerTableFrame->GetSize();
 
   nsSize   innerSize;
   nsMargin innerMargin, innerMarginNoAuto, innerPadding;
 
   // pass along the reflow command to the inner table, requesting the same info in our flags
-  nsHTMLReflowMetrics innerMet(aOuterMet.maxElementSize, aOuterMet.mFlags);
+  nsHTMLReflowMetrics innerMet(aOuterMet.mComputeMEW, aOuterMet.mFlags);
 
   // If the incremental reflow command is a StyleChanged reflow and
   // it's target is the current frame, then make sure we send
@@ -1719,14 +1711,12 @@ nsTableOuterFrame::IR_InnerTableReflow(nsIPresContext*           aPresContext,
   // from a reflow, then reflow or move the caption as needed
   if (mCaptionFrame) {
     nsPoint captionOrigin;
-    nsRect prevCaptionRect;
-    mCaptionFrame->GetRect(prevCaptionRect);
+    nsRect prevCaptionRect = mCaptionFrame->GetRect();
 
-    if (priorInnerRect.width != innerMet.width) {
+    if (priorInnerSize.width != innerMet.width) {
       nsMargin ignorePadding;
       // XXX only need to reflow if the caption is auto width
-      nsSize maxElementSize(0,0);
-      nsHTMLReflowMetrics captionMet((eReflowReason_StyleChange != reflowReason) ? nsnull : &maxElementSize);
+      nsHTMLReflowMetrics captionMet(eReflowReason_StyleChange == reflowReason);
       nscoord availWidth = GetCaptionAvailWidth(aPresContext, mCaptionFrame, aOuterRS, captionMargin,
                                                 ignorePadding, &innerSize.width, &innerMarginNoAuto);
       nsReflowStatus capStatus; // don't let the caption cause incomplete
@@ -1748,7 +1738,7 @@ nsTableOuterFrame::IR_InnerTableReflow(nsIPresContext*           aPresContext,
     }
     else {
       // reposition the caption frame if necessary and set the inner's origin
-      captionSize = GetFrameSize(*mCaptionFrame);
+      captionSize = mCaptionFrame->GetSize();
       nsMargin captionPadding;
       GetMarginPadding(aPresContext, aOuterRS, mCaptionFrame, aOuterRS.availableWidth, captionMargin, 
                        captionMarginNoAuto, captionPadding);
@@ -1772,17 +1762,20 @@ nsTableOuterFrame::IR_InnerTableReflow(nsIPresContext*           aPresContext,
 
   FinishReflowChild(mInnerTableFrame, aPresContext, nsnull, innerMet,
                     innerOrigin.x, innerOrigin.y, 0);
+  if (aOuterMet.mComputeMEW) {
+    aOuterMet.mMaxElementWidth = innerMet.mMaxElementWidth;
+  }
 
   UpdateReflowMetrics(aPresContext, captionSide, aOuterMet, innerMargin, innerMarginNoAuto, 
                       innerPadding, captionMargin, captionMarginNoAuto, aOuterRS.availableWidth);
   nsSize desSize(aOuterMet.width, aOuterMet.height);
-  InvalidateDamage(aPresContext, captionSide, desSize, (innerSize.width != priorInnerRect.width), captionMoved);
+  InvalidateDamage(aPresContext, captionSide, desSize, (innerSize.width != priorInnerSize.width), captionMoved);
 
   return rv;
 }
 
 /* the only difference between an insert and a replace is a replace 
-   checks the old maxElementSize and reflows the table only if it
+   checks the old maxElementWidth and reflows the table only if it
    has changed
 */
 nsresult 
@@ -1797,8 +1790,7 @@ nsTableOuterFrame::IR_CaptionInserted(nsIPresContext*           aPresContext,
   // reflow the caption frame, getting it's MES
   nsSize   captionSize;
   nsMargin captionMargin, captionMarginNoAuto, captionPadding;
-  nsSize maxElementSize(0,0);
-  nsHTMLReflowMetrics captionMet(&maxElementSize);
+  nsHTMLReflowMetrics captionMet(PR_TRUE);
   // reflow the caption
   nscoord availWidth = GetCaptionAvailWidth(aPresContext, mCaptionFrame, aOuterRS, captionMargin, captionPadding);
   nsReflowStatus capStatus; // don't let the caption cause incomplete
@@ -1808,20 +1800,19 @@ nsTableOuterFrame::IR_CaptionInserted(nsIPresContext*           aPresContext,
 
   if (NS_FAILED(rv)) return rv;
 
-  mMinCaptionWidth = maxElementSize.width;
+  mMinCaptionWidth = captionMet.mMaxElementWidth;
 
   nsPoint  captionOrigin(0,0); 
 
   nsMargin innerMargin, innerMarginNoAuto, innerPadding;
   nsPoint innerOrigin;
   nsSize containSize = GetContainingBlockSize(aOuterRS);
-  nsRect prevInnerRect;
-  mInnerTableFrame->GetRect(prevInnerRect);
+  nsPoint prevInnerOrigin = mInnerTableFrame->GetPosition();
   nscoord capMin = mMinCaptionWidth + captionMarginNoAuto.left + captionMarginNoAuto.right;
 
   // if the caption's MES + margins > outer width, reflow the inner table
   if (capMin > mRect.width) {
-    nsHTMLReflowMetrics innerMet(aDesiredSize.maxElementSize); 
+    nsHTMLReflowMetrics innerMet(aDesiredSize.mComputeMEW); 
     nsSize innerSize;
     nscoord availWidth = GetInnerTableAvailWidth(aPresContext, mInnerTableFrame, aOuterRS, 
                                                  &capMin, innerMargin, innerPadding);
@@ -1834,13 +1825,16 @@ nsTableOuterFrame::IR_CaptionInserted(nsIPresContext*           aPresContext,
                    captionMargin, innerSize, innerMargin, innerOrigin);
     rv = FinishReflowChild(mInnerTableFrame, aPresContext, nsnull, innerMet,
                            innerOrigin.x, innerOrigin.y, 0);
+    if (aDesiredSize.mComputeMEW) {
+      aDesiredSize.mMaxElementWidth = innerMet.mMaxElementWidth;
+    }
     if (NS_FAILED(rv)) return rv;
     GetCaptionOrigin(aPresContext, captionSide, containSize, innerSize, 
                      innerMargin, captionSize, captionMargin, captionOrigin);
   }
   else {
     // reposition the inner frame if necessary and set the caption's origin
-    nsSize innerSize = GetFrameSize(*mInnerTableFrame);
+    nsSize innerSize = mInnerTableFrame->GetSize();
     GetMarginPadding(aPresContext, aOuterRS, mInnerTableFrame, aOuterRS.availableWidth, innerMargin, 
                      innerMarginNoAuto, innerPadding);
     GetInnerOrigin(aPresContext, captionSide, containSize, captionSize, 
@@ -1856,7 +1850,7 @@ nsTableOuterFrame::IR_CaptionInserted(nsIPresContext*           aPresContext,
   UpdateReflowMetrics(aPresContext, captionSide, aDesiredSize, innerMargin, innerMarginNoAuto, 
                       innerPadding, captionMargin, captionMarginNoAuto, aOuterRS.availableWidth);
   nsSize desSize(aDesiredSize.width, aDesiredSize.height);
-  PRBool innerMoved = (innerOrigin.x != prevInnerRect.x) || (innerOrigin.y != prevInnerRect.y);
+  PRBool innerMoved = innerOrigin != prevInnerOrigin;
   InvalidateDamage(aPresContext, captionSide, desSize, innerMoved, PR_TRUE);
 
   return rv;
@@ -1875,9 +1869,7 @@ static PRBool
 IsPctHeight(nsIFrame* aFrame)
 {
   if (aFrame) {
-    nsCOMPtr<nsIStyleContext> styleContext;
-    aFrame->GetStyleContext(getter_AddRefs(styleContext)); 
-    nsStylePosition* position = (nsStylePosition*)styleContext->GetStyleData(eStyleStruct_Position);
+    const nsStylePosition* position = aFrame->GetStylePosition();
     if (eStyleUnit_Percent == position->mHeight.GetUnit()) {
       float percent = position->mHeight.GetPercentValue();
       if (percent > 0.0f) {
@@ -1919,9 +1911,8 @@ NS_METHOD nsTableOuterFrame::Reflow(nsIPresContext*          aPresContext,
 
   // Initialize out parameters
   aDesiredSize.width = aDesiredSize.height = 0;
-  if (nsnull != aDesiredSize.maxElementSize) {
-    aDesiredSize.maxElementSize->width =  0;
-    aDesiredSize.maxElementSize->height = 0;
+  if (aDesiredSize.mComputeMEW) {
+    aDesiredSize.mMaxElementWidth =  0;
   }
   aStatus = NS_FRAME_COMPLETE;
 
@@ -1958,9 +1949,7 @@ NS_METHOD nsTableOuterFrame::Reflow(nsIPresContext*          aPresContext,
 
       // Lay out the caption and get its maximum element size
       if (nsnull != mCaptionFrame) {
-        nsSize maxElementSize;
-        nsHTMLReflowMetrics captionMet(&maxElementSize);
-        captionMet.maxElementSize = &maxElementSize;
+        nsHTMLReflowMetrics captionMet(PR_TRUE);
         nsHTMLReflowState captionReflowState(aPresContext, aOuterRS, mCaptionFrame,
                                              nsSize(NS_UNCONSTRAINEDSIZE, NS_UNCONSTRAINEDSIZE),
                                              eReflowReason_Initial);
@@ -1968,16 +1957,22 @@ NS_METHOD nsTableOuterFrame::Reflow(nsIPresContext*          aPresContext,
         mCaptionFrame->WillReflow(aPresContext);
         rv = mCaptionFrame->Reflow(aPresContext, captionMet, captionReflowState, aStatus);
         mCaptionFrame->DidReflow(aPresContext, nsnull, NS_FRAME_REFLOW_FINISHED);
-        mMinCaptionWidth = maxElementSize.width;
+        mMinCaptionWidth = captionMet.mMaxElementWidth;
       }
     }
-    // At this point, we must have an inner table frame, and we might have a caption
-    NS_ASSERTION(mFrames.NotEmpty() && mInnerTableFrame, "incomplete children");
+    // At this point, we need an inner table frame, and we might have a 
+    // caption. Due to the logic in 
+    // nsCSSFrameConstructor::ConstructTableFrame, we can end here 
+    // without an inner table frame.
+    if (mFrames.IsEmpty() || !mInnerTableFrame) {
+      NS_ASSERTION(PR_FALSE, "incomplete children");
+      return NS_ERROR_FAILURE;
+    }
     nsSize   innerSize;
     nsMargin innerMargin, innerMarginNoAuto, innerPadding;
 
     // First reflow the inner table
-    nsHTMLReflowMetrics innerMet(aDesiredSize.maxElementSize);
+    nsHTMLReflowMetrics innerMet(aDesiredSize.mComputeMEW);
     
     nscoord capMin = mMinCaptionWidth;
     PctAdjustMinCaptionWidth(aPresContext, aOuterRS, captionSide, capMin);
@@ -2026,6 +2021,9 @@ NS_METHOD nsTableOuterFrame::Reflow(nsIPresContext*          aPresContext,
 
     FinishReflowChild(mInnerTableFrame, aPresContext, nsnull, innerMet,
                       innerOrigin.x, innerOrigin.y, 0);
+    if (aDesiredSize.mComputeMEW) {
+      aDesiredSize.mMaxElementWidth = innerMet.mMaxElementWidth;
+    }
 
     UpdateReflowMetrics(aPresContext, captionSide, aDesiredSize, innerMargin, innerMarginNoAuto, 
                         innerPadding, captionMargin, captionMarginNoAuto, aOuterRS.availableWidth);
@@ -2108,14 +2106,13 @@ void nsTableOuterFrame::DeleteChildsNextInFlow(nsIPresContext* aPresContext,
   nsIFrame* nextInFlow;
    
   aChild->GetNextInFlow(&nextInFlow);
-
   if (!nextInFlow) {
     NS_ASSERTION(PR_FALSE, "null next-in-flow");
     return;
   }
-  nsTableOuterFrame* parent;
-   
-  nextInFlow->GetParent((nsIFrame**)&parent);
+
+  nsTableOuterFrame* parent = NS_STATIC_CAST(nsTableOuterFrame*,
+                                             nextInFlow->GetParent());
   if (!parent) {
     NS_ASSERTION(PR_FALSE, "null parent");
     return;
@@ -2134,22 +2131,16 @@ void nsTableOuterFrame::DeleteChildsNextInFlow(nsIPresContext* aPresContext,
 
   // Take the next-in-flow out of the parent's child list
   if (parent->mFrames.FirstChild() == nextInFlow) {
-    nsIFrame* nextSibling;
-    nextInFlow->GetNextSibling(&nextSibling);
-    parent->mFrames.SetFrames(nextSibling);
+    parent->mFrames.SetFrames(nextInFlow->GetNextSibling());
   } else {
-    nsIFrame* nextSibling;
-
     // Because the next-in-flow is not the first child of the parent
     // we know that it shares a parent with aChild. Therefore, we need
     // to capture the next-in-flow's next sibling (in case the
     // next-in-flow is the last next-in-flow for aChild AND the
     // next-in-flow is not the last child in parent)
-    aChild->GetNextSibling(&nextSibling);
-    NS_ASSERTION(nextSibling == nextInFlow, "unexpected sibling");
+    NS_ASSERTION(aChild->GetNextSibling() == nextInFlow, "unexpected sibling");
 
-    nextInFlow->GetNextSibling(&nextSibling);
-    aChild->SetNextSibling(nextSibling);
+    aChild->SetNextSibling(nextInFlow->GetNextSibling());
   }
 
   // Delete the next-in-flow frame and adjust it's parent's child count
@@ -2226,15 +2217,4 @@ nsTableOuterFrame::GetFrameName(nsAString& aResult) const
 {
   return MakeFrameName(NS_LITERAL_STRING("TableOuter"), aResult);
 }
-
-NS_IMETHODIMP
-nsTableOuterFrame::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const
-{
-  if (!aResult) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  PRUint32 sum = sizeof(*this);
-  *aResult = sum;
-  return NS_OK;
-}
 #endif
