@@ -23,20 +23,13 @@
 
 
 #include "nsCOMPtr.h"
-#include "nsHTMLParts.h"
-#include "nsIHTMLContent.h"
 #include "nsFrame.h"
-#include "nsLineLayout.h"
-#include "nsHTMLIIDs.h"
 #include "nsIPresContext.h"
-#include "nsHTMLAtoms.h"
 #include "nsUnitConversion.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsStyleConsts.h"
-#include "nsINameSpaceManager.h"
 #include "nsIRenderingContext.h"
 #include "nsIFontMetrics.h"
-#include "nsStyleUtil.h"
 
 #include "nsMathMLmfencedFrame.h"
 
@@ -69,41 +62,56 @@ nsMathMLmfencedFrame::~nsMathMLmfencedFrame()
 }
 
 NS_IMETHODIMP
-nsMathMLmfencedFrame::Init(nsIPresContext*  aPresContext,
-                           nsIContent*      aContent,
-                           nsIFrame*        aParent,
-                           nsIStyleContext* aContext,
-                           nsIFrame*        aPrevInFlow)
+nsMathMLmfencedFrame::InheritAutomaticData(nsIPresContext* aPresContext,
+                                           nsIFrame*       aParent)
 {
-  nsresult rv = nsMathMLContainerFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
+  // let the base class get the default from our parent
+  nsMathMLContainerFrame::InheritAutomaticData(aPresContext, aParent);
 
-  mEmbellishData.flags |= NS_MATHML_STRETCH_ALL_CHILDREN_VERTICALLY;
+  mPresentationData.flags |= NS_MATHML_STRETCH_ALL_CHILDREN_VERTICALLY;
 
-  mOpenChar = nsnull;
-  mCloseChar = nsnull;
-  mSeparatorsChar = nsnull;
-  mSeparatorsCount = 0;
-
-#if defined(NS_DEBUG) && defined(SHOW_BOUNDING_BOX)
-  mPresentationData.flags |= NS_MATHML_SHOW_BOUNDING_METRICS;
-#endif
-  return rv;
+  return NS_OK;
 }
 
-
 NS_IMETHODIMP
 nsMathMLmfencedFrame::SetInitialChildList(nsIPresContext* aPresContext,
                                           nsIAtom*        aListName,
                                           nsIFrame*       aChildList)
 {
-  nsresult rv;
-
   // First, let the base class do its work
-  rv = nsMathMLContainerFrame::SetInitialChildList(aPresContext, aListName, aChildList);
+  nsresult rv = nsMathMLContainerFrame::SetInitialChildList(aPresContext, aListName, aChildList);
   if (NS_FAILED(rv)) return rv;
 
-  rv = CreateFencesAndSeparators(aPresContext);  
-  return rv;
+  // No need to tract the style contexts given to our MathML chars. 
+  // The Style System will use Get/SetAdditionalStyleContext() to keep them
+  // up-to-date if dynamic changes arise.
+  return CreateFencesAndSeparators(aPresContext);
+}
+
+NS_IMETHODIMP
+nsMathMLmfencedFrame::AttributeChanged(nsIPresContext* aPresContext,
+                                       nsIContent*     aContent,
+                                       PRInt32         aNameSpaceID,
+                                       nsIAtom*        aAttribute,
+                                       PRInt32         aModType, 
+                                       PRInt32         aHint)
+{
+  RemoveFencesAndSeparators();
+  CreateFencesAndSeparators(aPresContext);
+
+  return nsMathMLContainerFrame::
+         AttributeChanged(aPresContext, aContent, aNameSpaceID,
+                          aAttribute, aModType, aHint);
+}
+
+nsresult
+nsMathMLmfencedFrame::ChildListChanged(nsIPresContext* aPresContext,
+                                       PRInt32         aModType)
+{
+  RemoveFencesAndSeparators();
+  CreateFencesAndSeparators(aPresContext);
+
+  return nsMathMLContainerFrame::ChildListChanged(aPresContext, aModType);
 }
 
 void
@@ -112,7 +120,7 @@ nsMathMLmfencedFrame::RemoveFencesAndSeparators()
   if (mOpenChar) delete mOpenChar;
   if (mCloseChar) delete mCloseChar;
   if (mSeparatorsChar) delete[] mSeparatorsChar;
-  
+
   mOpenChar = nsnull;
   mCloseChar = nsnull;
   mSeparatorsChar = nsnull;
@@ -123,8 +131,8 @@ nsresult
 nsMathMLmfencedFrame::CreateFencesAndSeparators(nsIPresContext* aPresContext)
 {
   nsresult rv;
-
   nsAutoString value, data;
+  PRBool isMutable = PR_FALSE;
 
   //////////////  
   // see if the opening fence is there ...
@@ -137,13 +145,14 @@ nsMathMLmfencedFrame::CreateFencesAndSeparators(nsIPresContext* aPresContext)
   else if (NS_CONTENT_ATTR_NOT_THERE == rv)
     data = PRUnichar('('); // default as per the MathML REC
   else
-    data = nsAutoString();
+    data.Truncate();
 
-  if (0 < data.Length()) {
+  if (!data.IsEmpty()) {
     mOpenChar = new nsMathMLChar;
     if (!mOpenChar) return NS_ERROR_OUT_OF_MEMORY;
     mOpenChar->SetData(aPresContext, data);
-    ResolveMathMLCharStyle(aPresContext, mContent, mStyleContext, mOpenChar);
+    isMutable = nsMathMLOperators::IsMutableOperator(data);
+    ResolveMathMLCharStyle(aPresContext, mContent, mStyleContext, mOpenChar, isMutable);
   }
 
   //////////////
@@ -157,13 +166,14 @@ nsMathMLmfencedFrame::CreateFencesAndSeparators(nsIPresContext* aPresContext)
   else if (NS_CONTENT_ATTR_NOT_THERE == rv)
     data = PRUnichar(')'); // default as per the MathML REC
   else
-    data = nsAutoString();
+    data.Truncate();
 
-  if (0 < data.Length()) {
+  if (!data.IsEmpty()) {
     mCloseChar = new nsMathMLChar;
     if (!mCloseChar) return NS_ERROR_OUT_OF_MEMORY;
     mCloseChar->SetData(aPresContext, data);
-    ResolveMathMLCharStyle(aPresContext, mContent, mStyleContext, mCloseChar);
+    isMutable = nsMathMLOperators::IsMutableOperator(data);
+    ResolveMathMLCharStyle(aPresContext, mContent, mStyleContext, mCloseChar, isMutable);
   }
 
   //////////////
@@ -177,27 +187,26 @@ nsMathMLmfencedFrame::CreateFencesAndSeparators(nsIPresContext* aPresContext)
   else if (NS_CONTENT_ATTR_NOT_THERE == rv)
     data = PRUnichar(','); // default as per the MathML REC
   else
-    data = nsAutoString();
+    data.Truncate();
 
   mSeparatorsCount = data.Length();
   if (0 < mSeparatorsCount) {
-    PRInt32 sepCount = -1;
-    nsIFrame* childFrame = mFrames.FirstChild();
-    while (childFrame) {
-      sepCount++;
-      childFrame->GetNextSibling(&childFrame);
-    }
+    PRInt32 sepCount = mFrames.GetLength() - 1;
     if (0 < sepCount) {
       mSeparatorsChar = new nsMathMLChar[sepCount];
       if (!mSeparatorsChar) return NS_ERROR_OUT_OF_MEMORY;
       nsAutoString sepChar;
       for (PRInt32 i = 0; i < sepCount; i++) {
-        if (i < mSeparatorsCount)
+        if (i < mSeparatorsCount) {
           sepChar = data[i];
-        else
+          isMutable = nsMathMLOperators::IsMutableOperator(sepChar);
+        }
+        else {
           sepChar = data[mSeparatorsCount-1];
+          // keep the value of isMutable that was set earlier
+        }
         mSeparatorsChar[i].SetData(aPresContext, sepChar);
-        ResolveMathMLCharStyle(aPresContext, mContent, mStyleContext, &mSeparatorsChar[i]);
+        ResolveMathMLCharStyle(aPresContext, mContent, mStyleContext, &mSeparatorsChar[i], isMutable);
       }
     }
     mSeparatorsCount = sepCount;
@@ -212,12 +221,10 @@ nsMathMLmfencedFrame::Paint(nsIPresContext*      aPresContext,
                             nsFramePaintLayer    aWhichLayer,
                             PRUint32             aFlags)
 {
-  nsresult rv = NS_OK;
-
   /////////////
   // paint the content
-  rv = nsMathMLContainerFrame::Paint(aPresContext, aRenderingContext, 
-                                     aDirtyRect, aWhichLayer);
+  nsresult rv = nsMathMLContainerFrame::Paint(aPresContext, aRenderingContext, 
+                                              aDirtyRect, aWhichLayer);
   ////////////
   // paint fences and separators
   if (NS_SUCCEEDED(rv)) {
@@ -264,7 +271,17 @@ nsMathMLmfencedFrame::doReflow(nsIPresContext*          aPresContext,
 
   nsMathMLContainerFrame* mathMLFrame =
     NS_STATIC_CAST(nsMathMLContainerFrame*, aForFrame);
- 
+
+  PRInt32 i;
+  nsCOMPtr<nsIFontMetrics> fm;
+  aReflowState.rendContext->SetFont(aForFrame->GetStyleFont()->mFont, nsnull);
+  aReflowState.rendContext->GetFontMetrics(*getter_AddRefs(fm));
+  nscoord axisHeight, em;
+  GetAxisHeight(*aReflowState.rendContext, fm, axisHeight);
+  GetEmHeight(fm, em);
+  // leading to be left at the top and the bottom of stretched chars
+  nscoord leading = NSToCoordRound(0.2f * em); 
+
   /////////////
   // Reflow children
   // Asking each child to cache its bounding metrics
@@ -274,15 +291,23 @@ nsMathMLmfencedFrame::doReflow(nsIPresContext*          aPresContext,
   // the *same* initial aDesiredSize.mBoundingMetrics. If we were to use the base
   // method here, our stretchy frames will be stretched and placed, and we may
   // end up stretching our fences/separators with a different aDesiredSize.
+  // XXX The above decision was revisited in bug 121748 and this code can be
+  // refactored to use nsMathMLContainerFrame::Reflow() at some stage.
 
   PRInt32 count = 0;
   nsReflowStatus childStatus;
   nsSize availSize(aReflowState.mComputedWidth, aReflowState.mComputedHeight);
-  nsHTMLReflowMetrics childDesiredSize(aDesiredSize.maxElementSize, 
+  nsHTMLReflowMetrics childDesiredSize(aDesiredSize.mComputeMEW, 
                       aDesiredSize.mFlags | NS_REFLOW_CALC_BOUNDING_METRICS);
-  nsIFrame* fisrtChild;
-  aForFrame->FirstChild(aPresContext, nsnull, &fisrtChild);
-  nsIFrame* childFrame = fisrtChild;
+  nsIFrame* firstChild;
+  aForFrame->FirstChild(aPresContext, nsnull, &firstChild);
+  nsIFrame* childFrame = firstChild;
+  if (firstChild || aOpenChar || aCloseChar || aSeparatorsCount > 0) {
+    // We use the ASCII metrics to get our minimum height. This way, if we have
+    // borders or a background, they will fit better with other elements on the line
+    fm->GetMaxAscent(aDesiredSize.ascent);
+    fm->GetMaxDescent(aDesiredSize.descent);
+  }
   while (childFrame) {
     nsHTMLReflowState childReflowState(aPresContext, aReflowState,
                                        childFrame, availSize);
@@ -294,8 +319,7 @@ nsMathMLmfencedFrame::doReflow(nsIPresContext*          aPresContext,
     // At this stage, the origin points of the children have no use, so we will use the
     // origins as placeholders to store the child's ascent and descent. Later on,
     // we should set the origins so as to overwrite what we are storing there now.
-    childFrame->SetRect(aPresContext,
-                        nsRect(childDesiredSize.descent, childDesiredSize.ascent,
+    childFrame->SetRect(nsRect(childDesiredSize.descent, childDesiredSize.ascent,
                                childDesiredSize.width, childDesiredSize.height));
 
     // compute the bounding metrics right now for mfrac
@@ -308,7 +332,7 @@ nsMathMLmfencedFrame::doReflow(nsIPresContext*          aPresContext,
     else
       aDesiredSize.mBoundingMetrics += childDesiredSize.mBoundingMetrics;
 
-    childFrame->GetNextSibling(&childFrame);
+    childFrame = childFrame->GetNextSibling();
   }
 
   /////////////
@@ -317,9 +341,9 @@ nsMathMLmfencedFrame::doReflow(nsIPresContext*          aPresContext,
   nsBoundingMetrics containerSize;
   nsStretchDirection stretchDir = NS_STRETCH_DIRECTION_VERTICAL;
 
-  nsEmbellishData embellishData;
-  mathMLFrame->GetEmbellishData(embellishData);
-  if (!NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(embellishData.flags)) {
+  nsPresentationData presentationData;
+  mathMLFrame->GetPresentationData(presentationData);
+  if (!NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(presentationData.flags)) {
     // case when the call is made for mfrac, we only need to stretch the '/' separator
     containerSize = aDesiredSize.mBoundingMetrics; // computed earlier
   }
@@ -328,19 +352,18 @@ nsMathMLmfencedFrame::doReflow(nsIPresContext*          aPresContext,
     mathMLFrame->GetPreferredStretchSize(aPresContext, *aReflowState.rendContext,
                                          0, /* i.e., without embellishments */
                                          stretchDir, containerSize);
-    childFrame = fisrtChild;
+    childFrame = firstChild;
     while (childFrame) {
       nsIMathMLFrame* mathmlChild;
-      rv = childFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathmlChild);
-      if (NS_SUCCEEDED(rv) && mathmlChild) {
+      childFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathmlChild);
+      if (mathmlChild) {
         // retrieve the metrics that was stored at the previous pass
         GetReflowAndBoundingMetricsFor(childFrame, childDesiredSize, childDesiredSize.mBoundingMetrics);
 
         mathmlChild->Stretch(aPresContext, *aReflowState.rendContext, 
                              stretchDir, containerSize, childDesiredSize);
         // store the updated metrics
-        childFrame->SetRect(aPresContext,
-                            nsRect(childDesiredSize.descent, childDesiredSize.ascent,
+        childFrame->SetRect(nsRect(childDesiredSize.descent, childDesiredSize.ascent,
                                    childDesiredSize.width, childDesiredSize.height));
 
         if (aDesiredSize.descent < childDesiredSize.descent)
@@ -348,50 +371,46 @@ nsMathMLmfencedFrame::doReflow(nsIPresContext*          aPresContext,
         if (aDesiredSize.ascent < childDesiredSize.ascent)
           aDesiredSize.ascent = childDesiredSize.ascent;
       }
-      childFrame->GetNextSibling(&childFrame);
+      childFrame = childFrame->GetNextSibling();
     }
+    // bug 121748: for surrounding fences & separators, use a size that covers everything
+    mathMLFrame->GetPreferredStretchSize(aPresContext, *aReflowState.rendContext,
+                                         STRETCH_CONSIDER_EMBELLISHMENTS,
+                                         stretchDir, containerSize);
+  }
+  if (aDesiredSize.mComputeMEW) {
+    aDesiredSize.mMaxElementWidth = childDesiredSize.mMaxElementWidth;
   }
 
   //////////////////////////////////////////
   // Prepare the opening fence, separators, and closing fence, and
   // adjust the origin of children.
 
-  PRInt32 i;
-  const nsStyleFont* font;
-  aForFrame->GetStyleData(eStyleStruct_Font, (const nsStyleStruct *&)font);
-  nsCOMPtr<nsIFontMetrics> fm;
-  aReflowState.rendContext->SetFont(font->mFont);
-  aReflowState.rendContext->GetFontMetrics(*getter_AddRefs(fm));
-  nscoord axisHeight, em;
-  GetAxisHeight(*aReflowState.rendContext, fm, axisHeight);
-  GetEmHeight(fm, em);
-
   // we need to center around the axis
-  nscoord delta = PR_MAX(containerSize.ascent - axisHeight, 
-                         containerSize.descent + axisHeight);
-  containerSize.ascent = delta + axisHeight;
-  containerSize.descent = delta - axisHeight;
-
-  nsPresentationData presentationData;
-  mathMLFrame->GetPresentationData(presentationData);
+  if (firstChild) { // do nothing for an empty <mfenced></mfenced>
+    nscoord delta = PR_MAX(containerSize.ascent - axisHeight, 
+                           containerSize.descent + axisHeight);
+    containerSize.ascent = delta + axisHeight;
+    containerSize.descent = delta - axisHeight;
+  }
 
   /////////////////
   // opening fence ...
   ReflowChar(aPresContext, *aReflowState.rendContext, aOpenChar,
              NS_MATHML_OPERATOR_FORM_PREFIX, presentationData.scriptLevel, 
-             axisHeight, em, containerSize, aDesiredSize);
+             axisHeight, leading, em, containerSize, aDesiredSize);
   /////////////////
   // separators ...
   for (i = 0; i < aSeparatorsCount; i++) {
     ReflowChar(aPresContext, *aReflowState.rendContext, &aSeparatorsChar[i],
                NS_MATHML_OPERATOR_FORM_INFIX, presentationData.scriptLevel,
-               axisHeight, em, containerSize, aDesiredSize);
+               axisHeight, leading, em, containerSize, aDesiredSize);
   }
   /////////////////
   // closing fence ...
   ReflowChar(aPresContext, *aReflowState.rendContext, aCloseChar,
              NS_MATHML_OPERATOR_FORM_POSTFIX, presentationData.scriptLevel,
-             axisHeight, em, containerSize, aDesiredSize);
+             axisHeight, leading, em, containerSize, aDesiredSize);
 
   //////////////////
   // Adjust the origins of each child.
@@ -407,7 +426,7 @@ nsMathMLmfencedFrame::doReflow(nsIPresContext*          aPresContext,
     firstTime = PR_FALSE;
   }
 
-  childFrame = fisrtChild;
+  childFrame = firstChild;
   while (childFrame) {
     nsHTMLReflowMetrics childSize(nsnull);
     GetReflowAndBoundingMetricsFor(childFrame, childSize, bm);
@@ -428,7 +447,7 @@ nsMathMLmfencedFrame::doReflow(nsIPresContext*          aPresContext,
     }
     i++;
 
-    childFrame->GetNextSibling(&childFrame);
+    childFrame = childFrame->GetNextSibling();
   }
 
   if (aCloseChar) {
@@ -442,15 +461,15 @@ nsMathMLmfencedFrame::doReflow(nsIPresContext*          aPresContext,
   aDesiredSize.width = aDesiredSize.mBoundingMetrics.width;
   aDesiredSize.height = aDesiredSize.ascent + aDesiredSize.descent;
 
-  if (aDesiredSize.maxElementSize) {
-    aDesiredSize.maxElementSize->width = aDesiredSize.width;
-    aDesiredSize.maxElementSize->height = aDesiredSize.height;
+  if (aDesiredSize.mComputeMEW) {
+    aDesiredSize.mMaxElementWidth = aDesiredSize.width;
   }
 
   mathMLFrame->SetBoundingMetrics(aDesiredSize.mBoundingMetrics);
   mathMLFrame->SetReference(nsPoint(0, aDesiredSize.ascent));
 
   aStatus = NS_FRAME_COMPLETE;
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
 }
 
@@ -462,6 +481,7 @@ nsMathMLmfencedFrame::ReflowChar(nsIPresContext*      aPresContext,
                                  nsOperatorFlags      aForm,
                                  PRInt32              aScriptLevel,
                                  nscoord              axisHeight,
+                                 nscoord              leading,
                                  nscoord              em,
                                  nsBoundingMetrics&   aContainerSize,
                                  nsHTMLReflowMetrics& aDesiredSize)
@@ -470,10 +490,9 @@ nsMathMLmfencedFrame::ReflowChar(nsIPresContext*      aPresContext,
     nsOperatorFlags flags = 0;
     float leftSpace = 0.0f;
     float rightSpace = 0.0f;
- 
+
     nsAutoString data;
     aMathMLChar->GetData(data);
-    aMathMLChar->SetData(aPresContext, data); // XXX hack to reset, bug 45010
     PRBool found = nsMathMLOperators::LookupOperator(data, aForm,              
                                            &flags, &leftSpace, &rightSpace);
 
@@ -489,8 +508,7 @@ nsMathMLmfencedFrame::ReflowChar(nsIPresContext*      aPresContext,
                                         NS_STRETCH_DIRECTION_VERTICAL,
                                         aContainerSize, charSize);
 
-    if (NS_STRETCH_DIRECTION_UNSUPPORTED != aMathMLChar->GetStretchDirection())
-    {
+    if (NS_STRETCH_DIRECTION_UNSUPPORTED != aMathMLChar->GetStretchDirection()) {
       // has changed... so center the char around the axis
       nscoord height = charSize.ascent + charSize.descent;
       charSize.ascent = height/2 + axisHeight;
@@ -499,6 +517,7 @@ nsMathMLmfencedFrame::ReflowChar(nsIPresContext*      aPresContext,
     else {
       // either it hasn't changed or stretching the char failed (i.e.,
       // GetBoundingMetrics failed)
+      leading = 0;
       if (NS_FAILED(res)) {
         nsTextDimensions dimensions;
         aRenderingContext.GetTextDimensions(data.get(), data.Length(), dimensions);
@@ -511,10 +530,10 @@ nsMathMLmfencedFrame::ReflowChar(nsIPresContext*      aPresContext,
       }
     }
 
-    if (aDesiredSize.ascent < charSize.ascent) 
-      aDesiredSize.ascent = charSize.ascent;
-    if (aDesiredSize.descent < charSize.descent) 
-      aDesiredSize.descent = charSize.descent;
+    if (aDesiredSize.ascent < charSize.ascent + leading) 
+      aDesiredSize.ascent = charSize.ascent + leading;
+    if (aDesiredSize.descent < charSize.descent + leading) 
+      aDesiredSize.descent = charSize.descent + leading;
 
     // account the spacing
     charSize.width += NSToCoordRound((leftSpace + rightSpace) * em);
@@ -540,22 +559,20 @@ nsMathMLmfencedFrame::PlaceChar(nsMathMLChar*      aMathMLChar,
   // the char's y-origin was used to store the ascent ... 
   nsRect rect;
   aMathMLChar->GetRect(rect);
- 
+
   nscoord dy = aDesiredAscent - rect.y;
-  if (aMathMLChar->GetStretchDirection() != NS_STRETCH_DIRECTION_UNSUPPORTED)
-  {
+  if (aMathMLChar->GetStretchDirection() != NS_STRETCH_DIRECTION_UNSUPPORTED) {
     // the stretchy char will be centered around the axis
     // so we adjust the returned bounding metrics accordingly
     bm.descent = (bm.ascent + bm.descent) - rect.y;
     bm.ascent = rect.y;
   }
 
-  aMathMLChar->SetRect(nsRect(dx + rect.x, dy,
-                              bm.width, rect.height));
+  aMathMLChar->SetRect(nsRect(dx + rect.x, dy, bm.width, rect.height));
 
   bm.leftBearing += rect.x;
   bm.rightBearing += rect.x;
-  
+
   // return rect.width since it includes lspace and rspace
   bm.width = rect.width;
   dx += rect.width;
@@ -563,11 +580,9 @@ nsMathMLmfencedFrame::PlaceChar(nsMathMLChar*      aMathMLChar,
 
 // ----------------------
 // the Style System will use these to pass the proper style context to our MathMLChar
-NS_IMETHODIMP
-nsMathMLmfencedFrame::GetAdditionalStyleContext(PRInt32           aIndex, 
-                                                nsIStyleContext** aStyleContext) const
+nsStyleContext*
+nsMathMLmfencedFrame::GetAdditionalStyleContext(PRInt32 aIndex) const
 {
-  NS_PRECONDITION(aStyleContext, "null OUT ptr");
   PRInt32 openIndex = -1;
   PRInt32 closeIndex = -1;
   PRInt32 lastIndex = mSeparatorsCount-1;
@@ -581,25 +596,24 @@ nsMathMLmfencedFrame::GetAdditionalStyleContext(PRInt32           aIndex,
     closeIndex = lastIndex;
   }
   if (aIndex < 0 || aIndex > lastIndex) {
-    return NS_ERROR_INVALID_ARG;
+    return nsnull;
   }
 
-  *aStyleContext = nsnull;
   if (aIndex < mSeparatorsCount) {
-    mSeparatorsChar[aIndex].GetStyleContext(aStyleContext);
+    return mSeparatorsChar[aIndex].GetStyleContext();
   }
   else if (aIndex == openIndex) {
-    mOpenChar->GetStyleContext(aStyleContext);
+    return mOpenChar->GetStyleContext();
   }
   else if (aIndex == closeIndex) {
-    mCloseChar->GetStyleContext(aStyleContext);
+    return mCloseChar->GetStyleContext();
   }
-  return NS_OK;
+  return nsnull;
 }
 
-NS_IMETHODIMP
+void
 nsMathMLmfencedFrame::SetAdditionalStyleContext(PRInt32          aIndex, 
-                                                nsIStyleContext* aStyleContext)
+                                                nsStyleContext*  aStyleContext)
 {
   PRInt32 openIndex = -1;
   PRInt32 closeIndex = -1;
@@ -614,7 +628,7 @@ nsMathMLmfencedFrame::SetAdditionalStyleContext(PRInt32          aIndex,
     closeIndex = lastIndex;
   }
   if (aIndex < 0 || aIndex > lastIndex) {
-    return NS_ERROR_INVALID_ARG;
+    return;
   }
 
   if (aIndex < mSeparatorsCount) {
@@ -626,5 +640,4 @@ nsMathMLmfencedFrame::SetAdditionalStyleContext(PRInt32          aIndex,
   else if (aIndex == closeIndex) {
     mCloseChar->SetStyleContext(aStyleContext);
   }
-  return NS_OK;
 }
