@@ -61,11 +61,12 @@
 #include "nsIScrollableFrame.h"
 #include "nsIScrollbarFrame.h"
 #include "nsIScrollableView.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsIRenderingContext.h"
 #include "nsIDeviceContext.h"
 #include "nsIFontMetrics.h"
 #include "nsITimer.h"
+#include "nsAutoPtr.h"
 
 /////////////// nsListScrollSmoother //////////////////
 
@@ -225,7 +226,7 @@ NS_INTERFACE_MAP_END_INHERITING(nsBoxFrame)
 
 NS_IMETHODIMP
 nsListBoxBodyFrame::Init(nsIPresContext* aPresContext, nsIContent* aContent,
-                         nsIFrame* aParent, nsIStyleContext* aContext, nsIFrame* aPrevInFlow)
+                         nsIFrame* aParent, nsStyleContext* aContext, nsIFrame* aPrevInFlow)
 {
   nsresult rv = nsBoxFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
 
@@ -258,11 +259,10 @@ nsListBoxBodyFrame::Init(nsIPresContext* aPresContext, nsIContent* aContent,
 
   nsBoxLayoutState boxLayoutState(aPresContext);
 
-  const nsStyleFont* font = (const nsStyleFont*)aContext->GetStyleData(eStyleStruct_Font);
   nsCOMPtr<nsIDeviceContext> dc;
   aPresContext->GetDeviceContext(getter_AddRefs(dc));
   nsCOMPtr<nsIFontMetrics> fm;
-  dc->GetMetricsFor(font->mFont, *getter_AddRefs(fm));
+  dc->GetMetricsFor(aContext->GetStyleFont()->mFont, *getter_AddRefs(fm));
   fm->GetHeight(mRowHeight);
 
   return rv;
@@ -339,7 +339,7 @@ nsListBoxBodyFrame::DoLayout(nsBoxLayoutState& aBoxLayoutState)
 
   // if we are scrolled and the row height changed
   // make sure we are scrolled to a correct index.
-  if (mAdjustScroll) 
+  if (mAdjustScroll)
      PostReflowCallback();
 
   return rv;
@@ -555,30 +555,15 @@ nsListBoxBodyFrame::ScrollByLines(PRInt32 aNumLines)
   // we have to do a sync update for mac because if we scroll too quickly
   // w/out going back to the main event loop we can easily scroll the wrong
   // bits and it looks like garbage (bug 63465).
-  nsIFrame* frame = nsnull;
-  if (NS_FAILED(QueryInterface(NS_GET_IID(nsIFrame), (void **)&frame)))
-    return NS_ERROR_FAILURE;
     
-  nsIView* view = nsnull;
-  frame->GetView(mPresContext, &view);
-  if (!view) {
-    nsIFrame* frameWithView;
-    frame->GetParentWithView(mPresContext, &frameWithView);
-    if (frameWithView)
-      frameWithView->GetView(mPresContext, &view);
-    else
-      return NS_ERROR_FAILURE;
-  }
-  if (view) {
-    nsCOMPtr<nsIViewManager> vm;
-    if (view->GetViewManager(*getter_AddRefs(vm)) && nsnull != vm) {
-      // I'd use Composite here, but it doesn't always work.
-      // vm->Composite();
-      vm->ForceUpdate();
-  
-    }      
-  }
-     
+  nsCOMPtr<nsIPresShell> shell;
+  mPresContext->GetShell(getter_AddRefs(shell));
+  nsCOMPtr<nsIViewManager> vm;
+  shell->GetViewManager(getter_AddRefs(vm));
+  // I'd use Composite here, but it doesn't always work.
+  // vm->Composite();
+  vm->ForceUpdate();
+
   return NS_OK;
 }
 
@@ -600,10 +585,10 @@ nsListBoxBodyFrame::GetIndexOfItem(nsIDOMElement* aItem, PRInt32* _retval)
 
   for (PRInt32 childIndex = 0; childIndex < childCount; childIndex++) {
     nsCOMPtr<nsIContent> child;
-    listbox->ChildAt(childIndex, *getter_AddRefs(child));
+    listbox->ChildAt(childIndex, getter_AddRefs(child));
     
     nsCOMPtr<nsIAtom> childTag;
-    child->GetTag(*getter_AddRefs(childTag));
+    child->GetTag(getter_AddRefs(childTag));
 
     // we hit a treerow, count it
     if (childTag == nsXULAtoms::listitem) {
@@ -634,10 +619,10 @@ nsListBoxBodyFrame::GetItemAtIndex(PRInt32 aIndex, nsIDOMElement** _retval)
   PRInt32 itemCount = 0;
   for (PRInt32 childIndex = 0; childIndex < childCount; childIndex++) {
     nsCOMPtr<nsIContent> child;
-    listbox->ChildAt(childIndex, *getter_AddRefs(child));
+    listbox->ChildAt(childIndex, getter_AddRefs(child));
     
     nsCOMPtr<nsIAtom> childTag;
-    child->GetTag(*getter_AddRefs(childTag));
+    child->GetTag(getter_AddRefs(childTag));
 
     // we hit a treerow, count it
     if (childTag == nsXULAtoms::listitem) {
@@ -747,9 +732,9 @@ nsListBoxBodyFrame::ComputeIntrinsicWidth(nsBoxLayoutState& aBoxLayoutState)
   nsCOMPtr<nsIContent> firstRowContent(do_QueryInterface(firstRowEl));
 
   if (firstRowContent) {
-    nsCOMPtr<nsIStyleContext> styleContext;
-    aBoxLayoutState.GetPresContext()->ResolveStyleContextFor(firstRowContent, nsnull,
-                                                             getter_AddRefs(styleContext));
+    nsRefPtr<nsStyleContext> styleContext;
+    styleContext = aBoxLayoutState.GetPresContext()->ResolveStyleContextFor(firstRowContent,
+                                                                            nsnull);
 
     nscoord width = 0;
     nsMargin margin(0,0,0,0);
@@ -760,8 +745,7 @@ nsListBoxBodyFrame::ComputeIntrinsicWidth(nsBoxLayoutState& aBoxLayoutState)
 
     width += (margin.left + margin.right);
 
-    const nsStyleMargin* styleMargin = (const nsStyleMargin*)styleContext->GetStyleData(eStyleStruct_Margin);
-    styleMargin->GetMargin(margin);
+    styleContext->GetStyleMargin()->GetMargin(margin);
     width += (margin.left + margin.right);
 
     nsCOMPtr<nsIContent> listbox;
@@ -772,10 +756,10 @@ nsListBoxBodyFrame::ComputeIntrinsicWidth(nsBoxLayoutState& aBoxLayoutState)
 
     nsCOMPtr<nsIContent> child;
     for (PRInt32 i = 0; i < childCount && i < 100; ++i) {
-      listbox->ChildAt(i, *getter_AddRefs(child));
+      listbox->ChildAt(i, getter_AddRefs(child));
 
       nsCOMPtr<nsIAtom> tag;
-      child->GetTag(*getter_AddRefs(tag));
+      child->GetTag(getter_AddRefs(tag));
       if (tag == nsXULAtoms::listitem) {
         nsIPresContext* presContext = aBoxLayoutState.GetPresContext();
         nsIRenderingContext* rendContext = aBoxLayoutState.GetReflowState()->rendContext;
@@ -785,7 +769,7 @@ nsListBoxBodyFrame::ComputeIntrinsicWidth(nsBoxLayoutState& aBoxLayoutState)
           PRInt32 textCount;
           child->ChildCount(textCount);
           for (PRInt32 j = 0; j < textCount; ++j) {
-            child->ChildAt(j, *getter_AddRefs(textChild));
+            child->ChildAt(j, getter_AddRefs(textChild));
             nsCOMPtr<nsIDOMText> text(do_QueryInterface(textChild));
             if (text) {
               nsAutoString data;
@@ -793,11 +777,10 @@ nsListBoxBodyFrame::ComputeIntrinsicWidth(nsBoxLayoutState& aBoxLayoutState)
               value += data;
             }
           }
-          const nsStyleFont* font = (const nsStyleFont*)styleContext->GetStyleData(eStyleStruct_Font);
           nsCOMPtr<nsIDeviceContext> dc;
           presContext->GetDeviceContext(getter_AddRefs(dc));
           nsCOMPtr<nsIFontMetrics> fm;
-          dc->GetMetricsFor(font->mFont, *getter_AddRefs(fm));
+          dc->GetMetricsFor(styleContext->GetStyleFont()->mFont, *getter_AddRefs(fm));
           rendContext->SetFont(fm);
 
           nscoord textWidth;
@@ -827,9 +810,9 @@ nsListBoxBodyFrame::ComputeTotalRowCount()
   mRowCount = 0;
   for (PRInt32 i = 0; i < childCount; i++) {
     nsCOMPtr<nsIContent> childContent;
-    listbox->ChildAt(i, *getter_AddRefs(childContent));
+    listbox->ChildAt(i, getter_AddRefs(childContent));
     nsCOMPtr<nsIAtom> tag;
-    childContent->GetTag(*getter_AddRefs(tag));
+    childContent->GetTag(getter_AddRefs(tag));
     if (tag == nsXULAtoms::listitem)
       ++mRowCount;
   }
@@ -872,7 +855,7 @@ nsListBoxBodyFrame::DoScrollToIndex(PRInt32 aRowIndex, PRBool aForceDestruct)
   // This change has to happen immediately.
   // Flush any pending reflow commands.
   nsCOMPtr<nsIDocument> doc;
-  mContent->GetDocument(*getter_AddRefs(doc));
+  mContent->GetDocument(getter_AddRefs(doc));
   doc->FlushPendingNotifications();
 
   return NS_OK;
@@ -1163,13 +1146,14 @@ nsListBoxBodyFrame::GetFirstItemBox(PRInt32 aOffset, PRBool* aCreated)
     nsCOMPtr<nsIContent> topContent;
     mTopFrame->GetContent(getter_AddRefs(topContent));
     nsCOMPtr<nsIContent> topParent;
-    topContent->GetParent(*getter_AddRefs(topParent));
+    topContent->GetParent(getter_AddRefs(topParent));
     PRInt32 contentIndex;
     topParent->IndexOf(topContent, contentIndex);
     contentIndex -= aOffset;
     if (contentIndex < 0)
       return nsnull;
-    topParent->ChildAt(contentIndex-mRowsToPrepend, *getter_AddRefs(startContent));
+    topParent->ChildAt(contentIndex - mRowsToPrepend,
+                       getter_AddRefs(startContent));
   } else {
     // This will be the first item frame we create.  Use the content
     // at the current index, which is the first index scrolled into view
@@ -1221,13 +1205,13 @@ nsListBoxBodyFrame::GetNextItemBox(nsIBox* aBox, PRInt32 aOffset, PRBool* aCreat
     nsCOMPtr<nsIContent> prevContent;
     frame->GetContent(getter_AddRefs(prevContent));
     nsCOMPtr<nsIContent> parentContent;
-    prevContent->GetParent(*getter_AddRefs(parentContent));
+    prevContent->GetParent(getter_AddRefs(parentContent));
     parentContent->IndexOf(prevContent, i);
     parentContent->ChildCount(childCount);
     if (i+aOffset+1 < childCount) {
       // There is a content node that wants a frame.
       nsCOMPtr<nsIContent> nextContent;
-      parentContent->ChildAt(i+aOffset+1, *getter_AddRefs(nextContent));
+      parentContent->ChildAt(i + aOffset + 1, getter_AddRefs(nextContent));
 
       // Either append the new frame, or insert it after the current frame
       PRBool isAppend = result != mLinkupFrame && mRowsToPrepend <= 0;
@@ -1373,7 +1357,7 @@ nsListBoxBodyFrame::OnContentRemoved(nsIPresContext* aPresContext, nsIFrame* aCh
     nsCOMPtr<nsIContent> listboxContent;
     mContent->GetBindingParent(getter_AddRefs(listboxContent));
     nsCOMPtr<nsIContent> oldNextSiblingContent;
-    listboxContent->ChildAt(aIndex, *getter_AddRefs(oldNextSiblingContent));
+    listboxContent->ChildAt(aIndex, getter_AddRefs(oldNextSiblingContent));
     PRInt32 siblingIndex = -1;
     if (oldNextSiblingContent) {
       nsCOMPtr<nsIContent> nextSiblingContent;
@@ -1382,7 +1366,8 @@ nsListBoxBodyFrame::OnContentRemoved(nsIPresContext* aPresContext, nsIFrame* aCh
   
     // if the row being removed is off-screen and above the top frame, we need to
     // adjust our top index and tell the scrollbar to shift up one row.
-    if (siblingIndex >= 0 && siblingIndex-1 <= mCurrentIndex) {
+    if (siblingIndex >= 0 && siblingIndex-1 < mCurrentIndex) {
+      NS_PRECONDITION(mCurrentIndex > 0, "mCurrentIndex > 0");
       --mCurrentIndex;
       mYPosition = mCurrentIndex*mRowHeight;
       VerticalScroll(mYPosition);
@@ -1401,7 +1386,7 @@ nsListBoxBodyFrame::OnContentRemoved(nsIPresContext* aPresContext, nsIFrame* aCh
     listBoxContent->ChildCount(childCount);
     if (childCount > 0) {
       nsCOMPtr<nsIContent> lastChild;
-      listBoxContent->ChildAt(childCount-1, *getter_AddRefs(lastChild));
+      listBoxContent->ChildAt(childCount - 1, getter_AddRefs(lastChild));
       nsCOMPtr<nsIPresShell> shell;
       aPresContext->GetShell(getter_AddRefs(shell));
       nsIFrame* lastChildFrame = nsnull;
@@ -1447,9 +1432,9 @@ nsListBoxBodyFrame::GetListItemContentAt(PRInt32 aIndex, nsIContent** aContent)
   PRInt32 itemsFound = 0;
   for (PRInt32 i = 0; i < childCount; ++i) {
     nsCOMPtr<nsIContent> kid;
-    listboxContent->ChildAt(i, *getter_AddRefs(kid));
+    listboxContent->ChildAt(i, getter_AddRefs(kid));
     nsCOMPtr<nsIAtom> tag;
-    kid->GetTag(*getter_AddRefs(tag));
+    kid->GetTag(getter_AddRefs(tag));
     if (tag.get() == nsXULAtoms::listitem) {
       ++itemsFound;
       if (itemsFound-1 == aIndex) {
@@ -1475,9 +1460,9 @@ nsListBoxBodyFrame::GetListItemNextSibling(nsIContent* aListItem, nsIContent** a
   nsCOMPtr<nsIContent> prevKid;
   for (PRInt32 i = 0; i < childCount; ++i) {
     nsCOMPtr<nsIContent> kid;
-    listboxContent->ChildAt(i, *getter_AddRefs(kid));
+    listboxContent->ChildAt(i, getter_AddRefs(kid));
     nsCOMPtr<nsIAtom> tag;
-    kid->GetTag(*getter_AddRefs(tag));
+    kid->GetTag(getter_AddRefs(tag));
     if (tag.get() == nsXULAtoms::listitem) {
       ++aSiblingIndex;
       if (prevKid.get() == aListItem) {
