@@ -47,7 +47,7 @@
 #include "prtypes.h"
 #include "nsIAtom.h"
 #include "nsIPresContext.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsCSSRendering.h"
 #include "nsINameSpaceManager.h"
 #include "nsIViewManager.h"
@@ -71,6 +71,7 @@
 #include "nsIRootBox.h"
 #include "nsIDocShellTreeItem.h"
 #include "nsITimerInternal.h"
+#include "nsUnicharUtils.h"
 #ifdef XP_WIN
 #include "nsISound.h"
 #endif
@@ -168,7 +169,7 @@ NS_IMETHODIMP
 nsMenuPopupFrame::Init(nsIPresContext*  aPresContext,
                        nsIContent*      aContent,
                        nsIFrame*        aParent,
-                       nsIStyleContext* aContext,
+                       nsStyleContext*  aContext,
                        nsIFrame*        aPrevInFlow)
 {
   nsresult rv = nsBoxFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
@@ -191,8 +192,7 @@ nsMenuPopupFrame::Init(nsIPresContext*  aPresContext,
   // Now that we've made a view, remove it and insert it at the correct
   // position in the view hierarchy (as the root view).  We do this so that we
   // can draw the menus outside the confines of the window.
-  nsIView* ourView;
-  GetView(aPresContext, &ourView);
+  nsIView* ourView = GetView(aPresContext);
 
   nsCOMPtr<nsIViewManager> viewManager;
   ourView->GetViewManager(*getter_AddRefs(viewManager));
@@ -219,15 +219,14 @@ nsMenuPopupFrame::Init(nsIPresContext*  aPresContext,
   // Create a widget for ourselves.
   nsWidgetInitData widgetData;
   widgetData.mWindowType = eWindowType_popup;
-  widgetData.mContentType = nsIDocShellTreeItem::typeChrome;
   widgetData.mBorderStyle = eBorderStyle_default;
   widgetData.clipSiblings = PR_TRUE;
 
   nsCOMPtr<nsIContent> parentContent;
-  aContent->GetParent(*getter_AddRefs(parentContent));
+  aContent->GetParent(getter_AddRefs(parentContent));
   nsCOMPtr<nsIAtom> tag;
   if (parentContent)
-    parentContent->GetTag(*getter_AddRefs(tag));
+    parentContent->GetTag(getter_AddRefs(tag));
   widgetData.mDropShadow = !(tag && tag == nsXULAtoms::menulist);
   
   // XXX make sure we are hidden (shouldn't this be done automatically?)
@@ -237,10 +236,12 @@ nsMenuPopupFrame::Init(nsIPresContext*  aPresContext,
   printf("XP Popups: This is a nag to indicate that an inconsistent hack is being done on the Mac for popups.\n");
 #endif  
   static NS_DEFINE_IID(kCPopupCID,  NS_POPUP_CID);
-  ourView->CreateWidget(kCPopupCID, &widgetData, nsnull);
+  ourView->CreateWidget(kCPopupCID, &widgetData, nsnull, PR_TRUE, PR_TRUE, 
+                        eContentTypeUI);
 #else
   static NS_DEFINE_IID(kCChildCID,  NS_CHILD_CID);
-  ourView->CreateWidget(kCChildCID, &widgetData, nsnull);
+  ourView->CreateWidget(kCChildCID, &widgetData, nsnull, PR_TRUE, PR_TRUE, 
+                        eContentTypeUI);
 #endif   
 
   MoveToAttributePosition();
@@ -498,10 +499,15 @@ nsMenuPopupFrame::GetViewOffset(nsIView* aView, nsPoint& aPoint)
 ///////////////////////////////////////////////////////////////////////////////
 // GetRootViewForPopup
 //   Retrieves the view for the popup widget that contains the given frame. 
-//   If the given frame is not contained by a popup widget, return the root view.
+//   If the given frame is not contained by a popup widget, return the
+//   root view.  This is the root view of the pres context's
+//   viewmanager if aStopAtViewManagerRoot is true; otherwise it's the
+//   root view of the root viewmanager.
 void
 nsMenuPopupFrame::GetRootViewForPopup(nsIPresContext* aPresContext, 
-                                      nsIFrame* aStartFrame, nsIView** aResult)
+                                      nsIFrame* aStartFrame,
+                                      PRBool aStopAtViewManagerRoot,
+                                      nsIView** aResult)
 {
   *aResult = nsnull;
 
@@ -521,9 +527,16 @@ nsMenuPopupFrame::GetRootViewForPopup(nsIPresContext* aPresContext,
   }
 
   if (parentWithView) {
-    nsIView* view = nsnull;
-    nsIView* temp = nsnull;
-    parentWithView->GetView(aPresContext, &view);
+    nsIView* view = parentWithView->GetView(aPresContext);
+    NS_ASSERTION(view, "GetParentWithView returned frame with no view!");
+    nsIView* rootView = nsnull;
+    if (aStopAtViewManagerRoot) {
+      nsCOMPtr<nsIViewManager> viewManager;
+      view->GetViewManager(*getter_AddRefs(viewManager));
+      NS_ASSERTION(viewManager, "View must have a viewmanager");
+      viewManager->GetRootView(rootView);
+    }
+    
     while (view) {
       // Walk up the view hierachy looking for a view whose widget has a 
       // window type of eWindowType_popup - in other words a popup window
@@ -539,6 +552,12 @@ nsMenuPopupFrame::GetRootViewForPopup(nsIPresContext* aPresContext,
         }
       }
 
+      if (aStopAtViewManagerRoot && view == rootView) {
+        *aResult = view;
+        return;
+      }
+
+      nsIView* temp = nsnull;
       view->GetParent(temp);
       if (!temp) {
         // Otherwise, we've walked all the way up to the root view and not
@@ -601,7 +620,7 @@ nsMenuPopupFrame::AdjustClientXYForNestedDocuments ( nsIDOMXULDocument* inPopupD
   // nsXULPopupListener).
 
   nsCOMPtr<nsIAtom> tag;
-  mContent->GetTag(*getter_AddRefs(tag));
+  mContent->GetTag(getter_AddRefs(tag));
   nsCOMPtr<nsIDOMNode> targetNode;
   if (tag == nsXULAtoms::tooltip)
     inPopupDoc->GetTooltipNode(getter_AddRefs(targetNode));
@@ -613,7 +632,7 @@ nsMenuPopupFrame::AdjustClientXYForNestedDocuments ( nsIDOMXULDocument* inPopupD
   nsCOMPtr<nsIWidget> targetDocumentWidget;
   if ( targetAsContent ) {
     nsCOMPtr<nsIDocument> targetDocument;
-    targetAsContent->GetDocument(*getter_AddRefs(targetDocument));
+    targetAsContent->GetDocument(getter_AddRefs(targetDocument));
     if (targetDocument) {
       nsCOMPtr<nsIPresShell> shell;
       targetDocument->GetShellAt(0, getter_AddRefs(shell));
@@ -628,7 +647,7 @@ nsMenuPopupFrame::AdjustClientXYForNestedDocuments ( nsIDOMXULDocument* inPopupD
           nsCOMPtr<nsIPresContext> targetContext;
           shell->GetPresContext(getter_AddRefs(targetContext));
           if (targetContext) {
-            GetRootViewForPopup(targetContext, targetFrame, &parentView);
+            GetRootViewForPopup(targetContext, targetFrame, PR_TRUE, &parentView);
             GetWidgetForView(parentView, *getter_AddRefs(targetDocumentWidget));
           }
         }
@@ -686,8 +705,7 @@ nsMenuPopupFrame::AdjustPositionForAnchorAlign ( PRInt32* ioXPos, PRInt32* ioYPo
   nsAutoString popupAnchor(aPopupAnchor);
   nsAutoString popupAlign(aPopupAlign);
 
-  const nsStyleVisibility* vis = (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
-  if (vis->mDirection == NS_STYLE_DIRECTION_RTL) {
+  if (GetStyleVisibility()->mDirection == NS_STYLE_DIRECTION_RTL) {
     if (popupAnchor == NS_LITERAL_STRING("topright"))
       popupAnchor.Assign(NS_LITERAL_STRING("topleft"));
     else if (popupAnchor == NS_LITERAL_STRING("topleft"))
@@ -708,10 +726,8 @@ nsMenuPopupFrame::AdjustPositionForAnchorAlign ( PRInt32* ioXPos, PRInt32* ioYPo
   }
 
   // Adjust position for margins at the aligned corner
-  const nsStyleMargin* smargin = 
-    (const nsStyleMargin*)mStyleContext->GetStyleData(eStyleStruct_Margin);
   nsMargin margin;
-  smargin->GetMargin(margin);
+  GetStyleMargin()->GetMargin(margin);
   if (popupAlign == NS_LITERAL_STRING("topleft")) {
     *ioXPos += margin.left;
     *ioYPos += margin.top;
@@ -892,8 +908,7 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
   // |view|
   //   The root view for the popup window widget associated with this frame,
   //   or, the view associated with this frame. 
-  nsIView* view = nsnull;
-  GetView(aPresContext, &view);
+  nsIView* view = GetView(aPresContext);
 
   ///////////////////////////////////////////////////////////////////////////////
   //
@@ -963,7 +978,7 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
   nsCOMPtr<nsIContent> parentContent;
   aFrame->GetContent(getter_AddRefs(parentContent));
   nsCOMPtr<nsIAtom> tag;
-  mContent->GetTag(*getter_AddRefs(tag));
+  mContent->GetTag(getter_AddRefs(tag));
   PRBool sizedToPopup = (tag != nsXULAtoms::tooltip)
                         && (nsMenuFrame::IsSizedToPopup(parentContent, PR_FALSE));
   
@@ -998,9 +1013,7 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
     ypos = NSIntPixelsToTwips(newYPos, p2t);
 
     // Add in the top and left margins
-    const nsStyleMargin* smargin = 
-      (const nsStyleMargin*)mStyleContext->GetStyleData(eStyleStruct_Margin);
-    smargin->GetMargin(margin);
+    GetStyleMargin()->GetMargin(margin);
     
     xpos += margin.left;
     ypos += margin.top;
@@ -1070,7 +1083,7 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
   //   frames inside a toplevel window, this is the root view of the toplevel
   //   window.
   nsIView* parentView = nsnull;
-  GetRootViewForPopup(aPresContext, aFrame, &parentView);
+  GetRootViewForPopup(aPresContext, aFrame, PR_FALSE, &parentView);
   if (!parentView)
     return NS_OK;
 
@@ -1218,11 +1231,6 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
     // means necessary
     //
 
-    // add back in the parentPos offset. Not sure why, but we need this for mail/news
-    // context menus and we can't do this in the case where there popup is anchored.
-    screenViewLocX += parentPos.x;
-    screenViewLocY += parentPos.y;    
-    
     // shrink to fit onto the screen, vertically and horizontally
     if(mRect.width > screenWidthTwips) 
         mRect.width = screenWidthTwips;    
@@ -1448,14 +1456,14 @@ NS_IMETHODIMP nsMenuPopupFrame::ConsumeOutsideClicks(PRBool& aConsumeOutsideClic
   aConsumeOutsideClicks = PR_TRUE;
 
   nsCOMPtr<nsIContent> parentContent;
-  mContent->GetParent(*getter_AddRefs(parentContent));
+  mContent->GetParent(getter_AddRefs(parentContent));
 
   if (parentContent) {
     nsCOMPtr<nsIAtom> parentTag;
-    parentContent->GetTag(*getter_AddRefs(parentTag));
+    parentContent->GetTag(getter_AddRefs(parentTag));
     if (parentTag == nsXULAtoms::menulist)
       return NS_OK;  // Consume outside clicks for combo boxes on all platforms
-    if (parentTag == nsXULAtoms::menu) {
+    if (parentTag == nsXULAtoms::menu || parentTag == nsXULAtoms::popupset) {
 #ifdef XP_WIN
       // Don't consume outside clicks for menus in Windows
       aConsumeOutsideClicks = PR_FALSE;
@@ -1486,9 +1494,9 @@ nsIScrollableView* nsMenuPopupFrame::GetScrollableView(nsIFrame* aStart)
   // try start frame and siblings
   currFrame=aStart;
   do {
-    currFrame->GetView(mPresContext, &view);
+    view = currFrame->GetView(mPresContext);
     if ( view )
-      view->QueryInterface(NS_GET_IID(nsIScrollableView), (void**)&scrollableView);
+      CallQueryInterface(view, &scrollableView);
     if ( scrollableView )
       return scrollableView;
     currFrame->GetNextSibling(&currFrame);
@@ -1594,7 +1602,7 @@ NS_IMETHODIMP nsMenuPopupFrame::SetCurrentMenuItem(nsIMenuFrame* aMenuItem)
 NS_IMETHODIMP
 nsMenuPopupFrame::Escape(PRBool& aHandledFlag)
 {
-  mIncrementalString = NS_LITERAL_STRING("");
+  mIncrementalString.Truncate();
 
   // See if we have a context menu open.
   nsCOMPtr<nsIMenuParent> contextMenu;
@@ -1633,7 +1641,7 @@ nsMenuPopupFrame::Escape(PRBool& aHandledFlag)
 NS_IMETHODIMP
 nsMenuPopupFrame::Enter()
 {
-  mIncrementalString = NS_LITERAL_STRING("");
+  mIncrementalString.Truncate();
 
   // See if we have a context menu open.
   nsCOMPtr<nsIMenuParent> contextMenu;
@@ -1694,10 +1702,10 @@ nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, PRBool& doActi
   nsIMenuFrame* frameShortcut = nsnull;
 
   nsCOMPtr<nsIContent> parentContent;
-  mContent->GetParent(*getter_AddRefs(parentContent));
+  mContent->GetParent(getter_AddRefs(parentContent));
   if (parentContent) {
     nsCOMPtr<nsIAtom> tag;
-    parentContent->GetTag(*getter_AddRefs(tag));
+    parentContent->GetTag(getter_AddRefs(tag));
     if (tag != nsXULAtoms::menulist)
       isMenu = PR_TRUE;
   }
@@ -1723,20 +1731,26 @@ nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, PRBool& doActi
     return nsnull;
   }
   else {
-    nsAutoString pressKey(NS_STATIC_CAST(PRUnichar, charCode));
-    // UpperCase to compare-no-case
-    ToUpperCase(pressKey);
+    PRUnichar uniChar = ToLowerCase(NS_STATIC_CAST(PRUnichar, charCode));
     if (isMenu || // Menu supports only first-letter navigation
         keyTime - lastKeyTime > INC_TYP_INTERVAL) // Interval too long, treat as new typing
-      mIncrementalString = pressKey;
+      mIncrementalString = uniChar;
     else {
-      if (mIncrementalString.Length() != 1 ||
-          mIncrementalString.First() != pressKey.First())
-          // If user typed the same key more than once, we should do a cycled one-key navigation
-      mIncrementalString.do_AppendFromElement(pressKey.First());
+      mIncrementalString.Append(uniChar);
     }
   }
 
+  // See bug 188199 & 192346, if all letters in incremental string are same, just try to match the first one
+  nsAutoString incrementalString(mIncrementalString);
+  PRUint32 charIndex = 1, stringLength = incrementalString.Length();
+  while (charIndex < stringLength && incrementalString[charIndex] == incrementalString[charIndex - 1]) {
+    charIndex++;
+  }
+  if (charIndex == stringLength) {
+    incrementalString.Truncate(1);
+    stringLength = 1;
+  }
+
   lastKeyTime = keyTime;
 
   nsIFrame* currFrame;
@@ -1767,8 +1781,7 @@ nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, PRBool& doActi
       else
         isShortcut = PR_TRUE;
 
-      ToUpperCase(textKey);
-      if (Substring(textKey, 0, mIncrementalString.Length()) == mIncrementalString) {
+      if (Substring(textKey, 0, stringLength).Equals(incrementalString, nsCaseInsensitiveStringComparator())) {
         // mIncrementalString is a prefix of textKey
         nsCOMPtr<nsIMenuFrame> menuFrame = do_QueryInterface(currFrame);
         if (menuFrame) {
@@ -1799,7 +1812,7 @@ nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, PRBool& doActi
       current->GetAttr(kNameSpaceID_None, nsXULAtoms::menuactive, activeKey);
       if (activeKey == NS_LITERAL_STRING("true")) {
         foundActive = PR_TRUE;
-        if (mIncrementalString.Length() > 1) {
+        if (stringLength > 1) {
           // If there is more than one char typed, the current item has highest priority,
           //   otherwise the item next to current has highest priority
           nsCOMPtr<nsIMenuFrame> menuFrame = do_QueryInterface(currFrame);
@@ -1877,7 +1890,7 @@ nsMenuPopupFrame::KeyboardNavigation(PRUint32 aKeyCode, PRBool& aHandledFlag)
   nsNavigationDirection theDirection;
   NS_DIRECTION_FROM_KEY_CODE(theDirection, aKeyCode);
 
-  mIncrementalString = NS_LITERAL_STRING("");
+  mIncrementalString.Truncate();
 
   // This method only gets called if we're open.
   if (!mCurrentMenu && NS_DIRECTION_IS_INLINE(theDirection)) {
@@ -2050,7 +2063,8 @@ nsMenuPopupFrame::GetWidget(nsIWidget **aWidget)
 {
   // Get parent view
   nsIView * view = nsnull;
-  nsMenuPopupFrame::GetRootViewForPopup(mPresContext, this, &view);
+  // XXX should this be passing PR_FALSE or PR_TRUE for aStopAtViewManagerRoot?
+  nsMenuPopupFrame::GetRootViewForPopup(mPresContext, this, PR_FALSE, &view);
   if (!view)
     return NS_OK;
 
@@ -2075,7 +2089,7 @@ nsMenuPopupFrame::InstallKeyboardNavigator()
     return NS_OK;
 
   nsCOMPtr<nsIDocument> doc;
-  mContent->GetDocument(*getter_AddRefs(doc));
+  mContent->GetDocument(getter_AddRefs(doc));
   nsCOMPtr<nsIDOMEventReceiver> target = do_QueryInterface(doc);
   
   mTarget = target;
@@ -2110,7 +2124,7 @@ PRBool
 nsMenuPopupFrame::IsValidItem(nsIContent* aContent)
 {
   nsCOMPtr<nsIAtom> tag;
-  aContent->GetTag(*getter_AddRefs(tag));
+  aContent->GetTag(getter_AddRefs(tag));
   if (tag && (tag.get() == nsXULAtoms::menu ||
               tag.get() == nsXULAtoms::menuitem ||
               tag.get() == nsHTMLAtoms::option))
@@ -2280,7 +2294,9 @@ nsMenuPopupFrame::Notify(nsITimer* aTimer)
       } // if not the menu with an open submenu
     } // if menu open
     
-    mCloseTimer->Cancel();
+    if (mCloseTimer) {
+      mCloseTimer->Cancel();
+    }
   }
   
   mCloseTimer = nsnull;
@@ -2324,8 +2340,7 @@ nsMenuPopupFrame::MoveTo(PRInt32 aLeft, PRInt32 aTop)
   mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::left, left, PR_FALSE);
   mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::top, top, PR_FALSE);
 
-  nsIView* view = nsnull;
-  GetView(mPresContext, &view);   
+  nsIView* view = GetView(mPresContext);   
   
   // Retrieve screen position of parent view
   nsIView* parentView = nsnull;
