@@ -43,7 +43,7 @@
 //
 
 #include "nsStackFrame.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsIPresContext.h"
 #include "nsIContent.h"
 #include "nsCOMPtr.h"
@@ -107,7 +107,9 @@ nsStackFrame::GetFrameForPoint(nsIPresContext* aPresContext,
   GetInset(im);
   nsMargin border(0,0,0,0);
   nsStyleBorderPadding  bPad;
-  aFrame->GetStyle(eStyleStruct_BorderPaddingShortcut, (nsStyleStruct&)bPad);
+  nsStyleContext* styleContext;
+  aFrame->GetStyleContext(&styleContext);
+  styleContext->GetBorderPaddingFor(bPad);
   bPad.GetBorderPadding(borderPadding);
   r.Deflate(im);
   r.Deflate(border);    
@@ -134,13 +136,12 @@ nsStackFrame::GetFrameForPoint(nsIPresContext* aPresContext,
       rv = NS_ERROR_FAILURE;
 
   if (NS_FAILED(rv)) {
-        const nsStyleColor* color =
-    (const nsStyleColor*)mStyleContext->GetStyleData(eStyleStruct_Color);
+      const nsStyleBackground* color = GetStyleBackground();
 
       PRBool        transparentBG = NS_STYLE_BG_COLOR_TRANSPARENT ==
                                     (color->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT);
 
-      PRBool backgroundImage = (color->mBackgroundImage.Length() > 0);
+      PRBool backgroundImage = !color->mBackgroundImage.IsEmpty();
 
       if (!transparentBG || backgroundImage)
       {
