@@ -57,8 +57,7 @@
 #include "nsPIDOMWindow.h"
 #include "nsIPrompt.h"
 #include "nsIAuthPrompt.h"
-#include "nsICharsetConverterManager.h"
-#include "nsICharsetConverterManager2.h"
+#include "nsICharsetAlias.h"
 #include "nsIChannel.h"
 #include "nsIRequestObserver.h"
 #include "netCore.h"
@@ -70,7 +69,6 @@
 // XXX Remove
 #include "nsIWebShell.h"
 
-
 static NS_DEFINE_CID(kTransactionManagerCID, NS_TRANSACTIONMANAGER_CID);
 static NS_DEFINE_CID(kComponentManagerCID,  NS_COMPONENTMANAGER_CID);
 
@@ -83,7 +81,6 @@ nsMsgWindow::nsMsgWindow()
 {
   mCharsetOverride = PR_FALSE;
   m_stopped = PR_FALSE;
-  NS_INIT_ISUPPORTS();
 }
 
 nsMsgWindow::~nsMsgWindow()
@@ -248,11 +245,8 @@ NS_IMETHODIMP nsMsgWindow::SetMsgHeaderSink(nsIMsgHeaderSink * aMsgHdrSink)
 
 NS_IMETHODIMP nsMsgWindow::GetTransactionManager(nsITransactionManager * *aTransactionManager)
 {
-	if(!aTransactionManager)
-		return NS_ERROR_NULL_POINTER;
-
-	*aTransactionManager = mTransactionManager;
-	NS_IF_ADDREF(*aTransactionManager);
+	NS_ENSURE_ARG_POINTER(aTransactionManager);
+	NS_IF_ADDREF(*aTransactionManager = mTransactionManager);
 	return NS_OK;
 }
 
@@ -264,13 +258,9 @@ NS_IMETHODIMP nsMsgWindow::SetTransactionManager(nsITransactionManager * aTransa
 
 NS_IMETHODIMP nsMsgWindow::GetOpenFolder(nsIMsgFolder * *aOpenFolder)
 {
-	if(!aOpenFolder)
-		return NS_ERROR_NULL_POINTER;
-
-	*aOpenFolder = mOpenFolder;
-	NS_IF_ADDREF(*aOpenFolder);
+	NS_ENSURE_ARG_POINTER(aOpenFolder);
+	NS_IF_ADDREF(*aOpenFolder = mOpenFolder);
 	return NS_OK;
-
 }
 
 NS_IMETHODIMP nsMsgWindow::SetOpenFolder(nsIMsgFolder * aOpenFolder)
@@ -309,33 +299,31 @@ NS_IMETHODIMP nsMsgWindow::SetRootDocShell(nsIDocShell * aDocShell)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgWindow::GetMailCharacterSet(PRUnichar * *aMailCharacterSet)
+NS_IMETHODIMP nsMsgWindow::GetMailCharacterSet(char * *aMailCharacterSet)
 {
   if(!aMailCharacterSet)
     return NS_ERROR_NULL_POINTER;
 
-  *aMailCharacterSet = ToNewUnicode(mMailCharacterSet);
+  *aMailCharacterSet = ToNewCString(mMailCharacterSet);
   if (!(*aMailCharacterSet))
     return NS_ERROR_OUT_OF_MEMORY;
 
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgWindow::SetMailCharacterSet(const PRUnichar * aMailCharacterSet)
+NS_IMETHODIMP nsMsgWindow::SetMailCharacterSet(const char * aMailCharacterSet)
 {
   mMailCharacterSet.Assign(aMailCharacterSet);
 
   // Convert to a canonical charset name instead of using the charset name from the message header as is.
   // This is needed for charset menu item to have a check mark correctly.
   nsresult rv;
-  nsCOMPtr<nsICharsetConverterManager2> ccm2 = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
+  nsCOMPtr<nsICharsetAlias> calias =
+      do_GetService(NS_CHARSETALIAS_CONTRACTID, &rv);
+  
   if (NS_SUCCEEDED(rv)) 
-  {
-    nsCOMPtr <nsIAtom> charsetAtom;
-    rv = ccm2->GetCharsetAtom(mMailCharacterSet.get(), getter_AddRefs(charsetAtom));
-    if (NS_SUCCEEDED(rv)) 
-      rv = charsetAtom->ToString(mMailCharacterSet);
-  }
+      calias->GetPreferred(nsDependentCString(aMailCharacterSet),
+                           mMailCharacterSet);
 
   return NS_OK;
 }
@@ -427,7 +415,7 @@ NS_IMETHODIMP nsMsgWindow::StopUrls()
 // nsIURIContentListener support
 NS_IMETHODIMP nsMsgWindow::OnStartURIOpen(nsIURI* aURI, PRBool* aAbortOpen)
 {
-   return NS_OK;
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgWindow::DoContent(const char *aContentType, PRBool aIsContentPreferred,
