@@ -68,8 +68,8 @@ public:
   
   NS_IMETHOD StartFolderLoading(void);
   NS_IMETHOD EndFolderLoading(void);
-  NS_IMETHOD GetCharset(PRUnichar * *aCharset);
-  NS_IMETHOD SetCharset(const PRUnichar * aCharset);
+  NS_IMETHOD GetCharset(char * *aCharset);
+  NS_IMETHOD SetCharset(const char * aCharset);
   NS_IMETHOD GetCharsetOverride(PRBool *aCharsetOverride);
   NS_IMETHOD SetCharsetOverride(PRBool aCharsetOverride);
   NS_IMETHOD GetFirstNewMessage(nsIMsgDBHdr **firstNewMessage);
@@ -105,7 +105,7 @@ public:
 
   NS_IMETHOD GetSupportsOffline(PRBool *aSupportsOffline);
   NS_IMETHOD ShouldStoreMsgOffline(nsMsgKey msgKey, PRBool *result);
-  NS_IMETHOD GetOfflineFileTransport(nsMsgKey msgKey, PRUint32 *offset, PRUint32 *size, nsITransport **_retval);
+  NS_IMETHOD GetOfflineFileStream(nsMsgKey msgKey, PRUint32 *offset, PRUint32 *size, nsIInputStream **_retval);
   NS_IMETHOD HasMsgOffline(nsMsgKey msgKey, PRBool *result);
   NS_IMETHOD DownloadMessagesForOffline(nsISupportsArray *messages, nsIMsgWindow *msgWindow);
   NS_IMETHOD DownloadAllForOffline(nsIUrlListener *listener, nsIMsgWindow *msgWindow);
@@ -123,7 +123,8 @@ public:
   NS_IMETHOD GetStringProperty(const char *propertyName, char **propertyValue);
   NS_IMETHOD SetStringProperty(const char *propertyName, const char *propertyValue);
   NS_IMETHOD CallFilterPlugins(nsIMsgWindow *aMsgWindow);
-
+  NS_IMETHOD GetLastMessageLoaded(nsMsgKey *aMsgKey);
+  NS_IMETHOD SetLastMessageLoaded(nsMsgKey aMsgKey);
 
 protected:
   virtual nsresult ReadDBFolderInfo(PRBool force);
@@ -160,6 +161,7 @@ protected:
   PRBool mAddListener;
   PRBool mNewMessages;
   PRBool mGettingNewMessages;
+  nsMsgKey mLastMessageLoaded;
 
   nsCOMPtr <nsIMsgDBHdr> m_offlineHeader;
   PRInt32 m_numOfflineMsgLines;
@@ -171,6 +173,7 @@ protected:
   static nsIAtom* mFolderLoadedAtom;
   static nsIAtom* mDeleteOrMoveMsgCompletedAtom;
   static nsIAtom* mDeleteOrMoveMsgFailedAtom;
+  static nsIAtom* mJunkStatusChangedAtom;
   static nsrefcnt mInstanceCount;
 };
 
