@@ -82,9 +82,8 @@
 
 #define PREF_MAIL_WARN_FILTER_CHANGED "mail.warn_filter_changed"
 
-static NS_DEFINE_CID(kStandardUrlCID, NS_STANDARDURL_CID);
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
-static NS_DEFINE_CID(kIOServiceCID,              NS_IOSERVICE_CID);
+static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
 static NS_DEFINE_CID(kCollationFactoryCID, NS_COLLATIONFACTORY_CID);
 
 nsrefcnt nsMsgFolder::gInstanceCount  = 0;
@@ -156,7 +155,7 @@ nsMsgFolder::nsMsgFolder(void)
     initializeStrings();
     createCollationKeyGenerator();
 #ifdef MSG_FASTER_URI_PARSING
-    mParsingURL = do_CreateInstance(kStandardUrlCID);
+    mParsingURL = do_CreateInstance(NS_STANDARDURL_CONTRACTID);
 #endif
   }
 
@@ -184,14 +183,15 @@ nsMsgFolder::~nsMsgFolder(void)
 
   gInstanceCount--;
   if (gInstanceCount <= 0) {
-    NS_IF_RELEASE(kTotalMessagesAtom);
     NS_IF_RELEASE(kBiffStateAtom);
     NS_IF_RELEASE(kNewMessagesAtom);
     NS_IF_RELEASE(kNumNewBiffMessagesAtom);
-    NS_IF_RELEASE(kTotalUnreadMessagesAtom);
-    NS_IF_RELEASE(kFlaggedAtom);
-    NS_IF_RELEASE(kStatusAtom);
     NS_IF_RELEASE(kNameAtom);
+    NS_IF_RELEASE(kTotalUnreadMessagesAtom);
+    NS_IF_RELEASE(kTotalMessagesAtom);
+    NS_IF_RELEASE(kFolderSizeAtom);
+    NS_IF_RELEASE(kStatusAtom);
+    NS_IF_RELEASE(kFlaggedAtom);
     NS_IF_RELEASE(kSynchronizeAtom);
     NS_IF_RELEASE(kOpenAtom);
     NS_IF_RELEASE(kCollationKeyGenerator);
@@ -411,7 +411,7 @@ nsMsgFolder::GetSubFolders(nsIEnumerator* *result)
 }
 
 NS_IMETHODIMP
-nsMsgFolder::FindSubFolder(const char *subFolderName, nsIFolder **aFolder)
+nsMsgFolder::FindSubFolder(const char *aEscapedSubFolderName, nsIFolder **aFolder)
 {
   nsresult rv = NS_OK;
   nsCOMPtr<nsIRDFService> rdf(do_GetService(kRDFServiceCID, &rv));
@@ -419,15 +419,14 @@ nsMsgFolder::FindSubFolder(const char *subFolderName, nsIFolder **aFolder)
   if (NS_FAILED(rv))
     return rv;
 
-// XXX use necko here
+  // XXX use necko here
   nsCAutoString uri;
   uri.Append(mURI);
   uri.Append('/');
-
-  uri.Append(subFolderName);
+  uri.Append(aEscapedSubFolderName);
 
   nsCOMPtr<nsIRDFResource> res;
-  rv = rdf->GetResource(uri.get(), getter_AddRefs(res));
+  rv = rdf->GetResource(uri, getter_AddRefs(res));
   if (NS_FAILED(rv))
     return rv;
 
@@ -587,7 +586,7 @@ nsMsgFolder::parseURI(PRBool needServer)
 #ifdef MSG_FASTER_URI_PARSING
   nsMsgAutoBool parsingUrlState;
   if (mParsingURLInUse) {
-    url = do_CreateInstance(kStandardUrlCID, &rv);
+    url = do_CreateInstance(NS_STANDARDURL_CONTRACTID, &rv);
   }
 
   else {
@@ -597,9 +596,7 @@ nsMsgFolder::parseURI(PRBool needServer)
   }
 
 #else
-  rv = nsComponentManager::CreateInstance(kStandardUrlCID, nsnull,
-                                          NS_GET_IID(nsIURL),
-                                          (void **)getter_AddRefs(url));
+  url = do_CreateInstance(NS_STANDARDURL_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
 #endif
 
@@ -1048,43 +1045,6 @@ NS_IMETHODIMP nsMsgFolder::GetPrettiestName(PRUnichar **name)
   return GetName(name);
 }
 
-static PRBool
-nsCanBeInFolderPane(nsISupports* element, void* data)
-{
-#ifdef HAVE_PANE
-  nsIMsgFolder* subFolder = NS_STATIC_CAST(nsIMsgFolder*, element);
-  return subFolder->CanBeInFolderPane();
-#else
-  return PR_TRUE;
-#endif
-}
-
-NS_IMETHODIMP
-nsMsgFolder::GetVisibleSubFolders(nsIEnumerator* *result)
-{
-  nsresult rv;
-  nsCOMPtr<nsISupportsArray> vFolders;
-  rv = nsFilterBy(mSubFolders, nsCanBeInFolderPane, nsnull, getter_AddRefs(vFolders));
-  if (NS_FAILED(rv)) return rv;
-  rv = vFolders->Enumerate(result);
-  return rv;
-}
-
-#ifdef HAVE_ADMINURL
-NS_IMETHODIMP nsMsgFolder::GetAdminUrl(MWContext *context, MSG_AdminURLType type)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsMsgFolder::HaveAdminUrl(MSG_AdminURLType type, PRBool *haveAdminUrl)
-{
-  if (!haveAdminUrl)
-    return NS_ERROR_NULL_POINTER;
-
-  *haveAdminUrl = PR_FALSE;
-  return NS_OK;
-}
-#endif
 
 NS_IMETHODIMP nsMsgFolder::GetShowDeletedMessages(PRBool *showDeletedMessages)
 {
@@ -1252,10 +1212,6 @@ NS_IMETHODIMP nsMsgFolder::Rename(const PRUnichar *name, nsIMsgWindow *msgWindow
   status = SetName((PRUnichar *) unicharString.get());
   //After doing a SetName we need to make sure that broadcasting this message causes a
   //new sort to happen.
-#ifdef HAVE_MASTER
-  if (m_master)
-    m_master->BroadcastFolderChanged(this);
-#endif
   return status;
 
 }
@@ -1356,10 +1312,6 @@ NS_IMETHODIMP nsMsgFolder::UpdateSummaryTotals(PRBool /* force */)
 NS_IMETHODIMP nsMsgFolder::SummaryChanged()
 {
   UpdateSummaryTotals(PR_FALSE);
-#ifdef HAVE_MASTER
-    if (mMaster)
-        mMaster->BroadcastFolderChanged(this);
-#endif
   return NS_OK;
 }
 
@@ -1722,20 +1674,6 @@ NS_IMETHODIMP nsMsgFolder::GetExpansionArray(nsISupportsArray *expansionArray)
   return NS_OK;
 }
 
-#ifdef HAVE_PANE
-NS_IMETHODIMP nsMsgFolder::SetFlagInAllFolderPanes(PRUInt32 which)
-{
-
-}
-
-#endif
-
-#ifdef HAVE_NET
-NS_IMETHODIMP nsMsgFolder::EscapeMessageId(const char *messageId, const char **escapeMessageID)
-{
-
-}
-#endif
 
 NS_IMETHODIMP nsMsgFolder::GetExpungedBytes(PRUint32 *count)
 {
@@ -1813,9 +1751,6 @@ NS_IMETHODIMP nsMsgFolder::GetDisplayRecipients(PRBool *displayRecipients)
       // There's one FCC folder for sent mail, and one for sent news
       nsIMsgFolder *fccFolders[2];
       int numFccFolders = 0;
-#ifdef HAVE_MASTER
-      m_master->GetFolderTree()->GetFoldersWithFlag (MSG_FOLDER_FLAG_SENTMAIL, fccFolders, 2, &numFccFolders);
-#endif
       for (int i = 0; i < numFccFolders; i++)
       {
         PRBool isAncestor;
@@ -1872,13 +1807,6 @@ NS_IMETHODIMP nsMsgFolder::GetLocked(PRBool *isLocked)
   return  NS_OK;
 }
 
-#ifdef HAVE_PANE
-    MWContext *GetFolderPaneContext();
-#endif
-
-#ifdef HAVE_MASTER
-    MSG_Master  *GetMaster() {return m_master;}
-#endif
 
 NS_IMETHODIMP nsMsgFolder::GetRelativePathName(char **pathName)
 {
@@ -1906,29 +1834,6 @@ NS_IMETHODIMP nsMsgFolder::SetSizeOnDisk(PRUint32 aSizeOnDisk)
   return NS_OK;
 }
 
-#ifdef HAVE_NET
-NS_IMETHODIMP nsMsgFolder::ShouldPerformOperationOffline(PRBool *performOffline)
-{
-
-}
-#endif
-
-
-#ifdef DOES_FOLDEROPERATIONS
-NS_IMETHODIMP nsMsgFolder::DownloadToTempFileAndUpload(MessageCopyInfo *copyInfo,
-                                                       nsMsgKeyArray &keysToSave,
-                                                       MSG_FolderInfo *dstFolder,
-                                                       nsMsgDatabase *sourceDB)
-{
-
-}
-
-NS_IMETHODIMP nsMsgFolder::UpdateMoveCopyStatus(MWContext *context, PRBool isMove, int32 curMsgCount, int32 totMessages)
-{
-
-}
-
-#endif
 
 NS_IMETHODIMP nsMsgFolder::RememberPassword(const char *password)
 {
