@@ -20,6 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Lorenzo Colitti <lorenzo@colitti.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -322,14 +323,9 @@ public:
   NS_IMETHOD AddSearchResult(nsIImapProtocol* aProtocol, 
                          const char* searchHitLine);
   NS_IMETHOD HeaderFetchCompleted(nsIImapProtocol* aProtocol);
-  NS_IMETHOD UpdateSecurityStatus(nsIImapProtocol* aProtocol);
   // ****
   NS_IMETHOD SetBiffStateAndUpdate(nsIImapProtocol* aProtocol,
                                nsMsgBiffState biffState);
-  NS_IMETHOD GetStoredUIDValidity(nsIImapProtocol* aProtocol,
-                              uid_validity_info* aInfo);
-  NS_IMETHOD LiteSelectUIDValidity(nsIImapProtocol* aProtocol,
-                               PRUint32 uidValidity);
   NS_IMETHOD ProgressStatus(nsIImapProtocol* aProtocol,
                         PRUint32 aMsgId, const PRUnichar *extraInfo);
   NS_IMETHOD PercentProgress(nsIImapProtocol* aProtocol,
@@ -344,7 +340,7 @@ public:
   
   nsresult MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr, 
                              nsIMsgDatabase *sourceDB, 
-                             const char *destFolder,
+                             const nsACString& destFolder,
                              nsIMsgFilter *filter,
                              nsIMsgWindow *msgWindow);
   virtual nsresult SpamFilterClassifyMessage(const char *aURI, nsIMsgWindow *aMsgWindow, nsIJunkMailPlugin *aJunkMailPlugin);
@@ -364,7 +360,8 @@ protected:
 	void TweakHeaderFlags(nsIImapProtocol* aProtocol, nsIMsgDBHdr *tweakMe);
 
 	nsresult SyncFlags(nsIImapFlagAndUidState *flagState);
-        nsresult NotifyMessageFlagsFromHdr(nsIMsgDBHdr *dbHdr, nsMsgKey msgKey, PRUint32 flags);
+  nsresult HandleCustomFlags(nsMsgKey uidOfMessage, nsIMsgDBHdr *dbHdr, nsXPIDLCString &keywords);
+  nsresult NotifyMessageFlagsFromHdr(nsIMsgDBHdr *dbHdr, nsMsgKey msgKey, PRUint32 flags);
 
   nsresult SetupHeaderParseStream(PRUint32 size, const char *content_type, nsIMailboxSpec *boxSpec);
   nsresult  ParseAdoptedHeaderLine(const char *messageLine, PRUint32 msgKey);
@@ -477,6 +474,8 @@ protected:
   PRPackedBool m_folderNeedsAdded;
   PRPackedBool m_folderNeedsACLListed;
   PRPackedBool m_performingBiff;
+  PRPackedBool m_folderQuotaCommandIssued;
+  PRPackedBool m_folderQuotaDataIsValid;
 
   nsMsgIMAPFolderACL *m_folderACL;
 
@@ -487,6 +486,11 @@ protected:
   // offline imap support
   PRBool m_downloadMessageForOfflineUse;
   PRBool m_downloadingFolderForOfflineUse;
+
+  // Quota support
+  nsCString m_folderQuotaRoot;
+  PRUint32 m_folderQuotaUsedKB;
+  PRUint32 m_folderQuotaMaxKB;
 };
 
 
