@@ -45,19 +45,33 @@
 #include <files.h>
 #include <TextUtils.h>
 
+#ifdef XP_MACOSX
+#include "nsILocalFileMac.h"
+#include "MoreFilesX.h"
+
+static nsresult NS_FileSpecToILocalFileMac(nsFileSpec *aSpec, nsILocalFileMac **aLocalFileMac)
+{
+  nsCOMPtr<nsILocalFile> localFile;
+  nsresult rv = NS_FileSpecToIFile(aSpec, getter_AddRefs(localFile));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsCOMPtr<nsILocalFileMac> macFile = do_QueryInterface(localFile);
+  if (!macFile)
+    return NS_ERROR_FAILURE;
+   
+  NS_IF_ADDREF(*aLocalFileMac = macFile);
+  return NS_OK;
+}
+#else
 #include "MoreFiles.h"
 #include "MoreFilesExtras.h"
+#endif
 
 static NS_DEFINE_IID(kISupportsIID,			NS_ISUPPORTS_IID);
 static NS_DEFINE_CID(kComponentManagerCID, 	NS_COMPONENTMANAGER_CID);
-static NS_DEFINE_CID(kSmtpServiceCID,		NS_SMTPSERVICE_CID); 
-static NS_DEFINE_CID(kMsgAccountMgrCID,		NS_MSGACCOUNTMANAGER_CID);
-
 
 static const char *	kWhitespace = "\b\t\r\n ";
 
-
-
 nsEudoraMac::nsEudoraMac()
 {
 	m_mailImportLocation = nsnull;
@@ -125,7 +139,19 @@ PRBool nsEudoraMac::FindEudoraLocation( nsIFileSpec *pFolder, PRBool findIni, ns
 					if (NS_SUCCEEDED( rv)) {
 						// find a file with TEXT, CSOm that isn't the nicknames file
 						// or just cheat and look for more than 1 file?
+#ifdef XP_MACOSX
+  {
+    nsCOMPtr<nsILocalFileMac> macFile;
+    rv = NS_FileSpecToILocalFileMac(&spec, getter_AddRefs(macFile));
+    if (NS_SUCCEEDED(rv))
+    {
+      macFile->GetFileCreator(&creator);
+      macFile->GetFileType(&type);
+    }
+  }
+#else
 						rv = spec.GetFileTypeAndCreator( &type, &creator);
+#endif
 						if (NS_SUCCEEDED( rv)) {
 							if ((type == 'TEXT') && (creator == 'CSOm'))
 								count++;
@@ -140,7 +166,7 @@ PRBool nsEudoraMac::FindEudoraLocation( nsIFileSpec *pFolder, PRBool findIni, ns
 									PRBool	isBk = PR_FALSE;
 									PRInt32 len;
 									if (pLeafName) {
-										len = nsCRT::strlen( pLeafName);
+										len = strlen( pLeafName);
 										if (len > 5)
 											isBk = (nsCRT::strcasecmp( pLeafName + len - 5, ".bkup") == 0);
 										nsCRT::free( pLeafName);
@@ -148,7 +174,7 @@ PRBool nsEudoraMac::FindEudoraLocation( nsIFileSpec *pFolder, PRBool findIni, ns
 									if (!isBk) {
 										pLeafName = pref.GetLeafName();
 										if (pLeafName) {
-											len = nsCRT::strlen( pLeafName);
+											len = strlen( pLeafName);
 											if (len > 5)
 												isBk = (nsCRT::strcasecmp( pLeafName + len - 5, ".bkup") == 0);
 											nsCRT::free( pLeafName);
@@ -285,7 +311,19 @@ nsresult nsEudoraMac::IterateMailDir( nsIFileSpec *pFolder, nsISupportsArray *pA
 					if (NS_SUCCEEDED( rv)) {
 						type = 0;
 						creator = 0;
-						spec.GetFileTypeAndCreator( &type, &creator);
+#ifdef XP_MACOSX
+  {
+    nsCOMPtr<nsILocalFileMac> macFile;
+    rv = NS_FileSpecToILocalFileMac(&spec, getter_AddRefs(macFile));
+    if (NS_SUCCEEDED(rv))
+    {
+      macFile->GetFileCreator(&creator);
+      macFile->GetFileType(&type);
+    }
+  }
+#else
+    spec.GetFileTypeAndCreator( &type, &creator);
+#endif
 						if ((type == 'TEXT') && IsValidMailboxName( fName) && IsValidMailboxFile( entry)) {
 							rv = FoundMailbox( entry, fName.get(), pArray, pImport);
 						}
@@ -394,7 +432,24 @@ PRBool nsEudoraMac::CreateTocFromResource( nsIFileSpec *pMail, nsIFileSpec *pToc
 	nsresult rv = pMail->GetFileSpec( &spec);
 	if (NS_FAILED( rv))
 		return( PR_FALSE);
-	short resFile = FSpOpenResFile( spec.GetFSSpecPtr(), fsRdPerm);
+        short resFile = -1;
+#ifdef XP_MACOSX
+        {
+          nsCOMPtr<nsILocalFileMac> macFile;
+          rv = NS_FileSpecToILocalFileMac(&spec, getter_AddRefs(macFile));
+          if (NS_FAILED(rv))
+            return PR_FALSE;
+
+          FSSpec fsSpec;
+          rv = macFile->GetFSSpec(&fsSpec);
+	  if (NS_FAILED(rv))
+            return PR_FALSE;
+
+          resFile = FSpOpenResFile( &fsSpec, fsRdPerm);
+        }
+#else
+        resFile = FSpOpenResFile( spec.GetFSSpecPtr(), fsRdPerm);
+#endif
 	if (resFile == -1)
 		return( PR_FALSE);
 	Handle	resH = nil;
@@ -468,7 +523,19 @@ nsresult nsEudoraMac::FindTOCFile( nsIFileSpec *pMailFile, nsIFileSpec **ppTOCFi
 		nsFileSpec	spec;
 		rv = (*ppTOCFile)->GetFileSpec( &spec);
 		if (NS_SUCCEEDED( rv))
+#ifdef XP_MACOSX
+  {
+    nsCOMPtr<nsILocalFileMac> macFile;
+    rv = NS_FileSpecToILocalFileMac(&spec, getter_AddRefs(macFile));
+    if (NS_SUCCEEDED(rv))
+    {
+      macFile->GetFileCreator(&creator);
+      macFile->GetFileType(&type);
+    }
+  }
+#else
 			spec.GetFileTypeAndCreator( &type, &creator);
+#endif
 	}
 	
 
@@ -515,8 +582,25 @@ PRBool nsEudoraMac::GetSettingsFromResource( nsIFileSpec *pSettings, short resId
 	nsresult rv = pSettings->GetFileSpec( &spec);
 	if (NS_FAILED( rv))
 		return( PR_FALSE);
-		
-	short resFile = FSpOpenResFile( spec.GetFSSpecPtr(), fsRdPerm);
+
+        short resFile = -1;
+#ifdef XP_MACOSX
+        {
+          nsCOMPtr<nsILocalFileMac> macFile;
+          rv = NS_FileSpecToILocalFileMac(&spec, getter_AddRefs(macFile));
+	  if (NS_FAILED(rv))
+            return PR_FALSE;
+
+          FSSpec fsSpec;
+          rv = macFile->GetFSSpec(&fsSpec);
+	  if (NS_FAILED(rv))
+            return PR_FALSE;
+
+          resFile = FSpOpenResFile( &fsSpec, fsRdPerm);
+        }
+#else
+	resFile = FSpOpenResFile( spec.GetFSSpecPtr(), fsRdPerm);
+#endif
 	if (resFile == -1)
 		return( PR_FALSE);
 		
@@ -537,7 +621,7 @@ PRBool nsEudoraMac::GetSettingsFromResource( nsIFileSpec *pSettings, short resId
 		GetIndString( pStr[2], resId, kReturnAddressID); 
 		GetIndString( pStr[3], resId, kFullNameID);
 		GetIndString( pStr[4], resId, kLeaveMailOnServerID);
-		CloseResFile( resFile);
+		CloseResFile( resFile); 
 		
 		theStr = pStr[0];
 		if (*theStr) {
@@ -591,18 +675,17 @@ PRBool nsEudoraMac::GetSettingsFromResource( nsIFileSpec *pSettings, short resId
 		return( PR_TRUE);
 	}
 	else {
-		CloseResFile( resFile);
+		CloseResFile( resFile); 
 		return( PR_FALSE);
 	}
 }
 
 PRBool nsEudoraMac::ImportSettings( nsIFileSpec *pIniFile, nsIMsgAccount **localMailAccount)
 {
-	PRBool		result = PR_FALSE;
 	nsresult	rv;
 
 	nsCOMPtr<nsIMsgAccountManager> accMgr = 
-	         do_GetService(kMsgAccountMgrCID, &rv);
+	         do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
     if (NS_FAILED(rv)) {
 		IMPORT_LOG0( "*** Failed to create a account manager!\n");
 		return( PR_FALSE);
@@ -824,7 +907,7 @@ void nsEudoraMac::SetIdentities(nsIMsgAccountManager *accMgr, nsIMsgAccount *acc
 		IMPORT_LOG1( "\temail: %s\n", pStrs[kReturnAddressStr]->get());
 	}
 
-	SetSmtpServer( accMgr, acc, pStrs[kSmtpServerStr]->get(), nsnull);
+	SetSmtpServer( accMgr, acc, pStrs[kSmtpServerStr]->get(), userName);
 		
 }
 
@@ -833,7 +916,7 @@ void nsEudoraMac::SetSmtpServer( nsIMsgAccountManager *pMgr, nsIMsgAccount *pAcc
 {
 	nsresult	rv;
 	
-	nsCOMPtr<nsISmtpService> smtpService(do_GetService(kSmtpServiceCID, &rv)); 
+	nsCOMPtr<nsISmtpService> smtpService(do_GetService(NS_SMTPSERVICE_CONTRACTID, &rv)); 
 	if (NS_SUCCEEDED(rv) && smtpService) {
 		nsCOMPtr<nsISmtpServer>		foundServer;
 	
@@ -856,10 +939,8 @@ void nsEudoraMac::SetSmtpServer( nsIMsgAccountManager *pMgr, nsIMsgAccount *pAcc
 }
 
 
-
 nsresult nsEudoraMac::GetAttachmentInfo( const char *pFileName, nsIFileSpec *pSpec, nsCString& mimeType)
 {
-
 	mimeType.Truncate();
 
 	// Sample attachment line
@@ -924,7 +1005,37 @@ nsresult nsEudoraMac::GetAttachmentInfo( const char *pFileName, nsIFileSpec *pSp
 	IMPORT_LOG3( "\tAttachment type: %s, creator: %s, fileNum: %ld\n", typeStr.get(), creatStr.get(), fNum);
 	IMPORT_LOG1( "\tAttachment file name: %s\n", str.get());
 #endif
-	
+	FSSpec	spec;
+	memset( &spec, 0, sizeof( spec));
+#ifdef XP_MACOSX
+        {
+          nsresult rv = pSpec->SetNativePath(str.get());
+          if (NS_FAILED(rv)) {
+            IMPORT_LOG0("\tfailed to set native path\n");
+            return rv;
+          }
+  
+          nsFileSpec tempFileSpec;
+          rv = pSpec->GetFileSpec(&tempFileSpec);
+          if (NS_FAILED(rv)) {
+            IMPORT_LOG0("\tfailed to get file spec\n");
+            return rv;
+          }
+
+          nsCOMPtr<nsILocalFileMac> macFile;
+          rv = NS_FileSpecToILocalFileMac(&tempFileSpec, getter_AddRefs(macFile));
+          if (NS_FAILED(rv)) {
+            IMPORT_LOG0("\tfailed to get local mac file\n");
+            return rv;
+          }
+          
+          rv = macFile->GetFSSpec(&spec);
+          if (NS_FAILED(rv)) {
+            IMPORT_LOG0("\tfailed to get FSSpec\n");
+            return rv;
+          }
+        }
+#else
 	// Now we have all of the pertinent info, find out if the file exists?
 	nsCString	fileName;
 	if ((str.CharAt( 0) == '"') && (str.Last() == '"')) {
@@ -943,7 +1054,6 @@ nsresult nsEudoraMac::GetAttachmentInfo( const char *pFileName, nsIFileSpec *pSp
 	
 	// Create a FSSpec from the volume name, fileName, and folderNumber
 	// Assume that we are looking for a file on the volume with macFileId
-	FSSpec	spec;
 	Str63	str63;
 	short	vRefNum = 0;
 	if (volumeName.Length() > 63) {
@@ -961,7 +1071,6 @@ nsresult nsEudoraMac::GetAttachmentInfo( const char *pFileName, nsIFileSpec *pSp
 		return( NS_ERROR_FAILURE);
 	}
 	
-	nsCRT::memset( &spec, 0, sizeof( spec));
 	err = FSpResolveFileIDRef( nil, vRefNum, (long) fNum, &spec);
 	if (err != noErr) {
 		IMPORT_LOG1( "\t*** Error, cannot resolve fileIDRef: %ld\n", (long) err);
@@ -978,18 +1087,23 @@ nsresult nsEudoraMac::GetAttachmentInfo( const char *pFileName, nsIFileSpec *pSp
 	
 	nsFileSpec	fSpec( spec);
 	pSpec->SetFromFileSpec( fSpec);
+#endif
 	
-	// Need to find the mime type for the attachment?
-	long	dataSize = 0;
-	long	rsrcSize = 0;
-	
-	err = FSpGetFileSize( &spec, &dataSize, &rsrcSize);
-	
-	// TLR: FIXME: Need to get the mime type from the Mac file type.
-	// Currently we just applsingle if there is a resource fork, otherwise,
-	// just default to something.
-	
-	if (rsrcSize)
+#ifdef XP_MACOSX
+	if (HasResourceFork(&spec)) 
+#else
+        // Need to find the mime type for the attachment?
+        long    dataSize = 0;
+        long    rsrcSize = 0;
+ 
+        err = FSpGetFileSize( &spec, &dataSize, &rsrcSize);
+ 
+        // TLR: FIXME: Need to get the mime type from the Mac file type.
+        // Currently we just applsingle if there is a resource fork, otherwise,
+        // just default to something.
+ 
+        if (rsrcSize)
+#endif
 		mimeType = "application/applefile";
 	else
 		mimeType = "application/octet-stream";
@@ -998,7 +1112,18 @@ nsresult nsEudoraMac::GetAttachmentInfo( const char *pFileName, nsIFileSpec *pSp
 	
 	return( NS_OK);
 }
-
+		
+PRBool nsEudoraMac::HasResourceFork(FSSpec *fsSpec)
+{
+  FSRef fsRef;
+  if (::FSpMakeFSRef(fsSpec, &fsRef) == noErr)
+  {
+    FSCatalogInfo catalogInfo;
+    OSErr err = ::FSGetCatalogInfo(&fsRef, kFSCatInfoRsrcSizes, &catalogInfo, nsnull, nsnull, nsnull);
+    return (err == noErr && catalogInfo.rsrcLogicalSize != 0);
+  }
+  return PR_FALSE;
+}
 
 
 #define		kNumBadFolderNames		7
@@ -1018,7 +1143,7 @@ PRBool nsEudoraMac::IsValidMailFolderName( nsCString& name)
 		return( PR_TRUE);
 	
 	for (int i = 0; i < kNumBadFolderNames; i++) {
-		if (!name.CompareWithConversion( cBadFolderNames[i], PR_TRUE))
+		if (name.Equals( cBadFolderNames[i], nsCaseInsensitiveCStringComparator()))
 			return( PR_FALSE);
 	}
 	
@@ -1030,7 +1155,7 @@ PRBool nsEudoraMac::IsValidMailboxName( nsCString& fName)
 {
 	if (m_depth > 1)
 		return( PR_TRUE);
-	if (!Compare(fName, NS_LITERAL_CSTRING("Eudora Nicknames"), nsCaseInsensitiveCStringComparator()))
+	if (fName.Equals(NS_LITERAL_CSTRING("Eudora Nicknames"), nsCaseInsensitiveCStringComparator()))
 		return( PR_FALSE);
 	return( PR_TRUE);
 }
@@ -1181,7 +1306,19 @@ nsresult nsEudoraMac::FindAddressBooks( nsIFileSpec *pRoot, nsISupportsArray **p
 				if (NS_SUCCEEDED( rv)) {
 					type = 0;
 					creator = 0;
+#ifdef XP_MACOSX
+  {
+    nsCOMPtr<nsILocalFileMac> macFile;
+    rv = NS_FileSpecToILocalFileMac(&fSpec, getter_AddRefs(macFile));
+    if (NS_SUCCEEDED(rv))
+    {
+      macFile->GetFileCreator(&creator);
+      macFile->GetFileType(&type);
+    }
+  }
+#else
 					fSpec.GetFileTypeAndCreator( &type, &creator);
+#endif
 					if (type == 'TEXT') {
 						rv = impSvc->CreateNewABDescriptor( getter_AddRefs( desc));
 						if (NS_SUCCEEDED( rv)) {
