@@ -57,7 +57,6 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(nsPop3Sink, nsIPop3Sink)
 
 nsPop3Sink::nsPop3Sink()
 {
-    NS_INIT_ISUPPORTS();
     m_authed = PR_FALSE;
     m_accountUrl = nsnull;
     m_biffState = 0;
@@ -257,10 +256,41 @@ nsPop3Sink::EndMailDelivery()
       (void) filterList->FlushLogIfNecessary();
   }
 
+  // fix for bug #161999
+  // we should update the summary totals for the folder (inbox)
+  // in case it's not the open folder
+  m_folder->UpdateSummaryTotals(PR_TRUE);
+ 
+  // check if the folder open in this window is not the current folder, and if it has new
+  // message, in which case we need to try to run the
+  // filter plugin.
+  if (m_newMailParser)
+  {
+    nsCOMPtr <nsIMsgWindow> msgWindow;
+    m_newMailParser->GetMsgWindow(getter_AddRefs(msgWindow));
+    if (msgWindow)
+    {
+      nsCOMPtr <nsIMsgFolder> openFolder;
+      (void) msgWindow->GetOpenFolder(getter_AddRefs(openFolder));
+      if (openFolder && openFolder != m_folder)
+      {
+        // only call filter plugins if folder is a local folder, because only
+        // local folders get messages filtered into them synchronously by pop3.
+        nsCOMPtr<nsIMsgLocalMailFolder> localFolder = do_QueryInterface(openFolder);
+        if (localFolder)
+        {
+          PRBool hasNew;
+          (void) openFolder->GetHasNewMessages(&hasNew);
+          if (hasNew)
+            openFolder->CallFilterPlugins(nsnull);
+        }
+      }
+    }
+  }
 #ifdef DEBUG
-    printf("End mail message delivery.\n");
+  printf("End mail message delivery.\n");
 #endif 
-    return NS_OK;
+  return NS_OK;
 }
 
 nsresult 
