@@ -156,7 +156,7 @@ ProcessBodyAsAttachment(MimeObject *obj, nsMsgAttachmentData **data)
   {
     char *fname = NULL;
     fname = mime_decode_filename(tmp->real_name, charset, obj->options);
-    PR_FREEIF(charset);
+    nsMemory::Free(charset);
     if (fname && fname != tmp->real_name)
     {
       PR_Free(tmp->real_name);
@@ -330,12 +330,22 @@ GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayO
   else
   {
     isIMAPPart = PR_FALSE;
-    urlSpec = mime_set_url_part(aMessageURL, part.get(), PR_TRUE);
+    char *no_part_url = nsnull;
+    if (options->part_to_load && options->format_out == nsMimeOutput::nsMimeMessageBodyDisplay)
+      no_part_url = mime_get_base_url(aMessageURL);
+    if (no_part_url) {
+      urlSpec = mime_set_url_part(no_part_url, part.get(), PR_TRUE);
+      PR_Free(no_part_url);
+    }
+    else
+      urlSpec = mime_set_url_part(aMessageURL, part.get(), PR_TRUE);
   }
 
   if (!urlSpec)
     return NS_ERROR_OUT_OF_MEMORY;
 
+  if ((options->format_out == nsMimeOutput::nsMimeMessageBodyDisplay) && (nsCRT::strncasecmp(aMessageURL, urlSpec, strlen(urlSpec)) == 0))
+    return NS_OK;
   nsMsgAttachmentData *tmp = &(aAttachData[attIndex++]);
   nsresult rv = nsMimeNewURI(&(tmp->url), urlSpec, nsnull);
 
@@ -357,7 +367,7 @@ GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayO
       for (i = 0; i < 2 && !tmp->real_name; i ++)
       {
         PR_FREEIF(disp);
-        PR_FREEIF(charset);
+        nsMemory::Free(charset);
         disp = MimeHeaders_get(((MimeContainer *)object)->children[i]->headers, HEADER_CONTENT_DISPOSITION, PR_FALSE, PR_FALSE);
         tmp->real_name = MimeHeaders_get_parameter(disp, "filename", &charset, nsnull);
       }
@@ -372,7 +382,7 @@ GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayO
 
       char *fname = nsnull;
       fname = mime_decode_filename(tmp->real_name, charset, options);
-      PR_FREEIF(charset);
+      nsMemory::Free(charset);
 
       if (fname && fname != tmp->real_name)
       {
@@ -398,7 +408,7 @@ GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayO
         for (i = 0; i < 2 && !tmp->real_name; i ++)
         {
           PR_FREEIF(disp);
-          PR_FREEIF(charset);
+          nsMemory::Free(charset);
           disp = MimeHeaders_get(((MimeContainer *)object)->children[i]->headers, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE);
           tmp->real_name = MimeHeaders_get_parameter(disp, "name", &charset, nsnull);
         }
@@ -413,7 +423,7 @@ GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayO
 
         char *fname = nsnull;
         fname = mime_decode_filename(tmp->real_name, charset, options);
-        PR_FREEIF(charset);
+        nsMemory::Free(charset);
 
         if (fname && fname != tmp->real_name)
         {
@@ -695,8 +705,8 @@ SetMailCharacterSetToMsgWindow(MimeObject *obj, const char *aCharacterSet)
             msgurl->GetMsgWindow(getter_AddRefs(msgWindow));
             if (msgWindow)
               rv = msgWindow->SetMailCharacterSet(!nsCRT::strcasecmp(aCharacterSet, "us-ascii") ?
-                                                  NS_LITERAL_STRING("ISO-8859-1").get() :
-                                                  NS_ConvertASCIItoUCS2(aCharacterSet).get());
+                                                  "ISO-8859-1" :
+                                                  aCharacterSet);
           }
         }
       }
@@ -833,7 +843,7 @@ mime_output_fn(char *buf, PRInt32 size, void *stream_closure)
   
   // Now, write to the WriteBody method if this is a message body and not
   // a part retrevial
-  if (!msd->options->part_to_load)
+  if (!msd->options->part_to_load || msd->options->format_out == nsMimeOutput::nsMimeMessageBodyDisplay)
   {
     if (msd->output_emitter)
     {
@@ -922,7 +932,7 @@ mime_display_stream_complete (nsMIMESession *stream)
     // Ok, now we are going to process the attachment data by getting all
     // of the attachment info and then driving the emitter with this data.
     //
-    if (!msd->options->part_to_load)
+    if (!msd->options->part_to_load || msd->options->format_out == nsMimeOutput::nsMimeMessageBodyDisplay)
     {
       nsMsgAttachmentData *attachments;
       nsresult rv = MimeGetAttachmentList(obj, msd->url_name, &attachments);
@@ -1236,14 +1246,24 @@ mime_get_main_object(MimeObject* obj)
     }
     else
     {
-      // We don't care about a signed/smime object; Go inside to the 
-      // thing that we signed or smime'ed
-      //
-      cobj = (MimeContainer*) obj;
-      if (cobj->nchildren > 0)
-        obj = cobj->children[0];
+      if (mime_subclass_p(obj->clazz, (MimeObjectClass*)&mimeContainerClass))
+      {
+        // We don't care about a signed/smime object; Go inside to the 
+        // thing that we signed or smime'ed
+        //
+        cobj = (MimeContainer*) obj;
+        if (cobj->nchildren > 0)
+          obj = cobj->children[0];
+        else
+          obj = nsnull;
+      }
       else
-        obj = nsnull;
+      {
+        // we received a message with a child object that looks like a signed
+        // object, but it is not a subclass of mimeContainer, so let's
+        // return the given child object.
+        return obj;
+      }
     }
   }
   return nsnull;
@@ -1567,8 +1587,8 @@ mime_bridge_create_display_stream(
   }
  
   if (msd->options->headers == MimeHeadersMicro &&
-     (msd->url_name == NULL || (nsCRT::strncmp(msd->url_name, "news:", 5) != 0 &&
-              nsCRT::strncmp(msd->url_name, "snews:", 6) != 0)) )
+     (msd->url_name == NULL || (strncmp(msd->url_name, "news:", 5) != 0 &&
+              strncmp(msd->url_name, "snews:", 6) != 0)) )
     msd->options->headers = MimeHeadersMicroPlus;
 
   msd->options->url = msd->url_name;
@@ -1605,7 +1625,7 @@ mime_bridge_create_display_stream(
 
   // If this is a part, then we should emit the HTML to render the data
   // (i.e. embedded images)
-  if (msd->options->part_to_load)
+  if (msd->options->part_to_load && msd->options->format_out != nsMimeOutput::nsMimeMessageBodyDisplay)
     msd->options->write_html_p = PR_FALSE;
 
   if (msd->options->prefs)
@@ -2049,10 +2069,8 @@ nsresult GetMailNewsFont(MimeObject *obj, PRBool styleFixed,  PRInt32 *fontPixel
     else
       charset.Assign(text->charset);
 
-    nsCOMPtr<nsICharsetConverterManager2> charSetConverterManager2;
-    nsCOMPtr <nsIAtom> charsetAtom;
+    nsCOMPtr<nsICharsetConverterManager> charSetConverterManager2;
     nsCOMPtr<nsIAtom> langGroupAtom;
-    const PRUnichar* langGroup = nsnull;
     nsCAutoString prefStr;
 
     ToLowerCase(charset);
@@ -2062,21 +2080,16 @@ nsresult GetMailNewsFont(MimeObject *obj, PRBool styleFixed,  PRInt32 *fontPixel
       return rv;
 
     // get a language, e.g. x-western, ja
-    rv = charSetConverterManager2->GetCharsetAtom2(charset.get(),getter_AddRefs(charsetAtom));
+    rv = charSetConverterManager2->GetCharsetLangGroup(charset.get(), getter_AddRefs(langGroupAtom));
     if (NS_FAILED(rv))
       return rv;
-    rv = charSetConverterManager2->GetCharsetLangGroup(charsetAtom,getter_AddRefs(langGroupAtom));
+    rv = langGroupAtom->ToUTF8String(fontLang);
     if (NS_FAILED(rv))
       return rv;
-    rv = langGroupAtom->GetUnicode(&langGroup);
-    if (NS_FAILED(rv))
-      return rv;
-
-    fontLang.AssignWithConversion(langGroup);
 
     // get a font size from pref
     prefStr.Assign(!styleFixed ? "font.size.variable." : "font.size.fixed.");
-    prefStr.AppendWithConversion(langGroup);
+    prefStr.Append(fontLang);
     rv = prefs->GetIntPref(prefStr.get(), fontPixelSize);
     if (NS_FAILED(rv))
       return rv;
