@@ -37,10 +37,16 @@
 #include "nspr.h"
 #include "png.h"
 
-PR_STATIC_CALLBACK(void) info_callback(png_structp png_ptr, png_infop info_ptr);
-PR_STATIC_CALLBACK(void) row_callback(png_structp png_ptr, png_bytep new_row,
-                                      png_uint_32 row_num, int pass);
-PR_STATIC_CALLBACK(void) end_callback(png_structp png_ptr, png_infop info_ptr);
+static void PNGAPI info_callback(png_structp png_ptr, png_infop info_ptr);
+static void PNGAPI row_callback(png_structp png_ptr, png_bytep new_row,
+                           png_uint_32 row_num, int pass);
+static void PNGAPI end_callback(png_structp png_ptr, png_infop info_ptr);
+static void PNGAPI error_callback(png_structp png_ptr, png_const_charp error_msg);
+static void PNGAPI warning_callback(png_structp png_ptr, png_const_charp warning_msg);
+
+#ifdef PR_LOGGING
+PRLogModuleInfo *gPNGLog = PR_NewLogModule("PNGDecoder");
+#endif
 
 NS_IMPL_ISUPPORTS1(nsPNGDecoder, imgIDecoder)
 
@@ -50,7 +56,6 @@ nsPNGDecoder::nsPNGDecoder() :
   interlacebuf(nsnull), ibpr(0),
   mError(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsPNGDecoder::~nsPNGDecoder()
@@ -78,8 +83,7 @@ NS_IMETHODIMP nsPNGDecoder::Init(imgILoad *aLoad)
   /* Always decode to 24 bit pixdepth */
 
   mPNG = png_create_read_struct(PNG_LIBPNG_VER_STRING, 
-                                NULL, NULL, 
-                                NULL);
+                                NULL, error_callback, warning_callback);
   if (!mPNG) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -198,14 +202,16 @@ info_callback(png_structp png_ptr, png_infop info_ptr)
       png_set_gray_to_rgb(png_ptr);
 
 
-#if defined(XP_PC) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
+#if defined(XP_WIN) || defined(XP_OS2) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
   // windows likes BGR
   png_set_bgr(png_ptr);
 #endif
 
   if (png_get_gAMA(png_ptr, info_ptr, &aGamma)) {
-      if (aGamma < 0)
+      if ((aGamma <= 0.0) || (aGamma > 21474.83)) {
           aGamma = 0.45455;
+          png_set_gAMA(png_ptr, info_ptr, aGamma);
+      }
       png_set_gamma(png_ptr, 2.2, aGamma);
   }
   else
@@ -249,7 +255,7 @@ info_callback(png_structp png_ptr, png_infop info_ptr)
   nsPNGDecoder *decoder = NS_STATIC_CAST(nsPNGDecoder*, png_get_progressive_ptr(png_ptr));
 
   if (decoder->mObserver)
-    decoder->mObserver->OnStartDecode(nsnull, nsnull);
+    decoder->mObserver->OnStartDecode(nsnull);
 
   decoder->mImage = do_CreateInstance("@mozilla.org/image/container;1");
   if (!decoder->mImage)
@@ -261,7 +267,7 @@ info_callback(png_structp png_ptr, png_infop info_ptr)
   decoder->mImage->Init(width, height, decoder->mObserver);
 
   if (decoder->mObserver)
-    decoder->mObserver->OnStartContainer(nsnull, nsnull, decoder->mImage);
+    decoder->mObserver->OnStartContainer(nsnull, decoder->mImage);
 
   decoder->mFrame = do_CreateInstance("@mozilla.org/gfx/image/frame;2");
   if (!decoder->mFrame)
@@ -279,20 +285,20 @@ info_callback(png_structp png_ptr, png_infop info_ptr)
     }
   }
 
-#if defined(XP_PC) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
+#if defined(XP_WIN) || defined(XP_OS2) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
   // XXX this works...
   format += 1; // RGB to BGR
 #endif
 
   // then initalize the frame and append it to the container
-  nsresult rv = decoder->mFrame->Init(0, 0, width, height, format);
+  nsresult rv = decoder->mFrame->Init(0, 0, width, height, format, 24);
   if (NS_FAILED(rv))
     longjmp(decoder->mPNG->jmpbuf, 5); // NS_ERROR_OUT_OF_MEMORY
 
   decoder->mImage->AppendFrame(decoder->mFrame);
 
   if (decoder->mObserver)
-    decoder->mObserver->OnStartFrame(nsnull, nsnull, decoder->mFrame);
+    decoder->mObserver->OnStartFrame(nsnull, decoder->mFrame);
 
   PRUint32 bpr, abpr;
   decoder->mFrame->GetImageBytesPerRow(&bpr);
@@ -463,7 +469,7 @@ row_callback(png_structp png_ptr, png_bytep new_row,
     }
 
     nsRect r(0, row_num, width, 1);
-    decoder->mObserver->OnDataAvailable(nsnull, nsnull, decoder->mFrame, &r);
+    decoder->mObserver->OnDataAvailable(nsnull, decoder->mFrame, &r);
   }
 }
 
@@ -487,9 +493,9 @@ end_callback(png_structp png_ptr, png_infop info_ptr)
   nsPNGDecoder *decoder = NS_STATIC_CAST(nsPNGDecoder*, png_get_progressive_ptr(png_ptr));
 
   if (decoder->mObserver) {
-    decoder->mObserver->OnStopFrame(nsnull, nsnull, decoder->mFrame);
-    decoder->mObserver->OnStopContainer(nsnull, nsnull, decoder->mImage);
-    decoder->mObserver->OnStopDecode(nsnull, nsnull, NS_OK, nsnull);
+    decoder->mObserver->OnStopFrame(nsnull, decoder->mFrame);
+    decoder->mObserver->OnStopContainer(nsnull, decoder->mImage);
+    decoder->mObserver->OnStopDecode(nsnull, NS_OK, nsnull);
   }
 
   // We are never going to change the data of this frame again.  Let the OS
@@ -497,3 +503,17 @@ end_callback(png_structp png_ptr, png_infop info_ptr)
   decoder->mFrame->SetMutable(PR_FALSE);
 }
 
+
+void
+error_callback(png_structp png_ptr, png_const_charp error_msg)
+{
+  PR_LOG(gPNGLog, PR_LOG_ERROR, ("libpng error: %s\n", error_msg));
+  longjmp(png_ptr->jmpbuf, 1);
+}
+
+
+void
+warning_callback(png_structp png_ptr, png_const_charp warning_msg)
+{
+  PR_LOG(gPNGLog, PR_LOG_WARNING, ("libpng warning: %s\n", warning_msg));
+}
