@@ -61,7 +61,7 @@
 #include <sys/utsname.h>
 #endif
 
-#if defined(XP_PC) && !defined(XP_OS2)
+#if defined(XP_WIN)
 #include <windows.h>
 #endif
 
@@ -95,6 +95,31 @@ static NS_DEFINE_CID(kSocketProviderServiceCID, NS_SOCKETPROVIDERSERVICE_CID);
 #define UA_PREF(_pref) UA_PREF_PREFIX _pref
 #define HTTP_PREF(_pref) HTTP_PREF_PREFIX _pref
 
+//-----------------------------------------------------------------------------
+
+static nsresult
+NewURI(const nsACString &aSpec,
+       const char *aCharset,
+       nsIURI *aBaseURI,
+       PRInt32 aDefaultPort,
+       nsIURI **aURI)
+{
+    nsStandardURL *url = new nsStandardURL();
+    if (!url)
+        return NS_ERROR_OUT_OF_MEMORY;
+    NS_ADDREF(url);
+
+    nsresult rv = url->Init(nsIStandardURL::URLTYPE_AUTHORITY,
+                            aDefaultPort, aSpec, aCharset, aBaseURI);
+    if (NS_FAILED(rv)) {
+        NS_RELEASE(url);
+        return rv;
+    }
+
+    *aURI = url; // no QI needed
+    return NS_OK;
+}
+
 //-----------------------------------------------------------------------------
 // nsHttpHandler <public>
 //-----------------------------------------------------------------------------
@@ -119,10 +144,6 @@ nsHttpHandler::nsHttpHandler()
     , mRedirectionLimit(10)
     , mLastUniqueID(NowInSeconds())
     , mSessionStartTime(0)
-    //, mActiveConnections(0)
-    //, mIdleConnections(0)
-    //, mTransactionQ(0)
-    //, mConnectionLock(nsnull)
     , mUserAgentIsDirty(PR_TRUE)
     , mUseCache(PR_TRUE)
     , mSendSecureXSiteReferrer(PR_TRUE)
@@ -168,10 +189,6 @@ nsHttpHandler::Init()
         return rv;
     }
 
-    //mConnectionLock = PR_NewLock();
-    //if (!mConnectionLock)
-    //    return NS_ERROR_OUT_OF_MEMORY;
-
     InitUserAgentComponents();
 
     // monitor some preference changes
@@ -661,7 +678,7 @@ nsHttpHandler::InitUserAgentComponents()
     "Macintosh"
 #elif defined(XP_BEOS)
     "BeOS"
-#elif defined(NO_X11)
+#elif !defined(MOZ_X11)
     "?"
 #else
     "X11"
@@ -682,8 +699,8 @@ nsHttpHandler::InitUserAgentComponents()
     else if (os2ver == 45)
         mOscpu.Adopt(nsCRT::strdup("Warp 4.5"));
 
-#elif defined(XP_PC)
-    OSVERSIONINFO info = { sizeof OSVERSIONINFO };
+#elif defined(XP_WIN)
+    OSVERSIONINFO info = { sizeof(OSVERSIONINFO) };
     if (GetVersionEx(&info)) {
         if (info.dwPlatformId == VER_PLATFORM_WIN32_NT) {
             if (info.dwMajorVersion      == 3)
@@ -1128,16 +1145,17 @@ PrepareAcceptLanguages(const char *i_AcceptLanguages, nsACString &o_AcceptLangua
          token != (char *) 0;
          token = nsCRT::strtok(p, ",", &p))
     {
-        while (*token == ' ' || *token == '\x9') token++;
+        token = net_FindCharNotInSet(token, HTTP_LWS);
         char* trim;
-        trim = PL_strpbrk(token, "; \x9");
+        trim = net_FindCharInSet(token, ";" HTTP_LWS);
         if (trim != (char*)0)  // remove "; q=..." if present
             *trim = '\0';
 
         if (*token != '\0') {
             comma = n++ != 0 ? "," : ""; // delimiter if not first item
-            if (q < 0.9995)
-                wrote = PR_snprintf(p2, available, "%s%s;q=0.%01u", comma, token, QVAL_TO_UINT(q));
+            PRUint32 u = QVAL_TO_UINT(q);
+            if (u < 10)
+                wrote = PR_snprintf(p2, available, "%s%s;q=0.%u", comma, token, u);
             else
                 wrote = PR_snprintf(p2, available, "%s%s", comma, token);
             q -= dec;
@@ -1181,7 +1199,7 @@ nsHttpHandler::SetAcceptLanguages(const char *aAcceptLanguages)
 static nsresult
 PrepareAcceptCharsets(const char *i_AcceptCharset, nsACString &o_AcceptCharset)
 {
-    PRUint32 n, size, wrote;
+    PRUint32 n, size, wrote, u;
     PRInt32 available;
     double q, dec;
     char *p, *p2, *token, *q_Accept, *o_Accept;
@@ -1225,16 +1243,17 @@ PrepareAcceptCharsets(const char *i_AcceptCharset, nsACString &o_AcceptCharset)
     for (token = nsCRT::strtok(o_Accept, ",", &p);
          token != (char *) 0;
          token = nsCRT::strtok(p, ",", &p)) {
-        while (*token == ' ' || *token == '\x9') token++;
+        token = net_FindCharNotInSet(token, HTTP_LWS);
         char* trim;
-        trim = PL_strpbrk(token, "; \x9");
+        trim = net_FindCharInSet(token, ";" HTTP_LWS);
         if (trim != (char*)0)  // remove "; q=..." if present
             *trim = '\0';
 
         if (*token != '\0') {
             comma = n++ != 0 ? "," : ""; // delimiter if not first item
-            if (q < 0.9995)
-                wrote = PR_snprintf(p2, available, "%s%s;q=0.%01u", comma, token, QVAL_TO_UINT(q));
+            u = QVAL_TO_UINT(q);
+            if (u < 10)
+                wrote = PR_snprintf(p2, available, "%s%s;q=0.%u", comma, token, u);
             else
                 wrote = PR_snprintf(p2, available, "%s%s", comma, token);
             q -= dec;
@@ -1245,8 +1264,9 @@ PrepareAcceptCharsets(const char *i_AcceptCharset, nsACString &o_AcceptCharset)
     }
     if (add_utf) {
         comma = n++ != 0 ? "," : ""; // delimiter if not first item
-        if (q < 0.9995)
-            wrote = PR_snprintf(p2, available, "%sutf-8;q=0.%01u", comma, QVAL_TO_UINT(q));
+        u = QVAL_TO_UINT(q);
+        if (u < 10)
+            wrote = PR_snprintf(p2, available, "%sutf-8;q=0.%u", comma, u);
         else
             wrote = PR_snprintf(p2, available, "%sutf-8", comma);
         q -= dec;
@@ -1262,9 +1282,9 @@ PrepareAcceptCharsets(const char *i_AcceptCharset, nsACString &o_AcceptCharset)
         // the non-wildcard always receives preference.
 
         q += dec;
-        if (q < 0.9995) {
-            wrote = PR_snprintf(p2, available, "%s*;q=0.%01u", comma, QVAL_TO_UINT(q));
-        }
+        u = QVAL_TO_UINT(q);
+        if (u < 10)
+            wrote = PR_snprintf(p2, available, "%s*;q=0.%u", comma, u);
         else
             wrote = PR_snprintf(p2, available, "%s*", comma);
         available -= wrote;
@@ -1331,7 +1351,7 @@ nsHttpHandler::GetScheme(nsACString &aScheme)
 NS_IMETHODIMP
 nsHttpHandler::GetDefaultPort(PRInt32 *result)
 {
-    *result = 80;
+    *result = NS_HTTP_DEFAULT_PORT;
     return NS_OK;
 }
 
@@ -1348,24 +1368,8 @@ nsHttpHandler::NewURI(const nsACString &aSpec,
                       nsIURI *aBaseURI,
                       nsIURI **aURI)
 {
-    nsresult rv = NS_OK;
-
     LOG(("nsHttpHandler::NewURI\n"));
-
-    nsCOMPtr<nsIStandardURL> url;
-    NS_NEWXPCOM(url, nsStandardURL);
-    if (!url)
-        return NS_ERROR_OUT_OF_MEMORY;
-
-    // use the correct default port
-    PRInt32 defaultPort;
-    GetDefaultPort(&defaultPort);
-
-    rv = url->Init(nsIStandardURL::URLTYPE_AUTHORITY,
-                   defaultPort, aSpec, aCharset, aBaseURI);
-    if (NS_FAILED(rv)) return rv;
-
-    return CallQueryInterface(url, aURI);
+    return ::NewURI(aSpec, aCharset, aBaseURI, NS_HTTP_DEFAULT_PORT, aURI);
 }
 
 NS_IMETHODIMP
@@ -1625,7 +1629,6 @@ nsHttpHandler::Observe(nsISupports *subject,
         // XXX should probably shutdown and init the conn mgr.
     }
     else if (strcmp(topic, "profile-change-net-teardown")    == 0 ||
-             strcmp(topic, "session-logout")                 == 0 ||
              strcmp(topic, NS_XPCOM_SHUTDOWN_OBSERVER_ID)    == 0) {
 
         // kill off the "prune dead connections" timer
@@ -1642,6 +1645,14 @@ nsHttpHandler::Observe(nsISupports *subject,
         // depend on this value.
         mSessionStartTime = NowInSeconds();
     }
+    else if (strcmp(topic, "session-logout") == 0) {
+        // clear cache of all authentication credentials.
+        mAuthCache.ClearAll();
+
+        // need to reset the session start time since cache validation may
+        // depend on this value.
+        mSessionStartTime = NowInSeconds();
+    }
     else if (strcmp(topic, "profile-change-net-restore") == 0) {
         // initialize connection manager
         InitConnectionMgr();
@@ -1691,7 +1702,7 @@ nsHttpsHandler::GetScheme(nsACString &aScheme)
 NS_IMETHODIMP
 nsHttpsHandler::GetDefaultPort(PRInt32 *aPort)
 {
-    *aPort = 443;
+    *aPort = NS_HTTPS_DEFAULT_PORT;
     return NS_OK;
 }
 
@@ -1707,7 +1718,7 @@ nsHttpsHandler::NewURI(const nsACString &aSpec,
                        nsIURI *aBaseURI,
                        nsIURI **_retval)
 {
-    return gHttpHandler->NewURI(aSpec, aOriginCharset, aBaseURI, _retval);
+    return ::NewURI(aSpec, aOriginCharset, aBaseURI, NS_HTTPS_DEFAULT_PORT, _retval);
 }
 
 NS_IMETHODIMP
