@@ -19,7 +19,6 @@
 #include "nsJARURI.h"
 #include "nsNetUtil.h"
 #include "nsIIOService.h"
-#include "nsFileSpec.h"
 #include "nsCRT.h"
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
@@ -32,7 +31,6 @@
 nsJARURI::nsJARURI()
     : mJAREntry(nsnull)
 {
-    NS_INIT_ISUPPORTS();
 }
  
 nsJARURI::~nsJARURI()
@@ -328,8 +326,7 @@ nsJARURI::Resolve(const nsACString &relativePath, nsACString &result)
 {
     nsresult rv;
 
-    nsCAutoString scheme;
-    rv = net_ExtractURLScheme(relativePath, nsnull, nsnull, &scheme);
+    rv = net_ExtractURLScheme(relativePath, nsnull, nsnull, nsnull);
     if (NS_SUCCEEDED(rv)) {
         // then aSpec is absolute
         result = relativePath;
@@ -337,12 +334,27 @@ nsJARURI::Resolve(const nsACString &relativePath, nsACString &result)
     }
 
     nsCAutoString path(mJAREntry);
-    PRInt32 pos = path.RFind("/");
-    if (pos >= 0)
-        path.Truncate(pos + 1);
-    else
-        path = "";
+    PRInt32 pos = 0;
 
+    char first = relativePath.Length() > 0 ? relativePath.First() : '#';
+
+    switch (first) {
+    case '/':
+        path = "";
+        break;
+    case '?':
+    case '#':
+        pos = path.RFindChar(first);
+        if (pos >= 0)
+            path.Truncate(pos);
+        break;
+    default:
+        pos = path.RFindChar('/');
+        if (pos >= 0)
+            path.Truncate(pos + 1);
+        else
+            path = "";
+    }
     nsCAutoString resolvedEntry;
     rv = net_ResolveRelativePath(relativePath, path,
                                  resolvedEntry);
