@@ -246,6 +246,9 @@ nsProfile::nsProfile()
     mIsContentLocaleSpecified = PR_FALSE;
     
     mShutdownProfileToreDownNetwork = PR_FALSE;
+    
+    mProfileChangeVetoed = PR_FALSE;
+    mProfileChangeFailed = PR_FALSE;
 }
 
 nsProfile::~nsProfile() 
@@ -456,7 +459,7 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
     
     GetProfileCount(&numProfiles);
 
-    if (profileURLStr.Length() == 0)
+    if (profileURLStr.IsEmpty())
     {
         // If this flag is TRUE, it makes the multiple profile case
         // just like the single profile case - the profile will be
@@ -465,7 +468,12 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
         // multiple profiles.
         
         PRBool startWithLastUsedProfile = PR_FALSE;
-        GetStartWithLastUsedProfile(&startWithLastUsedProfile);
+
+        // But first, make sure this app supports this.
+        PRBool cantAutoSelect;
+        rv = prefBranch->GetBoolPref("profile.manage_only_at_launch", &cantAutoSelect);
+        if (NS_SUCCEEDED(rv) && !cantAutoSelect)
+          GetStartWithLastUsedProfile(&startWithLastUsedProfile);
         
         // This means that there was no command-line argument to force
         // profile UI to come up. But we need the UI anyway if there
@@ -507,7 +515,7 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
             profileURLStr = PROFILE_SELECTION_URL;
     }
 
-    if (profileURLStr.Length() != 0)
+    if (!profileURLStr.IsEmpty())
     {
         if (!canInteract) return NS_ERROR_PROFILE_REQUIRES_INTERACTION;
 
@@ -734,7 +742,7 @@ nsProfile::ProcessArgs(nsICmdLineService *cmdLineArgs,
             }
             else {
                 // get a platform charset
-                nsAutoString charSet;
+                nsCAutoString charSet;
                 rv = GetPlatformCharset(charSet);
                 NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get a platform charset");
 
@@ -803,7 +811,7 @@ nsProfile::ProcessArgs(nsICmdLineService *cmdLineArgs,
             }
             else {
                 // get a platform charset
-                nsAutoString charSet;
+                nsCAutoString charSet;
                 rv = GetPlatformCharset(charSet);
                 NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get a platform charset");
 
@@ -1217,6 +1225,8 @@ nsProfile::SetCurrentProfile(const PRUnichar * aCurrentProfile)
         
         // Phase 3: Notify observers of a profile change
         observerService->NotifyObservers(subject, "profile-before-change", context.get());        
+        if (mProfileChangeFailed)
+          return NS_ERROR_ABORT;
         
         UpdateCurrentProfileModTime(PR_FALSE);        
     }
@@ -1245,21 +1255,29 @@ nsProfile::SetCurrentProfile(const PRUnichar * aCurrentProfile)
         // Bring network back online
         observerService->NotifyObservers(subject, "profile-change-net-restore", context.get());
         mShutdownProfileToreDownNetwork = PR_FALSE;
+        if (mProfileChangeFailed)
+          return NS_ERROR_ABORT;
     }
 
     // Phase 4: Notify observers that the profile has changed - Here they respond to new profile
     observerService->NotifyObservers(subject, "profile-do-change", context.get());
+    if (mProfileChangeFailed)
+      return NS_ERROR_ABORT;
 
     // Phase 5: Now observers can respond to something another observer did in phase 4
     observerService->NotifyObservers(subject, "profile-after-change", context.get());
-      
+    if (mProfileChangeFailed)
+      return NS_ERROR_ABORT;
+
     // Now that a profile is established, set the profile defaults dir for the locale of this profile
     rv = DefineLocaleDefaultsDir();
     NS_ASSERTION(NS_SUCCEEDED(rv), "nsProfile::DefineLocaleDefaultsDir failed");
 
     // Phase 6: One last notification after the new profile is established
     observerService->NotifyObservers(subject, "profile-initial-state", context.get());
-      
+    if (mProfileChangeFailed)
+      return NS_ERROR_ABORT;
+
     return NS_OK;
 }
 
@@ -2460,6 +2478,11 @@ NS_IMETHODIMP nsProfile::VetoChange()
     return NS_OK;
 }
 
+NS_IMETHODIMP nsProfile::ChangeFailed()
+{
+    mProfileChangeFailed = PR_TRUE;
+    return NS_OK;
+}
 
 NS_IMETHODIMP
 nsProfile::GetRegStrings(const PRUnichar *aProfileName, 
