@@ -62,10 +62,12 @@
 #include "nsIRDFNode.h"
 #include "nsIRDFObserver.h"
 #include "nsIRDFInMemoryDataSource.h"
+#include "nsIRDFPropagatableDataSource.h"
 #include "nsIRDFPurgeableDataSource.h"
 #include "nsIRDFService.h"
 #include "nsIServiceManager.h"
 #include "nsISupportsArray.h"
+#include "nsCOMArray.h"
 #include "nsAutoLock.h"
 #include "nsEnumeratorUtils.h"
 #include "nsVoidArray.h"  // XXX introduces dependency on raptorbase
@@ -282,6 +284,7 @@ class InMemoryResourceEnumeratorImpl;
 
 class InMemoryDataSource : public nsIRDFDataSource,
                            public nsIRDFInMemoryDataSource,
+                           public nsIRDFPropagatableDataSource,
                            public nsIRDFPurgeableDataSource
 {
 protected:
@@ -295,7 +298,7 @@ protected:
     PLDHashTable mForwardArcs; 
     PLDHashTable mReverseArcs; 
 
-    nsCOMPtr<nsISupportsArray> mObservers;  
+    nsCOMArray<nsIRDFObserver> mObservers;  
     PRUint32                   mNumObservers;
 
     static PLDHashOperator PR_CALLBACK
@@ -338,6 +341,9 @@ public:
     // nsIRDFInMemoryDataSource methods
     NS_DECL_NSIRDFINMEMORYDATASOURCE
 
+    // nsIRDFPropagatableDataSource methods
+    NS_DECL_NSIRDFPROPAGATABLEDATASOURCE
+
     // nsIRDFPurgeableDataSource methods
     NS_DECL_NSIRDFPURGEABLEDATASOURCE
 
@@ -395,6 +401,7 @@ public:
     // This datasource's monitor object.
     PRLock* mLock;
 #endif
+    PRBool  mPropagateChanges;
 };
 
 //----------------------------------------------------------------------
@@ -449,8 +456,7 @@ public:
     Destroy(InMemoryAssertionEnumeratorImpl* aEnumerator) {
         // Keep the datasource alive for the duration of the stack
         // frame so its allocator stays valid.
-        nsCOMPtr<nsIRDFDataSource> kungFuDeathGrip =
-            dont_QueryInterface(aEnumerator->mDataSource);
+        nsCOMPtr<nsIRDFDataSource> kungFuDeathGrip = aEnumerator->mDataSource;
 
         // Grab the pool from the datasource; since we keep the
         // datasource alive, this has to be safe.
@@ -483,8 +489,6 @@ InMemoryAssertionEnumeratorImpl::InMemoryAssertionEnumeratorImpl(
       mTruthValue(aTruthValue),
       mNextAssertion(nsnull)
 {
-    NS_INIT_ISUPPORTS();
-
     NS_ADDREF(mDataSource);
     NS_IF_ADDREF(mSource);
     NS_ADDREF(mProperty);
@@ -657,8 +661,7 @@ public:
     Destroy(InMemoryArcsEnumeratorImpl* aEnumerator) {
         // Keep the datasource alive for the duration of the stack
         // frame so its allocator stays valid.
-        nsCOMPtr<nsIRDFDataSource> kungFuDeathGrip =
-            dont_QueryInterface(aEnumerator->mDataSource);
+        nsCOMPtr<nsIRDFDataSource> kungFuDeathGrip = aEnumerator->mDataSource;
 
         // Grab the pool from the datasource; since we keep the
         // datasource alive, this has to be safe.
@@ -689,7 +692,6 @@ InMemoryArcsEnumeratorImpl::InMemoryArcsEnumeratorImpl(InMemoryDataSource* aData
       mTarget(aTarget),
       mCurrent(nsnull)
 {
-    NS_INIT_ISUPPORTS();
     NS_ADDREF(mDataSource);
     NS_IF_ADDREF(mSource);
     NS_IF_ADDREF(mTarget);
@@ -842,6 +844,7 @@ NS_NewRDFInMemoryDataSource(nsISupports* aOuter, const nsIID& aIID, void** aResu
     NS_PRECONDITION(aResult != nsnull, "null ptr");
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
+    *aResult = nsnull;
 
     if (aOuter && !aIID.Equals(NS_GET_IID(nsISupports))) {
         NS_ERROR("aggregation requires nsISupports");
@@ -851,21 +854,16 @@ NS_NewRDFInMemoryDataSource(nsISupports* aOuter, const nsIID& aIID, void** aResu
     InMemoryDataSource* datasource = new InMemoryDataSource(aOuter);
     if (! datasource)
         return NS_ERROR_OUT_OF_MEMORY;
+    NS_ADDREF(datasource);
 
-    nsresult rv;
-
-    rv = datasource->Init();
+    nsresult rv = datasource->Init();
     if (NS_SUCCEEDED(rv)) {
         datasource->fAggregated.AddRef();
         rv = datasource->AggregatedQueryInterface(aIID, aResult); // This'll AddRef()
         datasource->fAggregated.Release();
-
-        if (NS_SUCCEEDED(rv))
-            return rv;
     }
 
-    delete datasource;
-    *aResult = nsnull;
+    NS_RELEASE(datasource);
     return rv;
 }
 
@@ -891,6 +889,7 @@ InMemoryDataSource::InMemoryDataSource(nsISupports* aOuter)
 #ifdef MOZ_THREADSAFE_RDF
     mLock = nsnull;
 #endif
+    mPropagateChanges = PR_TRUE;
 }
 
 
@@ -909,12 +908,6 @@ InMemoryDataSource::Init()
                       sizeof(Entry),
                       PL_DHASH_MIN_SIZE);
 
-    // allocate "mObservers" at Init() time so
-    // don't have to null-check it before usage later
-    nsresult rv;
-    rv = NS_NewISupportsArray(getter_AddRefs(mObservers));
-    if (NS_FAILED(rv)) return rv;
-
 #ifdef MOZ_THREADSAFE_RDF
     mLock = PR_NewLock();
     if (! mLock)
@@ -994,6 +987,9 @@ InMemoryDataSource::AggregatedQueryInterface(REFNSIID aIID, void** aResult)
     else if (aIID.Equals(NS_GET_IID(nsIRDFInMemoryDataSource))) {
         *aResult = NS_STATIC_CAST(nsIRDFInMemoryDataSource*, this);
     }
+    else if (aIID.Equals(NS_GET_IID(nsIRDFPropagatableDataSource))) {
+        *aResult = NS_STATIC_CAST(nsIRDFPropagatableDataSource*, this);
+    }
     else if (aIID.Equals(NS_GET_IID(nsIRDFPurgeableDataSource))) {
         *aResult = NS_STATIC_CAST(nsIRDFPurgeableDataSource*, this);
     }
@@ -1401,16 +1397,15 @@ InMemoryDataSource::Assert(nsIRDFResource* aSource,
     }
 
     // notify observers
-    for (PRInt32 i = (PRInt32)mNumObservers - 1; i >= 0; --i) {
-        nsIRDFObserver* obs = (nsIRDFObserver*) mObservers->ElementAt(i);
+    for (PRInt32 i = (PRInt32)mNumObservers - 1; mPropagateChanges && i >= 0; --i) {
+        nsIRDFObserver* obs = mObservers[i];
 
         // XXX this should never happen, but it does, and we can't figure out why.
-        NS_ASSERTION(obs != nsnull, "observer array corrupted!");
+        NS_ASSERTION(obs, "observer array corrupted!");
         if (! obs)
           continue;
 
         obs->OnAssert(this, aSource, aProperty, aTarget);
-        NS_RELEASE(obs);
         // XXX ignore return value?
     }
 
@@ -1554,16 +1549,15 @@ InMemoryDataSource::Unassert(nsIRDFResource* aSource,
     }
 
     // Notify the world
-    for (PRInt32 i = PRInt32(mNumObservers) - 1; i >= 0; --i) {
-        nsIRDFObserver* obs = (nsIRDFObserver*) mObservers->ElementAt(i);
+    for (PRInt32 i = PRInt32(mNumObservers) - 1; mPropagateChanges && i >= 0; --i) {
+        nsIRDFObserver* obs = mObservers[i];
 
         // XXX this should never happen, but it does, and we can't figure out why.
-        NS_ASSERTION(obs != nsnull, "observer array corrupted!");
+        NS_ASSERTION(obs, "observer array corrupted!");
         if (! obs)
           continue;
 
         obs->OnUnassert(this, aSource, aProperty, aTarget);
-        NS_RELEASE(obs);
         // XXX ignore return value?
     }
 
@@ -1609,16 +1603,15 @@ InMemoryDataSource::Change(nsIRDFResource* aSource,
     }
 
     // Notify the world
-    for (PRInt32 i = PRInt32(mNumObservers) - 1; i >= 0; --i) {
-        nsIRDFObserver* obs = (nsIRDFObserver*) mObservers->ElementAt(i);
+    for (PRInt32 i = PRInt32(mNumObservers) - 1; mPropagateChanges && i >= 0; --i) {
+        nsIRDFObserver* obs = mObservers[i];
 
         // XXX this should never happen, but it does, and we can't figure out why.
-        NS_ASSERTION(obs != nsnull, "observer array corrupted!");
+        NS_ASSERTION(obs, "observer array corrupted!");
         if (! obs)
           continue;
 
         obs->OnChange(this, aSource, aProperty, aOldTarget, aNewTarget);
-        NS_RELEASE(obs);
         // XXX ignore return value?
     }
 
@@ -1664,16 +1657,15 @@ InMemoryDataSource::Move(nsIRDFResource* aOldSource,
     }
 
     // Notify the world
-    for (PRInt32 i = PRInt32(mNumObservers) - 1; i >= 0; --i) {
-        nsIRDFObserver* obs = (nsIRDFObserver*) mObservers->ElementAt(i);
+    for (PRInt32 i = PRInt32(mNumObservers) - 1; mPropagateChanges && i >= 0; --i) {
+        nsIRDFObserver* obs = mObservers[i];
 
         // XXX this should never happen, but it does, and we can't figure out why.
-        NS_ASSERTION(obs != nsnull, "observer array corrupted!");
+        NS_ASSERTION(obs, "observer array corrupted!");
         if (! obs)
           continue;
 
         obs->OnMove(this, aOldSource, aNewSource, aProperty, aTarget);
-        NS_RELEASE(obs);
         // XXX ignore return value?
     }
 
@@ -1689,8 +1681,8 @@ InMemoryDataSource::AddObserver(nsIRDFObserver* aObserver)
         return NS_ERROR_NULL_POINTER;
 
     NS_AUTOLOCK(mLock);
-    mObservers->AppendElement(aObserver);
-    (void)mObservers->Count(&mNumObservers);
+    mObservers.AppendObject(aObserver);
+    mNumObservers = mObservers.Count();
 
     return NS_OK;
 }
@@ -1703,10 +1695,10 @@ InMemoryDataSource::RemoveObserver(nsIRDFObserver* aObserver)
         return NS_ERROR_NULL_POINTER;
 
     NS_AUTOLOCK(mLock);
-    mObservers->RemoveElement(aObserver);
+    mObservers.RemoveObject(aObserver);
     // note: use Count() instead of just decrementing
     // in case aObserver wasn't in list, for example
-    (void)mObservers->Count(&mNumObservers);
+    mNumObservers = mObservers.Count();
 
     return NS_OK;
 }
@@ -1859,6 +1851,27 @@ InMemoryDataSource::DoCommand(nsISupportsArray/*<nsIRDFResource>*/* aSources,
     return NS_OK;
 }
 
+NS_IMETHODIMP
+InMemoryDataSource::BeginUpdateBatch()
+{
+    for (PRInt32 i = PRInt32(mNumObservers) - 1; mPropagateChanges && i >= 0; --i) {
+        nsIRDFObserver* obs = mObservers[i];
+        obs->OnBeginUpdateBatch(this);
+    }
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+InMemoryDataSource::EndUpdateBatch()
+{
+    for (PRInt32 i = PRInt32(mNumObservers) - 1; mPropagateChanges && i >= 0; --i) {
+        nsIRDFObserver* obs = mObservers[i];
+        obs->OnEndUpdateBatch(this);
+    }
+    return NS_OK;
+}
+
+
 
 ////////////////////////////////////////////////////////////////////////
 // nsIRDFInMemoryDataSource methods
@@ -1915,6 +1928,23 @@ InMemoryDataSource::EnsureFastContainment(nsIRDFResource* aSource)
 }
 
 
+////////////////////////////////////////////////////////////////////////
+// nsIRDFPropagatableDataSource methods
+NS_IMETHODIMP
+InMemoryDataSource::GetPropagateChanges(PRBool* aPropagateChanges)
+{
+    *aPropagateChanges = mPropagateChanges;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+InMemoryDataSource::SetPropagateChanges(PRBool aPropagateChanges)
+{
+    mPropagateChanges = aPropagateChanges;
+    return NS_OK;
+}
+
+
 ////////////////////////////////////////////////////////////////////////
 // nsIRDFPurgeableDataSource methods
 
@@ -2015,8 +2045,9 @@ InMemoryDataSource::Sweep()
 #endif
         if (!(as->mHashEntry))
         {
-            for (PRInt32 i = PRInt32(mNumObservers) - 1; i >= 0; --i) {
-                nsIRDFObserver* obs = (nsIRDFObserver*) mObservers->ElementAt(i);
+            for (PRInt32 i = PRInt32(mNumObservers) - 1; mPropagateChanges && i >= 0; --i) {
+                nsIRDFObserver* obs = mObservers[i];
+                // XXXbz other loops over mObservers null-check |obs| here!
                 obs->OnUnassert(this, as->mSource, as->u.as.mProperty, as->u.as.mTarget);
                 // XXX ignore return value?
             }
