@@ -25,6 +25,7 @@
 #include "nsMemory.h"
 #include "nsString.h"
 #include "nsCRT.h"
+#include "nsUTF8Utils.h"
 
 #ifndef nsStringTraits_h___
 #include "nsStringTraits.h"
@@ -147,7 +148,7 @@ class LossyConvertEncoding
 
 NS_COM
 void
-CopyUCS2toASCII( const nsAString& aSource, nsACString& aDest )
+LossyCopyUTF16toASCII( const nsAString& aSource, nsACString& aDest )
   {
       // right now, this won't work on multi-fragment destinations
     aDest.SetLength(aSource.Length());
@@ -162,7 +163,7 @@ CopyUCS2toASCII( const nsAString& aSource, nsACString& aDest )
 
 NS_COM
 void
-CopyASCIItoUCS2( const nsACString& aSource, nsAString& aDest )
+CopyASCIItoUTF16( const nsACString& aSource, nsAString& aDest )
   {
       // right now, this won't work on multi-fragment destinations
     aDest.SetLength(aSource.Length());
@@ -175,6 +176,165 @@ CopyASCIItoUCS2( const nsACString& aSource, nsAString& aDest )
     copy_string(aSource.BeginReading(fromBegin), aSource.EndReading(fromEnd), converter);
   }
 
+NS_COM
+void
+CopyUTF16toUTF8( const nsAString& aSource, nsACString& aDest )
+  {
+    aDest.Truncate();
+    AppendUTF16toUTF8(aSource, aDest);
+  }
+
+NS_COM
+void
+CopyUTF8toUTF16( const nsACString& aSource, nsAString& aDest )
+  {
+    aDest.Truncate();
+    AppendUTF8toUTF16(aSource, aDest);
+  }
+
+NS_COM
+void
+CopyUTF16toUTF8( const PRUnichar* aSource, nsACString& aDest )
+  {
+    aDest.Truncate();
+    AppendUTF16toUTF8(aSource, aDest);
+  }
+
+NS_COM
+void
+CopyUTF8toUTF16( const char* aSource, nsAString& aDest )
+  {
+    aDest.Truncate();
+    AppendUTF8toUTF16(aSource, aDest);
+  }
+
+NS_COM
+void
+AppendUTF16toUTF8( const nsAString& aSource, nsACString& aDest )
+  {
+    nsAString::const_iterator source_start, source_end;
+    CalculateUTF8Size calculator;
+    copy_string(aSource.BeginReading(source_start),
+                aSource.EndReading(source_end), calculator);
+
+    PRUint32 count = calculator.Size();
+
+    if (count)
+      {
+        PRUint32 old_dest_length = aDest.Length();
+
+        // Grow the buffer if we need to.
+        aDest.SetLength(old_dest_length + count);
+
+        nsACString::iterator dest;
+        aDest.BeginWriting(dest);
+
+        dest.advance(old_dest_length);
+
+        if (count <= (PRUint32)dest.size_forward())
+          {
+            // aDest has enough room in the fragment just past the end
+            // of its old data that it can hold what we're about to
+            // append. Append using copy_string().
+
+            // All ready? Time to convert
+
+            ConvertUTF16toUTF8 converter(dest.get());
+            copy_string(aSource.BeginReading(source_start),
+                        aSource.EndReading(source_end), converter);
+
+            if (converter.Size() != count)
+              {
+                NS_ERROR("Input invalid or incorrect length was calculated");
+
+                aDest.SetLength(old_dest_length);
+              }
+          }
+        else
+          {
+            // This isn't the fastest way to do this, but it gets
+            // complicated to convert UTF16 into a fragmented UTF8
+            // string, so we'll take the easy way out here in this
+            // rare situation.
+
+            aDest.Replace(old_dest_length, count,
+                          NS_ConvertUTF16toUTF8(aSource));
+          }
+      }
+  }
+
+NS_COM
+void
+AppendUTF8toUTF16( const nsACString& aSource, nsAString& aDest )
+  {
+    nsACString::const_iterator source_start, source_end;
+    CalculateUTF8Length calculator;
+    copy_string(aSource.BeginReading(source_start),
+                aSource.EndReading(source_end), calculator);
+
+    PRUint32 count = calculator.Length();
+
+    if (count)
+      {
+        PRUint32 old_dest_length = aDest.Length();
+
+        // Grow the buffer if we need to.
+        aDest.SetLength(old_dest_length + count);
+
+        nsAString::iterator dest;
+        aDest.BeginWriting(dest);
+
+        dest.advance(old_dest_length);
+
+        if (count <= (PRUint32)dest.size_forward())
+          {
+            // aDest has enough room in the fragment just past the end
+            // of its old data that it can hold what we're about to
+            // append. Append using copy_string().
+
+            // All ready? Time to convert
+
+            ConvertUTF8toUTF16 converter(dest.get());
+            copy_string(aSource.BeginReading(source_start),
+                        aSource.EndReading(source_end), converter);
+
+            if (converter.Length() != count)
+              {
+                NS_ERROR("Input wasn't UTF8 or incorrect length was calculated");
+                aDest.SetLength(old_dest_length);
+              }
+          }
+        else
+          {
+            // This isn't the fastest way to do this, but it gets
+            // complicated to convert parts of a UTF8 string into a
+            // UTF16 string, so we'll take the easy way out here in
+            // this rare situation.
+
+            aDest.Replace(old_dest_length, count,
+                          NS_ConvertUTF8toUTF16(aSource));
+          }
+      }
+  }
+
+NS_COM
+void
+AppendUTF16toUTF8( const PRUnichar* aSource, nsACString& aDest )
+  {
+    if (aSource) {
+      AppendUTF16toUTF8(nsDependentString(aSource), aDest);
+    }
+  }
+
+NS_COM
+void
+AppendUTF8toUTF16( const char* aSource, nsAString& aDest )
+  {
+    if (aSource) {
+      AppendUTF8toUTF16(nsDependentCString(aSource), aDest);
+    }
+  }
+
 
   /**
    * A helper function that allocates a buffer of the desired character type big enough to hold a copy of the supplied string (plus a zero terminator).
@@ -208,22 +368,18 @@ NS_COM
 char*
 ToNewUTF8String( const nsAString& aSource )
   {
-    NS_ConvertUCS2toUTF8 temp(aSource);
+    nsAString::const_iterator start, end;
+    CalculateUTF8Size calculator;
+    copy_string(aSource.BeginReading(start), aSource.EndReading(end),
+                calculator);
 
-    char* result;
-    if (temp.GetOwnsBuffer()) {
-      // We allocated. Trick the string into not freeing its buffer to
-      // avoid an extra allocation.
-      result = temp.mStr;
+    char *result = NS_STATIC_CAST(char*,
+        nsMemory::Alloc(calculator.Size() + 1));
 
-      temp.mStr=0;
-      temp.SetOwnsBuffer(PR_FALSE);
-    }
-    else {
-      // We didn't allocate a buffer, so we need to copy it out of the
-      // nsCAutoString's storage.
-      result = ToNewCString(temp);
-    }
+    ConvertUTF16toUTF8 converter(result);
+    copy_string(aSource.BeginReading(start), aSource.EndReading(end),
+                converter).write_terminator();
+    NS_ASSERTION(calculator.Size() == converter.Size(), "length mismatch");
 
     return result;
   }
@@ -268,6 +424,26 @@ ToNewUnicode( const nsACString& aSource )
     return result;
   }
 
+NS_COM
+PRUnichar*
+UTF8ToNewUnicode( const nsACString& aSource )
+  {
+    nsACString::const_iterator start, end;
+    CalculateUTF8Length calculator;
+    copy_string(aSource.BeginReading(start), aSource.EndReading(end),
+                calculator);
+
+    PRUnichar *result = NS_STATIC_CAST(PRUnichar*,
+        nsMemory::Alloc(sizeof(PRUnichar) * (calculator.Length() + 1)));
+
+    ConvertUTF8toUTF16 converter(result);
+    copy_string(aSource.BeginReading(start), aSource.EndReading(end),
+                converter).write_terminator();
+    NS_ASSERTION(calculator.Length() == converter.Length(), "length mismatch");
+
+    return result;
+  }
+
 NS_COM
 PRUnichar*
 CopyUnicodeTo( const nsAString& aSource, PRUint32 aSrcOffset, PRUnichar* aDest, PRUint32 aLength )
@@ -440,7 +616,7 @@ IsUTF8( const nsACString& aString )
                       }
                   }
                 else
-                  return PR_FALSE; // Not UTF8 string
+                  return PR_FALSE; // Not UTF-8 string
               }
               
               while (ptr < fragmentEnd && state)
@@ -456,7 +632,7 @@ IsUTF8( const nsACString& aString )
 
                   if ( !UTF8traits::isInSeq(c) || overlong && c <= olupper || 
                        surrogate && slower <= c || nonchar && !state )
-                    return PR_FALSE; // Not UTF8 string
+                    return PR_FALSE; // Not UTF-8 string
                   overlong = surrogate = PR_FALSE;
                 }
             }
@@ -1080,6 +1256,54 @@ CountCharInReadable( const nsACString& aStr,
   return count;
 }
 
+NS_COM PRBool
+StringBeginsWith( const nsAString& aSource, const nsAString& aSubstring,
+                  const nsStringComparator& aComparator )
+  {
+    nsAString::size_type src_len = aSource.Length(),
+                         sub_len = aSubstring.Length();
+    if (sub_len > src_len)
+      return PR_FALSE;
+    return Substring(aSource, 0, sub_len).Equals(aSubstring, aComparator);
+  }
+
+NS_COM PRBool
+StringBeginsWith( const nsACString& aSource, const nsACString& aSubstring,
+                  const nsCStringComparator& aComparator )
+  {
+    nsACString::size_type src_len = aSource.Length(),
+                          sub_len = aSubstring.Length();
+    if (sub_len > src_len)
+      return PR_FALSE;
+    return Substring(aSource, 0, sub_len).Equals(aSubstring, aComparator);
+  }
+
+NS_COM PRBool
+StringEndsWith( const nsAString& aSource, const nsAString& aSubstring,
+                const nsStringComparator& aComparator )
+  {
+    nsAString::size_type src_len = aSource.Length(),
+                         sub_len = aSubstring.Length();
+    if (sub_len > src_len)
+      return PR_FALSE;
+    return Substring(aSource, src_len - sub_len, sub_len).Equals(aSubstring,
+                                                                 aComparator);
+  }
+
+NS_COM PRBool
+StringEndsWith( const nsACString& aSource, const nsACString& aSubstring,
+                const nsCStringComparator& aComparator )
+  {
+    nsACString::size_type src_len = aSource.Length(),
+                          sub_len = aSubstring.Length();
+    if (sub_len > src_len)
+      return PR_FALSE;
+    return Substring(aSource, src_len - sub_len, sub_len).Equals(aSubstring,
+                                                                 aComparator);
+  }
+
+
+
 template <class CharT>
 class CalculateHashCode
   {
