@@ -42,7 +42,6 @@
 #include "nsIPresContext.h"
 #include "nsIWidget.h"
 #include "nsIButton.h"
-#include "nsIScrollbar.h"
 #include "nsGUIEvent.h"
 #include "nsIDeviceContext.h"
 #include "nsIComponentManager.h"
@@ -73,11 +72,9 @@ nsEventStatus PR_CALLBACK HandleEvent(nsGUIEvent *aEvent)
   nsEventStatus result = nsEventStatus_eIgnore;
   nsView       *view = nsView::GetViewFor(aEvent->widget);
 
-  if (nsnull != view)
+  if (view)
   {
-    nsViewManager *vm = view->GetViewManager();
-
-    vm->DispatchEvent(aEvent, &result);
+    view->GetViewManager()->DispatchEvent(aEvent, &result);
   }
 
   return result;
@@ -100,11 +97,21 @@ nsView::nsView()
   mChildRemoved = PR_FALSE;
 }
 
+void nsView::DropMouseGrabbing() {
+  // check to see if we are grabbing events
+  if (mViewManager->GetMouseEventGrabber() == this) {
+    // we are grabbing events. Move the grab to the parent if we can.
+    PRBool boolResult; //not used
+    // if GetParent() returns null, then we release the grab, which is the best we can do
+    mViewManager->GrabMouseEvents(GetParent(), boolResult);
+  }
+}
+
 nsView::~nsView()
 {
   MOZ_COUNT_DTOR(nsView);
 
-  while (GetFirstChild() != nsnull)
+  while (GetFirstChild())
   {
     nsView* child = GetFirstChild();
     if (child->GetViewManager() == mViewManager) {
@@ -115,14 +122,16 @@ nsView::~nsView()
     }
   }
 
-  if (nsnull != mViewManager)
+  DropMouseGrabbing();
+  
+  if (mViewManager)
   {
     nsView *rootView = mViewManager->GetRootView();
     
-    if (nsnull != rootView)
+    if (rootView)
     {
       // Root views can have parents!
-      if (nsnull != mParent)
+      if (mParent)
       {
         mViewManager->RemoveChild(this);
       }
@@ -133,33 +142,26 @@ nsView::~nsView()
         mViewManager->SetRootView(nsnull);
       }
     }
-    else if (nsnull != mParent)
+    else if (mParent)
     {
       mParent->RemoveChild(this);
     }
     
-    nsView* grabbingView = mViewManager->GetMouseEventGrabber(); //check to see if we are capturing!!!
-    if (grabbingView == this)
-    {
-      PRBool boolResult;//not used
-      mViewManager->GrabMouseEvents(nsnull,boolResult);
-    }
-
     mViewManager = nsnull;
   }
-  else if (nsnull != mParent)
+  else if (mParent)
   {
     mParent->RemoveChild(this);
   }
 
-  if (nsnull != mZParent)
+  if (mZParent)
   {
     mZParent->RemoveReparentedView();
     mZParent->Destroy();
   }
 
   // Destroy and release the widget
-  if (nsnull != mWindow)
+  if (mWindow)
   {
     mWindow->SetClientData(nsnull);
     mWindow->Destroy();
@@ -198,15 +200,15 @@ nsrefcnt nsView::Release()
 
 nsView* nsView::GetViewFor(nsIWidget* aWidget)
 {           
-  void*    clientData;
-
   NS_PRECONDITION(nsnull != aWidget, "null widget ptr");
 	
   // The widget's client data points back to the owning view
-  if (aWidget && NS_SUCCEEDED(aWidget->GetClientData(clientData))) {
+  if (aWidget) {
+    void* clientData;
+    aWidget->GetClientData(clientData);
     nsISupports* data = (nsISupports*)clientData;
     
-    if (nsnull != data) {
+    if (data) {
       nsIView* view = nsnull;
       if (NS_SUCCEEDED(data->QueryInterface(NS_GET_IID(nsIView), (void **)&view))) {
         return NS_STATIC_CAST(nsView*, view);
@@ -317,18 +319,6 @@ NS_IMETHODIMP nsView::IgnoreSetPosition(PRBool aShouldIgnore)
 
 void nsView::SetPosition(nscoord aX, nscoord aY)
 {
-  nscoord x = aX;
-  nscoord y = aY;
-  if (IsRoot()) {
-    // Add view manager's coordinate offset to the root view
-    // This allows the view manager to offset it's coordinate space
-    // while allowing layout to assume it's coordinate space origin is (0,0)
-    nscoord offsetX;
-    nscoord offsetY;
-    mViewManager->GetWindowOffset(&offsetX, &offsetY);
-    x += offsetX;
-    y += offsetY;
-  }
   mDimBounds.x += aX - mPosX;
   mDimBounds.y += aY - mPosY;
   mPosX = aX;
@@ -472,13 +462,10 @@ NS_IMETHODIMP nsView::GetPosition(nscoord *x, nscoord *y) const
     *x = *y = 0;
   else
   {
-
     *x = mPosX;
     *y = mPosY;
-
   }
 
-
   return NS_OK;
 }
 
@@ -555,13 +542,7 @@ NS_IMETHODIMP nsView::SetVisibility(nsViewVisibility aVisibility)
 
   if (aVisibility == nsViewVisibility_kHide)
   {
-    nsView* grabbingView = mViewManager->GetMouseEventGrabber(); //check to see if we are grabbing events
-    if (grabbingView == this)
-    {
-      //if yes then we must release them before we become hidden and can't get them
-      PRBool boolResult;//not used
-      mViewManager->GrabMouseEvents(nsnull,boolResult);
-    }
+    DropMouseGrabbing();
   }
 
   if (nsnull != mWindow)
@@ -600,10 +581,8 @@ NS_IMETHODIMP nsView::SetFloating(PRBool aFloatingView)
 
 #if 0
 	// recursively make all sub-views "floating" grr.
-	nsIView *child = mFirstChild;
-	while (nsnull != child) {
+	for (nsView* child = mFirstChild; chlid; child = child->GetNextSibling()) {
 		child->SetFloating(aFloatingView);
-		child->GetNextSibling(child);
 	}
 #endif
 
@@ -622,6 +601,12 @@ NS_IMETHODIMP nsView::GetParent(nsIView *&aParent) const
   return NS_OK;
 }
 
+NS_IMETHODIMP nsView::GetFirstChild(nsIView *&aChild) const
+{
+  aChild = mFirstChild;
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsView::GetNextSibling(nsIView *&aNextSibling) const
 {
   aNextSibling = mNextSibling;
@@ -637,12 +622,10 @@ void nsView::InsertChild(nsView *aChild, nsView *aSibling)
     if (nsnull != aSibling)
     {
 #ifdef NS_DEBUG
-      nsView* siblingParent = aSibling->GetParent();
-      NS_ASSERTION(siblingParent == this, "tried to insert view with invalid sibling");
+      NS_ASSERTION(aSibling->GetParent() == this, "tried to insert view with invalid sibling");
 #endif
       //insert after sibling
-      nsView* siblingNextSibling = aSibling->GetNextSibling();
-      aChild->SetNextSibling(siblingNextSibling);
+      aChild->SetNextSibling(aSibling->GetNextSibling());
       aSibling->SetNextSibling(aChild);
     }
     else
@@ -651,7 +634,6 @@ void nsView::InsertChild(nsView *aChild, nsView *aSibling)
       mFirstChild = aChild;
     }
     aChild->SetParent(this);
-    mNumKids++;
   }
 }
 
@@ -667,13 +649,11 @@ void nsView::RemoveChild(nsView *child)
     while (nsnull != kid) {
       if (kid == child) {
         if (nsnull != prevKid) {
-          nsView* kidNextSibling = kid->GetNextSibling();
-          prevKid->SetNextSibling(kidNextSibling);
+          prevKid->SetNextSibling(kid->GetNextSibling());
         } else {
           mFirstChild = kid->GetNextSibling();
         }
         child->SetParent(nsnull);
-        mNumKids--;
         found = PR_TRUE;
         break;
       }
@@ -687,7 +667,7 @@ void nsView::RemoveChild(nsView *child)
 
 nsView* nsView::GetChild(PRInt32 aIndex) const
 { 
-  for (nsView* child = GetFirstChild(); child != nsnull; child = child->GetNextSibling()) {
+  for (nsView* child = GetFirstChild(); child; child = child->GetNextSibling()) {
     if (aIndex == 0) {
       return child;
     }
@@ -737,10 +717,11 @@ NS_IMETHODIMP nsView::GetClientData(void *&aData) const
 }
 
 NS_IMETHODIMP nsView::CreateWidget(const nsIID &aWindowIID,
-                                     nsWidgetInitData *aWidgetInitData,
-                                     nsNativeWidget aNative,
-                                     PRBool aEnableDragDrop,
-                                     PRBool aResetVisibility)
+                                   nsWidgetInitData *aWidgetInitData,
+                                   nsNativeWidget aNative,
+                                   PRBool aEnableDragDrop,
+                                   PRBool aResetVisibility,
+                                   nsContentType aContentType)
 {
   nsIDeviceContext  *dx;
   nsRect            trect = mDimBounds;
@@ -761,17 +742,24 @@ NS_IMETHODIMP nsView::CreateWidget(const nsIID &aWindowIID,
 
     if (PR_TRUE == usewidgets)
     {
+      PRBool initDataPassedIn = PR_TRUE;
+      nsWidgetInitData initData;
+      if (!aWidgetInitData) {
+        // No initData, we're a child window
+        // Create initData to pass in params
+        initDataPassedIn = PR_FALSE;
+        initData.clipChildren = PR_TRUE; // Clip child window's children
+        aWidgetInitData = &initData;
+      }
+      aWidgetInitData->mContentType = aContentType;
+
       if (aNative)
         mWindow->Create(aNative, trect, ::HandleEvent, dx, nsnull, nsnull, aWidgetInitData);
       else
       {
-        nsWidgetInitData initData;
-        if (nsnull == aWidgetInitData && nsnull != GetParent()) {
-          if (GetParent()->GetViewManager() != mViewManager) {
-            initData.mListenForResizes = PR_TRUE;
-            aWidgetInitData = &initData;
-          }
-        }
+        if (!initDataPassedIn && GetParent() && 
+          GetParent()->GetViewManager() != mViewManager)
+          initData.mListenForResizes = PR_TRUE;
 
         nsIWidget *parent;
         GetOffsetFromWidget(nsnull, nsnull, parent);
@@ -791,11 +779,8 @@ NS_IMETHODIMP nsView::CreateWidget(const nsIID &aWindowIID,
   //make sure visibility state is accurate
   
   if (aResetVisibility) {
-    nsViewVisibility vis;
-    
-    GetVisibility(vis);
-    SetVisibility(vis);
-    }
+    SetVisibility(GetVisibility());
+  }
 
   NS_RELEASE(dx);
 
@@ -836,8 +821,8 @@ NS_IMETHODIMP nsView::GetWidget(nsIWidget *&aWidget) const
 
 NS_IMETHODIMP nsView::HasWidget(PRBool *aHasWidget) const
 {
-	*aHasWidget = (mWindow != nsnull);
-	return NS_OK;
+  *aHasWidget = (mWindow != nsnull);
+  return NS_OK;
 }
 
 //
@@ -879,17 +864,14 @@ NS_IMETHODIMP nsView::List(FILE* out, PRInt32 aIndent) const
             nonclientBounds.x, nonclientBounds.y,
             windowBounds.width, windowBounds.height);
   }
-  nsRect brect;
-  GetBounds(brect);
+  nsRect brect = GetBounds();
   fprintf(out, "{%d,%d,%d,%d}",
           brect.x, brect.y, brect.width, brect.height);
   PRBool  hasTransparency;
   HasTransparency(hasTransparency);
   fprintf(out, " z=%d vis=%d opc=%1.3f tran=%d clientData=%p <\n", mZIndex, mVis, mOpacity, hasTransparency, mClientData);
-  nsView* kid = mFirstChild;
-  while (nsnull != kid) {
+  for (nsView* kid = mFirstChild; kid; kid = kid->GetNextSibling()) {
     kid->List(out, aIndent + 1);
-    kid = kid->GetNextSibling();
   }
   for (i = aIndent; --i >= 0; ) fputs("  ", out);
   fputs(">\n", out);
@@ -906,14 +888,15 @@ NS_IMETHODIMP nsView::GetOffsetFromWidget(nscoord *aDx, nscoord *aDy, nsIWidget
   // to 0 rather than relying on the caller to do so...
   while (nsnull != ancestor)
   {
-    ancestor->GetWidget(aWidget);
-	  if (nsnull != aWidget) {
+    aWidget = ancestor->GetWidget();
+    if (aWidget) {
+      NS_ADDREF(aWidget);
       // the widget's (0,0) is at the top left of the view's bounds, NOT its position
       nsRect r;
       ancestor->GetDimensions(r);
       aDx -= r.x;
       aDy -= r.y;
-	    return NS_OK;
+      return NS_OK;
     }
 
     if ((nsnull != aDx) && (nsnull != aDy))
@@ -921,13 +904,12 @@ NS_IMETHODIMP nsView::GetOffsetFromWidget(nscoord *aDx, nscoord *aDy, nsIWidget
       ancestor->ConvertToParentCoords(aDx, aDy);
     }
 
-	  ancestor = ancestor->GetParent();
+    ancestor = ancestor->GetParent();
   }
 
-  
   if (nsnull == aWidget) {
-       // The root view doesn't have a widget
-       // but maybe the view manager does.
+    // The root view doesn't have a widget
+    // but maybe the view manager does.
     GetViewManager()->GetWidget(&aWidget);
   }
 
@@ -947,7 +929,7 @@ nsresult nsView::GetDirtyRegion(nsIRegion*& aRegion)
   return NS_OK;
 }
 
-PRBool nsView::IsRoot()
+PRBool nsView::IsRoot() const
 {
   NS_ASSERTION(mViewManager != nsnull," View manager is null in nsView::IsRoot()");
   return mViewManager->GetRootView() == this;
@@ -984,7 +966,7 @@ NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, P
   aEmpty = PR_FALSE;
   aIsClipped = PR_FALSE;
   
-  GetBounds(aClippedRect);
+  aClippedRect = GetBounds();
   PRBool lastViewIsFloating = GetFloating();
 
   // Walk all of the way up the views to see if any
@@ -992,12 +974,12 @@ NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, P
   // don't consider non-floating ancestors of a floating view.
   const nsView* view = this;
   while (PR_TRUE) {
-    const nsView* zParent = view->GetZParent();
     const nsView* parentView = view->GetParent();
 #if 0
     // For now, we're disabling this code. This means that we only honor clipping
     // set by parent elements which are containing block ancestors of this content.
 
+    const nsView* zParent = view->GetZParent();
     if (zParent) {
       // This view was reparented. We need to continue collecting clip
       // rects from the zParent.
@@ -1006,7 +988,7 @@ NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, P
       // coordinate system.  They are the offset of this view within
       // parentView
       const nsView* zParentChain;
-      for (zParentChain = zParent; zParentChain != parentView && zParentChain != nsnull;
+      for (zParentChain = zParent; zParentChain != parentView && zParentChain;
            zParentChain = zParentChain->GetParent()) {
         zParentChain->ConvertFromParentCoords(&ancestorX, &ancestorY);
       }
@@ -1016,7 +998,7 @@ NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, P
         // case we walked from zParentChain all the way up to the
         // root, subtracting from ancestorX/Y. So now we walk from
         // parentView up to the root, adding to ancestorX/Y.
-        while (parentView != nsnull) {
+        while (parentView) {
           parentView->ConvertToParentCoords(&ancestorX, &ancestorY);
           parentView = parentView->GetParent();
         }
@@ -1036,18 +1018,24 @@ NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, P
     }
 
     if (parentView->GetClipChildren()) {
-      aIsClipped = PR_TRUE;
       // Adjust for clip specified by ancestor
       nsRect clipRect;
       parentView->GetChildClip(clipRect);
       //Offset the cliprect by the amount the child offsets from the parent
       clipRect.x -= ancestorX;
       clipRect.y -= ancestorY;
+
+      nsRect oldClippedRect = aClippedRect;
       PRBool overlap = aClippedRect.IntersectRect(clipRect, aClippedRect);
       if (!overlap) {
+        aIsClipped = PR_TRUE;
         aEmpty = PR_TRUE; // Does not intersect so the rect is empty.
         return NS_OK;
       }
+
+      if (oldClippedRect != aClippedRect) {
+        aIsClipped = PR_TRUE;
+      }
     }
 
     parentView->ConvertToParentCoords(&ancestorX, &ancestorY);
