@@ -40,9 +40,9 @@
 #include "nsIDOMDocument.h"
 #include "nsIURL.h"
 #include "nsIChannel.h"
+#include "nsNetUtil.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIFilePicker.h"
-#include "nsIFileURL.h"
 #include "nsILookAndFeel.h"
 #include "nsIComponentManager.h"
 #include "nsIFactory.h"
@@ -156,6 +156,7 @@ static NS_DEFINE_CID(kWalletServiceCID, NS_WALLETSERVICE_CID);
 #endif
 #define BUTTON_WIDTH 90
 #define BUTTON_HEIGHT THROBBER_HEIGHT
+#define MAX_TEXT_LENGTH 30000
 
 #ifdef INSET_WEBSHELL
 #define WEBSHELL_LEFT_INSET 5
@@ -397,6 +398,18 @@ NS_IMETHODIMP nsBrowserWindow::SetEnabled(PRBool aEnabled)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsBrowserWindow::GetBlurSuppression(PRBool *aBlurSuppression)
+{
+  NS_ENSURE_ARG_POINTER(aBlurSuppression);
+  *aBlurSuppression = PR_FALSE;
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP nsBrowserWindow::SetBlurSuppression(PRBool aBlurSuppression)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP nsBrowserWindow::GetMainWidget(nsIWidget** aMainWidget)
 {
    NS_ENSURE_ARG_POINTER(aMainWidget);
@@ -466,11 +479,10 @@ GetPresShellFor(nsIDocShell* aDocShell)
       nsIDocumentViewer* docv = nsnull;
       cv->QueryInterface(kIDocumentViewerIID, (void**) &docv);
       if (nsnull != docv) {
-        nsIPresContext* cx;
-        docv->GetPresContext(cx);
+        nsCOMPtr<nsIPresContext> cx;
+        docv->GetPresContext(getter_AddRefs(cx));
         if (nsnull != cx) {
           cx->GetShell(&shell);
-          NS_RELEASE(cx);
         }
         NS_RELEASE(docv);
       }
@@ -1138,16 +1150,15 @@ nsBrowserWindow::DoFileOpen()
 
   if (GetFileFromFileSelector(parentWindow, getter_AddRefs(file),
                               getter_AddRefs(mOpenFileDirectory))) {
-    nsCOMPtr<nsIFileURL> fileURL = do_CreateInstance(NS_STANDARDURL_CONTRACTID);
-    if (fileURL) {
-      fileURL->SetFile(file);
-
-      nsCAutoString url;
-      fileURL->GetSpec(url);
+    nsCOMPtr<nsIURI> uri;
+    NS_NewFileURI(getter_AddRefs(uri), file);
+    if (uri) {
+      nsCAutoString spec;
+      uri->GetSpec(spec);
       
       // Ask the Web widget to load the file URL
       nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(mWebBrowser));
-      webNav->LoadURI(NS_ConvertUTF8toUCS2(url).get(),
+      webNav->LoadURI(NS_ConvertUTF8toUCS2(spec).get(),
                       nsIWebNavigation::LOAD_FLAGS_NONE,
                       nsnull,
                       nsnull,
@@ -1322,7 +1333,6 @@ GetTitleSuffix(void)
 // Note: operator new zeros our memory
 nsBrowserWindow::nsBrowserWindow()
 {
-  NS_INIT_REFCNT();
   if (!gTitleSuffix) {
 #if XXX
     gTitleSuffix = GetTitleSuffix();
@@ -1541,15 +1551,14 @@ nsnull, r.x, r.y, r.width, r.height);
 
 
   // Create a document viewer and bind it to the webshell
-  nsIDocumentViewer* docv;
-  aDocumentViewer->CreateDocumentViewerUsing(aPresContext, docv);
+  nsCOMPtr<nsIDocumentViewer> docv;
+  aDocumentViewer->CreateDocumentViewerUsing(aPresContext,
+                                             getter_AddRefs(docv));
   docv->SetContainer(mWebBrowser);
   nsCOMPtr<nsIContentViewerContainer> cvContainer = do_QueryInterface(mDocShell);
   cvContainer->Embed(docv, "duh", nsnull);
 
-
   webBrowserWin->SetVisibility(PR_TRUE);
-  NS_RELEASE(docv);
 
   return NS_OK;
 }
@@ -1664,6 +1673,7 @@ nsBrowserWindow::CreateToolBar(PRInt32 aWidth)
     PRUint32 size;
     nsAutoString empty;
     mLocation->SetText(empty, size);
+    mLocation->SetMaxTextLength(MAX_TEXT_LENGTH);
     NS_RELEASE(widget);
   }
 
@@ -1723,6 +1733,7 @@ nsBrowserWindow::CreateStatusBar(PRInt32 aWidth)
     widget->SetForegroundColor(NS_RGB(0, 0, 0));
     PRUint32 size;
     mStatus->SetText(nsAutoString(),size);
+    mStatus->SetMaxTextLength(MAX_TEXT_LENGTH);
 
     nsITextWidget* textWidget = nsnull;
     if (NS_OK == mStatus->QueryInterface(kITextWidgetIID,(void**)&textWidget))
@@ -2228,12 +2239,10 @@ nsIDOMDocument* nsBrowserWindow::GetDOMDocument(nsIDocShell *aDocShell)
     if (nsnull != mCViewer) {
       nsIDocumentViewer* mDViewer;
       if (NS_OK == mCViewer->QueryInterface(kIDocumentViewerIID, (void**) &mDViewer)) {
-        nsIDocument* mDoc;
-        mDViewer->GetDocument(mDoc);
+        nsCOMPtr<nsIDocument> mDoc;
+        mDViewer->GetDocument(getter_AddRefs(mDoc));
         if (nsnull != mDoc) {
-          if (NS_OK == mDoc->QueryInterface(kIDOMDocumentIID, (void**) &domDoc)) {
-          }
-          NS_RELEASE(mDoc);
+          mDoc->QueryInterface(kIDOMDocumentIID, (void**) &domDoc);
         }
         NS_RELEASE(mDViewer);
       }
@@ -2334,8 +2343,8 @@ nsBrowserWindow::DoEditorMode(nsIDocShell *aDocShell)
       nsIDocumentViewer* mDViewer;
       if (NS_OK == mCViewer->QueryInterface(kIDocumentViewerIID, (void**) &mDViewer)) 
       {
-        nsIDocument* mDoc;
-        mDViewer->GetDocument(mDoc);
+        nsCOMPtr<nsIDocument> mDoc;
+        mDViewer->GetDocument(getter_AddRefs(mDoc));
         if (nsnull != mDoc) {
           nsIDOMDocument* mDOMDoc;
           if (NS_OK == mDoc->QueryInterface(kIDOMDocumentIID, (void**) &mDOMDoc)) 
@@ -2345,7 +2354,6 @@ nsBrowserWindow::DoEditorMode(nsIDocShell *aDocShell)
             NS_RELEASE(mDOMDoc);
             NS_IF_RELEASE(shell);
           }
-          NS_RELEASE(mDoc);
         }
         NS_RELEASE(mDViewer);
       }
@@ -2374,8 +2382,8 @@ nsBrowserWindow::DoEditorTest(nsIDocShell *aDocShell, PRInt32 aCommandID)
       nsIDocumentViewer* mDViewer;
       if (NS_OK == mCViewer->QueryInterface(kIDocumentViewerIID, (void**) &mDViewer)) 
       {
-        nsIDocument* mDoc;
-        mDViewer->GetDocument(mDoc);
+        nsCOMPtr<nsIDocument> mDoc;
+        mDViewer->GetDocument(getter_AddRefs(mDoc));
         if (nsnull != mDoc) {
           nsIDOMDocument* mDOMDoc;
           if (NS_OK == mDoc->QueryInterface(kIDOMDocumentIID, (void**) &mDOMDoc)) 
@@ -2383,7 +2391,6 @@ nsBrowserWindow::DoEditorTest(nsIDocShell *aDocShell, PRInt32 aCommandID)
             NS_DoEditorTest(aCommandID);
             NS_RELEASE(mDOMDoc);
           }
-          NS_RELEASE(mDoc);
         }
         NS_RELEASE(mDViewer);
       }
@@ -2401,7 +2408,6 @@ nsBrowserWindow::DoEditorTest(nsIDocShell *aDocShell, PRInt32 aCommandID)
 #include "nsIContent.h"
 #include "nsIFrame.h"
 #include "nsIFrameDebug.h"
-#include "nsIStyleContext.h"
 #include "nsIStyleSet.h"
 
 
@@ -2517,7 +2523,7 @@ DumpFramesRecurse(nsIDocShell* aDocShell, FILE* out, nsString *aFilterName)
         shell->GetPresContext(&presContext);
 
         nsIFrameDebug* fdbg;
-        if (NS_SUCCEEDED(root->QueryInterface(NS_GET_IID(nsIFrameDebug), (void**)&fdbg))) {
+        if (NS_SUCCEEDED(CallQueryInterface(root, &fdbg))) {
           fdbg->List(presContext, out, 0);
         }
         NS_IF_RELEASE(presContext);
@@ -2552,191 +2558,6 @@ nsBrowserWindow::DumpFrames(FILE* out, nsString *aFilterName)
 
 //----------------------------------------------------------------------
 
-#include "nsISizeOfHandler.h"
-#include "nsQuickSort.h"
-
-struct SizeReportEntry {
-  nsIAtom* mKey;
-  PRUint32 mCount;
-  PRUint32 mTotalSize;
-  PRUint32 mMinSize;
-  PRUint32 mMaxSize;
-};
-
-static void
-GatherSizeReportData(nsISizeOfHandler* aHandler,
-                     nsIAtom* aKey,
-                     PRUint32 aCount,
-                     PRUint32 aTotalSize,
-                     PRUint32 aMinSize,
-                     PRUint32 aMaxSize,
-                     void* arg)
-{
-  if (aKey) {
-    nsVoidArray* array = (nsVoidArray*) arg;
-    SizeReportEntry* entry = new SizeReportEntry();
-    entry->mKey = aKey;
-    entry->mCount = aCount;
-    entry->mTotalSize = aTotalSize;
-    entry->mMinSize = aMinSize;
-    entry->mMaxSize = aMaxSize;
-    array->AppendElement((void*) entry);
-  }
-}
-
-static int PR_CALLBACK CompareEntries(const void* ve1, const void* ve2, void* closure)
-{
-  SizeReportEntry* e1 = (SizeReportEntry*) ve1;
-  SizeReportEntry* e2 = (SizeReportEntry*) ve2;
-  if (e1->mKey == e2->mKey) {
-    return 0;
-  }
-  nsAutoString k1, k2;
-  e1->mKey->ToString(k1);
-  e2->mKey->ToString(k2);
-  if (k1 < k2) {
-    return -1;
-  }
-  return 1;
-}
-
-static SizeReportEntry*
-SortFrameSizeList(nsVoidArray& sizeReportData)
-{
-  // Convert data in void array into a real array, freeing the old data
-  PRInt32 i, count = sizeReportData.Count();
-  SizeReportEntry* all = new SizeReportEntry[count];
-  SizeReportEntry* to = all;
-  for (i = 0; i < count; i++) {
-    SizeReportEntry* e = (SizeReportEntry*) sizeReportData[i];
-    *to = *e;
-    to++;
-    sizeReportData.ReplaceElementAt(nsnull, i);
-    delete e;
-  }
-
-  // Now sort the array
-  NS_QuickSort(all, count, sizeof(SizeReportEntry), CompareEntries, nsnull);
-
-  return all;
-}
-
-static void ShowReport(FILE* out, nsISizeOfHandler* aHandler)
-{
-  nsVoidArray sizeReportData;
-  aHandler->Report(GatherSizeReportData, (void*) &sizeReportData);
-
-  if (sizeReportData.Count()) {
-    PRInt32 count = sizeReportData.Count();
-    SizeReportEntry* entries = SortFrameSizeList(sizeReportData);
-
-    // Display headers
-    fprintf(out, "%-20s %-5s %-9s %-7s %-7s %-7s\n",
-            "Frame Type", "Count", "TotalSize",
-            "MinSize", "MaxSize", "AvgSize");
-    fprintf(out, "%-20s %-5s %-9s %-7s %-7s %-7s\n",
-            "----------", "-----", "---------",
-            "-------", "-------", "-------");
-
-    SizeReportEntry* entry = entries;
-    SizeReportEntry* end = entries + count;
-    for (; entry < end; entry++) {
-      nsAutoString type;
-      entry->mKey->ToString(type);
-
-      fprintf(out, "%-20s %5d %9d %7d %7d %7d\n",
-              NS_LossyConvertUCS2toASCII(type).get(),
-              entry->mCount, entry->mTotalSize,
-              entry->mMinSize, entry->mMaxSize,
-              entry->mCount ? entry->mTotalSize / entry->mCount : 0);
-    }
-    delete entries;
-
-    // Display totals and trailer
-    PRUint32 totalCount, totalSize;
-    aHandler->GetTotals(&totalCount, &totalSize);
-    fprintf(out, "%-20s %5d %9d\n",
-            "*** Total ***", totalCount, totalSize);
-  }
-}
-
-//----------------------------------------
-
-static void
-GatherFrameDataSizes(nsIPresContext* aPresContext, nsISizeOfHandler* aHandler, nsIFrame* aFrame)
-{
-  if (aFrame) {
-    // Add in the frame
-    nsCOMPtr<nsIAtom> frameType;
-    aFrame->GetFrameType(getter_AddRefs(frameType));
-    PRUint32 frameDataSize;
-    nsIFrameDebug* fdbg;
-    if (NS_SUCCEEDED(aFrame->QueryInterface(NS_GET_IID(nsIFrameDebug), (void**) &fdbg))) {
-      fdbg->SizeOf(aHandler, &frameDataSize);
-      aHandler->AddSize(frameType, frameDataSize);
-    }
-
-    // And all of its children
-    PRInt32 listIndex = 0;
-    nsIAtom* listName = nsnull;
-    do {
-      nsIFrame* kid;
-      aFrame->FirstChild(aPresContext, listName, &kid);
-      while (kid) {
-        GatherFrameDataSizes(aPresContext, aHandler, kid);
-        kid->GetNextSibling(&kid);
-      }
-      NS_IF_RELEASE(listName);
-      aFrame->GetAdditionalChildListName(listIndex, &listName);
-      listIndex++;
-    } while (nsnull != listName);
-  }
-}
-
-static void
-GatherFrameDataSizes(nsISizeOfHandler* aHandler, nsIDocShell* aDocShell)
-{
-  if (nsnull != aDocShell) {
-    nsIPresShell* shell = GetPresShellFor(aDocShell);
-    if (nsnull != shell) {
-      nsCOMPtr<nsIPresContext> presContext;
-      nsIFrame* root;
-      shell->GetRootFrame(&root);
-      shell->GetPresContext(getter_AddRefs(presContext));
-      if (nsnull != root) {
-        GatherFrameDataSizes(presContext, aHandler, root);
-      }
-      NS_RELEASE(shell);
-    }
-
-    // dump the frames of the sub documents
-    PRInt32 i, n;
-    nsCOMPtr<nsIDocShellTreeNode> docShellAsNode(do_QueryInterface(aDocShell));
-    docShellAsNode->GetChildCount(&n);
-    for (i = 0; i < n; i++) {
-      nsCOMPtr<nsIDocShellTreeItem> child;
-      docShellAsNode->GetChildAt(i, getter_AddRefs(child));
-      nsCOMPtr<nsIDocShell> childAsShell(do_QueryInterface(child));
-      if (childAsShell) {
-        GatherFrameDataSizes(aHandler, childAsShell);
-      }
-    }
-  }
-}
-
-void
-nsBrowserWindow::ShowFrameSize(FILE* out)
-{
-  nsCOMPtr<nsISizeOfHandler> handler;
-  nsresult rv = NS_NewSizeOfHandler(getter_AddRefs(handler));
-  if (NS_SUCCEEDED(rv) && handler) {
-    GatherFrameDataSizes(handler, mDocShell);
-    ShowReport(out, handler);
-  }
-}
-
-//----------------------------------------------------------------------
-
 static
 void
 DumpViewsRecurse(nsIDocShell* aDocShell, FILE* out)
@@ -2816,15 +2637,7 @@ nsBrowserWindow::DumpStyleContexts(FILE* out)
       if (nsnull == root) {
         fputs("null root frame\n", out);
       } else {
-        nsIStyleContext* rootContext;
-        root->GetStyleContext(&rootContext);
-        if (nsnull != rootContext) {
-          styleSet->ListContexts(rootContext, out);
-          NS_RELEASE(rootContext);
-        }
-        else {
-          fputs("null root context", out);
-        }
+        styleSet->ListContexts(root, out);
       }
     }
     NS_RELEASE(shell);
@@ -2969,129 +2782,6 @@ nsBrowserWindow::GetStringPref(const char * aPrefName, nsString& aValue)
 }
 
 #ifdef NS_DEBUG
-
-//----------------------------------------------------------------------
-
-static void
-GatherContentDataSizes(nsISizeOfHandler* aHandler, nsIContent* aContent)
-{
-  if (aContent) {
-    // Add in the content
-    nsCOMPtr<nsIAtom> tag;
-    aContent->GetTag(*getter_AddRefs(tag));
-    if (!tag) {
-      tag = getter_AddRefs(NS_NewAtom(":content-no-tag"));
-    }
-    PRUint32 contentDataSize;
-    aContent->SizeOf(aHandler, &contentDataSize);
-    aHandler->AddSize(tag, contentDataSize);
-
-    // And all of its children
-    PRInt32 i, childCount;
-    aContent->ChildCount(childCount);
-    for (i = 0; i < childCount; i++) {
-      nsIContent* kid;
-      aContent->ChildAt(i, kid);
-      if (kid) {
-        GatherContentDataSizes(aHandler, kid);
-        NS_RELEASE(kid);
-      }
-    }
-  }
-}
-
-static void
-GatherContentDataSizes(nsISizeOfHandler* aHandler, nsIDocShell* aDocShell)
-{
-  if (nsnull != aDocShell) {
-    nsIPresShell* shell = GetPresShellFor(aDocShell);
-    if (nsnull != shell) {
-      nsIDocument* doc;
-      shell->GetDocument(&doc);
-      if (doc) {
-        nsCOMPtr<nsIContent> rootContent;
-        doc->GetRootContent(getter_AddRefs(rootContent));
-        if (rootContent) {
-          GatherContentDataSizes(aHandler, rootContent.get());
-        }
-        NS_RELEASE(doc);
-      }
-      NS_RELEASE(shell);
-    }
-
-    // dump the frames of the sub documents
-    PRInt32 i, n;
-    nsCOMPtr<nsIDocShellTreeNode> docShellAsNode(do_QueryInterface(aDocShell));
-    docShellAsNode->GetChildCount(&n);
-    for (i = 0; i < n; i++) {
-      nsCOMPtr<nsIDocShellTreeItem> child;
-      docShellAsNode->GetChildAt(i, getter_AddRefs(child));
-      nsCOMPtr<nsIDocShell> childAsShell(do_QueryInterface(child));
-      if (childAsShell) {
-        GatherContentDataSizes(aHandler, childAsShell);
-      }
-    }
-  }
-}
-
-void
-nsBrowserWindow::ShowContentSize(FILE* out)
-{
-  nsCOMPtr<nsISizeOfHandler> handler;
-  nsresult rv = NS_NewSizeOfHandler(getter_AddRefs(handler));
-  if (NS_SUCCEEDED(rv) && handler) {
-    GatherContentDataSizes(handler, mDocShell);
-    ShowReport(out, handler);
-  }
-}
-
-//----------------------------------------------------------------------
-
-static void
-GatherStyleDataSizes(nsISizeOfHandler* aHandler, nsIDocShell* aDocShell)
-{
-  if (nsnull != aDocShell) {
-    nsIPresShell* shell = GetPresShellFor(aDocShell);
-    if (nsnull != shell) {
-      nsIStyleSet *styleSet;
-      shell->GetStyleSet(&styleSet);
-      if(styleSet){
-        PRUint32 ssSize;
-        styleSet->ResetUniqueStyleItems();
-        // size of the style set
-        styleSet->SizeOf(aHandler, ssSize);
-        NS_RELEASE(styleSet);
-      }
-      nsIFrame* root;
-      shell->GetRootFrame(&root);
-      if (nsnull == root) {
-        puts("null root frame\n");
-      } else {
-        nsIStyleContext* rootContext;
-        root->GetStyleContext(&rootContext);
-        if (nsnull != rootContext) {
-          PRUint32 contextSize;
-          // get the sizes from the root context
-          rootContext->SizeOf(aHandler, contextSize);
-          NS_RELEASE(rootContext);
-        }
-      }
-      NS_RELEASE(shell);
-    }
-  }
-}
-
-void
-nsBrowserWindow::ShowStyleSize(FILE* out)
-{
-  nsCOMPtr<nsISizeOfHandler> handler;
-  nsresult rv = NS_NewSizeOfHandler(getter_AddRefs(handler));
-  if (NS_SUCCEEDED(rv) && handler) {
-    GatherStyleDataSizes(handler, mDocShell);
-    ShowReport(out, handler);
-  }
-}
-
 void
 nsBrowserWindow::DoDebugSave()
 {
@@ -3213,21 +2903,6 @@ nsBrowserWindow::DispatchDebugMenu(PRInt32 aID)
       result = nsEventStatus_eConsumeNoDefault;
       break;
 
-    case VIEWER_SHOW_CONTENT_SIZE:
-      ShowContentSize();
-      result = nsEventStatus_eConsumeNoDefault;
-      break;
-
-    case VIEWER_SHOW_FRAME_SIZE:
-      ShowFrameSize();
-      result = nsEventStatus_eConsumeNoDefault;
-      break;
-
-    case VIEWER_SHOW_STYLE_SIZE:
-      ShowStyleSize();
-      result = nsEventStatus_eConsumeNoDefault;
-      break;
-
     case VIEWER_SHOW_CONTENT_QUALITY:
       if (nsnull != mDocShell) {
         nsIPresShell   *ps = GetPresShellFor(mDocShell);
@@ -3283,7 +2958,7 @@ nsBrowserWindow::SetCompatibilityMode(PRUint32 aMode)
   if (pref) { 
     int32 prefInt = USE_DTD;
     if (STANDARD == aMode) {
-      prefInt = eCompatibility_Standard;
+      prefInt = eCompatibility_FullStandards;
     }
     else if (QUIRKS == aMode) {
       prefInt = eCompatibility_NavQuirks;
