@@ -50,9 +50,9 @@
 #include "nsGfxUtils.h"
 #include "DefProcFakery.h"
 #include "nsMacResources.h"
-#include "nsRegionMac.h"
 #include "nsIRollupListener.h"
 #include "nsCRT.h"
+#include "nsWidgetSupport.h"
 
 #if TARGET_CARBON
 #include <CFString.h>
@@ -90,17 +90,15 @@ long CallSystemWDEF ( short inCode, WindowPtr inWindow, short inMessage, long in
 
 #define kWindowPositionSlop 20
 
-//  TODO: these should come from the system, not be hard-coded. What if I'm running
-// an elaborate theme with wide window borders?
+// These are only initial guesses. Real values are filled in
+// after window creation.
 const short kWindowTitleBarHeight = 22;
 const short kWindowMarginWidth = 6;
 const short kDialogTitleBarHeight = 26;
 const short kDialogMarginWidth = 6;
 
-
 #pragma mark -
 
-
 pascal OSErr
 nsMacWindow :: DragTrackingHandler ( DragTrackingMessage theMessage, WindowPtr theWindow, 
                     void *handlerRefCon, DragReference theDrag)
@@ -112,7 +110,6 @@ nsMacWindow :: DragTrackingHandler ( DragTrackingMessage theMessage, WindowPtr t
   if ( !theWindow || !windowEventSink )
     return dragNotAcceptedErr;
     
-  nsresult rv = NS_OK;
   switch ( theMessage ) {
   
     case kDragTrackingEnterHandler:
@@ -188,15 +185,14 @@ NS_IMPL_ISUPPORTS_INHERITED4(nsMacWindow, Inherited, nsIEventSink, nsPIWidgetMac
 //-------------------------------------------------------------------------
 nsMacWindow::nsMacWindow() : Inherited()
   , mWindowMadeHere(PR_FALSE)
-  , mIsDialog(PR_FALSE)
   , mIsSheet(PR_FALSE)
   , mIgnoreDeactivate(PR_FALSE)
-  , mMacEventHandler(nsnull)
   , mAcceptsActivation(PR_TRUE)
   , mIsActive(PR_FALSE)
   , mZoomOnShow(PR_FALSE)
   , mZooming(PR_FALSE)
   , mResizeIsFromUs(PR_FALSE)
+  , mMacEventHandler(nsnull)
 #if !TARGET_CARBON
   , mPhantomScrollbar(nsnull)
   , mPhantomScrollbarData(nsnull)
@@ -208,6 +204,8 @@ nsMacWindow::nsMacWindow() : Inherited()
   // create handlers for drag&drop
   mDragTrackingHandlerUPP = NewDragTrackingHandlerUPP(DragTrackingHandler);
   mDragReceiveHandlerUPP = NewDragReceiveHandlerUPP(DragReceiveHandler);
+  mBoundsOffset.v = kWindowTitleBarHeight; // initial guesses
+  mBoundsOffset.h = kWindowMarginWidth;
 }
 
 
@@ -255,7 +253,10 @@ nsMacWindow::~nsMacWindow()
     if (mustResetPort)
       nsGraphicsUtils::SetPortToKnownGoodPort();
   }
-  
+  else if ( mWindowPtr && !mWindowMadeHere ) {
+    (void)::RemoveWindowProperty(mWindowPtr, kTopLevelWidgetPropertyCreator,
+        kTopLevelWidgetRefPropertyTag);
+  }
 }
 
 
@@ -281,17 +282,18 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
   // build the main native window
   if (aNativeParent == nsnull)
   {
-    nsWindowType windowType;
     if (aInitData) {
       mWindowType = aInitData->mWindowType;
       // if a toplevel window was requested without a titlebar, use a dialog windowproc
       if (aInitData->mWindowType == eWindowType_toplevel &&
-        (aInitData->mBorderStyle == eBorderStyle_none ||
-         aInitData->mBorderStyle != eBorderStyle_all && !(aInitData->mBorderStyle & eBorderStyle_title)))
-        windowType = eWindowType_dialog;
+          (aInitData->mBorderStyle == eBorderStyle_none ||
+           aInitData->mBorderStyle != eBorderStyle_all &&
+           aInitData->mBorderStyle != eBorderStyle_default &&
+           !(aInitData->mBorderStyle & eBorderStyle_title)))
+        mWindowType = eWindowType_dialog;
     } 
     else
-      mWindowType = (mIsDialog ? eWindowType_dialog : eWindowType_toplevel);
+      mWindowType = eWindowType_toplevel;
 
     short     wDefProcID = kWindowDocumentProc;
     Boolean   goAwayFlag = false;
@@ -437,7 +439,7 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
     {
       if (aInitData->mBorderStyle == eBorderStyle_none ||
           aInitData->mBorderStyle == eBorderStyle_default &&
-          windowType == eWindowType_dialog ||
+          mWindowType == eWindowType_dialog ||
           !(aInitData->mBorderStyle & eBorderStyle_close))
         goAwayFlag = false;
     }
@@ -468,11 +470,18 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
     }
     mWindowPtr = ::NewCWindow(nil, &wRect, "\p", false, wDefProcID, (WindowRef)-1, goAwayFlag, (long)nsnull);
     mWindowMadeHere = PR_TRUE;
+
+    Rect content, structure;
+    ::GetWindowBounds(mWindowPtr, kWindowStructureRgn, &structure);
+    ::GetWindowBounds(mWindowPtr, kWindowContentRgn, &content);
+    mBoundsOffset.v = content.top - structure.top;
+    mBoundsOffset.h = content.left - structure.left;
   }
   else
   {
     mWindowPtr = (WindowPtr)aNativeParent;
     mWindowMadeHere = PR_FALSE;
+    mVisible = PR_TRUE;
   }
 
   if (mWindowPtr == nsnull)
@@ -483,7 +492,9 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
   // event handlers to get our widget or event sink when all they have
   // is a native WindowPtr.
   nsIWidget* temp = NS_STATIC_CAST(nsIWidget*, this);
-  OSStatus swpStatus = ::SetWindowProperty ( mWindowPtr, 'MOSS', 'GEKO', sizeof(nsIWidget*), &temp );
+  OSStatus swpStatus = ::SetWindowProperty ( mWindowPtr,
+                          kTopLevelWidgetPropertyCreator, kTopLevelWidgetRefPropertyTag,
+                          sizeof(nsIWidget*), &temp );
   NS_ASSERTION ( swpStatus == noErr, "couldn't set a property on the window, event handling will fail" );
   if ( swpStatus != noErr )
     return NS_ERROR_FAILURE;
@@ -1106,7 +1117,6 @@ NS_IMETHODIMP nsMacWindow::Move(PRInt32 aX, PRInt32 aY)
   StPortSetter setOurPortForLocalToGlobal ( mWindowPtr );
   
   if (eWindowType_popup == mWindowType) {
-    PRInt32 xOffset=0,yOffset=0;
     nsRect  localRect,globalRect;
 
     // convert to screen coordinates
@@ -1134,8 +1144,13 @@ NS_IMETHODIMP nsMacWindow::Move(PRInt32 aX, PRInt32 aY)
       if ( abs(currBounds.left-aX) > kMoveThreshold || abs(currBounds.top-aY) > kMoveThreshold ) {
         ::MoveWindow(mWindowPtr, aX, aY, false);
         
-        Rect newBounds;
-        ::GetWindowBounds ( mWindowPtr, kWindowGlobalPortRgn, &newBounds );
+        // update userstate to match, if appropriate
+        PRInt32 sizeMode;
+        nsBaseWidget::GetSizeMode ( &sizeMode );
+        if ( sizeMode == nsSizeMode_Normal ) {
+          ::GetWindowBounds ( mWindowPtr, kWindowGlobalPortRgn, &currBounds );
+          ::SetWindowUserState ( mWindowPtr, &currBounds );
+        }  
       }  
     }
 
@@ -1144,13 +1159,8 @@ NS_IMETHODIMP nsMacWindow::Move(PRInt32 aX, PRInt32 aY)
     Rect portBounds;
     ::GetWindowPortBounds(mWindowPtr, &portBounds);
 
-    if (mIsDialog) {
-      aX += kDialogMarginWidth;
-      aY += kDialogTitleBarHeight;
-    } else {
-      aX += kWindowMarginWidth;
-      aY += kWindowTitleBarHeight;
-    }
+    aX += mBoundsOffset.h;
+    aY += mBoundsOffset.v;
 
     nsCOMPtr<nsIScreenManager> screenmgr = do_GetService(sScreenManagerContractID);
     if (screenmgr) {
@@ -1173,11 +1183,18 @@ NS_IMETHODIMP nsMacWindow::Move(PRInt32 aX, PRInt32 aY)
 
     // move the window if it has not been moved yet
     // (ie. if this function isn't called in response to a DragWindow event)
-    Point macPoint = topLeft(portBounds);
-    ::LocalToGlobal(&macPoint);
-    if (macPoint.h != aX || macPoint.v != aY)
+    ::LocalToGlobal((Point *) &portBounds.top);
+    ::LocalToGlobal((Point *) &portBounds.bottom);
+    if (portBounds.left != aX || portBounds.top != aY) {
       ::MoveWindow(mWindowPtr, aX, aY, false);
 
+      // update userstate to match, if appropriate
+      PRInt32 sizeMode;
+      GetSizeMode(&sizeMode);
+      if (sizeMode == nsSizeMode_Normal)
+        ::SetWindowUserState(mWindowPtr, &portBounds);
+    }
+
     // propagate the event in global coordinates
     Inherited::Move(aX, aY);
 
@@ -1286,6 +1303,9 @@ nsMacWindow::UserStateForResize()
 
   // finally, reset our internal state to match
   nsBaseWidget::SetSizeMode(nsSizeMode_Normal);
+  // Tell container window we've moved. We stopped reporting move events
+  // when we were in the StandardState.
+  ReportMoveEvent();
 }
 
 
@@ -1444,14 +1464,7 @@ void nsMacWindow::MoveToGlobalPoint(PRInt32 aX, PRInt32 aY)
     }
   }
 
-  if (mIsDialog) {
-    aX -= kDialogMarginWidth;
-    aY -= kDialogTitleBarHeight;
-  } else {
-    aX -= kWindowMarginWidth;
-    aY -= kWindowTitleBarHeight;
-  }
-  Move(aX, aY);
+  Move(aX-mBoundsOffset.h, aY-mBoundsOffset.v);
 }
 
 //-------------------------------------------------------------------------
@@ -1497,6 +1510,16 @@ NS_IMETHODIMP nsMacWindow::Resize(PRInt32 aWidth, PRInt32 aHeight, PRBool aRepai
       // make sure that we don't infinitely recurse if live-resize is on
       mResizeIsFromUs = PR_TRUE;
       ::SizeWindow(mWindowPtr, aWidth, aHeight, aRepaint);
+
+      // update userstate to match, if appropriate
+      PRInt32 sizeMode;
+      GetSizeMode(&sizeMode);
+      if (sizeMode == nsSizeMode_Normal) {
+        Rect portBounds;
+        ::GetWindowBounds(mWindowPtr, kWindowGlobalPortRgn, &portBounds);
+        ::SetWindowUserState(mWindowPtr, &portBounds);
+      }
+
       mResizeIsFromUs = PR_FALSE;
 
 #if defined(XP_MACOSX)
@@ -1535,10 +1558,7 @@ NS_IMETHODIMP nsMacWindow::GetScreenBounds(nsRect &aRect) {
     }
   }
  
-  if (mIsDialog)
-    aRect.MoveBy(-kDialogMarginWidth, -kDialogTitleBarHeight-yAdjust);
-  else
-    aRect.MoveBy(-kWindowMarginWidth, -kWindowTitleBarHeight-yAdjust);
+  aRect.MoveBy(-mBoundsOffset.h, -mBoundsOffset.v-yAdjust);
 
   return NS_OK;
 }
@@ -1671,6 +1691,15 @@ nsMacWindow::HandlePosToOffset(PRInt16 currentPointX, PRInt16 currentPointY,
   return NS_OK;
 }
 
+/* OSStatus HandleGetSelectedText (out AString selectedText); */
+NS_IMETHODIMP 
+nsMacWindow::HandleGetSelectedText(nsAString & selectedText, OSStatus *_retval)
+{
+  *_retval = noErr;
+  NS_ENSURE_TRUE(mMacEventHandler.get(), NS_ERROR_FAILURE);
+  return mMacEventHandler->HandleUnicodeGetSelectedText(selectedText);
+}
+
 #pragma mark -
 
 
