@@ -57,7 +57,7 @@
 
 #include "nsWidgetAtoms.h"
 
-#include "nsIDOMXULDocument.h"
+#include "nsIDOMDocument.h"
 
 #include <Menus.h>
 #include <TextUtils.h>
@@ -113,7 +113,6 @@ nsMenuBar::nsMenuBar()
   
   nsPreviousMenuStackUnwind(nsnull, nsnull);
 
-  NS_INIT_ISUPPORTS();
   mNumMenus       = 0;
   mParent         = nsnull;
   mIsMenuBarAdded = PR_FALSE;
@@ -268,7 +267,7 @@ nsMenuBar :: GetDocument ( nsIWebShell* inWebShell, nsIDocument** outDocument )
       nsCOMPtr<nsIDocumentViewer> docv(do_QueryInterface(cv));
       if (!docv)
         return;
-      docv->GetDocument(*outDocument);    // addrefs
+      docv->GetDocument(outDocument);    // addrefs
     }
   }
 }
@@ -481,7 +480,7 @@ NS_METHOD nsMenuBar::AddMenu(nsIMenu * aMenu)
       aMenu->GetMenuContent(getter_AddRefs(menu));
       if (menu) {
         nsCOMPtr<nsIDocument> doc;
-        menu->GetDocument(*getter_AddRefs(doc));
+        menu->GetDocument(getter_AddRefs(doc));
         if (doc) {
           nsCOMPtr<nsIDOMDocument> domdoc ( do_QueryInterface(doc) );
           if ( domdoc ) {
@@ -658,7 +657,7 @@ nsMenuBar::ContentAppended( nsIDocument * aDocument, nsIContent  * aContainer,
       obs->ContentInserted ( aDocument, aContainer, aNewIndexInContainer );
     else {
       nsCOMPtr<nsIContent> parent;
-      aContainer->GetParent(*getter_AddRefs(parent));
+      aContainer->GetParent(getter_AddRefs(parent));
       if(parent) {
         Lookup ( parent, getter_AddRefs(obs) );
         if ( obs )
@@ -712,7 +711,7 @@ nsMenuBar::ContentRemoved( nsIDocument * aDocument, nsIContent * aContainer,
       obs->ContentRemoved ( aDocument, aChild, aIndexInContainer );
     else {
       nsCOMPtr<nsIContent> parent;
-      aContainer->GetParent(*getter_AddRefs(parent));
+      aContainer->GetParent(getter_AddRefs(parent));
       if(parent) {
         Lookup ( parent, getter_AddRefs(obs) );
         if ( obs )
@@ -738,7 +737,7 @@ nsMenuBar::ContentInserted( nsIDocument * aDocument, nsIContent * aContainer,
       obs->ContentInserted ( aDocument, aChild, aIndexInContainer );
     else {
       nsCOMPtr<nsIContent> parent;
-      aContainer->GetParent(*getter_AddRefs(parent));
+      aContainer->GetParent(getter_AddRefs(parent));
       if(parent) {
         Lookup ( parent, getter_AddRefs(obs) );
         if ( obs )
