@@ -56,7 +56,7 @@
 #include "nsIDocumentViewer.h"
 #include "nsIDocumentObserver.h"
 
-#include "nsIDOMXULDocument.h"
+#include "nsIDOMDocument.h"
 #include "nsWidgetAtoms.h"
 
 #include <Menus.h>
@@ -93,8 +93,6 @@ EventHandlerUPP nsMenuBarX::sCommandEventHandler = nsnull;
 nsMenuBarX::nsMenuBarX()
   : mNumMenus(0), mParent(nsnull), mIsMenuBarAdded(PR_FALSE), mDocument(nsnull), mCurrentCommandID(1)
 {
-  NS_INIT_ISUPPORTS();
-
   OSStatus status = ::CreateNewMenu(0, 0, &mRootMenu);
   NS_ASSERTION(status == noErr, "nsMenuBarX::nsMenuBarX:  creation of root menu failed.");
   
@@ -220,7 +218,7 @@ nsMenuBarX :: GetDocument ( nsIWebShell* inWebShell, nsIDocument** outDocument )
       nsCOMPtr<nsIDocumentViewer> docv(do_QueryInterface(cv));
       if (!docv)
         return;
-      docv->GetDocument(*outDocument);    // addrefs
+      docv->GetDocument(outDocument);    // addrefs
     }
   }
 }
@@ -259,7 +257,7 @@ void
 nsMenuBarX :: AquifyMenuBar ( )
 {
   nsCOMPtr<nsIDocument> containingDoc;
-  mMenuBarContent->GetDocument ( *getter_AddRefs(containingDoc) );
+  mMenuBarContent->GetDocument ( getter_AddRefs(containingDoc) );
   nsCOMPtr<nsIDOMDocument> domDoc ( do_QueryInterface(containingDoc) );
   if ( domDoc ) {
     // remove quit item and its separator
@@ -341,13 +339,15 @@ nsMenuBarX :: CommandEventHandler ( EventHandlerCallRef inHandlerChain, EventRef
         
         case kHICommandAbout:
         {
-          // the 'about' command is special because we don't have a nsIMenu or nsIMenuItem
-          // for the apple menu. Grovel for the content node with an id of "aboutName" 
-          // and call it directly.
-          nsCOMPtr<nsIDOMXULDocument> xulDoc = do_QueryInterface(self->mDocument);
-	        if ( xulDoc ) {
+          // the 'about' command is special because we don't have a
+          // nsIMenu or nsIMenuItem for the apple menu. Grovel for the
+          // content node with an id of "aboutName" and call it
+          // directly.
+          nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(self->mDocument);
+	        if ( domDoc ) {
       	    nsCOMPtr<nsIDOMElement> domElement;
-      	    xulDoc->GetElementById(NS_LITERAL_STRING("aboutName"), getter_AddRefs(domElement));
+      	    domDoc->GetElementById(NS_LITERAL_STRING("aboutName"),
+                                   getter_AddRefs(domElement));
       	    nsCOMPtr<nsIContent> aboutContent ( do_QueryInterface(domElement) );
       	    self->ExecuteCommand(aboutContent);
           }
@@ -481,10 +481,10 @@ nsMenuBarX::MenuConstruct( const nsMenuEvent & aMenuEvent, nsIWidget* aParentWin
   mMenuBarContent->ChildCount(count);
   for ( int i = 0; i < count; ++i ) { 
     nsCOMPtr<nsIContent> menu;
-    mMenuBarContent->ChildAt ( i, *getter_AddRefs(menu) );
+    mMenuBarContent->ChildAt ( i, getter_AddRefs(menu) );
     if ( menu ) {
       nsCOMPtr<nsIAtom> tag;
-      menu->GetTag ( *getter_AddRefs(tag) );
+      menu->GetTag ( getter_AddRefs(tag) );
       if (tag == nsWidgetAtoms::menu) {
         nsAutoString menuName;
         nsAutoString menuAccessKey(NS_LITERAL_STRING(" "));
@@ -631,7 +631,7 @@ nsMenuBarX :: CreateAppleMenu ( nsIMenu* inMenu )
     inMenu->GetMenuContent(getter_AddRefs(menu));
     if (menu) {
       nsCOMPtr<nsIDocument> doc;
-      menu->GetDocument(*getter_AddRefs(doc));
+      menu->GetDocument(getter_AddRefs(doc));
       if (doc) {
         nsCOMPtr<nsIDOMDocument> domdoc ( do_QueryInterface(doc) );
         if ( domdoc ) {
@@ -772,7 +772,7 @@ nsMenuBarX::ContentAppended( nsIDocument * aDocument, nsIContent  * aContainer,
       obs->ContentInserted ( aDocument, aContainer, aNewIndexInContainer );
     else {
       nsCOMPtr<nsIContent> parent;
-      aContainer->GetParent(*getter_AddRefs(parent));
+      aContainer->GetParent(getter_AddRefs(parent));
       if(parent) {
         Lookup ( parent, getter_AddRefs(obs) );
         if ( obs )
@@ -826,7 +826,7 @@ nsMenuBarX::ContentRemoved( nsIDocument * aDocument, nsIContent * aContainer,
       obs->ContentRemoved ( aDocument, aChild, aIndexInContainer );
     else {
       nsCOMPtr<nsIContent> parent;
-      aContainer->GetParent(*getter_AddRefs(parent));
+      aContainer->GetParent(getter_AddRefs(parent));
       if(parent) {
         Lookup ( parent, getter_AddRefs(obs) );
         if ( obs )
@@ -852,7 +852,7 @@ nsMenuBarX::ContentInserted( nsIDocument * aDocument, nsIContent * aContainer,
       obs->ContentInserted ( aDocument, aChild, aIndexInContainer );
     else {
       nsCOMPtr<nsIContent> parent;
-      aContainer->GetParent(*getter_AddRefs(parent));
+      aContainer->GetParent(getter_AddRefs(parent));
       if(parent) {
         Lookup ( parent, getter_AddRefs(obs) );
         if ( obs )
@@ -982,7 +982,7 @@ MenuHelpersX::WebShellToPresContext (nsIWebShell* inWebShell, nsIPresContext** o
   if ( contentViewer ) {
     nsCOMPtr<nsIDocumentViewer> docViewer ( do_QueryInterface(contentViewer) );
     if ( docViewer )
-      docViewer->GetPresContext(*outContext);     // AddRefs for us
+      docViewer->GetPresContext(outContext);     // AddRefs for us
     else
       retval = NS_ERROR_FAILURE;
   }
@@ -1016,7 +1016,7 @@ MenuHelpersX::DispatchCommandTo(nsIWeakReference* aWebShellWeakRef,
   aTargetContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::command, command);
   if (!command.IsEmpty()) {
     nsCOMPtr<nsIDocument> doc;
-    aTargetContent->GetDocument(*getter_AddRefs(doc));
+    aTargetContent->GetDocument(getter_AddRefs(doc));
     nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(doc));
     nsCOMPtr<nsIDOMElement> commandElt;
     domDoc->GetElementById(command, getter_AddRefs(commandElt));
