@@ -38,7 +38,7 @@
 #include "nsCOMPtr.h"
 #include "nsIDocument.h"
 #include "nsIContent.h"
-#include "nsIDOMXULDocument.h"
+#include "nsIDOMDocument.h"
 #include "nsIDocumentViewer.h"
 #include "nsIDocumentObserver.h"
 #include "nsIComponentManager.h"
@@ -46,7 +46,7 @@
 #include "prinrval.h"
 
 #include "nsMenuX.h"
-#include "nsMenubarX.h"
+#include "nsMenuBarX.h"
 #include "nsIMenu.h"
 #include "nsIMenuBar.h"
 #include "nsIMenuItem.h"
@@ -71,6 +71,7 @@
 #include "nsGUIEvent.h"
 
 #include "nsDynamicMDEF.h"
+#include "nsCRT.h"
 
 
 static OSStatus InstallMyMenuEventHandler(MenuRef menuRef, void* userData, EventHandlerRef* outHandler) ;
@@ -100,7 +101,6 @@ public:
 
     nsDummyMenuItemX()
     {
-        NS_INIT_REFCNT();
     }
 };
 
@@ -121,8 +121,6 @@ nsMenuX::nsMenuX()
         mIsHelpMenu(PR_FALSE), mIsEnabled(PR_TRUE), mDestroyHandlerCalled(PR_FALSE),
         mNeedsRebuild(PR_TRUE), mConstructed(PR_FALSE), mVisible(PR_TRUE), mHandler(nsnull)
 {
-  NS_INIT_REFCNT();
-
 #if DEBUG
   ++gMenuCounterX;
 #endif 
@@ -155,7 +153,7 @@ nsMenuX::~nsMenuX()
 // Create
 //
 NS_METHOD 
-nsMenuX::Create(nsISupports * aParent, const nsAReadableString &aLabel, const nsAReadableString &aAccessKey, 
+nsMenuX::Create(nsISupports * aParent, const nsAString &aLabel, const nsAString &aAccessKey, 
                 nsIChangeManager* aManager, nsIWebShell* aShell, nsIContent* aNode )
 {
   mWebShellWeakRef = getter_AddRefs(NS_GetWeakReference(aShell));
@@ -208,7 +206,7 @@ NS_METHOD nsMenuX::GetLabel(nsString &aText)
 
 
 //-------------------------------------------------------------------------
-NS_METHOD nsMenuX::SetLabel(const nsAReadableString &aText)
+NS_METHOD nsMenuX::SetLabel(const nsAString &aText)
 {
   mLabel = aText;
 
@@ -228,7 +226,7 @@ NS_METHOD nsMenuX::GetAccessKey(nsString &aText)
 }
 
 //-------------------------------------------------------------------------
-NS_METHOD nsMenuX::SetAccessKey(const nsAReadableString &aText)
+NS_METHOD nsMenuX::SetAccessKey(const nsAString &aText)
 {
   return NS_OK;
 }
@@ -587,11 +585,11 @@ nsEventStatus nsMenuX::MenuConstruct(
   menuPopup->ChildCount(count);
   for ( PRInt32 i = 0; i < count; ++i ) {
     nsCOMPtr<nsIContent> child;
-    menuPopup->ChildAt(i, *getter_AddRefs(child));
+    menuPopup->ChildAt(i, getter_AddRefs(child));
     if ( child ) {
       // depending on the type, create a menu item, separator, or submenu
       nsCOMPtr<nsIAtom> tag;
-      child->GetTag ( *getter_AddRefs(tag) );
+      child->GetTag ( getter_AddRefs(tag) );
       if ( tag == nsWidgetAtoms::menuitem )
         LoadMenuItem(this, child);
       else if ( tag == nsWidgetAtoms::menuseparator )
@@ -633,11 +631,11 @@ nsEventStatus nsMenuX::HelpMenuConstruct(
   menuPopup->ChildCount(count);
   for ( PRInt32 i = 0; i < count; ++i ) {
     nsCOMPtr<nsIContent> child;
-    menuPopup->ChildAt(i, *getter_AddRefs(child));
+    menuPopup->ChildAt(i, getter_AddRefs(child));
     if ( child ) {      
       // depending on the type, create a menu item, separator, or submenu
       nsCOMPtr<nsIAtom> tag;
-      child->GetTag ( *getter_AddRefs(tag) );
+      child->GetTag ( getter_AddRefs(tag) );
       if ( tag == nsWidgetAtoms::menuitem )
         LoadMenuItem(this, child);
       else if ( tag == nsWidgetAtoms::menuseparator )
@@ -880,16 +878,16 @@ void nsMenuX::LoadMenuItem( nsIMenu* inParentMenu, nsIContent* inMenuItemContent
 
     // Try to find the key node. Get the document so we can do |GetElementByID|
     nsCOMPtr<nsIDocument> document;
-    inMenuItemContent->GetDocument(*getter_AddRefs(document));
+    inMenuItemContent->GetDocument(getter_AddRefs(document));
     if ( !document ) 
       return;
-    nsCOMPtr<nsIDOMXULDocument> xulDocument = do_QueryInterface(document);
-    if ( !xulDocument )
+    nsCOMPtr<nsIDOMDocument> domDocument = do_QueryInterface(document);
+    if ( !domDocument )
       return;
-  
+
     nsCOMPtr<nsIDOMElement> keyElement;
     if (!keyValue.IsEmpty())
-      xulDocument->GetElementById(keyValue, getter_AddRefs(keyElement));
+      domDocument->GetElementById(keyValue, getter_AddRefs(keyElement));
     if ( keyElement ) {
       nsCOMPtr<nsIContent> keyContent ( do_QueryInterface(keyElement) );
       nsAutoString keyChar(NS_LITERAL_STRING(" "));
@@ -1029,16 +1027,16 @@ nsMenuX::OnCreate()
   // be updated.
   if (popupContent) {
     nsCOMPtr<nsIDocument> doc;
-    popupContent->GetDocument(*getter_AddRefs(doc));
+    popupContent->GetDocument(getter_AddRefs(doc));
     nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(doc));
 
     PRInt32 count;
     popupContent->ChildCount(count);
     for (PRInt32 i = 0; i < count; i++) {
       nsCOMPtr<nsIContent> grandChild;
-      popupContent->ChildAt(i, *getter_AddRefs(grandChild));
+      popupContent->ChildAt(i, getter_AddRefs(grandChild));
       nsCOMPtr<nsIAtom> tag;
-      grandChild->GetTag(*getter_AddRefs(tag));
+      grandChild->GetTag(getter_AddRefs(tag));
       if (tag.get() == nsWidgetAtoms::menuitem) {
         // See if we have a command attribute.
         nsAutoString command;
@@ -1232,7 +1230,7 @@ nsMenuX::GetMenuPopupContent(nsIContent** aResult)
   for (PRInt32 i = 0; i < count; i++) {
     PRInt32 dummy;
     nsCOMPtr<nsIContent> child;
-    mMenuContent->ChildAt(i, *getter_AddRefs(child));
+    mMenuContent->ChildAt(i, getter_AddRefs(child));
     nsCOMPtr<nsIAtom> tag;
     xblService->ResolveTag(child, &dummy, getter_AddRefs(tag));
     if (tag && tag.get() == nsWidgetAtoms::menupopup) {
@@ -1334,17 +1332,24 @@ nsMenuX::AttributeChanged(nsIDocument *aDocument, PRInt32 aNameSpaceID, nsIAtom
     
     mMenuContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::label, mLabel);
 
-    // reuse the existing menu, to avoid invalidating root menu bar.
-    NS_ASSERTION(mMacMenuHandle != NULL, "nsMenuX::AttributeChanged: invalid menu handle.");
-    RemoveAll();
-    CFStringRef titleRef = ::CFStringCreateWithCharacters(kCFAllocatorDefault, (UniChar*)mLabel.get(), mLabel.Length());
-    NS_ASSERTION(titleRef, "nsMenuX::AttributeChanged: CFStringCreateWithCharacters failed.");
-    ::SetMenuTitleWithCFString(mMacMenuHandle, titleRef);
-    ::CFRelease(titleRef);
-    
-    if (menubarParent)
-        ::DrawMenuBar();
-
+    // invalidate my parent. If we're a submenu parent, we have to rebuild
+    // the parent menu in order for the changes to be picked up. If we're
+    // a regular menu, just change the title and redraw the menubar.
+    if ( !menubarParent ) {
+      nsCOMPtr<nsIMenuListener> parentListener ( do_QueryInterface(mParent) );
+      parentListener->SetRebuild(PR_TRUE);
+    }
+    else {
+      // reuse the existing menu, to avoid rebuilding the root menu bar.
+      NS_ASSERTION(mMacMenuHandle != NULL, "nsMenuX::AttributeChanged: invalid menu handle.");
+      RemoveAll();
+      CFStringRef titleRef = ::CFStringCreateWithCharacters(kCFAllocatorDefault, (UniChar*)mLabel.get(), mLabel.Length());
+      NS_ASSERTION(titleRef, "nsMenuX::AttributeChanged: CFStringCreateWithCharacters failed.");
+      ::SetMenuTitleWithCFString(mMacMenuHandle, titleRef);
+      ::CFRelease(titleRef);
+      
+      ::DrawMenuBar();
+    }
   }
   else if(aAttribute == nsWidgetAtoms::hidden || aAttribute == nsWidgetAtoms::collapsed) {
     SetRebuild(PR_TRUE);
