@@ -50,8 +50,6 @@
 #include "nsIPlatformCharset.h"
 #include "nsIServiceManager.h"
 
-#define ABS(i) ( (i)<0 ? 0-(i) : (i) )
-
 // Variables for grabbing
 PRBool   nsWindow::sIsGrabbing = PR_FALSE;
 nsWindow *nsWindow::sGrabWindow = nsnull;
@@ -456,7 +454,7 @@ NS_IMETHODIMP nsWindow::Resize(PRInt32 aWidth,
     if (mIsTooSmall)
     {
       // if we are not shown, we don't want to force a show here, so check and see if Show(TRUE) has been called
-      NeedToShow = mShown;
+      NeedToShow = mIsShown;
       mIsTooSmall = PR_FALSE;
     }
   }
@@ -679,21 +677,21 @@ NS_IMETHODIMP nsWindow::Scroll(PRInt32 aDx, PRInt32 aDy, nsRect *aClipRect)
  
   if (aDx < 0 || aDy < 0)
   {
-    srcX = mBounds.x + ABS(aDx);
-    srcY = mBounds.y + ABS(aDy);
-    destX = mBounds.x;
-    destY = mBounds.y;
-    width = mBounds.width - ABS(aDx);
-    height = mBounds.height - ABS(aDy);
+    srcX   = mBounds.x + PR_ABS(aDx);
+    srcY   = mBounds.y + PR_ABS(aDy);
+    destX  = mBounds.x;
+    destY  = mBounds.y;
+    width  = mBounds.width  - PR_ABS(aDx);
+    height = mBounds.height - PR_ABS(aDy);
   } 
   else if (aDx > 0 || aDy > 0)
   {
-    srcX = mBounds.x;
-    srcY = mBounds.y;
-    destX = mBounds.x + ABS(aDx);
-    destY = mBounds.y + ABS(aDy);
-    width = mBounds.width - ABS(aDx);
-    height = mBounds.height - ABS(aDy);
+    srcX   = mBounds.x;
+    srcY   = mBounds.y;
+    destX  = mBounds.x + PR_ABS(aDx);
+    destY  = mBounds.y + PR_ABS(aDy);
+    width  = mBounds.width  - PR_ABS(aDx);
+    height = mBounds.height - PR_ABS(aDy);
   }
 
   XCopyArea(mDisplay, mBaseWindow, mBaseWindow, mScrollGC,
@@ -795,18 +793,18 @@ NS_IMETHODIMP nsWindow::SetTitle(const nsString& aTitle)
 
   nsCOMPtr<nsIUnicodeEncoder> encoder;
   /* get the charset */
-  nsAutoString platformCharset;
+  nsCAutoString platformCharset;
   nsCOMPtr <nsIPlatformCharset> platformCharsetService = do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
   if (NS_SUCCEEDED(rv))
     rv = platformCharsetService->GetCharset(kPlatformCharsetSel_Menu, platformCharset);
   
   if (NS_FAILED(rv))
-    platformCharset.Assign(NS_LITERAL_STRING("ISO-8859-1"));
+    platformCharset.Assign(NS_LITERAL_CSTRING("ISO-8859-1"));
 
   /* get the encoder */
   nsCOMPtr<nsICharsetConverterManager> ccm = 
            do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);  
-  rv = ccm->GetUnicodeEncoder(&platformCharset, getter_AddRefs(encoder));
+  rv = ccm->GetUnicodeEncoder(platformCharset.get(), getter_AddRefs(encoder));
 
   /* Estimate out length and allocate the buffer based on a worst-case estimate, then do
      the conversion. */
