@@ -38,10 +38,12 @@
 
 #include "nsPrimitiveHelpers.h"
 #include "nsCOMPtr.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsITransferable.h"
 #include "nsIComponentManager.h"
 #include "nsLinebreakConverter.h"
+#include "nsReadableUtils.h"
 
 #include "nsIServiceManager.h"
 #include "nsICharsetConverterManager.h"
@@ -55,7 +57,7 @@
 //
 // Given some data and the flavor it corresponds to, creates the appropriate
 // nsISupports* wrapper for passing across IDL boundaries. Right now, everything
-// creates a two-byte |nsISupportsWString|, except for "text/plain" and native
+// creates a two-byte |nsISupportsString|, except for "text/plain" and native
 // platform HTML (CF_HTML on win32)
 //
 void
@@ -66,23 +68,25 @@ nsPrimitiveHelpers :: CreatePrimitiveForData ( const char* aFlavor, void* aDataB
     return;
 
   if ( strcmp(aFlavor,kTextMime) == 0 || strcmp(aFlavor,kNativeHTMLMime) == 0 ) {
-    nsCOMPtr<nsISupportsString> primitive;
-    nsComponentManager::CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, nsnull, 
-                                       NS_GET_IID(nsISupportsString), getter_AddRefs(primitive));
+    nsCOMPtr<nsISupportsCString> primitive;
+    nsComponentManager::CreateInstance(NS_SUPPORTS_CSTRING_CONTRACTID, nsnull, 
+                                       NS_GET_IID(nsISupportsCString), getter_AddRefs(primitive));
     if ( primitive ) {
-      primitive->SetDataWithLength ( aDataLen, NS_STATIC_CAST(char*, aDataBuff) );
+      const char * start = (const char*)aDataBuff;
+      primitive->SetData(Substring(start, start + aDataLen));
       nsCOMPtr<nsISupports> genericPrimitive ( do_QueryInterface(primitive) );
       *aPrimitive = genericPrimitive;
       NS_ADDREF(*aPrimitive);
     }
   }
   else {
-    nsCOMPtr<nsISupportsWString> primitive;
-    nsresult rv = nsComponentManager::CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID, nsnull, 
-                                                      NS_GET_IID(nsISupportsWString), getter_AddRefs(primitive));
+    nsCOMPtr<nsISupportsString> primitive;
+    nsresult rv = nsComponentManager::CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, nsnull, 
+                                                      NS_GET_IID(nsISupportsString), getter_AddRefs(primitive));
     if (NS_SUCCEEDED(rv) && primitive ) {
-      // recall that SetDataWithLength() takes length as characters, not bytes
-      primitive->SetDataWithLength ( aDataLen / 2, NS_STATIC_CAST(PRUnichar*, aDataBuff) );
+      // recall that length takes length as characters, not bytes
+      const PRUnichar* start = (const PRUnichar*)aDataBuff;
+      primitive->SetData(Substring(start, start + (aDataLen / 2)));
       nsCOMPtr<nsISupports> genericPrimitive ( do_QueryInterface(primitive) );
       *aPrimitive = genericPrimitive;
       NS_ADDREF(*aPrimitive);
@@ -107,14 +111,20 @@ nsPrimitiveHelpers :: CreateDataFromPrimitive ( const char* aFlavor, nsISupports
     return;
 
   if ( strcmp(aFlavor,kTextMime) == 0 ) {
-    nsCOMPtr<nsISupportsString> plainText ( do_QueryInterface(aPrimitive) );
-    if ( plainText )
-      plainText->GetData ( NS_REINTERPRET_CAST(char**, aDataBuff) );
+    nsCOMPtr<nsISupportsCString> plainText ( do_QueryInterface(aPrimitive) );
+    if ( plainText ) {
+      nsCAutoString data;
+      plainText->GetData ( data );
+      *aDataBuff = ToNewCString(data);
+    }
   }
   else {
-    nsCOMPtr<nsISupportsWString> doubleByteText ( do_QueryInterface(aPrimitive) );
-    if ( doubleByteText )
-      doubleByteText->GetData ( NS_REINTERPRET_CAST(PRUnichar**, aDataBuff) );
+    nsCOMPtr<nsISupportsString> doubleByteText ( do_QueryInterface(aPrimitive) );
+    if ( doubleByteText ) {
+      nsAutoString data;
+      doubleByteText->GetData ( data );
+      *aDataBuff = ToNewUnicode(data);
+    }
   }
 
 }
@@ -128,12 +138,12 @@ nsPrimitiveHelpers :: CreateDataFromPrimitive ( const char* aFlavor, nsISupports
 // string, not the # of bytes in the buffer. The |outPlainTextData| is null terminated, 
 // but its length parameter, |outPlainTextLen|, does not reflect that.
 //
-void
+nsresult
 nsPrimitiveHelpers :: ConvertUnicodeToPlatformPlainText ( PRUnichar* inUnicode, PRInt32 inUnicodeLen, 
                                                             char** outPlainTextData, PRInt32* outPlainTextLen )
 {
   if ( !outPlainTextData || !outPlainTextLen )
-    return;
+    return NS_ERROR_INVALID_ARG;
 
   // Get the appropriate unicode encoder. We're guaranteed that this won't change
   // through the life of the app so we can cache it.
@@ -141,28 +151,29 @@ nsPrimitiveHelpers :: ConvertUnicodeToPlatformPlainText ( PRUnichar* inUnicode,
   nsCOMPtr<nsIUnicodeEncoder> encoder;
 
   // get the charset
-  nsAutoString platformCharset;
+  nsCAutoString platformCharset;
   nsCOMPtr <nsIPlatformCharset> platformCharsetService = do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
   if (NS_SUCCEEDED(rv))
     rv = platformCharsetService->GetCharset(kPlatformCharsetSel_PlainTextInClipboard, platformCharset);
   if (NS_FAILED(rv))
-    platformCharset.Assign(NS_LITERAL_STRING("ISO-8859-1"));
+    platformCharset.Assign(NS_LITERAL_CSTRING("ISO-8859-1"));
+  
 
   // use transliterate to convert things like smart quotes to normal quotes for plain text
-  nsCAutoString cPlatformCharset;
-  cPlatformCharset.AssignWithConversion(platformCharset);
 
   nsCOMPtr<nsISaveAsCharset> converter = do_CreateInstance("@mozilla.org/intl/saveascharset;1");
-  converter->Init(cPlatformCharset.get(),
+  rv = converter->Init(platformCharset.get(),
                   nsISaveAsCharset::attr_EntityAfterCharsetConv +
                   nsISaveAsCharset::attr_FallbackQuestionMark,
                   nsIEntityConverter::transliterate);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  converter->Convert(inUnicode, outPlainTextData);
+  rv = converter->Convert(inUnicode, outPlainTextData);
   *outPlainTextLen = *outPlainTextData ? strlen(*outPlainTextData) : 0;
 
   NS_ASSERTION ( NS_SUCCEEDED(rv), "Error converting unicode to plain text" );
   
+  return rv;
 } // ConvertUnicodeToPlatformPlainText
 
 
@@ -174,12 +185,12 @@ nsPrimitiveHelpers :: ConvertUnicodeToPlatformPlainText ( PRUnichar* inUnicode,
 // but its length parameter, |outUnicodeLen|, does not reflect that. |outUnicodeLen| is
 // the length of the string in characters, not bytes.
 //
-void
+nsresult
 nsPrimitiveHelpers :: ConvertPlatformPlainTextToUnicode ( const char* inText, PRInt32 inTextLen, 
                                                             PRUnichar** outUnicode, PRInt32* outUnicodeLen )
 {
   if ( !outUnicode || !outUnicodeLen )
-    return;
+    return NS_ERROR_INVALID_ARG;
 
   // Get the appropriate unicode decoder. We're guaranteed that this won't change
   // through the life of the app so we can cache it.
@@ -188,17 +199,18 @@ nsPrimitiveHelpers :: ConvertPlatformPlainTextToUnicode ( const char* inText, PR
   static PRBool hasConverter = PR_FALSE;
   if ( !hasConverter ) {
     // get the charset
-    nsAutoString platformCharset;
+    nsCAutoString platformCharset;
     nsCOMPtr <nsIPlatformCharset> platformCharsetService = do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
     if (NS_SUCCEEDED(rv))
       rv = platformCharsetService->GetCharset(kPlatformCharsetSel_PlainTextInClipboard, platformCharset);
     if (NS_FAILED(rv))
-      platformCharset.Assign(NS_LITERAL_STRING("ISO-8859-1"));
+      platformCharset.Assign(NS_LITERAL_CSTRING("ISO-8859-1"));
       
     // get the decoder
     nsCOMPtr<nsICharsetConverterManager> ccm = 
              do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);  
-    rv = ccm->GetUnicodeDecoder(&platformCharset, getter_AddRefs(decoder));
+    rv = ccm->GetUnicodeDecoderRaw(platformCharset.get(),
+                                   getter_AddRefs(decoder));
 
     hasConverter = PR_TRUE;
   }
@@ -216,6 +228,7 @@ nsPrimitiveHelpers :: ConvertPlatformPlainTextToUnicode ( const char* inText, PR
 
   NS_ASSERTION ( NS_SUCCEEDED(rv), "Error converting plain text to unicode" );
 
+  return rv;
 } // ConvertPlatformPlainTextToUnicode
 
 
