@@ -85,10 +85,23 @@ nsLookAndFeelIntPref nsXPLookAndFeel::sIntPrefs[] =
     PR_FALSE, nsLookAndFeelTypeInt, 0 },
   { "ui.caretBlinkTime", eMetric_CaretBlinkTime, PR_FALSE, nsLookAndFeelTypeInt, 0 },
   { "ui.caretWidthTwips", eMetric_SingleLineCaretWidth, PR_FALSE, nsLookAndFeelTypeInt, 0 },
+  { "ui.caretVisibleWithSelection", eMetric_ShowCaretDuringSelection, PR_FALSE, nsLookAndFeelTypeInt, 0 },
   { "ui.submenuDelay", eMetric_SubmenuDelay, PR_FALSE, nsLookAndFeelTypeInt, 0 },
   { "ui.dragFullWindow", eMetric_DragFullWindow, PR_FALSE, nsLookAndFeelTypeInt, 0 },
+  { "ui.dragThresholdX", eMetric_DragThresholdX, PR_FALSE, nsLookAndFeelTypeInt, 0 },
+  { "ui.dragThresholdY", eMetric_DragThresholdY, PR_FALSE, nsLookAndFeelTypeInt, 0 },
   { "ui.menusCanOverlapOSBar", eMetric_MenusCanOverlapOSBar,
     PR_FALSE, nsLookAndFeelTypeInt, 0 },
+  { "ui.treeOpenDelay",
+    eMetric_TreeOpenDelay, PR_FALSE, nsLookAndFeelTypeInt, 0 },
+  { "ui.treeCloseDelay",
+    eMetric_TreeCloseDelay, PR_FALSE, nsLookAndFeelTypeInt, 0 },
+  { "ui.treeLazyScrollDelay",
+    eMetric_TreeLazyScrollDelay, PR_FALSE, nsLookAndFeelTypeInt, 0 },
+  { "ui.treeScrollDelay",
+    eMetric_TreeScrollDelay, PR_FALSE, nsLookAndFeelTypeInt, 0 },
+  { "ui.treeScrollLinesMax",
+    eMetric_TreeScrollLinesMax, PR_FALSE, nsLookAndFeelTypeInt, 0 },
 };
 
 nsLookAndFeelFloatPref nsXPLookAndFeel::sFloatPrefs[] =
@@ -158,6 +171,7 @@ char* nsXPLookAndFeel::sColorPrefs[] =
   "ui.window",
   "ui.windowframe",
   "ui.windowtext",
+  "ui.-moz-buttondefault",
   "ui.-moz-field",
   "ui.-moz-fieldtext",
   "ui.-moz-dialog",
@@ -183,7 +197,6 @@ PRBool nsXPLookAndFeel::sInitialized = PR_FALSE;
 
 nsXPLookAndFeel::nsXPLookAndFeel() : nsILookAndFeel()
 {
-  NS_INIT_ISUPPORTS();
 }
 
 static int PR_CALLBACK intPrefChanged (const char *newpref, void *data)
