@@ -64,13 +64,18 @@
 #endif
 #endif /* defined(DEBUG) */
 
+#define UNLOAD_DEPENDENT_LIBS
+#ifdef HPUX
+#undef UNLOAD_DEPENDENT_LIBS
+#endif
+
 #include "nsNativeComponentLoader.h"
 
 nsDll::nsDll(nsIFile *dllSpec, nsNativeComponentLoader *loader)
     : m_dllSpec(do_QueryInterface(dllSpec)),
-      m_loader(loader),
       m_instance(NULL), 
       m_moduleObject(NULL),
+      m_loader(loader),
       m_markForUnload(PR_FALSE)
 {
     NS_ASSERTION(loader, "Null loader when creating a nsDLL");
@@ -139,11 +144,6 @@ PRBool nsDll::Load(void)
     //   on the dependent libraries with the assumption that the 
     //   component library holds a reference via the OS so loader.
 
-    if (!m_loader->mLoadedDependentLibs) {
-        NS_ERROR("huh?  no dependent libs");
-        return PR_TRUE;
-    }
-
 #if defined(XP_UNIX)
     nsCOMPtr<nsIComponentLoaderManager> manager = do_QueryInterface(m_loader->mCompMgr);
     if (!manager)
@@ -152,7 +152,9 @@ PRBool nsDll::Load(void)
     nsXPIDLCString extraData;
     manager->GetOptionalData(m_dllSpec, nsnull, getter_Copies(extraData));
     
+#ifdef UNLOAD_DEPENDENT_LIBS
     nsVoidArray dependentLibArray;
+#endif
 
     // if there was any extra data, treat it as a listing of dependent libs
     if (extraData != nsnull) 
@@ -181,12 +183,12 @@ PRBool nsDll::Load(void)
         while (token!=nsnull)
         {
             nsCStringKey key(token);
-            if (m_loader->mLoadedDependentLibs->Get(&key)) {
+            if (m_loader->mLoadedDependentLibs.Get(&key)) {
                 token = nsCRT::strtok(newStr, " ", &newStr);
                 continue;
             }
 
-            m_loader->mLoadedDependentLibs->Put(&key, (void*)1);
+            m_loader->mLoadedDependentLibs.Put(&key, (void*)1);
 
             nsXPIDLCString libpath;
             file->SetNativeLeafName(nsDependentCString(token));
@@ -215,8 +217,10 @@ PRBool nsDll::Load(void)
             // if we couldn't load the dependent library.  We did the best we
             // can.  Now just let us fail later if this really was a required
             // dependency.
+#ifdef UNLOAD_DEPENDENT_LIBS
             if (lib) 
                 dependentLibArray.AppendElement((void*)lib);
+#endif
                 
             token = nsCRT::strtok(newStr, " ", &newStr);
         }
@@ -235,6 +239,7 @@ PRBool nsDll::Load(void)
     // the dependency library we just loaded.  
     // XXX should we unload later - or even at all?
 
+#ifdef UNLOAD_DEPENDENT_LIBS
     if (extraData != nsnull)
     {
         PRInt32 arrayCount = dependentLibArray.Count();
@@ -242,6 +247,7 @@ PRBool nsDll::Load(void)
             PR_UnloadLibrary((PRLibrary*)dependentLibArray.ElementAt(index));
     }
 #endif
+#endif
 
 #ifdef NS_BUILD_REFCNT_LOGGING
         nsTraceRefcnt::SetActivityIsLegal(PR_TRUE);
