@@ -28,7 +28,7 @@
 #include "nsMemory.h"
 #include "nsXPIDLString.h" 
 
-#include "nsLocalFileMac.h"
+#include "nsLocalFile.h"
 #include "nsNativeCharsetUtils.h"
 #include "nsISimpleEnumerator.h"
 #include "nsIComponentManager.h"
@@ -813,7 +813,6 @@ class nsDirEnumerator : public nsISimpleEnumerator
 
         nsDirEnumerator()
         {
-            NS_INIT_ISUPPORTS();
         }
 
         nsresult Init(nsILocalFileMac* parent) 
@@ -919,8 +918,6 @@ nsLocalFile::nsLocalFile() :
     mType('TEXT'),
     mCreator(kDefaultCreator)
 {
-    NS_INIT_ISUPPORTS();
-
     ClearFSSpec(mSpec);
     ClearFSSpec(mTargetSpec);
     
@@ -931,7 +928,6 @@ nsLocalFile::nsLocalFile() :
 
 nsLocalFile::nsLocalFile(const nsLocalFile& srcFile)
 {
-    NS_INIT_ISUPPORTS();
     *this = srcFile;
 }
 
@@ -945,8 +941,6 @@ nsLocalFile::nsLocalFile(const FSSpec& aSpec, const nsACString& aAppendedPath) :
     mType('TEXT'),
     mCreator(kDefaultCreator)
 {
-    NS_INIT_ISUPPORTS();
-    
     ClearFSSpec(mTargetSpec);
     
     InitClassStatics();
