@@ -56,10 +56,9 @@
 #include "nsCOMPtr.h"
 #include "nsMemory.h"
 #include "nsIFile.h"
-#include "nsILocalFile.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
-#include "nsLocalFileUnix.h"
+#include "nsLocalFile.h"
 #include "nsIComponentManager.h"
 #include "nsXPIDLString.h"
 #include "prproces.h"
@@ -121,7 +120,6 @@ nsDirEnumeratorUnix::nsDirEnumeratorUnix() :
                          mDir(nsnull), 
                          mEntry(nsnull)
 {
-    NS_INIT_REFCNT();
 }
 
 nsDirEnumeratorUnix::~nsDirEnumeratorUnix()
@@ -201,7 +199,6 @@ nsDirEnumeratorUnix::GetNextEntry()
 nsLocalFile::nsLocalFile() :
     mHaveCachedStat(PR_FALSE)
 {
-    NS_INIT_REFCNT();
 }
 
 nsLocalFile::~nsLocalFile()
@@ -492,7 +489,7 @@ nsLocalFile::Normalize()
     BPath be_p;
     status_t err;
     if ((err = be_e.GetPath(&be_p)) == B_OK) {
-        resolved_path_ptr = be_p.Path();
+        resolved_path_ptr = (char *)be_p.Path();
         PL_strncpyz(resolved_path, resolved_path_ptr, PATH_MAX - 1);
     }
 #else
@@ -709,8 +706,13 @@ nsLocalFile::CopyToNative(nsIFile *newParent, const nsACString &newName)
 
     // we copy the parent here so 'newParent' remains immutable
     nsCOMPtr <nsIFile> workParent;
-    if (NS_FAILED(rv = newParent->Clone(getter_AddRefs(workParent))))
-        return rv;
+    if (newParent) {
+        if (NS_FAILED(rv = newParent->Clone(getter_AddRefs(workParent))))
+            return rv;
+    } else {
+        if (NS_FAILED(rv = GetParent(getter_AddRefs(workParent))))
+            return rv;
+    }
     
     // check to see if we are a directory or if we are a file
     PRBool isDirectory;
@@ -1159,6 +1161,10 @@ nsLocalFile::GetParent(nsIFile **aParent)
     NS_ENSURE_ARG_POINTER(aParent);
     *aParent       = nsnull;
 
+    // if '/' we are at the top of the volume, return null
+    if (mPath.Equals("/"))
+        return  NS_OK;
+ 
     // <brendan, after jband> I promise to play nice
     char *buffer   = NS_CONST_CAST(char *, mPath.get()),
          *slashp   = buffer;
@@ -1203,6 +1209,47 @@ nsLocalFile::Exists(PRBool *_retval)
     return NS_OK;
 }
 
+#ifdef XP_BEOS
+// access() is buggy in BeOS POSIX implementation, at least for BFS, using stat() instead
+NS_IMETHODIMP
+nsLocalFile::IsWritable(PRBool *_retval)
+{
+    CHECK_mPath();
+    NS_ENSURE_ARG_POINTER(_retval);
+    struct stat buf;
+    *_retval = (stat(mPath.get(), &buf) == 0);
+    *_retval = *_retval && (buf.st_mode & (S_IWUSR | S_IWGRP | S_IWOTH ));
+    if (*_retval || errno == EACCES)
+        return NS_OK;
+    return NSRESULT_FOR_ERRNO();
+}
+
+NS_IMETHODIMP
+nsLocalFile::IsReadable(PRBool *_retval)
+{
+    CHECK_mPath();
+    NS_ENSURE_ARG_POINTER(_retval);
+    struct stat buf;
+    *_retval = (stat(mPath.get(), &buf) == 0);
+    *_retval = *_retval && (buf.st_mode & (S_IRUSR | S_IRGRP | S_IROTH ));
+    if (*_retval || errno == EACCES)
+        return NS_OK;
+    return NSRESULT_FOR_ERRNO();
+}
+
+NS_IMETHODIMP
+nsLocalFile::IsExecutable(PRBool *_retval)
+{
+    CHECK_mPath();
+    NS_ENSURE_ARG_POINTER(_retval);
+    struct stat buf;
+    *_retval = (stat(mPath.get(), &buf) == 0);
+    *_retval = *_retval && (buf.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH ));
+    if (*_retval || errno == EACCES)
+        return NS_OK;
+    return NSRESULT_FOR_ERRNO();
+}
+#else
 NS_IMETHODIMP
 nsLocalFile::IsWritable(PRBool *_retval)
 {
@@ -1238,11 +1285,12 @@ nsLocalFile::IsExecutable(PRBool *_retval)
         return NS_OK;
     return NSRESULT_FOR_ERRNO();
 }
-
+#endif
 NS_IMETHODIMP
 nsLocalFile::IsDirectory(PRBool *_retval)
 {
     NS_ENSURE_ARG_POINTER(_retval);
+    *_retval = PR_FALSE;
     VALIDATE_STAT_CACHE();
     *_retval = S_ISDIR(mCachedStat.st_mode);
     return NS_OK;
@@ -1252,6 +1300,7 @@ NS_IMETHODIMP
 nsLocalFile::IsFile(PRBool *_retval)
 {
     NS_ENSURE_ARG_POINTER(_retval);
+    *_retval = PR_FALSE;
     VALIDATE_STAT_CACHE();
     *_retval = S_ISREG(mCachedStat.st_mode);
     return NS_OK;
@@ -1366,7 +1415,7 @@ nsLocalFile::GetNativeTarget(nsACString &_retval)
 
     nsresult rv;
     PRBool isSymlink;
-    nsCOMPtr<nsIFile> self(dont_QueryInterface(this));
+    nsCOMPtr<nsIFile> self(this);
     nsCOMPtr<nsIFile> parent;
     while (NS_SUCCEEDED(rv = self->GetParent(getter_AddRefs(parent)))) {
         NS_ASSERTION(parent != nsnull, "no parent?!");
