@@ -84,7 +84,7 @@ nsCOMPtr<nsIDOMNode> GetDOMNodeFromDocShell(nsIDocShell *aShell)
     nsCOMPtr<nsIDocumentViewer> docv(do_QueryInterface(cv));
     if (docv) {
       nsCOMPtr<nsIDocument> doc;
-      docv->GetDocument(*getter_AddRefs(doc));
+      docv->GetDocument(getter_AddRefs(doc));
       if (doc) {
         nsCOMPtr<nsIDOMDocument> domdoc(do_QueryInterface(doc));
         if (domdoc) {
@@ -213,8 +213,6 @@ nsAppShellWindowEnumerator::nsAppShellWindowEnumerator (
     mWindowMediator(&aMediator), mType(aTypeString),
     mCurrentPosition(0)
 {
-  NS_INIT_ISUPPORTS();
-
   mWindowMediator->AddEnumerator(this);
   NS_ADDREF(mWindowMediator);
 }
@@ -328,7 +326,7 @@ nsWindowInfo *nsASDOMWindowEarlyToLateEnumerator::FindNext() {
 
   nsWindowInfo *info,
                *listEnd;
-  PRBool        allWindows = mType.Length() == 0;
+  PRBool        allWindows = mType.IsEmpty();
 
   // see nsXULWindowEarlyToLateEnumerator::FindNext
   if (!mCurrentPosition)
@@ -367,7 +365,7 @@ nsWindowInfo *nsASXULWindowEarlyToLateEnumerator::FindNext() {
 
   nsWindowInfo *info,
                *listEnd;
-  PRBool        allWindows = mType.Length() == 0;
+  PRBool        allWindows = mType.IsEmpty();
 
   /* mCurrentPosition null is assumed to mean that the enumerator has run
      its course and is now basically useless. It could also be interpreted
@@ -412,7 +410,7 @@ nsWindowInfo *nsASDOMWindowFrontToBackEnumerator::FindNext() {
 
   nsWindowInfo *info,
                *listEnd;
-  PRBool        allWindows = mType.Length() == 0;
+  PRBool        allWindows = mType.IsEmpty();
 
   // see nsXULWindowEarlyToLateEnumerator::FindNext
   if (!mCurrentPosition)
@@ -452,7 +450,7 @@ nsWindowInfo *nsASXULWindowFrontToBackEnumerator::FindNext() {
 
   nsWindowInfo *info,
                *listEnd;
-  PRBool        allWindows = mType.Length() == 0;
+  PRBool        allWindows = mType.IsEmpty();
 
   // see nsXULWindowEarlyToLateEnumerator::FindNext
   if (!mCurrentPosition)
@@ -492,7 +490,7 @@ nsWindowInfo *nsASDOMWindowBackToFrontEnumerator::FindNext() {
 
   nsWindowInfo *info,
                *listEnd;
-  PRBool        allWindows = mType.Length() == 0;
+  PRBool        allWindows = mType.IsEmpty();
 
   // see nsXULWindowEarlyToLateEnumerator::FindNext
   if (!mCurrentPosition)
@@ -535,7 +533,7 @@ nsWindowInfo *nsASXULWindowBackToFrontEnumerator::FindNext() {
 
   nsWindowInfo *info,
                *listEnd;
-  PRBool        allWindows = mType.Length() == 0;
+  PRBool        allWindows = mType.IsEmpty();
 
   // see nsXULWindowEarlyToLateEnumerator::FindNext
   if (!mCurrentPosition)
