@@ -64,7 +64,6 @@
 #include "nsIWebNavigation.h"
 #include "nsIWindowWatcher.h"
 
-#include "nsIXULPopupListener.h"
 #include "nsIDOMXULElement.h"
 
 #include "nsGUIEvent.h"
@@ -144,20 +143,12 @@
 /* Define Class IDs */
 static NS_DEFINE_CID(kWindowCID,           NS_WINDOW_CID);
 static NS_DEFINE_CID(kWebShellCID,         NS_WEB_SHELL_CID);
-static NS_DEFINE_CID(kAppShellServiceCID,  NS_APPSHELL_SERVICE_CID);
-static NS_DEFINE_CID(kAppShellCID,         NS_APPSHELL_CID);
 
 #include "nsWidgetsCID.h"
 static NS_DEFINE_CID(kMenuBarCID,          NS_MENUBAR_CID);
 static NS_DEFINE_CID(kMenuCID,             NS_MENU_CID);
 static NS_DEFINE_CID(kMenuItemCID,         NS_MENUITEM_CID);
 
-static NS_DEFINE_CID(kPrefCID,             NS_PREF_CID);
-static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
-
-static NS_DEFINE_CID(kLayoutDocumentLoaderFactoryCID, NS_LAYOUT_DOCUMENT_LOADER_FACTORY_CID);
-static NS_DEFINE_CID(kXULPopupListenerCID, NS_XULPOPUPLISTENER_CID);
-
 
 #ifdef DEBUG_rods
 #define DEBUG_MENUSDEL 1
@@ -165,8 +156,6 @@ static NS_DEFINE_CID(kXULPopupListenerCID, NS_XULPOPUPLISTENER_CID);
 
 #include "nsIWebShell.h"
 
-static NS_DEFINE_CID(kStringBundleServiceCID,     NS_STRINGBUNDLESERVICE_CID);
-
 #define SIZE_PERSISTENCE_TIMEOUT 500 // msec
 
 const char * kPrimaryContentTypeValue  = "content-primary";
@@ -199,8 +188,6 @@ struct nsWebShellInfo {
 
 nsWebShellWindow::nsWebShellWindow() : nsXULWindow()
 {
-  NS_INIT_ISUPPORTS();
-
   mWebShell = nsnull;
   mWindow   = nsnull;
   mLockedUntilChromeLoad = PR_FALSE;
@@ -339,7 +326,7 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
                          nsnull,
                          nsnull,
                          nsnull);
-    NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
+    NS_ENSURE_SUCCESS(rv, rv);
   }
                      
   return rv;
@@ -735,7 +722,7 @@ NS_IMETHODIMP nsWebShellWindow::LoadMenuItem(
     // Try to find the key node.
     nsCOMPtr<nsIDocument> document;
     nsCOMPtr<nsIContent> content = do_QueryInterface(domElement);
-    if (NS_FAILED(rv = content->GetDocument(*getter_AddRefs(document)))) {
+    if (NS_FAILED(rv = content->GetDocument(getter_AddRefs(document)))) {
       NS_ERROR("Unable to retrieve the document.");
       return rv;
     }
@@ -970,7 +957,7 @@ void nsWebShellWindow::DynamicLoadMenus(nsIDOMDocument * aDOMDoc, nsIWidget * aP
       }
 
       nsCOMPtr<nsIPresContext> presContext;
-      if (NS_FAILED(rv = docViewer->GetPresContext(*getter_AddRefs(presContext)))) {
+      if (NS_FAILED(rv = docViewer->GetPresContext(getter_AddRefs(presContext)))) {
           NS_ERROR("Unable to retrieve the doc viewer's presentation context.");
           return;
       }
@@ -1182,6 +1169,7 @@ nsWebShellWindow::SetPersistenceTimer(PRBool aSize, PRBool aPosition, PRBool aMo
     nsresult rv;
     mSPTimer = do_CreateInstance("@mozilla.org/timer;1", &rv);
     if (NS_SUCCEEDED(rv)) {
+      NS_ADDREF_THIS(); // for the timer, which holds a reference to this window
       mSPTimer->InitWithFuncCallback(FirePersistenceTimer, this,
                                      SIZE_PERSISTENCE_TIMEOUT, nsITimer::TYPE_ONE_SHOT);
       mSPTimerSize = aSize;
@@ -1269,7 +1257,7 @@ nsWebShellWindow::OnStateChange(nsIWebProgress *aProgress,
       return NS_OK;
 
     nsCOMPtr<nsIDocument> doc;
-    docv->GetDocument(*getter_AddRefs(doc));
+    docv->GetDocument(getter_AddRefs(doc));
     if (!doc)
       return NS_OK;
 
@@ -1396,7 +1384,7 @@ nsCOMPtr<nsIDOMDocument> nsWebShellWindow::GetNamedDOMDoc(const nsAString & aWeb
     return domDoc;
 
   nsCOMPtr<nsIDocument> doc;
-  docv->GetDocument(*getter_AddRefs(doc));
+  docv->GetDocument(getter_AddRefs(doc));
   if (doc)
     return nsCOMPtr<nsIDOMDocument>(do_QueryInterface(doc));
 
@@ -1420,7 +1408,7 @@ void nsWebShellWindow::LoadContentAreas() {
     nsCOMPtr<nsIDocumentViewer> docViewer = do_QueryInterface(contentViewer);
     if (docViewer) {
       nsCOMPtr<nsIDocument> doc;
-      docViewer->GetDocument(*getter_AddRefs(doc));
+      docViewer->GetDocument(getter_AddRefs(doc));
       nsCOMPtr<nsIURI> mainURL;
       doc->GetDocumentURL(getter_AddRefs(mainURL));
       if (mainURL) {
@@ -1435,7 +1423,7 @@ void nsWebShellWindow::LoadContentAreas() {
 
   // content URLs are specified in the search part of the URL
   // as <contentareaID>=<escapedURL>[;(repeat)]
-  if (searchSpec.Length() > 0) {
+  if (!searchSpec.IsEmpty()) {
     PRInt32     begPos,
                 eqPos,
                 endPos;
@@ -1504,7 +1492,7 @@ PRBool nsWebShellWindow::ExecuteCloseHandler()
         nsCOMPtr<nsIDocumentViewer> docViewer;
         nsCOMPtr<nsIPresContext> presContext;
         docViewer = do_QueryInterface(contentViewer);
-        if (docViewer && NS_SUCCEEDED(docViewer->GetPresContext(*getter_AddRefs(presContext)))) {
+        if (docViewer && NS_SUCCEEDED(docViewer->GetPresContext(getter_AddRefs(presContext)))) {
           nsEventStatus status = nsEventStatus_eIgnore;
           nsMouseEvent event;
           event.eventStructType = NS_EVENT;
@@ -1654,7 +1642,7 @@ NS_IMETHODIMP nsWebShellWindow::Destroy()
    if(docv)
       {
       nsCOMPtr<nsIDocument> doc;
-      docv->GetDocument(*getter_AddRefs(doc));
+      docv->GetDocument(getter_AddRefs(doc));
       if(doc)
          doc->RemoveObserver(NS_STATIC_CAST(nsIDocumentObserver*, this));
       }
@@ -1666,6 +1654,7 @@ NS_IMETHODIMP nsWebShellWindow::Destroy()
     mSPTimer->Cancel();
     mSPTimer = nsnull;
     PersistPositionAndSize(mSPTimerPosition, mSPTimerSize, mSPTimerMode);
+    NS_RELEASE_THIS(); // the timer held a reference to us
   }
   PR_Unlock(mSPTimerLock);
 
