@@ -109,8 +109,6 @@ nsWindowMediator::nsWindowMediator() :
   mEnumeratorList(), mOldestWindow(0), mTopmostWindow(0),
   mTimeStamp(0), mUpdateBatchNest(0), mListLock(0)
 {
-   NS_INIT_ISUPPORTS();
-
    // This should really be done in the static constructor fn.
    nsresult rv;
    rv = Init();
@@ -337,7 +335,7 @@ nsWindowMediator::MostRecentWindowInfo(const PRUnichar* inType)
 {
   PRInt32       lastTimeStamp = -1;
   nsAutoString  typeString(inType);
-  PRBool        allWindows = !inType || typeString.Length() == 0;
+  PRBool        allWindows = !inType || typeString.IsEmpty();
 
   // Find the most window with the highest time stamp that matches
   // the requested type
